!===============================================================================
! CVS $Id: flux_albedo.F90,v 1.6 2001/12/07 22:13:35 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/flux_albedo.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

SUBROUTINE flux_albo( So )

   USE buf_mod
   USE control_mod
   USE shr_orb_mod
   USE shr_timer_mod ! timing routines
   use const_mod     ! provides physical constants

   implicit none

   !----- arguments -----
   real    :: So(n_o,nSo)         ! ocn state buffer
 
   !----- local -----
   integer :: n                   ! loop index
   real    :: rlat                ! gridcell latitude in radians
   real    :: rlon                ! gridcell longitude in radians
   real    :: delta               ! Solar declination angle in radians
   real    :: eccf                ! Earth-sun distance factor
   real    :: cosz                ! Cosine of solar zenith angle
   real    :: calday              ! calendar day including fraction, at 0e
   real    :: calendar(12)        ! days of year past at beginning of each month
   logical :: first_call = .true. ! flags 1st envocation of this routine
   integer :: timer_n             ! system-clock-timer number


   real,parameter :: albdif     = 0.06 ! 60 deg ref albedo, diffuse
   real,parameter :: albdir     = 0.07 ! 60 deg ref albedo, direct 

   data calendar /0,31,59,90,120,151,181,212,243,273,304,334/

   SAVE

   !----- formats -----
   character(len=*),parameter :: F00 = "('(flux_albo) ',3a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   if flux_albav/=0 (ie. "on" or "true")
!     Compute four effective daily avg surface albedos for all 
!     combinations of visible/near-infrared and direct/diffuse radiation
!     without accounting for zenith angle (ie. a "daily average" albedo)
!   else
!     Compute four surface albedos for all combinations of visible/
!     near-infrared and direct/diffuse radiation, accounting for 
!     instantaneous zenith angle
!
! NOTE:
!   o upon input, albedos are assumed to be a 60 degree reference albedo
!   o albedos are computed by taking the 60 deg reference albedo
!     and then adjusting this value based on zenith angle
!   o Albedos are independent of spectral interval and other physical
!     factors such as surface wind speed.
!
!   For more details see Briegleb, Bruce P., 1992: Delta-Eddington
!   Approximation for Solar Radiation in the NCAR Community Climate 
!   Model, Journal of Geophysical Research, Vol 97, D7, pp7603-7612.
!
! CODE HISTORY:
!   Original version:           CCM1
!   Standardized:               J. Rosinski, June 1992
!   Rewritten for land only:    J. Rosinski, May  1994
!   Rewritten for ocean only:   B. Kauffman, July 1994
!  
!-------------------------------------------------------------------------------

   if (first_call) call shr_timer_get(timer_n,'flux_albo')
   call shr_timer_start(timer_n)

   IF (flx_albav /= 0) THEN
     !--------------------------------------------------------------------
     if (first_call) write(6,F00) 'ocn albedo is zenith angle independant'
     !--------------------------------------------------------------------
     do n=1,n_o
        So(n,kSo_anidr) = albdif
        So(n,kSo_avsdr) = albdif
        So(n,kSo_anidf) = albdif
        So(n,kSo_avsdf) = albdif
     end do
   ELSE
     !--------------------------------------------------------------------
     if (first_call) write(6,F00) 'ocn albedo is zenith angle dependant'
     !--------------------------------------------------------------------

     !--- julian day ---
     calday = calendar(month) + day + (sec+ashift)/86400.0

     !--- solar declination ---
     call shr_orb_decl(calday,orb_eccen,orb_mvelpp,orb_lambm0,orb_obliqr,delta,eccf)

!$OMP PARALLEL DO PRIVATE(n,rlat,rlon,cosz)
     do n=1,n_o
         rlat = DEG2RAD*yc_o(n)
         rlon = DEG2RAD*xc_o(n)
         cosz = shr_orb_cosz( calday, rlat, rlon, delta )
         if (cosz  >  0.0) then !--- sun hit --
           So(n,kSo_anidr) = (.026/(cosz**1.7 + 0.065)) +   &
           &                 (.150*(cosz      - 0.10 )  *   &
           &                       (cosz      - 0.50 )  *   &
           &                       (cosz      - 1.00 )  )
           So(n,kSo_avsdr) = So(n,kSo_anidr)
           So(n,kSo_anidf) = albdif
           So(n,kSo_avsdf) = albdif
         else !--- dark side of earth ---
           So(n,kSo_anidr) = 1.0
           So(n,kSo_avsdr) = 1.0
           So(n,kSo_anidf) = 1.0
           So(n,kSo_avsdf) = 1.0
         end if
     end do
   END IF

   call shr_timer_stop(timer_n)
   first_call=.false.
END

!===============================================================================

SUBROUTINE flux_albi( Si )

   USE buf_mod
   USE control_mod
   USE shr_orb_mod
   USE shr_timer_mod ! timing routines
   use const_mod     ! provides physical constants

   implicit none

   !----- arguments -----
   real    :: Si(n_i,nSi)         ! ice state buffer
 
   !----- local -----
   integer :: n                   ! loop index
   real    :: rlat                ! gridcell latitude in radians
   real    :: rlon                ! gridcell longitude in radians
   real    :: delta               ! Solar declination angle in radians
   real    :: eccf                ! Earth-Sun distance factor
   real    :: calday              ! calendar day including fraction, at 0e
   real    :: cosz                ! Cosine of solar zenith angle
   real    :: calendar(12)        ! days of year past at beginning of each month
   logical :: first_call = .true. ! flags 1st invocation of this routine
   integer :: timer_n             ! system-clock-timer number

   data calendar /0,31,59,90,120,151,181,212,243,273,304,334/

   SAVE

   !----- formats -----
   character(len=*),parameter :: F00 = "('(flux_albi) ',3a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!   if flux_albav/=0 (ie. "on" or "true")
!     Impose a zenith angle dependance on ice model albedos.
!     Currently this only involves setting albedos to zero
!     on the dark side of the earth.
!   else
!     do not alter ice albedos 
! NOTE:
!   o upon input, albedos are assumed to be a 60 degree reference albedo
!-------------------------------------------------------------------------------

   if (first_call) call shr_timer_get(timer_n,'flux_albi')
   call shr_timer_start(timer_n)

   IF (flx_albav /= 0) THEN
     !--------------------------------------------------------------------
     if (first_call) write(6,F00) 'ice albedo is zenith angle independant'
     !--------------------------------------------------------------------
   ELSE
     !--------------------------------------------------------------------
     if (first_call) write(6,F00) 'ice albedo is zenith angle dependant'
     !--------------------------------------------------------------------

     !--- julian day ---
     calday = calendar(month) + day + (sec+ashift)/86400.0

     !--- solar declination ---
     call shr_orb_decl(calday,orb_eccen,orb_mvelpp,orb_lambm0,orb_obliqr,delta,eccf)
   
!$OMP PARALLEL DO PRIVATE(n,rlat,rlon,cosz)
     do n=1,n_i
        rlat = DEG2RAD*yc_o(n)
        rlon = DEG2RAD*xc_o(n)
        cosz = shr_orb_cosz( calday, rlat, rlon, delta )
        if (cosz < 0.0) then !--- dark side of earth ---
          Si(n,kSi_anidr) = 1.0
          Si(n,kSi_avsdr) = 1.0
          Si(n,kSi_anidf) = 1.0
          Si(n,kSi_avsdf) = 1.0
        end if
     end do
   END IF

   call shr_timer_stop(timer_n)
   first_call=.false.
END

!===============================================================================
