!===============================================================================
! CVS $Id: map_npfix.F90,v 1.4 2001/12/07 22:13:36 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/cpl/cpl5/map_npfix.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

SUBROUTINE map_npfix(Sa_in,Sa_out)

   USE buf_mod       ! model data fields
   USE control_mod   ! control flags
   USE map_mod       ! map data, eg. cell areas & interface fractions
   USE const_mod     ! physical constants, like pi
   USE shr_timer_mod ! timing routines

   IMPLICIT none

   !----- local -----
   real,intent(in)    :: Sa_in (n_a,nSa) ! atm states on the input  (atm) grid
   real,intent(inout) :: Sa_out(n_o,nSa) ! atm states on the output (ocn) grid

   !----- local -----
   integer :: n           ! index relative to output array (subroutine arg)
   integer :: npts        ! number of output array points that need fixing
   integer :: nn          ! index that goes from 1 to npts
   integer :: m           ! index relative to input array (subroutine arg)
   integer :: mpts        ! number of grid points in top row of input grid
   integer :: mm          ! index that goes from 1 to mpts
   integer :: m1st        ! 1st m st y(m)=max input latitude

   real    :: npu         ! north-pole u-velocity
   real    :: npv         ! north-pole v-velocity

   real   ,allocatable ::  wght(:,:) ! 4 bilin map weights
   integer,allocatable ::    mw(:,:) ! index on input grid
   integer,allocatable ::    nw  (:) ! index on output grid
   real   ,allocatable :: Sa_in_u(:) ! local NP u-velocity "on" input grid
   real   ,allocatable :: Sa_in_v(:) ! local NP v-velocity "on" input grid
   real   ,allocatable ::  theta (:) ! rotation angle wrt 0 deg east (radians)

   real    :: alpha,beta    ! used to compute bilin weights
   real    :: dx,dy         ! used to compute biline weights
   real    :: xm,ym         ! x/y-coords of output data for bilin weights
   real    :: xl,xh,yl,yh   ! x/y-coords of  input data for bilin weights
   real    :: w1,w2,w3,w4   ! 4 bilin weights
   integer :: m1,m2,m3,m4   ! 4 bilin indicies wrt input grid
   integer :: mm3,mm4       ! bilin indicies wrt fabricated NP input grid row
   real    :: f1,f2,f3,f4   ! 4 bilin function values on input grid

   logical :: first_call = .true. ! flags 1st invocation of routine
   integer :: timer_n             ! system-clock-timer number

   save  

   !----- formats -----
   character(len=*),parameter :: F00 = "('(map_npfix) ',4a)"

!-------------------------------------------------------------------------------
! PURPOSE:
!    Compute/correct atm wind velocties on ocn (or ice) grid.
!
! ASSUMPTIONS:
!  o the input (atm) grid has a "top row" with a constant latitude value
!  o the ocn & ice have identical grids, this routine works for both ocn & ice
!
! NOTATION:
!
!           ^  f4      f3
!       b   |
!       e  dy      f                interpolate to f from f1,f2,f3,f4
!       t   |                     
!       a   v  f1      f2
!              <--dx--->
!               alpha
!
!   f  is located at (xm,ym)
!   f1 is located at (xl,yl)
!   f2 is located at (xh,yl)
!   f3 is located at (xh,yh)
!   f4 is located at (xl,yh)
!
!-------------------------------------------------------------------------------

   !----------------------------------------------------------------------------
   ! one-time setup computations
   !----------------------------------------------------------------------------
   if (first_call) then
      write(6,F00) "compute bilinear weights & indicies for NP region."

      !--- determine number of points in top row of atm (input) data ---
      mpts = 0
      m1st = 0
      do m=1,n_a
        if (domain_a%yc(m) >= domain_a%yc(n_a) ) then
           m1st = m               ! 1st point in top row
           mpts = n_a - m1st + 1  ! number of points in top row
           exit
        end if 
      end do

      !--- allocate memory for a new, north-pole row ---
      allocate(Sa_in_u(mpts))  ! u-velocity
      allocate(Sa_in_v(mpts))  ! v-velocity
      allocate( theta (mpts))  ! rotation angle wrt prime-meridian

      !--- compute (u,v) rotation angles relative to prime-meridian ---
      do m=m1st,n_a
        mm = m - m1st + 1                   ! index into local NP row
        theta(mm) = domain_a%xc(m)*deg2rad  ! units are radians
      end do

      !--- determine number of points that need correcting (output) data ---
      npts = 0
      do n=1,n_o
        if (domain_o%yc(n) > domain_a%yc(n_a) ) npts = npts + 1
      end do

      !--- allocate bilin-map memory given number of output points involved ---
      allocate(   nw(  npts) ) ! indices to output points
      allocate(   mw(4,npts) ) ! for each output point, 4 input  indicies
      allocate( wght(4,npts) ) ! for each output point, 4 bilinear weights

      !--- compute bilin-map data that utilizes new/local NP row ---
      nn = 0
      do n=1,n_o
        if (domain_o%yc(n) > domain_a%yc(n_a) ) then
           nn = nn + 1
           nw(nn) = n              ! output index associated with weights
           xm = domain_o%xc(n)     ! x-coord of output
           ym = domain_o%yc(n)     ! y-coord of output
           yl = domain_a%yc(n_a)   ! constant y-coord of top atm row
           yh = 90.0               ! constant y-coord of north pole
           if ( xm < domain_a%xc(m1st) ) then
             m1 = n_a              ! input indicies associated with weights
             m2 = m1st
             m3 = m2   + mpts      ! note: this index goes beyond actual data
             m4 = m1   + mpts      ! note: this index goes beyond actual data
             xl = domain_a%xc(m1) - 360.0
             xh = domain_a%xc(m2)
           else if ( xm < domain_a%xc(n_a) ) then
             do m=m1st,n_a - 1
               if ( xm < domain_a%xc(m+1) ) then
                 m1 = m            ! input indicies associated with weights
                 m2 = m+1
                 m3 = m2  + mpts   ! note: this index goes beyond actual data
                 m4 = m1  + mpts   ! note: this index goes beyond actual data
                 xl = domain_a%xc(m1)
                 xh = domain_a%xc(m2)
                 exit
               end if
             end do
           else
             m1 = n_a              ! input indicies associated with weights
             m2 = m1st
             m3 = m2   + mpts      ! note: this index goes beyond actual data
             m4 = m1   + mpts      ! note: this index goes beyond actual data
             xl = domain_a%xc(m1)
             xh = domain_a%xc(m2) + 360.0
           end if
           dx = xh - xl 
           dy = yh - yl 
           alpha = (xm - xl)/dx
           beta  = (ym - yl)/dy
           wght(1,nn) = (1.0-alpha)*(1.0-beta) ! 4 input bilinear weights
           wght(2,nn) = (    alpha)*(1.0-beta)
           wght(3,nn) = (    alpha)*(    beta)
           wght(4,nn) = (1.0-alpha)*(    beta) 
           mw  (1,nn) = m1                     ! input indicies wrt with weights
           mw  (2,nn) = m2   
           mw  (3,nn) = m3  
           mw  (4,nn) = m4 
        end if
      end do

      call shr_timer_get(timer_n,'map_npfix')

      first_call = .false.
   end if

   call shr_timer_start(timer_n)

   !----------------------------------------------------------------------------
   ! fabricate multiple instances of atm north-pole data w/ various rotations
   !----------------------------------------------------------------------------

   npu = 0  ! avg NP value with y-basis vector pointing north along
   npv = 0  ! prime-meridian and x-basis vector pointing east at prime-meridian
   do mm=1,mpts
      m = n_a - mpts + mm
      npu = npu + cos(theta(mm))*Sa_in(m,kSa_u) - sin(theta(mm))*Sa_in(m,kSa_v)
      npv = npv + sin(theta(mm))*Sa_in(m,kSa_u) + cos(theta(mm))*Sa_in(m,kSa_v)
   end do
   npu = npu/mpts
   npv = npv/mpts

   !--- copies of NP value with basis-vectors aligned with cell's longitude
   do mm=1,mpts
      Sa_in_u(mm) =  cos(theta(mm))*npu + sin(theta(mm))*npv
      Sa_in_v(mm) = -sin(theta(mm))*npu + cos(theta(mm))*npv
   end do

   !----------------------------------------------------------------------------
   ! bilinear interpolation from atm to select set of ocn/ice grid points
   !----------------------------------------------------------------------------
   do nn=1,npts
      n   = nw(nn)            ! output grid index

      m1  = mw(1,nn)          ! input grid indicies on atm grid
      m2  = mw(2,nn)          ! input grid indicies on atm grid
      mm3 = mw(3,nn) - n_a    ! input grid indicies on local "NP grid" array
      mm4 = mw(4,nn) - n_a    ! input grid indicies on local "NP grid" array

      w1 = wght(1,nn)         ! 4 mapping weights
      w2 = wght(2,nn)
      w3 = wght(3,nn)
      w4 = wght(4,nn)

      f1 = Sa_in  ( m1,kSa_u)  ! 4 input velocities
      f2 = Sa_in  ( m2,kSa_u)
      f3 = Sa_in_u(mm3)        ! local NP pole value
      f4 = Sa_in_u(mm4)        ! local NP pole value
      Sa_out(n,kSa_u) = w1*f1 + w2*f2 + w3*f3 + w4*f4

      f1 = Sa_in  ( m1,kSa_v)
      f2 = Sa_in  ( m2,kSa_v)
      f3 = Sa_in_v(mm3)        ! local NP pole value
      f4 = Sa_in_v(mm4)        ! local NP pole value
      Sa_out(n,kSa_v) = w1*f1 + w2*f2 + w3*f3 + w4*f4

   end do

   call shr_timer_stop(timer_n)

END

!===============================================================================
