c CVS: $Id: ice_ocean.F,v 1.13 2002/04/15 14:24:08 schramm Exp $
c CVS: $Source: /fs/cgd/csm/models/CVS.REPOS/ice/csim4_src/source/ice_ocean.F,v $
c CVS: $Name: ccsm2_0_beta46 $
!
!BOP
!
! !MODULE: ice_ocean - ocean mixed layer internal to sea ice model
!
! !DESCRIPTION:
!
! Ocean mixed layer calculation (internal to sea ice model). \\
! Open ocean has stability based flux calculations for uncoupled
! runs. Uses monthly mean ocean forcing data from external file to 
! drive the ocean mixed layer. Initial sst either from the ice
! restart file or set to 1 Jan from external file data./initial \\
!
! List of ocean forcing fields: Note that order is important! \\
! (order is determined by field list in vname).
! 
! For ocean mixed layer-----------------------------units 
! 
! 1  sst------temperature---------------------------(C)   \\
! 2  sss------salinity------------------------------(ppt) \\
! 3  hbl------depth---------------------------------(m)   \\ 
! 4  u--------surface u current---------------------(m/s) \\
! 5  v--------surface v current---------------------(m/s) \\
! 6  dhdx-----surface tilt x direction--------------(m/m) \\
! 7  dhdy-----surface tilt y direction--------------(m/m) \\
! 8  qdp------ocean sub-mixed layer heat flux-------(W/m2)\\ 
!
! !REVISION HISTORY:
!
! original authors John Weatherly, C.M. Bitz,
!                  Elizabeth C. Hunke, Bruce P. Briegleb
! significantly modified by  Bruce P. Briegleb
!
! !INTERFACE:
!
      module ice_ocean
!
! !USES:
!
      use ice_flux
      use ice_calendar
      use ice_prnpnt
!
!EOP
!
      implicit none

      logical (kind=log_kind) :: 
     &  oceanmixed_ice           ! if true, use ocean mixed layer
     &, oceanmixed_ice_sst_init  ! if true, use ocn frc sst for init
     &, prntdiag_oceanmixed      ! if true, print out some diagnostics

      character(char_len) :: 
     &  oceanmixed_ice_file  ! netCDF file name for ocean forcing data

      integer (kind=int_kind), parameter :: 
     &  nfld = 8    ! number of fields to search for on netCDF file
      character(len=16) ::
     &  vname(nfld) ! variable names to search for on netCDF file
      data vname / 
     &     'T',      'S',      'hblt',  'U',     'V',
     &     'dhdx',   'dhdy',   'qdp' /

      integer (kind=int_kind) ::
     &  fid         ! file id for netCDF routines
     &, dimid       ! dimension id for netCDF file
     &, varid(nfld) ! variable id for field in netCDF file
     &, ntim        ! number of times of data for netCDF file

      real (kind=dbl_kind) :: 
     &  workg(imt_global,jmt_global)       ! work array for global input
     &, ocn_frc(imt_local,jmt_local,nfld)  ! time intrpltd ocn forcing data
     &, ocn_frc_m(ilo:ihi,jlo:jhi,nfld,12) ! ocn mixed layer data for 12 months

      real (kind=dbl_kind) ::
     &  sst_mixedlayer(ilo:ihi,jlo:jhi)    ! save value of sst
     &, frzmlt_mixedlayer(ilo:ihi,jlo:jhi) ! save value of frzmlt
!
      contains
!
!BOP
!
! !IROUTINE: init_oceanmixed_ice - initialize mixed layer forcing data
!
! !INTERFACE:
!
      subroutine init_oceanmixed_ice
!
! !DESCRIPTION:
!
! Initialize ocean forcing data for the mixed layer. Check netCDF \\
! file for correct dimensions, fields and other attributes, and then 
! read in all 12 months of forcing data. \\
!                    
! Assumes 12 months of ocean forcing data, ordered from mean
! January to mean December; dates of these months are ignored. \\
!                    
! Note the assumption that if lat/lon dimensions of the netCDF files
! are identical to those in the code, then the grids of the netCDF 
! file and the code are also.  \\                 
!
! !REVISION HISTORY:
! 
! author  Bruce P. Briegleb, NCAR
!
! !USES:
!
      use ice_mpi_internal
      use shr_sys_mod, only : shr_sys_flush, shr_sys_abort
      include "netcdf.inc"
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      character(len=80) :: 
     &  fileocn  ! netCDF file name for netCDF ice concentration data
     &, title    ! global title for netCDF file

      integer (kind=int_kind) ::
     &  status   ! status variable from netCDF routine 
     &, status1  ! status variable from netCDF routine 
     &, status2  ! status variable from netCDF routine 
     &, nlon     ! number of longitudes of data for netCDF file
     &, nlat     ! number of latitudes  of data for netCDF file
     &, start(3) ! start location for netCDF data reads
     &, count(3) ! number of data items to read in

      integer (kind=int_kind) ::
     &  i        ! longitude index
     &, j        ! latitude  index
     &, n        ! field     index
     &, m        ! month     index

      !-----------------------------------------------------------------
      ! master_task does check of netCDF data
      !-----------------------------------------------------------------

      if (my_task.eq.master_task) then

        !---------------------------------------------------------------
        ! Read in ocean forcing data from an existing local netCDF file
        !---------------------------------------------------------------
        fileocn  = oceanmixed_ice_file
        write (6,*) 'open netCDF file containing ocean forcing data'
        write (6,*) 'ocean mixed layer forcing data file = ',fileocn
        call shr_sys_flush(6)

        status = nf_open(fileocn, NF_NOWRITE, fid)
        if (status /= NF_NOERR) then
          call shr_sys_abort ('ice: no netCDF file with ocn forcing data')
        endif
        write(6,*) 'Successful open of ocean forcing file'
        call shr_sys_flush(6)

        status = nf_get_att_text(fid, NF_GLOBAL, 'title', title)
        if (status /= NF_NOERR) then
          call shr_sys_abort ('ice: no title in netCDF ocean forcing file')
        endif
        write(6,*) 'title =',title 
        call shr_sys_flush(6)

        !---------------------------------------------------------------
        ! Read in dimensions and check
        !---------------------------------------------------------------

        write (6,*) 'Reading dimensions and checking'
        call shr_sys_flush(6)

        status1 = nf_inq_dimid(fid,'time',dimid)
        status2 = nf_inq_dimlen(fid,dimid,ntim)
        if (status1 /= NF_NOERR .or. status2 /= NF_NOERR ) then
          call shr_sys_abort ('ice: no time in netCDF ocean forcing file')
        endif
        write(6,*) 'number of data time values = ',ntim
        if( ntim .le. 11 ) then
          write(6,*) 'ocean forcing file must have 12 months'
          call shr_sys_abort ('ice: ntim < 12 in nc file for ocn frc data')
        endif 

        status1 = nf_inq_dimid(fid,'nlon',dimid)
        status2 = nf_inq_dimlen(fid,dimid,nlon)
        if (status1 /= NF_NOERR .or. status2 /= NF_NOERR ) then
          call shr_sys_abort ('ice: no nlon in netCDF ocean forcing file')
        endif
        write(6,*) 'number of longitude values = ',nlon
        call shr_sys_flush(6)

        status1 = nf_inq_dimid(fid,'nlat',dimid)
        status2 = nf_inq_dimlen(fid,dimid,nlat)
        if (status1 /= NF_NOERR .or. status2 /= NF_NOERR ) then
          call shr_sys_abort ('ice: no nlat in netCDF ocean forcing file')
        endif
        write(6,*) 'number of latitude values  = ',nlat
        call shr_sys_flush(6)

        if( nlon .ne. imt_global ) then
          call shr_sys_abort ('ice: ocn frc file nlon ne imt_global')
        endif
        if( nlat .ne. jmt_global ) then
          call shr_sys_abort ('ice: ocn frc file nlat ne jmt_global')
        endif

        !---------------------------------------------------------------
        ! Check if required fields exist
        !---------------------------------------------------------------

        do n=1,nfld

          status1  = nf_inq_varid(fid,vname(n),varid(n))
          if (status1 /= NF_NOERR ) then
            write(6,*) 'ice error- cannot find field ',vname(n)
            call shr_sys_abort ('ice: cannot find ocn frc field')
          endif
          write (6,*) 'Ocean forcing field found = ',vname(n)
          call shr_sys_flush(6)

        enddo

      endif

      !---------------------------------------------------------------
      ! Read in ocean forcing data for all 12 months
      !---------------------------------------------------------------

      do n=1,nfld

        do m=1,12

          if (my_task.eq.master_task) then

            start(1) = 1
            start(2) = 1
            start(3) = m
            count(1) = imt_global
            count(2) = jmt_global
            count(3) = 1
            !---------------------------------------------------------------
            ! Note: netCDF does single to double conversion if necessary
            !---------------------------------------------------------------
          status2  = nf_get_vara_double(fid,varid(n),start,count,workg)
            if (status2 /= NF_NOERR ) then
              write(6,*) 'ice error- could not read in ocn frc field ',vname(n)
              call shr_sys_abort ('ice: could not read in ocn frc field ')
            endif
            if( prntdiag_oceanmixed ) then
              write(6,*)
     $     'ice- successful read of ocn frc field = ',vname(n),' for month =',m
            endif

          endif

          !-----------------------------------------------------------------
          ! Scatter to all processors
          !-----------------------------------------------------------------
 
          call global_scatter(workg,ocn_frc_m(ilo,jlo,n,m)) 

        enddo               ! month loop

      enddo               ! field loop


      !-----------------------------------------------------------------
      ! Set initial sst to forcing data (~1Jan) if desired
      !-----------------------------------------------------------------

      if( oceanmixed_ice_sst_init ) then
        do j=jlo,jhi
          do i=ilo,ihi
            sst_mixedlayer(i,j) = 0.5_dbl_kind*ocn_frc_m(i,j,1,1) +
     $                            0.5_dbl_kind*ocn_frc_m(i,j,1,12) 
          enddo
        enddo
        if (my_task.eq.master_task) then
          write (6,*) 'Initial ocean sst set to 1 Jan from forcing data'
          call shr_sys_flush(6)
        endif
      endif

      if (my_task.eq.master_task) then
        write (6,*) 'Ocean forcing data successfully read'
        call shr_sys_flush(6)
      endif

      end subroutine init_oceanmixed_ice
!
!BOP
!
! !IROUTINE: time_intrplt_ocean_forcing - time interpolates ocean forcing
!
! !INTERFACE:
!
      subroutine time_intrplt_ocean_forcing
!
! !DESCRIPTION:
!
! Finds two months of ocean forcing data that bracket current date
! (for which year is ignored), and interpolate linearly in time 
! between those two values. 
!
! !REVISION HISTORY:
!                                       
! author  Bruce P. Briegleb, NCAR
!
! !USES:
!
      use ice_mpi_internal
      use ice_grid
      use shr_sys_mod, only : shr_sys_flush, shr_sys_abort
      include "netcdf.inc"
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      integer (kind=int_kind) ::
     &  n        ! field index
     &, m        ! month index
     &, m1       ! month index for first  month
     &, m2       ! month index for second month
     &, i        ! longitude index
     &, j        ! latitude  index
     &, month_in ! integer month number for input
     &, day_in   ! integer day number for input

      integer (kind=int_kind) ::
     &  date1       ! encoded date (YYYYMMDD) for first  month data
     &, date2       ! encoded date (YYYYMMDD) for second month data

      integer (kind=int_kind), dimension(13) ::
     &  calendar = (/0,31,59,90,120,151,181,212,243,273,304,334,365/)

      real (kind=dbl_kind) ::
     &  f1      ! fraction used in time interpolation 
     &, f2      ! fraction used in time interpolation
     &, ad_1    ! integer day number since 0 AD for first time value
     &, ad_in   ! integer day number since 0 AD for input
     &, ad_2    ! integer day number since 0 AD for second time value

      !-----------------------------------------------------------------
      ! Find months bracketing current idate (without year) 
      ! for time interpolation, and set dates in relative days
      !-----------------------------------------------------------------

      month_in = int( mod(idate,10000)/100 )
      day_in   = mod( idate, 100 )
      ad_in    = real(365 + calendar(month_in) + day_in)

      m1 = 0
      m2 = 0

      if( real(calendar(month_in)+day_in) .lt. 
     $    real(calendar(1)+16) ) then

        m1    = 12
        date1 =      calendar(12)  + 16

        m2    =  1 
        date2 = 365 + calendar(1)  + 16
        
      else if( real(calendar(month_in)+day_in) .gt. 
     $         real(calendar(12)+16) ) then

        m1    = 12
        date1 = 365 + calendar(12)  + 16

        m2    =  1 
        date2 = 730 + calendar(1)  + 16
        
      else 

        do m=2,12
     
          if( real(calendar(m-1)+16) .le.
     $        real(calendar(month_in)+day_in) .and. 
     $        real(calendar(month_in)+day_in) .le. 
     $        real(calendar(m)+16) ) then

            m1    = m-1
            date1 = 365 + calendar(m-1) + 16

            m2    = m
            date2 = 365 + calendar(m)   + 16

          endif

        enddo

      endif

      !-----------------------------------------------------------------
      ! error exit
      !-----------------------------------------------------------------

      if( m1 .eq. 0 .or. m2 .eq. 0 ) then
        write(6,*) ' error in time_intrplt_ocean_forcing'
        call shr_sys_abort ('ice: month indices m1 and m2 not found ')
      endif

      !-----------------------------------------------------------------
      ! Time interpolate ocean forcing data
      !-----------------------------------------------------------------

      ad_1     = real(date1)
      ad_2     = real(date2)

      f1 = (ad_2  - ad_in) / (ad_2 - ad_1)
      f2 = (ad_in - ad_1 ) / (ad_2 - ad_1)

      do n=1,nfld
        do j = jlo,jhi
          do i = ilo,ihi
            ocn_frc(i,j,n) = c1
            if (tmask(i,j)) then
              ocn_frc(i,j,n) = f1*ocn_frc_m(i,j,n,m1) + 
     &                         f2*ocn_frc_m(i,j,n,m2)
            endif
          enddo
        enddo 
      enddo        ! field loop 

      if (my_task.eq.master_task) then
        if( prntdiag_oceanmixed ) then
          write (6,*) 'time_intrplt data  time idate date1 date2 f1 f2'
          write (6,*) time,idate,date1,date2,f1,f2
          call shr_sys_flush(6)
        endif
      endif

      end subroutine time_intrplt_ocean_forcing
!
!BOP
!
! !IROUTINE: set_oceanmixed_ice - set sst and frzmlt
!
! !INTERFACE:
!
      subroutine set_oceanmixed_ice
!
! !DESCRIPTION:
!
! Sets sst and frzmlt with ocean mixed layer values
!
! !REVISION HISTORY:
! 
! author  Bruce P. Briegleb, NCAR
!
! !USES:
!
      use shr_sys_mod, only : shr_sys_flush
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      integer (kind=int_kind) ::
     &  i        ! longitude index
     &, j        ! latitude  index

       do j=jlo,jhi
         do i=ilo,ihi
           sst(i,j)    = sst_mixedlayer(i,j)
           frzmlt(i,j) = frzmlt_mixedlayer(i,j)
         enddo
       enddo

      if (my_task.eq.master_task) then
        if( prntdiag_oceanmixed ) then
          write (6,*) 'Ocean sst and frzmlt data set'
          call shr_sys_flush(6)  
        endif
      endif

      end subroutine set_oceanmixed_ice
!
!BOP
!
! !IROUTINE: compute_oceanmixed_ice - update ocean state fields
!
! !INTERFACE:
!
      subroutine compute_oceanmixed_ice
!
! !DESCRIPTION:
!
! Update sst and other ocean state fields
!
! !REVISION HISTORY:
! 
! author  Bruce P. Briegleb, NCAR
!
! !USES:
!
      use shr_sys_mod, only : shr_sys_flush
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      integer (kind=int_kind) ::
     &  i        ! longitude index
     &, j        ! latitude  index

       do j=jlo,jhi
         do i=ilo,ihi
           call mixed_layer(i,j)
         enddo
       enddo

      if (my_task.eq.master_task) then
        if( prntdiag_oceanmixed ) then
          write (6,*) 'Ocean mixed layer updated'
          call shr_sys_flush(6)
        endif 
      endif

      end subroutine compute_oceanmixed_ice
!
!BOP
!
! !IROUTINE: mixed_layer(i,j) - computes open ocean fluxes
!
! !INTERFACE:
!
      subroutine mixed_layer(i,j)
!
! !DESCRIPTION:
!
! Calculate flux exchange over open ocean and update mixed layer sst
!
! !REVISION HISTORY:
!                                       
! author  Elizabeth Hunke, LANL, and Bruce P. Briegleb, NCAR
!
! !USES:
!
      use ice_grid
      use ice_atmo
      use ice_constants, only : TTTocn, qqqocn
!
! !INPUT/OUTPUT PARAMETERS:
!
      integer (kind=int_kind), intent(in) :: i,j
!
!EOP
!
      real (kind=dbl_kind) ::
     &   delt   ! potential T difference   (K)
     &,  delq   ! humidity difference      (kg/kg)
     &,  rdn_in ! initial value for rdn in stability routine
     &,  vmag   ! surface wind magnitude   (m/s)
     &,  strx   ! dummy argument
     &,  stry   ! dummy argument
     &,  Trf    ! dummy argument
     &,  dssqdt ! dummy argument

      real (kind=dbl_kind), parameter ::
     &   umin =  c1          ! minimum wind speed (m/s)

      if (tmask(i,j)) then

         ! set mixed layer temperature

         sst(i,j) = sst_mixedlayer(i,j)

         vmag   = max(umin, wind(i,j))      ! CSM Ocean wind-speed formula 
         rdn_in = sqrt(0.0027_dbl_kind/vmag 
     &     + .000142_dbl_kind + .0000764_dbl_kind*vmag) ! for rdn_in

         qqq=qqqocn
         TTT=TTTocn
         call stability( i,j,1, sst(i,j), rdn_in,
     $         strx   ,stry   ,Trf, dssqdt,  delt, delq)

         call ocnheat(i,j, delt, delq)  ! update sst, determine frzmlt

      endif

      end subroutine mixed_layer
!
!BOP
!
! !IROUTINE: ocnheat(i,j, delt, delq) - update ocean fields 
!
! !INTERFACE:
!
      subroutine ocnheat(i,j, delt, delq)
!
! !DESCRIPTION:
!
! Update sst and freezing/melting potential; set other ocean fields 
! from forcing data; thermal effect of ice/ocn heat will be included
! in the sst change after this routine is called.
! 
! !REVISION HISTORY:
!
! Modified by Bruce P. Briegleb
!
! !USES:
!
      use ice_constants
      use ice_state
      use ice_albedo
!
! !INPUT/OUTPUT PARAMETERS:
!
      integer (kind=int_kind) :: i,j

      real (kind=dbl_kind), intent(in) ::
     &   delt   ! potential T difference   (K)
     &,  delq   ! humidity difference      (kg/kg)
!
!EOP
!
      integer (kind=int_kind) :: n
      real (kind=dbl_kind) ::
     &   sst_in    ! ocn frc data time interpolated sst
     &,  sss_in    ! ocn frc data time interpolated sss
     &,  hbl_in    ! ocn frc data time interpolated hbl
     &,  u_in      ! ocn frc data time interpolated u
     &,  v_in      ! ocn frc data time interpolated v
     &,  dhdx_in   ! ocn frc data time interpolated dhdx
     &,  dhdy_in   ! ocn frc data time interpolated dhdy
     &,  qdp_in    ! ocn frc data time interpolated qdp

      real (kind=dbl_kind) ::
     &   TsfK    ! surface temperature in Kelvin       (K)
     &,  hbl     ! depth of mixed or boundary layer    (m)
     &,  fsh     ! sensible         heat flux        (W/m**2)
     &,  flh     ! latent           heat flux        (W/m**2)
     &,  fswabs  ! srfc absrbd short-wave heat flux  (W/m**2)
     &,  flwdn   ! long-wave downward heat flux      (W/m**2)
     &,  flwup   ! long-wave upward heat flux        (W/m**2)

      real (kind=dbl_kind) ::
     &   ft     ! fraction reduction of positive qdp
     &,  qdp    ! deep ocean heat flux               (W/m**2)

      ! set ocean forcing data

      sst_in    = ocn_frc(i,j,1)    !  sst
      sss_in    = ocn_frc(i,j,2)    !  sss
      hbl_in    = ocn_frc(i,j,3)    !  hbl
      u_in      = ocn_frc(i,j,4)    !  u
      v_in      = ocn_frc(i,j,5)    !  v
      dhdx_in   = ocn_frc(i,j,6)    !  dhdx
      dhdy_in   = ocn_frc(i,j,7)    !  dhdy
      qdp_in    = ocn_frc(i,j,8)    !  qdp

      ! set specific local values

      hbl       = hbl_in
      qdp       = qdp_in

      ! print out diagnostics if desired

      if(print_points) then
       do n=1,npnt
        if( my_task .eq. nmbtsk(n) ) then
         if( j .eq. jindx(n) ) then
           if( i .eq. iindx(n) ) then
             iunit = untout(n)
             write(iunit,100) idate,aice(i,j),hbl,sst(i,j),sss_in,
     $                u_in,v_in,qdp_in,dhdx_in,dhdy_in
 100                  format(' ocn  in',1x,i6,1x,f5.4,1x,f5.1,1x,
     $                f6.3,1x,f5.2,2(f6.3,1x),f6.1,1x,2(1pe11.4,1x))
             call shr_sys_flush(iunit)
           endif
         endif
        endif
       enddo
      endif

      ! ocean surface temperature in Kelvin
      TsfK = sst(i,j)+Tffresh

      ! shortwave radiative flux absorbed
      fswabs = (c1 - albocn) * Fsw(i,j)

      ! longwave radiative fluxes
      flwdn  = Flw(i,j)
      flwup  = -stefan_boltzmann * TsfK**4

      ! downward latent and sensible heat fluxes
      flh = lhcoef(i,j,1) * delq  ! using lhcoef(i,j,1), shcoef(i,j,1)
      fsh = shcoef(i,j,1) * delt  ! as temporary arrays for ocean

      ! first, compute sst change due to exchange with atm above
      ! over open water:
      sst(i,j) = sst(i,j) +((fsh + flh + flwup + flwdn + fswabs)
     &            *(c1-aice(i,j)))*dt/(cp_ocn*rhow*hbl)

      ! adjust qdp if cooling of mixed layer will occur when sst le Tf;
      ! Reduce qdp > 0 down to zero
      if( sst(i,j) .le. Tf(i,j) .and. qdp .gt. 0. ) then
        qdp = c0
      endif

      ! compute T change due to exchange with deep layers:
      sst(i,j) = sst(i,j) - qdp*dt/(cp_ocn*rhow*hbl)

      ! compute T change due to heat used by ice model:
      sst(i,j) = sst(i,j) + aice(i,j)*Fhnet(i,j)*dt/(cp_ocn*rhow*hbl)

      ! compute potential to freeze or melt ice
      frzmlt(i,j) = (Tf(i,j)-sst(i,j))*(cp_ocn*rhow*hbl)/dt          

      ! if sst is below freezing, reset sst to Tf
      if(sst(i,j) .le. Tf(i,j)) sst(i,j) = Tf(i,j)

      ! save mixed layer sst and frzmlt
      sst_mixedlayer(i,j)    = sst(i,j)
      frzmlt_mixedlayer(i,j) = frzmlt(i,j)

      ! set other ocean fields with forcing data

      sss(i,j)      =  sss_in
      uocn(i,j)     =  u_in
      vocn(i,j)     =  v_in
      strtltx(i,j)  =  dhdx_in
      strtlty(i,j)  =  dhdy_in

      ! print out diagnostics if desired

      if(print_points) then
       do n=1,npnt
        if( my_task .eq. nmbtsk(n) ) then
         if( j .eq. jindx(n) ) then
           if( i .eq. iindx(n) ) then
             iunit = untout(n)
             write(iunit,200) idate,hbl,sst(i,j),Fsw(i,j),fswabs,
     $                  flwdn,flwup,flh,fsh,qdp,frzmlt(i,j)
 200                    format(' ocn out',1x,i6,1x,f5.1,1x,f6.3,
     $                  1x,7(f6.1,1x),1pe11.4)
             call shr_sys_flush(iunit)
           endif
         endif
        endif
       enddo
      endif

      end subroutine ocnheat
!
      end module ice_ocean
!
