c CVS: $Id: ice_prescribed.F,v 1.16 2002/04/15 14:24:08 schramm Exp $
c CVS: $Source: /fs/cgd/csm/models/CVS.REPOS/ice/csim4_src/source/ice_prescribed.F,v $
c CVS: $Name: ccsm2_0_beta46 $
!
!BOP
!
! !MODULE: ice_prescribed - Prescribed Ice
!
! !DESCRIPTION:
!
! Prescribed ice.  \\
!   
! Set ice concentration with prescribed data from netCDF data file; 
! then internally prescribe ice thickness, temperature, snow cover, 
! layer linear temperature profile; finally, compute air/ice fluxes,
! but no other ice thermo or dynamics; set ice/ocean fluxes to zero. 
!
! !REVISION HISTORY:
!
! author: Bruce P. Briegleb, NCAR
!
! !INTERFACE:
!
      module ice_prescribed
!
! !USES:
!
      use ice_model_size
      use ice_constants
      use ice_domain
      use ice_grid
      use ice_calendar
!
!EOP
!
      implicit none

      logical (kind=log_kind) :: prescribed_ice             ! if true, prescribed ice
      logical (kind=log_kind) :: prescribed_ice_climatology ! if true, use climate prescribed ice

      character(char_len_long) :: 
     &  prescribed_ice_file ! netCDF file name for prescribed ice

      integer (kind=int_kind) ::
     &  tmindex     ! time index for first month ice cov netCDF data
     &, date1       ! encoded date (YYYYMMDD) for first  month data
     &, date2       ! encoded date (YYYYMMDD) for second month data
     &, date3       ! encoded date (YYYYMMDD) for third month data
     &, newdata     ! if 1, new data read in needing scattering

      integer (kind=int_kind) ::
     &  fid      ! file id for netCDF routines
     &, dateid   ! variable id for date in netCDF file
     &, varid    ! variable id for field in netCDF file
     &, ntim     ! number of times of data for netCDF file

      real (kind=dbl_kind) :: 
     &  ice_cov(imt_local,jmt_local)  ! time interpolated ice concentration
     &, ice_cov_m1_imt(imt_local,jmt_local) 
     &, ice_cov_m2_imt(imt_local,jmt_local) 
     &, ice_cov_m(ilo:ihi,jlo:jhi,12) ! ice concentration for all 12 climatology months

      real (kind=dbl_kind) :: 
     &  workg1(imt_global,jmt_global)    ! work array for global input
     &, workg2(imt_global,jmt_global)    ! work array for global input
     &, workg3(imt_global,jmt_global)    ! work array for global input

!
      contains
!
!BOP
!
! !IROUTINE: init_prescribed_ice - initialize prescribed ice data
!
! !INTERFACE:
!
      subroutine init_prescribed_ice
!
! !DESCRIPTION:
!
! Initialize climatological ice concentration data by checking 
! existing netCDF file for correct dimensions, fields and other 
! attributes, and then reading in first 12 months of ice cover data
! for climatology case and 2 months (12 and 13) for non-climatology. \\
!                    
! Note the assumption that if lat/lon dimensions of the netCDF files
! are identical to those in the code, then the grids of the netCDF
! file and the code are also.                   
!
! !REVISION HISTORY:
!
! author:  Bruce P. Briegleb, NCAR
!
! !USES:
!
      use ice_mpi_internal
      use shr_sys_mod, only : shr_sys_flush, shr_sys_abort
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      include "netcdf.inc"

      character(char_len_long) :: 
     &  title    ! global title for netCDF file

      integer (kind=int_kind) ::
     &  status   ! status variable from netCDF routine 
     &, status1  ! status variable from netCDF routine 
     &, status2  ! status variable from netCDF routine 
     &, nlon     ! number of longitudes of data for netCDF file
     &, nlat     ! number of latitudes  of data for netCDF file
     &, start(3) ! start location for netCDF data reads
     &, count(3) ! number of data items to read in

      integer (kind=int_kind) ::
     &  i        ! longitude index
     &, j        ! latitude  index
     &, m        ! month  index
     &, n_months ! number of months to read in initially from file
     &, offset   ! difference between first month of file and that read in

      real (kind=dbl_kind) :: 
     &  areanh   ! nh ice area for printout
     &, areash   ! sh ice area for printout

      !-----------------------------------------------------------------
      ! master_task does check of netCDF data
      !-----------------------------------------------------------------
      if (.not.prescribed_ice_climatology) then
        n_months = 3
        offset = 11
      else
        n_months = 12
        offset = 0
      endif

      if (my_task.eq.master_task) then

        !---------------------------------------------------------------
        ! Read in ice coverage data from an existing local netCDF file
        !---------------------------------------------------------------
        write (6,*) 'open netCDF file containing ice_cov data'
        write (6,*) 'sea ice coverage file = ',prescribed_ice_file
        call shr_sys_flush(6)

        status = nf_open(prescribed_ice_file, NF_NOWRITE, fid)
        if (status /= NF_NOERR) then
          call shr_sys_abort ('ice: cannot find nc file with ice_cov data')
        endif
        write(6,*) 'Successful open of sea ice concentration file'
        call shr_sys_flush(6)

        status = nf_get_att_text(fid, NF_GLOBAL, 'title', title)
        if (status == NF_NOERR) then
          write(6,*) 'title =',trim(title)
          call shr_sys_flush(6)
        endif

        !---------------------------------------------------------------
        ! Read in dimensions and check
        !---------------------------------------------------------------

        write (6,*) 'Reading dimensions and checking'
        call shr_sys_flush(6)

        status1 = nf_inq_dimid(fid,'time',varid)
        status2 = nf_inq_dimlen(fid,varid,ntim)
        if (status1 /= NF_NOERR .or. status2 /= NF_NOERR ) then
          call shr_sys_abort ('ice: cannot find time in netCDF file')
        endif
        write(6,*) 'number of data time values = ',ntim
        if( ntim .le. 11 ) then
          write(6,*) 'climatological ice must have 12 months'
          call shr_sys_abort ('ice: ntim < 12 in nc file for climate data')
        endif 

        status1 = nf_inq_dimid(fid,'nlon',varid)
        status2 = nf_inq_dimlen(fid,varid,nlon)
        if (status1 /= NF_NOERR .or. status2 /= NF_NOERR ) then
          call shr_sys_abort ('ice: cannot find nlon in netCDF file')
        endif
        write(6,*) 'number of longitude values = ',nlon
        call shr_sys_flush(6)

        status1 = nf_inq_dimid(fid,'nlat',varid)
        status2 = nf_inq_dimlen(fid,varid,nlat)
        if (status1 /= NF_NOERR .or. status2 /= NF_NOERR ) then
          call shr_sys_abort ('ice: cannot find nlat in netCDF file')
        endif
        write(6,*) 'number of latitude values = ',nlat
        call shr_sys_flush(6)

        if( nlon .ne. imt_global ) then
          call shr_sys_abort ('ice: nlon ne imt_global')
        endif
        if( nlat .ne. jmt_global ) then
          write(6,*) 'ice error- nlat ne jmt_global'
          call shr_sys_abort ('ice: nlat ne jmt_global')
        endif

        !---------------------------------------------------------------
        ! Find and read in first two dates:
        !---------------------------------------------------------------

        if (.not.prescribed_ice_climatology) then
          write (6,*) 'Reading in first date'
          start(1)  = offset+1       ! Read in Dec of first year
          count(1)  = 1
          status1 = nf_inq_varid(fid,'date',dateid)
          status2 = nf_get_vara_int(fid,dateid,start,count,date1)
          if (status1 /= NF_NOERR .or. status2 /= NF_NOERR ) then
            call shr_sys_abort ('ice: cannot find date in netCDF file')
          endif
          write(6,*) 'first month date  = ',date1

          write (6,*) 'Reading in second date'
          start(1)  = offset+2      ! Read in Jan of second year
          count(1)  = 1
          status2 = nf_get_vara_int(fid,dateid,start,count,date2)
          write(6,*) 'second month date = ',date2
          call shr_sys_flush(6)

          write (6,*) 'Reading in third date'
          start(1)  = offset+3      ! Read in Feb of second year
          count(1)  = 1
          status2 = nf_get_vara_int(fid,dateid,start,count,date3)
          write(6,*) 'third month date = ',date3
          call shr_sys_flush(6)
        endif

        !---------------------------------------------------------------
        ! Check if ice concentration data exists:
        !---------------------------------------------------------------

        status1  = nf_inq_varid(fid,'ice_cov',varid)
        if (status1 /= NF_NOERR ) then
          write(6,*) 'ice error- cannot find ice_cov field '
          call shr_sys_abort ('ice: cannot find ice_cov field')
        endif
        write (6,*) 'Sea ice coverage field found'
        call shr_sys_flush(6)

      endif

      !---------------------------------------------------------------
      ! Read in ice coverage data for n_months
      !---------------------------------------------------------------

      do m=1,n_months

        if (my_task.eq.master_task) then

          start(1) = 1
          start(2) = 1
          start(3) = m + offset
          count(1) = imt_global
          count(2) = jmt_global
          count(3) = 1
          !---------------------------------------------------------------
          ! Note: netCDF does single to double conversion if necessary
          !---------------------------------------------------------------
          status2  = nf_get_vara_double(fid,varid,start,count,workg1)
          if (status2 /= NF_NOERR ) then
            call shr_sys_abort ('ice: could not read in ice_cov field')
          endif
          write(6,*)
     $    'ice- successful read of ice coverage data for month =',start(3)

        endif

        !-----------------------------------------------------------------
        ! Scatter to all processors
        !-----------------------------------------------------------------
 
        call global_scatter(workg1,ice_cov_m(ilo:ihi,jlo:jhi,m)) 

      enddo

      !---------------------------------------------------------------
      ! Set time index to indicate first of two dates read in
      !---------------------------------------------------------------

      if (my_task.eq.master_task.and.(.not.prescribed_ice_climatology))
     $          tmindex=start(3)

      !-----------------------------------------------------------------
      ! Set ice_cov over land to zero 
      !-----------------------------------------------------------------

      do m=1,n_months
        do j = jlo,jhi
          do i = ilo,ihi
            if (.not. tmask(i,j)) then
              ice_cov_m(i,j,m) = c0
            endif
          enddo
        enddo 
      enddo

      !-----------------------------------------------------------------
      ! Compute hemispheric ice areas and print out
      !-----------------------------------------------------------------

      do m=1,n_months

        ice_cov_m1_imt(ilo:ihi,jlo:jhi) = ice_cov_m(ilo:ihi,jlo:jhi,m)
        call bound(ice_cov_m1_imt)

        call get_sum(0,tarean,one,ice_cov_m1_imt,areanh)
        call get_sum(0,tareas,one,ice_cov_m1_imt,areash)

        if (my_task.eq.master_task) then

          areanh = areanh / 1.0e12 
          areash = areash / 1.0e12 

          write(6,*) 
     $    ' month  nh,sh areas (10**6 km**2) = ',m,areanh,areash

        endif

      enddo

      end subroutine init_prescribed_ice
!
!BOP
!
! !IROUTINE: read_prescribed_ice - read in two ice concentration fields
!
! !INTERFACE:
!
      subroutine read_prescribed_ice
!
! !DESCRIPTION:
!
! Read in two ice concentration fields that bracket current date,
! and interpolate linearlly in time between those two values. Make
! sure final ice concentration exceeds a small constant value. \\
!                    
! No time extrapolation allowed; if date out of range, exit.                   
! Data may have values < 0 or > 1; limit 1.0e-4 to 1 after time
! interpolation.
!
! !REVISION HISTORY:
!                    
! author:  Bruce P. Briegleb, NCAR
!
! !USES:
!
      use ice_mpi_internal
      use shr_sys_mod, only : shr_sys_flush, shr_sys_abort
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      include "netcdf.inc"

      integer (kind=int_kind) ::
     &  n        ! time index
     &, i        ! longitude index
     &, j        ! latitude  index
     &, status   ! status variable from netCDF routine 
     &, start(3) ! start location for netCDF data reads
     &, count(3) ! number of data items to read in
     &, year_1   ! integer year number for first time value
     &, year_in  ! integer year number for input
     &, year_2   ! integer year number for second time value
     &, month_1  ! integer month number for first time value
     &, month_in ! integer month number for input
     &, month_2  ! integer month number for second time value
     &, day_1    ! integer day number for first time value
     &, day_in   ! integer day number for input
     &, day_2    ! integer day number for second time value

      integer (kind=int_kind), dimension(12) ::
     &  calendar = (/0,31,59,90,120,151,181,212,243,273,304,334/)

      real (kind=dbl_kind) ::
     &  f1      ! fraction used in time interpolation 
     &, f2      ! fraction used in time interpolation
     &, ad_1    ! integer day number since 0 AD for first time value
     &, ad_in   ! integer day number since 0 AD for input
     &, ad_2    ! integer day number since 0 AD for second time value
     &, areanh  ! nh ice area for printout
     &, areash  ! sh ice area for printout
     &, sump    ! temporary sum for setting pmask

      !-----------------------------------------------------------------
      ! Check current model date against dates of data
      !-----------------------------------------------------------------

      if (my_task.eq.master_task) then

        newdata = 0
     
        if( idate .lt. date1 ) then

          write(6,*) 'idate = ',idate,' smallest date = ',date1
          write(6,*) 'no time extrapolation allowed'
          call shr_sys_abort ('ice: idate < smallest data date')

        else if( idate .gt. date2 ) then

          !-------------------------------------------------------------
          ! Try to find date1,date2 that bracket idate
          !-------------------------------------------------------------

          do n=tmindex+1,ntim
            date1     = date2
            start(1)  = n
            count(1)  = 1
            status = nf_get_vara_int(fid,dateid,start,count,date2)
            if (status /= NF_NOERR ) then
              call shr_sys_abort ('could not read date in prescribed_ice')
            endif
            if( date1 .le. idate .and. idate .le. date2 ) goto 100
          enddo
          if( idate .gt. date2 ) then

            write(6,*) 'idate = ',idate,' largest date = ',date2
            write(6,*) 'no time extrapolation allowed'
            call shr_sys_abort ('ice: idate > largest data date')

          endif
 100      tmindex = n-1
          ! get date 3

          start(1)  = n+1
          count(1)  = 1
          status = nf_get_vara_int(fid,dateid,start,count,date3)
          if (status /= NF_NOERR ) then
            call shr_sys_abort ('could not read date in prescribed_ice')
          endif

          !-------------------------------------------------------------
          ! Read in data for date1 and date2 that bracket idate
          !      plus data for date3
          !-------------------------------------------------------------

          start(1) = 1
          start(2) = 1
          start(3) = tmindex
          count(1) = imt_global
          count(2) = jmt_global
          count(3) = 1
          !-------------------------------------------------------------
          ! Note: netCDF does single to double conversion if necessary
          !-------------------------------------------------------------
          status   = nf_get_vara_double(fid,varid,start,count,workg1)
          if (status /= NF_NOERR ) then
            call shr_sys_abort ('ice: could not read ice coverage data')
          endif
          write(6,*) 
     $    'ice- successful read of ice_cov data for date = ',date1

          start(1) = 1
          start(2) = 1
          start(3) = tmindex + 1
          count(1) = imt_global
          count(2) = jmt_global
          count(3) = 1
          !-------------------------------------------------------------
          ! Note: netCDF does single to double conversion if necessary
          !-------------------------------------------------------------
          status   = nf_get_vara_double(fid,varid,start,count,workg2)
          if (status /= NF_NOERR ) then
            write(6,*) 'ice error- could not read ice coverage data '
            call shr_sys_abort ('ice: could not read ice coverage data')
          endif
          write(6,*) 
     $    'ice- successful read of ice_cov data for date = ',date2

          start(1) = 1
          start(2) = 1
          start(3) = tmindex + 2
          count(1) = imt_global
          count(2) = jmt_global
          count(3) = 1
          !-------------------------------------------------------------
          ! Note: netCDF does single to double conversion if necessary
          !-------------------------------------------------------------
          status   = nf_get_vara_double(fid,varid,start,count,workg3)
          if (status /= NF_NOERR ) then
            write(6,*) 'ice error- could not read ice coverage data '
            call shr_sys_abort ('ice: could not read ice coverage data')
          endif
          write(6,*) 
     $    'ice- successful read of ice_cov data for date = ',date3

          newdata = 1

        endif

      endif

      !-----------------------------------------------------------------
      ! Scatter scalars for new data scattering and time interpolation
      !-----------------------------------------------------------------
      call ice_bcast_iscalar(newdata)
      call ice_bcast_iscalar(date1)
      call ice_bcast_iscalar(date2)
      call ice_bcast_iscalar(date3)

      !-----------------------------------------------------------------
      ! If new data read in, scatter it
      !-----------------------------------------------------------------

      if( newdata .eq. 1 ) then

        call global_scatter(workg1,ice_cov_m(ilo:ihi,jlo:jhi,1)) 
        call global_scatter(workg2,ice_cov_m(ilo:ihi,jlo:jhi,2))
        call global_scatter(workg3,ice_cov_m(ilo:ihi,jlo:jhi,3))

        !---------------------------------------------------------------
        ! Set ice_cov over land to zero 
        !---------------------------------------------------------------

        do j = jlo,jhi
          do i = ilo,ihi
            if (.not. tmask(i,j)) then
              ice_cov_m(i,j,1) = c0
              ice_cov_m(i,j,2) = c0
              ice_cov_m(i,j,3) = c0
            endif
          enddo
        enddo 

        !---------------------------------------------------------------
        ! Compute hemispheric ice areas and print out
        !---------------------------------------------------------------

        ice_cov_m1_imt(ilo:ihi,jlo:jhi) = ice_cov_m(ilo:ihi,jlo:jhi,1)
        ice_cov_m2_imt(ilo:ihi,jlo:jhi) = ice_cov_m(ilo:ihi,jlo:jhi,2)
        call bound(ice_cov_m1_imt)
        call bound(ice_cov_m2_imt)

        call get_sum(0,tarean,one,ice_cov_m1_imt,areanh)
        call get_sum(0,tareas,one,ice_cov_m1_imt,areash)

        if (my_task.eq.master_task) then

          areanh = areanh / 1.0e12 
          areash = areash / 1.0e12 

          write(6,*) ' date1  nh area (10**6 km**2) = ',date1,areanh
          write(6,*) ' date1  sh area (10**6 km**2) = ',date1,areash

        endif

        call get_sum(0,tarean,one,ice_cov_m2_imt,areanh)
        call get_sum(0,tareas,one,ice_cov_m2_imt,areash)

        if (my_task.eq.master_task) then

          areanh = areanh / 1.0e12 
          areash = areash / 1.0e12 

          write(6,*) ' date2  nh area (10**6 km**2) = ',date2,areanh
          write(6,*) ' date2  sh area (10**6 km**2) = ',date2,areash

        endif

      endif 

      !-----------------------------------------------------------------
      ! Time interpolate available ice concentration data
      !-----------------------------------------------------------------

      year_1   = int( date1/10000 )
      month_1  = int( mod(date1,10000)/ 100)
      day_1    = mod( date1, 100 )
      ad_1     = real(year_1*365 + calendar(month_1) + day_1)

      year_in  = int( idate/10000 )
      month_in = int( mod(idate,10000)/ 100)
      day_in   = mod( idate, 100 )
      ad_in    = real(year_in*365 + calendar(month_in) + day_in)

      year_2   = int( date2/10000 )
      month_2  = int( mod(date2,10000)/ 100)
      day_2    = mod( date2, 100 )
      ad_2     = real(year_2*365 + calendar(month_2) + day_2)

      f1 = (ad_2  - ad_in) / (ad_2 - ad_1)
      f2 = (ad_in - ad_1 ) / (ad_2 - ad_1)

      do j = jlo,jhi
        do i = ilo,ihi
          ice_cov(i,j) = c0
          pmask(i,j) = .false.
          if (tmask(i,j)) then
            ice_cov(i,j) = f1*ice_cov_m(i,j,1) + f2*ice_cov_m(i,j,2)
            if( ice_cov(i,j) .lt. eps04 ) ice_cov(i,j) = c0
            if( ice_cov(i,j) .gt. c1)     ice_cov(i,j) = c1
            if (ice_cov_m(i,j,1) .gt. c0) pmask(i,j) = .true.
            if (ice_cov_m(i,j,2) .gt. c0) pmask(i,j) = .true.
            if (ice_cov_m(i,j,3) .gt. c0) pmask(i,j) = .true.
          endif
        enddo
      enddo 

      end subroutine read_prescribed_ice
!
!BOP
!
! !IROUTINE: read_prescribed_ice_climate - read two climate ice concentrations
!
! !INTERFACE:
!
      subroutine read_prescribed_ice_climate
!
! !DESCRIPTION:
!
! Finds two months of climatological ice concentration that bracket 
! current date (for which year is ignored), and interpolate linearlly 
! in time between those two values. Final time inerpolated ice 
! concentration is limited to the range 1.0e-4 to 1.
!
! !REVISION HISTORY:
!                                       
! author:  Bruce P. Briegleb, NCAR
!
! !USES:
!
      use ice_mpi_internal
      use shr_sys_mod, only : shr_sys_flush, shr_sys_abort
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      include "netcdf.inc"

      integer (kind=int_kind) ::
     &  m        ! month index
     &, m1       ! month index for first  month
     &, m2       ! month index for second month
     &, m3       ! month index for third month
     &, i        ! longitude index
     &, j        ! latitude  index
     &, month_in ! integer month number for input
     &, day_in   ! integer day number for input

      integer (kind=int_kind), dimension(13) ::
     &  calendar = (/0,31,59,90,120,151,181,212,243,273,304,334,365/)

      real (kind=dbl_kind) ::
     &  f1      ! fraction used in time interpolation 
     &, f2      ! fraction used in time interpolation
     &, ad_1    ! integer day number since 0 AD for first time value
     &, ad_in   ! integer day number since 0 AD for input
     &, ad_2    ! integer day number since 0 AD for second time value

      !-----------------------------------------------------------------
      ! Find months bracketing current idate (without year) 
      ! for time interpolation, and set dates in relative days
      !-----------------------------------------------------------------

      month_in = int( mod(idate,10000)/100 )
      day_in   = mod( idate, 100 )
      ad_in    = real(       365 + calendar(month_in) + day_in)

      m1 = 0
      m2 = 0

      if( real(calendar(month_in)+day_in) .lt. 
     $    real(calendar(1)+16) .or.
     $    real(calendar(month_in)+day_in) .gt. 
     $    real(calendar(12)+16) ) then

        m1    = 12
        date1 =      calendar(12)  + 16

        m2    =  1 
        date2 = 365 + calendar(1)  + 16
        
      else 

        do m=2,12
     
          if( real(calendar(m-1)+16) .le.
     $        real(calendar(month_in)+day_in) .and. 
     $        real(calendar(month_in)+day_in) .le. 
     $        real(calendar(m)+16) ) then

            m1    = m-1
            date1 = 365 + calendar(m-1) + 16

            m2    = m
            date2 = 365 + calendar(m)   + 16

          endif

        enddo

      endif

      !-----------------------------------------------------------------
      ! error exit
      !-----------------------------------------------------------------

      if( m1 .eq. 0 .or. m2 .eq. 0 ) then
        write(6,*) ' error in read_prescribed_ice_climate'
        call shr_sys_abort ('ice: month indices m1 and m2 not found')
      endif

      !-----------------------------------------------------------------
      ! Time interpolate available ice concentration data
      !-----------------------------------------------------------------

      ad_1     = real(date1)
      ad_2     = real(date2)

      f1 = (ad_2  - ad_in) / (ad_2 - ad_1)
      f2 = (ad_in - ad_1 ) / (ad_2 - ad_1)

      m3 = mod(m2,12) + 1
      do j = jlo,jhi
        do i = ilo,ihi
          ice_cov(i,j) = c0
          pmask(i,j) = .false.
          if (tmask(i,j)) then
            ice_cov(i,j) = f1*ice_cov_m(i,j,m1) + f2*ice_cov_m(i,j,m2)
            if( ice_cov(i,j) .lt. eps04 ) ice_cov(i,j) = c0
            if( ice_cov(i,j) .gt. c1)     ice_cov(i,j) = c1
            if (ice_cov_m(i,j,m1) .gt. c0 ) pmask(i,j) = .true.
            if (ice_cov_m(i,j,m2) .gt. c0 ) pmask(i,j) = .true.
            if (ice_cov_m(i,j,m3) .gt. c0 ) pmask(i,j) = .true.
          endif
        enddo
      enddo 

      end subroutine read_prescribed_ice_climate
!
!BOP
!
! !IROUTINE: set_prescribed_ice_state - set prescribed ice state
!
! !INTERFACE:
!
      subroutine set_prescribed_ice_state
!
! !DESCRIPTION:
!
! Set prescribed ice state using input ice concentration; set snow
! to specified value; set surface ice temperature to atmospheric 
! value; use linear temperature gradient in ice to ocean temperature.
!                    
! !REVISION HISTORY:
!
! author:  Bruce P. Briegleb, NCAR
!
! !USES:
!
      use ice_flux
      use ice_vthermo
      use ice_state
      use ice_itd
      use ice_dh
      use shr_sys_mod, only : shr_sys_flush, shr_sys_abort
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      integer (kind=int_kind) :: 
     &   layer    ! level index
     &,  nc       ! ice category index
     &,   i       ! longitude index
     &,   j       ! latitude index  
     &,   k       ! level index 
      real (kind=dbl_kind) ::
     &  slope     ! diff in underlying ocean tmp and ice surface tmp
     &, Ti        ! ice level temperature
     &, hi        ! ice prescribed (hemispheric) ice thickness 
      integer (kind=int_kind) ::
     &   m1,m2      ! two months    used in time interpolation
      real (kind=dbl_kind) :: 
     &   f1,f2      ! two fractions used in time interpolation
     &,  snwc(13)   ! mean snow cover (m) first of month (nrthrn hmsphr)
     &,  snwcnh     ! snow cover (m) for northern hemisphere
     &,  snwcsh     ! snow cover (m) for southern hemisphere
     &,  snw        ! actual snow cover (m) applied
      data snwc   /  .23,  .25,  .27,  .29,  .33,  .18,
     &                0.,   0.,  .02,  .12,  .18,  .21,  .23/ 

      !-----------------------------------------------------------------
      ! Set time parameters
      !-----------------------------------------------------------------
      m1   = month
      m2   = month + 1
      f2   = (yday-daycal(m1))/(daycal(m2)-daycal(m1))
      f1   = c1 - f2

      !-----------------------------------------------------------------
      ! Compute hemispheric snow covers; phase shift sh six months
      !-----------------------------------------------------------------
      snwcnh = f1*snwc(m1) + f2*snwc(m2)
      m1 = m1 + 6
      if( m1 .gt. 13 ) m1 = m1 - 12
      m2 = m2 + 6
      if( m2 .gt. 13 ) m2 = m2 - 12
      snwcsh = f1*snwc(m1) + f2*snwc(m2)

      !-----------------------------------------------------------------
      ! Initialize ice state
      !-----------------------------------------------------------------
      do nc=1,ncat
        do j=1,jmt_local
          do i=1,imt_local
            aicen(i,j,nc) = c0
            vicen(i,j,nc) = c0
            vsnon(i,j,nc) = c0
          enddo
        enddo
      enddo

      do nc=1,ncat
        Tsfcn(ilo:ihi,jlo:jhi,nc) = Tf(ilo:ihi,jlo:jhi)
        call bound(Tsfcn(:,:,nc))
      enddo

      do layer=1,ntilay
        do j=1,jmt_local
          do i=1,imt_local
            eicen(i,j,layer) = c0
          enddo
        enddo
      enddo

      do j = jlo,jhi
       do i = ilo,ihi
        !---------------------------------------------------------------
        ! Only over ocean
        !---------------------------------------------------------------
        if (tmask(i,j)) then
         !--------------------------------------------------------------
         ! Place ice where ice concentration > .0001
         !--------------------------------------------------------------
         if ( ice_cov(i,j) .ge. eps04 ) then

          if (.not. rmask(i,j)) then
            write(6,*) ' set_prescribed_ice_state, ice forming,',
     &                 ' rmask=false, stopping'
            call shr_sys_flush(6)
            call shr_sys_abort(' set_prescribed_ice_state')
          endif

          hi = 0.0_dbl_kind
          !-------------------------------------------------------------
          ! Set ice thickness in each hemisphere
          !-------------------------------------------------------------
          if( ULAT(i,j).gt. 40.0_dbl_kind/rad_to_deg ) then
            hi  = 2.0_dbl_kind
            snw = snwcnh
          else if( ULAT(i,j).lt.-40.0_dbl_kind/rad_to_deg ) then
            hi  = 1.0_dbl_kind
            snw = snwcsh
          endif

          !-------------------------------------------------------------
          ! All ice in appropriate thickness category
          !-------------------------------------------------------------
          do nc = 1,ncat
            if( hin_max(nc-1).lt.hi .and. hi.lt.hin_max(nc) ) then
              aicen(i,j,nc) = ice_cov(i,j)
              vicen(i,j,nc) = hi*aicen(i,j,nc) 
              !---------------------------------------------------------
              ! keep snow/ice boundary above sea level by reducing snow
              !---------------------------------------------------------
              vsnon(i,j,nc) = min(snw*aicen(i,j,nc),p333*vicen(i,j,nc))
              Tsfcn(i,j,nc) = min(Tair(i,j)-Tffresh,-p2)   ! deg C       
              !---------------------------------------------------------
              ! make linear temp profile and compute enthalpy
              !---------------------------------------------------------
              slope = Tf(i,j) - Tsfcn(i,j,nc)
              k=1
              do layer=layer1(nc),layern(nc)
                Ti=Tsfcn(i,j,nc)+slope*(real(k)-p5)/nilay(nc)
         eicen(i,j,layer)=energ(Ti,saltz(k,nc))*vicen(i,j,nc)/nilay(nc)
                k=k+1
              enddo
            endif
          enddo

         endif
        endif
       enddo
      enddo

      ! compute aggregate ice state and open water area
      call aggregate 

      end subroutine set_prescribed_ice_state
!
!BOP
!
! !IROUTINE: set_prescribed_ice_flux - Set non-computed fluxes to zero
!
! !INTERFACE:
!
      subroutine set_prescribed_ice_flux
!
! !DESCRIPTION:
!
! Set non-computed fluxes for prescribed ice model to zero
!
! !REVISION HISTORY:
!
! author: Bruce P. Briegleb, NCAR
!
! !USES:
!
      use ice_dyn_evp
      use ice_flux
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      integer i,j

      do j=jlo,jhi
      do i=ilo,ihi
      !-----------------------------------------------------------------
      ! fluxes received
      !-----------------------------------------------------------------
        Fsnow(i,j)    = c0     ! snowfall rate (kg/m2/s)
        Frain(i,j)    = c0     ! rainfall rate (kg/m2/s)
        frzmlt(i,j)   = c0     ! freezing/melting potential (W/m^2)
      !-----------------------------------------------------------------
      ! ice velocities
      !-----------------------------------------------------------------
        u(i,j)        = c0     ! ice velocity u
        v(i,j)        = c0     ! ice velocity v
      !-----------------------------------------------------------------
      ! derived or computed fields
      !-----------------------------------------------------------------
        strocnxT(i,j) = c0     ! ice-ocean stress, x-direction (T-cell)
        strocnyT(i,j) = c0     ! ice-ocean stress, y-direction (T-cell)
      !-----------------------------------------------------------------
      ! other atm and ocn fluxes
      !-----------------------------------------------------------------
        call init_flux_atm(i,j)
        call init_flux_ocn(i,j)
      enddo
      enddo

      end subroutine set_prescribed_ice_flux
!
!BOP
!
! !IROUTINE: exit_prescribed_ice - close netCDF file for prescribed ice
!
! !INTERFACE:
!
      subroutine exit_prescribed_ice
!
! !DESCRIPTION:
!
! Close netCDF file for prescribed ice
!
! !REVISION HISTORY:
!
! author: Bruce P. Briegleb, NCAR
!
! !INPUT/OUTPUT PARAMETERS:
!
!EOP
!
      include "netcdf.inc"

      integer (kind=int_kind) ::
     &  status   ! status variable from netCDF routine 

      !-----------------------------------------------------------------
      ! close output dataset
      !-----------------------------------------------------------------

      if (my_task.eq.master_task) then
        status = nf_close(fid)
        write(6,*) ' '
        write(6,*) 'Close netCDF sea ice concentration file =',
     &              prescribed_ice_file
        write(6,*) ' '
        if (status /= NF_NOERR ) then
          write(6,*) 'error ice- Could not close netCDF file '
        endif
      endif

      end subroutine exit_prescribed_ice
!
      end module ice_prescribed
!

