!===============================================================================
! CVS $Id: flux_ai.F90,v 1.4 2001/12/11 22:45:25 kauff Exp $
! CVS $Source: /fs/cgd/csm/models/CVS.REPOS/ice/dice5/flux_ai.F90,v $
! CVS $Name: ccsm2_0_beta46 $
!===============================================================================

      subroutine flux_ai(imax  ,mask  ,zbot  ,ubot  ,vbot   &
      &                 ,thbot ,qbot  ,rbot  ,tbot  ,ts     &
      &                 ,sen   ,lat   ,lwup  ,evap  ,taux   &
      &                 ,tauy  ,tref  )

      use const_mod  ! physical constants

      implicit none

      !--- input arguments --------------------------------
      integer imax           ! array dimensions
      integer mask (imax)    ! 0 <=> cell NOT in model domain
      real    zbot (imax)    ! atm level height  (m)
      real    ubot (imax)    ! atm u wind        (m/s)
      real    vbot (imax)    ! atm v wind        (m/s)
      real    thbot(imax)    ! atm potential T   (K)
      real    qbot (imax)    ! atm specific humidity (kg/kg)
      real    rbot (imax)    ! atm air density   (kg/m^3)
      real    tbot (imax)    ! atm T             (K) 
      real    ts   (imax)    ! surface temperature

      !--- output arguments -------------------------------
      real    sen  (imax)    ! sensible         heat flux  (W/m^2)
      real    lat  (imax)    ! latent           heat flux  (W/m^2)
      real    lwup (imax)    ! long-wave upward heat flux  (W/m^2)
      real    evap (imax)    ! evaporative water flux ((kg/s)/m^2)
      real    taux (imax)    ! x surface stress (N)
      real    tauy (imax)    ! y surface stress (N)
      real    tref (imax)    ! 2m reference height temperature
 
      !--- local constants --------------------------------
      real,parameter :: umin   =  1.0   ! minimum wind speed (m/s)
      real,parameter :: zref   = 10.0   ! reference height (m)
      real,parameter :: ztref  =  2.0   ! reference height for air T ~ m

      !--- local variables --------------------------------
      integer i      ! vector index
      real    vmag   ! surface wind magnitude   (m/s)
      real    thvbot ! virtual temperature      (K)
      real    ssq    ! sea surface humidity     (kg/kg)
      real    dssqdt ! derivative of ssq wrt Ts (kg/kg/K)
      real    delt   ! potential T difference   (K)
      real    delq   ! humidity difference      (kg/kg)
      real    stable ! stability factor
      real    rdn    ! sqrt of neutral exchange coefficient (momentum)
      real    rhn    ! sqrt of neutral exchange coefficient (heat)
      real    ren    ! sqrt of neutral exchange coefficient (water)
      real    rd     ! sqrt of exchange coefficient (momentum)
      real    rh     ! sqrt of exchange coefficient (heat)
      real    re     ! sqrt of exchange coefficient (water)            
      real    ustar  ! ustar
      real    qstar  ! qstar
      real    tstar  ! tstar
      real    hol    ! H (at zbot) over L
      real    xsq    ! temporary variable
      real    xqq    ! temporary variable
      real    psimh  ! stability function at zbot (momentum)
      real    psixh  ! stability function at zbot (heat and water)
      real    alz    ! ln(zbot/z10)
      real    ltheat ! latent heat for surface
      real    tau    ! stress at zbot
      real    cp     ! specific heat of moist air

      real    bn     ! exchange coef funct for interpolation
      real    bh     ! exchange coef funct for interpolation
      real    fac    ! interpolation factor
      real    ln0    ! log factor for interpolation
      real    ln3    ! log factor for interpolation

      !--- local functions --------------------------------
      real   Tk      ! temperature (K)
      real   qsat    ! the saturation humididty of air (kg/m^3)
      real   dqsatdt ! derivivative of qsat wrt surface temperature
      real   xd      ! dummy argument  
      real   psimhu  ! unstable part of psimh
      real   psixhu  ! unstable part of psimx

      qsat(Tk)    = 640380. / exp(5107.4/Tk)
      dqsatdt(Tk) = (5107.4 / Tk**2) * 640380. / exp(5107.4/Tk)
      psimhu(xd)  = log((1.+xd*(2.+xd))*(1.+xd*xd)/8.) - 2.*atan(xd) + 1.571
      psixhu(xd)  =  2. * log((1. + xd*xd)/2.)

!-------------------------------------------------------------------------------
! PURPOSE:
!   using atm & ice state variables,
!   compute atm/ice fluxes and diagnostic air temperature
!
! NOTE: 
!   o all fluxes are positive downward
!   o net heat flux = net sw + lw up + lw down + sen + lat
!   o here, tstar = <WT>/U*, and qstar = <WQ>/U*.
!   o wind speeds should all be above a minimum speed (eg. 1.0 m/s)
! 
! ASSUME:
!   o The saturation humidity of air at T(K): qsat(T)  (kg/m^3)
!-------------------------------------------------------------------------------

      do i=1,imax

        if (mask(i) == 0) then
          sen  (i) = 1.0e30
          lat  (i) = 1.0e30
          lwup (i) = 1.0e30
          evap (i) = 1.0e30
          taux (i) = 1.0e30
          tauy (i) = 1.0e30
          tref (i) = 1.0e30
        else if (tbot(i) == 0.0) then ! zero/equitorial/no-data region?
          sen  (i) = 0.0
          lat  (i) = 0.0
          lwup (i) = 0.0
          evap (i) = 0.0
          taux (i) = 0.0
          tauy (i) = 0.0
          tref (i) = 0.0
        else
          !--- define some needed variables ---
          vmag   = max(umin, sqrt(ubot(i)**2+vbot(i)**2))
          thvbot = thbot(i)*(1. + zvir * qbot(i)) ! virtual pot temp (K)
           ssq   =  qsat  (ts(i)) / rbot(i)       ! sea surf hum (kg/kg)
          dssqdt = dqsatdt(ts(i)) / rbot(i)       ! deriv of ssq wrt Ts 
          delt   = thbot(i) - ts(i)               ! pot temp diff (K)
          delq   = qbot(i) - ssq                  ! spec hum dif (kg/kg)
          alz    = log(zbot(i)/zref) 
          cp     = cpdair*(1. + cpvir*ssq) 
          ltheat = latvap + latice

          !----------------------------------------------------------
          ! first estimate of Z/L and ustar, tstar and qstar
          !----------------------------------------------------------

          !--- neutral coefficients, z/L = 0.0 ---
          rdn = karman/log(zref/zzsice)
          rhn = rdn
          ren = rdn

          !--- ustar,tstar,qstar ----
          ustar = rdn * vmag
          tstar = rhn * delt  
          qstar = ren * delq  

          !--- compute stability & evaluate all stability functions ---
          hol    = karman * g * zbot(i) &
          &        * (tstar/thvbot+qstar/(1./zvir+qbot(i))) / ustar**2
          hol    = sign( min(abs(hol),10.), hol )
          stable = 0.5 + sign(0.5 , hol)
          xsq    = max(sqrt(abs(1. - 16.*hol)) , 1.)
          xqq    = sqrt(xsq)
          psimh  = -5.*hol*stable + (1.-stable)*psimhu(xqq)
          psixh  = -5.*hol*stable + (1.-stable)*psixhu(xqq)

          !--- shift all coeffs to measurement height and stability ---
          rd = rdn / (1.+rdn/karman*(alz-psimh))
          rh = rhn / (1.+rhn/karman*(alz-psixh))
          re = ren / (1.+ren/karman*(alz-psixh))

          !--- update ustar, tstar, qstar w/ updated, shifted coeffs --
          ustar = rd * vmag 
          tstar = rh * delt 
          qstar = re * delq 

          !----------------------------------------------------------
          ! iterate to converge on Z/L, ustar, tstar and qstar
          !----------------------------------------------------------

          !--- compute stability & evaluate all stability functions ---
          hol    = karman * g * zbot(i) &
          &        * (tstar/thvbot+qstar/(1./zvir+qbot(i))) / ustar**2
          hol    = sign( min(abs(hol),10.), hol )
          stable = 0.5 + sign(0.5 , hol)
          xsq    = max(sqrt(abs(1. - 16.*hol)) , 1.)
          xqq    = sqrt(xsq)
          psimh  = -5.*hol*stable + (1.-stable)*psimhu(xqq)
          psixh  = -5.*hol*stable + (1.-stable)*psixhu(xqq)

          !--- shift all coeffs to measurement height and stability ---
          rd = rdn / (1.+rdn/karman*(alz-psimh)) 
          rh = rhn / (1.+rhn/karman*(alz-psixh)) 
          re = ren / (1.+ren/karman*(alz-psixh)) 

          !--- update ustar, tstar, qstar w/ updated, shifted coeffs --
          ustar = rd * vmag 
          tstar = rh * delt 
          qstar = re * delq 

          !----------------------------------------------------------
          ! compute the fluxes
          !----------------------------------------------------------

          tau = rbot(i) * ustar * ustar 
      
          !--- momentum flux ---
          taux(i) = tau * ubot(i) / vmag 
          tauy(i) = tau * vbot(i) / vmag 
       
          !--- heat flux ---
          sen (i) =      cp * tau * tstar / ustar 
          lat (i) =  ltheat * tau * qstar / ustar
          lwup(i) = -stebol * ts(i)**4 
     
          !--- water flux ---
          evap(i) = lat(i)/ltheat 

          !----------------------------------------------------------
          ! compute diagnostic: 2m reference height temperature
          !----------------------------------------------------------

          !--- Compute function of exchange coefficients. Assume that 
          !--- cn = rdn*rdn, cm=rd*rd and ch=rh*rd, and therefore 
          !--- 1/sqrt(cn(i))=1/rdn and sqrt(cm(i))/ch(i)=1/rh 
          bn = karman/rdn
          bh = karman/rh

          !--- Interpolation factor for stable and unstable cases
          ln0 = log(1.0 + (ztref/zbot(i))*(exp(bn) - 1.0))
          ln3 = log(1.0 + (ztref/zbot(i))*(exp(bn - bh) - 1.0))
          fac = (ln0 - ztref/zbot(i)*(bn - bh))/bh * stable &
          &   + (ln0 - ln3)/bh * (1.-stable)
          fac = min(max(fac,0.),1.)

          !--- actual interpolation
          tref(i) = ts(i) + (tbot(i) - ts(i))*fac

        endif
      enddo 

      RETURN 
      END
 
!===============================================================================
