module data_in_mod

  use kind_mod              ! defines precision 
  use shr_sys_mod           ! system calls
  use buf_mod               ! allocatables and parameters 
  use control_mod           ! control info, date/time
  use constant_mod          ! physical constants
  use calendar_mod          ! calendar info 
  implicit none 

  integer , private              :: nx_sst,ny_sst ! SST file grid dimensions  
  integer , private              :: ntim_sst      ! SST file number of time samples
  real(r8), private, allocatable :: x_sst(:)      ! SST longitude coordinate
  real(r8), private, allocatable :: y_sst(:)      ! SST latitude coordinate
  real(r8), private, allocatable :: T_sst(:,:,:)  ! SST temperatures

  real(r8), public , allocatable :: eday_sst(:)   ! SST input elapsed day + seconds
  integer , private, allocatable :: date_sst(:)   ! SST input dates
  integer , private, allocatable :: secs_sst(:)   ! SST seconds of input dates 

  real(r8), public, allocatable :: r_t   (:,:,:)  ! spatially interpolated temperature            ~ Kelvin
  real(r8), public, allocatable :: r_s   (:,:,:)  ! spatially interpolated salinity               ~ Kelvin
  real(r8), public, allocatable :: r_u   (:,:,:)  ! spatially interpolated velocity, zonal        ~ m/s
  real(r8), public, allocatable :: r_v   (:,:,:)  ! spatially interpolated velocity, meridional   ~ m/s
  real(r8), public, allocatable :: r_dhdx(:,:,:)  ! spatially interpolated dh/dx (surface slope)  ~ m/m
  real(r8), public, allocatable :: r_dhdy(:,:,:)  ! spatially interpolated dh/dy (surface slope)  ~ m/m
  real(r8), public, allocatable :: r_Q   (:,:,:)  ! spatially interpolated heat of fusion (q > 0) ~ W/m^2

  logical, private :: dointerp ! if true, do spatial interpolation
  integer, public  :: nm,np    ! input indices for prev and next month of data (multi-year only)
  integer, private :: ntp1     ! input current forward time index (multi-year only)
  integer, private :: fid      ! input sst file nc id
  integer, private :: sstid    ! input sst data nc id

!===========================================================================
 CONTAINS
!===========================================================================

  subroutine data_in_init

    use calendar_mod, only : dsm, valid_date

    implicit none

#include <netcdf.inc>                  ! netCDF defs

    !----- local ------
    character,allocatable :: str(:)    ! variable length char string
    character(len=80)     :: fn        ! file name
    integer               :: vid       ! nc variable ID     
    integer               :: dimid     ! nc dimension id    
    integer               :: dimlen    ! nc dimension length
    integer               :: rcode     ! nc return code     

    integer               :: i,j,k,n   ! generic loop indicies
    integer               :: year_sst  ! input year (temporary)
    integer               :: month_sst ! input month (temporary) 
    integer               :: dayom_sst ! input day of month (temporary)

    ! netcdf temporaries     
    character*(NF_MAX_NAME) tmpname
    integer tmptype
    integer ndims
    integer dims(NF_MAX_VAR_DIMS)
    integer natt
    real(r8), allocatable :: temp(:,:)

!-----------------------------------------------------------------------
! PURPOSE: Obtain SST data from file
!-----------------------------------------------------------------------

    ! Open SST file
    call getfil (trim(data_file), data_dir, fn)
    write(6,*) '(data_in_init) open SST netCDF file ',fn
    call wrap_open (fn, NF_NOWRITE, fid)

    ! Print title info 
    call wrap_inq_attlen  (fid, NF_GLOBAL, 'title', n)
    allocate(str(n))
    call wrap_get_att_text(fid, NF_GLOBAL, 'title', str)
    write(6,*) '(data_in_init) title = ',str(1:n)
    deallocate(str)
    call wrap_inq_attlen  (fid, NF_GLOBAL, 'history', n)
    allocate(str(n))
    call wrap_get_att_text(fid, NF_GLOBAL, 'history', str)
    write(6,*) '(data_in_init) history = ',str(1:n)
    deallocate(str)

    ! Retrieve input grid 
    write(6,*) '(data_in_init) read SST grid data...'

    call wrap_inq_varid (fid, trim(data_lonname), vid)
    call wrap_inq_var(fid, vid, tmpname, tmptype, ndims, dims, natt)
    if (ndims == 1) then
       call wrap_inq_dimlen(fid, dims(1), nx_sst ) 
       allocate(x_sst(nx_sst))
       call wrap_get_var_realx (fid, vid, x_sst)
    elseif (ndims == 2) then
       call wrap_inq_dimlen(fid, dims(1), nx_sst ) 
       call wrap_inq_dimlen(fid, dims(2), ny_sst ) 
       allocate(temp(nx_sst,ny_sst))
       call wrap_get_var_realx (fid, vid, temp)
       dointerp = .false.
       if (nx_sst /= nx .or. ny_sst /= ny) then
          call shr_sys_abort('cannot do interp for non-rectilinear sst coords')
       endif
       do j = 1,ny_sst
          do i = 1,nx_sst
             if (abs(temp(i,j)-xc(i,j)) > 1.e-2) dointerp=.true.
          end do
       end do
       if (dointerp) then
          call shr_sys_abort('cannot do interp for non-rectilinear sst coords')
       endif
       deallocate (temp)
    else
       call shr_sys_abort('dimension of coordinate variable must be 1 or 2')
    endif

    call wrap_inq_varid (fid, trim(data_latname), vid)
    call wrap_inq_var(fid, vid, tmpname, tmptype, ndims, dims, natt)
    if (ndims == 1) then
       call wrap_inq_dimlen(fid, dims(1), ny_sst ) 
       allocate(y_sst(ny_sst))
       call wrap_get_var_realx (fid, vid, y_sst)
    elseif (ndims == 2) then
       call wrap_inq_dimlen(fid, dims(1), nx_sst ) 
       call wrap_inq_dimlen(fid, dims(2), ny_sst ) 
       allocate(temp(nx_sst,ny_sst))
       call wrap_get_var_realx (fid, vid, temp)
       dointerp = .false.
       if (nx_sst /= nx .or. ny_sst /= ny) then
          call shr_sys_abort('cannot do interp for non-rectilinear sst coords')
       endif
       do j = 1,ny_sst
          do i = 1,nx_sst
             if (abs(temp(i,j)-yc(i,j)) > 1.e-2) dointerp=.true.
          end do
       end do
       if (dointerp) then
          call shr_sys_abort('cannot do interp for non-rectilinear sst coords')
       endif
       deallocate (temp)
    else
       call shr_sys_abort('dimension of coordinate variable must be 1 or 2')
    endif

    ! Determine if need to do spatial interpolation 
    ! Note: spatial interpolation can only currently be done for rectilinear coordinates
    if (ndims == 1) then
       if (nx_sst==nx .and. ny_sst==ny) then
          dointerp=.false.
          do i=1,nx_sst
             if (abs(x_sst(i)-xc(i,1)) > 1.e-2) dointerp=.true.
          end do
          do i=1,ny_sst
             if (abs(y_sst(i)-yc(1,i)) > 1.e-2) dointerp=.true.
          end do
       else
          dointerp=.true.
       endif
    endif
    if (dointerp) then
       write(6,*)'(data_in_init) will interpolate SST data to domain grid'
    else
       write(6,*)'(data_in_init) model domain and SST grid the same'
    endif

    ! Retrieve entire date and sec variables 
    call wrap_inq_dimid ( fid, "time", dimid )
    call wrap_inq_dimlen( fid, dimid , ntim_sst )
    if (data_form(1:6)=='annual' .and. ntim_sst/=12 ) then
       write(6,*)'(data_in_init) When cycling SST data, must have 12' 
       write(6,*)'consecutive months of data starting with Jan'
       write(6,*)'Current dataset has ',ntim_sst,' months'
       call shr_sys_abort('need 12 time samples for climatology data')
    end if
    allocate(date_sst(ntim_sst))
    allocate(secs_sst(ntim_sst))
    call wrap_inq_varid   (fid, 'date', vid)
    call wrap_get_var_int (fid, vid, date_sst)
    call wrap_inq_varid   (fid, 'datesec', vid)
    call wrap_get_var_int (fid, vid, secs_sst)

    ! Determine elapsed days of total SST dataset
    allocate(eday_sst(ntim_sst))
    do n = 1,ntim_sst
       if (.not. valid_date(date_sst(n))) then
          call shr_sys_abort('data_in_init: not valid date')
       endif
       if ( data_form(1:6) == 'annual' ) then
          if ( mod(date_sst(n),10000)/100/=n ) then
             write(6,*)'(data_in_init) When cycling sst data - must have 12 consecutive months of data'
             write(6,*)'month ',n,' of sst data set is out of order'
             call shr_sys_abort()
          end if
          year_sst = 0
       else
          year_sst = int(date_sst(n)/10000)
       endif
       month_sst = int(mod(date_sst(n),10000)/100)
       dayom_sst = mod(date_sst(n),100) 
       eday_sst(n) = year_sst*365. + dsm(month_sst) + (dayom_sst-1) + secs_sst(n)/86400.
    end do

    ! Get SST variable id
    call wrap_inq_varid(fid, trim(data_sstname), sstid)

    ! Allocate memory for SST data
    if (data_form(1:6) == 'annual') then
       allocate(T_sst(nx_sst,ny_sst,12))
    else
       allocate(T_sst(nx_sst,ny_sst,2))
    endif

    ! Read in annual or multi-year datasets
    if (data_form(1:6) == 'annual') then
       call data_in_annual()
       call wrap_close(fid)
    else if (data_form(1:9) == 'multiyear') then
       call data_in_multiyr_init()
    else
       write(6,*)'(data_in_init) ERROR data_form must be set to either annual or multiyear'
    endif

    return
  end subroutine data_in_init

!===========================================================================

  SUBROUTINE data_in_annual()

    implicit none

#include <netcdf.inc>         ! netCDF defs

    !----- local ------
    integer :: i,j,k,n        ! generic loop indicies
    integer :: start(3)       ! vector of start indices needed by get_vara
    integer :: count(3)       ! vector of lengths needed by get_vara

!-----------------------------------------------------------------------
! PURPOSE: read in 12 months of raw SST climatological data 
!-----------------------------------------------------------------------

    ! Retrieve SST data 
    write(6,*) '(data_in_annual) reading 12 months of SST data...'
    start(1) = 1; count(1) = nx_sst
    start(2) = 1; count(2) = ny_sst
    start(3) = 1; count(3) = 12
    call wrap_get_vara_realx(fid, sstid, start, count, T_sst)

    ! Convert to Kelvin 
    write(6,*) '(data_in_annual) convert SST climatology to Kelvin'
    T_sst(:,:,:) = T_sst(:,:,:) + Tzro

    ! Spatially interpolate if necessary
    if (dointerp) then
       write(6,*) '(data_in_annual) map SST from input grid to model grid...'
       do n=1,12
          call Interp2D(x_sst  , y_sst , T_sst(1,1,n), nx_sst, ny_sst , &
                        xc     , yc    , r_t(1,1,n)  , nx    , ny     , &
                        1.0e30, 0)
       end do
    else
       r_t(:,:,:) = T_sst(:,:,:)
    endif

    ! Set necessary default values
    write(6,*) '(data_in_annual) set default values for u,v,dh/dx,dy/dy,Q'
    r_s    (:,:,:) =   34.7   ! salinity                  ~ Kelvin
    r_u    (:,:,:) =   0.0    ! velocity, zonal           ~ m/sr
    r_v    (:,:,:) =   0.0    ! velocity, meridional      ~ m/s
    r_dhdx (:,:,:) =   0.0    ! surface slope, meridional ~ m/m
    r_dhdy (:,:,:) =   0.0    ! surface slope, zonal      ~ m/m
    r_q    (:,:,:) =   0.0    ! heat: fusion xor melt pot ~ Q/m^2

    ! Deallocate unnecessary memory
    deallocate(x_sst)
    deallocate(y_sst)
    deallocate(T_sst)

    return
  end SUBROUTINE data_in_annual

!===========================================================================

  SUBROUTINE data_in_multiyr_init()

    implicit none

#include <netcdf.inc>         ! netCDF defs

    !----- local ------
    integer :: i,j,k,n        ! generic loop indicies
    integer :: start(3)       ! vector of start indices needed by get_vara
    integer :: count(3)       ! vector of lengths needed by get_vara

!-----------------------------------------------------------------------
! PURPOSE:
! read in raw SST multi year dataset and position input dataset by
! reading in two timeslices bracketing current day
!-----------------------------------------------------------------------

    ! Position SST dataset for current date 
    write(6,*) '(data_in_multiyr_init) reading SST data...'
    nm = 1
    np = 2
    start(1) = 1 ; count(1) = nx_sst
    start(2) = 1 ; count(2) = ny_sst
    start(3) = 1 ; count(3) = 1
    do n=1,ntim_sst-1
       if (eday>=eday_sst(n) .AND. eday<eday_sst(n+1)) then
          start(3) = n  
          call wrap_get_vara_realx(fid, sstid, start, count, T_sst(1,1,nm))
          start(3) = n+1
          call wrap_get_vara_realx(fid, sstid, start, count, T_sst(1,1,np))
          eday_sst(nm) = eday_sst(n)
          eday_sst(np) = eday_sst(n+1)
          ntp1 = n + 1
          write(6,*)'(data_in_multiyr_init) Read SST data for ', &
                     date_sst(n)  ,' and ',secs_sst(n)  ,' sec and ', &
                     date_sst(n+1),' and ',secs_sst(n+1),' sec'
          goto 10
       endif
    end do
    write(6,*)'(data_in_multiyr_init) Failed to find input dates bracketing ',cdate,sec
    call shr_sys_abort()
10  continue

    ! Convert to Kelvin 
    write(6,*) '(data_in_multiyr_init) convert SST climatology to Kelvin'
    T_sst(:,:,:) = T_sst(:,:,:) + Tzro

    ! Spatially interpolate if necessary 
    if (dointerp) then
       do n=1,2
          if (n == 1) then
             write(6,*) '(data_in_multiyr_init) map SST from STR grid to model grid...'
          endif
          call Interp2D(x_sst , y_sst ,T_sst(1,1,n), nx_sst, ny_sst, &
                        xc    , yc    ,r_t(1,1,n)  , nx    , ny    , & 
                        1.0e30, 0)
       end do
    else
       r_t(:,:,:) = T_sst(:,:,:)
    endif

    ! Set necessary default values
    write(6,*) '(data_in_multiyr_init) set default values for u,v,dh/dx,dy/dy,Q'
    r_s    (:,:,:) =   34.7   ! salinity                  ~ Kelvin
    r_u    (:,:,:) =   0.0    ! velocity, zonal           ~ m/s
    r_v    (:,:,:) =   0.0    ! velocity, meridional      ~ m/s
    r_dhdx (:,:,:) =   0.0    ! surface slope, meridional ~ m/m
    r_dhdy (:,:,:) =   0.0    ! surface slope, zonal      ~ m/m
    r_q    (:,:,:) =   0.0    ! heat: fusion xor melt pot ~ Q/m^2

    RETURN
  end SUBROUTINE data_in_multiyr_init

!===========================================================================

  SUBROUTINE data_in_multiyr()

    implicit none

#include <netcdf.inc>           ! netCDF defs

    !     ----- local ------
    integer :: i,j,k,n        ! generic loop indicies
    integer :: ntmp           ! temporary variable 
    integer :: start(3)       ! vector of start indices needed by get_vara
    integer :: count(3)       ! vector of lengths needed by get_vara

!-----------------------------------------------------------------------
! PURPOSE:
!   read in raw SST climatological data 
!-----------------------------------------------------------------------

    ! If model time is past current forward timeslice, 
    ! read in the next timeslice for time interpolation.  

    if (eday > eday_sst(ntp1)) then

       ! Read in next time slice
       start(1) = 1; count(1) = nx_sst
       start(2) = 1; count(2) = ny_sst
       start(3) = 1; count(3) = 1
       if (ntp1 > ntim_sst) then
          call shr_sys_abort('data_in_multiyr: attempt to read past end of SST dataset')
       end if
       if (eday > eday_sst(ntp1+1)) then
          write(6,*)'data_in_multiyr: Input date',date_sst(ntp1+1),' sec ',secs_sst(ntp1), &
               ' must exceed model date',cdate,' sec ',sec
          call shr_sys_abort()
       end if
       ntp1 = ntp1 + 1
       ntmp = nm
       nm   = np
       np   = ntmp
       start(3) = ntp1
       call wrap_get_vara_realx(fid, sstid, start, count, T_sst(1,1,np))
       eday_sst(nm) = eday_sst(ntp1-1)
       eday_sst(np) = eday_sst(ntp1  )
       write(6,*)'(data_in_multiyr) Read SST data for ',date_sst(ntp1),' and ',secs_sst(ntp1),' sec '

       ! Convert to Kelvin
       write(6,*) '(data_in_multiyr) convert SST climatology to Kelvin'
       T_sst(:,:,np) = T_sst(:,:,np) + Tzro

       ! Spatially interpolate if necessary 
       if (dointerp) then
          write(6,*) '(data_in_multiyr) map SST from STR grid to model grid...'
          call Interp2D(x_sst, y_sst, T_sst(1,1,np), nx_sst, ny_sst, &
                        xc    ,yc   , r_t(1,1,np)  , nx    , ny    , &
                        1.0e30,0)
       else
          r_t(:,:,np) = T_sst(:,:,np)
       endif

    endif

    RETURN
  end SUBROUTINE data_in_multiyr

!===========================================================================

end MODULE data_in_mod


