!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module communicate

!***********************************************************************
!
!     this module contains the necessary routines and variables for
!     communicating between processors and setting up local domain.
!
!     CVS:$Id: communicate.F,v 1.3.2.1 2001/03/13 22:54:24 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain

      implicit none
      save
      private
      public :: init_communicate, exit_message_environment

!-----------------------------------------------------------------------

      !***
      !*** processors laid out in Cartesian geometry
      !*** NPROC_X, NPROC_Y currently defined in makefile
      !***

      integer (kind=int_kind), parameter, public ::
     &   nproc_x = NPROC_X,         ! num of processors in x-direction
     &   nproc_y = NPROC_Y          ! num of processors in y-direction

      integer (kind=int_kind), parameter, public ::
     &   nproc_s = nproc_x*nproc_y  ! total num of processors

      integer (kind=int_kind), parameter, public :: 
     &   mpitag_nshift = 1,         ! MPI tags for various 
     &   mpitag_sshift = 2,         ! communication patterns
     &   mpitag_eshift = 3,
     &   mpitag_wshift = 4,
     &   mpitag_io     = 5,
     &   mpitag_gthr   = 6

      integer (kind=int_kind), public ::
     &   nbr_west, nbr_east,        ! processor numbers for the 
     &   nbr_north, nbr_south       ! n,s,e,w neighbor processors

      integer (kind=int_kind), dimension(4,nproc_s), public ::
     &   local_block                ! global i,j range for each processor

      integer (kind=int_kind), dimension(nproc_s), public ::
     &   mpi_interior_int,          ! MPI data type definitions for
     &   mpi_interior_real,         ! physical domain blocks
     &   mpi_interior_dble,         ! different for each processor
     &   mpi_interior_int_global,   ! in case of padding
     &   mpi_interior_real_global,
     &   mpi_interior_dble_global

      integer (kind=int_kind), public ::
     &   comm_2d,                   ! MPI data types for other 
     &   mpi_ew_type, mpi_ns_type,  ! strided data patterns
     &   mpi_ew_type_int, mpi_ns_type_int,
     &   MPI_COMM_OCN               ! MPI communicator for ocn comms

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_communicate

!-----------------------------------------------------------------------
!
!     this routine sets up MPI environment and uses MPI topology
!     creation routines to map cartesian geometry to processors.
!     process number (my_task) and neighboring process numbers are
!     determined.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      include 'mpif.h'   ! MPI Fortran include file

      integer (kind=int_kind) ::
     &   i, j, n,                 ! dummy counters
     &   interior_i, interior_j,  ! dummies for interior blocks
     &   ierr,                    ! MPI error flag
     &   ndims,                   ! number of processor dimensions  
     &   domain_error,            ! flag to check for domain errors
     &   errsum                   ! global sum of domain_error

      integer (kind=int_kind), dimension(2) ::
     &   dims,                    ! processor dimensions and coordinates 
     &   coords = (/0,0/)         ! for Cartesian processor layout

      logical (kind=log_kind), dimension(2) ::
     &   periods                  ! MPI flag for periodic coordinates

      logical (kind=log_kind) ::
     &   reorder

!-----------------------------------------------------------------------
!
!     initiate mpi environment and create communicator for internal
!     ocean communications
!     if model is coupled with NCAR flux coupler, MPI_INIT and
!     MPI_COMM_OCN intialization must occur in initial.F since MPI
!     will be used for coupler comms
!
!-----------------------------------------------------------------------

#ifndef coupled
      call MPI_INIT(ierr)
      call MPI_COMM_DUP(MPI_COMM_WORLD, MPI_COMM_OCN, ierr)
#endif

      master_task = 0

!-----------------------------------------------------------------------
!
!     create cartesian decomposition and initialize two-dimensional 
!     cartesian shifts
!
!-----------------------------------------------------------------------

      ndims      = 2
      dims   (1) = nproc_y  ! transpose the axes to make consistent
      dims   (2) = nproc_x  ! with shmem version
c      dims   (1) = nproc_x
c      dims   (2) = nproc_y
      periods(1) = .true. 
      periods(2) = .true. 
      reorder    = .false.

      call MPI_CART_CREATE(MPI_COMM_OCN, ndims, dims, periods,
     &                     reorder, comm_2d, ierr)

!-----------------------------------------------------------------------
!
!     compute shifts for cartesian decomposition
!
!-----------------------------------------------------------------------

      call MPI_CART_SHIFT(comm_2d, 1, 1, nbr_west, nbr_east, ierr)
      call MPI_CART_SHIFT(comm_2d, 0, 1, nbr_south, nbr_north, ierr)
c      call MPI_CART_SHIFT(comm_2d, 0, 1, nbr_west, nbr_east, ierr)
c      call MPI_CART_SHIFT(comm_2d, 1, 1, nbr_south, nbr_north, ierr)

      call MPI_COMM_RANK  (comm_2d, my_task, ierr)
      call MPI_CART_COORDS(comm_2d, my_task, 2, coords, ierr)

!-----------------------------------------------------------------------
!
!     set up data type for strided (east-west) communication
!     and non-strided (north-south) communication
!
!-----------------------------------------------------------------------

      call MPI_TYPE_VECTOR(jmt, num_ghost_cells, imt, 
     &                     mpi_double_precision, mpi_ew_type, ierr)
      call MPI_TYPE_COMMIT(mpi_ew_type, ierr)

      call MPI_TYPE_VECTOR(num_ghost_cells, imt, imt, 
     &                     mpi_double_precision, mpi_ns_type, ierr)
      call MPI_TYPE_COMMIT(mpi_ns_type, ierr)

      call MPI_TYPE_VECTOR(jmt, num_ghost_cells, imt, 
     &                     mpi_integer, mpi_ew_type_int, ierr)
      call MPI_TYPE_COMMIT(mpi_ew_type_int, ierr)

      call MPI_TYPE_VECTOR(num_ghost_cells, imt, imt, 
     &                     mpi_integer, mpi_ns_type_int, ierr)
      call MPI_TYPE_COMMIT(mpi_ns_type_int, ierr)

!-----------------------------------------------------------------------
!
!     set up domain information
!
!-----------------------------------------------------------------------

      interior_i = imt - 2*num_ghost_cells
      interior_j = jmt - 2*num_ghost_cells

      iphys_b = num_ghost_cells + 1
      jphys_b = num_ghost_cells + 1

      do i=1,imt
        i_global(i) = interior_i*coords(2) + i - num_ghost_cells
c        i_global(i) = interior_i*coords(1) + i - num_ghost_cells
      end do
      iphys_e = min(imt - num_ghost_cells,
     &              imt_global - i_global(1) + 1)

      do j=1,jmt
        j_global(j) = interior_j*coords(1) + j - num_ghost_cells
c        j_global(j) = interior_j*coords(2) + j - num_ghost_cells
      end do
      jphys_e = min(jmt - num_ghost_cells,
     &              jmt_global - j_global(1) + 1)

      !***
      !*** perform some checks on domain info
      !***

      domain_error = 0
      if (iphys_e < iphys_b .or. jphys_e < jphys_b) then
        write(6,*) 'Some PE domains have no cells:'
        write(6,*) 'Task = ',my_task
        write(6,*) 'iphys_e,jphys_e = ',iphys_e,jphys_e
        domain_error = 1
      endif

      if ((iphys_e - iphys_b) < num_ghost_cells .or. 
     &    (jphys_e - jphys_b) < num_ghost_cells) then
        write(6,*) 'Not enough points in physical domain to perform'
        write(6,*) '    proper ghost cell updates'
        write(6,*) 'Task = ',my_task
        write(6,*) 'iphys_e,jphys_e = ',iphys_e,jphys_e
        domain_error = 1
      endif

      call MPI_ALLREDUCE(domain_error, errsum, 1, MPI_INTEGER, MPI_SUM,
     &                   MPI_COMM_OCN, ierr)

      if (errsum /= 0) then
        call exit_message_environment(ierr)
        stop
      endif

!-----------------------------------------------------------------------
!
!     for routines which gather/scatter physical blocks of data, 
!     broadcast the global index of each processors
!     block in the global domain.  also define types for interior
!     blocks (global and local).
!
!-----------------------------------------------------------------------

      do n=1,nproc_s
        if (n-1 .eq. my_task) then
          i = i_global(iphys_b)
          j = j_global(jphys_b)
        endif
        call MPI_bcast(i, 1, MPI_INTEGER, n-1, 
     &                       MPI_COMM_OCN, ierr)
        call MPI_barrier(MPI_COMM_OCN, ierr)
        call MPI_bcast(j, 1, MPI_INTEGER, n-1, 
     &                       MPI_COMM_OCN, ierr)
        call MPI_barrier(MPI_COMM_OCN, ierr)
        local_block(1,n) = i
        local_block(2,n) = j
        if (n-1 .eq. my_task) then
          i = i_global(iphys_e)
          j = j_global(jphys_e)
        endif
        call MPI_bcast(i, 1, MPI_INTEGER, n-1, 
     &                       MPI_COMM_OCN, ierr)
        call MPI_barrier(MPI_COMM_OCN, ierr)
        call MPI_bcast(j, 1, MPI_INTEGER, n-1, 
     &                       MPI_COMM_OCN, ierr)
        call MPI_barrier(MPI_COMM_OCN, ierr)
        local_block(3,n) = i
        local_block(4,n) = j

        interior_i = local_block(3,n) - local_block(1,n) + 1
        interior_j = local_block(4,n) - local_block(2,n) + 1

        call MPI_TYPE_VECTOR(interior_j, interior_i, imt, 
     &                       mpi_integer, mpi_interior_int(n), ierr)
        call MPI_TYPE_COMMIT(mpi_interior_int(n), ierr)

        call MPI_TYPE_VECTOR(interior_j, interior_i, imt, 
     &                       mpi_real, mpi_interior_real(n), ierr)
        call MPI_TYPE_COMMIT(mpi_interior_real(n), ierr)

        call MPI_TYPE_VECTOR(interior_j, interior_i, imt, 
     &                       mpi_double_precision, 
     &                       mpi_interior_dble(n), ierr)
        call MPI_TYPE_COMMIT(mpi_interior_dble(n), ierr)

        call MPI_TYPE_VECTOR(interior_j, interior_i, imt_global, 
     &                   mpi_integer, mpi_interior_int_global(n), ierr)
        call MPI_TYPE_COMMIT(mpi_interior_int_global(n), ierr)

        call MPI_TYPE_VECTOR(interior_j, interior_i, imt_global, 
     &                      mpi_real, mpi_interior_real_global(n), ierr)
        call MPI_TYPE_COMMIT(mpi_interior_real_global(n), ierr)

        call MPI_TYPE_VECTOR(interior_j, interior_i, imt_global, 
     &                       mpi_double_precision, 
     &                       mpi_interior_dble_global(n), ierr)
        call MPI_TYPE_COMMIT(mpi_interior_dble_global(n), ierr)

      end do

!-----------------------------------------------------------------------

      end subroutine init_communicate

!***********************************************************************

      subroutine exit_message_environment(ierr)

      implicit none

      include 'mpif.h'   ! MPI Fortran include file

      integer (kind=int_kind) :: ierr   ! MPI error flag

      call MPI_FINALIZE(ierr)

      end subroutine exit_message_environment

!***********************************************************************

      end module communicate

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
