!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module global_reductions

!***********************************************************************
!
!     This module contains all the global reduction routines.  This
!     particular version contains MPI versions of these routines.
!
!     CVS:$Id: global_reductions.F,v 1.2.2.1 2001/03/13 22:54:24 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use communicate
      use boundary

      implicit none

!-----------------------------------------------------------------------
!
!     generic interfaces for module procedures
!
!-----------------------------------------------------------------------

      interface broadcast_scalar
        module procedure broadcast_scalar_real,
     &                   broadcast_scalar_real4,
     &                   broadcast_scalar_int,
     &                   broadcast_scalar_log,
     &                   broadcast_scalar_char
      end interface

      interface broadcast_vector
        module procedure broadcast_vector_real,
     &                   broadcast_vector_real4,
     &                   broadcast_vector_int,
     &                   broadcast_vector_log
      end interface

      interface global_sum
        module procedure global_real_sum,
     &                   global_real4_sum,
     &                   global_int_sum
      end interface

      interface global_count
        module procedure global_real_count,
     &                   global_real4_count,
     &                   global_int_count,
     &                   global_log_count
      end interface

      interface global_maxval
        module procedure global_real_maxval,
     &                   global_real4_maxval,
     &                   global_int_maxval
      end interface

      interface global_minval
        module procedure global_real_minval,
     &                   global_real4_minval,
     &                   global_int_minval
      end interface

      interface gather_global
        module procedure gather_global_real,
     &                   gather_global_real4,
     &                   gather_global_int
      end interface

      interface scatter_global
        module procedure scatter_global_real,
     &                   scatter_global_real4,
     &                   scatter_global_int
      end interface

!***********************************************************************

      contains

!***********************************************************************

      subroutine broadcast_scalar_real(scalar, root_pe)

      implicit none

!-----------------------------------------------------------------------
!
!     broadcasts a scalar real variable from one processor (root_pe)
!     to all other processors.
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      integer (kind=int_kind), intent(in) :: 
     &   root_pe              ! processor number to broadcast from

      real (kind=dbl_kind), intent(inout) :: 
     &   scalar               ! scalar to be broadcast

      integer (kind=int_kind) :: ierr  ! local MPI error flag

!-----------------------------------------------------------------------

      call MPI_BCAST(scalar, 1, MPI_DOUBLE_PRECISION, root_pe, 
     &                       MPI_COMM_OCN, ierr)
      call MPI_BARRIER(MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end subroutine broadcast_scalar_real

!***********************************************************************

      subroutine broadcast_scalar_real4(scalar, root_pe)

      implicit none

!-----------------------------------------------------------------------
!
!     broadcasts a scalar single precision variable from one processor 
!     (root_pe) to all other processors.
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      integer (kind=int_kind), intent(in) :: 
     &   root_pe              ! processor number to broadcast from

      real (kind=real_kind), intent(inout) :: 
     &   scalar               ! scalar to be broadcast

      integer (kind=int_kind) :: ierr  ! local MPI error flag

!-----------------------------------------------------------------------

      call MPI_BCAST(scalar, 1, MPI_REAL, root_pe, 
     &                       MPI_COMM_OCN, ierr)
      call MPI_BARRIER(MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end subroutine broadcast_scalar_real4

!***********************************************************************

      subroutine broadcast_scalar_int(scalar, root_pe)

      implicit none

!-----------------------------------------------------------------------
!
!     broadcasts a scalar integer variable from one processor (root_pe)
!     to all other processors.
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      integer (kind=int_kind), intent(in) ::
     &    root_pe              ! processor number to broadcast from 

      integer (kind=int_kind), intent(inout) ::
     &   scalar                ! scalar to be broadcast

      integer (kind=int_kind) :: ierr  ! local MPI error flag

!-----------------------------------------------------------------------

      call MPI_BCAST(scalar, 1, MPI_INTEGER, root_pe, 
     &                       MPI_COMM_OCN, ierr)
      call MPI_BARRIER(MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end subroutine broadcast_scalar_int

!***********************************************************************

      subroutine broadcast_scalar_log(scalar, root_pe)

      implicit none

!-----------------------------------------------------------------------
!
!     broadcasts a scalar logical variable from one processor (root_pe)
!     to all other processors.
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      integer (kind=int_kind), intent(in) ::
     &    root_pe              ! processor number to broadcast from 

      logical (kind=log_kind), intent(inout) ::
     &   scalar                ! scalar to be broadcast

      integer (kind=int_kind) :: 
     &    reduce_int,          ! local temporary 
     &    ierr                 ! MPI error flag

!-----------------------------------------------------------------------

      if (scalar) then
        reduce_int = 1
      else
        reduce_int = 0
      endif

      call MPI_BCAST(reduce_int, 1, MPI_INTEGER, root_pe, 
     &                       MPI_COMM_OCN, ierr)
      call MPI_BARRIER(MPI_COMM_OCN, ierr)

      if (reduce_int .eq. 1) then
        scalar = .true.
      else
        scalar = .false.
      endif

!-----------------------------------------------------------------------

      end subroutine broadcast_scalar_log

!***********************************************************************

      subroutine broadcast_scalar_char(scalar, root_pe)

      implicit none

!-----------------------------------------------------------------------
!
!     broadcasts a scalar character variable from one processor 
!     (root_pe) to all other processors.
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      integer (kind=int_kind), intent(in) ::
     &    root_pe              ! processor number to broadcast from 

      character (*), intent(inout) ::
     &   scalar                ! scalar to be broadcast

      integer (kind=int_kind) :: 
     &    clength              ! length of character
     &,   ierr                 ! MPI error flag

!-----------------------------------------------------------------------

      clength = len(scalar)

      call MPI_BCAST(scalar, clength, MPI_CHARACTER, root_pe, 
     &                       MPI_COMM_OCN, ierr)
      call MPI_BARRIER(MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end subroutine broadcast_scalar_char

!***********************************************************************

      subroutine broadcast_vector_real(vector, root_pe, nelements)

      implicit none

!-----------------------------------------------------------------------
!
!     broadcasts a vector of real variables from one processor (root_pe)
!     to all other processors.
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      integer (kind=int_kind), intent(in) :: 
     &    root_pe,             ! processor number to broadcast from 
     &    nelements            ! size of vector to be broadcast

      real (kind=dbl_kind), dimension(nelements), intent(inout) :: 
     &    vector               ! vector to be broadcast

      integer (kind=int_kind) :: ierr ! local MPI error flag

!-----------------------------------------------------------------------

      call MPI_BCAST(vector, nelements, MPI_DOUBLE_PRECISION, root_pe, 
     &                       MPI_COMM_OCN, ierr)
      call MPI_BARRIER(MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end subroutine broadcast_vector_real

!***********************************************************************

      subroutine broadcast_vector_real4(vector, root_pe, nelements)

      implicit none

!-----------------------------------------------------------------------
!
!     broadcasts a vector of single precision variables from one 
!     processor (root_pe) to all other processors.
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      integer (kind=int_kind), intent(in) :: 
     &    root_pe,             ! processor number to broadcast from 
     &    nelements            ! size of vector to be broadcast

      real (kind=real_kind), dimension(nelements), intent(inout) :: 
     &    vector               ! vector to be broadcast

      integer (kind=int_kind) :: ierr ! local MPI error flag

!-----------------------------------------------------------------------

      call MPI_BCAST(vector, nelements, MPI_REAL, root_pe, 
     &                       MPI_COMM_OCN, ierr)
      call MPI_BARRIER(MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end subroutine broadcast_vector_real4

!***********************************************************************

      subroutine broadcast_vector_int(vector, root_pe, nelements)

      implicit none

!-----------------------------------------------------------------------
!
!     broadcasts a vector of integer variables from one processor 
!     (root_pe) to all other processors.
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      integer (kind=int_kind), intent(in) :: 
     &    root_pe,             ! processor number to broadcast from 
     &    nelements            ! size of vector to be broadcast

      integer (kind=int_kind), dimension(nelements), intent(inout) :: 
     &    vector               ! vector to be broadcast

      integer (kind=int_kind) :: ierr ! local MPI error flag

!-----------------------------------------------------------------------

      call MPI_BCAST(vector, nelements, MPI_INTEGER, root_pe, 
     &                       MPI_COMM_OCN, ierr)
      call MPI_BARRIER(MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end subroutine broadcast_vector_int

!***********************************************************************

      subroutine broadcast_vector_log(vector, root_pe, nelements)

      implicit none

!-----------------------------------------------------------------------
!
!     broadcasts a vector of logical variables from one processor 
!     (root_pe) to all other processors.
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      integer (kind=int_kind), intent(in) :: 
     &    root_pe,             ! processor number to broadcast from 
     &    nelements            ! size of vector to be broadcast

      logical (kind=log_kind), dimension(nelements), intent(inout) :: 
     &    vector               ! vector to be broadcast

      integer (kind=int_kind), dimension(nelements) :: 
     &    vector_int           ! temporary vector for MPI bcast 

      integer (kind=int_kind) :: ierr ! local MPI error flag

!-----------------------------------------------------------------------

      where (vector)
        vector_int = 1
      elsewhere
        vector_int = 0
      end where

      call MPI_BCAST(vector_int, nelements, MPI_INTEGER, root_pe, 
     &                       MPI_COMM_OCN, ierr)
      call MPI_BARRIER(MPI_COMM_OCN, ierr)

      where (vector_int == 1)
        vector = .true.
      elsewhere
        vector = .false.
      end where

!-----------------------------------------------------------------------

      end subroutine broadcast_vector_log

!***********************************************************************

      function global_int_sum (X, MASK)

      implicit none

!-----------------------------------------------------------------------
!
!     computes the global sum of the _physical domain_ of a 2-d
!     array.
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      integer (kind=int_kind), dimension(imt,jmt), intent(in) :: 
     &   X                    ! array to be summed

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   MASK                 ! real multiplicative mask

      integer (kind=int_kind) :: 
     &   i,j,                 ! local counters
     &   local_int_sum,       ! sum of local subdomain
     &   global_int_sum,      ! resulting sum of global domain
     &   ierr                 ! MPI error flag

!-----------------------------------------------------------------------

      local_int_sum = 0
      do j=jphys_b,jphys_e
        do i=iphys_b,iphys_e
          local_int_sum = local_int_sum + X(i,j)*MASK(i,j)
        end do
      end do
     
      call MPI_ALLREDUCE(local_int_sum, global_int_sum, 1,
     &                   MPI_INTEGER, MPI_SUM, MPI_COMM_OCN, ierr)
    
!-----------------------------------------------------------------------

      end function global_int_sum
      
!***********************************************************************

      function global_real4_sum (X, MASK)

      implicit none

!-----------------------------------------------------------------------
!
!     computes the global sum of the _physical domain_ of a 2-d
!     array.
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      real (kind=real_kind), dimension(imt,jmt), intent(in) :: 
     &   X                    ! array to be summed

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   MASK                 ! real multiplicative mask

      real (kind=real_kind) :: 
     &   local_sum,           ! sum of local subdomain
     &   global_real4_sum     ! resulting sum of global domain

      integer (kind=int_kind) :: 
     &   i,j,                 ! local counters
     &   ierr                 ! MPI error flag

!-----------------------------------------------------------------------

      local_sum = c0
      do j=jphys_b,jphys_e
        do i=iphys_b,iphys_e
          local_sum = local_sum + X(i,j)*MASK(i,j)
        end do
      end do
     
      call MPI_ALLREDUCE(local_sum, global_real4_sum, 1,
     &                   MPI_REAL, MPI_SUM, MPI_COMM_OCN, ierr)
    
!-----------------------------------------------------------------------

      end function global_real4_sum
      
!***********************************************************************

      function global_real_sum (X, MASK)

      implicit none

!-----------------------------------------------------------------------
!
!     computes the global sum of the _physical domain_ of a 2-d
!     array.
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X,                   ! array to be summed
     &   MASK                 ! real multiplicative mask

      real (kind=dbl_kind) :: 
     &   local_sum,           ! sum of local subdomain
     &   global_real_sum      ! resulting global sum

      integer (kind=int_kind) :: 
     &   i,j,                 ! local counters
     &   ierr                 ! MPI error flag

!-----------------------------------------------------------------------

      local_sum = c0
      do j=jphys_b,jphys_e
        do i=iphys_b,iphys_e
          local_sum = local_sum + X(i,j)*MASK(i,j)
        end do
      end do
     
      call MPI_ALLREDUCE(local_sum, global_real_sum, 1,
     &                   MPI_DOUBLE_PRECISION, MPI_SUM, 
     &                   MPI_COMM_OCN, ierr)
    
!-----------------------------------------------------------------------

      end function global_real_sum
      
!***********************************************************************

      function global_real_sum_prod (X,Y,MASK)

      implicit none

!-----------------------------------------------------------------------
!
!     this routine performs a global sum over the physical domain
!     of a product of two 2-d arrays.
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X,                 ! first array in product to be summed
     &   Y,                 ! second array in product to be summed
     &   MASK               ! real multiplicative mask

      real (kind=dbl_kind) :: 
     &   local_real_sum_prod,  ! sum of local subdomain
     &   global_real_sum_prod  ! global sum result

      integer (kind=int_kind) :: 
     &   i, j,              ! dummy counters
     &   ierr               ! MPI error flag

!-----------------------------------------------------------------------

      local_real_sum_prod = c0
      do j=jphys_b,jphys_e
        do i=iphys_b,iphys_e
          local_real_sum_prod = local_real_sum_prod + 
     &                          X(i,j)*Y(i,j)*MASK(i,j)
        enddo
      enddo

      call MPI_ALLREDUCE(local_real_sum_prod, global_real_sum_prod, 1,
     &                   MPI_DOUBLE_PRECISION, MPI_SUM, 
     &                   MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end function global_real_sum_prod

!***********************************************************************

      function global_real_count (MASK)

      implicit none

!-----------------------------------------------------------------------
!
!     this function returns the number of true or non-zero elements
!     in the physical domain of a 2-d array
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   MASK

      integer (kind=int_kind) :: 
     &   global_real_count,    ! resulting global count
     &   local_count,          ! local count of subdomain
     &   ierr,                 ! MPI error flag
     &   i,j                   ! dummy counters

!-----------------------------------------------------------------------

      local_count = count(MASK(iphys_b:iphys_e,jphys_b:jphys_e) /= c0)

      call MPI_ALLREDUCE(local_count, global_real_count, 1,
     &                   MPI_INTEGER, MPI_SUM, MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end function global_real_count

!***********************************************************************

      function global_real4_count (MASK)

      implicit none

!-----------------------------------------------------------------------
!
!     this function returns the number of true or non-zero elements
!     in the physical domain of a 2-d array
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      real (kind=real_kind), dimension(imt,jmt), intent(in) :: 
     &   MASK

      integer (kind=int_kind) :: 
     &   global_real4_count,   ! resulting global count
     &   local_count,          ! local count of subdomain
     &   ierr,                 ! MPI error flag
     &   i,j                   ! dummy counters

!-----------------------------------------------------------------------

      local_count = count(MASK(iphys_b:iphys_e,jphys_b:jphys_e) /= 0.0)

      call MPI_ALLREDUCE(local_count, global_real4_count, 1,
     &                   MPI_INTEGER, MPI_SUM, MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end function global_real4_count

!***********************************************************************

      function global_int_count (MASK)

      implicit none

!-----------------------------------------------------------------------
!
!     this function returns the number of true or non-zero elements
!     in the physical domain of a 2-d array
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      integer (kind=int_kind), dimension(imt,jmt), intent(in) :: 
     &   MASK

      integer (kind=int_kind) :: 
     &   global_int_count,     ! resulting global count
     &   local_count,          ! local count of subdomain
     &   ierr,                 ! MPI error flag
     &   i,j                   ! dummy counters

!-----------------------------------------------------------------------

      local_count = count(MASK(iphys_b:iphys_e,jphys_b:jphys_e) /= 0)

      call MPI_ALLREDUCE(local_count, global_int_count, 1,
     &                   MPI_INTEGER, MPI_SUM, MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end function global_int_count

!***********************************************************************

      function global_log_count (MASK)

      implicit none

!-----------------------------------------------------------------------
!
!     this function returns the number of true or non-zero elements
!     in the physical domain of a 2-d array
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      logical (kind=log_kind), dimension(imt,jmt), intent(in) :: 
     &   MASK

      integer (kind=int_kind) :: 
     &   global_log_count,     ! resulting global count
     &   local_count,          ! local count of subdomain
     &   ierr,                 ! MPI error flag
     &   i,j                   ! dummy counters

!-----------------------------------------------------------------------

      local_count = count(mask(iphys_b:iphys_e,jphys_b:jphys_e))

      call MPI_ALLREDUCE(local_count, global_log_count, 1,
     &                   MPI_INTEGER, MPI_SUM, MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end function global_log_count

!***********************************************************************
 
      function global_real_maxval (X,LMASK)

      implicit none

!-----------------------------------------------------------------------
!
!     this function computes the global maxval of the physical domain
!     of a 2-d field
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X            ! array containing field for which max required

      logical (kind=log_kind), dimension(imt,jmt), intent(in), 
     &         optional :: LMASK

      real (kind=dbl_kind) :: 
     &   global_real_maxval,  ! resulting max val of global domain
     &   local_maxval         ! max value of local subdomain

      integer (kind=int_kind) :: 
     &   ierr                 ! MPI error flag
   
!-----------------------------------------------------------------------

      if (present(LMASK)) then
        local_maxval = maxval(X(iphys_b:iphys_e,jphys_b:jphys_e),
     &               MASK=LMASK(iphys_b:iphys_e,jphys_b:jphys_e))
      else
        local_maxval = maxval(X(iphys_b:iphys_e,jphys_b:jphys_e))
      endif

      call MPI_ALLREDUCE(local_maxval, global_real_maxval, 1,
     &                   MPI_DOUBLE_PRECISION, MPI_MAX, 
     &                   MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end function global_real_maxval

!***********************************************************************
 
      function global_real4_maxval (X,LMASK)

      implicit none

!-----------------------------------------------------------------------
!
!     this function computes the global maxval of the physical domain
!     of a 2-d field
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      real (kind=real_kind), dimension(imt,jmt), intent(in) :: 
     &   X            ! array containing field for which max required

      logical (kind=log_kind), dimension(imt,jmt), intent(in), 
     &         optional :: LMASK

      real (kind=real_kind) :: 
     &   global_real4_maxval,  ! resulting max val of global domain
     &   local_maxval          ! max value of local subdomain

      integer (kind=int_kind) :: 
     &   ierr                 ! MPI error flag
   
!-----------------------------------------------------------------------

      if (present(LMASK)) then
        local_maxval = maxval(X(iphys_b:iphys_e,jphys_b:jphys_e),
     &               MASK=LMASK(iphys_b:iphys_e,jphys_b:jphys_e))
      else
        local_maxval = maxval(X(iphys_b:iphys_e,jphys_b:jphys_e))
      endif

      call MPI_ALLREDUCE(local_maxval, global_real4_maxval, 1,
     &                   MPI_REAL, MPI_MAX, MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end function global_real4_maxval

!***********************************************************************
 
      function global_int_maxval (X,LMASK)

      implicit none

!-----------------------------------------------------------------------
!
!     this function computes the global maxval of the physical domain
!     of a 2-d field
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      integer (kind=int_kind), dimension(imt,jmt), intent(in) :: 
     &   X            ! array containing field for which max required

      logical (kind=log_kind), dimension(imt,jmt), intent(in), 
     &         optional :: LMASK

      integer (kind=int_kind) :: 
     &   global_int_maxval,   ! resulting max val of global domain
     &   local_maxval,        ! max value of local subdomain
     &   ierr                 ! MPI error flag
   
!-----------------------------------------------------------------------

      if (present(LMASK)) then
        local_maxval = maxval(X(iphys_b:iphys_e,jphys_b:jphys_e),
     &               MASK=LMASK(iphys_b:iphys_e,jphys_b:jphys_e))
      else
        local_maxval = maxval(X(iphys_b:iphys_e,jphys_b:jphys_e))
      endif

      call MPI_ALLREDUCE(local_maxval, global_int_maxval, 1,
     &                   MPI_INTEGER, MPI_MAX, MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end function global_int_maxval

!***********************************************************************
 
      function global_real_minval (X, LMASK)

      implicit none

!-----------------------------------------------------------------------
!
!     this function computes the global minval of the physical domain
!     of a 2-d field
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   X            ! array containing field for which min required

      logical (kind=log_kind), dimension(imt,jmt), intent(in), 
     &         optional :: LMASK

      real (kind=dbl_kind) :: 
     &   global_real_minval,  ! resulting min val of global domain
     &   local_minval         ! min value of local subdomain

      integer (kind=int_kind) :: 
     &   ierr                 ! MPI error flag
   
!-----------------------------------------------------------------------

      if (present(LMASK)) then
        local_minval = minval(X(iphys_b:iphys_e,jphys_b:jphys_e),
     &             MASK = LMASK(iphys_b:iphys_e,jphys_b:jphys_e))
      else
        local_minval = minval(X(iphys_b:iphys_e,jphys_b:jphys_e))
      endif

      call MPI_ALLREDUCE(local_minval, global_real_minval, 1,
     &                   MPI_DOUBLE_PRECISION, MPI_MIN, 
     &                   MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end function global_real_minval

!***********************************************************************
 
      function global_real4_minval (X,LMASK)

      implicit none

!-----------------------------------------------------------------------
!
!     this function computes the global minval of the physical domain
!     of a 2-d field
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      real (kind=real_kind), dimension(imt,jmt), intent(in) :: 
     &   X            ! array containing field for which min required

      logical (kind=log_kind), dimension(imt,jmt), intent(in), 
     &         optional :: LMASK

      real (kind=real_kind) :: 
     &   global_real4_minval,  ! resulting min val of global domain
     &   local_minval          ! min value of local subdomain

      integer (kind=int_kind) :: 
     &   ierr                 ! MPI error flag
   
!-----------------------------------------------------------------------

      if (present(LMASK)) then
        local_minval = minval(X(iphys_b:iphys_e,jphys_b:jphys_e),
     &             MASK = LMASK(iphys_b:iphys_e,jphys_b:jphys_e))
      else
        local_minval = minval(X(iphys_b:iphys_e,jphys_b:jphys_e))
      endif

      call MPI_ALLREDUCE(local_minval, global_real4_minval, 1,
     &                   MPI_REAL, MPI_MIN, MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end function global_real4_minval

!***********************************************************************
 
      function global_int_minval (X,LMASK)

      implicit none

!-----------------------------------------------------------------------
!
!     this function computes the global minval of the physical domain
!     of a 2-d field
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      integer (kind=int_kind), dimension(imt,jmt), intent(in) :: 
     &   X            ! array containing field for which min required

      logical (kind=log_kind), dimension(imt,jmt), intent(in), 
     &         optional :: LMASK

      integer (kind=int_kind) :: 
     &   global_int_minval,   ! resulting min val of global domain
     &   local_minval,        ! min value of local subdomain
     &   ierr                 ! MPI error flag
   
!-----------------------------------------------------------------------

      if (present(LMASK)) then
        local_minval = minval(X(iphys_b:iphys_e,jphys_b:jphys_e),
     &             MASK = LMASK(iphys_b:iphys_e,jphys_b:jphys_e))
      else
        local_minval = minval(X(iphys_b:iphys_e,jphys_b:jphys_e))
      endif

      call MPI_ALLREDUCE(local_minval, global_int_minval, 1,
     &                   MPI_INTEGER, MPI_MIN, MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end function global_int_minval

!***********************************************************************

      subroutine global_real_maxloc(WORK,ival,jval,val)

      implicit none

!-----------------------------------------------------------------------
!
!     this routine finds the location of the global maximum for the
!     physical domain of a 2-d field
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &    WORK             ! array for which maxloc required

      real (kind=dbl_kind), intent(out) :: 
     &    val              ! maximum value of the field

      integer (kind=int_kind), intent(out) :: 
     &    ival, jval       ! i,j location of maximum

      integer (kind=int_kind) :: 
     &    i, j, n,         ! dummy counters
     &    ierr             ! MPI error flag

      real (kind=dbl_kind), dimension(3) :: 
     &    local_val        ! local array to hold local i,j,val

      real (kind=dbl_kind), dimension(0:(3*nproc_s)-1) :: 
     &    global_val       ! array to hold gathered i,j,val from others

!-----------------------------------------------------------------------

      !***
      !*** find local maxval with global location
      !***

      val = -1.0E+30
      do j=jphys_b,jphys_e
        do i=iphys_b,iphys_e
          if (WORK(i,j) .gt. val) then
            val = WORK(i,j)
            local_val(1) = val
            local_val(2) = i_global(i)
            local_val(3) = j_global(j)
          endif
        end do
      end do

      !***
      !*** gather all similar results from other processors
      !***

      call MPI_ALLGATHER(local_val , 3, MPI_DOUBLE_PRECISION,
     &                   global_val, 3, MPI_DOUBLE_PRECISION,
     &                   MPI_COMM_OCN, ierr)

      !***
      !*** find max and location by comparing the gathered results
      !***

      val  = -1.0E+30
      ival = 0
      jval = 0
      do n=0,3*nproc_s-1,3
        if(global_val(n) .GT. val) then
          val  = global_val(n  )
          ival = global_val(n+1)
          jval = global_val(n+2)
        end if
      end do

!-----------------------------------------------------------------------

      end subroutine global_real_maxloc

!***********************************************************************

      function global_real_scalar_maxval (local_scalar)

!-----------------------------------------------------------------------
!
!     this function returns the maximum scalar value across processors
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      real (kind=dbl_kind), intent(inout) :: 
     &   local_scalar                ! local scalar to be compared

      real (kind=dbl_kind) :: 
     &   global_real_scalar_maxval   ! resulting global max

      integer (kind=int_kind) :: ierr ! MPI error flag
    
!-----------------------------------------------------------------------

      call MPI_ALLREDUCE(local_scalar, global_real_scalar_maxval, 1,
     &                   MPI_DOUBLE_PRECISION, MPI_MAX, 
     &                   MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end function global_real_scalar_maxval

!***********************************************************************

      function global_real_scalar_minval (local_scalar)

      implicit none

!-----------------------------------------------------------------------
!
!     this function returns the minimum scalar value across processors
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      real (kind=dbl_kind), intent(inout) :: 
     &   local_scalar                ! local scalar to be compared

      real (kind=dbl_kind) :: 
     &   global_real_scalar_minval   ! resulting global min

      integer (kind=int_kind) :: ierr ! MPI error flag
    
!-----------------------------------------------------------------------

      call MPI_ALLREDUCE(local_scalar, global_real_scalar_minval, 1,
     &                   MPI_DOUBLE_PRECISION, MPI_MIN, 
     &                   MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end function global_real_scalar_minval

!***********************************************************************

      function global_real_scalar_sum (local_scalar)

      implicit none

!-----------------------------------------------------------------------
!
!     this function returns the sum of scalar value across processors
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

      real (kind=dbl_kind), intent(inout) :: 
     &   local_scalar                ! local scalar to be compared

      real (kind=dbl_kind) :: 
     &   global_real_scalar_sum   ! resulting global sum

      integer (kind=int_kind) :: ierr ! MPI error flag
    
!-----------------------------------------------------------------------

      call MPI_ALLREDUCE(local_scalar, global_real_scalar_sum, 1,
     &                   MPI_DOUBLE_PRECISION, MPI_SUM, 
     &                   MPI_COMM_OCN, ierr)

!-----------------------------------------------------------------------

      end function global_real_scalar_sum

!***********************************************************************

      subroutine gather_global_int(ARRAY_G, ARRAY, dst_task)

      implicit none

!-----------------------------------------------------------------------
!
!     this routine gathers a distributed array and strips off ghost
!     cells to create a local array with global dimensions - used in 
!     coupling routines and I/O for gathering 2-d global information
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   dst_task ! task which will hold global result array

      integer (kind=int_kind), dimension(imt,jmt), intent(in) :: 
     &   ARRAY ! array to be gathered

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt_global,jmt_global), 
     &                         intent(out) :: 
     &    ARRAY_G ! global array

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n, ier, send_req
      integer (kind=int_kind) :: recv_req(nproc_s)
      integer (kind=int_kind) :: recv_status(MPI_STATUS_SIZE,nproc_s),
     &        send_status(MPI_STATUS_SIZE)

!-----------------------------------------------------------------------
!
!     exchange messages with processors one at a time
!     destination task receives chunks from processors who own them
!
!-----------------------------------------------------------------------

      if (my_task == dst_task) then
        do n=1,nproc_s
          call MPI_IRECV(ARRAY_G(local_block(1,n),local_block(2,n)),
     &                  1, mpi_interior_int_global(n), n-1,
     &                  mpitag_gthr, MPI_COMM_OCN, recv_req(n), ier)
        end do
      endif

!-----------------------------------------------------------------------
!
!     current processor sends its local chunk to the destination task
!
!-----------------------------------------------------------------------

      call MPI_ISEND(ARRAY(iphys_b,jphys_b), 
     &               1, mpi_interior_int(my_task+1), dst_task,
     &               mpitag_gthr, MPI_COMM_OCN, send_req, ier)
      call MPI_WAIT(send_req, send_status, ier)

!-----------------------------------------------------------------------
!
!     wait for completion and unpack local buffer into global buffer
!
!-----------------------------------------------------------------------

      if (my_task .eq. dst_task) then
        call MPI_WAITALL(nproc_s, recv_req, recv_status, ier)
      endif

      call MPI_barrier(MPI_COMM_OCN, ier)

!-----------------------------------------------------------------------

      end subroutine gather_global_int

!***********************************************************************

      subroutine gather_global_real4(ARRAY_G, ARRAY, dst_task)

      implicit none

!-----------------------------------------------------------------------
!
!     this routine gathers a distributed array and strips off ghost
!     cells to create a local array with global dimensions - used in 
!     coupling routines (and I/O?) for gathering 2-d global information
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   dst_task ! task which will hold global result array

      real(kind=real_kind), dimension(imt,jmt), intent(in) :: 
     &   ARRAY ! array to be gathered

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      real(kind=real_kind), dimension(imt_global,jmt_global), 
     &    intent(out) :: 
     &    ARRAY_G ! global array

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n, ier, send_req
      integer (kind=int_kind) :: recv_req(nproc_s)
      integer (kind=int_kind) :: recv_status(MPI_STATUS_SIZE,nproc_s),
     &        send_status(MPI_STATUS_SIZE)

!-----------------------------------------------------------------------
!
!     exchange messages with processors one at a time
!     destination task receives chunks from processors who own them
!
!-----------------------------------------------------------------------

      if (my_task == dst_task) then
        do n=1,nproc_s
          call MPI_IRECV(ARRAY_G(local_block(1,n),local_block(2,n)),
     &                   1, mpi_interior_real_global(n), n-1,
     &                   mpitag_gthr, MPI_COMM_OCN, recv_req(n), ier)
        end do
      endif

!-----------------------------------------------------------------------
!
!     current processor sends its local chunk to the destination task
!
!-----------------------------------------------------------------------

      call MPI_ISEND(ARRAY(iphys_b,jphys_b), 
     &               1, mpi_interior_real(my_task+1), dst_task,
     &               mpitag_gthr, MPI_COMM_OCN, send_req, ier)
      call MPI_WAIT(send_req, send_status, ier)

!-----------------------------------------------------------------------
!
!     wait for completion and unpack local buffer into global buffer
!
!-----------------------------------------------------------------------

      if (my_task .eq. dst_task) then
        call MPI_WAITALL(nproc_s, recv_req, recv_status, ier)
      endif

      call MPI_barrier(MPI_COMM_OCN, ier)

!-----------------------------------------------------------------------

      end subroutine gather_global_real4

!***********************************************************************

      subroutine gather_global_real(ARRAY_G, ARRAY, dst_task)

      implicit none

!-----------------------------------------------------------------------
!
!     this routine gathers a distributed array and strips off ghost
!     cells to create a local array with global dimensions - used in 
!     coupling routines (and I/O?) for gathering 2-d global information
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   dst_task ! task which will hold global result array

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   ARRAY ! array to be gathered

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt_global,jmt_global), 
     &    intent(out) :: 
     &    ARRAY_G ! global array

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n, ier, send_req
      integer (kind=int_kind) :: recv_req(nproc_s)
      integer (kind=int_kind) :: recv_status(MPI_STATUS_SIZE,nproc_s),
     &        send_status(MPI_STATUS_SIZE)

!-----------------------------------------------------------------------
!
!     exchange messages with processors one at a time
!     destination task receives chunks from processors who own them
!
!-----------------------------------------------------------------------

      if (my_task == dst_task) then
        do n=1,nproc_s
          call MPI_IRECV(ARRAY_G(local_block(1,n),local_block(2,n)),
     &                   1, mpi_interior_dble_global(n), n-1,
     &                   mpitag_gthr, MPI_COMM_OCN, recv_req(n), ier)
        end do
      endif

!-----------------------------------------------------------------------
!
!     current processor sends its local chunk to the destination task
!
!-----------------------------------------------------------------------

      call MPI_ISEND(ARRAY(iphys_b,jphys_b), 
     &               1, mpi_interior_dble(my_task+1), dst_task,
     &               mpitag_gthr, MPI_COMM_OCN, send_req, ier)
      call MPI_WAIT(send_req, send_status, ier)

!-----------------------------------------------------------------------
!
!     wait for completion and unpack local buffer into global buffer
!
!-----------------------------------------------------------------------

      if (my_task .eq. dst_task) then
        call MPI_WAITALL(nproc_s, recv_req, recv_status, ier)
      endif

      call MPI_barrier(MPI_COMM_OCN, ier)

!-----------------------------------------------------------------------

      end subroutine gather_global_real

!***********************************************************************

      subroutine scatter_global_int(ARRAY, ARRAY_G, src_task)

      implicit none

!-----------------------------------------------------------------------
!
!     this routine scatters a global array and adds ghost
!     cells to create a distributed array - used by coupling routines
!     (and I/O?) for distributing 2-d global information
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &    src_task ! task from which to scatter global array

      integer (kind=int_kind), dimension(imt_global,jmt_global), 
     &    intent(in) :: 
     &    ARRAY_G ! global array to be distributed

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(out) :: 
     &    ARRAY ! local part of distributed array

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n, ier, recv_req, send_req
      integer (kind=int_kind) :: send_status(MPI_STATUS_SIZE),
     &        recv_status(MPI_STATUS_SIZE)

!-----------------------------------------------------------------------
!
!     send chunks to processors who own them and fill ghost cells
!
!-----------------------------------------------------------------------

      call MPI_IRECV(ARRAY(iphys_b,jphys_b), 
     &               1, mpi_interior_int(my_task+1), src_task,
     &               mpitag_io, MPI_COMM_OCN, recv_req, ier)

      if (my_task .eq. src_task) then
        do n=1,nproc_s
          call MPI_ISEND(ARRAY_G(local_block(1,n),local_block(2,n)),
     &                   1, mpi_interior_int_global(n), n-1,
     &                   mpitag_io, MPI_COMM_OCN, send_req, ier)
          call MPI_WAIT(send_req, send_status, ier)
        end do
      endif

      call MPI_WAIT(recv_req, recv_status, ier)
      call MPI_barrier(MPI_COMM_OCN,ier)
      call boundary_2d(ARRAY)
      call MPI_barrier(MPI_COMM_OCN,ier)

!-----------------------------------------------------------------------

      end subroutine scatter_global_int

!***********************************************************************

      subroutine scatter_global_real4(ARRAY, ARRAY_G, src_task)

      implicit none

!-----------------------------------------------------------------------
!
!     this routine scatters a global array and adds ghost
!     cells to create a distributed array - used by coupling routines
!     (and I/O?) for distributing 2-d global information
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &    src_task ! task from which to scatter global array

      real (kind=real_kind), dimension(imt_global,jmt_global), 
     &    intent(in) :: 
     &    ARRAY_G ! global array to be distributed

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      real (kind=real_kind), dimension(imt,jmt), intent(out) :: 
     &    ARRAY ! local part of distributed array

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n, ier, recv_req, send_req
      integer (kind=int_kind) :: send_status(MPI_STATUS_SIZE),
     &        recv_status(MPI_STATUS_SIZE)

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &    WORK ! for converting between real/double

!-----------------------------------------------------------------------
!
!     send chunks to processors who own them and fill ghost cells
!
!-----------------------------------------------------------------------

      call MPI_IRECV(ARRAY(iphys_b,jphys_b), 
     &               1, mpi_interior_real(my_task+1), src_task,
     &               mpitag_io, MPI_COMM_OCN, recv_req, ier)

      if (my_task .eq. src_task) then
        do n=1,nproc_s
          call MPI_ISEND(ARRAY_G(local_block(1,n),local_block(2,n)),
     &                   1, mpi_interior_real_global(n), n-1,
     &                   mpitag_io, MPI_COMM_OCN, send_req, ier)
          call MPI_WAIT(send_req, send_status, ier)
        end do
      endif

      call MPI_WAIT(recv_req, recv_status, ier)
      call MPI_barrier(MPI_COMM_OCN,ier)
      WORK = ARRAY
      call boundary_2d(WORK)
      call MPI_barrier(MPI_COMM_OCN,ier)
      ARRAY = WORK

!-----------------------------------------------------------------------

      end subroutine scatter_global_real4

!***********************************************************************

      subroutine scatter_global_real(ARRAY, ARRAY_G, src_task)

      implicit none

!-----------------------------------------------------------------------
!
!     this routine scatters a global array and adds ghost
!     cells to create a distributed array - used by coupling routines
!     (and I/O?) for distributing 2-d global information
!
!-----------------------------------------------------------------------

      include 'mpif.h'  ! MPI Fortran include file

!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &    src_task ! task from which to scatter global array

      real (kind=dbl_kind), dimension(imt_global,jmt_global), 
     &    intent(in) :: 
     &    ARRAY_G ! global array to be distributed

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &    ARRAY ! local part of distributed array

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n, ier, recv_req, send_req
      integer (kind=int_kind) :: send_status(MPI_STATUS_SIZE),
     &        recv_status(MPI_STATUS_SIZE)

!-----------------------------------------------------------------------
!
!     send chunks to processors who own them and fill ghost cells
!
!-----------------------------------------------------------------------

      call MPI_IRECV(ARRAY(iphys_b,jphys_b), 
     &               1, mpi_interior_dble(my_task+1), src_task,
     &               mpitag_io, MPI_COMM_OCN, recv_req, ier)

      if (my_task .eq. src_task) then
        do n=1,nproc_s
          call MPI_ISEND(ARRAY_G(local_block(1,n),local_block(2,n)),
     &                   1, mpi_interior_dble_global(n), n-1,
     &                   mpitag_io, MPI_COMM_OCN, send_req, ier)
          call MPI_WAIT(send_req, send_status, ier)
        end do
      endif

      call MPI_WAIT(recv_req, recv_status, ier)
      call MPI_barrier(MPI_COMM_OCN,ier)
      call boundary_2d(ARRAY)
      call MPI_barrier(MPI_COMM_OCN,ier)

!-----------------------------------------------------------------------

      end subroutine scatter_global_real

!***********************************************************************

      end module global_reductions

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
