!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module io

!***********************************************************************
!
!     This module provides a generic parallel input/output interface
!     for writing arrays.  The module also contains an i/o unit 
!     manager.
!
!     CVS:$Id: io.F,v 1.3.2.1 2001/03/13 22:54:25 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use communicate
      use global_reductions
      use exit_mod

      implicit none
      save

!-----------------------------------------------------------------------
!
!     module variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), parameter ::
     &   max_units = 99        ! maximum number of open units
     &,  stdin     =  5        ! reserved unit for standard input
     &,  stdout    =  6        ! reserved unit for standard output
     &,  stderr    =  6        ! reserved unit for standard error
     &,  nml_in    = 10        ! reserved unit for namelist input
     &,  recl_int  =  4        ! record length factor for integers
     &,  recl_real =  4        ! record length factor for reals
     &,  recl_dbl  =  8        ! record length factor for doubles

      logical (kind=log_kind), dimension(max_units) ::
     &   in_use                ! flag=.true. if unit currently open

      logical (kind=log_kind) ::
     &   lredirect_stdout      ! redirect stdout to log file
     &,  luse_pointer_files    ! use pointer files to determine
                               !   name/location of restart files

      character (char_len) ::
     &   log_filename          ! root name for log file
     &,  pointer_filename      ! root name of pointer files

      integer (kind=int_kind) ::
     &   num_iotasks           ! number of processes doing i/o

      integer (kind=int_kind), dimension(max_units) :: 
     &   nrec                  ! record num for direct access file
                               ! attached to a unit

      character (6), parameter :: 
     &   nml_filename = 'pop_in'  ! input namelist filename

!-----------------------------------------------------------------------
!
!     generic interfaces for i/o routines
!
!-----------------------------------------------------------------------

      interface write_array
         module procedure write_int_2d,
     &                    write_real4_2d,
     &                    write_real8_2d,
     &                    write_int_3d,
     &                    write_real4_3d,
     &                    write_real8_3d
      end interface

      interface read_array
         module procedure read_int_2d,
     &                    read_real4_2d,
     &                    read_real8_2d,
     &                    read_int_3d,
     &                    read_real4_3d,
     &                    read_real8_3d
      end interface

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_io

!-----------------------------------------------------------------------
!
!     this routine initializes some i/o arrays and checks the validity
!     of the i/o processor number
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  nml_error             ! namelist i/o error flag
     &, cindx,cindx2          ! indices into character strings

      character (8) ::
     &  logdate               ! actual date at model startup 
                              !   (not simulation date)

      character (10) ::
     &  logtime               ! wallclock time at model startup

      character (char_len) ::
     &  char_tmp              ! temp character string for filenames

      namelist /io_nml/ num_iotasks
     &,                 lredirect_stdout, log_filename
     &,                 luse_pointer_files, pointer_filename

!-----------------------------------------------------------------------

      in_use = .false.            ! no unit in use

      in_use(stdin)  = .true.    ! reserved units
      in_use(stdout) = .true.
      in_use(stderr) = .true.
      in_use(nml_in)  = .true.

      nrec = 0

      lredirect_stdout   = .false.
      log_filename       = 'pop.out'
      luse_pointer_files = .false.
      pointer_filename = 'pop_pointer'
      num_iotasks = min(km, nproc_s)  ! set default num io tasks

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=io_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading io_nml')
      endif

      call broadcast_scalar(num_iotasks,        master_task)
      call broadcast_scalar(lredirect_stdout,   master_task)
      call broadcast_scalar(log_filename,       master_task)
      call broadcast_scalar(luse_pointer_files, master_task)
      call broadcast_scalar(pointer_filename,   master_task)

      if (num_iotasks > nproc_s) then
        num_iotasks = min(km, nproc_s) ! reset num iotasks if too big
      endif

!-----------------------------------------------------------------------
!
!     redirect stdout to a log file if requested
!
!-----------------------------------------------------------------------

      if (lredirect_stdout .and. my_task == master_task) then
        char_tmp = log_filename
        log_filename = char_blank
        cindx2 = len_trim(char_tmp)
        log_filename(1:cindx2) = trim(char_tmp)

!        call date_and_time(date=logdate,time=logtime)
!
!        cindx2 = cindx2 + 1
!        log_filename(cindx2:cindx2) = '.'
!        cindx  = cindx2 + 1
!        cindx2 = cindx + 7
!        log_filename(cindx:cindx2) = logdate
!        cindx2 = cindx2 + 1
!        log_filename(cindx2:cindx2) = '.'
!        cindx  = cindx2 + 1
!        cindx2 = cindx + 5
!        log_filename(cindx:cindx2) = logtime(1:6)
!
!        open(stdout, file=log_filename, form='formatted', 
!     &               status='unknown')
        open(stdout, file=log_filename, position='append')
      endif

!-----------------------------------------------------------------------

      end subroutine init_io

!***********************************************************************

      subroutine get_unit(iunit)

!-----------------------------------------------------------------------
!
!     this routine returns the next available i/o unit
!     note that _all_ processors must call get_unit (even if only
!     the master_task is doing the i/o) to insure that
!     the in_use and nrec arrays remain synchronized.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     output variable
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(out) ::
     &   iunit                 ! next free i/o unit

!-----------------------------------------------------------------------
!
!     find next free unit
!
!-----------------------------------------------------------------------

      srch_units: do iunit=1,max_units
        if (.not. in_use(iunit)) exit srch_units
      end do srch_units

      in_use(iunit) = .true.    ! mark iunit as being in use
      nrec(iunit) = 1           ! reset record number for unit

!-----------------------------------------------------------------------

      end subroutine get_unit

!***********************************************************************

      subroutine release_unit(iunit)

!-----------------------------------------------------------------------
!
!     this routine releases an i/o unit (marks it as available).
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variable
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &   iunit                 ! i/o unit to be released

!-----------------------------------------------------------------------

      in_use(iunit) = .false.

!-----------------------------------------------------------------------

      end subroutine release_unit

!***********************************************************************

      subroutine open_parallel_file(nu,filename,recl_factor)

!-----------------------------------------------------------------------
!
!     this routine opens a file for parallel i/o
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   recl_factor      ! factor for multiplying grid size to get
                          ! record length

      character (*), intent(in) ::
     &   filename   ! file to open and assign to unit nu

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(out) :: 
     &   nu         ! unit number attached to newly-opened file

!-----------------------------------------------------------------------

      call get_unit(nu)

      if (my_task < num_iotasks) then
        open(nu,file=filename,access='direct',form='unformatted',
     &          recl=imt_global*jmt_global*recl_factor,status='unknown')
      endif

!-----------------------------------------------------------------------

      end subroutine open_parallel_file

!***********************************************************************

      subroutine close_parallel_file(nu)

!-----------------------------------------------------------------------
!
!     this routine closes a parallel file
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   nu         ! unit number to close

!-----------------------------------------------------------------------

      if (my_task .lt. num_iotasks) then
        close(nu)
      endif
      call release_unit(nu)

!-----------------------------------------------------------------------

      end subroutine close_parallel_file

!***********************************************************************

      subroutine read_int_2d(nu,INT2D)

!-----------------------------------------------------------------------
!
!     reads an integer horizontal slab array
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   nu                      ! file unit number

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(out) :: 
     &   INT2D                  ! array to be read

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt_global,jmt_global) :: 
     &   IOBUFI                ! local global-sized buffer

!-----------------------------------------------------------------------
!
!     read in global 2-d slice from one processor
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        read(nu,rec=nrec(nu)) IOBUFI
      endif

      nrec(nu) = nrec(nu) + 1

!-----------------------------------------------------------------------
!
!     send chunks to processors who own them
!
!-----------------------------------------------------------------------

      call scatter_global(INT2D, IOBUFI, master_task)

!-----------------------------------------------------------------------

      end subroutine read_int_2d

!***********************************************************************

      subroutine read_real4_2d(nu,REAL2D)

!-----------------------------------------------------------------------
!
!     reads a 32-bit real horizontal slab array
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   nu                      ! file unit number

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      real (kind=real_kind), dimension(imt,jmt), intent(out) :: 
     &   REAL2D  ! array to be read

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=real_kind), dimension(imt_global,jmt_global) ::
     &   IOBUFR   ! local global-sized buffer array

!-----------------------------------------------------------------------
!
!     read in global 2-d slice from one processor
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        read(nu,rec=nrec(nu)) IOBUFR
      endif

      nrec(nu) = nrec(nu) + 1

!-----------------------------------------------------------------------
!
!     send chunks to processors who own them and fill ghost cells
!
!-----------------------------------------------------------------------

      call scatter_global(REAL2D, IOBUFR, master_task)

!-----------------------------------------------------------------------

      end subroutine read_real4_2d

!***********************************************************************

      subroutine read_real8_2d(nu,REAL2D)

!-----------------------------------------------------------------------
!
!     reads a 64-bit real horizontal slab array
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   nu                      ! file unit number

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &   REAL2D  ! array to be read

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt_global,jmt_global) :: 
     &   IOBUFD     ! temporary global array buffer

!-----------------------------------------------------------------------
!
!     read in global 2-d slice from one processor
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        read(nu,rec=nrec(nu)) IOBUFD
      endif

      nrec(nu) = nrec(nu) + 1

!-----------------------------------------------------------------------
!
!     send chunks to processors who own them and fill ghost cells
!
!-----------------------------------------------------------------------

      call scatter_global(REAL2D, IOBUFD, master_task)

!-----------------------------------------------------------------------

      end subroutine read_real8_2d

!***********************************************************************

      subroutine read_int_3d(nu,INT3D)

!-----------------------------------------------------------------------
!
!     read 3-d integer array
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   nu                      ! file unit number

!-----------------------------------------------------------------------
!
!     output variables - declared inout to use automatic array
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(:,:,:), intent(inout) :: 
     &   INT3D  ! array to be read

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) k,n,krecs,klvl,nz,ierr

      integer (kind=int_kind), dimension(imt_global,jmt_global) :: 
     &   IOBUFI                ! global-sized array buffer

!-----------------------------------------------------------------------
!
!     determine the number of records each i/o process must read to
!     get all the records
!
!-----------------------------------------------------------------------

      nz = size(INT3D, DIM=3)

      if (mod(nz,num_iotasks) == 0) then
        krecs = nz/num_iotasks
      else
        krecs = nz/num_iotasks + 1
      endif

!-----------------------------------------------------------------------
!
!     each i/o process reads a horizontal slab from a record and sends
!     the data to processors who own it. read and distribute num_iotasks
!     records at a time to keep the messages from getting mixed up.
!
!-----------------------------------------------------------------------

      do k=1,krecs
        call MPI_barrier(MPI_COMM_OCN,ierr)

        if (my_task .lt. num_iotasks) then
          klvl = (k-1)*num_iotasks + my_task + 1
          if (klvl .le. nz) then
            read(nu, rec=nrec(nu)+klvl-1) IOBUFI
          endif
        endif

        do n=1,num_iotasks
          klvl = (k-1)*num_iotasks + n
          if (klvl .le. nz) then
            call scatter_global(INT3D(:,:,klvl), IOBUFI, n-1)
          endif
        end do

      end do

      nrec(nu) = nrec(nu) + nz

!-----------------------------------------------------------------------

      end subroutine read_int_3d

!***********************************************************************

      subroutine read_real4_3d(nu,REAL3D)

!-----------------------------------------------------------------------
!
!     read 3-d 32-bit real array
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   nu                    ! file unit number

!-----------------------------------------------------------------------
!
!     output variables - declared inout to use assumed-size array 
!
!-----------------------------------------------------------------------

      real (kind=real_kind), dimension(:,:,:), intent(inout) :: 
     &   REAL3D              ! array to read

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   k,n,       ! dummy counters
     &   krecs,     ! number of records each iotask must read
     &   klvl,      ! k level associated with a record
     &   nz,        ! number of levels in array to be read
     &   ierr       ! MPI error flag

      real (kind=real_kind), dimension(imt_global, jmt_global) :: 
     &   IOBUFR     ! local global-sized buffer

!-----------------------------------------------------------------------
!
!     determine the number of records each i/o process must read to
!     get all the records
!
!-----------------------------------------------------------------------

      nz = size(REAL3D, DIM=3)

      if (mod(nz,num_iotasks) .eq. 0) then
        krecs = nz/num_iotasks
      else
        krecs = nz/num_iotasks + 1
      endif

!-----------------------------------------------------------------------
!
!     each i/o process reads a horizontal slab from a record and sends
!     the data to processors who own it. read and distribute num_iotasks
!     records at a time to keep the messages from getting mixed up.
!
!-----------------------------------------------------------------------

      do k=1,krecs
        call MPI_barrier(MPI_COMM_OCN,ierr)

        if (my_task .lt. num_iotasks) then
          klvl = (k-1)*num_iotasks + my_task + 1
          if (klvl .le. nz) then
            read(nu, rec=nrec(nu)+klvl-1) IOBUFR
          endif
        endif

        do n=1,num_iotasks
          klvl = (k-1)*num_iotasks + n
          if (klvl .le. nz) then
            call scatter_global(REAL3D(:,:,klvl),IOBUFR,n-1)
          endif
        end do

      end do

      nrec(nu) = nrec(nu) + nz

!-----------------------------------------------------------------------

      end subroutine read_real4_3d

!***********************************************************************

      subroutine read_real8_3d(nu,REAL3D)

!-----------------------------------------------------------------------
!
!     read 3-d 64-bit real array
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   nu                    ! file unit number

!-----------------------------------------------------------------------
!
!     output variables - declared inout to use assumed-size array 
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(:,:,:), intent(inout) :: 
     &   REAL3D             ! array to read

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   k,n,       ! dummy counters
     &   krecs,     ! number of records each iotask must read
     &   klvl,      ! k level associated with a record
     &   nz,        ! number of levels in array to be read
     &   ierr       ! MPI error flag

      real (kind=dbl_kind), dimension(imt_global, jmt_global) :: 
     &   IOBUFD     ! local global-sized buffer

!-----------------------------------------------------------------------
!
!     determine the number of records each i/o process must read to
!     get all the records
!
!-----------------------------------------------------------------------

      nz = size(REAL3D, DIM=3)

      if (mod(nz,num_iotasks) .eq. 0) then
        krecs = nz/num_iotasks
      else
        krecs = nz/num_iotasks + 1
      endif

!-----------------------------------------------------------------------
!
!     each i/o process reads a horizontal slab from a record and sends
!     the data to processors who own it. read and distribute num_iotasks
!     records at a time to keep the messages from getting mixed up.
!
!-----------------------------------------------------------------------

      do k=1,krecs
        call MPI_barrier(MPI_COMM_OCN,ierr)

        if (my_task .lt. num_iotasks) then
          klvl = (k-1)*num_iotasks + my_task + 1
          if (klvl .le. nz) then
            read(nu, rec=nrec(nu)+klvl-1) IOBUFD
          endif
        endif

        do n=1,num_iotasks
          klvl = (k-1)*num_iotasks + n
          if (klvl .le. nz) then
            call scatter_global(REAL3D(:,:,klvl),IOBUFD,n-1)
          endif
        end do

      end do

      nrec(nu) = nrec(nu) + nz

!-----------------------------------------------------------------------

      end subroutine read_real8_3d

!***********************************************************************

      subroutine write_int_2d(nu,INT2D)

!-----------------------------------------------------------------------
!
!     writes a 2-d integer array
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   nu                    ! file unit number

      integer (kind=int_kind), dimension(imt,jmt), intent(in) :: 
     &   INT2D                 ! array to be written

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt_global, jmt_global) :: 
     &   IOBUFI                ! local global-sized buffer

!-----------------------------------------------------------------------
!
!     receive chunks from processors who own them
!
!-----------------------------------------------------------------------

      call gather_global(IOBUFI, INT2D, master_task)

!-----------------------------------------------------------------------
!
!     write global 2-d slice from one processor
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        write(nu,rec=nrec(nu)) IOBUFI
      endif
      nrec(nu) = nrec(nu) + 1

!-----------------------------------------------------------------------

      end subroutine write_int_2d

!***********************************************************************

      subroutine write_real4_2d(nu,REAL2D)

!-----------------------------------------------------------------------
!
!     writes a 2-d 32-bit real array
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   nu                    ! file unit number

      real (kind=real_kind), dimension(imt,jmt), intent(in) :: 
     &   REAL2D              ! array to write

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=real_kind), dimension(imt_global, jmt_global) :: 
     &   IOBUFR              ! global-sized array buffer

!-----------------------------------------------------------------------
!
!     receive chunks from processors who own them
!
!-----------------------------------------------------------------------

      call gather_global(IOBUFR, REAL2D, master_task)

!-----------------------------------------------------------------------
!
!     write global 2-d slice from one processor
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        write(nu,rec=nrec(nu)) IOBUFR
      endif
      nrec(nu) = nrec(nu) + 1

!-----------------------------------------------------------------------

      end subroutine write_real4_2d

!***********************************************************************

      subroutine write_real8_2d(nu,REAL2D)

!-----------------------------------------------------------------------
!
!     writes a 2-d 64-bit real array
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   nu                    ! file unit number

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   REAL2D             ! array to write

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt_global, jmt_global) :: 
     &   IOBUFD             ! global-sized buffer

!-----------------------------------------------------------------------
!
!     receive chunks from processors who own them
!
!-----------------------------------------------------------------------

      call gather_global(IOBUFD, REAL2D, master_task)

!-----------------------------------------------------------------------
!
!     write global 2-d slice from one processor
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        write(nu,rec=nrec(nu)) IOBUFD
      endif
      nrec(nu) = nrec(nu) + 1

!-----------------------------------------------------------------------

      end subroutine write_real8_2d

!***********************************************************************

      subroutine write_int_3d(nu,INT3D)

!-----------------------------------------------------------------------
!
!     writes a 3-d integer array
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   nu                    ! file unit number

      integer (kind=int_kind), dimension(:,:,:), intent(in) :: 
     &   INT3D                 ! array to be written

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   k, n,      ! dummy counters
     &   krecs,     ! number of records each iotask must write
     &   klvl,      ! k index corresponding to current record
     &   nz,        ! size of 3rd dimension of 3-d array to be written
     &   ierr       ! MPI error flag

      integer (kind=int_kind), dimension(imt_global, jmt_global) :: 
     &   IOBUFI                ! global-sized buffer array

!-----------------------------------------------------------------------
!
!     determine the number of records each i/o process must write to
!     get all the records
!
!-----------------------------------------------------------------------

      nz = size(INT3D, DIM=3)

      if (mod(nz,num_iotasks) == 0) then
        krecs = nz/num_iotasks
      else
        krecs = nz/num_iotasks + 1
      endif

!-----------------------------------------------------------------------
!
!     gather and write num_iotasks records at a time to keep the 
!     messages from getting mixed up.
!
!-----------------------------------------------------------------------

      do k=1,krecs
        call MPI_barrier(MPI_COMM_OCN,ierr)

!-----------------------------------------------------------------------
!
!       gather a global slice on each iotask
!
!-----------------------------------------------------------------------

        do n=1,num_iotasks
          klvl = (k-1)*num_iotasks + n
          if (klvl <= nz) then
            call gather_global(IOBUFI, INT3D(:,:,klvl), n-1)
          endif
        end do

!-----------------------------------------------------------------------
!
!       iotasks wait for the gather to be complete and write global
!       slabs to the file
!
!-----------------------------------------------------------------------

        if (my_task < num_iotasks) then
          klvl = (k-1)*num_iotasks + my_task + 1
          if (klvl <= nz) then
            write(nu, rec=nrec(nu)+klvl-1) IOBUFI
          endif
        endif

!-----------------------------------------------------------------------

      end do
      nrec(nu) = nrec(nu) + nz

!-----------------------------------------------------------------------

      end subroutine write_int_3d

!***********************************************************************

      subroutine write_real4_3d(nu,REAL3D)

!-----------------------------------------------------------------------
!
!     writes a 3-d 32-bit real array
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   nu                    ! file unit number

      real (kind=real_kind), dimension(:,:,:), intent(in) :: 
     &   REAL3D              ! array to write

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   k, n,      ! dummy counters
     &   krecs,     ! number of records each iotask must write
     &   klvl,      ! k index corresponding to current record
     &   nz,        ! size of 3rd dimension of 3-d array to be written
     &   ierr       ! MPI error flag

      real (kind=real_kind), dimension(imt_global, jmt_global) :: 
     &   IOBUFR     ! global-sized array buffer

!-----------------------------------------------------------------------
!
!     determine the number of records each i/o process must write to
!     get all the records and convert to single precision.
!
!-----------------------------------------------------------------------

      nz = size(REAL3D, DIM=3)

      if (mod(nz,num_iotasks) == 0) then
        krecs = nz/num_iotasks
      else
        krecs = nz/num_iotasks + 1
      endif

!-----------------------------------------------------------------------
!
!     gather and write num_iotasks records at a time to keep the 
!     messages from getting mixed up.
!
!-----------------------------------------------------------------------

      do k=1,krecs

        call MPI_barrier(MPI_COMM_OCN,ierr)
!-----------------------------------------------------------------------
!
!       gather global slice on each iotask
!
!-----------------------------------------------------------------------

        do n=1,num_iotasks
          klvl = (k-1)*num_iotasks + n
          if (klvl <= nz) then
            call gather_global(IOBUFR, REAL3D(:,:,klvl), n-1)
          endif
        end do

!-----------------------------------------------------------------------
!
!       iotasks wait for the gather to be complete and write global
!       slabs to the file
!
!-----------------------------------------------------------------------

        if (my_task < num_iotasks) then
          klvl = (k-1)*num_iotasks + my_task + 1
          if (klvl <= nz) then
            write(nu, rec=nrec(nu)+klvl-1) IOBUFR
          endif
        endif

!-----------------------------------------------------------------------

      end do
      nrec(nu) = nrec(nu) + nz

c-----------------------------------------------------------------------

      end subroutine write_real4_3d


!***********************************************************************

      subroutine write_real8_3d(nu,REAL3D)

!-----------------------------------------------------------------------
!
!     writes a 3-d 64-bit real array
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &   nu                    ! file unit number

      real (kind=dbl_kind), dimension(:,:,:), intent(in) :: 
     &   REAL3D             ! array to write

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   k, n,      ! dummy counters
     &   krecs,     ! number of records each iotask must write
     &   klvl,      ! k index corresponding to current record
     &   nz,        ! size of 3rd dimension of 3-d array to be written
     &   ierr       ! MPI error flag

      real (kind=dbl_kind), dimension(imt_global, jmt_global) :: 
     &   IOBUFD     ! global-sized buffer array

!-----------------------------------------------------------------------
!
!     determine the number of records each i/o process must write to
!     get all the records
!
!-----------------------------------------------------------------------

      nz = size(REAL3D, DIM=3)

      if (mod(nz,num_iotasks) == 0) then
        krecs = nz/num_iotasks
      else
        krecs = nz/num_iotasks + 1
      endif

!-----------------------------------------------------------------------
!
!     gather and write num_iotasks records at a time to keep the 
!     messages from getting mixed up.
!
!-----------------------------------------------------------------------

      do k=1,krecs
        call MPI_barrier(MPI_COMM_OCN,ierr)

!-----------------------------------------------------------------------
!
!       gather global slice on each iotask
!
!-----------------------------------------------------------------------

        do n=1,num_iotasks
          klvl = (k-1)*num_iotasks + n
          if (klvl <= nz) then
            call gather_global(IOBUFD, REAL3D(:,:,klvl), n-1)
          endif
        end do

!-----------------------------------------------------------------------
!
!       iotasks wait for the gather to be complete and write global
!       slabs to the file
!
!-----------------------------------------------------------------------

        if (my_task < num_iotasks) then
          klvl = (k-1)*num_iotasks + my_task + 1
          if (klvl <= nz) then
            write(nu, rec=nrec(nu)+klvl-1) IOBUFD
          endif
        endif

!-----------------------------------------------------------------------

      end do
      nrec(nu) = nrec(nu) + nz

!-----------------------------------------------------------------------

      end subroutine write_real8_3d

!***********************************************************************

      end module io

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
