!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module advection

!***********************************************************************
!
!     this module contains arrays and variables necessary for performing
!     advection of momentum and tracer quantities.  both standard
!     advection and 3-rd order upwinding advection are available for
!     tracers.
!
!     CVS:$Id: advection.F,v 1.3.2.4 2001/09/07 19:51:40 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use prognostic
      use stencils
      use operators
      use grid
      use diagnostics
      use tavg
      use time_management
      use exit_mod
      use horizontal_mix
      use timers

      implicit none
      save

!-----------------------------------------------------------------------
!
!     flux velocities times horizontal length 
!     (see comments in 'adv_flux')
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension (imt,jmt) :: 
     &   UTE,UTW,VTN,VTS,  ! tracer flux velocities across E,W,N,S faces
     &   UUE,UUW,VUN,VUS,  ! momen  flux velocities across E,W,N,S faces
     &   WUK,              ! vert velocity at top of U box on level k
     &   WTK               ! vert velocity at top of T box on level k

!-----------------------------------------------------------------------
!
!     choices for tracer advection
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), parameter ::
     &  tadvect_standard = 1,  ! standard leap-frog tracer advection
     &  tadvect_upwind3  = 2   ! 3rd-order upwind tracer advection

      integer (kind=int_kind) ::
     &  tadvect_itype

!-----------------------------------------------------------------------
!
!     geometric arrays necessary for upwind advection
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(:), allocatable ::
     &  talfzp,tbetzp,tgamzp,
     &  talfzm,tbetzm,tdelzm

      real (kind=dbl_kind), dimension(:,:), allocatable ::
     &  TALFXP,TBETXP,TGAMXP,TALFYP,TBETYP,TGAMYP,
     &  TALFXM,TBETXM,TDELXM,TALFYM,TBETYM,TDELYM

      integer (kind=int_kind), private ::
     &  timer_advt
     &, timer_advu
 
!***********************************************************************

      contains

!***********************************************************************

      subroutine init_advection 

!-----------------------------------------------------------------------
!
!     This subroutine initializes variables associated with advection
!     schemes.
!
!-----------------------------------------------------------------------

      character (char_len) ::
     &  tadvect_ctype      ! character string for tracer advect choice

      namelist /advect_nml/ tadvect_ctype

!-----------------------------------------------------------------------
!
!     local variables for setting up upwind coefficients
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  i,j,k,                ! dummy loop indices
     &  nml_error             ! error flag for namelist read

      real (kind=dbl_kind), dimension(:,:), allocatable :: 
     &  DXC,DXCW,DXCE,DXCE2,
     &  DYC,DYCS,DYCN,DYCN2          

      real (kind=dbl_kind), dimension(:), allocatable :: 
     &  dzc

!-----------------------------------------------------------------------
!
!     read input namelist for choice of tracer advection method
!
!-----------------------------------------------------------------------

      tadvect_ctype = 'standard'

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=advect_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      end if

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading advection namelist')
      endif
     
      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout,advect_nml)
          write(stdout,*) ' '
      endif
 
      if (my_task == master_task) then
        write(stdout,1000)
        write(stdout,*)
        write(stdout,*) 'Advection options'
        write(stdout,*)
        write(stdout,1000)
 1000   format(72('-'))

        select case (tadvect_ctype(1:6))

        case ('standa')
          tadvect_itype = tadvect_standard
          write(stdout,*) 'Using leapfrog advection for tracers.'
        case ('upwind')
          tadvect_itype = tadvect_upwind3
          write(stdout,*) 
     &            'Using 3rd-order upwind advection for tracers.'
        case default
          tadvect_itype = -1000
        end select

      endif
      call broadcast_scalar(tadvect_itype, master_task)

      if (tadvect_itype < 0) then
        call exit_POP('ERROR: unknown tracer advection method')
      endif

!-----------------------------------------------------------------------
!
!     initialize advection timers
!
!-----------------------------------------------------------------------
 
      select case (tadvect_itype)
      case (tadvect_standard)
        call get_timer(timer_advt, 'ADVECTION_STANDARD_TRACER')
      case (tadvect_upwind3)
        call get_timer(timer_advt, 'ADVECTION_UPWIND3_TRACER')
      end select
 
      call get_timer(timer_advu, 'ADVECTION_MOMENTUM')
 
!-----------------------------------------------------------------------
!
!     initialize flux velocities
!
!-----------------------------------------------------------------------

      UTE = c0
      UTW = c0
      VTN = c0
      VTS = c0
      UUE = c0
      UUW = c0
      VUN = c0
      VUS = c0

!-----------------------------------------------------------------------
!
!     allocate and initialize upwinding grid arrays if necessary
!
!-----------------------------------------------------------------------

      if (tadvect_itype == tadvect_upwind3) then

        allocate (talfzp(km),
     &            tbetzp(km),
     &            tgamzp(km),
     &            talfzm(km),
     &            tbetzm(km),
     &            tdelzm(km))

        allocate (TALFXP(imt,jmt),
     &            TBETXP(imt,jmt),
     &            TGAMXP(imt,jmt),
     &            TALFYP(imt,jmt),
     &            TBETYP(imt,jmt),
     &            TGAMYP(imt,jmt),
     &            TALFXM(imt,jmt),
     &            TBETXM(imt,jmt),
     &            TDELXM(imt,jmt),
     &            TALFYM(imt,jmt),
     &            TBETYM(imt,jmt),
     &            TDELYM(imt,jmt),
     &            KMTNN (imt,jmt),
     &            KMTEE (imt,jmt))

        allocate (dzc(0:km+1))

        !***
        !*** vertical grid coefficients     
        !***

        dzc(0)    = dz(1)
        do k=1,km
          dzc(k)  = dz(k)
        enddo
        dzc(km+1) = dzc(km)

        do k=1,km-1
          talfzp(k) =  dz(k)*(c2*dz(k)+dzc(k-1))/
     &                ((dz(k)+dz(k+1))*(dzc(k-1)+c2*dz(k)+dz(k+1)))
          tbetzp(k) =  dz(k+1)*(c2*dz(k)+dzc(k-1))/
     &               ((dz(k)+dz(k+1))*(dz(k)+dzc(k-1)          ))
          tgamzp(k) =  -(dz(k)*dz(k+1))/
     &               ((dz(k)+dzc(k-1))*(dz(k+1)+dzc(k-1)+c2*dz(k)))
        enddo 
        tbetzp(1) = tbetzp(1) + tgamzp(1)
        tgamzp(1) = c0
        talfzp(km) = c0
        tbetzp(km) = c0
        tgamzp(km) = c0
 
        do k=1,km-1
          talfzm(k) =  dz(k)*(c2*dz(k+1)+dzc(k+2))/
     &               ((dz(k)+dz(k+1))*(dz(k+1)+dzc(k+2)            ))
          tbetzm(k) =  dz(k+1)*(c2*dz(k+1)+dzc(k+2))/
     &               ((dz(k)+dz(k+1))*(dz(k)+dzc(k+2)+c2*dz(k+1)))
          tdelzm(k) =  -(dz(k)*dz(k+1))/
     &               ((dz(k+1)+dzc(k+2))*(dz(k)+dzc(k+2)+c2*dz(k+1)))
        enddo    
        talfzm(km-1) = talfzm(km-1) + tdelzm(km-1)
        tdelzm(km-1) = c0
        talfzm(km) = c0
        tbetzm(km) = c0
        tdelzm(km) = c0

        deallocate (dzc)

        !***
        !*** zonal grid coefficients     
        !***

        allocate (DXC  (imt,jmt),
     &            DXCW (imt,jmt),
     &            DXCE (imt,jmt),
     &            DXCE2(imt,jmt))

        DXC = DXT      
        call w_shift(DXCW,DXC) 
        call e_shift(DXCE,DXC) 
        call e_shift(DXCE2,DXCE) 

        TALFXP = DXC *(c2*DXC+DXCW)/((DXC+DXCE)*(DXCW+c2*DXC+DXCE))
        TBETXP = DXCE*(c2*DXC+DXCW)/((DXC+DXCW)*(        DXC+DXCE))
        TGAMXP =     -(   DXC*DXCE)/((DXC+DXCW)*(DXCW+c2*DXC+DXCE))
 
        TALFXM = DXC *(c2*DXCE+DXCE2)/((DXC+DXCE)*(       DXCE+DXCE2))
        TBETXM = DXCE*(c2*DXCE+DXCE2)/((DXC+DXCE)*(DXC+c2*DXCE+DXCE2))
        TDELXM =     -(   DXC *DXCE )/((DXCE2+DXCE)*(DXC+c2*DXCE+DXCE2))

        deallocate (DXC, DXCW, DXCE, DXCE2)

        !***
        !*** poloidal grid coefficients     
        !***

        allocate (DYC  (imt,jmt),
     &            DYCS (imt,jmt),
     &            DYCN (imt,jmt),
     &            DYCN2(imt,jmt))

        DYC = DYT      
        call s_shift(DYCS,DYC) 
        call n_shift(DYCN,DYC) 
        call n_shift(DYCN2,DYCN) 

        TALFYP = DYC *(c2*DYC+DYCS)/((DYC+DYCN)*(DYCS+c2*DYC+DYCN))
        TBETYP = DYCN*(c2*DYC+DYCS)/((DYC+DYCN)*(DYCS+   DYC     ))
        TGAMYP =     -(   DYC*DYCN)/((DYC+DYCS)*(DYCS+c2*DYC+DYCN))
 
        TALFYM = DYC *(c2*DYCN+DYCN2)/((DYC+DYCN)*(       DYCN+DYCN2))
        TBETYM = DYCN*(c2*DYCN+DYCN2)/((DYC+DYCN)*(DYC+c2*DYCN+DYCN2))
        TDELYM =     -(   DYC *DYCN )/((DYCN2+DYCN)*(DYC+c2*DYCN+DYCN2))

        deallocate (DYC, DYCS, DYCN, DYCN2)

        call n_shift(KMTNN,KMTN)
        call e_shift(KMTEE,KMTE)

      endif

!-----------------------------------------------------------------------

      end subroutine init_advection 

!***********************************************************************

      subroutine adv_flux(k)

!-----------------------------------------------------------------------
!
!     calculate advection fluxes at level k on
!     faces of T boxes (Ut,Vt) and U boxes (Uu,Vu).
!
!     Ut = Ay(U*dy)    ! flux velocities times horizontal length
!     Vt = Ax(V*dx)
!
!     Uu = Ax{Ay(Ut*dy)}
!     Vu = Ay{Ax(Vt*dx)}
!
!   * the advection flux on a face of a U box is the average of the
!     corresponding fluxes on the four surrounding T boxes.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k ! depth level index

!-----------------------------------------------------------------------
!
!     local variables:
!
!-----------------------------------------------------------------------
 
      real (kind=dbl_kind), dimension (imt,jmt) :: 
     &  WORK                ! local temp space

!-----------------------------------------------------------------------
!
!     advection fluxes for T box.
!
!-----------------------------------------------------------------------

      WORK = p5*UVEL(:,:,k,curtime)*DYU
      call s_eoshift(UTE,WORK)
      UTE = WORK + UTE
      call w_eoshift(UTW,UTE)

      WORK = p5*VVEL(:,:,k,curtime)*DXU
      call w_eoshift(VTN,WORK)
      VTN = WORK + VTN
      call s_eoshift(VTS,VTN)

!-----------------------------------------------------------------------
!
!     advection fluxes for U box.
!
!-----------------------------------------------------------------------

      WORK = p25*(UTE + UTW)
      call n_eoshift(UUW,WORK)
      UUW = WORK + UUW
   
      WORK = p25*(VTN + VTS)
      call e_eoshift(VUS,WORK)
      VUS = WORK + VUS

!-----------------------------------------------------------------------

      end subroutine adv_flux
      
!***********************************************************************

      subroutine advu(k,LUK,LVK)

!-----------------------------------------------------------------------
!
!     advection of momentum
!
!     L(U) = (1/dy)*Dx(dy*Uu*Ax(U)) + (1/dx)*Dy(dx*Vu*Ay(U))
!             + Dz(Wu*Az(U))
!             + U*V*ky - V*V*kx   ! metric term
!
!     L(V) = (1/dy)*Dx(dy*Uu*Ax(V)) + (1/dx)Dy(dx*Vu*Ay(V))
!             + Dz(Wu*Az(V))
!             + U*V*kx - U*U*ky   ! metric term
!
!     where   kx = (1/dy)Dx(dy)
!             ky = (1/dx)Dy(dx)
!
!     comments:
!
!   * this routine must be called successively with k = 1,2,3,...
!
!   * the horizontal part of the advection operator is a  5-point
!     stencil.  the coefficients of this stencil consist of the
!     the advection velocities multiplied by geometric factors.
!
!   * the vertical velocity Wu in U columns is determined by 
!     integrating the continuity equation L(1) = 0 from the surface
!     down to level k.  in the rigid-lid formulation, the integration
!     starts with Wu = 0 at the surface.  in the free-surface 
!     formulation, the integration starts with Wu = the area-weighted
!     average of dh/dt at surrounding T points (dh/dt is the time
!     change of the surface height, and satisfies the barotropic 
!     continuity equation dh/dt + Div{H*([U],[V])} = 0, where ([U],[V]) 
!     is the barotropic velocity).
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k ! depth level index

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &  LUK,                ! advection of U-momentum
     &  LVK                 ! advection of V-momentum

!-----------------------------------------------------------------------
!
!     local variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  FVN,FUE,            ! stencil coeffs used by tavg
     &  WUKB                ! vert velocity at bottom of level k U box

!      include 'drifters.h'

       
      call timer_start (timer_advu)
 
!-----------------------------------------------------------------------
!
!     set vertical velocity at surface
!
!-----------------------------------------------------------------------
 
      if (k == 1) WUK = DHU  ! free surface

!-----------------------------------------------------------------------
!
!     compute north and east velocity fluxes from south and west fluxes
!     if implicit vertical mixing, this has been called from a 
!     different k-loop than adv_flux and the fluxes have been stored
!     in UVEL,VVEL(newtime).
!
!-----------------------------------------------------------------------

#if impvmix
      UUW = UVEL(:,:,k,newtime)
      VUS = VVEL(:,:,k,newtime)
#endif
      call e_eoshift(UUE,UUW)
      call n_eoshift(VUN,VUS)

!-----------------------------------------------------------------------
!
!     calculate vertical velocity at bottom of kth level
!     (vertical velocity is nonzero at bottom of U columns
!     if topography is varying)
!
!-----------------------------------------------------------------------

      WUKB = WUK + c2dz(k)*p5*(VUN - VUS + UUE - UUW)*UAREA_R

!#if drifter_particles
!#if drifter_particles != 2
!!-----------------------------------------------------------------------
!!
!!     save the vertical velocity for advecting drifters
!!
!!-----------------------------------------------------------------------
!      WVEL(:,:,k) = WUKB
!#endif
!#endif

!-----------------------------------------------------------------------
!
!     advect momentum
!
!     horizontal advection
!
!-----------------------------------------------------------------------

      call advu_fivept(LUK,LVK,UUW,VUS,UAREA_R,
     &                 UVEL(:,:,k,curtime), VVEL(:,:,k,curtime))

!-----------------------------------------------------------------------
!
!     vertical advection through top of U box
!
!-----------------------------------------------------------------------

      if (k == 1) then
        LUK = LUK + dzr(k)*WUK*UVEL(:,:,k,curtime)
        LVK = LVK + dzr(k)*WUK*VVEL(:,:,k,curtime)
      else
        LUK = LUK + dz2r(k)*WUK*(UVEL(:,:,k-1,curtime) + 
     &                           UVEL(:,:,k  ,curtime))
        LVK = LVK + dz2r(k)*WUK*(VVEL(:,:,k-1,curtime) + 
     &                           VVEL(:,:,k  ,curtime))
      endif

!-----------------------------------------------------------------------
!
!     vertical advection through bottom of U box
!     for k=km, UVEL(k+1) is not defined, but WUKB=0
!
!-----------------------------------------------------------------------

      if (k < km) then  
        LUK = LUK - dz2r(k)*WUKB*(UVEL(:,:,k  ,curtime) + 
     &                            UVEL(:,:,k+1,curtime))
        LVK = LVK - dz2r(k)*WUKB*(VVEL(:,:,k  ,curtime) + 
     &                            VVEL(:,:,k+1,curtime))
      endif

!-----------------------------------------------------------------------
!
!     add metric terms, and zero fields at land points
!
!-----------------------------------------------------------------------

      where (k <= KMU)
        LUK = LUK + UVEL(:,:,k,curtime)*VVEL(:,:,k,curtime)*KYU - 
     &              VVEL(:,:,k,curtime)**2*KXU
        LVK = LVK + UVEL(:,:,k,curtime)*VVEL(:,:,k,curtime)*KXU - 
     &              UVEL(:,:,k,curtime)**2*KYU
      elsewhere
        LUK = c0
        LVK = c0
      endwhere

!-----------------------------------------------------------------------
!
!     accumulate time average if necessary
!
!-----------------------------------------------------------------------

      if (ltavg_on .and. mix_pass /= 1) then
        FVN =  VUN*p5*UAREA_R
        FUE =  UUE*p5*UAREA_R
        call tavg_advu(k,FUE,FVN,WUK)
      endif

!-----------------------------------------------------------------------
!
!     reset W for next k level
!
!-----------------------------------------------------------------------

      WUK = WUKB   ! bottom value becomes top value for next pass

      call timer_stop  (timer_advu)
 
!-----------------------------------------------------------------------

      end subroutine advu

!***********************************************************************

      subroutine advt(k,LTK)

!-----------------------------------------------------------------------
!
!     advection of tracers - this routine actually computes only
!     vertical velocities and then calls individual routines based
!     on the type of advection chosen
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! depth level index

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,nt), intent(out) ::
     &  LTK                 ! returned as L(T) for nth tracer at level k

!-----------------------------------------------------------------------
!
!     local or common variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n   ! dummy tracer index

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  FC,                 ! local temp space
     &  FVN,FUE,            ! north and east advective stencil weights
     &  WTKB                ! vertical velocity at bottom of T box

 
        call timer_start (timer_advt)
 
!-----------------------------------------------------------------------
!
!     set vertical velocity at surface
!     w = DH = dh/dt or dh/dt - Fw depending on surface type
!
!-----------------------------------------------------------------------

      if (k == 1) WTK = DH
 
      if ( sfc_layer_type /= sfc_layer_varthick .and.
     &     k == 1 .and. ltavg_on .and. mix_pass /= 1)
     &  call tavg_free_sfc_resid_flx

      if (ldiag_global) then

        if (k == 1) then
          where (CALCT) DIAG_PE_2D = dzw(0)*grav*RHO(:,:,1,curtime)*WTK

          if (sfc_layer_type /= sfc_layer_varthick) then
            do n=1,nt
              where (CALCT) 
     &          DIAG_TRACER_ADV_2D(:,:,n) = WTK*TRACER(:,:,1,n,curtime)
              sfc_tracer_flux(n) = 
     &          global_sum(DIAG_TRACER_ADV_2D(:,:,n)*TAREA,RCALCT)
            end do
          endif

        else

          where (k <= KMT)
            DIAG_PE_2D = DIAG_PE_2D + dzw(k-1)*WTK*grav*p5*
     &                  (RHO(:,:,k-1,curtime) + RHO(:,:,k,curtime))
          endwhere

        endif

      endif

!-----------------------------------------------------------------------
!
!     calculate vertical velocity at bottom of kth level
!     (vertical velocity is zero at bottom of T columns)
!
!-----------------------------------------------------------------------

      FC = p5*(VTN - VTS + UTE - UTW)*TAREA_R
      WTKB = merge(WTK+c2dz(k)*FC, c0, k < KMT)

!-----------------------------------------------------------------------
!
!     advect tracers
!
!-----------------------------------------------------------------------

      select case (tadvect_itype)

      case (tadvect_upwind3)

        call advt_upwind3(k,LTK,WTKB)
 
      case(tadvect_standard)

        call advt_standard(k,LTK,WTKB)

      case default

        call exit_POP('ERROR: unknown advection type for tracers')

      end select

!-----------------------------------------------------------------------
!
!     compute diagnostics if necessary
!
!-----------------------------------------------------------------------

      if (ltavg_on .and. mix_pass /= 1 ) then
        FVN =  p5*VTN*TAREA_R
        FUE =  p5*UTE*TAREA_R
        call tavg_advt(k,FUE,FVN,WTK,WTKB,LTK)
      endif

      call cfl_advect(k,UTE,UTW,VTN,VTS,WTK,WTKB)

!-----------------------------------------------------------------------
!
!     set bottom vertical velocity to top for next k-level
!
!-----------------------------------------------------------------------

      WTK = WTKB

        call timer_stop  (timer_advt)
!-----------------------------------------------------------------------

      end subroutine advt

!***********************************************************************

      subroutine advt_standard(k,LTK,WTKB)

!-----------------------------------------------------------------------
!
!     this subroutine computes leap-frog advection
!
!     L(T) = (1/dy)*Dx(dy*Ut*Ax(T)) + (1/dx)*Dy(dx*Vt*Ay(T))
!             + Dz(Wt*Az(T))
!
!     where:
!
!       Ut = Ay(U*dy)    ! flux velocities times horizontal length
!       Vt = Ax(V*dx)
!
!       Uu = Ax{Ay(Ut*dy)}
!       Vu = Ay{Ax(Vt*dx)}
!
!     comments:
!
!   * this routine must be called successively with k = 1,2,3,...
!
!   * the horizontal part of the advection operator is a  5-point
!     stencil.  the coefficients of this stencil consist of the
!     the advection velocities multiplied by geometric factors.
!
!   * the vertical velocity Wt in T columns is determined by
!     integrating the continuity equation L(1) = 0 from the surface
!     down to level k.  in the rigid-lid formulation, the integration
!     starts with Wt = 0 at the surface.  in the free-surface
!     formulation, the integration starts with Wt = dh/dt at the
!     surface (dh/dt is the time change of the surface height,
!     and satisfies the barotropic continuity equation:
!     dh/dt + Div{H*([U],[V])} = 0, where  ([U],[V]) is the 
!     barotropic velocity).  dh/dt is calculated in routine 'dhdt'.
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! depth level index

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &  WTKB                ! vert velocity at bottom of level k T box

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,nt), intent(out) ::
     &  LTK                 ! returned as L(T) for nth tracer at level k

!-----------------------------------------------------------------------
!
!     local variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n   ! tracer loop index

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  WORK1, WORK2

!-----------------------------------------------------------------------
!
!     advect tracers
!
!     horizontal advection
!
!-----------------------------------------------------------------------

      WORK1 = VTN
      WORK2 = UTE

      if ( hmix_tracer_itype == hmix_tracer_type_gm
     &                                  .and. gm_bolus ) then
        WORK1 = WORK1 + VISOP * HTN
        WORK2 = WORK2 + UISOP * HTE
      endif

      call advt_fivept(k, LTK, WORK1, WORK2, TAREA_R,
     &                 TRACER(:,:,:,:,curtime))

!-----------------------------------------------------------------------
!
!     vertical advection through top and bottom of T box
!
!-----------------------------------------------------------------------

      WORK1 = WTK
      WORK2 = WTKB

      if ( hmix_tracer_itype == hmix_tracer_type_gm
     &                                  .and. gm_bolus ) then
        WORK1 = WORK1 + WISOPT
        WORK2 = WORK2 + WISOPB
      endif

      do n = 1,nt

        !*** no advection thru surface (k=1) in vs model

        if (k == 1) then
          if (sfc_layer_type /= sfc_layer_varthick) then
            LTK(:,:,n) = LTK(:,:,n) + 
     &                   dzr(k)*WTK*TRACER(:,:,k,n,curtime)
          endif
        else
          LTK(:,:,n) = LTK(:,:,n) +
     &                 dz2r(k)*WORK1*(TRACER(:,:,k-1,n,curtime) + 
     &                                TRACER(:,:,k  ,n,curtime))
        endif

        !***
        !*** for k=km, TRACER(k+1) is not defined, but WORK2=0
        !***

        if (k < km) then
          LTK(:,:,n) = LTK(:,:,n) -
     &                 dz2r(k)*WORK2*(TRACER(:,:,k  ,n,curtime) + 
     &                                TRACER(:,:,k+1,n,curtime))
        endif

      enddo

!-----------------------------------------------------------------------

      end subroutine advt_standard

!***********************************************************************

      subroutine advt_upwind3(k,LTK,WTKB)

!-----------------------------------------------------------------------
!
!     advection of tracers using 3rd-order upwinding
!
!     L(T) = (1/dy)*Dx(dy*Ut*Ax(T)) + (1/dx)*Dy(dx*Vt*Ay(T))
!             + Dz(Wt*Az(T))
!
!     where:
!
!       Ut = Ay(U*dy)    ! flux velocities times horizontal length
!       Vt = Ax(V*dx)
!
!       Uu = Ax{Ay(Ut*dy)}
!       Vu = Ay{Ax(Vt*dx)}
!
!     comments:
!
!   * this routine must be called successively with k = 1,2,3,...
!
!   * the horizontal part of the advection operator is a  5-point
!     stencil.  the coefficients of this stencil consist of the
!     the advection velocities multiplied by geometric factors.
!
!   * the vertical velocity Wt in T columns is determined by
!     integrating the continuity equation L(1) = 0 from the surface
!     down to level k.  in the rigid-lid formulation, the integration
!     starts with Wt = 0 at the surface.  in the free-surface
!     formulation, the integration starts with Wt = dh/dt at the
!     surface (dh/dt is the time change of the surface height,
!     and satisfies the barotropic continuity equation:
!     dh/dt + Div{H*([U],[V])} = 0, where  ([U],[V]) is the 
!     barotropic velocity).  dh/dt is calculated in routine 'dhdt'.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! depth level index

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &  WTKB                ! vert velocity at bottom of level k T box

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,nt), intent(out) ::
     &  LTK                 ! returned as L(T) for nth tracer at level k

!-----------------------------------------------------------------------
!
!     local variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n   ! tracer loop index

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  FVN,FVS,FUE,FUW,    ! stencil weights
     &  TPLUS,TMINUS,AZMINUS,DZMINUS, WORK1, WORK2

      real (kind=dbl_kind), dimension(imt,jmt,nt), save :: 
     &  AUX

      real (kind=dbl_kind), dimension(imt,jmt,nt) :: 
     &  AUXB

!-----------------------------------------------------------------------
!
!     advect tracers
!
!     horizontal advection
!
!-----------------------------------------------------------------------

      where ( k < KMT-1 )
        AZMINUS = talfzm(k)
        DZMINUS = tdelzm(k)
      elsewhere
        AZMINUS = talfzm(k) + tdelzm(k)
        DZMINUS = c0
      endwhere

      FVN =  p5*VTN*TAREA_R
      FVS = -p5*VTS*TAREA_R  ! note sign change
      FUE =  p5*UTE*TAREA_R
      FUW = -p5*UTW*TAREA_R  ! note sign change

      if ( hmix_tracer_itype == hmix_tracer_type_gm
     &                                  .and. gm_bolus ) then

        WORK1 = VISOP * HTN
        call s_shift( WORK2, WORK1 )
        FVN = FVN + p5*WORK1*TAREA_R
        FVS = FVS - p5*WORK2*TAREA_R

        WORK1 = UISOP * HTE
        call w_shift( WORK2, WORK1 )
        FUE = FUE + p5*WORK1*TAREA_R
        FUW = FUW - p5*WORK2*TAREA_R

      endif

      call hupw3(k,LTK,TRACER(:,:,:,:,curtime),FVN,FVS,FUE,FUW,
     &                 KMTN,KMTS,KMTE,KMTW,KMTNN,KMTEE,
     &                 TALFXP,TBETXP,TGAMXP,TALFYP,TBETYP,TGAMYP,
     &                 TALFXM,TBETXM,TDELXM,TALFYM,TBETYM,TDELYM,
     &                 talfzp,tbetzp,tgamzp,talfzm,tbetzm,tdelzm)

!-----------------------------------------------------------------------
!
!     vertical advection through top and bottom of T box
!
!-----------------------------------------------------------------------

      WORK1 = WTKB
      if ( hmix_tracer_itype == hmix_tracer_type_gm  .and. gm_bolus )
     &  WORK1 = WORK1 + WISOPB

      do n = 1,nt

        if ( k < km-1 .and. k > 1) then
          TPLUS = talfzp(k)*TRACER(:,:,k+1,n,curtime) 
     &          + tbetzp(k)*TRACER(:,:,k,n,curtime)
     &          + tgamzp(k)*TRACER(:,:,k-1,n,curtime)
          TMINUS = AZMINUS  *TRACER(:,:,k+1,n,curtime) 
     &           + tbetzm(k)*TRACER(:,:,k,n,curtime)
     &           + DZMINUS  *TRACER(:,:,k+2,n,curtime)
          AUXB(:,:,n) = (WORK1-abs(WORK1))*TPLUS +
     &                  (WORK1+abs(WORK1))*TMINUS
        else if ( k == 1) then
          TPLUS = talfzp(k)*TRACER(:,:,k+1,n,curtime) 
     &          + tbetzp(k)*TRACER(:,:,k,n,curtime)
          TMINUS = AZMINUS  *TRACER(:,:,k+1,n,curtime) 
     &           + tbetzm(k)*TRACER(:,:,k,n,curtime)
     &           + DZMINUS  *TRACER(:,:,k+2,n,curtime)
          AUXB(:,:,n) = (WORK1-abs(WORK1))*TPLUS +
     &                  (WORK1+abs(WORK1))*TMINUS
        else if ( k == km-1) then
          TPLUS = talfzp(k)*TRACER(:,:,k+1,n,curtime) 
     &          + tbetzp(k)*TRACER(:,:,k,n,curtime)
     &          + tgamzp(k)*TRACER(:,:,k-1,n,curtime)
          TMINUS = AZMINUS  *TRACER(:,:,k+1,n,curtime) 
     &         + tbetzm(k)*TRACER(:,:,k,n,curtime)
          AUXB(:,:,n) = (WORK1-abs(WORK1))*TPLUS +
     &                  (WORK1+abs(WORK1))*TMINUS
        else
          AUXB(:,:,n) = c0
        endif

        !*** no advection thru surface (k=1) in vs model

        if (k == 1) then
          if (sfc_layer_type /= sfc_layer_varthick) then
            LTK(:,:,n) = LTK(:,:,n)
     &                 + dzr(k)*WTK*TRACER(:,:,k,n,curtime) 
     &                 - dz2r(k)*AUXB(:,:,n)
          else
            LTK(:,:,n) = LTK(:,:,n)
     &                 - dz2r(k)*AUXB(:,:,n)
          endif
        else
          LTK(:,:,n) = LTK(:,:,n)
     &               + dz2r(k)*(AUX(:,:,n) - AUXB(:,:,n))
        endif

        AUX(:,:,n) = AUXB(:,:,n) ! set bottom to top for next k level

      enddo

!-----------------------------------------------------------------------

      end subroutine advt_upwind3

!***********************************************************************

      end module advection

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
