!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module baroclinic

!***********************************************************************
!
!     contains routines and variables for computing the baroclinic
!     velocities and tracer fields.
!
!     CVS:$Id: baroclinic.F,v 1.3.2.4 2001/09/07 19:51:42 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use prognostic
      use global_reductions
      use boundary
      use grid
      use advection
      use pressure_grad
      use convection
      use horizontal_mix
      use vertical_mix
      use diagnostics
      use state_mod
      use ice
      use time_management
      use io
      use tavg
      use forcing
      use forcing_shf
      use forcing_coupled
      use exit_mod
      use forcing_sfwf

      implicit none
      save

!-----------------------------------------------------------------------
!
!     module variables
!
!-----------------------------------------------------------------------

      logical (kind=log_kind) ::
     &  reset_to_freezing     ! flag to prevent very cold water

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_baroclinic

!-----------------------------------------------------------------------
!
!     initializes some baroclinic options
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  nml_error             ! namelist i/o error flag

      namelist /baroclinic_nml/ reset_to_freezing

!-----------------------------------------------------------------------
!
!     read options from namelist and broadcast
!
!-----------------------------------------------------------------------

      reset_to_freezing = .true.

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=baroclinic_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      end if

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR: reading baroclinic_nml')
      endif

      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, baroclinic_nml) 
          write(stdout,*) ' '
      endif
 
      if (my_task == master_task) then
        write(stdout,1000)
        write(stdout,*)
        write(stdout,*) 'Baroclinic options'
        write(stdout,*)
        write(stdout,1000)
 1000   format(72('-')) 
        if (reset_to_freezing) then
          write(stdout,*) 'Surface temperature reset to freezing on'
        else
          write(stdout,*) 'Surface temperature reset to freezing off'
        endif
      endif

      call broadcast_scalar(reset_to_freezing,master_task)

!-----------------------------------------------------------------------

      end subroutine init_baroclinic

!***********************************************************************

      subroutine baroclinic_driver

!-----------------------------------------------------------------------
!
!     explicit time integration of baroclinic velocities 
!     (Up,Vp) and tracer fields T. 
!
!     tracer equations:
!
!       (Ta-Tb)/(2*dt) = -L(T) + HDiff(Tb) + VDiff(Tb)
!                        + source term
!
!     momentum equations:
!
!       (Upp-Ub)/(2*dt) - f*alpha*(Vpp-Vb) = Fx
!       (Vpp-Vb)/(2*dt) + f*alpha*(Upp-Ub) = Fy
!
!       Up = Upp - [Upp]
!       Vp = Vpp - [Vpp]
!
!   * this routine calculates baroclinic velocities and tracers at
!     the new time level and stores them in arrays UVEL,VVEL,and TRACER
!     with time index newtime.  the arrays UVEL,VVEL, and TRACER
!     with time indices curtime and oldtime are not updated for the next
!     timestep until near the end of routine step.
!
!   * the above equations are written for the case of (leapfrog)
!     implicit treatment of the coriolis terms.  if these terms are
!     treated explicitly then the coriolis terms appear only in the 
!     forcing terms (Fx,Fy), which are calculated in clinic. 
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  n,k,                ! dummy indices for vertical level, tracer
     &  kp1,km1             ! level index for k+1, k-1 levels

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  FX,FY,              ! sum of r.h.s. forcing terms
     &  WORK1,WORK2         ! local work space

!-----------------------------------------------------------------------
!
!     initialize arrays for vertical sums.
!
!-----------------------------------------------------------------------

      ZX = c0
      ZY = c0

!-----------------------------------------------------------------------
!
!     main loop over vertical levels.
!
!-----------------------------------------------------------------------

      do k = 1,km 

        kp1 = k+1
        km1 = k-1
        if (k.eq.1) km1 = 1
        if (k.eq.km) kp1 = km

!-----------------------------------------------------------------------
!
!       compute vertical viscosity and diffusion coeffs
!
!-----------------------------------------------------------------------

        call vmix(k)

!-----------------------------------------------------------------------
!
!       calculate horizontal advection velocities at level k
!       for T and U boxes.
!
!-----------------------------------------------------------------------

        call adv_flux(k)

!-----------------------------------------------------------------------
!
!       calculate level k tracers at new time
!
!-----------------------------------------------------------------------

        call tracer_update(k)
         
#if impvmix

!-----------------------------------------------------------------------
!
!       temporarily store flux velocity in UVEL(newtime) and 
!       VVEL(newtime) and break the k loop
!
!-----------------------------------------------------------------------

        UVEL(:,:,k,newtime) = UUW
        VVEL(:,:,k,newtime) = VUS

      enddo

!-----------------------------------------------------------------------
!
!     update all tracers for rigid lid or old free surface form
!     if a variable-thickness surface layer is used and pressure
!       averaging is on, need a new density based on a prediction of 
!       the new T,S
!
!-----------------------------------------------------------------------

      if (sfc_layer_type /= sfc_layer_varthick) then
        call impvmixt(1,nt,-1,oldtime)

      else if (lpressure_avg .and. leapfrogts) then

        !*** predictor update of T,S
        !*** with PSURF(curtime) on the LHS at k=1

        call impvmixt(1,2,curtime,oldtime)

      endif

      do k = 1,km

#endif
!-----------------------------------------------------------------------
!
!       if pressure averaging is on and it's a leapfrog time step
!         we need the updated density for the pressure averaging
!
!-----------------------------------------------------------------------

        if (lpressure_avg .and. leapfrogts) then
          call boundary_2d(TRACER(:,:,k,1,newtime))
          call boundary_2d(TRACER(:,:,k,2,newtime))

          call state(k,k,TRACER(:,:,k,1,newtime),
     &                   TRACER(:,:,k,2,newtime),
     &                 RHOOUT=RHO(:,:,k,newtime))

        endif

#if impvmix
      enddo

      do k = 1,km
#endif
        kp1 = k+1
        km1 = k-1
        if (k.eq.1) km1 = 1
        if (k.eq.km) kp1 = km

!-----------------------------------------------------------------------
!
!       calculate forcing terms (Fx,Fy) at level k.
!
!-----------------------------------------------------------------------

        call clinic(k, FX, FY)

!-----------------------------------------------------------------------
!
!       store forces temporarily in UVEL(newtime),VVEL(newtime).
!
!-----------------------------------------------------------------------

        if (impcor) then   ! implicit treatment

          WORK1 = c2dtu*beta*FCOR
          WORK2 = c2dtu/(c1 + WORK1**2)
          UVEL(:,:,k,newtime) = (FX + WORK1*FY)*WORK2 
          VVEL(:,:,k,newtime) = (FY - WORK1*FX)*WORK2 

        else               ! explicit treatment

          UVEL(:,:,k,newtime) = c2dtu*FX
          VVEL(:,:,k,newtime) = c2dtu*FY

        endif

!-----------------------------------------------------------------------
!
!       increment sum for vertically-averaged forcing ([Fx],[Fy]).
!
!-----------------------------------------------------------------------

        ZX = ZX + FX*dz(k)
        ZY = ZY + FY*dz(k)

      enddo

!-----------------------------------------------------------------------
!
!     end of main loop over vertical levels.
!
!-----------------------------------------------------------------------

#if drifter_particles
!-----------------------------------------------------------------------
!
!     advect drifters with old time level velocities.  then if its
!     time to sample ocean variables at drifter locations, do it.
!
!-----------------------------------------------------------------------

      if(mod(iday,10).eq.0 .and. newday) then
         if((ndrifters + array_size) .gt. ndrifters_total) then
            write(stdout,*)' '
            write(stdout,*)
     &              ' No more drifter arrays remaining for deployment'
            write(stdout,*)' '
         else
            arrays_deployed = arrays_deployed + 1
            ndrifters = ndrifters + array_size
            write(stdout,*)' '
            write(stdout,*)' Deploying drifter array # ',arrays_deployed
            write(stdout,*)' '
         endif
      endif

      call drifter_move
      if(mod(ihour,n_write_drifters).eq.0 .and. newhour) then
         call drifter_prop
      endif
#endif

!-----------------------------------------------------------------------
!
!     normalize sums for vertical averages ([Fx],[Fy]) by dividing
!     by depth at U points.
!
!-----------------------------------------------------------------------

      ZX = ZX*HUR
      ZY = ZY*HUR

#if impvmix
!-----------------------------------------------------------------------
!
!     solve tridiagonal system with implicit treatment of vertical 
!     diffusion of velocity.
!
!-----------------------------------------------------------------------

      call impvmixu

#endif

!-----------------------------------------------------------------------
!
!     calculate unnormalized baroclinic velocities (Upp,Vpp)
!
!-----------------------------------------------------------------------

      UVEL(:,:,:,newtime) = UVEL(:,:,:,oldtime) + 
     &                      UVEL(:,:,:,newtime)  ! holds c2dtu*Fx
      VVEL(:,:,:,newtime) = VVEL(:,:,:,oldtime) + 
     &                      VVEL(:,:,:,newtime)  ! holds c2dtu*Fy

!-----------------------------------------------------------------------
!
!     find vertical averages ([Upp],[Vpp]).
!
!-----------------------------------------------------------------------

      WORK1 = c0  ! initialize sums
      WORK2 = c0

      do k = 1,km
        WORK1 = WORK1 + UVEL(:,:,k,newtime)*dz(k)
        WORK2 = WORK2 + VVEL(:,:,k,newtime)*dz(k)
      enddo

      WORK1 = WORK1*HUR  ! normalize by dividing by depth
      WORK2 = WORK2*HUR

!-----------------------------------------------------------------------
!
!     normalize baroclinic velocites by subtracting vertical mean:
!     (Up,Vp) = (Upp,Vpp) - ([Upp],[Vpp]), zero velocities at land pts.
!
!-----------------------------------------------------------------------

      do k = 1,km
        where (k .le. KMU)
          UVEL(:,:,k,newtime) = UVEL(:,:,k,newtime) - WORK1
          VVEL(:,:,k,newtime) = VVEL(:,:,k,newtime) - WORK2
        elsewhere 
          UVEL(:,:,k,newtime) = c0
          VVEL(:,:,k,newtime) = c0
        endwhere
      enddo

!-----------------------------------------------------------------------
!
!     note:  at this point UVEL(newtime) and VVEL(newtime) contain only 
!     the baroclinic velocities (Up,Vp) at the new time.  they are later
!     updated to the full velocities in step after barotropic is
!     is called, which calculates the barotropic velocites ([U],[V])
!     at the new time.  UVEL and VVEL at time levels oldtime ond curtime 
!     always contain the full velocites.
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     check cfl limits
!
!-----------------------------------------------------------------------

      if (ldiag_cfl) then
        call cfl_check
      endif

!-----------------------------------------------------------------------

      end subroutine baroclinic_driver

!***********************************************************************

      subroutine baroclinic_correct_adjust

!-----------------------------------------------------------------------
!
!     This subroutine finishes updating tracers by performing
!     adjustment-like physics (convection and ice) and completes
!     the corrector step for tracers using the new surface pressure
!     to update tracers.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: k,n

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  WORK1, WORK2

      real (kind=dbl_kind), dimension(:,:,:), allocatable ::
     &  RHS1                ! r.h.s. for impvmix on corrector step

!-----------------------------------------------------------------------
!
!     do the corrector step for variable thickness surface layer
!
!-----------------------------------------------------------------------

      if (sfc_layer_type == sfc_layer_varthick) then

#ifdef impvmix
!-----------------------------------------------------------------------
!
!     if implicit vertical mixing and pressure averaging:
!     correct new T and S and update remaining passive tracers
!     with tridiagonal solves
!
!     if implicit vertical mixing but no pressure averaging:
!     update all tracers with tridiagonal solves
!
!-----------------------------------------------------------------------

      if (lpressure_avg .and. leapfrogts) then
        
        allocate(RHS1(imt,jmt,2))

        do n = 1,2   ! corrector for T,S only
          where (KMT > 0)  ! corrector RHS at k=1
            RHS1(:,:,n) =
     &        ((c2*TRACER(:,:,1,n,curtime) - TRACER(:,:,1,n,oldtime))
     &        *(PSURF(:,:,curtime) - PSURF(:,:,oldtime))
     &        - TRACER(:,:,1,n,newtime)
     &        *(PSURF(:,:,newtime) - PSURF(:,:,curtime)))/(grav*dz(1)) 
          endwhere
        enddo

        !*** T,S update on corrector step
        call impvmixt(1,2,newtime,newtime,RHS=RHS1) 

        deallocate(RHS1)

        do n = 3,nt
          where (KMT > 0) ! surface RHS for passive tracers with press avg
            TRACER(:,:,1,n,newtime) = TRACER(:,:,1,n,newtime)
     &       - TRACER(:,:,1,n,oldtime) 
     &        *(PSURF(:,:,newtime) - PSURF(:,:,oldtime))/(grav*dz(1))
          endwhere
        enddo

        !*** standard update of all passive tracers
        !***  n=3,nt with PSURF(newtime) on LHS at k=1

        call impvmixt(3,nt,newtime,oldtime) 

      else

        do n = 1,nt
          where (KMT > 0)   !  surface RHS for tracers with pressure avg
            TRACER(:,:,1,n,newtime) = TRACER(:,:,1,n,newtime) 
     &        - TRACER(:,:,1,n,oldtime) 
     &         *(PSURF(:,:,newtime) - PSURF(:,:,mixtime))/(grav*dz(1))
          endwhere
        enddo

        !*** standard update all tracers:  n=1,nt
        !*** with PSURF(newtime) on the LHS at k=1
        call impvmixt(1,nt,newtime,oldtime)

      endif

#else
!-----------------------------------------------------------------------
!
!     if explicit vertical mixing and pressure averaging:
!     correct new tracers level k=1
!
!-----------------------------------------------------------------------

      if (lpressure_avg .and. leapfrogts) then

        do n = 1,2
          where (KMT > 0)   ! correct surface T and S with pressure avg
            TRACER(:,:,1,n,newtime) =  
     &        (TRACER(:,:,1,n,newtime)*(dz(1) + PSURF(:,:,curtime)/grav)
     &        +(c2*TRACER(:,:,1,n,curtime) - TRACER(:,:,1,n,oldtime))
     &        *(PSURF(:,:,curtime) - PSURF(:,:,oldtime))/grav)
     &        /(dz(1) + PSURF(:,:,newtime)/grav) 
          elsewhere
            TRACER(:,:,1,n,newtime) = c0  ! zero tracers at land points
          endwhere
        enddo

        do n = 3,nt
          where (KMT > 0)
            TRACER(:,:,1,n,newtime) =
     &       (TRACER(:,:,1,n,oldtime)*(dz(1)+PSURF(:,:,oldtime)/grav)
     &       + dz(1)*TRACER(:,:,1,n,newtime))
     &       /(dz(1) + PSURF(:,:,newtime)/grav)
          elsewhere
            TRACER(:,:,1,n,newtime) = c0  ! zero tracers at land points
          endwhere
        enddo

      else

        do n = 1,nt

          where (KMT > 0)    ! exact update of all tracers in surface layer
            TRACER(:,:,1,n,newtime) =
     &        (TRACER(:,:,1,n,oldtime)*(dz(1)+PSURF(:,:,mixtime)/grav)
     &        + dz(1)*TRACER(:,:,1,n,newtime))
     &        /(dz(1) + PSURF(:,:,newtime)/grav)
          elsewhere
            TRACER(:,:,1,n,newtime) = c0  ! zero tracers at land points
          endwhere

        enddo

      endif
#endif

      endif ! variable thickness surface layer

!-----------------------------------------------------------------------
!
!     check for surface temperatures below freezing
!     do not reset if ice formation option is on
!
!-----------------------------------------------------------------------

      if (reset_to_freezing .and. .not. liceform) then
        TRACER(:,:,1,1,newtime) = max(TRACER(:,:,1,1,newtime),-c2)
      endif

!-----------------------------------------------------------------------
!
!     convective adjustment of tracers
!
!-----------------------------------------------------------------------

      if (convection_itype == convect_type_adjust) then
        call convad
      endif

!-----------------------------------------------------------------------
!
!     compute ice formation and adjust temperature due to ice formation
!     if this option was requested
!
!-----------------------------------------------------------------------

      if (liceform .and. mix_pass /= 1) then
        call ice_formation(STF(:,:,1)+SHF_QSW, lfw_as_salt_flx)
      endif

!-----------------------------------------------------------------------
!
!     compute new density based on new tracers
!
!-----------------------------------------------------------------------

      do k = 1,km  ! recalculate new density
        if (ltavg_on .and. convection_itype == convect_type_adjust) then
          WORK1 = RHO(:,:,k,newtime)
        endif

        call boundary_2d(TRACER(:,:,k,1,newtime))
        call boundary_2d(TRACER(:,:,k,2,newtime))
        call state(k,k,TRACER(:,:,k,1,newtime),
     &                 TRACER(:,:,k,2,newtime), 
     &               RHOOUT=RHO(:,:,k,newtime))

        if (ltavg_on .and. convection_itype == convect_type_adjust
     &               .and. mix_pass /= 1) then
          WORK2 = RHO(:,:,k,newtime) - WORK1
          call tavg_convad(k,WORK2)
        endif
      enddo

!-----------------------------------------------------------------------

      end subroutine baroclinic_correct_adjust

!***********************************************************************

      subroutine clinic(k,FX,FY)

!-----------------------------------------------------------------------
!
!     calculate forcing terms on r.h.s. of baroclinic momentum eqns.
!
!     Fx = -L(U) + f*V - Gradx(P) + HDiff(Ub) + VDiff(Ub)
!     Fy = -L(V) - f*U - Grady(P) + HDiff(Vb) + VDiff(Vb)
!
!   * the above equations are written for the case of explicit
!     treatment of the Coriolis terms.  if these terms are treated
!     implicitly, then the coriolis terms above should be replaced by:
!
!       +f*V -> +f*(gamma*V + (1-gamma)*Vb)
!       -f*U -> -f*(gamma*U + (1-gamma)*Ub)  (on leapfrog timesteps)
!
!       +f*V -> +f*Vb
!       -f*U -> -f*Ub                        (on matsuno  timesteps)
!
!     (gamma is a parameter used to vary the time-centering of the
!     coriolis and pressure gradient terms on leapfrog steps).
!
!   * the small metric terms for advection and diffusion of the
!     velocity field are calculated in 'advu' and 'del2u' or 'del4u'.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k    ! depth level index

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &  FX,        ! sum of terms contributing to Fx at level k 
     &  FY         ! sum of terms contributing to Fy at level k 

!-----------------------------------------------------------------------
!
!     local variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  WORKX,WORKY     ! local work space to hold forcing terms

!-----------------------------------------------------------------------
!
!     initialize work arrays
!
!-----------------------------------------------------------------------

      WORKX = c0     ! IS THIS NEEDED ANY MORE?
      WORKY = c0     ! IS THIS NEEDED ANY MORE?

!-----------------------------------------------------------------------
!
!     advection L(U),L(V)
!
!-----------------------------------------------------------------------

      call advu(k,WORKX,WORKY)

      FX =  -WORKX   ! advu returns WORKX = +L(U) 
      FY =  -WORKY   ! advu returns WORKY = +L(V) 

      if (ldiag_global) then
        DIAG_KE_ADV_2D = DIAG_KE_ADV_2D - 
     &                   dz(k)*(UVEL(:,:,k,curtime)*WORKX + 
     &                          VVEL(:,:,k,curtime)*WORKY)
      endif

!-----------------------------------------------------------------------
!
!     coriolis terms
!
!-----------------------------------------------------------------------

      if (impcor .and. leapfrogts) then          ! implicit, leapfrog

        WORKX =  FCOR*(gamma*VVEL(:,:,k,curtime) + 
     &          (c1 - gamma)*VVEL(:,:,k,oldtime))
        WORKY = -FCOR*(gamma*UVEL(:,:,k,curtime) + 
     &          (c1 - gamma)*UVEL(:,:,k,oldtime))

      elseif(.not.impcor .and. leapfrogts) then  ! explicit, leapfrog

        WORKX =  FCOR*VVEL(:,:,k,curtime)
        WORKY = -FCOR*UVEL(:,:,k,curtime)

      else                                  ! matsuno or foward euler

        WORKX =  FCOR*VVEL(:,:,k,oldtime)
        WORKY = -FCOR*UVEL(:,:,k,oldtime)

      endif

      FX = FX + WORKX
      FY = FY + WORKY

!-----------------------------------------------------------------------
!
!     hydrostatic pressure gradients
!
!-----------------------------------------------------------------------

      call gradp(k,WORKX,WORKY)

      FX = FX - WORKX   ! gradp returns WORKX as +Gradx(p)
      FY = FY - WORKY   ! gradp returns WORKY as +Grady(p)

      WORKX =  -dz(k)*(UVEL(:,:,k,curtime)*WORKX + 
     &                 VVEL(:,:,k,curtime)*WORKY)

      if (ltavg_on .and. mix_pass /=1) then
        call tavg_gradp(k,WORKX)
      endif

      if (ldiag_global) then
        DIAG_KE_PRESS_2D = DIAG_KE_PRESS_2D + WORKX
      endif
               
!-----------------------------------------------------------------------
!
!     horizontal diffusion HDiff(Ub),HDiff(Vb)
!
!-----------------------------------------------------------------------

      call hdiffu(k,WORKX,WORKY)

      FX = FX + WORKX
      FY = FY + WORKY

      if (ldiag_global) then
        DIAG_KE_HMIX_2D = DIAG_KE_HMIX_2D + 
     &                    dz(k)*(UVEL(:,:,k,curtime)*WORKX + 
     &                           VVEL(:,:,k,curtime)*WORKY)
      endif

!-----------------------------------------------------------------------
!
!     vertical diffusion VDiff(Ub),VDiff(Vb)
!
!-----------------------------------------------------------------------

      call vdiffu(k,WORKX,WORKY)

      FX = FX + WORKX
      FY = FY + WORKY

      if (ldiag_global) then
        DIAG_KE_VMIX_2D = DIAG_KE_VMIX_2D + 
     &                    dz(k)*(UVEL(:,:,k,curtime)*WORKX + 
     &                           VVEL(:,:,k,curtime)*WORKY)
      endif

!-----------------------------------------------------------------------
!
!     zero forces (and hence velocities) at land points
!
!-----------------------------------------------------------------------

      where (k > KMU)
        FX = c0
        FY = c0
      endwhere

!-----------------------------------------------------------------------

      end subroutine clinic

!***********************************************************************

      subroutine tracer_update(k)

!-----------------------------------------------------------------------
!
!     update tracer fields at level k
!
!     (Ta-Tb)/(2*dt) = -L(T) + HDiff(Tb) + VDiff(Tb)
!                      + source terms
!
!   * there are nt tracer fields, the 1st and 2nd are always
!     temperature and salinity.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k     ! depth level index

!-----------------------------------------------------------------------
!         
!     local variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  n                     ! dummy tracer index
     & ,now

      real (kind=dbl_kind), dimension(imt,jmt,nt) :: 
     &  FT,                 ! sum of terms in dT/dt for the nth tracer
     &  WORKN               ! work array used for various dT/dt terms 

!-----------------------------------------------------------------------
!
!     initialize some arrays
!
!-----------------------------------------------------------------------

      WORKN = c0
      FT    = c0

!-----------------------------------------------------------------------
!
!     horizontal diffusion HDiff(T)
!
!-----------------------------------------------------------------------

      call hdifft(k,WORKN)

      FT = FT + WORKN

      if (ldiag_global) then
        do n=1,nt
          where (k <= KMT) DIAG_TRACER_HDIFF_2D(:,:,n) = 
     &                     DIAG_TRACER_HDIFF_2D(:,:,n) + 
     &                     WORKN(:,:,n)*dz(k)
        end do
      endif

!-----------------------------------------------------------------------
!
!     advection L(T)
!
!-----------------------------------------------------------------------

      call advt(k,WORKN)

      FT = FT - WORKN   ! advt returns WORKN = +L(T) 

      if (ldiag_global) then
        do n=1,nt
          where (k <= KMT) DIAG_TRACER_ADV_2D(:,:,n) = 
     &                     DIAG_TRACER_ADV_2D(:,:,n) - 
     &                     WORKN(:,:,n)*dz(k)
        end do
      endif

!-----------------------------------------------------------------------
!
!     vertical diffusion VDiff(T)
!
!-----------------------------------------------------------------------

      call vdifft(k,WORKN)

      FT = FT + WORKN

      if (ldiag_global) then
        do n=1,nt
          where (k <= KMT) DIAG_TRACER_VDIFF_2D(:,:,n) = 
     &                     DIAG_TRACER_VDIFF_2D(:,:,n) + 
     &                     WORKN(:,:,n)*dz(k)
        end do
      endif

!-----------------------------------------------------------------------
!
!     add tracer change in surface layer due to freshwater flux
!     if using variable thickness surface layer
!
!-----------------------------------------------------------------------

      if (k == 1 .and. sfc_layer_type == sfc_layer_varthick) then
        do n = 1,nt
          FT(:,:,n) = FT(:,:,n) + dzr(1)*TFW(:,:,n)
        enddo

        if (ldiag_global) then
          do n = 1,nt
            sfc_tracer_flux(n) = global_sum(TFW(:,:,n)*TAREA,RCALCT)
          enddo
        endif
      endif

!-----------------------------------------------------------------------
!
!     add source terms
!
!-----------------------------------------------------------------------

      WORKN = c0

      if (pt_interior_data_type /= 'none') then
        call set_pt_interior(k,WORKN(:,:,1))
      endif

      if (s_interior_data_type /= 'none') then
        call set_s_interior(k,WORKN(:,:,2))
      endif

!-----------------------------------------------------------------------
!
!     add source terms from KPP and from shortwave solar absorption
!       if necessary.
!     NOTE:  this is here instead of in set_{pt,s}_interior in case
!       KPP and/or shortwave solar absorption are turned on but 
!       bulk restoring is not.
!
!-----------------------------------------------------------------------

      if (vmix_itype == vmix_type_kpp) then
        WORKN = WORKN + SRC_KPP(:,:,k,:)
      endif

      if (shf_formulation == 'Barnier-restoring' .or.
     &    shf_formulation == 'bulk-NCEP' .or. lcoupled) then

        SHF_QSW = max(SHF_QSW,c0)  ! no neg QSW
        where (k < KMT)
          WORKN(:,:,1) = WORKN(:,:,1) 
     &    + SHF_QSW *(sw_absorb(k-1) - sw_absorb(k))*dzr(k)
        elsewhere  !  do not allow energy absorption by the ground
          WORKN(:,:,1) = WORKN(:,:,1) 
     &    + SHF_QSW *(sw_absorb(k-1))*dzr(k)
        endwhere

      endif

      FT = FT + WORKN

      if (ldiag_global) then
        do n=1,nt
          where (k <= KMT) DIAG_TRACER_SOURCE_2D(:,:,n) = 
     &                     DIAG_TRACER_SOURCE_2D(:,:,n) + 
     &                     WORKN(:,:,n)*dz(k)
        end do
      endif

#ifdef impvmix
!-----------------------------------------------------------------------
!
!     save the explicit part of the RHS in TRACER(newtime)
!     if there is implicit vertical mixing
!
!     with pressure averaging and variable thickness surface layer, 
!     the RHS contains the surface height contribution for the 
!     predictor step (for T,S at k=1 only)
!
!-----------------------------------------------------------------------

      if (sfc_layer_type == sfc_layer_varthick .and. k == 1 .and.
     &    lpressure_avg .and. leapfrogts) then

        do n = 1,2
          where (KMT > 0)  ! RHS for predictor
            TRACER(:,:,1,n,newtime) = c2dtt(1)*FT(:,:,n) 
     &                              - c2*TRACER(:,:,1,n,curtime)
     &        *(PSURF(:,:,curtime) - PSURF(:,:,oldtime))/(grav*dz(1))
          endwhere
        enddo

        do n = 3,nt
          TRACER(:,:,k,n,newtime) = merge(c2dtt(k)*FT(:,:,n),c0,k<=KMT)
        enddo

      else

        do n = 1,nt
          TRACER(:,:,k,n,newtime) = merge(c2dtt(k)*FT(:,:,n),c0,k<=KMT)
        enddo

      endif

#else
!-----------------------------------------------------------------------
!
!     for variable thickness surface layer, update all but surface
!       layers. 
!     at the surface:
!       if explicit vertical mixing and pressure averaging:
!         predict surface T,S and store RHS for all other tracers
!       otherwise
!         store RHS for all tracers for later update with new Psurf
!
!     if not a variable thickness surface layer, update tracers here
!     
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     do time differencing if there is no implicit vertical mixing
!
!-----------------------------------------------------------------------

      if (sfc_layer_type == sfc_layer_varthick .and. k == 1) then
        if (lpressure_avg .and. leapfrogts) then

          !*** predict surface T and S with pressure avg

          do n = 1,2
            where (KMT > 0)
              TRACER(:,:,1,n,newtime) = TRACER(:,:,1,n,oldtime) 
     &        + (c1/(c1 + PSURF(:,:,curtime)/(grav*dz(1))))*
     &          (c2dtt(1)*FT(:,:,n) - c2*TRACER(:,:,1,n,curtime)*
     &          (PSURF(:,:,curtime)-PSURF(:,:,oldtime))/(grav*dz(1)))
            elsewhere
              TRACER(:,:,1,n,newtime) = c0  ! zero tracers on land pts
            endwhere
          end do

          !*** store RHS for other tracer surface layers

          do n = 3,nt
            TRACER(:,:,k,n,newtime) = merge(c2dtt(k)*FT(:,:,n),
     &                                      c0, k <= KMT)
          enddo

        else

          !*** store RHS for all tracer surface layers

          do n = 1,nt
            TRACER(:,:,k,n,newtime) = merge(c2dtt(k)*FT(:,:,n),
     &                                      c0, k <= KMT)
          enddo

        endif

      else   !*** update all tracers to new time

        do n = 1,nt
          where (k <= KMT)
            TRACER(:,:,k,n,newtime) = TRACER(:,:,k,n,oldtime) 
     &                                + c2dtt(k)*FT(:,:,n)
          elsewhere
            TRACER(:,:,k,n,newtime) = c0  ! zero tracers at land points
          endwhere
        enddo

      endif
#endif
!-----------------------------------------------------------------------

      end subroutine tracer_update

!***********************************************************************

      end module baroclinic

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
