!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module barotropic

!***********************************************************************
!
!     contains the routine for solving the barotropic equations
!
!     CVS:$Id: barotropic.F,v 1.2.2.1 2001/03/13 22:54:39 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use prognostic
      use solvers
      use operators
      use grid
      use time_management
      use forcing

      implicit none
      save

!***********************************************************************

      contains

!***********************************************************************

      subroutine barotropic_driver

!-----------------------------------------------------------------------
!
!     solve barotropic equations for the surface pressure and barotropic
!     velocity field using the implicit free-surface formulation.
!
!     leapfrog steps:
!
!       momentum equations, solve for auxiliary velocity ([Uh],[Vh]):
!
!         ([Uh] - [Ub]) - 2*dt*alpha*f*([Vh] - [Vb]) 
!           = 2*dt*{[Fx] - gamma*Gradx(P) - (1-gamma)*Gradx(Pb)} 
!    
!         ([Vh] - [Vb]) + 2*dt*alpha*f*([Uh] - [Ub]) 
!           = 2*dt*{[Fy] - gamma*Grady(P) - (1-gamma)*Grady(Pb)} 
!  
!       elliptic equation for new pressure Pa:
!
!         Div{H*Grad(Pa)} - Pa/(alpha*2*dt**2*grav)
!           = Div(H*{([Uh],[Vh])/(alpha*2*dt) + Grad(Pb)})
!              - P/(alpha*2*dt**2*grav - Fw/(alpha*2*dt)
!
!       construct ([Ua],[Va]) from ([Uh],[Vh]), Pa, Pb:
!
!         [Ua] = [Uh] - alpha*2*dt*Gradx(Pa - Pb)
!         [Va] = [Vh] - alpha*2*dt*Grady(Pa - Pb)
!
!     matsuno steps, 1st pass:
!
!       momentum equations, solve for auxiliary velocity ([Uh],[Vh])
!
!         ([Uh] - [U]) - dt*theta*f*([Vh] - [V])
!           = dt*{[Fx] - Gradx(P)}
!   
!         ([Vh] - [V]) + dt*theta*f*([Uh] - [U])
!           = dt*{[Fy] - Grady(P)}
! 
!       elliptic equation for new pressure Pa:
!
!         Div{H*Grad(Pa')} - Pa'/(theta*dt**2*grav)
!           = Div(H*{([Uh],[Vh])/(theta*dt) + Grad(P)})
!              - P/(theta*dt**2*grav) - Fw/(theta*dt)
!
!       construct ([Ua],[Va]) from ([Uh],[Vh]), Pa, P:
!
!         [Ua] = [Uh] - theta*dt*Gradx(Pa - P)
!         [Va] = [Vh] - theta*dt*Grady(Pa - P)
!
!     matsuno steps, 2nd pass:
!
!       momentum equations, solve for auxiliary velocity ([Uh],[Vh])
!
!         ([Uh] - [U]) - dt*theta*f*([Vh] - [V])
!           = dt*{[F'x] - theta*Gradx(Pa') - (1-theta)*Gradx(P)}
!   
!         ([Vh] - [V]) + dt*theta*f*([Uh] - [U])
!           = dt*{[F'y] - theta*Grady(Pa') - (1-theta)*Grady(P)}
! 
!       elliptic equation for new pressure Pa:
!
!         Div{H*Grad(Pa)} - Pa/(theta*dt**2*grav)
!           = Div(H*{([Uh],[Vh])/(theta*dt) + Grad(Pa)})
!              - P/(theta*dt**2*grav) - Fw/(theta*dt)
!
!       construct ([Ua],[Va]) from ([Uh],[Vh]), Pa, Pa:
!
!         [Ua] = [Uh] - theta*dt*Gradx(Pa - Pa)
!         [Va] = [Vh] - theta*dt*Grady(Pa - Pa)
!
!     comments:
!
!   * the above equations are written for the case of implicit treatment
!     of the coriolis terms.  alpha and gamma are parameters
!     used to vary the time-centering of the coriolis terms and surface-
!     pressure gradients, which enter the equations centered in time
!     as follows:
!
!       {alpha*Qa + gamma*Q + (1-alpha-gamma)*Qb}  (leapfrog timesteps)
!       {theta*Qa + (1-theta)*Q}                   (matsuno  timesteps)
!
!     where Q is the coriolis term or surface-pressure gradient. 
!     the force terms ([Fx],[Fy]) contain r.h.s. coriolis terms which
!     vary depending on the type of timestep (see comments in clinic).
!     if the coriolis terms are treated explicitly, then they are 
!     simply evaluated at time (n), and appear only in the force terms.
!
!   * in the 2nd pass of a matsuno timestep, the force terms
!     ([F'x],[F'y]) are constructed in 'clinic' using the predicted
!     prognostic fields (Ua',Va'), Ta from the 1st pass.
!
!   * the auxiliary velocities ([Uh],[Vh]) are solutions of the momentum
!     equation using an earlier known pressure.  the elliptic equation
!     solves for the correction to this pressure, and its gradient is
!     added to ([Uh],[Vh]) to obtain the new velocites ([Ua],[Va]).
!
!   * the elliptic equation is multiplied by the T-cell area to make
!     the operator matrix used by the cg solver symmetric.
!
!   * Fw is the surface freshwater flux in cm/s for the variable
!     thickness surface layer option.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     code variables:
!
!       (ZX,ZY) = ([Fx],[Fy]) vertically integrated forcing terms
!         calculated in 'baroclinic'
!       {(P,PX,PY),(PB,PXB,PYB)} = surface pressure and its gradient
!         at time level {(n),(n-1)}
!       PGUESS = guess for new pressure extrapolated from three
!         previous time levels, used to calculate first guess for
!         the change in pressure D in the elliptic solver
!
!       PA = surface pressure Pa at new time (also used for predicted
!'         surface pressure Pa' from 1st pass of matsuno timestep)
!       (PXA,PYA) = gradient of surface pressure Grad(Pa) at new time
!'         (also used for predicted surface-pressure gradient Grad(Pa')
!         from 1st pass of matsuno timestep)
!       (SUA,SVA) = barotropic velocities ([Ua],[Va]) at new time
!
!       (UH,UH) = ([Uh],[Vh]) auxiliary velocities (solution of the
!         momentum equations with an earlier known pressure gradient)
!       mix_pass = parameter controlling type of time step (= 0 for
!         leapfrog, 1 for 1st pass matsuno, 2 for 2nd pass matsuno)
!       impcor = logical flag to control implicit/explicit treatment
!         of coriolis terms (implicit if impcor=true, explicit if not)
!       gamma = parameter used to control centering of coriolis
!         and surface-pressure gradient terms on leapfrog timesteps
!       theta = parameter to control centering of coriolis and
!         surface-pressure gradient terms on matsuno timesteps
!       beta = {alpha,theta} on {leapfrog,matsuno} timesteps
!         (see comments in 'step')
!       B = r.h.s. of elliptic equation times T-cell area
!       AC = central coefficient for 9-point operator, before diagonal
!         term from free-surface formulation is added.
!       A0 = central coefficient for 9-point operator, after  diagonal
!         term from free-surface formulation is added.
!       FW = surface freshwater flux in cm/s (present only in
!            variable thickness surface layer).
!
!-----------------------------------------------------------------------
 
      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  UH,VH,              ! see comments above
     &  RHS,                ! r.h.s. of elliptic eqn times T-cell area
     &  WORKX,WORKY,        ! local temp space
     &  WORK1,WORK2

!-----------------------------------------------------------------------
!
!     calculate r.h.s. of barotropic momentum equations.
!
!-----------------------------------------------------------------------

      if (leapfrogts) then               ! leapfrog

        WORKX = c2dtp*(ZX -       gamma *GRADPX(:,:,curtime) - 
     &                      (c1 - gamma)*GRADPX(:,:,oldtime))
        WORKY = c2dtp*(ZY -       gamma *GRADPY(:,:,curtime) - 
     &                      (c1 - gamma)*GRADPY(:,:,oldtime))

      elseif (mix_pass == 1 .or. f_euler_ts) then   ! matsuno 1st pass
 
        WORKX = c2dtp*(ZX - GRADPX(:,:,curtime))
        WORKY = c2dtp*(ZY - GRADPY(:,:,curtime))
 
      else ! (mix_pass == 2)                ! matsuno 2nd pass
 
        WORKX = c2dtp*(ZX -       theta *GRADPX(:,:,newtime) - 
     &                      (c1 - theta)*GRADPX(:,:,curtime))
        WORKY = c2dtp*(ZY -       theta *GRADPY(:,:,newtime) - 
     &                      (c1 - theta)*GRADPY(:,:,curtime))

      endif

!-----------------------------------------------------------------------
!
!     calculate negative gradient of surface atmospheric pressure
!     and add it to r.h.s. forcing
!
!-----------------------------------------------------------------------

      if(ap_data_type /= 'none') then

        call grad(1,WORK1,WORK2,ATM_PRESS)

        WORKX = WORKX - c2dtp*WORK1
        WORKY = WORKY - c2dtp*WORK2
      endif

!-----------------------------------------------------------------------
!
!     solve for auxiliary velocities ([Uh],[Vh]) 
!
!-----------------------------------------------------------------------

      if (impcor) then          ! implicit coriolis

        WORK1 = c2dtp*beta*FCOR
        WORK2 = c1/(c1 + WORK1**2)
        UH = WORK2*(WORKX + WORK1*WORKY) + UBTROP(:,:,oldtime)
        VH = WORK2*(WORKY - WORK1*WORKX) + VBTROP(:,:,oldtime)

      else                      ! explicit coriolis

        UH = WORKX + UBTROP(:,:,oldtime)
        VH = WORKY + VBTROP(:,:,oldtime)

      endif

!-----------------------------------------------------------------------
!
!     calculate r.h.s. of elliptic equation
!
!-----------------------------------------------------------------------

      if (leapfrogts) then               ! leapfrog
 
        WORKX = HU*(UH + beta*c2dtp*GRADPX(:,:,oldtime))
        WORKY = HU*(VH + beta*c2dtp*GRADPY(:,:,oldtime))

      elseif (mix_pass == 1 .or. f_euler_ts) then   ! matsuno 1st pass
 
        WORKX = HU*(UH + beta*c2dtp*GRADPX(:,:,curtime))
        WORKY = HU*(VH + beta*c2dtp*GRADPY(:,:,curtime))
 
      else ! (mix_pass == 2)                ! matsuno 2nd pass
 
        WORKX = HU*(UH + beta*c2dtp*GRADPX(:,:,newtime))
        WORKY = HU*(VH + beta*c2dtp*GRADPY(:,:,newtime))
 
      endif

      call div(1,RHS,WORKX,WORKY)  ! div returns T-cell area * divergence

      RHS = RHS/(beta*c2dtp)

!-----------------------------------------------------------------------
!
!     add diagonal term to central coefficient in implicit free-surface 
!     formulation, and add correction to r.h.s.
!
!-----------------------------------------------------------------------

      select case (sfc_layer_type)

      case(sfc_layer_varthick)
        A0 = merge(TAREA/(beta*c2dtp*dtp*grav),c0,CALCT)
        RHS = RHS - A0*PSURF(:,:,curtime) - FW*TAREA/(beta*c2dtp)
        A0 = AC - A0

      case(sfc_layer_rigid)
        A0 = AC

      case(sfc_layer_oldfree)
        A0 = merge(TAREA/(beta*c2dtp*dtp*grav),c0,CALCT)
        RHS = RHS - A0*PSURF(:,:,curtime)
        A0 = AC - A0

      end select

!-----------------------------------------------------------------------
!
!     initial guess for solver
!
!-----------------------------------------------------------------------

      PSURF(:,:,newtime) = PGUESS         ! initial guess for solver
 
      if (mix_pass == 2) then             ! in matsuno 2nd pass,
                                          ! temporarily store solutions
        WORKX = GRADPX(:,:,newtime)       ! for pressure gradient from
        WORKY = GRADPY(:,:,newtime)       ! 1st pass in scratch arrays
 
      endif

!-----------------------------------------------------------------------
!
!     solve elliptic equation with cg routine
!
!-----------------------------------------------------------------------

      select case(solv_itype)
      case (solv_pcg)
        call pcg(PSURF(:,:,newtime),RHS) ! precond conjg grad solver
      case (solv_cgr)
        call cgr(PSURF(:,:,newtime),RHS) ! conjugate residual solver
      case (solv_jac)
        call jacobi(PSURF(:,:,newtime),RHS)   ! simple jacobi solver
      end select

!-----------------------------------------------------------------------
!
!     calculate gradient of PSURF(:,:,newtime)
!
!-----------------------------------------------------------------------

      call grad(1,GRADPX(:,:,newtime),GRADPY(:,:,newtime),
     &            PSURF(:,:,newtime))

!-----------------------------------------------------------------------
!
!     update new barotropic velocity, pressure, pressure gradient
!
!-----------------------------------------------------------------------

      if (leapfrogts) then               ! leapfrog
 
        UBTROP(:,:,newtime) = UH - beta*c2dtp*(GRADPX(:,:,newtime) - 
     &                                         GRADPX(:,:,oldtime))
        VBTROP(:,:,newtime) = VH - beta*c2dtp*(GRADPY(:,:,newtime) - 
     &                                         GRADPY(:,:,oldtime))
 
      elseif (mix_pass == 1 .or. f_euler_ts) then  ! matsuno 1st pass
 
        UBTROP(:,:,newtime) = UH - beta*c2dtp*(GRADPX(:,:,newtime) -
     &                                         GRADPX(:,:,curtime))
        VBTROP(:,:,newtime) = VH - beta*c2dtp*(GRADPY(:,:,newtime) -
     &                                         GRADPY(:,:,curtime))
 
      else ! (mix_pass == 2)                ! matsuno 2nd pass
 
        UBTROP(:,:,newtime) = UH - beta*c2dtp*(GRADPX(:,:,newtime) - 
     &                                                       WORKX)
        VBTROP(:,:,newtime) = VH - beta*c2dtp*(GRADPY(:,:,newtime) - 
     &                                                       WORKY)
 
      endif

!-----------------------------------------------------------------------

      end subroutine barotropic_driver

!***********************************************************************

      end module barotropic

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
