!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module convection

!***********************************************************************
!
!     this module contains routines for treating convection if it is
!     not already treated in a vertical mixing module
!
!     CVS:$Id: convection.F,v 1.3.2.2 2001/09/07 19:51:47 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use prognostic
      use grid
      use state_mod
      use time_management
      use io
      use exit_mod

      implicit none
      save

!-----------------------------------------------------------------------
!
!     module variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  convection_itype  ! method to use for convection
     &, nconvad           ! number of times to convectively adjust

      real (kind=dbl_kind) ::
     &  convect_diff      ! tracer diffusion to use for convection
     &, convect_visc      ! viscosity to use for convection (momentum)

      integer (kind=int_kind), parameter ::
     &  convect_type_adjust = 1
     &, convect_type_diff   = 2

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_convection

!-----------------------------------------------------------------------
!
!     initializes some convection variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  nml_error              ! namelist i/o error flag
     &, convect_error          ! convect options error flag

      character (char_len) ::
     &  convection_type        ! input to choose method for convection

      namelist /convection_nml/ convection_type, nconvad
     &,                         convect_diff, convect_visc

!-----------------------------------------------------------------------
!
!     read namelist input and broadcast
!
!-----------------------------------------------------------------------

      convection_type = 'diffusion'
      nconvad = 2
      convect_diff = 1000.0_dbl_kind
      convect_visc = 1000.0_dbl_kind

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=convection_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      end if

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR: reading convection_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, convection_nml)    
          write(stdout,*) ' '
      endif

      convect_error = 0

      if (my_task == master_task) then
        write(stdout,1000)
        write(stdout,1010)
        write(stdout,1020)
        write(stdout,1010)
        write(stdout,1000)

 1000   format(72('-'))
 1010   format(' ')
 1020   format(' Convection options')

        select case (convection_type)
        case ('adjustment')
          convection_itype = convect_type_adjust
          write(stdout,*) ' Convective adjustment enabled'
          write(stdout,*) ' Using ',nconvad,' passes'
        case ('diffusion')
          convection_itype = convect_type_diff
          write(stdout,*) ' Convection simulated by strong diffusion'
          write(stdout,*) '   convect_diff = ',convect_diff
          write(stdout,*) '   convect_visc = ',convect_visc
        case default
          convect_error = -1000
        end select
        
      endif

      call broadcast_scalar(convect_error,    master_task)
      call broadcast_scalar(convection_itype, master_task)
      call broadcast_scalar(nconvad,          master_task)
      call broadcast_scalar(convect_diff,     master_task)
      call broadcast_scalar(convect_visc,     master_task)

      if (convect_error /= 0) then
        call exit_POP('ERROR: Unknown option for convection type')
      endif

#ifdef impvmix
      if (convection_itype == convect_type_adjust) then
        call exit_POP('ERROR: ' //
     &   'Convective adjustment should not be used with implicit vmix')
      endif
#else
      if (convection_itype == convect_type_diff) then
        call exit_POP( 'ERROR: ' //
     &    'Implicit vertical mixing required for diffusive convection')
      endif
#endif

!-----------------------------------------------------------------------

      end subroutine init_convection

!***********************************************************************

      subroutine convad

!-----------------------------------------------------------------------
!
!     convectively adjust tracers at adjacent depth
!     levels if they are gravitationally unstable
!
!   * this routine convectively adjusts tracers at the new
!     time (before they are updated at the end of 'step').
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     local variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  nc,   ! dummy index for number of adjustment passes
     &  n,    ! dummy index for tracer number
     &  k, ks ! dummy vertical level indices

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  RHOK,      ! density of water after adiab. displacement to k+1
     &  RHOKP,     ! actual density of water at level k+1
     &  WORK       ! local work space

!-----------------------------------------------------------------------
!
!     perform convective adjustment
!
!-----------------------------------------------------------------------

      do nc = 1,nconvad  ! repeat nconvad times

        do ks = 1,2
          do k = ks,km-1,2

            call state(k  ,k+1,TRACER(:,:,k  ,1,newtime),
     &                         TRACER(:,:,k  ,2,newtime),RHOOUT=RHOK )
            call state(k+1,k+1,TRACER(:,:,k+1,1,newtime),
     &                         TRACER(:,:,k+1,2,newtime),RHOOUT=RHOKP)
              
            do n = 1,nt
              where ((RHOK > RHOKP) .and. (k < KMT))
                TRACER(:,:,k,n,newtime) = dzwxcel(k)*
     &                       (dztxcel(k  )*TRACER(:,:,k  ,n,newtime) 
     &                      + dztxcel(k+1)*TRACER(:,:,k+1,n,newtime))
                TRACER(:,:,k+1,n,newtime) = TRACER(:,:,k,n,newtime)
              endwhere
            enddo

          enddo
        enddo

      enddo

!-----------------------------------------------------------------------

      end subroutine convad

!***********************************************************************

      end module convection

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
