!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module diag_bsf 

      use kinds_mod 
      use model_size
      use domain
      use constants
      use exit_mod
      use global_reductions
      use grid
      use io
      use stencils

      implicit none
      save

      integer (kind=int_kind), parameter :: mxisle = 200  ! max number of islands

      real (kind=dbl_kind) :: r_norm   ! normalization constant

      integer (kind=int_kind) ::
     &  nisle                          ! number of islands

      integer (kind=int_kind), dimension(mxisle) ::
     &  npts_is                        ! number of island boundary points

      integer (kind=int_kind), dimension(imt,jmt) ::
     &  ISMASK                         ! island mask

!     coefficients for the 9-point operator

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  BSFC, BSFN, BSFS, BSFE, BSFW, BSFNE, BSFSE, BSFNW, BSFSW

      real (kind=dbl_kind), dimension(mxisle) :: 
     &  aislandr                       ! island diagonal coefficient

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_diag_bsf 

!-----------------------------------------------------------------------
!
!     initialize the necessary fields for the diagnostic barotropic
!     streamfunction calculation
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: isle

      integer (kind=int_kind), dimension(imt,jmt) :: 
     &  WORK1, WORK2

      logical (kind=log_kind), dimension(imt,jmt) :: 
     &  MASKI, MASKT

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  XNE, XSE, XNW, XSW, YNE, YSE, YNW, YSW, WORK

      if ( my_task == master_task ) then
        write (stdout,*) ' '
        write (stdout,*) 'Initializing diagnostic BSF variables ....'
      endif

!-----------------------------------------------------------------------
!
!     Calculate normalization constant
!
!-----------------------------------------------------------------------

      r_norm = c0

      WORK   = TAREA**2
      r_norm = c1/global_sum(WORK,RCALCT)

      WORK1 = 0 
      WORK2 = merge( 1, 0, KMU > 0 )   ! WORK2 = (0,1) on (land,ocean)

!-----------------------------------------------------------------------
!
!     9-point coefficients
!
!-----------------------------------------------------------------------

      XSE = c0         ! initialize before passing as argument
      XNW = c0
      XSW = c0
      YSE = c0
      YNW = c0
      YSW = c0

      XNE = p25*WORK2*DXUR*DYU
      YNE = p25*WORK2*DYUR*DXU

      call s_shift(XSE,XNE)
      call w_shift(XNW,XNE)
      call s_shift(XSW,XNW)

      call s_shift(YSE,YNE)
      call w_shift(YNW,YNE)
      call s_shift(YSW,YNW)

      BSFNE = XNE + YNE
      BSFSE = XSE + YSE
      BSFNW = XNW + YNW
      BSFSW = XSW + YSW

      BSFE  = XNE + XSE - YNE - YSE
      BSFW  = XNW + XSW - YNW - YSW
      BSFN  = YNE + YNW - XNE - XNW
      BSFS  = YSE + YSW - XSE - XSW

      BSFC  = -(BSFNE + BSFSE + BSFNW + BSFSW)   ! diagonal coefficient

!-----------------------------------------------------------------------
!
!     calculate island mask
!
!-----------------------------------------------------------------------

      call island_mask_diag_bsf

!-----------------------------------------------------------------------
!     calculate island coefficients
!-----------------------------------------------------------------------

      do isle=1,mxisle
        aislandr(isle) = c0
      enddo

      if ( my_task == master_task ) then
        write (stdout,*) ' '
        write (stdout,*) ' island coefficients: '
      endif

      do isle=1,nisle

        WORK = c0
        MASKI = ISMASK  ==  isle

        call n_shift (WORK1,ISMASK)
        MASKT = MASKI .and. (WORK1 == 0)
        where (MASKT)  WORK = BSFN                  ! north faces

        call e_shift (WORK2,WORK1)
        MASKT = MASKI .and. (WORK2 == 0)
        where (MASKT)  WORK = WORK + BSFNE          ! northeast faces

        call w_shift (WORK2,WORK1)
        MASKT = MASKI .and. (WORK2 == 0)
        where (MASKT)  WORK = WORK + BSFNW          ! northwest faces

        call s_shift (WORK1,ISMASK)
        MASKT = MASKI .and. (WORK1 == 0)
        where (MASKT)  WORK = WORK + BSFS           ! south faces

        call e_shift (WORK2,WORK1)
        MASKT = MASKI .and. (WORK2 == 0)
        where (MASKT)  WORK = WORK + BSFSE          ! southeast faces

        call w_shift (WORK2,WORK1)
        MASKT = MASKI .and. (WORK2 == 0)
        where (MASKT)  WORK = WORK + BSFSW          ! southwest faces

        call e_shift (WORK1,ISMASK)
        MASKT = MASKI .and. (WORK1 == 0)
        where (MASKT)  WORK = WORK + BSFE           ! east  faces

        call w_shift (WORK1,ISMASK)
        MASKT = MASKI .and. (WORK1 == 0)
        where (MASKT)  WORK = WORK + BSFW           ! west  faces

        aislandr(isle) = - c1 / global_sum(WORK, ONE) ! diagonal coeff. on island

        if ( my_task == master_task ) then 
          write (stdout,*) ' island ', isle,
     &      ' coefficient = ', c1/aislandr(isle)
        endif

      enddo

!-----------------------------------------------------------------------

      end subroutine init_diag_bsf 

!***********************************************************************

      subroutine pcg_diag_bsf ( X, B )

!-----------------------------------------------------------------------
!
!     conjugate-gradient solver for diagnosed barotropic streamfunction 
!     in free-surface formulation
!
!-----------------------------------------------------------------------
!     equations:
!
!     Ax = b , A = operator, x = field, b = r.h.s.
!-----------------------------------------------------------------------
!     comments:
!
!   * the matrix A is a 9-point operator given by the
!     coefficient arrays BSF{C,N,S,E,W,NE,SE,NW,SW}.
!     a 5-point approximation may be used for the 
!     streamfunction calculation. 
!
!   * every ncheck steps the residual norm is calculated
!     in order to test for convergence.
!
!   * diagonal preconditioning is used
!
!-----------------------------------------------------------------------
!
!     input variables:
!
!       BSF{C,N,S,E,W,NE,SE,NW,SW} arrays of coefficients for 9-point
!         operator A
!       B = r.h.s.
!       X input as initial guess
!
!       the actual ninept_4 solver makes use of only FOUR of these 
!       coefficients, because the others can be obtained from these
!       using do-loop constructs.
!
!     output variables:
!
!       X returned as solution
!
!     local or common variables:
!
!       R = residual (b - Ax)
!       S = conjugate direction vector
!       Q = As (operator applied to conjugate direction vector)
!       eta{0,1,2} are scalars in the cg routine
!       rms_residual = sqrt{(r,r)/(da,da)}  (da = area of T cell)
!       ncheck:  calculate residual norm every ncheck steps 
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &  B

!-----------------------------------------------------------------------
!
!     input/output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(inout) :: 
     &  X

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), parameter :: mxscan =1000, ncheck = 100 

      real (kind=dbl_kind) :: err = 1.0e-6_dbl_kind

      integer (kind=int_kind) :: m, isle, mscan

      real (kind=dbl_kind) :: 
     &  eta0, eta1, eta2, rr, risle, rms_residual

      logical (kind=log_kind), dimension(imt,jmt) :: 
     &  CALCZ

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  R, S, Q, WORK1, WORK2, WORK3, BSFCR

!-----------------------------------------------------------------------
!
!     initialization 
!
!-----------------------------------------------------------------------

      R     = c0  
      Q     = c0
      WORK1 = c0
      WORK2 = c0

      CALCZ = ISMASK == 0  ! mask for interior ocean points

!-----------------------------------------------------------------------
!
!     diagonal preconditioner
!
!-----------------------------------------------------------------------

      BSFCR = merge( c1/BSFC, c0, BSFC /= c0)

      where ( .not. CALCZ )  BSFCR = c0

      do isle=1,nisle
        where ( abs(ISMASK) == isle )  BSFCR = aislandr(isle)
      enddo

!-----------------------------------------------------------------------
!
!     compute initial residual
!
!-----------------------------------------------------------------------

      call ninept_4 ( WORK1, BSFC, BSFN, BSFE, BSFNE, X)

      WORK1 = B - WORK1
      where (CALCZ) 
        R     = WORK1
        WORK2 = WORK1
      endwhere

      do isle=1,nisle                           ! island loop
        WORK3 = merge(c1, c0, ISMASK == isle)
        risle = global_sum( WORK1, WORK3 )
        where ( abs(ISMASK) == isle )  R = risle
        where ( ISMASK  == isle )  WORK2 = risle/npts_is(isle)
      enddo

      rms_residual = sqrt(global_sum(WORK2*R,RCALCT)*r_norm)

      if ( my_task == master_task ) then
        write (stdout,*) ' '
        write (stdout,*) ' convergence info from pcg_diag_bsf: '
        write (stdout,*) ' iter = ', 0, ' rms_resid = ', rms_residual
      endif

!-----------------------------------------------------------------------
!
!     initialize fields and scalars
!
!-----------------------------------------------------------------------

      S     = c0
      eta0  = c1
      mscan = mxscan

!-----------------------------------------------------------------------
!
!     iterate
!
!-----------------------------------------------------------------------

      do m=1,mxscan

        WORK1 = R * BSFCR   ! use diagonal preconditioner

!-----------------------------------------------------------------------
!
!       update conjugate direction vector s
!
!-----------------------------------------------------------------------

        WORK2 = R
        do isle=1,nisle   ! island loop
          where ( ISMASK  == isle )  WORK2 = R/npts_is(isle)
        enddo

        eta1 = global_sum( WORK2*WORK1, RCALCT )  ! (r,(PC)r)
        eta2 = eta1/eta0

        S =  WORK1 + S*eta2

!-----------------------------------------------------------------------
!
!       compute As
!
!-----------------------------------------------------------------------

        call ninept_4 ( WORK1, BSFC, BSFN, BSFE, BSFNE, S)

        where (CALCZ)
          Q     = WORK1
          WORK2 = WORK1
        endwhere

        do isle=1,nisle   ! island loop
          WORK3 = merge(c1, c0, ISMASK == isle)
          risle = global_sum( WORK1, WORK3 )
          where ( abs(ISMASK) == isle ) Q     = risle
          where (     ISMASK  == isle ) WORK2 = risle/npts_is(isle)
        enddo

!-----------------------------------------------------------------------
!
!       compute next solution and residual
!
!-----------------------------------------------------------------------

        eta0 = eta1
        eta1 = eta0/global_sum( WORK2*S, RCALCT )

        X = X + eta1*S
        R = R - eta1*Q

!-----------------------------------------------------------------------
!
!       test for convergence
!
!-----------------------------------------------------------------------

        if (mod(m,ncheck) == 0) then

          call ninept_4 ( WORK1, BSFC, BSFN, BSFE, BSFNE, X)

          WORK1 = B - WORK1
          where (CALCZ)
            R     = WORK1
            WORK2 = WORK1
          endwhere

          do isle=1,nisle   ! island loop
            WORK3 = merge(c1, c0, ISMASK == isle)
            risle = global_sum( WORK1, WORK3 )
            where ( abs(ISMASK) == isle ) R     = risle
            where (     ISMASK  == isle ) WORK2 = risle/npts_is(isle)
          enddo

          rr = global_sum( WORK2*R, RCALCT )   ! (r,r)

          rms_residual = sqrt(rr*r_norm)
 
          if ( my_task == master_task ) then
            write (stdout,*) ' iter = ', m, 
     &                       ' rms_resid = ', rms_residual 
          endif

          if (rms_residual < err) then
            mscan = m
            exit
          endif

        endif

      enddo

      if ( mscan == mxscan ) then
        if ( my_task == master_task ) then
          write (stdout,*) 
     &      ' WARNING (pcg_diag_bsf): No convergence after ',
     &                                mxscan, ' scans'
        endif
      endif

!-----------------------------------------------------------------------

      end subroutine pcg_diag_bsf

!***********************************************************************

      subroutine island_mask_diag_bsf

!-----------------------------------------------------------------------
!
!     find island mask for the barotropic streamfunction calculation
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     output variables: nisle, npts_is, ISMASK
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: i, j, n, imax, error_code
      integer (kind=int_kind), dimension(imt_global,jmt_global) :: 
     &  KALL, KINT, I1, I2, ISMASK_G, WORK1, WORK2

      logical (kind=log_kind), dimension(imt_global,jmt_global) :: 
     &  LAND, LANDLEFT 

      call gather_global (WORK1, KMT, master_task)

      error_code = 0

      if ( my_task == master_task ) then

        LAND = .false.
        where ( WORK1 == 0 )  LAND = .true.

        do j=1,jmt_global
          do i=1,imt_global
            I1(i,j) = i
            I2(i,j) = j
          enddo
        enddo

        where ( .not. LAND )
          KINT = imt_global * jmt_global + 1
        elsewhere
          KINT = I1 + (I2-1) * imt_global
        endwhere

        KALL = KINT

        imax = imt_global + 2 * jmt_global

        do i=1,imax

          WORK1 = cshift(KALL, +1, 2)
          WORK1(:,jmt_global) = KALL(:,jmt_global)
          WORK2 = cshift(KALL, -1, 2)
          WORK2(:,1) = KALL(:,1)
          KALL = min(WORK1, WORK2, KALL)

          WORK1 = cshift(KALL, +1, 2)
          WORK1(:,jmt_global) = KALL(:,jmt_global)
          WORK2 = cshift(KALL, -1, 2)
          WORK2(:,1) = KALL(:,1)
          KALL = min(WORK1, WORK2, KALL)

          WORK1 = cshift(KALL, +1, 1)
          WORK2 = cshift(KALL, -1, 1)
          KALL  = min(WORK1, WORK2, KALL)

          WORK1 = cshift(KALL, +1, 1)
          WORK2 = cshift(KALL, -1, 1)
          KALL  = min(WORK1, WORK2, KALL)

          where (LAND)  KINT = KALL
          KALL = KINT

        enddo

        ISMASK_G = 0
        LANDLEFT = LAND

        where ( KINT == KINT(1,jmt_global) )  ! north continent
          ISMASK_G = 999                      ! NOTE: nisle must be less than 999
          LANDLEFT = .false.
        endwhere

        n = 0
        do i=1,imt_global
          do j=1,jmt_global
            if (LANDLEFT(i,j)) then
              n = n + 1 
              where ( KINT == KINT(i,j) )
                ISMASK_G = n
                LANDLEFT = .false.
              endwhere
            endif
          enddo
        enddo

        nisle = n

        write (stdout,*) ' '
        write (stdout,*) ' number of islands = ', nisle

        if ( nisle > mxisle )  error_code = 1 

      endif        ! if master_task

      call broadcast_scalar (error_code, master_task)

      if ( error_code /= 0) then
        call exit_POP('ERROR (island_mask_diag_bsf): nisle > mxisle')
      endif

      if ( my_task == master_task ) then

        WORK1 = cshift(ISMASK_G, +1, 1)
        WORK2 = cshift(ISMASK_G, -1, 1)
        KALL  = max(ISMASK_G, WORK1, WORK2)

        WORK1 = cshift(ISMASK_G, +1, 2)
        WORK2 = cshift(ISMASK_G, -1, 2)
        KALL  = max(KALL, WORK1, WORK2)

        WORK1 = cshift(ISMASK_G, +1, 2)
        WORK2 = cshift(WORK1,    +1, 1)
        KALL  = max(KALL, WORK2)

        WORK1 = cshift(ISMASK_G, +1, 2)
        WORK2 = cshift(WORK1,    -1, 1)
        KALL  = max(KALL, WORK2)

        WORK1 = cshift(ISMASK_G, -1, 2)  
        WORK2 = cshift(WORK1,    +1, 1)
        KALL  = max(KALL, WORK2)
 
        WORK1 = cshift(ISMASK_G, -1, 2)
        WORK2 = cshift(WORK1,    -1, 1)
        KALL  = max(KALL, WORK2)
  
        where (.not.LAND)
          ISMASK_G =  -KALL
        endwhere
        ISMASK_G = -ISMASK_G

        write (stdout,*) ' '
        do i=1,nisle
          npts_is(i) = count(ISMASK_G == i)
          write (stdout,*) ' island ', i,
     &      ' # points on boundary = ', npts_is(i)  
        enddo

      endif        ! if master_task

      call scatter_global ( ISMASK, ISMASK_G, master_task )

      call broadcast_scalar ( nisle,   master_task )
      call broadcast_vector ( npts_is, master_task, mxisle )

!-----------------------------------------------------------------------

      end subroutine island_mask_diag_bsf


!***********************************************************************

      end module diag_bsf

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
