!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module forcing

!***********************************************************************

!-----------------------------------------------------------------------
!     driver module for all of the surface and interior forcing.
!
!     defines variables containing surface and interior forcing 
!       at the current timestep.
!
!     contains calls to all of the initialization routines and updating
!       routines.
!
!     CVS:$Id: forcing.F,v 1.2.2.6 2002/01/31 22:30:47 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!-----------------------------------------------------------------------

      use constants
      use forcing_ws
      use forcing_shf
      use forcing_sfwf
      use forcing_pt_interior
      use forcing_s_interior
      use forcing_ap
      use forcing_coupled
      use forcing_tools
      use prognostic
      use time_management
      use exit_mod
      use ice
      
      implicit none
      save

      real (kind=dbl_kind), dimension(imt,jmt,2) ::
     &   SMF    !  surface momentum fluxes (wind stress)

      logical (kind=log_kind) ::
     &  lsmft_avail  ! true if SMFT is an available field

      real (kind=dbl_kind), dimension(:,:,:), allocatable ::
     &   SMFT   !  surface momentum fluxes on T points if avail

      real (kind=dbl_kind), dimension(imt,jmt,nt) ::
     &   STF    !  surface tracer fluxes

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   ATM_PRESS  !  atmospheric pressure forcing

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_forcing

!-----------------------------------------------------------------------
!
!     Initialize forcing by calling a separate routines for
!       wind stress, heat flux, fresh water flux, passive tracer flux,
!       interior restoring, and atmospheric pressure.
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  allocate_error_status                ! allocate error flag

!-----------------------------------------------------------------------
!     write out header for forcing options to stdout.
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        write(stdout,1001)
        write(stdout,1000)
        write(stdout,1001)
        write(stdout,1002)
        write(stdout,1001)
        write(stdout,1000)
      endif

 
 
 1000 format (72('-'))
 1001 format (' ')
 1002 format ('forcing options')

!-----------------------------------------------------------------------
!     call individual initialization routines
!-----------------------------------------------------------------------

      allocate (SMFT(imt,jmt,2))

      call init_ws(SMF,SMFT,lsmft_avail)

!  NOTE: with bulk NCEP forcing init_shf must be called before init_sfwf

      call init_shf (STF(:,:,1))

      call init_sfwf(STF(:,:,2))
      
      call init_pt_interior

      call init_s_interior
      
      call init_ap(ATM_PRESS)

      call init_coupled(SMF, SMFT, STF, SHF_QSW, lsmft_avail)

      if ( lcoupled .and. sfwf_formulation /= 'partially-coupled'
     &     .and. sfc_layer_type == sfc_layer_varthick .and.
     &     .not. lfw_as_salt_flx .and. liceform ) then

        sfwf_num_comps = 1
        sfwf_comp_cpl  = 1
        tfw_num_comps  = 1
        tfw_comp_cpl   = 1

        allocate ( SFWF_COMP(imt,jmt,sfwf_num_comps),
     &             STAT = allocate_error_status )
        call check_allocate (allocate_error_status 
     &,      exit_message = 'forcing.F -- SFWF_COMP')

        allocate ( TFW_COMP(imt,jmt,nt,tfw_num_comps),
     &             STAT = allocate_error_status )
        call check_allocate (allocate_error_status 
     &,      exit_message = 'forcing.F -- TFW_COMP' )


        SFWF_COMP = c0
        TFW_COMP  = c0

      endif

      if (.not. lsmft_avail) deallocate(SMFT)

!-----------------------------------------------------------------------
!
!     check coupled compatibility with other forcing options
!
!-----------------------------------------------------------------------

      if (lcoupled) then
        if (ws_data_type /= 'none') then
          call exit_POP(
     &         'ws_data_type must be set to none in coupled mode')
        endif
        if ( (shf_formulation  == 'partially-coupled' .and.
     &        sfwf_formulation /= 'partially-coupled') .or.
     &       (shf_formulation  /= 'partially-coupled' .and.
     &        sfwf_formulation == 'partially-coupled') ) then
          call exit_POP(
     &      'partially-coupled must be used for both shf and sfwf')
        endif
        if ( shf_formulation /= 'partially-coupled' .and.
     &       shf_data_type /= 'none') then
          call exit_POP(
     &      'shf_data_type must be set to either none or '//
     &      'partially_coupled when lcoupled is true' )
        endif
        if ( sfwf_formulation /= 'partially-coupled' .and.
     &       sfwf_data_type /= 'none') then
          call exit_POP(
     &      'sfwf_data_type must be set to either none or '//
     &      'partially_coupled when lcoupled is true' )
        endif
#ifndef coupled
        call exit_POP(
     &        'ERROR: code must be compiled with coupled ifdef option')
#endif
      endif

 
!-----------------------------------------------------------------------

      end subroutine init_forcing

!***********************************************************************

      subroutine set_surface_forcing

!-----------------------------------------------------------------------
!
!     call surface forcing routines if necessary
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!     if forcing does not depend on the ocean state, then update 
!       forcing if current time is greater than the appropriate
!       interpolation time or if it is the first step.
!
!     if forcing DOES depend on the ocean state, then call every 
!       timestep.  interpolation check will be done within the set_*
!       routine.
!
!     note that interior restoring is assumed to take place every
!       timestep and is set in subroutine tracer_update.
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) :: WORK1, WORK2

      if( thour00 >= ws_interp_next .or. nsteps_run == 0 ) then
        if (lsmft_avail) then
          call set_ws(SMF,SMFT=SMFT)
        else
          call set_ws(SMF)
        endif
        if(nsteps_run /= 0) 
     &    ws_interp_next = ws_interp_next + ws_interp_inc
      endif

      call set_coupled_forcing(SMF,SMFT,STF,SHF_QSW)

!  NOTE: with bulk NCEP and partially coupled forcing, 
!        set_shf must be called before set_sfwf
      
      call set_shf(STF(:,:,1))
     
      call set_sfwf(STF(:,:,2))

      if ( shf_formulation  == 'partially-coupled' .or. 
     &     sfwf_formulation == 'partially-coupled' ) then
         call set_combined_forcing(STF)
      endif

      if ( lcoupled .and. sfwf_formulation /= 'partially-coupled'
     &     .and. sfc_layer_type == sfc_layer_varthick .and.
     &     .not. lfw_as_salt_flx .and. liceform ) then
        FW  = SFWF_COMP(:,:,sfwf_comp_cpl)
        TFW = TFW_COMP(:,:,:,tfw_comp_cpl)
      endif

      if ( sfc_layer_type == sfc_layer_varthick .and.
     &     .not. lfw_as_salt_flx .and. liceform ) then
        FW = FW + FW_FREEZE
        TFW(:,:,1) = TFW(:,:,1) + FW_FREEZE 
     &                           * TFRZ(TRACER(:,:,1,2,curtime))
        TFW(:,:,2) = TFW(:,:,2) + FW_FREEZE * salice
      endif
      
      if( thour00 >= ap_interp_next .or. nsteps_run == 0 ) then
        call set_ap(ATM_PRESS)
        if(nsteps_run /= 0) 
     &    ap_interp_next = ap_interp_next + ap_interp_inc
      endif

      end subroutine set_surface_forcing
      
!***********************************************************************

      end module forcing

!***********************************************************************

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
