!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module forcing_ap

!***********************************************************************

!-----------------------------------------------------------------------
!
!     contains routines and variables used for determining the
!       surface atmopheric pressure.
!
!     CVS:$Id: forcing_ap.F,v 1.2.2.3 2002/01/25 23:24:57 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use io
      use forcing_tools
      use time_management
      use grid
      use exit_mod

      implicit none
      save

      real (kind=dbl_kind), allocatable, dimension(:,:,:,:) ::
     &   ATM_PRESS_DATA !  surface atm pressure at multiple times that
                      !  may be interpolated in time to get ATM_PRESS

      real (kind=dbl_kind), dimension(12) :: 
     &   ap_data_time  !  time (in hours) corresponding to pressure
                  !  in ATM_PRESS_DATA

      real (kind=dbl_kind), dimension(20) :: ap_data_renorm

      real (kind=dbl_kind) :: 
     &    ap_data_inc    ! time increment between values of forcing data
     &  , ap_data_next   ! time that will be used for the next value of forcing data that is needed
     &  , ap_data_update ! time when the a new forcing value needs to be added to interpolation set
     &  , ap_interp_inc  ! time increment between interpolation
     &  , ap_interp_next ! time when next interpolation will be done
     &  , ap_interp_last ! time when last interpolation was done

      integer (kind=int_kind) :: 
     &    ap_interp_order  !  order of temporal interpolation
     &  , ap_data_time_min_loc  !  index of the third dimension of ATM_PRESS_DATA ,ap_data_time containing the minimum forcing time

      character (char_len) :: 
     &    ap_data_type  !  period keyword for the period of the forcing data
     &  , ap_filename     !  name of file conainting forcing data
     &  , ap_interp_freq !  period keyword for the period of the temporal interpolation
     &  , ap_interp_type
     &  , ap_data_label
     &  , ap_formulation

!***********************************************************************

      contains

!***********************************************************************

!***********************************************************************

      subroutine init_ap(ATM_PRESS)
      
!-----------------------------------------------------------------------
!
!     initialize atm pressure forcing by either calculating or reading
!       in the atm pressure.  also do initial book-keeping concerning 
!       when new data is needed for the temporal interpolation and 
!       when the forcing will need to be updated.
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n, nu
     &, nml_error              ! namelist i/o error flag
     &, allocate_error_status  ! allocate error flag

      character (char_len) :: forcing_filename

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   ATM_PRESS    !  surface momentum fluxes at current timestep
     &  ,WORK

      namelist /forcing_ap_nml/ ap_data_type, ap_data_inc
     &         ,ap_interp_type, ap_interp_freq
     &         ,ap_interp_inc, ap_filename, ap_data_renorm

!-----------------------------------------------------------------------
!
!     read atmospheric pressure namelist input after setting 
!       default values.
!
!-----------------------------------------------------------------------

      ap_data_type   = 'none'
      ap_data_inc    = 1.e20_dbl_kind
      ap_interp_type = 'nearest'
      ap_interp_freq = 'never'
      ap_interp_inc  = 1.e20_dbl_kind
      ap_filename    = 'unknown-ap'
      ap_data_renorm = c1
!     ap_data_renorm = c10  ! convert from Pa to dynes/cm**2

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=forcing_ap_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error,   master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR: reading forcing_ap_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, forcing_ap_nml)
          write(stdout,*) ' '
      endif

      call broadcast_scalar(ap_data_type,   master_task)
      call broadcast_scalar(ap_data_inc,    master_task)
      call broadcast_scalar(ap_interp_type, master_task)
      call broadcast_scalar(ap_interp_freq, master_task)
      call broadcast_scalar(ap_interp_inc,  master_task)
      call broadcast_scalar(ap_filename,    master_task)
      call broadcast_vector(ap_data_renorm, master_task, 20)
      
      ap_formulation = ' '

!-----------------------------------------------------------------------
!     convert data_type to 'monthly-calendar' if input is 'monthly'
!-----------------------------------------------------------------------

      if (ap_data_type == 'monthly') ap_data_type = 'monthly-calendar'

!-----------------------------------------------------------------------
!     convert interp_type to corresponding integer value.
!-----------------------------------------------------------------------

      select case (ap_interp_type)
      case ('nearest')
        ap_interp_order = 1
        
      case ('linear')
        ap_interp_order = 2
        
      case ('4point')
        ap_interp_order = 4
        
      case default
        call exit_POP('init_ap: Unknown value for ap_interp_type') 
        
      end select
    
!-----------------------------------------------------------------------
!
!     set values of atm pressure arrays (ATM_PRESS or ATM_PRESS_DATA)
!       depending on the type of the atm pressure data.
!
!-----------------------------------------------------------------------

      select case (ap_data_type)
      
!-----------------------------------------------------------------------
!     no atm pressure, therefore no interpolation in time is needed 
!       (ap_interp_freq = 'none'), nor are there any new values 
!       to be used (ap_data_next = ap_data_update = never).
!-----------------------------------------------------------------------
      case ('none')
      
        ATM_PRESS = c0
        ap_data_next = never
        ap_data_update = never
        ap_interp_freq = 'never'
        
!-----------------------------------------------------------------------
!     simple analytic atm pressure that is constant in time, 
!       therefore no interpolation in time is needed 
!       (ap_interp_freq = 'none'), nor are there any new values 
!       to be used (ap_data_next = ap_data_update = never).
!-----------------------------------------------------------------------       
      case ('analytic')
      
        WORK = (ULAT *radian -  25.0_dbl_kind)**2 + 
     &         (ULONG*radian - 180.0_dbl_kind)**2
        where (CALCT .and. WORK < 100.0_dbl_kind**2)
          ATM_PRESS = grav*exp(-WORK*p5/15.0_dbl_kind**2)
          ATM_PRESS = WORK 
     &      - sum(WORK*TAREA,mask=CALCT)/sum(TAREA,mask=CALCT)
        elsewhere
          ATM_PRESS = c0
        end where
        
        if(ap_data_renorm(1) /= c1) 
     &    ATM_PRESS = ATM_PRESS*ap_data_renorm(1)

        ap_data_next = never
        ap_data_update = never
        ap_interp_freq = 'never'
        
!-----------------------------------------------------------------------
!     annual mean climatological atm pressure (read in from a file) that
!       is constant in time, therefore no interpolation in time is needed 
!       (ap_interp_freq = 'none'), nor are there any new values 
!       to be used (ap_data_next = ap_data_update = never).
!-----------------------------------------------------------------------               
      case ('annual')
      
        forcing_filename = ap_filename
        call open_parallel_file(nu,forcing_filename,recl_dbl)

        call read_array(nu,ATM_PRESS)

        call close_parallel_file(nu)
        
        if(ap_data_renorm(1) /= c1) 
     &    ATM_PRESS = ATM_PRESS*ap_data_renorm(1)
        
        ap_data_next = never
        ap_data_update = never
        ap_interp_freq = 'never'
      
        if(my_task.eq.master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' AP Annual file read: ', forcing_filename
        endif
        
!-----------------------------------------------------------------------
!     monthly mean climatological atm pressure. all 12 months are read 
!       in from a file. interpolation order (ap_interp_order) may be 
!       specified with namelist input.
!-----------------------------------------------------------------------
      case ('monthly-equal','monthly-calendar')
      
        allocate( ATM_PRESS_DATA(imt,jmt,1,12)
     &           ,STAT = allocate_error_status )
        call check_allocate (allocate_error_status
     &,      exit_message = 'forcing_ap.F -- ATM_PRESS_DATA' )
 
        ATM_PRESS_DATA = c0

        call find_forcing_times(ap_data_time, ap_data_inc
     &            , ap_interp_type, ap_data_next
     &            , ap_data_time_min_loc, ap_data_update, ap_data_type)
                
        forcing_filename = ap_filename
        call open_parallel_file(nu,forcing_filename,recl_dbl)

        call read_array(nu,ATM_PRESS_DATA(:,:,1,:))

        call close_parallel_file(nu)
        
        if(ap_data_renorm(1) /= c1) 
     &    ATM_PRESS_DATA = ATM_PRESS_DATA*ap_data_renorm(1)

        if(my_task.eq.master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Monthly AP file read: ', forcing_filename
        endif
                
!-----------------------------------------------------------------------
!     atm pressure specified every n-hours, where the n-hour increment
!       should be specified with namelist input (ap_data_inc). 
!       only as many times as are necessary based on the order
!       of the temporal interpolation scheme (ap_interp_order) reside
!       in memory at any given time.
!-----------------------------------------------------------------------     

      case ('n-hour')
      
        allocate( ATM_PRESS_DATA(imt,jmt,1,ap_interp_order)
     &           ,STAT = allocate_error_status )
        call check_allocate (allocate_error_status
     &,      exit_message = 'forcing_ap.F -- ATM_PRESS_DATA' )
 
        ATM_PRESS_DATA = c0

        call find_forcing_times(ap_data_time, ap_data_inc
     &      , ap_interp_type, ap_data_next
     &      , ap_data_time_min_loc, ap_data_update, ap_data_type)
                
        do n = 1, ap_interp_order
          call get_forcing_filename(forcing_filename,ap_filename
     &                             ,ap_data_time(n),ap_data_inc)

          call open_parallel_file(nu,forcing_filename,recl_dbl)

          call read_array(nu,ATM_PRESS_DATA(:,:,1,n))

          call close_parallel_file(nu)

          if(my_task.eq.master_task) then
            write(stdout,*) ' '
            write(stdout,*) ' AP n-hour file read: ', forcing_filename
          endif
        enddo
        
        if(ap_data_renorm(1) /= c1) 
     &    ATM_PRESS_DATA = ATM_PRESS_DATA*ap_data_renorm(1)
               
      case default

        call exit_POP('init_ap: Unknown value for ap_data_type') 
        
      end select

!-----------------------------------------------------------------------
!
!  now check interpolation period (ap_interp_freq) to set the
!    time for the next temporal interpolation (ap_interp_next).
!
!  if no interpolation is to be done, set next interpolation time 
!    to a large number so the atm pressure update test in routine 
!    set_surface_forcing will always be false.
!
!  if interpolation is to be done every n-hours, find the first 
!    interpolation time greater than the current time.
!
!  if interpolation is to be done every timestep, set next interpolation 
!    time to a large negative number so the atm pressure update test in 
!    routine set_surface_forcing will always be true.

!-----------------------------------------------------------------------

      select case (ap_interp_freq)
      
      case ('never')
      
        ap_interp_next = never
        ap_interp_last = never
        ap_interp_inc  = c0
        
      case ('n-hour')
      
        call find_interp_time(ap_interp_inc, ap_interp_next)

      case ('every-timestep')
      
        ap_interp_next = always
        ap_interp_inc  = c0
        
      case default

        call exit_POP('init_ap: Unknown value for ap_interp_freq')

      end select

      if(nsteps_total == 0) ap_interp_last = thour00

!-----------------------------------------------------------------------
!  echo forcing options to stdout.
!-----------------------------------------------------------------------

      ap_data_label = 'Surface Atmospheric Pressure'
      call echo_forcing_options(ap_data_type
     &   , ap_formulation, ap_data_inc, ap_interp_freq
     &   , ap_interp_type, ap_interp_inc, ap_data_label)

!-----------------------------------------------------------------------

      end subroutine init_ap

!***********************************************************************

      subroutine set_ap(ATM_PRESS)
      
!-----------------------------------------------------------------------
!
!     update the current value of the atm pressure array (ATM_PRESS) by
!       interpolating to the current time.  it may also be necessary to 
!       use new data values than were used in the previous interpolation.
!       
!-----------------------------------------------------------------------

      implicit none

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   ATM_PRESS    !  atmospheric pressure at current timestep

      integer (kind=int_kind) :: nu, first, second, third, fourth
     &  , ap_data_time_max_loc

!-----------------------------------------------------------------------
!     check if new data is necessary for interpolation.  if yes, then
!       shuffle indices in ATM_PRESS_DATA and ap_data_time arrays.  
!       and read in new data if necessary ('n-hour' case).  also 
!       increment values of ap_data_time_min_loc, ap_data_next and 
!       ap_data_update.  then do the temporal interpolation.
!-----------------------------------------------------------------------      

!      if(my_task == master_task) then
!        write(stdout,*)' tday = ',tday,' updating ap'
!      endif
            
      select case(ap_data_type)
        
      case ('monthly-equal','monthly-calendar')

        ap_data_label = 'AP Monthly'
        if(thour00 >= ap_data_update)               
     &    call update_forcing_data(ap_data_time, ap_data_time_min_loc
     &    , ap_interp_type, ap_data_next, ap_data_update
     &    , ap_data_type, ap_data_inc, ap_filename
     &    , ap_data_renorm, ATM_PRESS_DATA, 1, 12, ap_data_label)

        call interpolate_forcing(ATM_PRESS, ATM_PRESS_DATA(:,:,1,:)
     &      , ap_data_time, ap_interp_type, ap_data_time_min_loc, 12
     &      , ap_interp_freq, ap_interp_inc
     &      , ap_interp_next, ap_interp_last, nsteps_run)

      case('n-hour')
      
        ap_data_label = 'AP n-hour'
        if(thour00 >= ap_data_update)               
     &    call update_forcing_data(ap_data_time, ap_data_time_min_loc
     &    , ap_interp_type, ap_data_next, ap_data_update
     &    , ap_data_type, ap_data_inc, ap_filename, ap_data_renorm
     &    , ATM_PRESS_DATA, 1, ap_interp_order, ap_data_label)

        call interpolate_forcing(ATM_PRESS, ATM_PRESS_DATA(:,:,1,:)
     &      , ap_data_time, ap_interp_type, ap_data_time_min_loc
     &      , ap_interp_order
     &      , ap_interp_freq, ap_interp_inc
     &      , ap_interp_next, ap_interp_last, nsteps_run)
        
      end select
      

      end subroutine set_ap
!***********************************************************************

!***********************************************************************

      end module forcing_ap

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
