!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module forcing_pt_interior

!***********************************************************************

!-----------------------------------------------------------------------
!
!     contains routines and variables used for determining the
!       interior potential temperature restoring.
!
!     CVS:$Id: forcing_pt_interior.F,v 1.3.2.3 2002/01/25 23:24:58 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use io
      use forcing_tools
      use forcing_shf
      use time_management
      use prognostic
      use grid
      use exit_mod

      implicit none
      save

      real (kind=dbl_kind), allocatable, dimension(:,:,:,:) ::
     &   PT_INTERIOR_DATA

      real (kind=dbl_kind), allocatable, dimension(:,:) ::
     &   PT_RESTORE_RTAU !  inverse restoring timescale for variable interior restoring

      integer (kind=int_kind), allocatable, dimension(:,:) ::
     &   PT_RESTORE_MAX_LEVEL !  maximum level for applying variable interior restoring

      real (kind=dbl_kind), dimension(12) :: 
     &   pt_interior_data_time  !  

      real (kind=dbl_kind), dimension(20) :: pt_interior_data_renorm

      real (kind=dbl_kind) :: 
     &    pt_interior_data_inc    ! time increment between values of forcing data
     &  , pt_interior_data_next   ! time that will be used for the next value of forcing data that is needed
     &  , pt_interior_data_update ! time when the a new forcing value needs to be added to interpolation set
     &  , pt_interior_interp_inc  ! time increment between interpolation
     &  , pt_interior_interp_next ! time when next interpolation will be done
     &  , pt_interior_interp_last ! time when last interpolation was done
     &  , pt_interior_restore_tau
     &  , pt_interior_restore_rtau

      integer (kind=int_kind) :: 
     &    pt_interior_interp_order  !  order of temporal interpolation
     &  , pt_interior_data_time_min_loc  !  index of the third dimension of pt_interior_data_time containing the minimum forcing time
     &  , pt_interior_restore_max_level

      character (char_len) :: 
     &    pt_interior_data_type  !  period keyword for the period of the forcing data
     &  , pt_interior_filename     !  name of file conainting forcing data
     &  , pt_interior_interp_freq !  period keyword for the period of the temporal interpolation
     &  , pt_interior_interp_type
     &  , pt_interior_data_label
     &  , pt_interior_formulation
     &  , pt_interior_restore_filename

      logical (kind=log_kind) ::
     &    pt_interior_variable_restore

!***********************************************************************

      contains

!***********************************************************************

!***********************************************************************

      subroutine init_pt_interior
      
!-----------------------------------------------------------------------
!
!     initialize potential temperature interior forcing by either 
!       calculating or reading in the 3D temperature.  also do  
!       initial book-keeping concerning when new data is needed for  
!       the temporal interpolation and when the forcing will need 
!       to be updated.
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n, nu
     &, nml_error              ! namelist i/o error flag
     &, allocate_error_status  ! allocate error flag

      character (char_len) :: forcing_filename

      namelist /forcing_pt_interior_nml/ pt_interior_data_type
     &         ,pt_interior_data_inc,pt_interior_interp_type
     &         ,pt_interior_interp_freq,pt_interior_interp_inc
     &         ,pt_interior_restore_tau, pt_interior_filename
     &         ,pt_interior_restore_max_level, pt_interior_data_renorm
     &         ,pt_interior_formulation, pt_interior_variable_restore
     &         ,pt_interior_restore_filename
     
!-----------------------------------------------------------------------
!
!     read interior potential temperature restoring namelist input 
!       after setting default values.
!
!-----------------------------------------------------------------------

      pt_interior_formulation       = 'restoring'
      pt_interior_data_type         = 'none'
      pt_interior_data_inc          = 1.e20_dbl_kind
      pt_interior_interp_type       = 'nearest'
      pt_interior_interp_freq       = 'never'
      pt_interior_interp_inc        = 1.e20_dbl_kind
      pt_interior_restore_tau       = 1.e20_dbl_kind
      pt_interior_filename          = 'unknown-pt_interior'
      pt_interior_restore_max_level = 0
      pt_interior_data_renorm       = c1
      pt_interior_variable_restore  = .false.
      pt_interior_restore_filename  = 'unknown-pt_interior_restore'

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in,nml=forcing_pt_interior_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR: reading forcing_pt_interior_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, forcing_pt_interior_nml)
          write(stdout,*) ' '
      endif

      call broadcast_scalar(pt_interior_formulation,       master_task)
      call broadcast_scalar(pt_interior_data_type,         master_task)
      call broadcast_scalar(pt_interior_data_inc,          master_task)
      call broadcast_scalar(pt_interior_interp_type,       master_task)
      call broadcast_scalar(pt_interior_interp_freq,       master_task)
      call broadcast_scalar(pt_interior_interp_inc,        master_task)
      call broadcast_scalar(pt_interior_restore_tau,       master_task)
      call broadcast_scalar(pt_interior_filename,          master_task)
      call broadcast_scalar(pt_interior_restore_max_level, master_task)
      call broadcast_scalar(pt_interior_variable_restore,  master_task)
      call broadcast_scalar(pt_interior_restore_filename,  master_task)
      call broadcast_vector(pt_interior_data_renorm,   master_task, 20)
      
!-----------------------------------------------------------------------
!     convert data_type to 'monthly-calendar' if input is 'monthly'
!-----------------------------------------------------------------------

      if (pt_interior_data_type == 'monthly') 
     &    pt_interior_data_type = 'monthly-calendar'

!-----------------------------------------------------------------------
!     calculate inverse of restoring time scale and convert to seconds.
!-----------------------------------------------------------------------

      pt_interior_restore_rtau = 
     &                     c1/(seconds_in_day*pt_interior_restore_tau)
      
!-----------------------------------------------------------------------
!     convert interp_type to corresponding integer value.
!-----------------------------------------------------------------------

      select case (pt_interior_interp_type)
      case ('nearest')
        pt_interior_interp_order = 1
        
      case ('linear')
        pt_interior_interp_order = 2
        
      case ('4point')
        pt_interior_interp_order = 4
        
      case default
        call exit_POP( 
     &    'init_pt_interior: Unknown value for pt_interior_interp_type')
        
      end select

!-----------------------------------------------------------------------
!
!     set values of the interior PT array (PT_INTERIOR_DATA)
!       depending on the type of the PT data.
!
!-----------------------------------------------------------------------

      select case (pt_interior_data_type)
      
!-----------------------------------------------------------------------
!     no interior forcing, therefore no interpolation in time is needed 
!       (pt_interior_interp_freq = 'none'), nor are there any new values 
!       to be used (pt_interior_data_next = pt_interior_data_update = 
!       never).
!-----------------------------------------------------------------------
      case ('none')
      
        pt_interior_data_next = never
        pt_interior_data_update = never
        pt_interior_interp_freq = 'never'
       
!-----------------------------------------------------------------------
!     annual mean climatological interior potential temperature 
!       (read in from a file) that is constant in time, therefore no 
!       new values will be needed (pt_interior_data_next = 
!       pt_interior_data_update = never).
!-----------------------------------------------------------------------               
      case ('annual')
      
        allocate( PT_INTERIOR_DATA(imt,jmt,km,1)
     &          , STAT = allocate_error_status )
        call check_allocate (allocate_error_status
     &,    exit_message = 'forcing_pt_interior.F -- PT_INTERIOR_DATA')

        PT_INTERIOR_DATA = c0
      
        forcing_filename = pt_interior_filename
        call open_parallel_file(nu,forcing_filename,recl_dbl)

        call read_array(nu,PT_INTERIOR_DATA(:,:,:,1))

        call close_parallel_file(nu)
        
        if(pt_interior_data_renorm(1) /= c1) 
     &    PT_INTERIOR_DATA = PT_INTERIOR_DATA*pt_interior_data_renorm(1)

        pt_interior_data_next = never
        pt_interior_data_update = never
        pt_interior_interp_freq = 'never'
      
        if(my_task.eq.master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Interior PT Annual file read: '
     &      , forcing_filename
        endif
        
!-----------------------------------------------------------------------
!     monthly mean climatological interior potential temperature. 
!       all 12 months are read in from a file. interpolation order 
!       (pt_interior_interp_order) may be specified with namelist input.
!-----------------------------------------------------------------------               
      case ('monthly-equal','monthly-calendar')
      
        allocate( PT_INTERIOR_DATA(imt,jmt,km,0:12)
     &          , STAT = allocate_error_status )
        call check_allocate (allocate_error_status
     &,    exit_message = 'forcing_pt_interior.F -- PT_INTERIOR_DATA')

        PT_INTERIOR_DATA = c0

        call find_forcing_times(pt_interior_data_time
     &      , pt_interior_data_inc, pt_interior_interp_type
     &      , pt_interior_data_next, pt_interior_data_time_min_loc
     &      , pt_interior_data_update, pt_interior_data_type)

        forcing_filename = pt_interior_filename
        call open_parallel_file(nu,forcing_filename,recl_dbl)

        do n = 1, 12
          call read_array(nu,PT_INTERIOR_DATA(:,:,:,n))
        enddo

        call close_parallel_file(nu)

        if(pt_interior_data_renorm(1) /= c1) 
     &    PT_INTERIOR_DATA = PT_INTERIOR_DATA*pt_interior_data_renorm(1)

        if(my_task.eq.master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Interior PT Monthly file read: '
     &      , forcing_filename
        endif
        
!-----------------------------------------------------------------------
!     interior potential temperature specified every n-hours, where 
!       the n-hour increment should be specified with namelist input 
!       (pt_interior_data_inc). only as many times as are necessary 
!       based on the orderof the temporal interpolation scheme 
!       (pt_interior_interp_order) reside in memory at any given time.
!-----------------------------------------------------------------------     
        
      case ('n-hour')
      
        allocate(PT_INTERIOR_DATA(imt,jmt,km,0:pt_interior_interp_order)
     &           ,STAT = allocate_error_status )
        call check_allocate (allocate_error_status
     &,    exit_message = 'forcing_pt_interior.F -- PT_INTERIOR_DATA')

        PT_INTERIOR_DATA = c0

        call find_forcing_times(pt_interior_data_time
     &      , pt_interior_data_inc, pt_interior_interp_type
     &      , pt_interior_data_next, pt_interior_data_time_min_loc
     &      , pt_interior_data_update, pt_interior_data_type)
        
        do n = 1, pt_interior_interp_order
          call get_forcing_filename(forcing_filename
     &      ,pt_interior_filename,pt_interior_data_time(n)
     &      ,pt_interior_data_inc)

          call open_parallel_file(nu,forcing_filename,recl_dbl)

          call read_array(nu,PT_INTERIOR_DATA(:,:,:,n))

          call close_parallel_file(nu)

          if(my_task.eq.master_task) then
            write(stdout,*) ' '
            write(stdout,*) ' Interior PT n-hour file read: '
     &        , forcing_filename
          endif
        enddo
        
        if(pt_interior_data_renorm(1) /= c1) 
     &    PT_INTERIOR_DATA = PT_INTERIOR_DATA*pt_interior_data_renorm(1)

      case default

        call exit_POP( 
     &    'init_pt_interior: Unknown value for pt_interior_data_type') 
        
      end select

!-----------------------------------------------------------------------
!
!  now check interpolation period (pt_interior_interp_freq) to set 
!    the time for the next temporal interpolation 
!    (pt_interior_interp_next).
!
!  if no interpolation is to be done, set next interpolation time 
!    to a large number so the interior PT update test in routine 
!    set_surface_forcing will always be false.
!
!  if interpolation is to be done every n-hours, find the first
!    interpolation time greater than the current time.
!
!  if interpolation is to be done every timestep, set next interpolation 
!    time to a large negative number so the interior PT update  
!    test in routine set_surface_forcing will always be true.
!
!-----------------------------------------------------------------------

      select case (pt_interior_interp_freq)
      
      case ('never')
      
        pt_interior_interp_next = never
        pt_interior_interp_last = never
        pt_interior_interp_inc  = c0
        
      case ('n-hour')
        call find_interp_time(pt_interior_interp_inc
     &     , pt_interior_interp_next)

      case ('every-timestep')
      
        pt_interior_interp_next = always
        pt_interior_interp_inc  = c0
        
      case default

        call exit_POP( 
     &  'init_pt_interior: Unknown value for pt_interior_interp_freq')

      end select

      if(nsteps_total == 0) pt_interior_interp_last = thour00

!-----------------------------------------------------------------------
!  allocate and read in arrays used for variable interior restoring 
!    if necessary.
!-----------------------------------------------------------------------

      if (pt_interior_variable_restore) then

        allocate( PT_RESTORE_MAX_LEVEL(imt,jmt)
     &   , PT_RESTORE_RTAU(imt,jmt), STAT = allocate_error_status)
        call check_allocate (allocate_error_status
     &, exit_message = 'forcing_pt_interior.F -- PT_RESTORE_MAX_LEVEL')


        forcing_filename = pt_interior_restore_filename
        call open_parallel_file(nu,forcing_filename,recl_dbl)

        call read_array(nu,PT_RESTORE_RTAU)  ! use as a temporary
        PT_RESTORE_MAX_LEVEL = nint(PT_RESTORE_RTAU)
        call read_array(nu,PT_RESTORE_RTAU)
        PT_RESTORE_RTAU = PT_RESTORE_RTAU/seconds_in_day  ! convert from days to secs

        call close_parallel_file(nu)

      endif

!-----------------------------------------------------------------------
!  echo forcing options to stdout.
!-----------------------------------------------------------------------

      pt_interior_data_label = 'Interior Potential Temperature Forcing'
      if (pt_interior_variable_restore .and. my_task == master_task)
     &   write(stdout,*)
     &   'Variable Interior Potential Temperature Restoring enabled'
      call echo_forcing_options(pt_interior_data_type
     &   , pt_interior_formulation, pt_interior_data_inc
     &   , pt_interior_interp_freq, pt_interior_interp_type
     &   , pt_interior_interp_inc, pt_interior_data_label)

!-----------------------------------------------------------------------
      
      end subroutine init_pt_interior

!***********************************************************************

      subroutine set_pt_interior(k,DPT_INTERIOR)
      
!-----------------------------------------------------------------------
!
!     update the current value of the interior PT array 
!       (DPT_INTERIOR) by interpolating to the current time.  
!       it may also be necessary to use new data values than were 
!       used in the previous interpolation.
!       
!-----------------------------------------------------------------------

      implicit none

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   DPT_INTERIOR 
                      
      integer (kind=int_kind) :: k, nu, first, second, third, fourth
     &  , now, pt_interior_data_time_max_loc
     
!-----------------------------------------------------------------------
!     check if new data is necessary for interpolation.  if yes, then
!       shuffle indices in SST_RESTORE and pt_interior_data_time arrays  
!       and read in new data if necessary ('n-hour' case).  also 
!       increment values of pt_interior_data_time_min_loc, 
!       pt_interior_data_next and pt_interior_data_update.then do 
!       the temporal interpolation. note that no new data is necessary 
!       for 'analytic' and 'annual' cases.
!-----------------------------------------------------------------------
      
!-----------------------------------------------------------------------
!  no interior restoring for k = 1
!-----------------------------------------------------------------------          
      if( k == 1) then
        DPT_INTERIOR = c0
        return
      endif

      select case(pt_interior_data_type)
        
      case('annual')
        now = 1

      case ('monthly-equal','monthly-calendar')
        now = 0

        pt_interior_data_label = 'PT_INTERIOR Monthly'
        if(thour00 >= pt_interior_data_update .and. k == 2)
     &    call update_forcing_data(pt_interior_data_time
     &      , pt_interior_data_time_min_loc, pt_interior_interp_type
     &      , pt_interior_data_next, pt_interior_data_update
     &      , pt_interior_data_type, pt_interior_data_inc
     &      , pt_interior_filename, pt_interior_data_renorm
     &      , PT_INTERIOR_DATA(:,:,:,1:12), km, 12
     &      , pt_interior_data_label)

        if(thour00 >= pt_interior_interp_next .or. nsteps_run == 1) then
          call interpolate_forcing(PT_INTERIOR_DATA(:,:,k,now)
     &    , PT_INTERIOR_DATA(:,:,k,1:12), pt_interior_data_time
     &    , pt_interior_interp_type, pt_interior_data_time_min_loc, 12
     &    , pt_interior_interp_freq, pt_interior_interp_inc
     &    , pt_interior_interp_next, pt_interior_interp_last
     &    , nsteps_run - 1)

          if(k == km .and. nsteps_run /= 1) pt_interior_interp_next = 
     &      pt_interior_interp_next + pt_interior_interp_inc
        endif

      case('n-hour')
        now = 0
      
        pt_interior_data_label = 'PT_INTERIOR n-hour'
        if(thour00 >= pt_interior_data_update .and. k == 2)
     &    call update_forcing_data(pt_interior_data_time
     &      , pt_interior_data_time_min_loc, pt_interior_interp_type
     &      , pt_interior_data_next, pt_interior_data_update
     &      , pt_interior_data_type, pt_interior_data_inc
     &      , pt_interior_filename, pt_interior_data_renorm
     &      , PT_INTERIOR_DATA(:,:,:,1:pt_interior_interp_order), km
     &      , pt_interior_interp_order, pt_interior_data_label)

        if(thour00 >= pt_interior_interp_next .or. nsteps_run == 1) then
          call interpolate_forcing(PT_INTERIOR_DATA(:,:,k,now)
     &      , PT_INTERIOR_DATA(:,:,k,1:pt_interior_interp_order)
     &      , pt_interior_data_time, pt_interior_interp_type
     &      , pt_interior_data_time_min_loc, pt_interior_interp_order
     &      , pt_interior_interp_freq, pt_interior_interp_inc
     &      , pt_interior_interp_next, pt_interior_interp_last
     &      , nsteps_run - 1)

          if(k == km .and. nsteps_run /= 1) pt_interior_interp_next = 
     &      pt_interior_interp_next + pt_interior_interp_inc
        endif

      end select      

      if (pt_interior_variable_restore) then
        DPT_INTERIOR = PT_RESTORE_RTAU*merge( 
     &      (PT_INTERIOR_DATA(:,:,k,now) - TRACER(:,:,k,1,curtime))
     &      , c0, k <= PT_RESTORE_MAX_LEVEL)
      else
        if (k <= pt_interior_restore_max_level) then
          DPT_INTERIOR = pt_interior_restore_rtau* 
     &      (PT_INTERIOR_DATA(:,:,k,now) - TRACER(:,:,k,1,curtime))
        else
          DPT_INTERIOR = c0
        endif
      endif

      end subroutine set_pt_interior
!***********************************************************************

!***********************************************************************

      end module forcing_pt_interior

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
