!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module forcing_s_interior

!***********************************************************************

!-----------------------------------------------------------------------
!
!     contains routines and variables used for determining the
!       interior salinity restoring.
!
!     CVS:$Id: forcing_s_interior.F,v 1.3.2.3 2002/01/25 23:24:58 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use io
      use forcing_tools
      use time_management
      use prognostic
      use grid
      use exit_mod

      implicit none
      save

      real (kind=dbl_kind), allocatable, dimension(:,:,:,:) ::
     &   S_INTERIOR_DATA 

      real (kind=dbl_kind), allocatable, dimension(:,:) ::
     &   S_RESTORE_RTAU !  inverse restoring timescale for variable interior restoring

      integer (kind=int_kind), allocatable, dimension(:,:) ::
     &   S_RESTORE_MAX_LEVEL !  maximum level for applying variable interior restoring

      real (kind=dbl_kind), dimension(12) :: 
     &   s_interior_data_time 

      real (kind=dbl_kind), dimension(20) :: s_interior_data_renorm

      real (kind=dbl_kind) :: 
     &    s_interior_data_inc    ! time increment between values of forcing data
     &  , s_interior_data_next   ! time that will be used for the next value of forcing data that is needed
     &  , s_interior_data_update ! time when the a new forcing value needs to be added to interpolation set
     &  , s_interior_interp_inc  ! time increment between interpolation
     &  , s_interior_interp_next ! time when next interpolation will be done
     &  , s_interior_interp_last ! time when last interpolation was done
     &  , s_interior_restore_tau
     &  , s_interior_restore_rtau

      integer (kind=int_kind) :: 
     &    s_interior_interp_order  !  order of temporal interpolation
     &  , s_interior_data_time_min_loc  !  index of the third dimension of s_interior_data_time containing the minimum forcing time
     &  , s_interior_restore_max_level

      character (char_len) :: 
     &    s_interior_data_type  !  period keyword for the period of the forcing data
     &  , s_interior_filename     !  name of file conainting forcing data
     &  , s_interior_interp_freq !  period keyword for the period of the temporal interpolation
     &  , s_interior_interp_type
     &  , s_interior_data_label
     &  , s_interior_formulation
     &  , s_interior_restore_filename

      logical (kind=log_kind) ::
     &    s_interior_variable_restore

!***********************************************************************

      contains

!***********************************************************************

!***********************************************************************

      subroutine init_s_interior
      
!-----------------------------------------------------------------------
!
!     initialize salinity interior forcing by either 
!       calculating or reading in the 3D salinity.  also do  
!       initial book-keeping concerning when new data is needed for  
!       the temporal interpolation and when the forcing will need 
!       to be updated.
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n, nu
     &, nml_error  ! namelist i/o error flag
     &, allocate_error_status

      character (char_len) :: forcing_filename

      namelist /forcing_s_interior_nml/ s_interior_data_type
     &         ,s_interior_data_inc,s_interior_interp_type
     &         ,s_interior_interp_freq,s_interior_interp_inc
     &         ,s_interior_restore_tau, s_interior_filename
     &         ,s_interior_restore_max_level, s_interior_data_renorm
     &         ,s_interior_formulation, s_interior_variable_restore
     &         ,s_interior_restore_filename
     
!-----------------------------------------------------------------------
!
!     read interior S namelist input after setting default values.
!
!-----------------------------------------------------------------------

      s_interior_formulation       = 'restoring'
      s_interior_data_type         = 'none'
      s_interior_data_inc          = 1.e20_dbl_kind
      s_interior_interp_type       = 'nearest'
      s_interior_interp_freq       = 'never'
      s_interior_interp_inc        = 1.e20_dbl_kind
      s_interior_restore_tau       = 1.e20_dbl_kind
      s_interior_filename          = 'unknown-s_interior'
      s_interior_restore_max_level = 0
      s_interior_data_renorm       = c1
!     s_interior_data_renorm       = 1.e-3_dbl_kind  !  convert from psu to g/g
      s_interior_variable_restore  = .false.
      s_interior_restore_filename  = 'unknown-s_interior_restore'
      
      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in,nml=forcing_s_interior_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR: reading forcing_s_interior_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, forcing_s_interior_nml)
          write(stdout,*) ' '
      endif

      call broadcast_scalar(s_interior_formulation,       master_task)
      call broadcast_scalar(s_interior_data_type,         master_task)
      call broadcast_scalar(s_interior_data_inc,          master_task)
      call broadcast_scalar(s_interior_interp_type,       master_task)
      call broadcast_scalar(s_interior_interp_freq,       master_task)
      call broadcast_scalar(s_interior_interp_inc,        master_task)
      call broadcast_scalar(s_interior_restore_tau,       master_task)
      call broadcast_scalar(s_interior_filename,          master_task)
      call broadcast_scalar(s_interior_restore_max_level, master_task)
      call broadcast_scalar(s_interior_variable_restore,  master_task)
      call broadcast_scalar(s_interior_restore_filename,  master_task)
      call broadcast_vector(s_interior_data_renorm,    master_task,20)
      
!-----------------------------------------------------------------------
!     convert data_type to 'monthly-calendar' if input is 'monthly'
!-----------------------------------------------------------------------

      if (s_interior_data_type == 'monthly') 
     &    s_interior_data_type = 'monthly-calendar'

!-----------------------------------------------------------------------
!     calculate inverse of restoring time scale and convert to seconds.
!-----------------------------------------------------------------------

      s_interior_restore_rtau = c1/
     &                       (seconds_in_day*s_interior_restore_tau)

!-----------------------------------------------------------------------
!     convert interp_type to corresponding integer value.
!-----------------------------------------------------------------------

      select case (s_interior_interp_type)
      case ('nearest')
        s_interior_interp_order = 1
        
      case ('linear')
        s_interior_interp_order = 2
        
      case ('4point')
        s_interior_interp_order = 4
        
      case default
        call exit_POP(
     &     'init_s_interior: Unknown value for s_interior_interp_type') 
        
      end select

!-----------------------------------------------------------------------
!
!     set values of the interior S array (S_INTERIOR_DATA)
!       depending on the type of S data.
!
!-----------------------------------------------------------------------

      select case (s_interior_data_type)
      
!-----------------------------------------------------------------------
!     no interior forcing, therefore no interpolation in time is needed 
!       (s_interior_interp_freq = 'none'), nor are there any new values 
!       to be used (s_interior_data_next = s_interior_data_update = 
!       never).
!-----------------------------------------------------------------------
      case ('none')
      
        s_interior_data_next = never
        s_interior_data_update = never
        s_interior_interp_freq = 'never'
        
!-----------------------------------------------------------------------
!     annual mean climatological salinity temperature 
!       (read in from a file) that is constant in time, therefore no 
!       new values will be needed (s_interior_data_next = 
!       s_interior_data_update = never).
!-----------------------------------------------------------------------               
      case ('annual')
      
        allocate( S_INTERIOR_DATA(imt,jmt,km,1)
     &          , STAT = allocate_error_status )
        call check_allocate (allocate_error_status
     &,   exit_message = 'forcing_s_interior.F -- S_INTERIOR_DATA')
 
        S_INTERIOR_DATA = c0
      
        forcing_filename = s_interior_filename
        call open_parallel_file(nu,forcing_filename,recl_dbl)

        call read_array(nu,S_INTERIOR_DATA(:,:,:,1))

        call close_parallel_file(nu)
        
        if(s_interior_data_renorm(1) /= c1)
     &    S_INTERIOR_DATA = S_INTERIOR_DATA*s_interior_data_renorm(1)

        s_interior_data_next = never
        s_interior_data_update = never
        s_interior_interp_freq = 'never'
      
        if(my_task.eq.master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Interior S Annual file read: '
     &      , forcing_filename
        endif
        
!-----------------------------------------------------------------------
!     monthly mean climatological interior salinity. 
!       all 12 months are read in from a file. interpolation order 
!       (s_interior_interp_order) may be specified with namelist input.
!-----------------------------------------------------------------------               
      case ('monthly-equal','monthly-calendar')
      
        allocate( S_INTERIOR_DATA(imt,jmt,km,0:12)
     &          , STAT = allocate_error_status )
        call check_allocate (allocate_error_status
     &,   exit_message = 'forcing_s_interior.F -- S_INTERIOR_DATA')
        S_INTERIOR_DATA = c0

        call find_forcing_times(s_interior_data_time
     &      , s_interior_data_inc, s_interior_interp_type
     &      , s_interior_data_next, s_interior_data_time_min_loc
     &      , s_interior_data_update, s_interior_data_type)

        forcing_filename = s_interior_filename
        call open_parallel_file(nu,forcing_filename,recl_dbl)

        do n = 1, 12
          call read_array(nu,S_INTERIOR_DATA(:,:,:,n))
        enddo

        call close_parallel_file(nu)

        if(s_interior_data_renorm(1) /= c1)
     &    S_INTERIOR_DATA = S_INTERIOR_DATA*s_interior_data_renorm(1)

        if(my_task.eq.master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Interior S Monthly file read: '
     &      , forcing_filename
        endif       
        
!-----------------------------------------------------------------------
!     interior salinity specified every n-hours, where 
!       the n-hour increment should be specified with namelist input 
!       (s_interior_data_inc). only as many times as are necessary 
!       based on the orderof the temporal interpolation scheme 
!       (s_interior_interp_order) reside in memory at any given time.
!-----------------------------------------------------------------------     
      case ('n-hour')
      
        allocate( S_INTERIOR_DATA(imt,jmt,km,0:s_interior_interp_order)
     &           ,STAT = allocate_error_status )
        call check_allocate (allocate_error_status
     &,   exit_message = 'forcing_s_interior.F -- S_INTERIOR_DATA')
        S_INTERIOR_DATA = c0

        call find_forcing_times(s_interior_data_time
     &      , s_interior_data_inc, s_interior_interp_type
     &      , s_interior_data_next, s_interior_data_time_min_loc
     &      , s_interior_data_update, s_interior_data_type)
        
        do n = 1, s_interior_interp_order
          call get_forcing_filename(forcing_filename,s_interior_filename
     &                    ,s_interior_data_time(n),s_interior_data_inc)

          call open_parallel_file(nu,forcing_filename,recl_dbl)

          call read_array(nu,S_INTERIOR_DATA(:,:,:,n))

          call close_parallel_file(nu)

          if(my_task.eq.master_task) then
            write(stdout,*) ' '
            write(stdout,*) ' Interior S n-hour file read: '
     &        , forcing_filename
          endif
        enddo
        
        if(s_interior_data_renorm(1) /= c1)
     &    S_INTERIOR_DATA = S_INTERIOR_DATA*s_interior_data_renorm(1)

      case default

        call exit_POP( 
     &    'init_s_interior: Unknown value for s_interior_data_type') 
        
      end select

!-----------------------------------------------------------------------
!
!  now check interpolation period (s_interior_interp_freq) to set the
!    time for the next temporal interpolation (s_interior_interp_next).
!
!  if no interpolation is to be done, set next interpolation time 
!    to a large number so the interior S update test in routine 
!    set_surface_forcing will always be false.
!
!  if interpolation is to be done every n-hours, find the first
!    interpolation time greater than the current time.
!
!  if interpolation is to be done every timestep, set next interpolation 
!    time to a large negative number so the interior S update  
!    test in routine set_surface_forcing will always be true.
!
!-----------------------------------------------------------------------

      select case (s_interior_interp_freq)
      
      case ('never')
      
        s_interior_interp_next = never
        s_interior_interp_last = never
        s_interior_interp_inc  = c0
        
      case ('n-hour')
        call find_interp_time(s_interior_interp_inc
     &     , s_interior_interp_next)

      case ('every-timestep')
      
        s_interior_interp_next = always
        s_interior_interp_inc  = c0
        
      case default

        call exit_POP( 
     &  'init_s_interior: Unknown value for s_interior_interp_freq')

      end select

      if(nsteps_total == 0) s_interior_interp_last = thour00

!-----------------------------------------------------------------------
!  allocate and read in arrays used for variable interior restoring
!    if necessary.
!-----------------------------------------------------------------------

      if (s_interior_variable_restore) then

        allocate( S_RESTORE_MAX_LEVEL(imt,jmt)
     &   , S_RESTORE_RTAU(imt,jmt), STAT = allocate_error_status)
        call check_allocate (allocate_error_status
     &,   exit_message = 'forcing_s_interior.F -- S_RESTORE_RTAU')

        forcing_filename = s_interior_restore_filename
        call open_parallel_file(nu,forcing_filename,recl_dbl)

        call read_array(nu,S_RESTORE_RTAU)  ! use as a temporary
        S_RESTORE_MAX_LEVEL = nint(S_RESTORE_RTAU)
        call read_array(nu,S_RESTORE_RTAU)
        S_RESTORE_RTAU = S_RESTORE_RTAU/seconds_in_day  ! convert from days to secs

        call close_parallel_file(nu)

      endif

!-----------------------------------------------------------------------
!  echo forcing options to stdout.
!-----------------------------------------------------------------------

      s_interior_data_label = 'Interior Salinity Forcing'
      if (s_interior_variable_restore .and. my_task == master_task)
     &   write(stdout,*)'Variable Interior Salinity Restoring enabled'
      call echo_forcing_options(s_interior_data_type
     &   , s_interior_formulation, s_interior_data_inc
     &   , s_interior_interp_freq, s_interior_interp_type
     &   , s_interior_interp_inc, s_interior_data_label)

!-----------------------------------------------------------------------

      end subroutine init_s_interior

!***********************************************************************

      subroutine set_s_interior(k,DS_INTERIOR)
      
!-----------------------------------------------------------------------
!
!     update the current value of the interior S array 
!       (DS_INTERIOR) by interpolating to the current time.  
!       it may also be necessary to use new data values than were 
!       used in the previous interpolation.
!       
!-----------------------------------------------------------------------

      implicit none

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   DS_INTERIOR    !  surface heat flux at current timestep
                      
      integer (kind=int_kind) :: k, nu, first, second, third, fourth
     &  , now, s_interior_data_time_max_loc
      
!-----------------------------------------------------------------------
!     check if new data is necessary for interpolation.  if yes, then
!       shuffle indices in SSS_RESTORE and s_interior_data_time arrays  
!       and read in new data if necessary ('n-hour' case).  also 
!       increment values of s_interior_data_time_min_loc, 
!       s_interior_data_next and s_interior_data_update.then do 
!       the temporal interpolation. note that no new data is necessary 
!       for 'analytic' and 'annual' cases.
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!  no interior restoring for k = 1
!-----------------------------------------------------------------------                      
      if( k == 1) then
        DS_INTERIOR = c0
        return
      endif

      select case(s_interior_data_type)
        
      case('annual')
        now = 1

      case ('monthly-equal','monthly-calendar')
        now = 0

        s_interior_data_label = 'S_INTERIOR Monthly'
        if(thour00 >= s_interior_data_update .and. k == 2)
     &    call update_forcing_data(s_interior_data_time
     &      , s_interior_data_time_min_loc, s_interior_interp_type
     &      , s_interior_data_next, s_interior_data_update
     &      , s_interior_data_type, s_interior_data_inc
     &      , s_interior_filename, s_interior_data_renorm
     &      , S_INTERIOR_DATA(:,:,:,1:12), km, 12
     &      , s_interior_data_label)

      if(thour00 >= s_interior_interp_next .or. nsteps_run == 1) then
          call interpolate_forcing(S_INTERIOR_DATA(:,:,k,0)
     &    , S_INTERIOR_DATA(:,:,k,1:12), s_interior_data_time
     &    , s_interior_interp_type, s_interior_data_time_min_loc, 12
     &    , s_interior_interp_freq, s_interior_interp_inc
     &    , s_interior_interp_next, s_interior_interp_last
     &    , nsteps_run - 1)

          if(k == km .and. nsteps_run /= 1) s_interior_interp_next = 
     &      s_interior_interp_next + s_interior_interp_inc
        endif

      case('n-hour')
        now = 0
      
        s_interior_data_label = 'S_INTERIOR n-hour'
        if(thour00 >= s_interior_data_update .and. k == 2)
     &    call update_forcing_data(s_interior_data_time
     &      , s_interior_data_time_min_loc, s_interior_interp_type
     &      , s_interior_data_next, s_interior_data_update
     &      , s_interior_data_type, s_interior_data_inc
     &      , s_interior_filename, s_interior_data_renorm
     &      , S_INTERIOR_DATA(:,:,:,1:s_interior_interp_order), km
     &      , s_interior_interp_order, s_interior_data_label)

        if(thour00 >= s_interior_interp_next .or. nsteps_run == 1) then
          call interpolate_forcing(S_INTERIOR_DATA(:,:,k,0)
     &      , S_INTERIOR_DATA(:,:,k,1:s_interior_interp_order)
     &      , s_interior_data_time, s_interior_interp_type
     &      , s_interior_data_time_min_loc, s_interior_interp_order
     &      , s_interior_interp_freq, s_interior_interp_inc
     &      , s_interior_interp_next, s_interior_interp_last
     &      , nsteps_run - 1)

          if(k == km .and. nsteps_run /= 1) s_interior_interp_next = 
     &      s_interior_interp_next + s_interior_interp_inc
        endif

      end select      

      if (s_interior_variable_restore) then
        DS_INTERIOR = S_RESTORE_RTAU*merge(
     &      (S_INTERIOR_DATA(:,:,k,now) - TRACER(:,:,k,2,curtime))
     &      , c0, k <= S_RESTORE_MAX_LEVEL)
      else
        if (k <= s_interior_restore_max_level) then
          DS_INTERIOR = s_interior_restore_rtau*
     &      (S_INTERIOR_DATA(:,:,k,now) - TRACER(:,:,k,2,curtime))
        else
          DS_INTERIOR = c0
        endif
      endif

      end subroutine set_s_interior
!***********************************************************************

!***********************************************************************

      end module forcing_s_interior

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
