!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module forcing_tools

!***********************************************************************

!-----------------------------------------------------------------------
!     contains variables, parameters and routines that are used by all
!       of the individual forcing modules.
!
!     CVS:$Id: forcing_tools.F,v 1.3.2.1 2001/03/13 22:54:49 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use io
      use grid
      use interp
      use time_management
      use exit_mod

      implicit none
      save

      real (kind=dbl_kind), dimension(12) :: 
     &  thour00_midmonth, thour00_endmonth

      real (kind=dbl_kind), parameter ::
     &  never    =  1.e20_dbl_kind
     & ,always   = -1.e20_dbl_kind
     
!***********************************************************************

      contains

!***********************************************************************

!***********************************************************************

      subroutine find_interp_time(forcing_interp_inc
     &   , forcing_interp_next)
     
!-----------------------------------------------------------------------
!
!     find the next time at which temporal interpolation is needed
!       in the forcing.
!
!-----------------------------------------------------------------------

      implicit none

!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind) ::
     &  forcing_interp_inc, forcing_interp_next

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: nstart, n

      real (kind=dbl_kind) :: forcing_time_test

!-----------------------------------------------------------------------
!     find the first interpolation time that is greater than the current
!       time, which then becomes the next interpolation time.
!-----------------------------------------------------------------------
      
      nstart = int(thour00/forcing_interp_inc) - 2 
      
      do n = nstart, nstart + 5
         forcing_time_test = (n + p5)*forcing_interp_inc
         if (forcing_time_test > thour00) exit
      enddo

      forcing_interp_next = forcing_time_test

      end subroutine find_interp_time

!***********************************************************************

      subroutine get_forcing_filename(forcing_filename,forcing_infile
     &   ,forcing_time,forcing_data_inc)
     
!-----------------------------------------------------------------------
!
!     find the name of an n-hour file given the forcing time needed.
!       files can be labeled by the date of the beginning of the 
!       forcing interval or by the date of the middle in the forcing 
!       interval (which corresponds to the value of the variable 
!       forcing_time) depending on the value of 
!       filename_at_begin_of_interval (T or F).
!
!     file names are asumed to be of the form:
!       fileroot.year.day.hour (fileroot.yyyy.ddd.hh)
!       where fileroot is specified by namelist input, year is 
!       4 digits, day is 3 digits, and hour is 2 digits.
!       for example, Jan 1 of year 22 with a root of 'ws' would be
!       ws.0022.001.00 (labeled by beginning of interval)
!       ws.0022.001.12 (labeled by middle of 1-day interval)
!
!     remember, forcing times are relative to 01-01-0000 so results
!       may not be what you expect.  for example, if the forcing 
!       increment is 2 days, then even numbered years will have
!       mid-interval-referenced files that are on even numbered
!       days, and odd numbered years will reference odd numbered
!       days (assuming no leap years).
!-----------------------------------------------------------------------

      implicit none

!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind) :: forcing_time, forcing_data_inc

      character (char_len) :: forcing_filename, forcing_infile
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: file_year, file_day, file_hour, n
     &  , days_in_prev_year, cindx

      real (kind=dbl_kind) :: file_time, file_time00

      character (len = 5) file_cyear
      character (len = 4) file_cday
      character (len = 3) file_chour

      logical (kind=log_kind) :: filename_at_begin_of_interval

!-----------------------------------------------------------------------
!   filename_at_begin_of_interval = .true. for filenames defined at
!     the beginning of the forcing interval.
!   filename_at_begin_of_interval = .false. for filenames defined at
!     the middle of the forcing interval.
!-----------------------------------------------------------------------

      filename_at_begin_of_interval = .false.

!-----------------------------------------------------------------------
!     find the time associated with the beginning or middle of the 
!       forcing interval.
!-----------------------------------------------------------------------

      if (filename_at_begin_of_interval) then
        file_time = forcing_time - p5*forcing_data_inc + eps
      else  !  filename defined at forcing time (middle of forcing interval)
        file_time = forcing_time + eps
      endif
      
!-----------------------------------------------------------------------
!     if the file time is negative, find the first appropriate 
!       positive value and use it.
!-----------------------------------------------------------------------

      if (file_time < c0 ) then
        if (my_task == master_task) then
          write(stdout,*) 'WARNING: ',
     &    ' apparently need a forcing file earlier than 01-01-0000'
          write(stdout,*)
     &    ' will attempt to find first appropriate file'
        endif
        do n = 1,10
          if (filename_at_begin_of_interval) then
            file_time = forcing_time + (n-p5)*forcing_data_inc + eps
          else
            file_time = forcing_time + n*forcing_data_inc + eps
          endif
          if(file_time >= c0 ) exit
        enddo
      endif
      if (file_time < c0 ) then
        call exit_POP('get_forcing_filename confused')
      endif

!-----------------------------------------------------------------------
!   convert file time to time relative to the beginning of the year.
!     then find the day and hour corresponding to that time.
!
!   note that the use of 'eps' above is important here due to the 
!     'int' and 'nint' functions, especially if file_time00 < 0.
!-----------------------------------------------------------------------

      thour00_begin_this_year = thour00
     &   - (elapsed_days_this_year + frac_day)*24.0_dbl_kind
      file_time00 = file_time - thour00_begin_this_year
      file_day  = int(abs(file_time00)/24.0_dbl_kind)
      file_hour = nint(abs(file_time00) - 24*file_day)

!-----------------------------------------------------------------------
!   if file_time00 < 0, then the time is in the previous year.
!
!   if file_time00 > the number of hours in a year, then the time is 
!     in the next year.
!-----------------------------------------------------------------------

      if (file_time00 < c0 )then
        file_hour = 24 - file_hour  !  counting backwards
        file_year = iyear - 1
        days_in_prev_year = days_in_norm_year
        if (allow_leapyear .and. mod(file_year,4) == 0) then ! check if previous
          if (mod(file_year,100) /= 0)                       ! year was leapyear
     &      days_in_prev_year = days_in_leap_year
          if (mod(file_year,400) == 0) 
     &      days_in_prev_year = days_in_leap_year
        endif
        file_day = days_in_prev_year - file_day

      elseif (file_time00 > 24*days_in_year) then
        file_day = file_day - days_in_year + 1
        file_year = iyear + 1
      else
        file_day = file_day + 1  !  day counting starts at 1, not 0
        file_year = iyear
      endif

!-----------------------------------------------------------------------
!   write year, day, and hour into character variables and use them
!     to create filename.
!-----------------------------------------------------------------------

      write(file_chour,'(i3)') 100   + file_hour
      write(file_cday, '(i4)') 1000  + file_day
      write(file_cyear,'(i5)') 10000 + file_year

      forcing_filename = char_blank
      cindx = len_trim(forcing_infile)
      forcing_filename(1:cindx) = forcing_infile(1:cindx)
      cindx = cindx + 1
      forcing_filename(cindx:cindx) = '.'
      cindx = cindx + 1
      forcing_filename(cindx:cindx+3) = file_cyear(2:5)
      cindx = cindx + 4
      forcing_filename(cindx:cindx) = '.'
      cindx = cindx + 1
      forcing_filename(cindx:cindx+2) = file_cday(2:4)
      cindx = cindx + 3
      forcing_filename(cindx:cindx) = '.'
      cindx = cindx + 1
      forcing_filename(cindx:cindx+1) = file_chour(2:3)

      end subroutine get_forcing_filename


!***********************************************************************

      subroutine find_forcing_times(forcing_time, forcing_data_inc
     & ,forcing_interp_type, forcing_time_next, forcing_time_min_loc
     & ,forcing_data_update, forcing_data_type)
     
!-----------------------------------------------------------------------
!
!     find the times associated with forcing data needed for 
!       interpolation given the current time and time increment 
!       between forcing data fields.  forcing times are assumed 
!       to be centered;  for example, with daily forcing, the value 
!       is assumed to occur at noon (hour = 12).
!
!-----------------------------------------------------------------------

      implicit none

!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: forcing_time_min_loc

      real (kind=dbl_kind) ::
     &  forcing_data_inc, forcing_time_next, forcing_data_update

      real (kind=dbl_kind), dimension(12) :: forcing_time

      character (char_len) :: forcing_data_type, forcing_interp_type
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: nstart, n, second, third, fourth

      real (kind=dbl_kind) :: forcing_time_test, forcing_time_min

!-----------------------------------------------------------------------
!   set mid_month and end_month times for monthly options.
!-----------------------------------------------------------------------

      if (forcing_data_type == 'monthly-calendar') then
        thour00_midmonth = thour00_midmonth_calendar
        thour00_endmonth = thour00_endmonth_calendar
      elseif (forcing_data_type == 'monthly-equal') then
        thour00_midmonth = thour00_midmonth_equal
        thour00_endmonth = thour00_endmonth_equal
      endif

!-----------------------------------------------------------------------
!   for 'monthly-equal' and 'monthly-calendar' set the forcing times to
!     be the mid-month times plus the time at the beginning of the year.
!     then find the first forcing time greater than the current time 
!     and exit.  note that the loop will not exit if the current time 
!     is in the second half of December in which case n will have the 
!     value 13.
!   for 'n-hour' forcing, start at a time that is definitely less 
!     than the first forcing time greater than the current time and 
!     just increment the test time by the forcing increment until we 
!     find a forcing time greater than the current time.
!-----------------------------------------------------------------------

      select case (forcing_data_type)

        case('monthly-equal','monthly-calendar')
          forcing_time(1:12) = 
     &      thour00_begin_this_year + thour00_midmonth(1:12)
          do n = 1,12
            if (forcing_time(n) > thour00) exit
          enddo
 
        case('n-hour')
          nstart = int(thour00/forcing_data_inc) - 3        
          do n = nstart, nstart + 6
            forcing_time_test = (n - p5)*forcing_data_inc
            if (forcing_time_test > thour00) exit
          enddo
      
      end select

!-----------------------------------------------------------------------
!     depending on the type of temporal interpolation, set 
!       the forcing times.  also set forcing_time_min_loc, which is
!       the temporal index of the minimum forcing time to be used for
!       the interpolation on the forcing data arrays.  for 'monthly'
!       data, forcing_time_min_loc varies between 1 and 12.  for 
!       example, if a 4point interpolation is required and the months
!       that are needed are 3,4,5,6 then forcing_time_min_loc = 3.
!       for 'n-hour' data, forcing_time_min_loc = 1 always because
!       initially the data is read chronologically 1 file at a time.
!       also set the times for the next interpolation and next time
!       that new data will be needed.
!-----------------------------------------------------------------------

      select case(forcing_interp_type)
      
      case ('nearest')  !  select nearest forcing time

        select case (forcing_data_type)

          case('monthly-equal','monthly-calendar')
            do n = 1,12
              if (thour00 <= thour00_begin_this_year 
     &                     + thour00_endmonth(n)) exit
            enddo
            if (n == 13) 
     &        stop 'error finding current month in find_forcing_times'
            forcing_time_min_loc = n
            forcing_time_min_loc = mod(forcing_time_min_loc,12)
            if (forcing_time_min_loc <= 0) 
     &        forcing_time_min_loc = forcing_time_min_loc + 12
            if (n == 12) then
              forcing_time_next = forcing_time(1) + hours_in_year ! next Jan
            else
              forcing_time_next = forcing_time(forcing_time_min_loc + 1)
            endif
            forcing_data_update = thour00_begin_this_year 
     &                          + thour00_endmonth(forcing_time_min_loc)

          case('n-hour')
            if( (forcing_time_test-thour00) <= p5*forcing_data_inc )then
              forcing_time_min = (n - p5)*forcing_data_inc
            else
              forcing_time_min = (n - 1.5_dbl_kind)*forcing_data_inc
            endif
            forcing_time_min_loc = 1
            forcing_time(forcing_time_min_loc) = forcing_time_min
            forcing_time_next = 
     &        forcing_time(forcing_time_min_loc) + 1*forcing_data_inc
            forcing_data_update = forcing_time(forcing_time_min_loc) 
     &        + p5*forcing_data_inc

        end select
            
      case ('linear')  !  linear interpolation
            
        select case(forcing_data_type)       

          case('monthly-equal','monthly-calendar')
            forcing_time_min_loc = n - 1
            forcing_time_min_loc = mod(forcing_time_min_loc,12)
            if (forcing_time_min_loc <= 0) 
     &        forcing_time_min_loc = forcing_time_min_loc + 12
            second = mod(forcing_time_min_loc+1,12)  
            if (second == 0) second = 12
            if (n == 1) then
              forcing_time(forcing_time_min_loc) 
     &        = forcing_time(forcing_time_min_loc) - hours_in_year ! previous Dec
            elseif (n >= 3) then
              forcing_time(1:n-2) = forcing_time(1:n-2) + hours_in_year
            endif
            if (n == 12) then
              forcing_time_next = forcing_time(1) ! next Jan
            else
              forcing_time_next = forcing_time(second+1) ! next month
            endif

          case('n-hour')
            forcing_time_min_loc = 1
            second = 2
            forcing_time(forcing_time_min_loc) 
     &        = (n - 1.5_dbl_kind)*forcing_data_inc
            forcing_time(second) 
     &        = forcing_time(forcing_time_min_loc) + forcing_data_inc
            forcing_time_next 
     &        = forcing_time(forcing_time_min_loc) + 2*forcing_data_inc

        end select
        
        forcing_data_update = forcing_time(second)
        
      case ('4point')  !  4-point interpolation
      
        select case(forcing_data_type)       

          case('monthly-equal','monthly-calendar')
            forcing_time_min_loc = n - 2
            forcing_time_min_loc = mod(forcing_time_min_loc,12)
            if (forcing_time_min_loc <= 0) 
     &        forcing_time_min_loc = forcing_time_min_loc + 12
            second = mod(forcing_time_min_loc+1,12)
            third  = mod(forcing_time_min_loc+2,12)
            fourth = mod(forcing_time_min_loc+3,12)  
            if (second == 0) second = 12
            if (third  == 0) third  = 12
            if (fourth == 0) fourth = 12
            if (n == 1) then
              forcing_time(forcing_time_min_loc) 
     &          = forcing_time(forcing_time_min_loc) - hours_in_year ! previous Nov
              forcing_time(second) 
     &          = forcing_time(second)               - hours_in_year ! previous Dec
            elseif (n == 2) then
              forcing_time(forcing_time_min_loc) 
     &          = forcing_time(forcing_time_min_loc) - hours_in_year ! previous Dec
            elseif (n >= 4) then
              forcing_time(1:n-3) = forcing_time(1:n-3) + hours_in_year
            endif
            if (n == 11) then
              forcing_time_next = forcing_time(1) ! next Jan
            else
              forcing_time_next = forcing_time(fourth+1)                 ! next month
            endif

          case('n-hour')
            forcing_time_min_loc = 1
            second = 2
            third  = 3
            fourth = 4
            forcing_time(forcing_time_min_loc) 
     &        = (n - 2.5_dbl_kind)*forcing_data_inc
            forcing_time(second) 
     &        = forcing_time(forcing_time_min_loc) + forcing_data_inc
            forcing_time(third) 
     &        = forcing_time(forcing_time_min_loc) + 2*forcing_data_inc
            forcing_time(fourth) 
     &        = forcing_time(forcing_time_min_loc) + 3*forcing_data_inc
            forcing_time_next 
     &        = forcing_time(forcing_time_min_loc) + 4*forcing_data_inc

        end select        

        forcing_data_update = forcing_time(third)
        
      end select

      end subroutine find_forcing_times

!***********************************************************************

!***********************************************************************

      subroutine interpolate_forcing(INTERP, FIELD, forcing_time
     & ,forcing_interp_type, forcing_time_min_loc, dim3
     & ,forcing_interp_freq, forcing_interp_inc, forcing_interp_next
     & ,forcing_interp_last, nsteps_run_check)
     
!-----------------------------------------------------------------------
!     temporally interpolate FIELD and put the result in INTERP.
!-----------------------------------------------------------------------
      
      implicit none

!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  forcing_time_min_loc, dim3, nsteps_run_check

      real (kind=dbl_kind) :: forcing_interp_inc, forcing_interp_next
     & ,forcing_interp_last

      real (kind=dbl_kind), dimension(12) :: forcing_time
      
      real (kind=dbl_kind), dimension(:,:,:) ::
     &   FIELD   !  forcing field to be interpolated
     
      real (kind=dbl_kind), dimension(:,:) ::
     &   INTERP    !  result of interpolation
     
      character (char_len) :: forcing_interp_type, forcing_interp_freq

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: second, third, fourth

!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!     if interpolation is done every timestep, then interpolate to
!       the current time.
!     if interpolation is done every n-hours, then interpolate to
!       the current interpolation time (which is >= current time)
!       or if it is the first step of the current sub-run, the value
!       of forcing_interp_last was read in from the restart file.
!
!     NOTE:  for surface forcing, this routine is called  with
!       nsteps_run = 0.  for interior forcing, time_manager has been
!       called so nsteps_run = 1.  as a result, nsteps_run_check has been
!       added to distinguish the 2 types of forcing.  for surface forcing,
!       nsteps_run_check = nsteps_run, while for interior forcing
!       nsteps_run_check = nsteps_run - 1 so that interpolation will
!       always be done for the first timestep.
!      
!-----------------------------------------------------------------------

      if(forcing_interp_freq == 'every-timestep') then
        forcing_interp_last = thour00
      elseif(forcing_interp_freq == 'n-hour') then
        if(nsteps_run_check /= 0) 
     &    forcing_interp_last = forcing_interp_next
      else
        call exit_POP('Error in interpolate_forcing')
      endif

        select case(forcing_interp_type)
          
        case('nearest')
          INTERP =  FIELD(:,:,forcing_time_min_loc)

!mmdebug
!         if(my_task.eq.master_task.and.newday) then
!           write(stdout,*) ' '
!           write(stdout,*) 'interp time (hours) = '
!    &        ,forcing_interp_last,' using month = '
!    &        ,forcing_time_min_loc
!           write(stdout,*) 'interp time (hours) = '
!    &        ,forcing_interp_last,' using time (days) = '
!    &        ,forcing_time(forcing_time_min_loc)/24.
!         endif
!     if(my_task==master_task.and.newday)write(*,'(3e15.5," INTERP")')
!    &    tday00,INTERP(11,11),FIELD(11,11,forcing_time_min_loc)
!mmdebug

        case('linear')
          second = mod(forcing_time_min_loc+1,dim3)
          if (second == 0) second = dim3
          call interp_2pt(INTERP,FIELD(:,:,forcing_time_min_loc)
     &        ,FIELD(:,:,second),forcing_time(forcing_time_min_loc)
     &        ,forcing_time(second), forcing_interp_last)
    
!mmdebug
!         if(my_task.eq.master_task.and.newday) then
!           write(stdout,*) ' '
!           write(stdout,*) 'interp time (hours) = '
!    &        ,forcing_interp_last,' using months = '
!    &        ,forcing_time_min_loc, second
!           write(stdout,*) 'interp time (hours) = '
!    &        ,forcing_interp_last,' using times (days) = '
!    &        ,forcing_time(forcing_time_min_loc)/24.
!    &        ,forcing_time(second)/24.
!         endif
!     if(my_task==master_task.and.newday)write(*,'(4e15.5," INTERP")')
!    &    tday00,INTERP(11,11),FIELD(11,11,forcing_time_min_loc)
!    &    ,FIELD(11,11,second)
!mmdebug

        case('4point')
          second = mod(forcing_time_min_loc+1,dim3)
          third  = mod(forcing_time_min_loc+2,dim3)
          fourth = mod(forcing_time_min_loc+3,dim3)
          if (second == 0) second = dim3
          if (third  == 0) third  = dim3
          if (fourth == 0) fourth = dim3
          call interp_4pt(INTERP,FIELD(:,:,forcing_time_min_loc)
     &        ,FIELD(:,:,second),FIELD(:,:,third)
     &        ,FIELD(:,:,fourth),forcing_time(forcing_time_min_loc)
     &        ,forcing_time(second),forcing_time(third)
     &        ,forcing_time(fourth),forcing_interp_last)
     
!mmdebug
!         if(my_task.eq.master_task.and.newday) then
!           write(stdout,*) ' '
!           write(stdout,*) 'interp time (hours) = '
!    &        ,forcing_interp_last,' using months = '
!    &        ,forcing_time_min_loc, second, third, fourth
!           write(stdout,*) 'interp time (hours) = '
!    &        ,forcing_interp_last,' using times (days) = '
!    &        ,forcing_time(forcing_time_min_loc)/24.
!    &        ,forcing_time(second)/24.,forcing_time(third)/24.
!    &        ,forcing_time(fourth)/24.
!         endif
!     if(my_task==master_task.and.newday)write(*,'(6e15.5," INTERP")')
!    &    tday00,INTERP(11,11),FIELD(11,11,forcing_time_min_loc)
!    &    ,FIELD(11,11,second)
!    &    ,FIELD(11,11,third),FIELD(11,11,fourth)
!mmdebug

        end select
        
      end subroutine interpolate_forcing
        
!***********************************************************************

!***********************************************************************

      subroutine update_forcing_data(forcing_time, forcing_time_min_loc
     & , forcing_interp_type, forcing_data_next, forcing_data_update
     & , forcing_data_type, forcing_data_inc, forcing_infile
     & , forcing_data_rescale, FIELD, dim3, dim4, forcing_data_label)

!-----------------------------------------------------------------------
!     update the data to be used in the next temporal interpolation.
!       if data is 'monthly' this is a shuffling of indices since
!       all 12 months are in memory.  if data is 'n-hour', new data
!       needs to be read in as well as shuffling of indices.
!-----------------------------------------------------------------------
      
      implicit none
      
!-----------------------------------------------------------------------
!     input variables
!-----------------------------------------------------------------------      
      
      integer (kind=int_kind) :: 
     &  forcing_time_min_loc, dim3, dim4
     
      real (kind=dbl_kind) :: forcing_data_next, forcing_data_update
     & , forcing_data_inc

      real (kind=dbl_kind), dimension(12) :: forcing_time
      
      real (kind=dbl_kind), dimension(20) :: forcing_data_rescale

      real (kind=dbl_kind), dimension(:,:,:,:) ::
     &   FIELD   !  forcing field to be read if necessary
     
      character (char_len) :: 
     &  forcing_infile, forcing_data_type, forcing_interp_type
     & ,forcing_data_label
      
!-----------------------------------------------------------------------
!     local variables
!-----------------------------------------------------------------------      

      integer (kind=int_kind) :: 
     &  forcing_time_max_loc, nu, n
     
      character (char_len) :: forcing_filename
      
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------      
      
      select case(forcing_data_type)
            
        case('monthly-equal','monthly-calendar')

          forcing_time(forcing_time_min_loc) 
     &     = forcing_time(forcing_time_min_loc) + hours_in_year
          forcing_time_min_loc = forcing_time_min_loc + 1
          if (forcing_time_min_loc > 12) forcing_time_min_loc = 1
          
          select case(forcing_interp_type)
          
          case('nearest')
            forcing_time_max_loc = forcing_time_min_loc
            forcing_data_update = forcing_data_update 
     &        + 24.0_dbl_kind*days_in_month(forcing_time_max_loc)

          case('linear')
            forcing_time_max_loc = mod(forcing_time_min_loc + 1,12)
            if (forcing_time_max_loc == 0) forcing_time_max_loc = 12
            forcing_data_update = forcing_time(forcing_time_max_loc)

          case('4point')
            forcing_time_max_loc = mod(forcing_time_min_loc + 3,12)
            if (forcing_time_max_loc == 0) forcing_time_max_loc = 12
            n = forcing_time_max_loc - 1
            if (n == 0) n = 12
            forcing_data_update = forcing_time(n)

          end select

          forcing_time(forcing_time_max_loc) = forcing_data_next
          n = mod(forcing_time_max_loc + 1, 12)
          if (n == 0) n = 12
          forcing_data_next = forcing_time(n)

      case('n-hour')
                    
          call get_forcing_filename(forcing_filename,forcing_infile
     &                          ,forcing_data_next,forcing_data_inc)

          call open_parallel_file(nu,forcing_filename,recl_dbl)
          call read_array(nu,FIELD(:,:,:,forcing_time_min_loc))
          
!-----------------------------------------------------------------------
!     renormalize values if necessary to compensate for different units.
!-----------------------------------------------------------------------
          if(dim3 == km) then  !  3d interior foring field
            if(forcing_data_rescale(1) /= c1) 
     &        FIELD(:,:,:,forcing_time_min_loc) = 
     &        FIELD(:,:,:,forcing_time_min_loc)*forcing_data_rescale(1)
          else                  !  surface forcing component
            do n = 1,dim3
              if(forcing_data_rescale(n) /= c1) 
     &         FIELD(:,:,n,forcing_time_min_loc) = 
     &         FIELD(:,:,n,forcing_time_min_loc)*forcing_data_rescale(n)
            enddo
          endif

          call close_parallel_file(nu)

          if(my_task.eq.master_task) then
            write(stdout,*) ' '
         write(stdout,*) 'tday00 = ',tday00,' ',trim(forcing_data_label)
     &       ,' file read: ', forcing_filename
          endif
        
          forcing_time(forcing_time_min_loc) = forcing_data_next
  
          forcing_time_min_loc = forcing_time_min_loc + 1
          if (forcing_time_min_loc > dim4) 
     &       forcing_time_min_loc = 1
          forcing_data_next = forcing_data_next + forcing_data_inc
          forcing_data_update = forcing_data_update + forcing_data_inc        
          
      end select
      
      end subroutine update_forcing_data
!***********************************************************************

!***********************************************************************

      subroutine echo_forcing_options(forcing_data_type
     &   , forcing_formulation, forcing_data_inc
     &   , forcing_interp_freq, forcing_interp_type, forcing_interp_inc
     &   , forcing_label)

      implicit none

      real (kind=dbl_kind) ::
     &    forcing_data_inc, forcing_interp_inc 

      character (char_len) ::
     &    forcing_data_type
     &  , forcing_interp_freq
     &  , forcing_interp_type
     &  , forcing_label
     &  , forcing_formulation

!-----------------------------------------------------------------------
!     echo forcing selections to stdout.
!-----------------------------------------------------------------------

      if (my_task == master_task) then

!-----------------------------------------------------------------------
!     no forcing
!-----------------------------------------------------------------------
        if (forcing_data_type == 'none') then
          write(stdout,*)'No ',trim(forcing_label)

!-----------------------------------------------------------------------
!     if there is some kind of forcing, write out the type (eg, annual)
!       and the formulation (eg, restoring).
!     then write increment between forcing data if appropriate.
!-----------------------------------------------------------------------
        else

          if (len_trim(forcing_formulation).eq.0) then
          ! Work around current Linux runtime error
            write(stdout,*)
     &        trim(forcing_label)
     &        ,':'
     &        ,trim(forcing_data_type)
          else
            write(stdout,*)
     &        trim(forcing_label)
     &        ,':'
     &        ,trim(forcing_data_type)
     &        ,' '
     &        ,trim(forcing_formulation)
          end if

          if (forcing_data_type == 'n-hour') then
            write(stdout,*)trim(forcing_label)
     &         ,' Data Time Increment (days): '
     &         ,forcing_data_inc/24.0_dbl_kind
          endif

!-----------------------------------------------------------------------
!     write the frequency that the forcing is updated.
!-----------------------------------------------------------------------
          select case(forcing_interp_freq)

          case('never')
            write(stdout,*)trim(forcing_label)
     &        ,' data is never interpolated'

          case('n-hour')
            write(stdout,*)trim(forcing_label),
     &         ' data interpolated every '
     &         ,forcing_interp_inc/24.0_dbl_kind,' days'

          case('every-timestep')
            write(stdout,*)trim(forcing_label),
     &         ' data interpolated every timestep'

          end select

!-----------------------------------------------------------------------
!     write order of interpolation if appropriate.
!-----------------------------------------------------------------------
          if(forcing_data_type /= 'none'
     &      .and. forcing_data_type /= 'analytic'
     &      .and. forcing_data_type /= 'annual')
     &         write(stdout,*)trim(forcing_label)
     &           ,' Interpolation type: ',forcing_interp_type

        endif
        write(stdout,*) ' '

      endif

      end subroutine echo_forcing_options

!***********************************************************************

      end module forcing_tools
!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
