!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module forcing_ws

!***********************************************************************

!-----------------------------------------------------------------------
!
!     contains routines and variables used for determining the
!       surface wind stress.
!
!     CVS:$Id: forcing_ws.F,v 1.2.2.3 2002/01/25 23:25:00 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use io
      use forcing_tools
      use time_management
      use grid
      use exit_mod

      implicit none
      save

      real (kind=dbl_kind), allocatable, dimension(:,:,:,:) ::
     &   SMF_DATA !  surface momentum fluxes at multiple times that
                      !  may be interpolated in time to get SMF

      real (kind=dbl_kind), dimension(12) :: 
     &   ws_data_time  !  time (in hours) corresponding to surface fluxes
                  !  in SMF_DATA

      real (kind=dbl_kind), dimension(20) :: ws_data_renorm

      real (kind=dbl_kind) :: 
     &    ws_data_inc    ! time increment between values of forcing data
     &  , ws_data_next   ! time corresponding to next value of forcing data that is needed
     &  , ws_data_update ! time when the a new forcing value needs to be added to interpolation set
     &  , ws_interp_inc  ! time increment between interpolation
     &  , ws_interp_next ! time when next interpolation will be done
     &  , ws_interp_last ! time when last interpolation was done

      integer (kind=int_kind) :: 
     &    ws_interp_order  !  order of temporal interpolation
     &  , ws_data_time_min_loc  !  index of the fourth dimension of SMF_DATA, ws_data_time containing the minimum forcing time

      character (char_len) :: 
     &    ws_data_type  !  period keyword for the period of the forcing data
     &  , ws_filename     !  name of file conainting forcing data
     &  , ws_interp_freq !  period keyword for the period of the temporal interpolation
     &  , ws_interp_type
     &  , ws_data_label
     &  , ws_formulation

!***********************************************************************

      contains

!***********************************************************************

!***********************************************************************

      subroutine init_ws(SMF,SMFT,lsmft_avail)
      
!-----------------------------------------------------------------------
!
!     initialize wind stress forcing by either calculating or reading
!       in the wind stress.  also do initial book-keeping concerning 
!       when new data is needed for the temporal interpolation and 
!       when the forcing will need to be updated.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      logical (kind=log_kind), intent(out) ::
     &  lsmft_avail            ! true if SMFT field available

      real (kind=dbl_kind), dimension(imt,jmt,2), intent(out) ::
     &   SMF    !  surface momentum fluxes at current timestep
     &,  SMFT   !  surface momentum fluxes at T points

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n, nu
     &, nml_error  ! namelist i/o error flag
     &, allocate_error_status  ! allocate error flag

      character (char_len) :: forcing_filename

      namelist /forcing_ws_nml/ ws_data_type, ws_data_inc
     &         ,ws_interp_type, ws_interp_freq
     &         ,ws_interp_inc, ws_filename, ws_data_renorm

!-----------------------------------------------------------------------
!
!     read wind stress namelist input after setting default values.
!
!-----------------------------------------------------------------------

      ws_data_type   = 'analytic'
      ws_data_inc    = 1.e20_dbl_kind
      ws_interp_type = 'nearest'
      ws_interp_freq = 'never'
      ws_interp_inc  = 1.e20_dbl_kind
      ws_filename    = 'unknown-ws'
      ws_data_renorm = c1

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in,nml=forcing_ws_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading forcing_ws_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, forcing_ws_nml)
          write(stdout,*) ' '
      endif

      call broadcast_scalar(ws_data_type,   master_task)
      call broadcast_scalar(ws_data_inc,    master_task)
      call broadcast_scalar(ws_interp_type, master_task)
      call broadcast_scalar(ws_interp_freq, master_task)
      call broadcast_scalar(ws_interp_inc,  master_task)
      call broadcast_scalar(ws_filename,    master_task)
      call broadcast_vector(ws_data_renorm, master_task,20)
      
      ws_formulation = ' '

!-----------------------------------------------------------------------
!     convert data_type to 'monthly-calendar' if input is 'monthly'
!-----------------------------------------------------------------------

      if (ws_data_type == 'monthly') ws_data_type = 'monthly-calendar'

!-----------------------------------------------------------------------
!     convert interp_type to corresponding integer value.
!-----------------------------------------------------------------------

      select case (ws_interp_type)
      case ('nearest')
        ws_interp_order = 1
        
      case ('linear')
        ws_interp_order = 2
        
      case ('4point')
        ws_interp_order = 4
        
      case default
        call exit_POP('init_ws: Unknown value for ws_interp_type') 
        
      end select

!-----------------------------------------------------------------------
!
!     set values of the wind stress arrays (SMF or SMF_DATA)
!       depending on the type of the wind stress data.
!
!-----------------------------------------------------------------------

      select case (ws_data_type)
      
!-----------------------------------------------------------------------
!     no wind stress, therefore no interpolation in time is needed 
!       (ws_interp_freq = 'never'), nor are there any new values 
!       to be used (ws_data_next = ws_data_update = never).
!-----------------------------------------------------------------------
      case ('none')
      
        SMF = c0
        SMFT = c0
        lsmft_avail = .true.
        ws_data_next = never
        ws_data_update = never
        ws_interp_freq = 'never'
        
!-----------------------------------------------------------------------
!     simple analytic wind stress that is constant in time, 
!       therefore no interpolation in time is needed 
!       (ws_interp_freq = 'never'), nor are there any new values 
!       to be used (ws_data_next = ws_data_update = never).
!-----------------------------------------------------------------------       
      case ('analytic')
      
        SMF (:,:,1) = c1 - c2*cos(ULAT)**2   ! simple zonal windstress field
        SMFT(:,:,1) = c1 - c2*cos(TLAT)**2   ! simple zonal windstress field

        SMF (:,:,2) = -sin(ANGLE)*SMF (:,:,1)        ! rotate vector to grid
        SMFT(:,:,2) = -sin(ANGLE)*SMFT(:,:,1)        ! rotate vector to grid
        SMF (:,:,1) =  cos(ANGLE)*SMF (:,:,1)
        SMFT(:,:,1) =  cos(ANGLE)*SMFT(:,:,1)
        
        if (ws_data_renorm(1) /= c1) then
          SMF  = SMF *ws_data_renorm(1)
          SMFT = SMFT*ws_data_renorm(1)
        endif
        lsmft_avail = .true.
        
        ws_data_next = never
        ws_data_update = never
        ws_interp_freq = 'never'
        
!-----------------------------------------------------------------------
!     annual mean climatological wind stress (read in from a file) that
!       is constant in time, therefore no interpolation in time is needed 
!       (ws_interp_freq = 'none'), nor are there any new values 
!       to be used (ws_data_next = ws_data_update = never).
!-----------------------------------------------------------------------               
      case ('annual')
      
        lsmft_avail = .false.
        forcing_filename = ws_filename
        call open_parallel_file(nu,forcing_filename,recl_dbl)

        call read_array(nu,SMF)
        
        call close_parallel_file(nu)
        
        if(ws_data_renorm(1) /= c1)
     &    SMF = SMF*ws_data_renorm(1)
        
        ws_data_next = never
        ws_data_update = never
        ws_interp_freq = 'never'
      
        if(my_task.eq.master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' WS Annual file read: ', forcing_filename
        endif
        
!-----------------------------------------------------------------------
!     monthly mean climatological wind stress. all 12 months are read 
!       in from a file. interpolation order (ws_interp_order) may be 
!       specified with namelist input.
!-----------------------------------------------------------------------               
      case ('monthly-equal','monthly-calendar')
      
        lsmft_avail = .false.
        allocate( SMF_DATA(imt,jmt,2,12)
     &           ,STAT = allocate_error_status )
        call check_allocate (allocate_error_status
     &,    exit_message = 'forcing_ws.F -- SMF_DATA')
        SMF_DATA = c0

        call find_forcing_times(ws_data_time, ws_data_inc
     &            , ws_interp_type, ws_data_next
     &            , ws_data_time_min_loc, ws_data_update, ws_data_type)
        
        forcing_filename = ws_filename
        call open_parallel_file(nu,forcing_filename,recl_dbl)

        call read_array(nu,SMF_DATA(:,:,1,:))
        call read_array(nu,SMF_DATA(:,:,2,:))
        
        call close_parallel_file(nu)

        if(ws_data_renorm(1) /= c1)
     &    SMF_DATA = SMF_DATA*ws_data_renorm(1)
        
        if(my_task.eq.master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' WS Monthly file read: ', forcing_filename
        endif        
        
!-----------------------------------------------------------------------
!     wind stress specified every n-hours, where the n-hour increment
!       should be specified with namelist input (ws_data_inc). 
!       only as many times as are necessary based on the order
!       of the temporal interpolation scheme (ws_interp_order) reside
!       in memory at any given time.
!-----------------------------------------------------------------------     

      case ('n-hour')
      
        lsmft_avail = .false.
        allocate( SMF_DATA(imt,jmt,2,ws_interp_order)
     &           ,STAT = allocate_error_status )
        call check_allocate (allocate_error_status
     &,    exit_message = 'forcing_ws.F -- SMF_DATA')
        SMF_DATA = c0

        call find_forcing_times(ws_data_time, ws_data_inc
     &     , ws_interp_type, ws_data_next
     &     , ws_data_time_min_loc, ws_data_update, ws_data_type)
        
        do n = 1, ws_interp_order
          call get_forcing_filename(forcing_filename,ws_filename
     &                             ,ws_data_time(n),ws_data_inc)

          call open_parallel_file(nu,forcing_filename,recl_dbl)

          call read_array(nu,SMF_DATA(:,:,:,n))

          call close_parallel_file(nu)

          if(my_task.eq.master_task) then
            write(stdout,*) ' '
            write(stdout,*) ' WS n-hour file read: ', forcing_filename
          endif
        enddo
        
        if(ws_data_renorm(1) /= c1)
     &    SMF_DATA = SMF_DATA*ws_data_renorm(1)
                
      case default

        call exit_POP('init_ws: Unknown value for ws_data_type') 
        
      end select

!-----------------------------------------------------------------------
!
!  now check interpolation period (ws_interp_freq) to set the
!    time for the next temporal interpolation (ws_interp_next).
!
!  if no interpolation is to be done, set next interpolation time 
!    to a large number so the wind stress update test in routine 
!    set_surface_forcing will always be false.
!
!  if interpolation is to be done every n-hours, find the first 
!    interpolation time greater than the current time.
!
!  if interpolation is to be done every timestep, set next interpolation 
!    time to a large negative number so the wind stress update test in 
!    routine set_surface_forcing will always be true.
!
!-----------------------------------------------------------------------

      select case (ws_interp_freq)
      
      case ('never')
      
        ws_interp_next = never
        ws_interp_last = never
        ws_interp_inc  = c0
        
      case ('n-hour')
      
        call find_interp_time(ws_interp_inc, ws_interp_next)

      case ('every-timestep')
      
        ws_interp_next = always
        ws_interp_inc  = c0
        
      case default

        call exit_POP('init_ws: Unknown value for ws_interp_freq')

      end select

      if(nsteps_total == 0) ws_interp_last = thour00

!-----------------------------------------------------------------------
!  echo forcing options to stdout.
!-----------------------------------------------------------------------

      ws_data_label = 'Surface Wind Stress'
      call echo_forcing_options(ws_data_type
     &   , ws_formulation, ws_data_inc
     &   , ws_interp_freq, ws_interp_type
     &   , ws_interp_inc, ws_data_label)

!-----------------------------------------------------------------------

      end subroutine init_ws

!***********************************************************************

      subroutine set_ws(SMF,SMFT)

!-----------------------------------------------------------------------
!
!     update the current value of the wind stress arrays (SMF) by
!       interpolating to the current time.  it may also be necessary to 
!       use new data values than were used in the previous interpolation.
!       
!-----------------------------------------------------------------------

      implicit none

      real (kind=dbl_kind), dimension(imt,jmt,2) ::
     &   SMF    !  surface momentum fluxes at current timestep

      real (kind=dbl_kind),dimension(imt,jmt,2),intent(out),optional ::
     &   SMFT   !  surface momentum fluxes at T points

      integer (kind=int_kind) :: nu, first, second, third, fourth
     &  , ws_data_time_max_loc
      
!-----------------------------------------------------------------------
!     check if new data is necessary for interpolation.  if yes, then
!       shuffle indices in SMF_DATA, and ws_data_time arrays  
!       and read in new data if necessary ('n-hour' case).  also 
!       increment values of ws_data_time_min_loc, ws_data_next and 
!       ws_data_update.then do the temporal interpolation.
!-----------------------------------------------------------------------

!      if(my_task == master_task) then
!        write(stdout,*)' tday = ',tday,' updating ws'
!      endif
            
      select case(ws_data_type)
        
      case ('monthly-equal','monthly-calendar')

        ws_data_label = 'WS Monthly'
        if(thour00 >= ws_data_update)               
     &    call update_forcing_data(ws_data_time, ws_data_time_min_loc
     &    , ws_interp_type, ws_data_next, ws_data_update
     &    , ws_data_type, ws_data_inc, ws_filename
     &    , ws_data_renorm, SMF_DATA, 2, 12, ws_data_label)

        call interpolate_forcing(SMF(:,:,1), SMF_DATA(:,:,1,:)
     &      , ws_data_time ,ws_interp_type, ws_data_time_min_loc, 12
     &      , ws_interp_freq, ws_interp_inc
     &      , ws_interp_next, ws_interp_last, nsteps_run)
        call interpolate_forcing(SMF(:,:,2), SMF_DATA(:,:,2,:)
     &      , ws_data_time ,ws_interp_type, ws_data_time_min_loc, 12
     &      , ws_interp_freq, ws_interp_inc
     &      , ws_interp_next, ws_interp_last, nsteps_run)
        
      case('n-hour')
      
        ws_data_label = 'WS n-hour'
        if(thour00 >= ws_data_update)               
     &    call update_forcing_data(ws_data_time, ws_data_time_min_loc
     &    , ws_interp_type, ws_data_next, ws_data_update
     &    , ws_data_type, ws_data_inc, ws_filename
     &    , ws_data_renorm, SMF_DATA, 2, ws_interp_order, ws_data_label)

        call interpolate_forcing(SMF(:,:,1), SMF_DATA(:,:,1,:)
     &      , ws_data_time ,ws_interp_type, ws_data_time_min_loc
     &      , ws_interp_order, ws_interp_freq, ws_interp_inc
     &      , ws_interp_next, ws_interp_last, nsteps_run)
        call interpolate_forcing(SMF(:,:,2), SMF_DATA(:,:,2,:)
     &      , ws_data_time ,ws_interp_type, ws_data_time_min_loc
     &      , ws_interp_order, ws_interp_freq, ws_interp_inc
     &      , ws_interp_next, ws_interp_last, nsteps_run)
        
      end select

      end subroutine set_ws
!***********************************************************************

!***********************************************************************

      end module forcing_ws

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
