!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module grid

!***********************************************************************
!
!     This module contains grid info and routines for setting up the
!     POP grid quantities.
!
!     CVS:$Id: grid.F,v 1.3.2.7 2001/10/25 17:38:31 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use global_reductions
      use stencils
      use io
      use exit_mod

      implicit none
      save


!-----------------------------------------------------------------------
!
!     geometric scalars
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind) ::
     &   area_u, area_t,        ! total ocean area of U,T cells
     &   volume_u, volume_t,    ! total ocean volume of U,T cells
     &   uarea_equator,         ! area of equatorial cell
     &   volgt_m_marg,          ! volume of marginal seas 
     &   areag_m_marg           ! area of marginal seas

      real (kind=dbl_kind),  dimension (km) :: 
     &   volgk_m_marg           ! volume of marginal seas at each dpth

      integer (kind=int_kind) ::
     &   jeq,                    ! j index of equatorial cell
     &   nocean_u, nocean_t,     ! num of ocean U,T points
     &   nsurface_u, nsurface_t, ! num of ocean U,T points at surface
     &   sfc_layer_type          ! choice for type of surface layer

      integer (kind=int_kind), parameter ::
     &   sfc_layer_varthick = 1  ! variable thickness surface layer
     &,  sfc_layer_rigid    = 2  ! rigid lid surface layer
     &,  sfc_layer_oldfree  = 3  ! old free surface form

      logical (kind=log_kind) ::
     &   topo_smooth,            ! flag to smooth topography
     &   flat_bottom,            ! flag for flat-bottom topography
     &   lremove_points           ! flag for removing isolated points

!-----------------------------------------------------------------------
!
!     geometric 1d serial arrays
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(1:km) :: 
     &   dz,                ! thickness of layer k
     &   c2dz,              ! 2*dz
     &   dzr, dz2r,         ! reciprocals of dz, c2dz
     &   zt,                ! vert dist from sfc to midpoint of layer
     &   zw,                ! vert dist from sfc to bottom of layer
     &   tarea_k


      real (kind=dbl_kind), dimension(0:km) :: 
     &   dzw, dzwr          ! midpoint of k to midpoint of k+1
                            !   and its reciprocal

!-----------------------------------------------------------------------
!
!     geometric 2d arrays
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   DXU, DYU,              ! {x,y} spacing centered at U points
     &   DXT, DYT,              ! {x,y} spacing centered at T points
     &   DXUR, DYUR,            ! reciprocals of DXU, DYU
     &   DXTR, DYTR,            ! reciprocals of DXT, DYT
     &   HTN, HTE,              ! cell widths on {N,E} sides of T cell
     &   HUS, HUW,              ! cell widths on {S,W} sides of U cell
     &   ULAT, ULONG,           ! {latitude,longitude} of U points
     &   TLAT, TLONG,           ! {latitude,longitude} of U points
     &   ANGLE,                 ! angle grid makes with latitude line
     &   ANGLET,                ! angle grid makes with latitude line
                                !  on the T grid
     &   FCOR, FCORT,           ! coriolis parameter at U,T points
     &   UAREA, TAREA,          ! area of U,T cells
     &   UAREA_R, TAREA_R,      ! reciprocal of area of U,T cells
     &   HT, HU, HUR            ! ocean depth at T,U points

!-----------------------------------------------------------------------
!
!     2d landmasks
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt) :: 
     &   KMT,             ! k index of deepest grid cell on T grid
     &   KMU,             ! k index of deepest grid cell on U grid
     &   KMTOLD           ! KMT field before smoothing

      logical (kind=log_kind), dimension(imt,jmt) :: 
     &   CALCT,           ! flag=true if point is an ocean point
     &   CALCU            !   at the surface

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &   RCALCT,          ! real equiv of CALCT,U to use as more
     &   RCALCU           !   efficient multiplicative mask

      integer (kind=int_kind), dimension(imt,jmt) :: 
     &   KMTN,KMTS,KMTE,KMTW,    ! KMT field at neighbor points 
     &   KMUN,KMUS,KMUE,KMUW     ! KMU field at neighbor points

      integer (kind=int_kind), dimension(:,:), allocatable :: 
     &   KMTEE,KMTNN,     ! KMT field 2 cells away for upwind stencil
     &   REGION_MASK      ! mask defining regions, marginal seas

!-----------------------------------------------------------------------
!
!     define f90 types used with region masks and marginal seas balancing
!
!-----------------------------------------------------------------------
      integer (kind=int_kind)
     &, parameter               ::
     &     max_regions =   15    ! max no. ocean regions
     &,    max_ms      =    7    ! max no. marginal seas
 
      integer (kind=int_kind)   ::
     &  num_regions
     &, num_ms
 
      type ms_bal
        real      (kind=dbl_kind) :: lat         ! transport latitude
        real      (kind=dbl_kind) :: lon         ! transport longitude
        real      (kind=dbl_kind) :: area        ! total distribution area
        real      (kind=dbl_kind) :: transport   ! total excess/deficit (E+P+M+R)
        integer   (kind=int_kind) :: mask_index  ! index of m-s balancing mask
      end type ms_bal
 
      type regions                           ! region-mask info
        integer   (kind=int_kind) :: number 
        character (char_len)      :: name          
        logical   (kind=log_kind) :: marginal_sea
        real      (kind=dbl_kind) :: area
        real      (kind=dbl_kind) :: volume
        type      (ms_bal)        :: ms_bal
      end type regions       
 
      type (regions)
     &, dimension(max_regions)    :: region_info



!-----------------------------------------------------------------------
!
!     area-weighted averaging coefficients
!     AT{0,S,W,SW} = {central,s,w,sw} coefficients for area-weighted
!       averaging of four U points surrounding a T point
!     AU{0,N,E,NE} = {central,n,e,ne} coefficients for area-weighted
!       averaging of four T points surrounding a U point
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension (imt,jmt) :: 
     &   AT0,ATS,ATW,ATSW
     &  ,AU0,AUN,AUE,AUNE

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_grid

!-----------------------------------------------------------------------
!
!     intializes all grid quantities
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      character (char_len) ::
     &  horiz_grid_opt,      ! horizontal grid option
     &  vert_grid_opt,       ! vertical grid option
     &  sfc_layer_opt,       ! choice for surface layer type
     &  topography_opt,      ! topography (KMT) option
     &  horiz_grid_file,     ! input file for reading horiz grid info
     &  vert_grid_file,      ! input file for reading horiz grid info
     &  topography_file,     ! input file for reading horiz grid info
     &  region_mask_filename,! input file for region mask
     &  region_info_filename ! input file with region identification

      namelist /grid_nml/ horiz_grid_opt, vert_grid_opt, topography_opt,
     &                 horiz_grid_file, vert_grid_file, topography_file,
     &                 topo_smooth, flat_bottom, lremove_points,
     &                 region_mask_filename, sfc_layer_opt,
     &                 region_info_filename   

      integer (kind=int_kind) ::
     &  k,                  ! vertical level index
     &  nml_error           ! namelist i/o error flag
      integer (kind=int_kind) :: i,j

      integer (kind=int_kind), dimension(imt,jmt) ::
     &  IWORK               ! local integer temp space

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  WORK                ! local temp space
      real (kind=dbl_kind) :: angle_0, angle_w, angle_s, angle_sw

      integer (kind=int_kind) ::
     &  range_count         ! counts number of points where ANGLE
                            !   is outside its expected range
      logical (kind=log_kind), dimension(imt,jmt) ::
     &  OUT_OF_RANGE
 
!-----------------------------------------------------------------------
!
!     read input namelist for grid setup options
!
!-----------------------------------------------------------------------

      horiz_grid_opt       = 'internal'
      vert_grid_opt        = 'internal'
      sfc_layer_opt        = 'varthick'
      topography_opt       = 'internal'
      horiz_grid_file      = 'unknown_horiz_grid_file'
      vert_grid_file       = 'unknown_vert_grid_file'
      topography_file      = 'unknown_topography_file'
      region_mask_filename = 'unknown_region_mask'
      region_info_filename = 'unknown_region_info'
      topo_smooth          = .false.
      flat_bottom          = .false.
      lremove_points       = .false.

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=grid_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading grid_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, grid_nml)
          write(stdout,*) ' '
      endif

      if (my_task == master_task) then
        write(stdout,1000)
        write(stdout,1001)
        write(stdout,1002)
        write(stdout,1001)
        write(stdout,1000)

 1000   format(72('-'))
 1001   format(' ')
 1002   format(' Grid options')

      endif

      call broadcast_scalar(horiz_grid_opt,       master_task)
      call broadcast_scalar(vert_grid_opt ,       master_task)
      call broadcast_scalar(sfc_layer_opt ,       master_task)
      call broadcast_scalar(topography_opt,       master_task)
      call broadcast_scalar(topo_smooth,          master_task)
      call broadcast_scalar(flat_bottom,          master_task)
      call broadcast_scalar(lremove_points,       master_task)
      call broadcast_scalar(region_mask_filename, master_task)
      call broadcast_scalar(region_info_filename, master_task)

!-----------------------------------------------------------------------
!
!     set up horizontal grid
!
!-----------------------------------------------------------------------

      select case (horiz_grid_opt)
      case ('internal')
        if (my_task == master_task) then
          write(stdout,*) ' Creating horizontal grid internally'
        endif
        call horiz_grid_internal
      case ('file')
        if (my_task == master_task) then
          write(stdout,*) ' Reading horizontal grid from file:',
     &                    trim(horiz_grid_file)
        endif
        call broadcast_scalar(horiz_grid_file, master_task)
        call read_horiz_grid(horiz_grid_file)
      case default
        call exit_POP('ERROR: unknown horizontal grid option')
      end select

      !***
      !*** construct fundamental b-grid cell widths at U points.
      !***

      call e_shift(WORK,HTN)
      DXU = p5*(HTN + WORK)
      call n_shift(WORK,HTE)
      DYU = p5*(HTE + WORK)

      DXUR = c1/DXU
      DYUR = c1/DYU

      UAREA = DXU*DYU
      UAREA_R = c1/UAREA

      !***
      !*** construct T-grid cell widths
      !***

      call s_shift(WORK,HTN)
      DXT = p5*(HTN + WORK)
      call w_shift(WORK,HTE)

      DYT = p5*(HTE + WORK)
      where (DXT == c0) DXT=c1
      where (DYT == c0) DYT=c1

      DXTR = c1/DXT
      DYTR = c1/DYT

      TAREA = DXT*DYT
      TAREA_R = c1/TAREA

      !***
      !*** compute min area of equatorial cell for use in variable_hmix
      !***

      WORK = abs(ULAT)
      uarea_equator = global_minval(WORK)

      where (WORK == uarea_equator)
        WORK = UAREA
      elsewhere
        WORK = 1.e+20_dbl_kind
      end where

      uarea_equator = global_minval(WORK)

!-----------------------------------------------------------------------
!
!     calculate lat/lon of T points
!
!-----------------------------------------------------------------------

      call calc_tpoints

!-----------------------------------------------------------------------
!
!     set up vertical grid
!
!-----------------------------------------------------------------------

      select case (vert_grid_opt)
      case ('internal')
        if (my_task == master_task) then
          write(stdout,*) ' Creating vertical grid internally'
        endif
        call vert_grid_internal
      case ('file')
        if (my_task == master_task) then
          write(stdout,*) ' Reading vertical grid from file:',
     &                    trim(vert_grid_file)
        endif
        call broadcast_scalar(vert_grid_file, master_task)
        call read_vert_grid(vert_grid_file)
      case default
        call exit_POP('ERROR: unknown vertical grid option')
      end select

      select case (trim(sfc_layer_opt))
      case ('varthick')  ! variable thickness sfc layer
        sfc_layer_type = sfc_layer_varthick
      case ('rigid')     ! rigid lid
        sfc_layer_type = sfc_layer_rigid
      case ('oldfree')   ! old free surface implementation
        sfc_layer_type = sfc_layer_oldfree
      case default
        call exit_POP('ERROR: unknown surface layer option')
      end select

      !***
      !*** calculate other vertical grid quantities
      !***

      dzw(0)  = p5*dz(1)
      dzw(km) = p5*dz(km)
      dzwr(0) = c1/dzw(0)
      zw(1) = dz(1)
      zt(1) = dzw(0)

      do k = 1,km-1
        dzw(k) = p5*(dz(k) + dz(k+1))
        zw(k+1) = zw(k) + dz(k+1)
        zt(k+1) = zt(k) + dzw(k)
      enddo

      do k = 1,km
        c2dz(k) = c2*dz(k)
        dzr(k)  = c1/dz(k)
        dz2r(k) = c1/c2dz(k)
        dzwr(k) = c1/dzw(k)
      enddo

      if (my_task == master_task) then
        write(stdout,*) ' Vertical grid:'
        write(stdout,1030)
        write(stdout,1035)
        do k=1,km
          write(stdout,1040) k,dz(k),zt(k)
        end do
      endif

 1030 format(3x,' k ',3x,'Thickness (cm)',3x,' Depth (cm) ')
 1035 format(3x,'---',3x,'--------------',3x,'------------')
 1040 format(3x,i3,4x,1pe12.5,4x,1pe12.5)

!-----------------------------------------------------------------------
!
!     set up topography
!
!-----------------------------------------------------------------------

      select case (topography_opt)
      case ('internal')
        call topography_internal
        flat_bottom = .true.
      case ('file')
        call broadcast_scalar(topography_file, master_task)
        call read_topography(topography_file)
      case default
        call exit_POP('ERROR: unknown topography option')
      end select

      !***
      !*** remove isolated lakes and disconnected points from grid
      !***

      if (lremove_points) call remove_points

      !***
      !*** smooth topography
      !***

      if (topo_smooth) call smooth_topography

      !***
      !*** flat bottom
      !***

      if (flat_bottom) where (KMT /= 0) KMT = km

      !***
      !*** calculate number of levels at U points (KMU field)
      !***

      call e_shift(IWORK,KMT)
      KMU = min(KMT,IWORK)
      call n_shift(IWORK,KMU)
      KMU = min(KMU,IWORK)  ! KMU = minimum of surrounding KMTs

      !***
      !*** calculate depth field at T,U points
      !***

      HT  = c0
      HU  = c0
      HUR = c0

      do k = 1,km
        where (k == KMT) HT = zw(k)
        where (k == KMU) 
          HU  = zw(k)
          HUR = c1/zw(k)
        endwhere
      enddo

!-----------------------------------------------------------------------
!
!     landmasks
!
!-----------------------------------------------------------------------

      call landmasks

!-----------------------------------------------------------------------
!
!     calculate area, volume, # surface points, # ocean points
!
!-----------------------------------------------------------------------

      area_t   = global_sum(TAREA,    RCALCT)
      area_u   = global_sum(UAREA,    RCALCU)
      WORK = TAREA*HT
      volume_t = global_sum(WORK, RCALCT)
      WORK = UAREA*HU
      volume_u = global_sum(WORK, RCALCU)
      tarea_k(1) = area_t
      do k=2,km
        WORK = merge(c1, c0, k <= KMT)
        tarea_k(k) = global_sum(TAREA, WORK)
      end do

      nsurface_t = global_count(RCALCT)
      nsurface_u = global_count(RCALCU)
      nocean_t = global_sum(KMT,ONE)
      nocean_u = global_sum(KMU,ONE)

      if (my_task == master_task) then
        write(stdout,*) ' '
        write(stdout,*) ' # surface (T,U) points', 
     &           nsurface_t, nsurface_u
        write(stdout,*) ' # ocean   (T,U) points', 
     &           nocean_t, nocean_u
        write(stdout,*) ' T-area,   U-area   (km**2)', 
     &           area_t*1.0e-10_dbl_kind, area_u*1.0e-10_dbl_kind
        write(stdout,*) ' T-volume,   U-volume   (km**3)', 
     &           volume_t*1.0e-15_dbl_kind, volume_u*1.0e-15_dbl_kind
      endif

!-----------------------------------------------------------------------
!
!     set region-masks and calculate active-ocean and marginal-seas
!       areas and volumes
!
!-----------------------------------------------------------------------

      if (trim(region_mask_filename) /= 'unknown_region_mask') then
        call area_masks(region_mask_filename, region_info_filename)
      endif

!-----------------------------------------------------------------------
!
!     calculate stencil coefficients for area-averaging
!
!-----------------------------------------------------------------------

      call cf_area_avg  ! coefficients for area-weighted averages

!-----------------------------------------------------------------------
!
!     calculate ANGLET from ANGLE, now that area-weighting coefficients
!       are available 
!
!-----------------------------------------------------------------------
 
      select case (horiz_grid_opt)
      case ('internal')
        ANGLET = c0
      case ('file')
!-----------------------------------------------------------------------
!      first, ensure that -pi <= ANGLE <= pi
!-----------------------------------------------------------------------

        OUT_OF_RANGE = .false.
       
        where (ANGLE < -pi .or.  ANGLE > pi) OUT_OF_RANGE = .true.
             ! ANGLE is outside its expected range
 
        range_count = global_count (OUT_OF_RANGE)
 
        if (range_count > 0) 
     &  call exit_POP('ERROR: ANGLE is outside its expected range')
 
!-----------------------------------------------------------------------
!     compute ANGLE on T-grid
!-----------------------------------------------------------------------
 
        do j=jphys_b, jphys_e
          do i=iphys_b, iphys_e

            angle_0  = ANGLE(i,  j  )
            angle_w  = ANGLE(i-1,j  )
            angle_s  = ANGLE(i,  j-1)
            angle_sw = ANGLE(i-1,j-1)
 
            if ( angle_0 < c0 ) then
              if ( abs(angle_w -angle_0) > pi ) 
     &          angle_w  = angle_w  - pi2 
              if ( abs(angle_s -angle_0) > pi ) 
     &          angle_s  = angle_s  - pi2 
              if ( abs(angle_sw-angle_0) > pi ) 
     &          angle_sw = angle_sw - pi2 
            endif

            ANGLET(i,j) =  angle_0  * AT0 (i,j)
     &                   + angle_w  * ATW (i,j)
     &                   + angle_s  * ATS (i,j) 
     &                   + angle_sw * ATSW(i,j)

          enddo
        enddo
 
        call boundary_2D(ANGLET)
 
!-----------------------------------------------------------------------
!     set ANGLET to zero for all of (global) j=1 row 
!     (bottom row of ANGLET isn't used, but it is written to netcdf file)
!-----------------------------------------------------------------------
        do j=1,jmt
          if (j_global(j) == 1) ANGLET(:,j) = c0
        enddo
 
      end select

!-----------------------------------------------------------------------
!
!     compute coriolis parameter 2*omega*sin(true_latitude)
!
!-----------------------------------------------------------------------

      FCOR = c2*omega*sin(ULAT)                ! at u-points
      call sw_4pt(WORK,AT0,ATS,ATW,ATSW,ULAT)
      FCORT = c2*omega*sin(WORK)               ! at t-points

!-----------------------------------------------------------------------

      end subroutine init_grid

!***********************************************************************

      subroutine horiz_grid_internal

!-----------------------------------------------------------------------
!
!     creates a lat/lon grid with equal spacing in each direction
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: i,j

      real (kind=dbl_kind) ::
     &  dlat, dlon,        ! lat/lon spacing for idealized grid
     &  xdeg               ! temporary longitude variable

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  WORK               ! local work space
 
!-----------------------------------------------------------------------
!
!     calculate lat/lon coords of U points
!     long range (-180,180)
!
!-----------------------------------------------------------------------

      dlon = 360.0_dbl_kind/real(imt_global)
      dlat = 180.0_dbl_kind/real(jmt_global)

      do i=1,imt
        xdeg = i_global(i)*dlon
        if (xdeg > 180.0_dbl_kind) xdeg = xdeg - 360.0_dbl_kind
        ULONG(i,:) =  xdeg/radian 
      enddo

      do j = 1,jmt
        ULAT(:,j)  = (-90.0_dbl_kind + j_global(j)*dlat)/radian
      enddo

!-----------------------------------------------------------------------
!
!     calculate grid lengths
!
!-----------------------------------------------------------------------
      
      HTN   = dlon*radius/radian  ! convert to cm
      HTE   = dlat*radius/radian  ! convert to cm
      ANGLE = c0

      call n_shift(WORK,HTE)
      HUW = p5*(HTE + WORK)
      call e_shift(WORK,HTN)
      HUS = p5*(HTN + WORK)

      HTN = HTN*cos(ULAT)
      call s_shift(WORK,ULAT)
      HUS = HUS*cos(p5*(ULAT + WORK))

!-----------------------------------------------------------------------

      end subroutine horiz_grid_internal

!***********************************************************************

      subroutine read_horiz_grid(horiz_grid_file)

!-----------------------------------------------------------------------
!
!     read horizontal grid information from input grid file
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      character (char_len), intent(in) ::
     &  horiz_grid_file     ! filename of file containing grid data

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  nu                    ! i/o unit number


!-----------------------------------------------------------------------

      call open_parallel_file(nu,horiz_grid_file,recl_dbl)

      call read_array(nu,ULAT)
      call read_array(nu,ULONG)
      call read_array(nu,HTN)
      call read_array(nu,HTE)
      call read_array(nu,HUS)
      call read_array(nu,HUW)
      call read_array(nu,ANGLE)

      call close_parallel_file(nu)

 
      if (my_task == master_task) then
        write(stdout,*) ' '
        write(stdout,*) ' file read: ', horiz_grid_file
      endif

      where (HTN <= c0) HTN = c1
      where (HTE <= c0) HTE = c1
      where (HUS <= c0) HUS = c1
      where (HUW <= c0) HUW = c1

!-----------------------------------------------------------------------

      end subroutine read_horiz_grid

!***********************************************************************

      subroutine vert_grid_internal

!-----------------------------------------------------------------------
!
!     creates vertical grid layer thicknesses based on km
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     parameters for depth function choice
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), parameter ::
     &  zmax    = 5500.0_dbl_kind,   ! max depth in meters
     &  dz_sfc  =   25.0_dbl_kind,   ! thickness of sfc layer (meters)
     &  dz_deep =  400.0_dbl_kind    ! thick of deep ocn layers (meters)

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind) ::
     &  depth,              ! depth based on integrated thicknesses
     &  zlength,            ! adjustable parameter for thickness
     &  d0, d1,             ! depths used by midpoint search
     &  zl0, zl1,           ! parameter used by midpoint search
     &  dzl                 ! zl1-zl0

      integer (kind=int_kind) :: k  ! dummy vertical index

!-----------------------------------------------------------------------
!
!     initialize bisection search to find best value of zlength
!     parameter such that integrated depth = zmax
!
!-----------------------------------------------------------------------

      zl0 = eps
      zl1 = zmax
      dzl = zl1 - zl0

      call compute_dz(d0,zl0,dz_sfc,dz_deep)
      call compute_dz(d1,zl1,dz_sfc,dz_deep)

      if ((d0-zmax)*(d1-zmax) > c0) then
        if (my_task == master_task) write(stdout,*) d0,d1,zmax
        call exit_POP('vert_grid: zero point not in initial interval')
      endif

!-----------------------------------------------------------------------
!
!     do bisection search
!
!-----------------------------------------------------------------------

      do while ( (dzl/zmax) > eps)

        !***
        !*** compute profile at midpoint
        !***

        zlength = zl0 + p5*dzl

        call compute_dz(depth,zlength,dz_sfc,dz_deep)

        !***
        !*** find interval to use for continuing search
        !***

        if ((d0-zmax)*(depth-zmax) < c0) then
          d1 = depth
          zl1 = zlength
        else if ((d1-zmax)*(depth-zmax) < c0) then
          d0 = depth
          zl0 = zlength
        else
          if (my_task == master_task) write(stdout,*) d0,d1,depth,zmax
          call exit_POP('vert_grid: zero point not in interval')
        endif

        dzl = zl1 - zl0

      end do

      dz = dz*cmperm   ! convert to cm

!-----------------------------------------------------------------------
!
!     presumably, we have converged, but check to make sure
!
!-----------------------------------------------------------------------

      if (abs(depth-zmax)/zmax > 0.01_dbl_kind) then
        if (my_task == master_task) then
          write(stdout,*) 'Integrated depth = ',depth,'   zmax = ',zmax
        endif
        call exit_POP('Unable to compute vertical grid internally')
      endif

!-----------------------------------------------------------------------

      end subroutine vert_grid_internal

!***********************************************************************

      subroutine compute_dz(depth,zlength,dz_sfc,dz_deep)

!-----------------------------------------------------------------------
!
!     computes a thickness profile and total depth given the 
!     parameters for the thickness function
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), intent(in) ::
     &  zlength,           ! gaussian parameter for thickness func
     &  dz_sfc,            ! thickness of surface layer
     &  dz_deep            ! thickness of deep ocean layers

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), intent(out) ::
     &  depth               ! depth based on integrated thicknesses

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: k

!-----------------------------------------------------------------------

      depth = c0

      do k=1,km

        dz(k) = dz_deep - (dz_deep - dz_sfc)*exp(-(depth/zlength)**2)
        depth = depth + dz(k)

      end do

!-----------------------------------------------------------------------

      end subroutine compute_dz

!***********************************************************************

      subroutine read_vert_grid(vert_grid_file)

!-----------------------------------------------------------------------
!
!     reads in layer thicknesses from grid input file
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      character (char_len), intent(in) ::
     &  vert_grid_file

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  k,                    ! vertical level index
     &  nu                    ! i/o unit number

!-----------------------------------------------------------------------
!
!     read vertical layer thickness from file
!
!-----------------------------------------------------------------------

      call get_unit(nu)
      if (my_task == master_task) then 
        open(nu,file=vert_grid_file,status='old',form='formatted')
        do k = 1,km
          read(nu,*) dz(k)
        enddo
        close(nu)
      endif
      call release_unit(nu)

      call broadcast_vector(dz,master_task,km)

!-----------------------------------------------------------------------

      end subroutine read_vert_grid

!***********************************************************************

      subroutine topography_internal

!-----------------------------------------------------------------------
!
!     generates simple KMT field with idealized land masses
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: i,j,k

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  TLATD, TLOND        ! lat/lon of T points in degrees
!-----------------------------------------------------------------------

      TLATD = TLAT*radian
      TLOND = TLONG*radian

      KMT = km  ! flat bottom

      do j=1,jmt
        if (j_global(j) <= 1 .or. j_global(j) >= jmt_global) 
     &    KMT(:,j) = 0
      end do

      where ((TLATD  > -35.0_dbl_kind)
     &  .and.(TLOND > 210.0_dbl_kind)
     &  .and.(TLOND < 250.0_dbl_kind)) KMT = 0

      where ((TLATD >  25.0_dbl_kind)
     &  .and.(TLOND > 210.0_dbl_kind)
     &  .and.(TLOND < 330.0_dbl_kind)) KMT = 0

      where ((TLATD >  60.0_dbl_kind)
     &  .and.(TLOND > 210.0_dbl_kind)
     &  .and.(TLOND < 150.0_dbl_kind)) KMT = 0

      where ((TLATD > -60.0_dbl_kind)
     &  .and.(TLOND > 110.0_dbl_kind)
     &  .and.(TLOND < 150.0_dbl_kind)) KMT = 0

      where (abs(TLATD) > 75.0_dbl_kind) KMT = 0 

!-----------------------------------------------------------------------

      end subroutine topography_internal

!***********************************************************************

      subroutine read_topography(topography_file)

!-----------------------------------------------------------------------
!
!     read in a KMT field from file
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      character (char_len), intent(in) ::
     &  topography_file     ! input file containing KMT field

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: nu  ! i/o unit number

!-----------------------------------------------------------------------
!
!     read KMT field from file
!
!-----------------------------------------------------------------------

      call open_parallel_file(nu,topography_file,recl_int)
      call read_array(nu,KMT)
      call close_parallel_file(nu)

      if (my_task == master_task) then
        write(stdout,*) ' '
        write(stdout,*) ' file read: ', topography_file
      endif

!-----------------------------------------------------------------------

      end subroutine read_topography

!***********************************************************************

      subroutine remove_points

!-----------------------------------------------------------------------
!
!     remove isolated points from grid (KMT field)   
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  npoints_removed        ! number of points removed

      integer (kind=int_kind), dimension(imt,jmt) :: 
     &  IWORK1,IWORK2          ! integer work space

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  RMASK                  ! temporary real mask

!-----------------------------------------------------------------------
!
!     calculate number of levels at U points
!     (KMU field before modifying KMT field)
!
!-----------------------------------------------------------------------

      call e_shift(IWORK1,KMT)
      KMU = min(KMT,IWORK1)
      call n_shift(IWORK1,KMU)
      KMU = min(KMU,IWORK1)  ! KMU = minimum of surrounding KMTs

!-----------------------------------------------------------------------
!
!     remove disconnected points from grid.
!     if all KMUs surrounding a T point are zero, set KMT = 0.
!
!-----------------------------------------------------------------------

      call w_shift(IWORK1,KMU)
      IWORK1 = KMU  + IWORK1
      call s_shift(IWORK2,IWORK1)
      IWORK1 = IWORK2 + IWORK1  ! sum of surounding KMUs

      where ((IWORK1 == 0) .and. (KMT > 0)) 
        RMASK = c1
      elsewhere
        RMASK = c0
      end where

      npoints_removed = global_count(RMASK)

      if (my_task == master_task) then
        write(stdout,*) ' '
        write(stdout,*) ' points removed from grid:', npoints_removed
      endif

      where (IWORK1 == 0) KMT = 0

!-----------------------------------------------------------------------

      end subroutine remove_points

!***********************************************************************

      subroutine smooth_topography

!-----------------------------------------------------------------------
!
!     this routine smooths topography to create new KMT, depth fields
!
!   * the depth field HT is constructed from the KMT field and
!     depth profile dz, HT is smoothed by by application of a
!     9-point averaging stencil:
!
!              1 -- 2 -- 1
!              |    |    |
!              2 -- 4 -- 2
!              |    |    |
!              1 -- 2 -- 1
!
!     land points are not included in the smoothing, and the
!     stencil is modified to include only ocean points in the
!     averaging.  once the depth field has been smoothed,
!     a new KMT field is constructed from it. 
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     local variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: k     ! dummy vertical level index

      integer (kind=int_kind), dimension(imt,jmt) :: 
     &  NB,                    ! num points contributing to 9pt avg
     &  IWORK                  ! local work space

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  HTNEW,              ! smoothed depth field at T points
     &  WORK                ! local work space

!-----------------------------------------------------------------------
!
!     old depth field in T columns
!
!-----------------------------------------------------------------------

      HT = c0
      do k = 1,km 
        where (k == KMT) HT = zw(k)
      enddo

!-----------------------------------------------------------------------
!
!     smooth topography
!
!-----------------------------------------------------------------------

      where (KMT > 0) 
        NB = 1
        HTNEW = HT
      elsewhere
        NB = 0
        HTNEW = c0
      endwhere

      call e_shift(WORK,HTNEW)
      HTNEW = HTNEW + WORK
      call w_shift(WORK,HTNEW)
      HTNEW = HTNEW + WORK
      call n_shift(WORK,HTNEW)
      HTNEW = HTNEW + WORK
      call s_shift(WORK,HTNEW)
      HTNEW = HTNEW + WORK

      call e_shift(IWORK,NB)
      NB = NB + IWORK
      call w_shift(IWORK,NB)
      NB = NB + IWORK
      call n_shift(IWORK,NB)
      NB = NB + IWORK
      call s_shift(IWORK,NB)
      NB = NB + IWORK

!-----------------------------------------------------------------------
!
!     new depth field
!
!-----------------------------------------------------------------------

      where ((KMT /= 0) .and. (NB /= 0)) 
        HTNEW = HTNEW/float(NB)
      elsewhere
        HTNEW = c0
      endwhere

!-----------------------------------------------------------------------
!
!     new KMT field
!
!-----------------------------------------------------------------------

      KMTOLD = KMT
      do k = 1,km-1
        where (HTNEW > zt(k) .and. HTNEW <= zt(k+1)) KMT = k
      enddo
      where (HTNEW > zt(k)) KMT = km

!-----------------------------------------------------------------------

      end subroutine smooth_topography

!***********************************************************************

      subroutine landmasks

!-----------------------------------------------------------------------
!
!     calculate additional masks for land points at each depth level.
!     these include real masks for applying multiplicative masks
!     instead of logical masks and also KMT arrays for neighbor points
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     masks for surface ocean T or U points
!
!-----------------------------------------------------------------------

      where (KMT >= 1) 
        CALCT = .true.
        RCALCT = c1
      elsewhere
        CALCT = .false.
        RCALCT = c0
      endwhere

      where (KMU >= 1) 
        CALCU = .true.
        RCALCU = c1
      elsewhere
        CALCU = .false.
        RCALCU = c0
      endwhere

!-----------------------------------------------------------------------
!
!     depth level fields (KMT,KMU) to north,south,east,west
!
!-----------------------------------------------------------------------

      call n_shift(KMTN,KMT)
      call s_shift(KMTS,KMT)
      call e_shift(KMTE,KMT)
      call w_shift(KMTW,KMT)

      call n_shift(KMUN,KMU)
      call s_shift(KMUS,KMU)
      call e_shift(KMUE,KMU)
      call w_shift(KMUW,KMU)

!-----------------------------------------------------------------------

      end subroutine landmasks

!***********************************************************************

      subroutine area_masks(mask_filename, info_filename)

!-----------------------------------------------------------------------
!
!     this subroutine reads in file with regional area mask and
!     marginal seas defined
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variable
!
!-----------------------------------------------------------------------

      character(*), intent(in) ::
     &  mask_filename          ! name of file containing region masks
     &, info_filename          ! name of file containing region names

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  k, n, nu
     &, region

      real (kind=dbl_kind) :: volume_marg_k, volume_marg, area_marg

      real (kind=dbl_kind), allocatable, dimension(:,:) ::
     &   WORK



      if (my_task == master_task) then
        write(stdout,1000)
        write(stdout,1001)
        write(stdout,1002)
        write(stdout,1001)
        write(stdout,1000)
 1000   format(72('-'))
 1001   format(' ')
 1002   format(' Region mask information')
      endif


!-----------------------------------------------------------------------
!
!     read in regional area masks, including marginal seas.  then
!     calculate related variables.
!
!-----------------------------------------------------------------------

      allocate(REGION_MASK(imt,jmt))
      call open_parallel_file(nu, mask_filename, recl_int)
      call read_array(nu,REGION_MASK)
      call close_parallel_file(nu)
      if(my_task == master_task) write(stdout,*)' read file: '
     &  , mask_filename

      num_regions = global_maxval(abs(REGION_MASK))
 
      if (num_regions > max_regions ) then
        call exit_POP ('ERROR: num_regions > max_regions')
      endif
 
!---------------------------------------------------------------------
!
!       open and read file which contains region names and marginal-
!         seas balancing information
!
!---------------------------------------------------------------------

      if(info_filename == 'unknown_region_info') then
        call exit_POP ('ERROR: unknown region_info_filename')
      endif
 
      region_info(:)%name   = 'unknown_region_name'
      region_info(:)%area   = c0
      region_info(:)%volume = c0
 
      call get_unit(nu)
 
      if (my_task == master_task) then
 
        open(nu,file=info_filename,form='formatted'
     &,      status='unknown')
 
 
        do n = 1,num_regions
          read (nu,*) region_info(n)%number
     &,               region_info(n)%name
     &,               region_info(n)%ms_bal%lat
     &,               region_info(n)%ms_bal%lon
     &,               region_info(n)%ms_bal%area
        enddo
 
        close(nu)
 
      endif 
 
      call release_unit(nu)
 
 
      do n = 1,num_regions
        call broadcast_scalar(region_info(n)%name       ,master_task)
        call broadcast_scalar(region_info(n)%number     ,master_task)
        call broadcast_scalar(region_info(n)%ms_bal%lat ,master_task)
        call broadcast_scalar(region_info(n)%ms_bal%lon ,master_task)
        call broadcast_scalar(region_info(n)%ms_bal%area,master_task)
      enddo
 
 
!---------------------------------------------------------------------
!
!       determine if region is a marginal sea
!
!---------------------------------------------------------------------
      num_ms = 0
      do n = 1,num_regions
        if (region_info(n)%number < 0 )  then
            num_ms = num_ms + 1
            region_info(n)%marginal_sea = .true.
        else
            region_info(n)%marginal_sea = .false.
        endif
      enddo
 
      if (num_ms > max_ms) then
          if (my_task == master_task) then
          write(stdout,*)'area_masks: maximum number of marginal '
     &                  //' seas exceeded'   
          endif
          call exit_POP('ERROR: must increase max_ms in module grid.F')
       endif


!-----------------------------------------------------------------------
!
!     set initial values of volgk_m_marg(k), areag_m_marg, and
!     volgt_m_marg to be appropriate values for the entire ocean
!     including marginal seas.
!
!-----------------------------------------------------------------------

      allocate(WORK(imt,jmt))

      do k = 1, km
        WORK = merge(TAREA*dz(k), c0, k <= KMT)
        volgk_m_marg(k) = global_sum(WORK, ONE)
      enddo

      areag_m_marg = area_t
      volgt_m_marg = volume_t

!-----------------------------------------------------------------------
!
!     a negative value of REGION_MASK designates a marginal sea.  if
!     a region is a marginal sea, calculate the area and volume
!     and decrement values from global values.
!
!-----------------------------------------------------------------------


      do region = 1, num_regions

        WORK = merge(TAREA, c0, 1 <= KMT .and. REGION_MASK == -region)
        area_marg    = global_sum(WORK,ONE)
        areag_m_marg = areag_m_marg - area_marg

        volume_marg = c0
        do k = 1, km
          WORK = merge(TAREA*dz(k), c0, 
     &             k <= KMT .and. REGION_MASK == -region)
          volume_marg_k   = global_sum(WORK,ONE)
          volume_marg     = volume_marg + volume_marg_k
          volgk_m_marg(k) = volgk_m_marg(k) - volume_marg_k
          if(volgk_m_marg(k) == c0) volgk_m_marg(k) = 1.e20_dbl_kind
          volgt_m_marg    = volgt_m_marg - volume_marg_k
        enddo
          
        if(area_marg /= c0) then
          if (.not. region_info(region)%marginal_sea) then
            call exit_POP ('ERROR: marginal-sea mismatch')
          endif
          region_info(region)%area   = area_marg
          region_info(region)%volume = volume_marg
        endif

      enddo

      deallocate(WORK)

!---------------------------------------------------------------------
!
!       document regional information
!
!---------------------------------------------------------------------
 
      if (my_task == master_task) then
        write(stdout,1001)
        write(stdout,1003)
 
        do n = 1,num_regions
         if (region_info(n)%marginal_sea) then
           write(stdout,1004) region_info(n)%number
     &,                       trim(region_info(n)%name)
     &,                       region_info(n)%area  *1.0e-10_dbl_kind
     &,                       region_info(n)%volume*1.0e-15_dbl_kind
         else
           write(stdout,1004) region_info(n)%number
     &,                       trim(region_info(n)%name)
         endif
        enddo
 
 
        write(stdout,1001)
        write(stdout,1005)
 
 
        do n = 1,num_regions
         if (region_info(n)%marginal_sea) then
           write(stdout,1006) region_info(n)%number
     &,                       trim(region_info(n)%name)
     &,                       region_info(n)%ms_bal%lat
     &,                       region_info(n)%ms_bal%lon 
     &,                       region_info(n)%ms_bal%area
         endif
        enddo
 
      endif 
 
      if (my_task == master_task) then
        write(stdout,1001)
      endif
 


!-----------------------------------------------------------------------
!
!     convert cgs to mks
!
!-----------------------------------------------------------------------

      areag_m_marg = areag_m_marg*1.0e-4_dbl_kind  ! cm^2 -> m^2
      volgt_m_marg = volgt_m_marg*1.0e-6_dbl_kind  ! cm^3 -> m^3
      volgk_m_marg = volgk_m_marg*1.0e-6_dbl_kind  ! cm^3 -> m^3

      do n = 1,num_regions
        if (region_info(n)%marginal_sea) then
            region_info(n)%area  =region_info(n)%area  *1.0e-4_dbl_kind
            region_info(n)%volume=region_info(n)%volume*1.0e-6_dbl_kind
        endif
      enddo


1003  format (  30x, '+', 23('-'),'+'
     &,       /,30x, '|',  2x,  'Marginal Seas Only   |'
     &,       /,2x,'Region', 8x, 'Region ',   7x, '|',  2x,'Area'     
     &,       8x,'Volume   |'
     &,       /,2x,'Number', 8x,  'Name',10x, '| (km^2)', 7x, '(km^3)'
     &,       3x, '|'
     &,       /, 2x,6('-'), 1x,19('-'),2x,11('-'),2x
     &,       11('-') )
1004  format (2x, i4, a22, 2(1pe13.5) )
1005  format (/,3x, ' Marginal Sea (E+P+M+R) Balancing Information'
     &,       /,2x,'Region', 8x, 'Region ',  8x, 'Bal.'
     &,       4x,'Bal.',  4x, 'Search'
     &,       /,2x,'Number', 8x,  'Name',11x, 'Lat'  , 5x, 'Lon'
     &,       5x, 'Area'
     &,       /,47x, '(cm^2)'
     &,       /, 2x,6('-'), 1x,19('-'),2x, 6('-'),2x
     &,        6('-'), 2x, 11('-')  )
1006  format (1x, i4, a20, 3x,2(0pf8.2),   1pe13.5  )

         
!-----------------------------------------------------------------------

      end subroutine area_masks

!***********************************************************************

      subroutine cf_area_avg

!-----------------------------------------------------------------------
!
!     calculate the coefficients of the 4-point stencils for 
!     area-weighted averaging at T and U points
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     calculate {central,s,w,sw} coefficients for area-averaging
!     to T points.
!
!-----------------------------------------------------------------------

!     AT0  = UAREA
!     call s_shift(ATS,UAREA)
!     call w_shift(ATW,UAREA)
!     call s_shift(ATSW,ATW)

!     AT0  = AT0 *p25*TAREA_R
!     ATS  = ATS *p25*TAREA_R
!     ATW  = ATW *p25*TAREA_R
!     ATSW = ATSW*p25*TAREA_R

      AT0  = p25
      ATS  = p25
      ATW  = p25
      ATSW = p25

!-----------------------------------------------------------------------
!
!     calculate {central,n,e,ne} coefficients for area-averaging
!      to U points.
!
!-----------------------------------------------------------------------

      AU0  = TAREA
      call n_shift(AUN,TAREA)
      call e_shift(AUE,TAREA)
      call n_shift(AUNE,AUE)

      AU0  = AU0 *p25*UAREA_R
      AUN  = AUN *p25*UAREA_R
      AUE  = AUE *p25*UAREA_R
      AUNE = AUNE*p25*UAREA_R

!-----------------------------------------------------------------------

      end subroutine cf_area_avg

!***********************************************************************

      subroutine calc_tpoints

!-----------------------------------------------------------------------
!
!     calculates lat/lon coordinates of T points from U points.
!     this computation is identical to the method used in the ccsm
!     ice model subroutine Tlatlon in module ice_gird.F as of
!     14 September 2001
!
!-----------------------------------------------------------------------



      integer (kind=int_kind) ::
     &   i, j                          ! generic indices
     &,  im1
 
      real (kind=dbl_kind) ::
     &     z1,x1,y1,z2,x2,y2,z3,x3,y3,z4,x4,y4,tx,ty,tz,da
     &,    ULAT_G (imt_global,jmt_global)  
     &,    ULONG_G(imt_global,jmt_global)  
     &,    TLAT_G (imt_global,jmt_global)  
     &,    TLONG_G(imt_global,jmt_global)  

      call gather_global(ULONG_G,ULONG,master_task)
      call gather_global(ULAT_G, ULAT ,master_task)

      if (my_task == master_task) then

      do j=2,jmt_global
        do i=1,imt_global

            if (i.eq.1) then
               im1=imt_global
            else
               im1=i-1
            endif

            z1 = cos(ULAT_G(im1,j-1))
            x1 = cos(ULONG_G(im1,j-1))*z1
            y1 = sin(ULONG_G(im1,j-1))*z1
            z1 = sin(ULAT_G(im1,j-1))

            z2 = cos(ULAT_G(i,j-1))
            x2 = cos(ULONG_G(i,j-1))*z2
            y2 = sin(ULONG_G(i,j-1))*z2
            z2 = sin(ULAT_G(i,j-1))

            z3 = cos(ULAT_G(im1,j))
            x3 = cos(ULONG_G(im1,j))*z3
            y3 = sin(ULONG_G(im1,j))*z3
            z3 = sin(ULAT_G(im1,j))

            z4 = cos(ULAT_G(i,j))
            x4 = cos(ULONG_G(i,j))*z4
            y4 = sin(ULONG_G(i,j))*z4
            z4 = sin(ULAT_G(i,j))

            tx = (x1+x2+x3+x4)/c4
            ty = (y1+y2+y3+y4)/c4
            tz = (z1+z2+z3+z4)/c4
            da = sqrt(tx**2+ty**2+tz**2)

            tz = tz/da
            ! TLONG_G in radians
            TLONG_G(i,j) = c0
            if (tx.ne.c0.or.ty.ne.c0) 
     &           TLONG_G(i,j) = atan2(ty,tx)
            ! TLAT_G in radians
            TLAT_G(i,j) = asin(tz)

        end do
      end do

      ! j=1: linear approximation
      do i=1,imt_global
         TLONG_G(i,1) = TLONG_G(i,2)
         TLAT_G (i,1) = c2*TLAT_G(i,2) - TLAT_G(i,3)
      end do

      endif ! master_task

      call scatter_global(TLONG, TLONG_G, master_task)
      call scatter_global(TLAT , TLAT_G , master_task)

      where (TLONG > pi2) TLONG = TLONG - pi2
      where (TLONG < c0 ) TLONG = TLONG + pi2
 
!-----------------------------------------------------------------------

      end subroutine calc_tpoints

!***********************************************************************

      subroutine fill_points(k,F)

!-----------------------------------------------------------------------
!
!     if the depth-level field KMT has been smoothed, fill in
!     values of a 2-d field at new ocean points that have appeared
!     due to the smoothing using averaged values from nearby points. 
!
!   * given the smoothed KMT field and the original unsmoothed field
!     KMTOLD, values at new points of the field F (defined at level k)
!     are filled in with averaged values of F defined at nearby points.
!     the averaging is done using a 9-point averaging stencil like
!     the one used in 'smooth_topography':
!
!     9-point averaging stencil:
!
!              1 -- 2 -- 1
!              |    |    |
!              2 -- 4 -- 2
!              |    |    |
!              1 -- 2 -- 1
!
!     the stencil is modified to include only old ocean points in the
!     averaging. 
!
!   * since only those new points which have neighboring old points
!     will be filled in, this procedure must be applied several times
!     until all points are filled in. 
!
!   * if an area weighted average is desired, multiply the field F
!     by the cell area before calling the routine, then divide by
!     the cell area after the field is returned.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  k                      ! depth level at which field is defined
                               ! (k = 1 for sfc or vert-avg fields)

!-----------------------------------------------------------------------
!
!     input/output:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(inout) ::
     &  F             ! input as field well-defined at old ocean points
                      ! output as field with new points filled in

!-----------------------------------------------------------------------
!
!     local variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  i,j,                  ! dummy indices
     &  npass,                ! num of passes thru grid filling points
     &  nfill_points          ! num of points left to be filled

      logical (kind=log_kind), dimension(imt,jmt) ::
     &  MASKOLD,              ! true at old ocean points
     &  MASKNEW               ! true at new points not in old ocn field

      integer (kind=int_kind), dimension(imt,jmt) :: 
     &  NB,                   ! num points contributing to 9pt avg
     &  IWORK                 ! local work space

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  FNEW,                 ! smoothed F field after npass passes
     &  WORK                  ! local work space

!-----------------------------------------------------------------------
!
!     initialize land mask and pass counter
!
!-----------------------------------------------------------------------

      MASKOLD = (k <= KMTOLD)
      npass = 0
      nfill_points = 100

!-----------------------------------------------------------------------
!
!     pass through grid until all points are filled
!
!-----------------------------------------------------------------------

      do while (nfill_points /= 0)

        npass = npass + 1
        if (npass > imt)  then
          call exit_POP('error (fill_points): too many passes')
        endif

!-----------------------------------------------------------------------
!
!       find new ocean points not in old ocean field
!
!-----------------------------------------------------------------------

        MASKNEW = (k <= KMT) .and. (.not. MASKOLD)

!-----------------------------------------------------------------------
!
!       find smoothed field: fill all points in this field with
!       the (9-point) average of the surrounding neighboring points,
!       including in the average only old ocean points. 
!
!-----------------------------------------------------------------------

        where (MASKOLD) 
          FNEW = F
          NB = 1
        elsewhere
          NB = 0
          FNEW = c0
        endwhere

        call e_shift(WORK,FNEW)
        FNEW = FNEW + WORK
        call w_shift(WORK,FNEW)
        FNEW = FNEW + WORK
        call n_shift(WORK,FNEW)
        FNEW = FNEW + WORK
        call s_shift(WORK,FNEW)
        FNEW = FNEW + WORK

        call e_shift(IWORK,NB)
        NB = NB + IWORK
        call w_shift(IWORK,NB)
        NB = NB + IWORK
        call n_shift(IWORK,NB)
        NB = NB + IWORK
        call s_shift(IWORK,NB)
        NB = NB + IWORK

!-----------------------------------------------------------------------
!
!       fill in F at new ocean points with the smoothed field,
!       only fill in points which have neighboring old ocean points.
!
!-----------------------------------------------------------------------

        where (MASKNEW .and. (NB /= 0))
          F = FNEW/float(NB)
          MASKOLD = .true.  ! update for next pass
        endwhere

!-----------------------------------------------------------------------
!
!       return for another pass if more points remain to be filled.
!
!-----------------------------------------------------------------------

        nfill_points = global_count((MASKNEW .and. (NB == 0)))
      enddo

!-----------------------------------------------------------------------

      end subroutine fill_points

!***********************************************************************

      end module grid

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
