!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module hmix_aniso

!***********************************************************************
!
!     this module contains routines for computing horizontal friction
!     terms from the divergence of a stress derived from the 
!     rate-of-strain tensor; the stress is anisotropic in general,
!     although the viscous coefficients may be chosen to make the
!     stress isotropic.
!
!   CVS:$Id: hmix_aniso.F,v 1.2.2.9 2002/04/25 22:45:02 njn01 Exp $
!   CVS:$Name: ccsm2_0_beta46 $
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use io
      use operators
      use diagnostics
      use grid
      use exit_mod

      implicit none
      save

!-----------------------------------------------------------------------
!
!     anisotropic viscosity variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), parameter :: ! choices for alignment type
     &  hmix_alignment_type_flow = 1,
     &  hmix_alignment_type_east = 2,
     &  hmix_alignment_type_grid = 3

      integer (kind=int_kind) ::
     &  hmix_alignment_itype,  ! integer choice for alignment type
     &  vconst_5

      logical (kind=log_kind) ::
     &  lvariable_hmix_aniso,  ! spatially varying mixing coeffs
     &  lsmag_aniso            ! smagorinski nonlinear viscosity

      real (kind=dbl_kind) ::
     &  visc_para,     ! viscosity parallel      to alignment direction
     &  visc_perp,     ! viscosity perpendicular to alignment direction
     &  vconst_1,      ! spatially varying viscosity params
     &  vconst_2,            
     &  vconst_3,            
     &  vconst_4,
     &  vconst_6        
 

      !*** the following variables are used only with smag viscosity

      real (kind=dbl_kind) ::
     &  c_para,      ! dimensionless smag coefficient for
     &  c_perp,      !    parallel and perpendicular directions
     &  u_para,      ! velocities for grid reynolds number viscous 
     &  u_perp       !    limit in parallel and perpendicular directions

      real (kind=dbl_kind), dimension (:,:), allocatable :: 
     &   K1E,K1W,K2N,K2S,         ! metric factors
     &   H1E,H1W,H2N,H2S,         ! grid spacings
     &   AMAX_CFL,                ! 1/2 maximum cfl-allowed viscosity
     &   DSMIN,                   ! min(DXU, DYU)
     &   F_PARA_SMAG, F_PERP_SMAG ! horizontal variations of the
                                  ! smag coefficients

!-----------------------------------------------------------------------
!
!     F_PARA and F_PERP represent the actual anisotropic viscosities 
!     used when lvariable_hmix_aniso is true. For lsmag_aniso option,
!     they contain the time-invariant "numerical" parts of the
!     anisotropic viscosity coefficients.
!
!-----------------------------------------------------------------------
 
      real (kind=dbl_kind), dimension(:,:,:),allocatable ::  
     &   F_PARA,                  ! spatial dependence of viscosity
                                  ! parallel to alignment direction
     &   F_PERP                   ! spatial dependence of viscosity
                                  ! perpendicular to alignment direction

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_aniso

!-----------------------------------------------------------------------
!
!     initializes various choices and allocates necessary space
!     also computes some time-independent arrays
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     local variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: nml_error  ! error flag for namelist
     &, nu                                  ! io unit number
     &, k                                   ! do-loop index

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  WORKA,WORKB                         ! local work arrays

      real (kind=dbl_kind) ::
     &  f_para_min, f_para_max,
     &  f_perp_min, f_perp_max

      character (char_len) ::
     &  hmix_alignment_choice  ! character choice for direction which 
                               ! breaks isotropy
     &, horz_viscosity_outfile ! root filename for anisotropic horizontal
                               ! viscosity output
    

!-----------------------------------------------------------------------
!
!     input namelist for setting various anisotropic viscosity options
!
!-----------------------------------------------------------------------

      namelist /hmix_aniso_nml/ 
     &  hmix_alignment_choice, lvariable_hmix_aniso, lsmag_aniso,
     &  visc_para, visc_perp, c_para, c_perp, u_para, u_perp,
     &  vconst_1,vconst_2,vconst_3,vconst_4,
     &  vconst_5, vconst_6, horz_viscosity_outfile

!-----------------------------------------------------------------------
!
!     read input namelist for anisotropic viscosity
!
!     DEFAULT VALUES
!     hmix_alignment_choice     : flow-aligned
!     lvariable_hmix_aniso      : false
!     lsmag_aniso               : false
!     visc_para                 : zero
!     visc_perp                 : zero
!     c_para                    : zero
!     c_perp                    : zero
!     u_para                    : zero
!     u_perp                    : zero
!     vconst_1 = 1.e7_dbl_kind
!     vconst_2 = 24.5_dbl_kind
!     vconst_3 = 0.2_dbl_kind
!     vconst_4 = 1.e-8_dbl_kind
!     vconst_5 = 3
!     vconst_6 = 1.e7_dbl_kind
!     horz_viscosity_outfile = 'horz_viscosity_outfile'
!
!-----------------------------------------------------------------------

      hmix_alignment_choice = 'flow'
      lvariable_hmix_aniso = .false.
      lsmag_aniso = .false.
      visc_para = c0
      visc_perp = c0
      c_para = c0
      c_perp = c0
      u_para = c0
      u_perp = c0
      vconst_1 = 1.e7_dbl_kind
      vconst_2 = 24.5_dbl_kind
      vconst_3 = 0.2_dbl_kind
      vconst_4 = 1.e-8_dbl_kind
      vconst_5 = 3
      vconst_6 = 1.e7_dbl_kind
      horz_viscosity_outfile = 'horz_viscosity_outfile'

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=hmix_aniso_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading hmix_aniso_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, hmix_aniso_nml)
          write(stdout,*) ' '
      endif

      if (my_task == master_task) then

        write(stdout,*) ' '
        write(stdout,*) 'Anisotropic viscosity options:'
        write(stdout,*) ' '


        select case (hmix_alignment_choice(1:4))
        case ('flow')
          hmix_alignment_itype = hmix_alignment_type_flow
          write(stdout,*) 
     &      '  Anisotropy aligned along local flow direction'
        case ('east')
          hmix_alignment_itype = hmix_alignment_type_east
          write(stdout,*) 
     &      '  Anisotropy aligned along east-west direction'
        case ('grid')
          hmix_alignment_itype = hmix_alignment_type_grid
          write(stdout,*) 
     &      '  Anisotropy aligned along grid coordinates'
        case default
          hmix_alignment_itype = -1000 
        end select

        if (hmix_alignment_itype /= -1000) then
        if (.not.lvariable_hmix_aniso .and. .not.lsmag_aniso) then
          write(stdout,*) 
     &      '  Spatially variable anisotropic viscosity disabled'
        endif
 
        write(stdout,1001) ' horizontal aniostropic viscosity file =',
     &    horz_viscosity_outfile


        if ( .not.lvariable_hmix_aniso .and. .not.lsmag_aniso ) then
          write(stdout,*)
     &    'Using constant anisotropic viscosity coefficients:'
          write(stdout,1000) '  visc_para = ',visc_para
          write(stdout,1000) '  visc_perp = ',visc_perp
        endif
        if ( lvariable_hmix_aniso .or. lsmag_aniso ) then
          write(stdout,*) 
     &      'Using anisotropic viscosity variables:'
          write(stdout,1000) '   vconst_1 = ',vconst_1
          write(stdout,1000) '   vconst_2 = ',vconst_2
          write(stdout,1000) '   vconst_3 = ',vconst_3
          write(stdout,1000) '   vconst_4 = ',vconst_4
          write(stdout,1002) '   vconst_5 = ',vconst_5
          write(stdout,1000) '   vconst_6 = ',vconst_6
        endif
        if ( lsmag_aniso ) then 
          write(stdout,*) 'Using nonlinear Smagorinski viscosites'
          write(stdout,*) '  input anistropic smag coefficients:'
          write(stdout,1000) '     c_para = ',c_para
          write(stdout,1000) '     c_perp = ',c_perp
        endif
        endif ! hmix_alignment_itype /= -1000
      endif
1000  format(a14,2x,1pe12.5)
1001  format(a40,1x,a80)             
1002  format(a14,2x,i4)

      call broadcast_scalar(hmix_alignment_itype,  master_task)
      call broadcast_scalar(lvariable_hmix_aniso,  master_task)
      call broadcast_scalar(lsmag_aniso,           master_task)
      call broadcast_scalar(visc_para,             master_task)
      call broadcast_scalar(visc_perp,             master_task)
      call broadcast_scalar(c_para,                master_task)
      call broadcast_scalar(c_perp,                master_task)
      call broadcast_scalar(u_para,                master_task)
      call broadcast_scalar(u_perp,                master_task)
      call broadcast_scalar(vconst_1,              master_task)
      call broadcast_scalar(vconst_2,              master_task)
      call broadcast_scalar(vconst_3,              master_task)
      call broadcast_scalar(vconst_4,              master_task)
      call broadcast_scalar(vconst_5,              master_task)
      call broadcast_scalar(vconst_6,              master_task)
      call broadcast_scalar(horz_viscosity_outfile,master_task)
 
      if (hmix_alignment_itype == -1000) then
          call exit_POP( 
     &    ' Unknown type for anisotropic alignment direction')
      endif



!-----------------------------------------------------------------------
!
!     allocate and initialize various 2D arrays:
!
!        AMAX_CFL is 1/2 the maximum allowed viscosity due to CFL limit
!
!-----------------------------------------------------------------------

      allocate(K1E(imt,jmt), K1W(imt,jmt),
     &         K2N(imt,jmt), K2S(imt,jmt),
     &         H1E(imt,jmt), H1W(imt,jmt),
     &         H2N(imt,jmt), H2S(imt,jmt),
     &         AMAX_CFL(imt,jmt))

      H2S = HTE
      H1W = HTN
      call n_eoshift(H2N,H2S)
      call e_eoshift(H1E,H1W)

      WORKA = H2S + H2N
      call w_eoshift(WORKB,WORKA)
      K1W = c2*(WORKA - WORKB)/(WORKA + WORKB)/H1W
      call e_eoshift(K1E,K1W)

      WORKA = H1W + H1E
      call s_eoshift(WORKB,WORKA)
      K2S = c2*(WORKA - WORKB)/(WORKA + WORKB)/H2S
      call n_eoshift(K2N,K2S)

      AMAX_CFL = p125/(dtu*(DXUR**2 + DYUR**2))

      if (lsmag_aniso) then
        allocate(DSMIN(imt,jmt))
        DSMIN = min(DXU,DYU)
      endif

      if (lvariable_hmix_aniso .or. lsmag_aniso) then
 
        allocate(F_PARA(imt,jmt,km), F_PERP(imt,jmt,km))

!       compute or read in spatial dependence of
!       anisotropic viscosity coefficients here.

        call hmix_anis_spvar
 

        do k=1,km
          f_para_min = global_minval(F_PARA(:,:,k))
          f_para_max = global_maxval(F_PARA(:,:,k))
          f_perp_min = global_minval(F_PERP(:,:,k))
          f_perp_max = global_maxval(F_PERP(:,:,k))
          if (my_task == master_task) then
            write(stdout,1019) k
 1019       format('  vertical level = ', i3)
            write(stdout,1020) f_para_min, f_para_max
 1020       format('  Min F_PARA =',1x,1pe12.5,3x,
     &               'Max F_PARA =',1x,1pe12.5)
            write(stdout,1030) f_perp_min, f_perp_max
 1030       format('  Min F_PERP =',1x,1pe12.5,3x,
     &             'Max F_PERP =',1x,1pe12.5)
          endif
        enddo
 
      endif
 
 
      if (lvariable_hmix_aniso .and. .not. lsmag_aniso) then

!       check that viscosities are less than 1/2 CFL limit,
!       taper where necessary. also check that parallel is greater than
!       perpendicular viscosity.

        do k = 1,km

        if (ANY(F_PARA(:,:,k) .gt. AMAX_CFL)) then

          if (my_task == master_task) then
            write(stdout,*) 'WARNING (hmix_aniso):' 
            write(stdout,*) 
     &      '  some values of parallel viscosity > 1/2 CFL limit'
            write(stdout,*) 
     &      '  parallel viscosity will be tapered'
          endif

          where (F_PARA(:,:,k) > AMAX_CFL)
            F_PARA(:,:,k) = AMAX_CFL
          endwhere

        endif

        if (ANY(F_PERP(:,:,k) .gt. AMAX_CFL)) then

          if (my_task == master_task) then
            write(stdout,*) 'WARNING (hmix_aniso):' 
            write(stdout,*) 
     &      '  some values of perpendicular viscosity > 1/2 CFL limit'
            write(stdout,*) 
     &      '  perpendicualr viscosity will be tapered'
          endif

          where (F_PERP(:,:,k) > AMAX_CFL)
            F_PERP(:,:,k) = AMAX_CFL
          endwhere

        endif

        if (ANY(F_PARA(:,:,k) .lt. F_PERP(:,:,k)) ) then
          if (my_task == master_task) then
            write(stdout,*)
     &      'WARNING (hmix_aniso): some values of'
            write(stdout,*)
     &      '  parallel viscosity less than perpendicular viscosity'
            write(stdout,*)
     &      '  F_PARA value will be set to F_PERP value'
          endif
        endif

        where (F_PARA(:,:,k) < F_PERP(:,:,k))
          F_PARA(:,:,k) = F_PERP(:,:,k)
        endwhere

        enddo  ! end of k-loop

      else

        if ( lsmag_aniso ) then

          allocate(F_PARA_SMAG(imt,jmt), F_PERP_SMAG(imt,jmt))

          F_PARA_SMAG = c1

!       latitude dependence of F_PERP_SMAG for viscous CCSM runs

          WORKA = abs(ULAT)*radian
          where (WORKA .ge. 20.0_dbl_kind)
            F_PERP_SMAG = c1 - 0.98_dbl_kind
     &               *exp(-(WORKA - 20.0_dbl_kind)**2/98.0_dbl_kind)
          elsewhere
            F_PERP_SMAG = 0.02_dbl_kind
          endwhere

        endif

        if (.not.lsmag_aniso) then

          if (ANY(visc_para .gt. AMAX_CFL)) then
            if (my_task == master_task) then
              write(stdout,*) 
     &        'WARNING (hmix_aniso):' 
              write(stdout,*) 
     &        '  parallel viscosity > 1/2 CFL limit at some points'
            endif
          endif

          if (ANY(visc_perp .gt. AMAX_CFL)) then
            if (my_task == master_task) then
              write(stdout,*) 
     &        'WARNING (hmix_aniso):' 
              write(stdout,*) 
     &        '  perpendicular viscosity > 1/2 CFL limit at some points'
            endif
          endif

          if (visc_para .lt. visc_perp) then
            call exit_POP(
     &   'ERROR: parallel viscosity less than perpendicular viscosity')
          endif

        endif

      endif

      if (lsmag_aniso) then

        if (c_para .lt. c_perp) then
          if (my_task == master_task) then
            write(stdout,*) 
     &      'WARNING (hmix_aniso):'
            write(stdout,*)
     &      '  smagorinski c_para < c_perp '
          endif
        endif


      endif

      if (lvariable_hmix_aniso .or. lsmag_aniso) then
        call get_unit (nu)
        call open_parallel_file (nu, horz_viscosity_outfile, recl_dbl)
        call write_array (nu, F_PARA)
        call write_array (nu, F_PERP)
        call close_parallel_file (nu)
      endif

      if (my_task == master_task) then
        write(stdout,*) ' '
      endif


      end subroutine init_aniso

!***********************************************************************

      subroutine hdiffu_aniso(k,HDUK,HDVK)

!-----------------------------------------------------------------------
!
!     This routine computes the viscous terms in the momentum equation
!     as the divergence of a stress tensor which is linearly related
!     to the rate-of-strain tensor with viscous coefficents visc_para
!     and visc_perp.  These coefficients represent energy dissipation
!     in directions parallel and perpendicular to a specified 
!     alignment direction which breaks the isotropy of the dissipation.
!
!     A functional approach is used to derived the discrete operator,
!     which ensures positive-definite energy dissipation, provided
!     visc_para > visc_perp.
!
!     In the functional approach, each U-cell is subdivided horizontally
!     into 4 subcells (or "quarter-cells"), and the strain and stress
!     tensors are evaluated in each subcell.
!
!     The viscosities may optionally be evaluated with smagorinsky-like 
!     non-linear dependence on the deformation rate, which is 
!     proportional to the norm of the strain tensor.  With the 
!     smagorinski option, the viscosities are evalutated as
!    
!      visc_para -> min{max[c_para*F_PARA_SMAG*|D|*ds**2),F_PARA],AMAX_CFL}
!      visc_perp -> min{max[c_perp*F_PERP_SMAG*|D|*ds**2),F_PERP],AMAX_CFL}
!
!     with visc_para >= visc_perp for positive-definite energy dissipation.
!
!     where ds = min(dx,dy), the |D| is the deformation rate:
!     |D| = sqrt(2)|E|,  c_para and c_perp are dimensionless 
!     coefficients of order 1, F_PARA_SMAG and F_PERP_SMAG represent
!     the horizontal variations of the viscosity coefficients, 
!     F_PARA and F_PERP are the time-invariant "numerical"
!     viscosity parts, and AMAX_CFL is 1/2 the allowed CFL limit value.
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!       Indices for quarter-cells
!       X = U point
!       N,S,E,W label north,south,east,west faces of U cell
!       1,2,3,4 label SW,NW,NE,SE quarter cells surrounding a U-point
!
!
!              X         ------- X -------         X
!                       |        |        |   
!                       |   1    |   4    |   
!                       |        |        |   
!               ---------------- N ----------------        
!              |        |        |        |        |
!              |   3    |   2    |   3    |   2    |
!              |        |        |        |        |
!              X ------ W ------ X ------ E ------ X
!              |        |        |        |        |
!              |   4    |   1    |   4    |   1    |
!              |        |        |        |        |
!               ---------------- S ---------------- 
!                       |        |        |   
!                       |   2    |   3    |   
!                       |        |        |   
!              X         ------- X -------         X
!
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! depth level index

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) ::
     &  HDUK,               ! Hdiff(Ub) at level k
     &  HDVK                ! Hdiff(Vb) at level k

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: i,j,iq   ! dummy loop index

      real (kind=dbl_kind), dimension (imt,jmt) :: 
     &   HDIFFCFL                            ! for cfl diagnostics

      real (kind=dbl_kind) ::
     &  work1,work2,work3,work4,work5,work6,work7,work8
     &  ,A,B,C,D                             ! viscous coefficients
     &  ,FX,FY                               ! area*friction terms

      real (kind=dbl_kind), dimension (imt,jmt,4) :: 
     &   S11,S22,S12                         ! stress tensor

      real (kind=dbl_kind), dimension (4) :: 
     &   E11,E22,E12                         ! rate-of-strain tensor


!-----------------------------------------------------------------------
!
!     compute rate-of-strain tensor in each quarter-cell
!
!-----------------------------------------------------------------------

      do j=2,jmt-1
      do i=2,imt-1

      work1 = (UVEL(i,j,k,mixtime) - UVEL(i-1,j,k,mixtime))/H1W(i,j)
      work2 = (UVEL(i+1,j,k,mixtime) - UVEL(i,j,k,mixtime))/H1E(i,j)
      work3 = p5*K2S(i,j)*(VVEL(i,j,k,mixtime) + VVEL(i,j-1,k,mixtime))
      work4 = p5*K2N(i,j)*(VVEL(i,j,k,mixtime) + VVEL(i,j+1,k,mixtime))

      E11(1) = work1 + work3
      E11(2) = work1 + work4
      E11(3) = work2 + work4
      E11(4) = work2 + work3

      work1 = (VVEL(i,j,k,mixtime) - VVEL(i,j-1,k,mixtime))/H2S(i,j)
      work2 = (VVEL(i,j+1,k,mixtime) - VVEL(i,j,k,mixtime))/H2N(i,j)
      work3 = p5*K1W(i,j)*(UVEL(i,j,k,mixtime) + UVEL(i-1,j,k,mixtime))
      work4 = p5*K1E(i,j)*(UVEL(i,j,k,mixtime) + UVEL(i+1,j,k,mixtime))

      E22(1) = work1 + work3
      E22(2) = work2 + work3
      E22(3) = work2 + work4
      E22(4) = work1 + work4

      work1 = (UVEL(i,j,k,mixtime) - UVEL(i,j-1,k,mixtime))/H2S(i,j)
      work2 = (UVEL(i,j+1,k,mixtime) - UVEL(i,j,k,mixtime))/H2N(i,j)
      work3 = (VVEL(i,j,k,mixtime) - VVEL(i-1,j,k,mixtime))/H1W(i,j)
      work4 = (VVEL(i+1,j,k,mixtime) - VVEL(i,j,k,mixtime))/H1E(i,j)
      work5 = K2S(i,j)*(UVEL(i,j,k,mixtime) + UVEL(i,j-1,k,mixtime))
      work6 = K2N(i,j)*(UVEL(i,j,k,mixtime) + UVEL(i,j+1,k,mixtime))
      work7 = K1W(i,j)*(VVEL(i,j,k,mixtime) + VVEL(i-1,j,k,mixtime))
      work8 = K1E(i,j)*(VVEL(i,j,k,mixtime) + VVEL(i+1,j,k,mixtime))

      E12(1) = work1 + work3 - p5*(work5 + work7)
      E12(2) = work2 + work3 - p5*(work6 + work7)
      E12(3) = work2 + work4 - p5*(work6 + work8)
      E12(4) = work1 + work4 - p5*(work5 + work8)

!-----------------------------------------------------------------------
!
!     calculate viscous coeffs A,B,C,D
!     and stress tensor from strain tensor
!
!-----------------------------------------------------------------------

      if (hmix_alignment_itype == hmix_alignment_type_flow) then

        work5 = sqrt(UVEL(i,j,k,mixtime)**2 + VVEL(i,j,k,mixtime)**2)
        if (work5.ge.eps) then
          work1 = UVEL(i,j,k,mixtime)/work5   ! n1
          work2 = VVEL(i,j,k,mixtime)/work5   ! n2
        else
          work1 = c0
          work2 = c0
        endif

      endif

      if (lsmag_aniso) then    ! smagorinsky nonlinear viscosity 

        do iq = 1,4              ! loop over quarter cells

!         deformation rate |D| = sqrt(2)|E| (use temp work6)
!         note E12 is twice the (1,2) component of the strain tensor

          work6 =  sqrt(c2*(E11(iq)**2 + E22(iq)**2) + E12(iq)**2)

!         compute nonlinear viscosities  (ds = min(dx,dy))

          work3 = max(c_para*F_PARA_SMAG(i,j)*work6*DSMIN(i,j)
     &                *DSMIN(i,j), F_PARA(i,j,k))
          work4 = max(c_perp*F_PERP_SMAG(i,j)*work6*DSMIN(i,j)
     &                *DSMIN(i,j), F_PERP(i,j,k))

!         taper viscosities when greater than 1/2 CFL limit

          work3 = min(work3,AMAX_CFL(i,j))    ! parallel viscosity
          work4 = min(work4,AMAX_CFL(i,j))    ! perpendicular  "

!         enforce parallel_visc >= perpendicular_visc

          work3 = max(work3, work4)
        
!         compute coefficients multiplying elements strain tensor
!         use only one of the following anisotropic forms:
!         grid-aligned, east-aligned or flow-aligned 

          select case (hmix_alignment_itype)

          case (hmix_alignment_type_grid)

            A = work3
            B = work4 
            C = c0
            D = work4 

          case (hmix_alignment_type_east)

            work1 =  cos(ANGLE(i,j))
            work2 = -sin(ANGLE(i,j))

            A = work3 - c2*(work3-work4)*(work1*work2)**2
            B = work4 - c2*(work3-work4)*(work1*work2)**2
            C = (work3-work4)*work1*work2*(work1**2-work2**2)
            D = work4 + c2*(work3-work4)*(work1*work2)**2

          case (hmix_alignment_type_flow)

            A = work3 - c2*(work3-work4)*(work1*work2)**2
            B = work4 - c2*(work3-work4)*(work1*work2)**2
            C = (work3-work4)*work1*work2*(work1**2-work2**2)
            D = work4 + c2*(work3-work4)*(work1*work2)**2

          end select

!         compute stress tensor in each quarter-cell
  
          S11(i,j,iq) =   A*E11(iq) - B*E22(iq) + C*E12(iq)
          S22(i,j,iq) = - B*E11(iq) + A*E22(iq) - C*E12(iq)
          S12(i,j,iq) =   C*(E11(iq) - E22(iq)) + D*E12(iq)

        enddo

      else     ! no smagorinski nonlinearity

        if (lvariable_hmix_aniso) then
          work3 = F_PARA(i,j,k)         ! parallel viscosity
          work4 = F_PERP(i,j,k)         ! perpendicular "
        else
          work3 = visc_para
          work4 = visc_perp
        endif

!       compute coefficients multiplying elements strain tensor
!       use only one of the following anisotropic forms:
!       grid-aligned, east-aligned or flow-aligned 

        select case (hmix_alignment_itype)

        case (hmix_alignment_type_grid)

          A = work3
          B = work4 
          C = c0
          D = work4 

        case (hmix_alignment_type_east)

          work1 =  cos(ANGLE(i,j))
          work2 = -sin(ANGLE(i,j))

          A = work3 - c2*(work3-work4)*(work1*work2)**2
          B = work4 - c2*(work3-work4)*(work1*work2)**2
          C = (work3-work4)*work1*work2*(work1**2-work2**2)
          D = work4 + c2*(work3-work4)*(work1*work2)**2

        case (hmix_alignment_type_flow)

          A = work3 - c2*(work3-work4)*(work1*work2)**2
          B = work4 - c2*(work3-work4)*(work1*work2)**2
          C = (work3-work4)*work1*work2*(work1**2-work2**2)
          D = work4 + c2*(work3-work4)*(work1*work2)**2

        end select

!       compute stress tensor in each quarter-cell

        do iq = 1,4
          S11(i,j,iq) =   A*E11(iq) - B*E22(iq) + C*E12(iq)
          S22(i,j,iq) = - B*E11(iq) + A*E22(iq) - C*E12(iq)
          S12(i,j,iq) =   C*(E11(iq) - E22(iq)) + D*E12(iq)
        enddo

      endif

      enddo  ! (i-loop)
      enddo  ! (j-loop)

!-----------------------------------------------------------------------
!
!     compute friction terms from stresses
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     x-component
!
!-----------------------------------------------------------------------

      do j=3,jmt-2
      do i=3,imt-2

      work1 = H2S(i,j)*S11(i,j,1) + H2N(i,j)*S11(i,j,2)
      work2 = H2S(i,j)*S11(i,j,4) + H2N(i,j)*S11(i,j,3)
      work3 = H2S(i+1,j)*S11(i+1,j,1) + H2N(i+1,j)*S11(i+1,j,2)
      work4 = H2S(i-1,j)*S11(i-1,j,4) + H2N(i-1,j)*S11(i-1,j,3)

      FX = p25*(work2 + work3 - work1 - work4)  ! <h2*S11>_e - <h2*S11>_w

      work1 = H1W(i,j)*S12(i,j,1) + H1E(i,j)*S12(i,j,4)
      work2 = H1W(i,j)*S12(i,j,2) + H1E(i,j)*S12(i,j,3)
      work3 = H1W(i,j+1)*S12(i,j+1,1) + H1E(i,j+1)*S12(i,j+1,4)
      work4 = H1W(i,j-1)*S12(i,j-1,2) + H1E(i,j-1)*S12(i,j-1,3)

      FX = FX + p25*((work2 + work3)*(c1 + p5*H2N(i,j)*K2N(i,j)) 
     &             - (work1 + work4)*(c1 - p5*H2S(i,j)*K2S(i,j))) 
                                                !  <h1*S12>_n*(1+h2n*k2n/2) 
                                                ! -<h1*S12>_s*(1-h2s*k2s/2)

      work1 = H2S(i,j)*S22(i,j,1) + H2N(i,j)*S22(i,j,2)
      work2 = H2S(i,j)*S22(i,j,4) + H2N(i,j)*S22(i,j,3)
      work3 = H2S(i+1,j)*S22(i+1,j,1) + H2N(i+1,j)*S22(i+1,j,2)
      work4 = H2S(i-1,j)*S22(i-1,j,4) + H2N(i-1,j)*S22(i-1,j,3)

      FX = FX - p125*((work2 + work3)*H1E(i,j)*K1E(i,j)   ! -<h2*S22>_e*h1e*k1e/2  
     &             +  (work1 + work4)*H1W(i,j)*K1W(i,j))  ! -<h2*S22>_w*h1w*k1w/2

!-----------------------------------------------------------------------
!
!     y-component
!
!-----------------------------------------------------------------------

      work1 = H1W(i,j)*S22(i,j,1) + H1E(i,j)*S22(i,j,4)
      work2 = H1W(i,j)*S22(i,j,2) + H1E(i,j)*S22(i,j,3)
      work3 = H1W(i,j+1)*S22(i,j+1,1) + H1E(i,j+1)*S22(i,j+1,4)
      work4 = H1W(i,j-1)*S22(i,j-1,2) + H1E(i,j-1)*S22(i,j-1,3)

      FY = p25*(work2 + work3 - work1 - work4)  ! <h1*S22>_n - <h1*S22>_s

      work1 = H2S(i,j)*S12(i,j,1) + H2N(i,j)*S12(i,j,2)
      work2 = H2S(i,j)*S12(i,j,4) + H2N(i,j)*S12(i,j,3)
      work3 = H2S(i+1,j)*S12(i+1,j,1) + H2N(i+1,j)*S12(i+1,j,2)
      work4 = H2S(i-1,j)*S12(i-1,j,4) + H2N(i-1,j)*S12(i-1,j,3)

      FY = FY + p25*((work2 + work3)*(c1 + p5*H1E(i,j)*K1E(i,j)) 
     &             - (work1 + work4)*(c1 - p5*H1W(i,j)*K1W(i,j))) 
                                                !  <h2*S12>_e*(1+h1e*k1e/2) 
                                                ! -<h2*S12>_w*(1-h1w*k1w/2)

      work1 = H1W(i,j)*S11(i,j,1) + H1E(i,j)*S11(i,j,4)
      work2 = H1W(i,j)*S11(i,j,2) + H1E(i,j)*S11(i,j,3)
      work3 = H1W(i,j+1)*S11(i,j+1,1) + H1E(i,j+1)*S11(i,j+1,4)
      work4 = H1W(i,j-1)*S11(i,j-1,2) + H1E(i,j-1)*S11(i,j-1,3)

      FY = FY - p125*((work2 + work3)*H2N(i,j)*K2N(i,j)
     &             +  (work1 + work4)*H2S(i,j)*K2S(i,j))
                         ! -<h1*S11>_n*h2n*k2n/2 - <h1*S11>_s*h2s*k2s/2

!-----------------------------------------------------------------------
!
!     divide by U-cell area
!
!-----------------------------------------------------------------------

      if(KMU(i,j).ge.k) then
        HDUK(i,j) = FX/UAREA(i,j)
        HDVK(i,j) = FY/UAREA(i,j)
      else
        HDUK(i,j) = c0
        HDVK(i,j) = c0
      endif

      enddo  ! (i-loop)
      enddo  ! (j-loop)

!-----------------------------------------------------------------------
!
!     compute horiz diffusion cfl diagnostics if required
!     note that only the parallel component is considered here.
!
!-----------------------------------------------------------------------

      if (ldiag_cfl .and. .not.lsmag_aniso) then
      
        if (lvariable_hmix_aniso) then
          HDIFFCFL = merge(c4*F_PARA(:,:,k)
     &                     *(DXUR**2 + DYUR**2), c0, KMU > k)
        else
          HDIFFCFL = merge(c4*visc_para*(DXUR**2 + DYUR**2),c0,KMU > k)
        endif
        HDIFFCFL = abs(HDIFFCFL)
        call cfl_hdiff(k,HDIFFCFL,2)

      endif

!-----------------------------------------------------------------------


!-----------------------------------------------------------------------

!-----------------------------------------------------------------------

      end subroutine hdiffu_aniso

!***********************************************************************
 
      subroutine hmix_anis_spvar

!-----------------------------------------------------------------------
!
!     Spatially-varying anisotropic viscosity initialization
!
!
!     This routine defines NCOM-like spatial distributions of
!     viscosity coefficients F_PARA and F_PERP
!
!     written by:	Stephen Yeager 3/2000
!     modified by:      GD (08/2001)
!
!-----------------------------------------------------------------------
!-------------------------------------------------------------------------
!
!
!   "A_viscosity" = F_PARA 
!                  = max{0.5*visc_vel_scale(z)*A*max[dx,dy],vconst_6}
!
!   where
!          A = 0.425 * cos(pi*y*radian/30) + 0.575   for |y*radian| < 30
!          A = 0.15                                  otherwise 
!
!   Here, A provides a horizontal variation for visc_vel_scale.
!
!   "B_viscosity" = F_PERP = max( bu, bv)
!
!   and 
!        F_PARA = min(F_PARA, AMAX_CFL),
!        F_PERP = min(F_PERP, AMAX_CFL),
!        F_PARA = max(F_PARA, F_PERP)
!   are enforced in init_aniso and hdiffu_aniso for the lvariable_hmix_aniso
!   and lsmag_aniso choices, respectively. 
!
!   In the above equations, 
!
!        bu  = vconst_1 * ( 1 + vconst_2
!             * ( 1 + cos( 2*y + pi ) ) )
!        bv  = vconst_3 * beta_f * dx^3
!             * exp( - (vconst_4 * distance)^2 )
!
!   with 
!        beta_f         (x,y)   = 2 * omega * cos(ULAT(i,j)) / radius
!        distance       (x,y,z) = actual distance to "vconst_5" points
!                                 west of the nearest western boundary
!        dx             (x,y)   = DXU(i,j)
!        dy             (x,y)   = DYU(i,j)
!        visc_vel_scale (z)     = 100.0 * exp(-zt(k)/visc_vel_scale_length)
!        visc_vel_scale_length  = e-folding scale ( = 1500.0e2 cm)
!        y              (x,y)   = ULAT(i,j), latitude of "u/v" grid pts in radians 
!
!   Also, "vconst_#" are input parameters defined in namelist hmix_aniso_nml. 
!   note that "vconst_1", "vconst_6", and "vconst_4" have dimensions of cm^2/s,
!   cm^2/s, and 1/cm, respectively. "vconst_5" is an INTEGER.
!
!   NOTE: The nearest western boundary computations are done along the
!         model longitudinal grid lines. Therefore, the viscosity
!         coefficients based on these are only approximate in the high
!         Northern Hemisphere when used with non-grid-aligned options.
!
!-------------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     local variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) ::
     & beta_f,			! Coriolis parameter
     & dist			! distance to nearest western boundary

      integer (kind=int_kind) ::
     & i,j,k,ie,is,n,ii,    	! dummy loop counters
     & index,			! index of nearest western boundary point
     & indexo,        		! index of nwbp + buffer
     & ig,jg,igp1,     		! dummy loop counters (global)
     & ncount	       		! number of western boundaries 	

      real (kind=dbl_kind) ::
     & visc_vel_scale,		
     & bu,bv			! B_viscosity terms

      integer (kind=int_kind), dimension(imt) ::
     & iwp

      integer (kind=int_kind), dimension(imt_global,jmt_global) ::
     & nwbp_g			! nearest western boundary point (global)

      real (kind=dbl_kind), parameter ::
     & vvsl=1500.e2_dbl_kind, 	! visc_vel_scale_length (cm)
     & dist_max=1.e10_dbl_kind 	! distance for ACC region

      real (kind=dbl_kind), dimension(imt_global,jmt_global) ::
     & HTN_G, 			! zonal distance between U points
     & dist_g    		! distance to nwbp (cm)     

      integer (kind=int_kind), dimension(imt_global,jmt_global) ::
     & KMU_G   			! kmt array at U points      

!-----------------------------------------------------------------------

      beta_f = c2*omega*cos(ULAT)/radius
      F_PARA = c0
      F_PERP = c0

      call gather_global(HTN_G, HTN, master_task)
      call gather_global(KMU_G, KMU, master_task)

      do k=1,km

      nwbp_g = 0

      !***
      !*** determine nearest western boundary
      !*** (master loops over global domain)
      !***

        if (my_task == master_task) then
           do jg=1,jmt_global

              ncount = 0
              iwp = 0

              do ig=1,imt_global
                igp1 = ig + 1
                if ( ig == imt_global )  igp1 = 1
                if ( (KMU_G(ig  ,jg) .lt. k) .and.
     &               (KMU_G(igp1,jg) .ge. k) ) then 
                  ncount = ncount + 1
                  iwp(ncount) = ig
                endif
              enddo
              if ( ncount .gt. 0 ) then
                do n=1,ncount-1
                  is = iwp(n)
                  ie = iwp(n+1) - 1
                  do ig = is, ie
                    nwbp_g(ig,jg) = is
                  enddo
                enddo
                do ig = 1, imt_global
                  if (nwbp_g(ig,jg) .eq. 0 )
     &              nwbp_g(ig,jg) = iwp(ncount)
                enddo
              endif
           enddo

      !***
      !*** determine distance to nearest western boundary
      !*** (master loops over global domain)
      !***

           do jg=1,jmt_global
              do ig=1,imt_global
                index=nwbp_g(ig,jg)
                indexo = index + vconst_5
            if ( index .eq. 0) then
              dist_g(ig,jg) = dist_max
            elseif ( ig .ge. index  .and.  ig .le. indexo ) then
              dist_g(ig,jg) = c0
            elseif ( (ig .gt. indexo) ) then
                dist_g(ig,jg) = HTN_G(ig,jg)+dist_g(ig-1,jg)
            elseif ( ig .lt. index ) then
              if (indexo .le. imt_global) then
                if (ig .eq. 1) then
                  dist_g(ig,jg)=c0
                    do ii=indexo+1,imt_global
                      dist_g(ig,jg)=HTN_G(ii,jg) + dist_g(ig,jg)
                    enddo
                    dist_g(ig,jg) = HTN_G(ig,jg)+dist_g(ig,jg)
                else
                  dist_g(ig,jg) = HTN_G(ig,jg)+dist_g(ig-1,jg)
                endif
              else
                if (ig .le. (indexo - imt_global)) then
                  dist_g(ig,jg) = c0
                else
                  dist_g(ig,jg) = HTN_G(ig,jg)+dist_g(ig-1,jg)
                endif
              endif
            endif
	
              enddo
           enddo
        endif 	! (my_task == master_task)

      call scatter_global(dist, dist_g, master_task)

      !***
      !*** calculate viscosity coefficients
      !*** (all processors loop over local subdomain)
      !***

         do j=1,jmt
          do i=1,imt

           visc_vel_scale = 100.0_dbl_kind * exp(-zt(k)/vvsl)

!     use "bu" as temp

           bu = 0.15_dbl_kind
           if ( abs(ULAT(i,j)*radian) < 30._dbl_kind )
     &       bu = 0.425_dbl_kind*cos(pi*ULAT(i,j)*radian/30._dbl_kind)
     &          + 0.575_dbl_kind

           F_PARA(i,j,k) =  max( p5 * visc_vel_scale * bu
     &                    * max(DXU(i,j),DYU(i,j)), vconst_6 )

           bu = vconst_1*(c1 + vconst_2*(c1 +
     &          cos((c2*ULAT(i,j))+pi)))
           bv = vconst_3*beta_f(i,j)*(DXU(i,j)**3)
           bv = bv*exp(-(vconst_4*dist(i,j))**2)
           F_PERP(i,j,k) = max(bu,bv)
 
!---------------------------------------------------------------------
!          F_PARA >= F_PERP for positive energy dissipation and
!          the diffusive CFL criteria will be enforced in subroutine
!          init_aniso
!---------------------------------------------------------------------


        enddo	! i
       enddo	! j
      enddo	! k

!-----------------------------------------------------------------------

      end subroutine hmix_anis_spvar
!***********************************************************************

      end module hmix_aniso

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
