!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module hmix_gm

!***********************************************************************
!
!     this module contains routines for computing horizontal mixing
!     using the Gent-McWilliams eddy transport parameterization
!     and isopycnal diffusion.
!
!     CVS:$Id: hmix_gm.F,v 1.3.2.6 2001/09/07 19:52:07 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use prognostic
      use grid
      use io
      use stencils
      use vertical_mix
      use state_mod
      use time_management
      use diagnostics
      use restart
      use exit_mod
      use hmix_gm_uvwisop

      implicit none
      save

!-----------------------------------------------------------------------
!
!     variables to save from one call to next
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  ktp,kbt,kbt2,kn,ks    !   of KAPPA arrays

      real (kind=dbl_kind), dimension(1:km) ::
     &  kappa_depth         ! depth dependence for kappa

      real (kind=dbl_kind), dimension(:,:), allocatable ::
     &  HXY,                ! dx/dy for y-z plane
     &  HYX,                ! dy/dx for x-z plane
     &  HYXW, HXYS,         ! west and south-shifted values of above
     &  DTR2,               ! (1/DXT)**2 + (1/DYT)**2
     &  RBR,                ! Rossby radius
     &  RX,                 ! Dx(rho) on the east  face of T-cell
     &  RY,                 ! Dy(rho) on the north face of T-cell
     &  RXW,RYS,            ! same on west, south faces
     &  DRDT,DRDS           ! expansion coefficients d(rho)/dT,S

      real (kind=dbl_kind), dimension(:,:,:,:), allocatable ::
     &  SLX, SLY            ! slope of isopycnal sfcs in x,y-direction

      real (kind=dbl_kind), dimension(:,:,:), allocatable ::
     &  KAPPA               ! tracer diffusion coefficient
     &, KAPPA_B             ! thickness diffusion coefficient

      real (kind=dbl_kind), dimension(:,:,:,:), allocatable ::
     &  TX,TY,TZ            ! tracer differences in each direction

      real (kind=dbl_kind), dimension(:,:,:), allocatable ::
     &  TXP, TYP, TZP, TEMP

      real (kind=dbl_kind), dimension(:,:,:), allocatable ::
     &  FZTOP               ! vertical flux

      real (kind=dbl_kind), dimension(:,:), allocatable ::
     &  KSV,                ! temporary array for KAPPA
     &  BTP                 ! beta plane approximation

      logical (kind=log_kind) :: compute_kappa

      logical (kind=log_kind) ::
     &  diff_tapering       ! different tapering for two diffusivities

!-----------------------------------------------------------------------
!
!     GM specific options
!
!     kappa_type = constant or variable
!     slope_control = tanh function (Danabosoglu and McWilliams 1995) or
!                     DM95 with replacement function to tanh or
!                     slope clipping or
!                     method of Gerdes et al (1991)
!     gm_bolus = .true. for explicit bolus velocity computation.
!                these velocities are later used in the advective terms.
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), parameter ::
     &  kappa_type_const = 1,
     &  kappa_type_var   = 2,
     &  slope_control_tanh    = 1,
     &  slope_control_notanh  = 2,
     &  slope_control_clip    = 3,
     &  slope_control_Gerd    = 4

      integer (kind=int_kind) ::
     &  kappa_type,           ! choice of KAPPA
     &  slope_control         ! choice for slope control

      logical (kind=log_kind) ::
     &  gm_bolus              ! .true. for explicit bolus velocity computation

      real (kind=dbl_kind) ::
     &  ah_bolus,             ! diffusivity for GM bolus transport
     &  ah_bkg,               ! backgroud horizontal diffusivity at k = KMT
     &  slm_r,                ! max. slope slope allowed for redi diffusion
     &  slm_b                 ! max. slope slope allowed for bolus transport

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_gm

!-----------------------------------------------------------------------
!
!     initializes various choices and allocates necessary space
!     also computes some time-independent arrays
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input namelist for setting various GM options
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  nml_error,            ! error flag for namelist
     &  k                     ! vertical level index

      real (kind=dbl_kind) ::
     &  kappa_depth_1,        ! parameters for variation of kappa
     &  kappa_depth_2,        ! with depth
     &  kappa_depth_scale

      character (char_len) ::
     &  kappa_choice,         ! choice of KAPPA
     &  slope_control_choice  ! choice for slope control

      namelist /hmix_gm_nml/ kappa_choice, 
     &                       slope_control_choice,
     &                       kappa_depth_1, kappa_depth_2,
     &                       kappa_depth_scale, ah_bolus, 
     &                       ah_bkg, slm_r, slm_b, gm_bolus

!-----------------------------------------------------------------------
!
!     read input namelist for additional GM options
!
!     DEFAULT OPTION
!     kappa           : constant = ah
!     slope control   : method by DM95 with replacing tanh by polynomial
!     gm_bolus        : .false.
!
!     variation of kappa with depth is
!       kappa_depth_1 + kappa_depth_2*exp(-z/kappa_depth_scale)
!
!     ah_bolus         : diffusion for GM bolus transport
!     ah_bkg           : background horizontal diffusion
!     slm_r            : max. slope slope allowed for redi diffusion
!     slm_b            : max. slope slope allowed for bolus transport
!
!-----------------------------------------------------------------------

      kappa_type    = kappa_type_const
      slope_control = slope_control_notanh
      kappa_depth_1 = c1
      kappa_depth_2 = c0
      kappa_depth_scale = 150000.0_dbl_kind
      ah_bolus      = 0.8e7_dbl_kind
      ah_bkg        = c0
      slm_r = 0.01_dbl_kind
      slm_b = 0.01_dbl_kind
      gm_bolus = .false.

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=hmix_gm_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading hmix_gm_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout,  hmix_gm_nml)
          write(stdout,*) ' '
      endif

      if (my_task == master_task) then
        write(stdout,*) '  Gent-McWilliams options:'
        write(stdout,*) '    kappa choice is ',trim(kappa_choice)
        write(stdout,*) '    slope control choice is ',
     &                                 trim(slope_control_choice)
        write(stdout,'(a23,1pe13.6)') ' thickness diffusion = ',
     &                                  ah_bolus
        write(stdout,'(a23,1pe13.6)') ' backgroud diffusion = ',
     &                                  ah_bkg
        write(stdout,'(a23,1pe13.6)') ' max. slope for redi = ',
     &                                  slm_r
        write(stdout,'(a23,1pe13.6)') ' max. slope for bolus = ',
     &                                  slm_b
        if (kappa_depth_2 /= c0) then
          write(stdout,'(a27)') '    kappa varies with depth'
          write(stdout,'(a20,1pe13.6)') '    kappa_depth_1 = ',
     &                                       kappa_depth_1
          write(stdout,'(a20,1pe13.6)') '    kappa_depth_2 = ',
     &                                       kappa_depth_2
          write(stdout,'(a24,1pe13.6)') '    kappa_depth_scale = ',
     &                                       kappa_depth_scale
        else
          write(stdout,'(a33)') '   kappa does not vary with depth'
        endif

        if ( gm_bolus ) then
          write(stdout,'(a45)') 
     &        '    explicit bolus velocity computation is on'
        endif

        select case (kappa_choice(1:3))
        case ('con')
          kappa_type = kappa_type_const
        case ('var')
          kappa_type = kappa_type_var
        case default
          kappa_type = -1000
        end select

        select case (slope_control_choice(1:4))
        case ('tanh')
          slope_control = slope_control_tanh
        case ('nota')
          slope_control = slope_control_notanh
        case ('clip')
          slope_control = slope_control_clip
        case ('Gerd')
          slope_control = slope_control_Gerd
        case default
          slope_control = -1000
        end select

      endif

      call broadcast_scalar(kappa_type,    master_task)
      call broadcast_scalar(slope_control, master_task)
      call broadcast_scalar(kappa_depth_1, master_task)
      call broadcast_scalar(kappa_depth_2, master_task)
      call broadcast_scalar(kappa_depth_scale, master_task)
      call broadcast_scalar(ah_bolus,      master_task)
      call broadcast_scalar(ah_bkg  ,      master_task)
      call broadcast_scalar(slm_r,         master_task)
      call broadcast_scalar(slm_b,         master_task)
      call broadcast_scalar(gm_bolus,      master_task)

      if (kappa_type == -1000) then
        call exit_POP('unknown type for KAPPA in GM setup')
      endif
      if (slope_control == -1000) then
        call exit_POP('unknown slope control method in GM setup')
      endif


!-----------------------------------------------------------------------
!
!     allocate GM arrays
!
!-----------------------------------------------------------------------

      allocate (HXY (imt,jmt), HYX (imt,jmt),
     &          HYXW(imt,jmt), HXYS(imt,jmt),
     &          DTR2(imt,jmt), RBR (imt,jmt),
     &          RX  (imt,jmt), RY  (imt,jmt),
     &          RXW (imt,jmt), RYS (imt,jmt),
     &          DRDT(imt,jmt), DRDS(imt,jmt),
     &          UISOP (imt,jmt), VISOP (imt,jmt),
     &          WISOPT(imt,jmt), WISOPB(imt,jmt))

      allocate (SLX(imt,jmt,2,3), SLY(imt,jmt,2,3))

      allocate (KAPPA(imt,jmt,3))
      allocate (KAPPA_B(imt,jmt,3))

      allocate (TX(imt,jmt,2,nt), TY(imt,jmt,2,nt),
     &          TZ(imt,jmt,2,nt))

      allocate (TXP(imt,jmt,2), TYP(imt,jmt,2),
     &          TZP(imt,jmt,2), TEMP(imt,jmt,2))

      allocate (FZTOP(imt,jmt,nt))

      allocate  (KSV(imt,jmt), BTP(imt,jmt))

      HXY    = c0
      HYX    = c0
      HYXW   = c0
      HXYS   = c0
      DTR2   = c0
      RBR    = c0
      RX     = c0
      RY     = c0
      RXW    = c0
      RYS    = c0
      DRDT   = c0
      DRDS   = c0
      SLX    = c0
      SLY    = c0
      KAPPA  = c0
      KAPPA_B = c0
      TX     = c0
      TY     = c0
      TZ     = c0
      TXP    = c0
      TYP    = c0
      TZP    = c0
      FZTOP  = c0
      UISOP  = c0
      VISOP  = c0
      WISOPT = c0
      WISOPB = c0

!-----------------------------------------------------------------------
!
!     initialize various time-independent arrays
!
!-----------------------------------------------------------------------

      do k=1,km
        kappa_depth(k) = kappa_depth_1
     &                 + kappa_depth_2*exp(-zt(k)/kappa_depth_scale)
      enddo

      HYX = HTE/HUS       ! Hyx = dy/dx for x-z plane
      HXY = HTN/HUW       ! Hxy = dx/dy for y-z plane

      call w_shift(HYXW,HYX)
      call s_shift(HXYS,HXY)

      DTR2 = DXTR**2 + DYTR**2

      !***
      !*** compute the Rossby radius which will be used to
      !*** contol KAPPA Large et al (1997), JPO, 27,
      !*** pp 2418-2447. Rossby radius = c/(2*omg*sin(latitude))
      !*** where c=200cm/s is the first baroclinic wave speed.
      !*** 15km < Rossby radius < 100km
      !***

!     Inverse of Rossby radius
      RBR = abs(FCORT)/200.0_dbl_kind ! |f|/Cg, Cg = 2 m/s = 200 cm/s
      RBR = min(RBR,c1/1.5e+6_dbl_kind) ! Cg/|f| .ge. 15 km = 1.5e+6 cm
      RBR = max(RBR,1.e-7_dbl_kind) ! Cg/|f| .le. 100 km = 1.e+7 cm

      call sw_4pt(BTP,AT0,ATS,ATW,ATSW,ULAT)
      BTP = c2*omega*cos(BTP)/radius ! beta at t-points

      compute_kappa = .false.

      if(slm_r /= slm_b) then
         diff_tapering = .true.
      else
         diff_tapering = .false.
      endif

!-----------------------------------------------------------------------

      end subroutine init_gm

!***********************************************************************

      subroutine hdifft_gm(k,GTK,ah)

!-----------------------------------------------------------------------
!
!     Gent-McWilliams eddy transport parameterization
!     and isopycnal diffusion
!
!   * this routine must be called successively with k = 1,2,3,...
!
!     differences from the original routine:
!
!     (1) Functional approach with different methods to compute
!         derivatives of density and slopes (see Griffies et al, 1998).
!     (2) nonuniform diffusion coefficient
!     (3) different cfl criterions
!     (4) different methods to control diffusion coefficient
!     (5) different parametrization for bolus velocity (bv)
!     (6) skew flux form for bv
!
!-----------------------------------------------------------------------


!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! depth level index

      real (kind=dbl_kind), intent(in) ::
     &  ah                  ! diffusion coefficient

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,nt), intent(out) ::
     &  GTK     ! diffusion+bolus advection for nth tracer at level k

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), parameter ::
     &   ieast  = 1, iwest  = 2,
     &   jnorth = 1, jsouth = 2

      integer (kind=int_kind) ::
     &   i,j,n,kk,ii,jj,         ! dummy loop counters
     &   ikpam,kid,ktmp  ! array indices

      integer (kind=int_kind), dimension(3) ::
     &   kc                    ! vertical array index

      real (kind=dbl_kind) ::
     &   fz, dz_bottom

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   CX, CY,
     &   RZ, RZM,           ! Dz(rho)
     &   SLA,               ! absolute value of slope
     &   FX, FY,            ! fluxes across east, north faces
     &   WORK1, WORK2       ! local work space
     &,  WORK3, WORK4, WORK5! local work space
     &,  WORK6, WORK7       ! local work space
     &,  KMASK              ! ocean mask
     &,  KMASKE, KMASKN     ! ocean mask
     &,  TAPER1, TAPER2, TAPER3 ! tapering factor
     &,  UIB, VIB           ! work arrays for isopycnal mixing velocities

      real (kind=dbl_kind), dimension(imt,jmt), save ::
     &   UIT, VIT           ! work arrays for isopycnal mixing velocities

!-----------------------------------------------------------------------
!
!     initialize various quantities
!
!-----------------------------------------------------------------------


      if (k == 1) then

         ktp = 1
         kbt = 2
         kbt2 = 3
         kn=1
         ks=2

         UIT = c0
         VIT = c0
         UIB = c0
         VIB = c0
         WISOPB = c0

      endif

      WISOPT = WISOPB

      KMASK = merge(c1, c0, k < KMT)
      CX = merge(dz(k)*HYX*p25, c0, (k <= KMT) .and. (k <= KMTE))
      CY = merge(dz(k)*HXY*p25, c0, (k <= KMT) .and. (k <= KMTN))

      kc(1) = ktp
      kc(2) = kbt
      kc(3) = kbt2
      
      TAPER1 = c1
      TAPER2 = c1
      TAPER3 = c1

!-----------------------------------------------------------------------
!
!     compute RX=Dx(rho), RY=Dy(rho) on the east face of T-cell at k=1.
!     For other z levels, these are computed and saved in previous call.
!
!-----------------------------------------------------------------------

      if (k == 1) then

	 KMASKE = merge(c1, c0, k <= KMT .and. k <= KMTE)
	 KMASKN = merge(c1, c0, k <= KMT .and. k <= KMTN)

         TEMP(:,:,kn) = max(-c2, TRACER(:,:,k,1,mixtime))
         TXP(:,:,kn) = KMASKE*(eoshift(TEMP(:,:,kn),dim=1,shift=1)
     &	                             - TEMP(:,:,kn))
	 TYP(:,:,kn) = KMASKN*(eoshift(TEMP(:,:,kn),dim=2,shift=1)
     &                               - TEMP(:,:,kn))

         do n=1,nt

           TX(:,:,kn,n) = KMASKE*(
     &          eoshift(TRACER(:,:,k,n,mixtime),dim=1,shift=1)
     &          - TRACER(:,:,k,n,mixtime))

           TY(:,:,kn,n) = KMASKN*(
     &          eoshift(TRACER(:,:,k,n,mixtime),dim=2,shift=1)
     &          - TRACER(:,:,k,n,mixtime))

         end do

         !***
         !*** D_T(rho) & D_S(rho) at level 1
         !***

         call state(k,k,TRACER(:,:,k,1,mixtime),
     &                  TRACER(:,:,k,2,mixtime),
     &                  DRHODT=DRDT, DRHODS=DRDS)

         !***
         !*** RX = Dx(rho) = DRDT*Dx(T) + DRDS*Dx(S)
         !*** RY = Dy(rho) = DRDT*Dy(T) + DRDS*Dy(S)
         !***

         RX  = DRDT*TXP(:,:,kn) + DRDS*TX(:,:,kn,2)

         RXW = DRDT*eoshift(TXP(:,:,kn),dim=1,shift=-1)
     &       + DRDS*eoshift(TX(:,:,kn,2),dim=1,shift=-1)

         RY  = DRDT*TYP(:,:,kn) + DRDS*TY(:,:,kn,2)

         RYS = DRDT*eoshift(TYP(:,:,kn),dim=2,shift=-1)
     &       + DRDS*eoshift(TY(:,:,kn,2),dim=2,shift=-1)

      end if

!-----------------------------------------------------------------------
!
!     compute RZ=Dz(rho) at k=k and
!     SLX = RX / RZ = slope of isopycnal surfaces in x-direction
!     SLY = RY / RZ = slope of isopycnal surfaces in y-direction
!
!-----------------------------------------------------------------------

      if (k < km) then

	 TEMP(:,:,ks) = max(-c2, TRACER(:,:,k+1,1,mixtime))

         TZ(:,:,ks,1) = TRACER(:,:,k  ,1,mixtime)
     &                - TRACER(:,:,k+1,1,mixtime)
         TZ(:,:,ks,2) = TRACER(:,:,k  ,2,mixtime)
     &                - TRACER(:,:,k+1,2,mixtime)
         TZP(:,:,ks) = TEMP(:,:,kn) - TEMP(:,:,ks)

         !***
         !*** RZ = Dz(rho) = DRDT*Dz(T) + DRDS*Dz(S)
         !*** RZM will be used to control KAPPA later
         !***

         RZ = DRDT * TZP(:,:,ks) + DRDS * TZ(:,:,ks,2)
         RZ = min(RZ,-eps2)
         RZM = RZ

         SLX (:,:,ieast ,kbt) = KMASK*RX /RZ
         SLX (:,:,iwest ,kbt) = KMASK*RXW /RZ
         SLY (:,:,jnorth,kbt) = KMASK*RY  /RZ
         SLY (:,:,jsouth,kbt) = KMASK*RYS /RZ

!-----------------------------------------------------------------------
!
!        compute Dz(rho) at level k adiabatically displaced to k+1
!
!-----------------------------------------------------------------------


         !***
         !*** D_T(rho) & D_S(rho) at level k adiabatically displaced
         !*** to k+1
         !***

         call state(k,k+1,TRACER(:,:,k,1,mixtime),
     &                    TRACER(:,:,k,2,mixtime),
     &                    DRHODT=DRDT, DRHODS=DRDS)

         RZ = DRDT * TZP(:,:,ks) + DRDS * TZ(:,:,ks,2)  ! Dz(rho)
         RZ = min(RZ,-eps2)

!-----------------------------------------------------------------------
!
!        compute Dx(rho), Dy(rho) at level k+1
!
!-----------------------------------------------------------------------

	 KMASKE = merge(c1, c0, k+1 <= KMT .and. k+1 <= KMTE)
	 KMASKN = merge(c1, c0, k+1 <= KMT .and. k+1 <= KMTN)

         TXP(:,:,ks) = KMASKE*(eoshift(TEMP(:,:,ks),dim=1,shift=1)
     &	                             - TEMP(:,:,ks))
	 TYP(:,:,ks) = KMASKN*(eoshift(TEMP(:,:,ks),dim=2,shift=1)
     &                               - TEMP(:,:,ks))

         do n=1,nt

           TX(:,:,ks,n) = KMASKE*(
     &           eoshift(TRACER(:,:,k+1,n,mixtime),dim=1,shift=1)
     &                 - TRACER(:,:,k+1,n,mixtime))
           TY(:,:,ks,n) = KMASKN*(
     &            eoshift(TRACER(:,:,k+1,n,mixtime),dim=2,shift=1)
     &                 - TRACER(:,:,k+1,n,mixtime))

         end do

         !***
         !*** D_T(rho) & D_S(rho) at level k+1
         !***

         call state(k+1,k+1,TRACER(:,:,k+1,1,mixtime),
     &                      TRACER(:,:,k+1,2,mixtime),
     &                      DRHODT=DRDT, DRHODS=DRDS)

         RX  = DRDT*TXP(:,:,ks) + DRDS*TX(:,:,ks,2) ! Dx(rho)

         RXW = DRDT*eoshift(TXP(:,:,ks),dim=1,shift=-1)
     &       + DRDS*eoshift(TX(:,:,ks,2),dim=1,shift=-1)

         RY  = DRDT*TYP(:,:,ks) + DRDS*TY(:,:,ks,2) ! Dy(rho)

         RYS = DRDT*eoshift(TYP(:,:,ks),dim=2,shift=-1)
     &       + DRDS*eoshift(TY(:,:,ks,2),dim=2,shift=-1)

!-----------------------------------------------------------------------
!
!        compute slope of isopycnal surfaces  at level k+1
!
!-----------------------------------------------------------------------

         where (k+1 <= KMT)
            SLX (:,:,ieast, kbt2) = RX /RZ
            SLX (:,:,iwest, kbt2) = RXW /RZ
            SLY (:,:,jnorth,kbt2) = RY  /RZ
            SLY (:,:,jsouth,kbt2) = RYS /RZ
         end where

!-----------------------------------------------------------------------
!
!        compute diffusion coefficient KAPPA if required
!
!-----------------------------------------------------------------------

         if (kappa_type == kappa_type_var. and. k==1) then

               if(nsteps_total == 1) KSV = ah

               if(nsteps_run == 1. and. nsteps_total> 1)
     &              call diff_coeff

               if(compute_kappa) then
                  call diff_coeff
                  compute_kappa = .false.
               end if

               if(mod(nsteps_total,10) == 0 .or.
     &              time_to_do(restart_freq_iopt, restart_freq))
     &              compute_kappa = .true.

         endif           ! if (kappa_type == kappa_type_var. and. k==1)

!-----------------------------------------------------------------------
!
!        control slope of isopycnal surfaces or KAPPA
!
!-----------------------------------------------------------------------

         do kk = 2,3

           kid = k+kk-2

!-----------------------------------------------------------------------
!
!          control KAPPA to reduce the isopycanl mixing near the
!          ocean surface Large et al (1997), JPO, 27, pp 2418-2447.
!          WORK1 = ratio between the depth of water parcel and
!          the vertical displacement of isopycnal surfaces
!          where the vertical displacement =
!          Rossby radius * slope of isopycnal surfaces
!
!-----------------------------------------------------------------------

           SLA = dzw(kid)*sqrt(p5*(
     &        (SLX(:,:,1,kc(kk))**2+SLX(:,:,2,kc(kk))**2)/HUS**2
     &      + (SLY(:,:,1,kc(kk))**2+SLY(:,:,2,kc(kk))**2)/HUW**2))+eps

           if ( kid == 1 ) then
             dz_bottom = c0
           else
             dz_bottom = zt(kid-1)
           endif

           if (slope_control == slope_control_tanh) then

             WORK1 = min(c1,zt(kid)*RBR/SLA)
             TAPER1 = p5*(c1+sin(pi*(WORK1-p5)))

!	use the Rossby deformation radius tapering
!       only within the boundary layer

	     if ( vmix_itype == vmix_type_kpp ) then
	       TAPER1 = merge(TAPER1, c1, dz_bottom <= HBLT)
	     endif

           else
             !***
             !*** sine function is replaced by
             !*** function = 4.*x*(1.-abs(x)) for |x|<0.5
             !***

             WORK1 = min(c1,zt(kid)*RBR/SLA)
             TAPER1 = (p5+c2*(WORK1-p5)*(c1-abs(WORK1-p5)))

	     if ( vmix_itype == vmix_type_kpp ) then
	     	TAPER1 = merge(TAPER1, c1, dz_bottom <= HBLT)
	     endif

           endif

!-----------------------------------------------------------------------
!
!          control KAPPA for numerical stability
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!          methods to control slope
!
!-----------------------------------------------------------------------

           select case (slope_control)
           case (slope_control_tanh)
             !***  method by Danabasoglu & Mcwilliams (1995)

             TAPER2 = merge(p5*
     &          (c1-tanh(c10*SLA/slm_r-c4)), c0, SLA < slm_r)

             if(diff_tapering) then
                TAPER3 = merge(p5*
     &               (c1-tanh(c10*SLA/slm_b-c4)), c0, SLA < slm_b)
             endif

           case (slope_control_notanh)
             !*** similar to DM95 except replacing tanh by
             !*** function = x*(1.-0.25*abs(x)) for |x|<2
             !***          = sign(x)            for |x|>2
             !*** (faster than DM95)

             where (SLA > 0.2_dbl_kind*slm_r .and.
     &                SLA < 0.6_dbl_kind*slm_r)
                TAPER2 = p5*(c1-(2.5_dbl_kind*SLA/slm_r-c1)*
     &             (c4-abs(c10*SLA/slm_r-c4)))
             end where

             where (SLA >= 0.6_dbl_kind*slm_r)
                TAPER2 = c0
             end where

             if(diff_tapering) then
                where (SLA > 0.2_dbl_kind*slm_b .and.
     &               SLA < 0.6_dbl_kind*slm_b)
                TAPER3 = p5*(c1-(2.5_dbl_kind*SLA/slm_b-c1)*
     &               (c4-abs(c10*SLA/slm_b-c4)))
                end where
                where (SLA >= 0.6_dbl_kind*slm_b)
                  TAPER3 = c0
                end where
              endif

           case (slope_control_clip)
             !*** slope clipping

             do i=1,2
               where (abs(SLX(:,:,i,kc(kk))*dzw(kid)/HUS) > slm_r)
                 SLX(:,:,i,kc(kk))=
     &               sign(slm_r*HUS*dzwr(kid),SLX(:,:,i,kc(kk)))
               end where
             enddo

             do j=1,2
               where (abs(SLY(:,:,j,kc(kk))*dzw(kid)/HUW) > slm_r)
                SLY(:,:,j,kc(kk))=
     &               sign(slm_r*HUW*dzwr(kid),SLY(:,:,j,kc(kk)))
               end where
             enddo

           case (slope_control_Gerd)
             !*** method by Gerdes et al (1991)

             where (SLA > slm_r)
               TAPER2 = (slm_r/SLA)**2
             end where

             if(diff_tapering) then
                where (SLA > slm_b)
                   TAPER3 = (slm_b/SLA)**2
                end where
             endif

           end select
           
           if (kappa_type == kappa_type_var) then
              KAPPA(:,:,kc(kk)) = KSV*TAPER1*TAPER2*kappa_depth(kid)
              KAPPA_B(:,:,kc(kk)) = KAPPA(:,:,kc(kk))
              if(diff_tapering) then
                 KAPPA_B(:,:,kc(kk)) = KSV*TAPER1*TAPER3*
     &                kappa_depth(kid)
              else
                 if(ah /= ah_bolus) KAPPA_B(:,:,kc(kk)) = 
     &                KSV*TAPER1*TAPER2*kappa_depth(kid)
              endif
           else
              KAPPA(:,:,kc(kk)) = ah*TAPER1*TAPER2*kappa_depth(kid)
              KAPPA_B(:,:,kc(kk)) = KAPPA(:,:,kc(kk))
              if(diff_tapering) then
                 KAPPA_B(:,:,kc(kk)) = ah_bolus*TAPER1*TAPER3*
     &                kappa_depth(kid)
              else
                 if(ah /= ah_bolus) KAPPA_B(:,:,kc(kk)) = 
     &                ah_bolus*TAPER1*TAPER2*kappa_depth(kid)
              endif
           endif

         end do

!-----------------------------------------------------------------------
!
!        end of k < km if block
!
!-----------------------------------------------------------------------

      endif

!-----------------------------------------------------------------------
!
!     impose the boundary conditions by setting KAPPA=0
!     in the quarter cells adjacent to rigid boundaries
!
!-----------------------------------------------------------------------

      !***
      !*** B.C. at the top
      !***

      if (k == 1) then
         KAPPA(:,:,ktp) = c0
         KAPPA_B(:,:,ktp) = c0
         FZTOP = c0  ! zero flux B.C. at the surface
      end if

      !***
      !*** B.C. at the bottom
      !***

      where (k == KMT)
         KAPPA(:,:,kbt) = c0
         KAPPA(:,:,kbt2) = c0
         KAPPA_B(:,:,kbt)  = c0
         KAPPA_B(:,:,kbt2) = c0
      end where

#ifdef impvmix
!-----------------------------------------------------------------------
!
!     calculate effective vertical diffusion coefficient
!     NOTE: it is assumed that VDC has been set before this
!           in vmix_coeffs or something similar.
!
!     Dz(VDC * Dz(T)) where D is derivative rather than difference
!     VDC = (Az(dz*Ax(KAPPA*HYX*SLX**2)) + Az(dz*Ay(KAPPA*HXY*SLY**2)))*
!           dzw/TAREA
!
!-----------------------------------------------------------------------


      if (k < km) then

        WORK1 = dzw(k)*KMASK*TAREA_R*
     &         (dz(k  )*p25*KAPPA(:,:,kbt )*
     &         (HYX *SLX(:,:,ieast, kbt )**2
     &        + HYXW*SLX(:,:,iwest, kbt )**2
     &        + HXY *SLY(:,:,jnorth,kbt )**2
     &        + HXYS*SLY(:,:,jsouth,kbt )**2)
     &         +dz(k+1)*p25*KAPPA(:,:,kbt2)*
     &         (HYX *SLX(:,:,ieast, kbt2)**2
     &        + HYXW*SLX(:,:,iwest, kbt2)**2
     &        + HXY *SLY(:,:,jnorth,kbt2)**2
     &        + HXYS*SLY(:,:,jsouth,kbt2)**2))

        do n=1,size(VDC,DIM=4)
          VDC(:,:,k,n) = VDC(:,:,k,n) + WORK1
        end do

      end if
#else
      call exit_POP(
     &   'implicit vertical mixing must be used with GM horiz mixing')
#endif


!-----------------------------------------------------------------------
!
!     start loop over tracers
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!      add some horizontal background diffusion
!      for the bottom half of the cells next to the lower boundary
!
!-----------------------------------------------------------------------

      WORK3 = KAPPA(:,:,ktp) + KAPPA(:,:,kbt)
     &      + eoshift(KAPPA(:,:,ktp),dim=1,shift=1)
     &      + eoshift(KAPPA(:,:,kbt),dim=1,shift=1)

      WORK4 = KAPPA(:,:,ktp) + KAPPA(:,:,kbt)
     &      + eoshift(KAPPA(:,:,ktp),dim=2,shift=1)
     &      + eoshift(KAPPA(:,:,kbt),dim=2,shift=1)

      if(ah_bkg /= c0) then 
         WORK5 = merge(ah_bkg, c0, k == KMT) ! background diffusion 
         WORK6 = eoshift (WORK5,dim=1,shift=1)
         WORK7 = eoshift (WORK5,dim=2,shift=1)
         WORK3 = WORK3 + WORK5 + WORK6
         WORK4 = WORK4 + WORK5 + WORK7
      endif

      if ( ah /= ah_bolus .or. diff_tapering ) then
        WORK1 = KAPPA(:,:,ktp ) - KAPPA_B(:,:,ktp )
        WORK2 = KAPPA(:,:,kbt ) - KAPPA_B(:,:,kbt )
        WORK5 = KAPPA(:,:,kbt ) + KAPPA_B(:,:,kbt )
        WORK6 = KAPPA(:,:,kbt2) + KAPPA_B(:,:,kbt2)
      endif

      if ( gm_bolus ) then
        WORK1 = KAPPA(:,:,ktp )
        WORK2 = KAPPA(:,:,kbt )
        WORK5 = KAPPA(:,:,kbt )
        WORK6 = KAPPA(:,:,kbt2)  
      endif

      do n = 1,nt

!-----------------------------------------------------------------------
!
!        calculate horizontal fluxes thru vertical faces of T-cell
!        FX = dz*HYX*Ax(Az(KAPPA))*Dx(T) : flux in x-direction
!        FY = dz*HXY*Ay(Az(KAPPA))*Dy(T) : flux in y-direction
!
!-----------------------------------------------------------------------

         FX = CX*TX(:,:,kn,n)*WORK3
         FY = CY*TY(:,:,kn,n)*WORK4 

         if(ah /= ah_bolus .or. diff_tapering .or. gm_bolus) then

	  if (n > 2) TZ(:,:,ks,n) = TRACER(:,:,k  ,n,mixtime)
     &                            - TRACER(:,:,k+1,n,mixtime)

          FX = FX - CX*
     &         ( WORK1*SLX(:,:,ieast,ktp)*TZ(:,:,kn,n)
     &         + WORK2*SLX(:,:,ieast,kbt)*TZ(:,:,ks,n)
     &         + eoshift(WORK1
     &                *SLX(:,:,iwest,ktp)*TZ(:,:,kn,n),dim=1,shift=1)
     &         + eoshift(WORK2
     &                *SLX(:,:,iwest,kbt)*TZ(:,:,ks,n),dim=1,shift=1))

          FY = FY - CY*
     &         ( WORK1*SLY(:,:,jnorth,ktp)*TZ(:,:,kn,n)
     &         + WORK2*SLY(:,:,jnorth,kbt)*TZ(:,:,ks,n)
     &         + eoshift(WORK1
     &                *SLY(:,:,jsouth,ktp)*TZ(:,:,kn,n),dim=2,shift=1)
     &         + eoshift(WORK2
     &                *SLY(:,:,jsouth,kbt)*TZ(:,:,ks,n),dim=2,shift=1))

       endif
!-----------------------------------------------------------------------
!
!        calculate vertical fluxes thru horizontal faces of T-cell
!      - Az(dz*Ax(HYX*KAPPA*SLX*TX)) - Az(dz*Ay(HXY*KAPPA*SLY*TY))
!        calculate isopycnal diffusion from flux differences
!          DTK = (Dx(FX)+Dy(FY)+Dz(FZ)) / volume
!
!-----------------------------------------------------------------------

         if ( k < km ) then
           dz_bottom = dz(k+1)
         else
           dz_bottom = c0
         endif
            
         do j=jphys_b,jphys_e
         do i=iphys_b,iphys_e

           if(ah /= ah_bolus .or. diff_tapering .or. gm_bolus) then

           fz = -KMASK(i,j)*p25*
     &         (dz(k  )*WORK5(i,j)*
     &         (SLX(i,j,ieast ,kbt )*HYX(i  ,j)*TX(i  ,j,kn,n)
     &        + SLY(i,j,jnorth,kbt )*HXY(i  ,j)*TY(i  ,j,kn,n)
     &        + SLX(i,j,iwest ,kbt )*HYX(i-1,j)*TX(i-1,j,kn,n)
     &        + SLY(i,j,jsouth,kbt )*HXY(i,j-1)*TY(i,j-1,kn,n))
     &        + dz_bottom*WORK6(i,j)*
     &         (SLX(i,j,ieast ,kbt2)*HYX(i  ,j)*TX(i  ,j,ks,n)
     &        + SLY(i,j,jnorth,kbt2)*HXY(i  ,j)*TY(i  ,j,ks,n)
     &        + SLX(i,j,iwest ,kbt2)*HYX(i-1,j)*TX(i-1,j,ks,n)
     &        + SLY(i,j,jsouth,kbt2)*HXY(i,j-1)*TY(i,j-1,ks,n)))

           else

           fz = -KMASK(i,j)*p5*
     &         (dz(k  )*KAPPA(i,j,kbt )*
     &         (SLX(i,j,ieast ,kbt )*HYX(i  ,j)*TX(i  ,j,kn,n)
     &        + SLY(i,j,jnorth,kbt )*HXY(i  ,j)*TY(i  ,j,kn,n)
     &        + SLX(i,j,iwest ,kbt )*HYX(i-1,j)*TX(i-1,j,kn,n)
     &        + SLY(i,j,jsouth,kbt )*HXY(i,j-1)*TY(i,j-1,kn,n))
     &        + dz_bottom*KAPPA(i,j,kbt2)*
     &         (SLX(i,j,ieast ,kbt2)*HYX(i  ,j)*TX(i  ,j,ks,n)
     &        + SLY(i,j,jnorth,kbt2)*HXY(i  ,j)*TY(i  ,j,ks,n)
     &        + SLX(i,j,iwest ,kbt2)*HYX(i-1,j)*TX(i-1,j,ks,n)
     &        + SLY(i,j,jsouth,kbt2)*HXY(i,j-1)*TY(i,j-1,ks,n)))

           endif

           GTK(i,j,n) = (FX(i,j) - FX(i-1,j) + FY(i,j) - FY(i,j-1)
     &                + FZTOP(i,j,n) - fz)*dzr(k)*TAREA_R(i,j)

           FZTOP(i,j,n) = fz

         end do
         end do

!-----------------------------------------------------------------------
!
!        end of tracer loop
!
!-----------------------------------------------------------------------

      end do

      if ( gm_bolus ) then

        WORK1 = (        KAPPA_B(:,:,kbt)  * SLX(:,:,ieast,kbt)
     &         +         KAPPA_B(:,:,kbt2) * SLX(:,:,ieast,kbt2)
     &         + eoshift(KAPPA_B(:,:,kbt)  * SLX(:,:,iwest,kbt),
     &                   dim=1, shift=1)
     &         + eoshift(KAPPA_B(:,:,kbt2) * SLX(:,:,iwest,kbt2),
     &                   dim=1, shift=1) ) * p25 * HYX * dzw(k)

        WORK2 = (        KAPPA_B(:,:,kbt)  * SLY(:,:,jnorth,kbt)
     &         +         KAPPA_B(:,:,kbt2) * SLY(:,:,jnorth,kbt2)
     &         + eoshift(KAPPA_B(:,:,kbt)  * SLY(:,:,jsouth,kbt),
     &                   dim=2, shift=1)
     &         + eoshift(KAPPA_B(:,:,kbt2) * SLY(:,:,jsouth,kbt2),
     &                   dim=2, shift=1) ) * p25 * HXY * dzw(k)

        UIB = merge( WORK1, c0, k < KMT .and. k < KMTE )
        VIB = merge( WORK2, c0, k < KMT .and. k < KMTN )

        WORK1 = merge( UIT - UIB, c0, k <= KMT .and. k <= KMTE )
        WORK2 = merge( VIT - VIB, c0, k <= KMT .and. k <= KMTN )

        UISOP = WORK1 * dzr(k) / HTE
        VISOP = WORK2 * dzr(k) / HTN

        call w_shift ( WORK3, WORK1 )
        call s_shift ( WORK4, WORK2 )

        WISOPB = merge(WISOPT+(WORK1 - WORK3 + WORK2 - WORK4)*TAREA_R,
     &                 c0, k < KMT )

      endif

!-----------------------------------------------------------------------
!
!     update remaining bottom-face fields to top-face fields for next
!     pass
!
!-----------------------------------------------------------------------

      ktmp = kbt2
      kbt2 = kbt
      kbt  = ktp
      ktp  = ktmp

      ktmp = kn
      kn   = ks
      ks   = ktmp

      UIT = UIB
      VIT = VIB

!-----------------------------------------------------------------------
!
!     compute horiz diffusion cfl diagnostics if required
!
!-----------------------------------------------------------------------

      if (ldiag_cfl) then

        WORK1 = merge(c4*ah*(DXTR**2 + DYTR**2), c0, KMT > k)
        WORK2 = abs(WORK1)
        call cfl_hdiff(k,WORK2,1)

      endif



!-----------------------------------------------------------------------

      end subroutine hdifft_gm

!
!***********************************************************************
!
      subroutine diff_coeff

!-----------------------------------------------------------------------
!
!     variable kappa parametrization by Visbeck et al (1995):
!
!     kappa=KSV=constant*l**2/T
!     T = (Eady) time scale = f/sqrt(Ri)
!     l = Rossby radius
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  kp1,i,j,kk,k

      integer (kind=int_kind) ::
     &  k1,k2

      integer (kind=int_kind) ::
     &  mtime

      real (kind=dbl_kind) ::
     &  const
     &, zmin1, zmin2

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  UKT,VKT

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  UKPT, VKPT, RNUM

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  WORK1, WORK2

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  WORK3, WORK4

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  RHOT,RHOS

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  GRATE, LSC

!-----------------------------------------------------------------------
!     initialization
!-----------------------------------------------------------------------

      GRATE = c0
      LSC = c0
      k1 = 0
      k2 = 0
      mtime = oldtime

      vert_loop: do k=1,km

      !*** -1000m < z < -50m
      if(zt(k) >= 0.5e4_dbl_kind .and. zt(k) <= 1.0e5_dbl_kind) then

!-----------------------------------------------------------------------
!     compute the inverse of time scale
!-----------------------------------------------------------------------

         if (k1 == 0) then

            k1 = k              ! k1 is the upper limit for integration

!-----------------------------------------------------------------------
!     average velocities at T points of k=1
!-----------------------------------------------------------------------

            call sw_4pt(UKT,AT0,ATS,ATW,ATSW,UVEL(:,:,k,mtime))
            call sw_4pt(VKT,AT0,ATS,ATW,ATSW,VVEL(:,:,k,mtime))

         endif

!-----------------------------------------------------------------------
!     compute RZ=Dz(rho) at k=k
!-----------------------------------------------------------------------

         if (k < km) then
            kp1 = k + 1
         else
            kp1 = km
         endif

         call state(k,k,TRACER(:,:,k,1,mtime),
     &                  TRACER(:,:,k,2,mtime),
     &                  DRHODT=RHOT, DRHODS=RHOS)

         WORK1 = TRACER(:,:,k  ,1,mtime)
     &         - TRACER(:,:,kp1,1,mtime)
         WORK2 = TRACER(:,:,k  ,2,mtime)
     &         - TRACER(:,:,kp1,2,mtime)

         WORK3 = RHOT * WORK1 + RHOS * WORK2
         WORK3 = min(WORK3,-eps2)

!-----------------------------------------------------------------------
!        compute Dz(rho) at level k adiabatically displaced to k+1
!-----------------------------------------------------------------------

         call state(k,k+1,TRACER(:,:,k,1,mtime),
     &                    TRACER(:,:,k,2,mtime),
     &                    DRHODT=RHOT, DRHODS=RHOS)

         WORK4 = RHOT * WORK1 + RHOS * WORK2  ! Dz(rho)
         WORK4 = min(WORK4,-eps2)

!-----------------------------------------------------------------------
!     average velocities at T points of level k+1
!-----------------------------------------------------------------------

         call sw_4pt(UKPT,AT0,ATS,ATW,ATSW,UVEL(:,:,kp1,mtime))
         call sw_4pt(VKPT,AT0,ATS,ATW,ATSW,VVEL(:,:,kp1,mtime))

!-----------------------------------------------------------------------
!
!     local Richardson number at the bottom of T box, level k
!     = -g*Dz(RHO)/RHO_0/(Dz(u)**2+Dz(v)**2)
!
!-----------------------------------------------------------------------

         where (k < KMT)

         !*** RHO_0 = 1 in denominator (approximately) in cgs unit.

            RNUM =  -dzw(k)/
     &           ((UKT - UKPT)**2 + (VKT - VKPT)**2 + eps)

            GRATE = GRATE + p5*grav*RNUM*(dz(k)*WORK3 + dz(k+1)*WORK4)

	    LSC = LSC - p5*grav*(dz(k)*WORK3+dz(k+1)*WORK4)/dzw(k)

         end where

!-----------------------------------------------------------------------
!     bottom values become top values for next pass
!-----------------------------------------------------------------------

         UKT = UKPT
         VKT = VKPT

!-----------------------------------------------------------------------
!     end of computing the inverse of time scale
!-----------------------------------------------------------------------

      else

         if(k1.ne.0.and.k2.eq.0) then
            k2 = k              ! k2 is the lower limit for integration
            exit vert_loop
         end if

      endif                     ! if(zt(k).ge.0.5e4.and.zt(k).lt.1.0e5)

      end do vert_loop

      do j=1,jmt
      do i=1,imt
        kk=KMT(i,j)
        zmin1 = min(zt(k1),zt(kk))
        zmin2 = min(zt(k2),zt(kk))
        GRATE(i,j) = GRATE(i,j)/(zmin2-zmin1+eps) ! Ri
        LSC(i,j) = LSC(i,j)*1.e10_dbl_kind/(zmin2-zmin1+eps) ! c_g^2=N^2*H^2
      end do
      end do

!     equatorial inverse of time scale and square of length scale
      WORK1 = sqrt(c2*sqrt(LSC)*BTP) ! sqrt(c_g*2*beta)
      WORK2 = sqrt(LSC)/(c2*BTP) ! c_g/(2 beta)

!     inverse of time scale
      WORK1 = max(abs(FCORT),WORK1)
      GRATE = WORK1/sqrt(GRATE+eps) ! 1/T = f/sqrt(Ri)

!     square of length sclae
      LSC = LSC/(FCORT+eps)**2  ! L^2 = c_g^2/f^2
      LSC = min(LSC,WORK2)

!     grid size = lower limit of length scale
      WORK1 = min(DXT**2,DYT**2)
      LSC = max(LSC,WORK1)

!-----------------------------------------------------------------------
!     compute KAPPA
!-----------------------------------------------------------------------

!     const = 0.015_dbl_kind ! constant taken from Visbeck et al
      const = 0.13_dbl_kind   ! constant taken from Visbeck et al
      WORK1 = const*GRATE*LSC ! const*L**2/T

!     KAPPA is bounded by 3.0e6 < KAPPA < 2.0e7

      KSV = min(2.0e7_dbl_kind,WORK1)
      KSV = max(3.0e6_dbl_kind,KSV)

      where (KMT < k1)       ! KAPPA=3.0e6 when depth < 50m
        KSV = 3.0e6_dbl_kind
      end where

!-----------------------------------------------------------------------

      end subroutine diff_coeff

!***********************************************************************

      end module hmix_gm

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||


