!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module hmix_smag

!***********************************************************************
!
!     this module contains routines necessary for horizontal diffusion
!     using a Smagorinsky parameterization
!
!     CVS:$Id: hmix_smag.F,v 1.2.2.2 2001/07/05 22:43:26 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use stencils
      use operators
      use grid

      implicit none
      save

!-----------------------------------------------------------------------
!
!     these are the smagorinsky diffusion coefficients at level k
!
!-----------------------------------------------------------------------


      real (kind=dbl_kind), dimension(:,:), allocatable :: 
     &  ATX,ATY

!-----------------------------------------------------------------------
!
!     Arrays used in calculating the stencil coefficients for 
!     Smagorinsky viscosity subgridscale model.
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(:,:), allocatable :: 
     &  DTTNY, DTTSY, DTTEX, DTTWX,
     &  DXUEX, DXUWX, DXUNY, DXUSY,
     &  DXUCX, DXUCY, DXVEX, DXVNX,
     &  DYVCX, DYVCY, DYVWX, DYVEX,
     &  DYVSY, DYVNY, DYUNX, DYUEX,
     &  DXVCX, DYUCY, DXVCY,
     &  R2DXUBAR, R2DYUBAR,
     &  R2DXTBAR, R2DYTBAR

!-----------------------------------------------------------------------
!
!     AD{0,N,W,NW} = {central,n,w,nw} coefficients for area-weighted
!       averaging of four Strain Rate points.  Only defined when the
!       Smagorinsky viscosity is in use.
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(:,:), allocatable :: 
     &  AD0,ADN,ADW,ADNW

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_smag

!-----------------------------------------------------------------------
!
!     allocates and initializes various coefficient and stencil arrays
!
!     K{X,Y}U = k{x,y} coeficients for advective metric terms
!
!     D{X,Y,T}{U,V,T}{N,S,E,W}{X,Y} = coeffs for velocity stencils
!       NOTE: these are NOT the full stencil coefficients.  They
!             need to be multiplied by either amx/atx or amy/aty 
!             (this will be done in subroutine HDIFF) and in some 
!             cases will have terms added that result from gradients 
!             of amx/atx and amy/aty.These arrays simply hold 
!             quantities that are used every cycle but are constant.
!
!     The naming convention is as follows:
!        
!     D         -> denotes this array as pertaining to diffusion
!     {X,Y,T}   -> denotes whether this array is associated with 
!                  X or Y momentum equation or Tracers
!     {U,V,T}   -> denotes whether this array is associated with a 
!                  stencil acting on U, V or T
!     {N,S,E,W} -> denotes position of the coefficient in the stencil
!     {X,Y}     -> denotes whether this array is multiplied by amx/atx
!                  or amy/aty when constructing the full stencil 
!                  coefficient
!
!     For example, DXUEX is necessary for calculating the 
!     Eastern Diffusion stencil coefficient
!     acting on U in the X-momentum equation and will be multiplied
!     by amX.  Simple, is it not?
!
!     Note that not all possible permutations of the above are defined:
!     only 28 are and 23 of them can be found in cf_del_smag.h.
!     (The other 5 are calculated every cycle)
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     local variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  DXKX,DYKY,DXKY,DYKX,   ! D{X,Y}K{X,Y} = d{x,y}k{x,y}
     &  WORK1,WORK2            ! local work space

!-----------------------------------------------------------------------
!
!     allocate necessary arrays
!
!-----------------------------------------------------------------------


      allocate(ATX  (imt,jmt), ATY  (imt,jmt),
     &         DTTNY(imt,jmt), DTTSY(imt,jmt), 
     &         DTTEX(imt,jmt), DTTWX(imt,jmt),
     &         DXUEX(imt,jmt), DXUWX(imt,jmt), 
     &         DXUNY(imt,jmt), DXUSY(imt,jmt),
     &         DXUCX(imt,jmt), DXUCY(imt,jmt), 
     &         DXVEX(imt,jmt), DXVNX(imt,jmt),
     &         DYVCX(imt,jmt), DYVCY(imt,jmt), 
     &         DYVWX(imt,jmt), DYVEX(imt,jmt),
     &         DYVSY(imt,jmt), DYVNY(imt,jmt), 
     &         DYUNX(imt,jmt), DYUEX(imt,jmt),
     &         DXVCX(imt,jmt), DYUCY(imt,jmt), DXVCY(imt,jmt),
     &         R2DXUBAR(imt,jmt), R2DYUBAR(imt,jmt),
     &         R2DXTBAR(imt,jmt), R2DYTBAR(imt,jmt),
     &         AD0(imt,jmt), ADN(imt,jmt),ADW(imt,jmt),ADNW(imt,jmt))

!-----------------------------------------------------------------------
!
!     calculate {N,S,E,W} arrays needed for the tracer
!     fields (for tracers, the central coefficient is calculated as
!     minus the sum of these after boundary conditions are applied).
!
!-----------------------------------------------------------------------

      WORK1 = HTN/HUW
      DTTNY = WORK1*TAREA_R
      call s_shift(DTTSY,WORK1)
      DTTSY = DTTSY*TAREA_R

      WORK1 = HTE/HUS
      DTTEX = WORK1*TAREA_R
      call w_shift(DTTWX,WORK1)
      DTTWX = DTTWX*TAREA_R

!-----------------------------------------------------------------------
!
!     calculate the metric advection terms (KXU,KYU) and auxiliary
!     arrays needed for the the rest of the stencil arrays
!
!-----------------------------------------------------------------------

      call e_shift(KXU,HUW)
      KXU = (KXU - HUW)*UAREA_R
      call n_shift(KYU,HUS)
      KYU = (KYU - HUS)*UAREA_R

      call w_shift(WORK1,HTE)
      WORK1 = (HTE - WORK1)*TAREA_R

      call e_shift(DXKX,WORK1)
      DXKX = DXKX - WORK1
      call n_shift(WORK2,DXKX)
      DXKX = p5*(DXKX + WORK2)*DXUR

      call n_shift(DYKX,WORK1)
      DYKX = DYKX - WORK1
      call e_shift(WORK2,DXKX)
      DYKX = p5*(DYKX + WORK2)*DYUR

      call s_shift(WORK1,HTN)
      WORK1 = (HTN - WORK1)*TAREA_R

      call n_shift(DYKY,WORK1)
      DYKY = DYKY - WORK1
      call e_shift(WORK2,DYKY)
      DYKY = p5*(DYKY + WORK2)*DYUR

      call e_shift(DXKY,WORK1)
      DXKY = DXKY - WORK1
      call n_shift(WORK2,DXKY)
      DXKY = p5*(DXKY + WORK2)*DXUR

!-----------------------------------------------------------------------
!
!     the following are inverse lengths from center differences
!     that will be used when calculating the full stencil coefficients
!
!-----------------------------------------------------------------------

      call e_shift(WORK1,HTN)
      R2DXUBAR = c1/(HTN + WORK1)
      call n_shift(WORK1,HTE)
      R2DYUBAR = c1/(HTE + WORK1)

      call w_shift(WORK1,HUS)
      R2DXTBAR = c1/(HUS + WORK1)
      call s_shift(WORK1,HUW)
      R2DYTBAR = c1/(HUW + WORK1)

!-----------------------------------------------------------------------
!
!     calculate arrays associated with {N,S,E,W} stencil coefficients  
!     acting on momentum. 
!
!-----------------------------------------------------------------------

      WORK1 = HUS/HTE
      DXUSY = WORK1*UAREA_R
      call n_shift(DXUNY,WORK1)
      DXUNY = DXUNY*UAREA_R

      WORK1 = HUW/HTN
      DYVWX = WORK1*UAREA_R
      call e_shift(DYVEX,WORK1)
      DYVEX = DYVEX*UAREA_R

      DXUWX = R2DXUBAR*(c4/HTN - KXU)
      call e_shift(WORK1,HTN)  
      DXUEX = R2DXUBAR*(c4/WORK1 + KXU)

      DYVSY = R2DYUBAR*(c4/HTE - KYU)
      call n_shift(WORK1,HTE)  
      DYVNY = R2DYUBAR*(c4/WORK1 + KYU)

      DXUCX = -DXUEX - DXUWX - KXU*KXU
      DYVCY = -DYVNY - DYVSY - KYU*KYU

      WORK1 = -DXUEX + DYVEX  !  DXUEY
      WORK2 = -DXUWX + DYVWX  !  DXUWY
      DXUCY = -WORK1 - WORK2 - DXUNY   - DXUSY 
     &        -DXKX  - DYKY  - KXU*KXU - c2*KYU*KYU

      WORK1 = -DYVNY + DXUNY  !  DXVNX
      WORK2 = -DYVSY + DXUSY  !  DYVSX
      DYVCX = -DYVEX - DYVWX - WORK1   - WORK2
     &        -DYKY  - DXKX  - KYU*KYU - c2*KXU*KXU

      DXVEX = c2*KYU*R2DXUBAR
      DYUNX = c2*KXU*R2DYUBAR
      DXVNX = -KXU*R2DYUBAR
      DYUEX = -KYU*R2DXUBAR

      DXVCX = c2*DXKY + KXU*KYU
      DYUCY = c2*DYKX + KXU*KYU
      DXVCY = -DXKY - DYKX - KXU*KYU
      !***  ..note that DXUCX = DXVCY

!-----------------------------------------------------------------------
!
!     calculate {central,n,w,nw} coefficients for area-averaging
!     total strain rate (located at east face of T-cell) to north
!     edge of T-cell.
!
!-----------------------------------------------------------------------

      AD0 = HTE*HUS            !  area of D-cell
      call n_shift(ADN,AD0)
      call w_shift(ADW,AD0)
      call n_shift(ADNW,ADW)

      WORK1= HTN*HUW 
      AD0  = AD0 *p25/WORK1
      ADN  = ADN *p25/WORK1
      ADW  = ADW *p25/WORK1
      ADNW = ADNW*p25/WORK1

!-----------------------------------------------------------------------

      end subroutine init_smag

!***********************************************************************

      subroutine hdiffu_smag(k,HDUK,HDVK)

!-----------------------------------------------------------------------
!
!     Horizontial diffusion of momentum using Smagorinsky viscosity.
!
!     Hdiff(U) = amy*{(1/dy)*Dx[dy*Dx(U)]
!                    +(1/dx)*Dy[dx*Dy(U)]}
!              + (amx-amy)*{2*Dx[Dx(U)] + kx*Dx(U)}
!              + 2*amx*ky*Dx(V) - (amx+amy)*kx*Dy(V)
!              - U*[amx*kx**2 + amy*(dxkx+dyky+kx**2+2*ky**2)]  
!              + V*[amx*(2*dxky+kx*ky) - amy*(dxky+dykx+kx*ky)]
!              + [Dx(U) - Dy(V) + V*ky - U*kx]*Dx(amx)
!              + [Dy(U) + Dx(V) - U*ky - V*kx]*Dy(amy)
!
!     Hdiff(V) = amx*{(1/dy)*Dx[dy*Dx(V)]
!                    +(1/dx)*Dy[dx*Dy(V)]}
!              + (amy-amx)*{2*Dy[Dy(V)] + ky*Dy(V)}
!              + 2*amy*kx*Dy(U) - (amy+amx)*ky*Dx(U)
!              - V*[amy*ky**2 + amx*(dyky+dxkx+ky**2+2*kx**2)]  
!              + U*[amy*(2*dykx+ky*kx) - amx*(dykx+dxky+kx*ky)]
!              - [Dx(U) - Dy(V) + V*ky - U*kx]*Dy(amy)
!              + [Dy(U) + Dx(V) - U*ky - V*kx]*Dx(amx)
!
!     amx = x-direction Smagorinsky viscosity
!     amy = y-direction Smagorinsky viscosity
!
!     kx = (1/dy)*Dx(dy)
!     ky = (1/dx)*Dy(dx)
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k   ! depth level index

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &  HDUK,               ! Hdiff(U) at level k
     &  HDVK                ! Hdiff(V) at level k

!-----------------------------------------------------------------------
!
!     local or common variables:
!
!       dtotal= sqrt(2*(dtens**2 + dshear**2)) is the total deformation
!                                              (calculated in hdifft_smag)
!       amx = cx*dtotal
!       amy = cy*dtotal
!
!       cx = (1/2)*(c*dx)**2
!       cy = (1/2)*(c*dy)**2
!       c  = 0.14
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  AMX,AMY,
     &  DAXDX,DAYDY,
     &  WORK0,WORK1,WORK2,WORK3,WORK4,WORK5 ! local work space

!-----------------------------------------------------------------------


!-----------------------------------------------------------------------
!
!   Calculate the viscosity coefficients amx and amy.  Amx is located
!   at the east face of the U cell which is the same as the east_shifted
!   aty.  Amy is located on the north face of the U cell which
!   is the same as the north-shifted atx.  Thus we need to renormalize
!   atx and aty to reflect the appropriate U cell length scales, 
!   (ie, different cx and cy coefficients) then shift to get amx and amy.
!
!-----------------------------------------------------------------------

      call e_shift(WORK1,HTN)
      call n_shift(WORK2,HTE)
      WORK3 = ATY*WORK1*WORK1/(HUW*HUW)
      WORK4 = ATX*WORK2*WORK2/(HUS*HUS)

      call e_shift(AMX,WORK3)
      call n_shift(AMY,WORK4)

!-----------------------------------------------------------------------
!
!   Now calculate Dx(amx) and Dy(amy).
!
!-----------------------------------------------------------------------

      call w_shift(WORK1,AMX)
      call s_shift(WORK2,AMY)
      DAXDX = (AMX - WORK1)*c2*R2DXUBAR
      DAYDY = (AMY - WORK2)*c2*R2DYUBAR

!-----------------------------------------------------------------------
!
!   Now put it all together to calculate hdiff (see cf_del_smag.F for
!   an explanation of the stencil coefficients).
!
!-----------------------------------------------------------------------

      !***
      !*** First U momentum terms involving U
      !***

      WORK1 = AMY*DXUNY + DAYDY*R2DYUBAR                     ! north
      WORK2 = AMY*DXUSY - DAYDY*R2DYUBAR                     ! south
      WORK3 = AMX*DXUEX + AMY*(DYVEX-DXUEX) + DAXDX*R2DXUBAR ! east
      WORK4 = AMX*DXUWX + AMY*(DYVWX-DXUWX) - DAXDX*R2DXUBAR ! west
      WORK0 = AMX*DXUCX + AMY*DXUCY - DAXDX*KXU - DAYDY*KYU  ! central
      call fivept_nobndy(WORK5,WORK0,WORK1,WORK2,WORK3,WORK4,
     &                   UVEL(:,:,k,curtime))

      !***
      !*** Next, U momentum terms involving V
      !***

      WORK1 = AMX*DXVNX + AMY*DXVNX - DAXDX*R2DYUBAR         ! north
      WORK2 = -WORK1                                         ! south
      WORK3 = AMX*DXVEX + DAYDY*R2DXUBAR                     ! east
      WORK4 = -WORK3                                         ! west
      WORK0 = AMX*DXVCX + AMY*DXVCY + DAXDX*KYU - DAYDY*KXU  ! central
      call fivept_nobndy(HDUK,WORK0,WORK1,WORK2,WORK3,WORK4,
     &                   VVEL(:,:,k,curtime))

      HDUK = HDUK + WORK5

      !***
      !*** Next, V momentum terms involving V
      !***

      WORK1 = AMX*(DXUNY-DYVNY) + AMY*DYVNY + DAYDY*R2DYUBAR ! north
      WORK2 = AMX*(DXUSY-DYVSY) + AMY*DYVSY - DAYDY*R2DYUBAR ! south
      WORK3 = AMX*DYVEX + DAXDX*R2DXUBAR                     ! east
      WORK4 = AMX*DYVWX - DAXDX*R2DXUBAR                     ! west
      WORK0 = AMX*DYVCX + AMY*DYVCY - DAXDX*KXU - DAYDY*KYU  ! central
      call fivept_nobndy(WORK5,WORK0,WORK1,WORK2,WORK3,WORK4,
     &                   VVEL(:,:,k,curtime))

      !***
      !*** Next, V momentum terms involving U
      !***

      WORK1 = AMX*DYUNX + DAXDX*R2DYUBAR                     ! north
      WORK2 = -WORK1                                         ! south
      WORK3 = AMX*DYUEX + AMY*DYUEX - DAYDY*R2DXUBAR         ! east
      WORK4 = -WORK3                                         ! west

      !***
      !*** use DXVCY for DYUCX
      !***

      WORK0 = AMX*DXVCY + AMY*DYUCY - DAXDX*KYU + DAYDY*KXU  ! central
      call fivept_nobndy(HDVK,WORK0,WORK1,WORK2,WORK3,WORK4,
     &                   UVEL(:,:,k,curtime))

      HDVK = HDVK + WORK5


!-----------------------------------------------------------------------

      end subroutine hdiffu_smag

!***********************************************************************

      subroutine hdifft_smag(k,HDTK)

!-----------------------------------------------------------------------
!     Horizontial diffusion of tracers using Smagorinsky diffusion.
!
!     equations:
!
!     Hdiff(T) = atx*(1/dy)*Dx[dy*Dx(T)]
!              + aty*(1/dx)*Dy[dx*Dy(T)]
!              + Dx(T)*Dx(atx) + Dy(T)*Dy(aty)
!
!     atx = x-direction Smagorinsky diffusion
!     aty = y-direction Smagorinsky diffusion
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k   ! depth level index

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,nt), intent(out) :: 
     &  HDTK                ! Hdiff(T) for tracer n at level k

!-----------------------------------------------------------------------
!
!     local or common variables:
!
!       dtens = Dx(U) - Dy(V) - kx*U + ky*V  is the tension strain rate
!       dshear= Dx(V) + Dy(U) - ky*U - kx*V  is the shearing strain rate
!       dtotal= sqrt(2*(dtens**2 + dshear**2)) is the total deformation
!
!       kx = (1/dy)*Dx(dy)
!       ky = (1/dx)*Dy(dx)
!
!       atx = cx*dtotal
!       aty = cy*dtotal
!
!       cx = (1/2)*(c*dx)**2
!       cy = (1/2)*(c*dy)**2
!       c  = 0.14
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n   ! dummy tracer index

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  DTENS,DSHEAR,
     &  WORK1,WORK2,WORK3,WORK4   ! local work space

!-----------------------------------------------------------------------


!-----------------------------------------------------------------------
!
!   First calculate the tension and shearing rates of strain.
!   The strain rates are located on cell edges, east of T and south
!   of U,V.  Derivatives wrt X are center differences, while derivatives
!   wrt Y are backward differences.  Quantities which are located at U,V
!   points (ie, everything except derivatives wrt Y) are averaged over Y
!   to get the cell sided value.
!
!-----------------------------------------------------------------------

      call e_shift(WORK1,UVEL(:,:,k,mixtime))
      call w_shift(WORK2,UVEL(:,:,k,mixtime))
      call s_shift(WORK3,VVEL(:,:,k,mixtime))

      DTENS = (WORK1 - WORK2)*R2DXUBAR - 
     &        KXU*UVEL(:,:,k,mixtime) + KYU*VVEL(:,:,k,mixtime)
      call s_shift(WORK1,DTENS)
      DTENS = p5*(DTENS + WORK1) - 
     &        (VVEL(:,:,k,mixtime) - WORK3)*c2*R2DYUBAR 

      call e_shift(WORK1,VVEL(:,:,k,mixtime))
      call w_shift(WORK2,VVEL(:,:,k,mixtime))
      call s_shift(WORK3,UVEL(:,:,k,mixtime))

      DSHEAR = (WORK1 - WORK2)*R2DXUBAR - 
     &         KXU*VVEL(:,:,k,mixtime) - KYU*UVEL(:,:,k,mixtime)
      call s_shift(WORK1,DSHEAR)
      DSHEAR = p5*(DSHEAR + WORK1) - 
     &         (UVEL(:,:,k,mixtime) - WORK3)*c2*R2DYUBAR

!-----------------------------------------------------------------------
!
!   Next calculate the magnitude of the total deformation.
!   The cx and cy coefficients are proportional to the local grid
!   length--here we use the length that is centered about the location
!   of the diffusion coefficient.
!
!-----------------------------------------------------------------------

      WORK1 = sqrt(c2*(DTENS*DTENS + DSHEAR*DSHEAR))  !  DTOTAL

      WORK3 = p5*(0.14_dbl_kind*HUS)**2                        !  CX
      WORK4 = p5*(0.14_dbl_kind*HUW)**2                        !  CY

!-----------------------------------------------------------------------
!
!   Now calculate the diffusion coefficients atx and aty.  Atx is 
!   located at the same place as the deformation;  aty is located at 
!   the north edge of the T cell so we need to average the total 
!   deformation for aty.
!
!-----------------------------------------------------------------------

      call nw_4pt(WORK2,AD0,ADN,ADW,ADNW,WORK1)

      ATX = WORK3*WORK1
      ATY = WORK4*WORK2

!-----------------------------------------------------------------------
!
!     Now calculate Dx(atx) and Dy(aty).
!     NOTE:  these should maybe be set to zero at the coast
!
!-----------------------------------------------------------------------

      call w_shift(WORK1,ATX)
      call s_shift(WORK2,ATY)
      WORK3 = (ATX - WORK1)*c2*R2DXTBAR
      WORK4 = (ATY - WORK2)*c2*R2DYTBAR

!-----------------------------------------------------------------------
!
!     Now put it all together to calculate hdiff (see init_smag above 
!     an explanation of the stencil coefficients).
!     NOTE:  the ATX,ATY here should probably be averages since they are
!            not cell centered
!
!-----------------------------------------------------------------------

      WORK1 = ATX*DTTEX + WORK3*R2DXTBAR                 ! east
      WORK2 = ATX*DTTWX - WORK3*R2DXTBAR                 ! west
      WORK3 = ATY*DTTNY + WORK4*R2DYTBAR                 ! north
      WORK4 = ATY*DTTSY - WORK4*R2DYTBAR                 ! south
      DTENS = -WORK1 - WORK2 - WORK3 - WORK4             ! central

      do n = 1,nt
       call fivept_nobndy(HDTK(:,:,n),DTENS,WORK3,WORK4,WORK1,WORK2,
     &                    TRACER(:,:,k,n,mixtime))
      enddo

!-----------------------------------------------------------------------


!-----------------------------------------------------------------------

      end subroutine hdifft_smag

!***********************************************************************

      end module hmix_smag

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
