!***********************************************************************

      module ice

!-----------------------------------------------------------------------
!
!     this module currently contains routines for computing ice 
!     formation and the heat flux associated with ice formation.
!     this heat flux is sent to the ice model via the flux coupler.
!     in the future, this module could contain the driver for a
!     subroutinized ice model.
!
!     CVS:$Id: ice.F,v 1.2.2.10 2002/02/22 22:54:06 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use domain
      use constants
      use prognostic
      use io
      use time_management
      use grid
      use exit_mod
      use qflux_mod

      implicit none
      save

!-----------------------------------------------------------------------
!
!     module variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  kmxice                ! lowest level from which to integrate 
                              ! ice formation

      integer (kind=int_kind), private ::
     &  ice_flag              ! time flag id for ice formation
     &, ice_cpl_flag          ! time flag id for coupled timestep

      logical (kind=log_kind) ::
     &  liceform              ! flag to turn on/off ice formation

      real (kind=dbl_kind), parameter ::
     &  cp_over_lhfusion = rho_sw*cp_sw/(latent_heat_fusion*rho_fw)

      real (kind=dbl_kind) ::
     &  salice,               ! sea ice salinity in msu
     &  salref,               ! ocean ref salinity in msu
     &  tlast_ice             ! time since last ice flux computed

      real (kind=dbl_kind), dimension(:,:), allocatable ::
     &  QICE,               ! tot column cooling from ice form (in C*cm)
     &  AQICE,              ! accumulated ice heat flux
     &  QFLUX               ! ice formation / melt heat flux in  W/m^2 


!-----------------------------------------------------------------------
!
!     lactive_ice = .T.     ocean is coupled to an active ice model,
!                   .F.     ocean is coupled to a dummy ice model.
!
!-----------------------------------------------------------------------

      logical (kind=log_kind) :: lactive_ice

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_ice

!-----------------------------------------------------------------------
!
!     this routine initializes ice variables.  it must be called
!     before initializing restarts because turning the ice on adds
!     an additional 2-d field to the restart file.
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: nml_error  ! namelist i/o error flag

      character (char_len) ::
     &  ice_freq_opt        ! option for frequency of computing ice

      integer (kind=int_kind) ::
     &  ice_freq_iopt       ! int option for freq units
     &, ice_freq            ! freq for computing ice in above units

      namelist /ice_nml/ kmxice, ice_freq_opt, ice_freq, lactive_ice

!-----------------------------------------------------------------------
!
!     read input namelists
!
!-----------------------------------------------------------------------

      kmxice           = 1
      ice_freq_opt     = 'never'
      ice_freq         = 100000
      lactive_ice      = .false.

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=ice_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue

        !***
        !*** define salice and salref in msu
        !***
 
        salice = sea_ice_salinity*ppt_to_salt
        salref = ocn_ref_salinity*ppt_to_salt

        liceform = .true.

        select case (ice_freq_opt)
        case ('never')
          liceform = .false.
          ice_freq_iopt = freq_opt_never
        case ('coupled')
          ice_freq_iopt = freq_opt_never ! check coupler flag instead
        case ('nyear')
          ice_freq_iopt = freq_opt_nyear
        case ('nmonth')
          ice_freq_iopt = freq_opt_nmonth
        case ('nday')
          ice_freq_iopt = freq_opt_nday
        case ('nhour')
          ice_freq_iopt = freq_opt_nhour
        case ('nsecond')
          ice_freq_iopt = freq_opt_nsecond
        case ('nstep')
          ice_freq_iopt = freq_opt_nstep
        case default
          ice_freq_iopt = -1000
        end select

      endif

      call broadcast_scalar(nml_error    , master_task)

      if (nml_error /= 0) then
        call exit_POP('ERROR reading ice_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, ice_nml)
          write(stdout,*) ' '
      endif


      call broadcast_scalar(ice_freq_iopt, master_task)

      if (ice_freq_iopt == -1000) then
        call exit_POP('unknown restart frequency option')
      endif

      call broadcast_scalar(liceform,      master_task)
      call broadcast_scalar(ice_freq,      master_task)
      call broadcast_scalar(lactive_ice,   master_task)

!-----------------------------------------------------------------------
!
!     if ice turned on, broadcast remaining vars and allocate memory
!
!-----------------------------------------------------------------------

      if (liceform) then
        call broadcast_scalar(kmxice,   master_task)
        call broadcast_scalar(salice,   master_task)
        call broadcast_scalar(salref,   master_task)
 
        !***
        !*** set up time flag
        !***

        ice_flag = init_time_flag('ice',default=.false.,
     &                            freq_opt = ice_freq_iopt,
     &                            freq     = ice_freq)

        ice_cpl_flag = init_time_flag('coupled_ts')

        !***
        !*** must keep track of time since last ice flux computed
        !***

        tlast_ice = c0

        !***
        !*** allocate and initialize ice flux arrays
        !***

        allocate( QICE(imt,jmt), AQICE(imt,jmt),QFLUX(imt,jmt))

        QICE  = c0
        AQICE = c0
        QFLUX = c0

      endif

!-----------------------------------------------------------------------

      end subroutine init_ice

!***********************************************************************

      subroutine ice_formation(SHF_IN, lfw_as_salt_flx)

!-----------------------------------------------------------------------
!
!     this subroutine computes ocean heat flux to the sea-ice. it forms
!     the necessary ice in the ocean and adjusts the potential 
!     temperature and salinity fields accordingly. the logic of this 
!     subroutine is based on William Large''s 1-d model and is based
!     on a version from the NCOM model written by Gokhan Danabasoglu. 
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) ::
     &  SHF_IN              ! surface heat flux

      logical (kind=log_kind), intent(in) ::
     &  lfw_as_salt_flx     ! treat fw flux as virtual salt flux
                            ! even with var.thickness sfc layer

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  k                   ! vertical level index

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  POTICE,             ! potential amt of ice formation
     &  WORK1               ! work array

      real (kind=dbl_kind) :: time_weight

!-----------------------------------------------------------------------
!
!     increment time since last evaluation
!     exit if this is not time to compute ice formation
!
!-----------------------------------------------------------------------

      if (avg_ts .or. back_to_back) then
        time_weight = p5 
      else
        time_weight = c1
      endif
      tlast_ice = tlast_ice + time_weight * dtt

!-----------------------------------------------------------------------
!
!     presently, the check_time_flag doesn't produce the same results
!     as ice_ts.  for now, ice_ts will be set in time_management and
!     used here
!-----------------------------------------------------------------------
!=====if (check_time_flag(ice_flag) .or.
!====&    check_time_flag(ice_cpl_flag)) then
 
!========================
      if ( ice_ts ) then
!========================

!-----------------------------------------------------------------------
!
!     initialize flux to zero
!
!-----------------------------------------------------------------------
 
        QICE = c0
        POTICE = c0
 
!-----------------------------------------------------------------------
!
!     compute frazil ice formation for sub-surface layers. if ice
!     forms in lower layers but layers above are warm - the heat is
!     used to melt the ice. the ice formation occurs at salinity, Si.
!     this volume is replaced with an equal volume at the salinity of
!     the layer above. the total ice heat flux is accumulated. 
!
!     WARNING: unless a monotone advection scheme is in place, 
!     advective errors could lead to temps that are far below freezing
!     in some locations and this scheme will form lots of ice.
!     ice formation should be limited to the top layer (kmxice=1)
!     if the advection scheme is not monotone.
!
!-----------------------------------------------------------------------
 
        do k=kmxice,2,-1

        !***
        !*** potice is the potential amount of ice formation 
        !*** (potice>0) or melting (potice<0) in layer k
        !***

          where (k <= KMT)
            POTICE = (TFRZ(TRACER(:,:,k,2,newtime))  
     &                   - TRACER(:,:,k,1,newtime)) * dz(k)
          endwhere

        !***
        !*** if potice < 0, use the heat to melt any ice
        !*** from lower layers
        !*** if potice > 0, keep on freezing (QICE < 0)
        !***

          POTICE = max(POTICE, QICE)

        !***
        !*** adjust tracer values based on freeze/melt
        !***

          TRACER(:,:,k,1,newtime) = TRACER(:,:,k,1,newtime) 
     &                             + POTICE / dz(k)  

          if ( sfc_layer_type == sfc_layer_varthick .and.
     &         .not. lfw_as_salt_flx ) then
            TRACER(:,:,k,2,newtime) = ( TRACER(:,:,k,2,newtime)
     &         * (dz(k) + cp_over_lhfusion * QICE)
     &         + cp_over_lhfusion * (TRACER(:,:,k-1,2,newtime)
     &         * (POTICE - QICE) - salice * POTICE) ) / dz(k)
          else
            TRACER(:,:,k,2,newtime) = TRACER(:,:,k,2,newtime)
     &         + (salref - salice) * POTICE * cp_over_lhfusion / dz(k)
          endif

          QICE = QICE - POTICE  ! accumulate freezing potential

        enddo

!-----------------------------------------------------------------------
!
!     now repeat the above algorithm for the surface layer. when fresh
!     water flux formulation is used, the surface layer does not get
!     any salt from other layers. instead, its volume changes. 
!
!-----------------------------------------------------------------------

        k = 1
      
        WORK1 = dz(k)
        if ( sfc_layer_type == sfc_layer_varthick )
     &    WORK1 = WORK1 + PSURF(:,:,newtime) / grav + eps2

        where ( k <= KMT )
          POTICE = (TFRZ(TRACER(:,:,k,2,newtime)) 
     &                 - TRACER(:,:,k,1,newtime)) * WORK1
        endwhere 

        POTICE = max(POTICE, QICE)

        TRACER(:,:,k,1,newtime) = TRACER(:,:,k,1,newtime)
     &                           + POTICE / WORK1
     
        if ( sfc_layer_type == sfc_layer_varthick .and.
     &       .not. lfw_as_salt_flx ) then
          TRACER(:,:,k,2,newtime) = ( TRACER(:,:,k,2,newtime)
     &       * (WORK1 + cp_over_lhfusion * QICE)
     &       - salice * QICE * cp_over_lhfusion ) / WORK1
        else
          TRACER(:,:,k,2,newtime) = TRACER(:,:,k,2,newtime)
     &       + (salref - salice) * POTICE * cp_over_lhfusion / WORK1
        endif

        QICE = QICE - POTICE

!-----------------------------------------------------------------------
!
!     let any residual heat in the upper layer melt previously 
!     formed ice
!
!-----------------------------------------------------------------------
 
        AQICE = AQICE + time_weight * QICE

!-----------------------------------------------------------------------
!
!     recalculate freezing potential based on adjusted T.
!     only interested in melt potential now (POTICE < 0) - use this 
!     melt to offset any accumulated freezing (AQICE < 0) and
!     adjust T and S to reflect this melting. when freshwater flux
!     formulation, compute the associated freshwater flux instead of
!     adjusting S.
!
!-----------------------------------------------------------------------

        where (k <= KMT)
          POTICE = (TFRZ(TRACER(:,:,k,2,newtime))  
     &                 - TRACER(:,:,k,1,newtime)) * WORK1 
        endwhere

        POTICE = max(POTICE, AQICE)

        TRACER(:,:,k,1,newtime) = TRACER(:,:,k,1,newtime) 
     &                           + POTICE / WORK1

        if ( sfc_layer_type == sfc_layer_varthick .and.
     &       .not. lfw_as_salt_flx ) then
          FW_FREEZE = time_weight * min(POTICE, QICE) 
     &               * cp_over_lhfusion / dt(k)
        else
          TRACER(:,:,k,2,newtime) = TRACER(:,:,k,2,newtime)  
     &       + (salref - salice) * POTICE * cp_over_lhfusion / WORK1 
        endif

        AQICE = AQICE - time_weight * POTICE

      endif ! time to do ice

!-----------------------------------------------------------------------

      end subroutine ice_formation

!***********************************************************************

      subroutine ice_flx_to_coupler

!-----------------------------------------------------------------------
!
!     this subroutine sets up the ice formation / melting potential
!     heat fluxes to be sent to the coupler. ice formation heat flux
!     is accumulated for time averaging.
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  k                   ! vertical level index

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  WORK1, WORK2        ! work arrays

!-----------------------------------------------------------------------
!
!     compute the first layer thickness
!
!-----------------------------------------------------------------------

      k = 1 

      WORK1 = dz(k)
      if ( sfc_layer_type == sfc_layer_varthick ) 
     &  WORK1 = WORK1 + PSURF(:,:,curtime)/grav + eps2

      if ( liceform .and. check_time_flag(ice_cpl_flag) ) then

!-----------------------------------------------------------------------
!
!     first compute the melt potential
!
!-----------------------------------------------------------------------

        WORK2 = c0
        where ( k <= KMT )
          WORK2 = (TFRZ(TRACER(:,:,k,2,curtime)) 
     &                - TRACER(:,:,k,1,curtime)) * WORK1
        endwhere

!-----------------------------------------------------------------------
!
!     adjust ice formation amount when mixing step is tavg
!
!-----------------------------------------------------------------------

        if ( tmix_iopt /= tmix_matsuno ) then
          AQICE = p5 * AQICE
        endif

!-----------------------------------------------------------------------
!
!     merge the ice formation and melt potential fluxes and compute
!     QFLUX in W/m^2
!
!-----------------------------------------------------------------------

        where ( AQICE < c0 ) 
          QICE = -AQICE
        elsewhere
          QICE = WORK2
        endwhere

        QFLUX = QICE / tlast_ice / hflux_factor

!-----------------------------------------------------------------------
!
!     accumulate ice formation heat flux for time averaging
!
!-----------------------------------------------------------------------

        call tavg_qflux_compute (QFLUX, tlast_ice)

      endif

      end subroutine ice_flx_to_coupler

!***********************************************************************

      function TFRZ(SALT)

!-----------------------------------------------------------------------
!
!     this function computes the freezing point of salt water
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) ::
     &  SALT                ! salinity in model units (g/g)

!-----------------------------------------------------------------------
!
!     output variable
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  TFRZ                ! freezing temperature of water in deg C

!-----------------------------------------------------------------------
!
!     use only the first salinity term in the expansion
!
!-----------------------------------------------------------------------

c     TFRZ = -0.0544_dbl_kind*SALT*salt_to_ppt
      TFRZ = -1.8_dbl_kind

!-----------------------------------------------------------------------

      end function TFRZ

!***********************************************************************

      function TMLT(SALT)

!-----------------------------------------------------------------------
!
!     this function sets the melting point temperature of ice.
!     for now, TMLT is a separate function than TFRZ.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variable
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) ::
     &  SALT                ! salinity in model units (g/g)

!-----------------------------------------------------------------------
!
!     output variable
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  TMLT                ! melting temperature in deg C

      if ( lactive_ice ) then
        TMLT = c0
      else
        TMLT = TFRZ(SALT)
      endif

      end function TMLT

!***********************************************************************

      end module ice

!***********************************************************************
