!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module initial

!***********************************************************************

!-----------------------------------------------------------------------
!
!     this module contains routines for initializing a POP simulation
!
!     CVS:$Id: initial.F,v 1.3.2.14 2002/05/10 22:42:46 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use communicate
      use prognostic
      use solvers
      use global_reductions
      use stencils
      use operators
      use grid
      use io
      use baroclinic
      use pressure_grad
      use convection
      use vertical_mix
      use horizontal_mix
      use advection
      use diagnostics
      use state_mod
      use time_management
      use topostress
      use ice
      use xdisplay
      use output
      use tavg
      use hydro_sections
      use current_meters
      use drifters
      use forcing
      use forcing_coupled
      use exit_mod
      use restart
      use ms_balance
      use budget_diagnostics

      implicit none
      save

!-----------------------------------------------------------------------
!
!     module variables
!
!-----------------------------------------------------------------------

!***********************************************************************

      contains

!***********************************************************************

      subroutine initialize_POP

!-----------------------------------------------------------------------
!
!     this routine initializes a POP run by calling various module
!     initialization routines and setting up the initial temperature
!     and salinity
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: del_mom, del_tracer

      integer (kind=int_kind) ::
     &  k,                    ! dummy vertical level index
     &  nu,                   ! i/o unit number
     &  ier                   ! error flag

!-----------------------------------------------------------------------
!
!     intialize message-passing or other communication protocol
!
!-----------------------------------------------------------------------

#ifdef coupled
      call MPI_INIT(ier)
      call mpi_coupled
#endif
      call init_communicate

!-----------------------------------------------------------------------
!
!     initialize constants and i/o stuff
!
!-----------------------------------------------------------------------

      call init_constants
      call init_io
      call init_timers
 
      if ( my_task == master_task )  call shr_sys_flush(stdout)

!-----------------------------------------------------------------------
!
!     output grid size information to log file
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        write(stdout,1000)
        write(stdout,1010)
        write(stdout,1020)
        write(stdout,1021)
        write(stdout,1022) '$Name: ccsm2_0_beta46 $'
        write(stdout,1010)
        write(stdout,1000)

        write(stdout,1030)
        write(stdout,1010)
        write(stdout,1040)
        write(stdout,1010)
        write(stdout,1030)

        write(stdout,1050) imt_global, jmt_global, km
        write(stdout,1060) nproc_s, nproc_x, nproc_y
        write(stdout,1070) imt,jmt
        write(stdout,1080) iphys_e-iphys_b+1,jphys_e-jphys_b+1
        call shr_sys_flush(stdout)
      endif

 1000 format(72('*'))
 1010 format(' ')
 1020 format(' Parallel Ocean Program (POP) ')
 1021 format(' Version 1.4.3  Released 22 May 2001 ')
 1022 format(' CCSM modification tag: ', a)
 1030 format(72('-'))
 1040 format(' Problem and domain sizes')
 1050 format(' Global problem size:',2x,i6,' x ',i6,' x ',i6)
 1060 format(' Using ',i6,' processors in a ',i6,' x ',i6,
     &           ' Cartesian decomposition')
 1070 format(' Local array size is:',2x,i6,' x ',i6)
 1080 format(' Physical domain is (approximately):',2x,i6,' x ',i6)

!-----------------------------------------------------------------------
!
!     set up computational grid
!
!-----------------------------------------------------------------------

      call init_grid

      if ( my_task == master_task )  call shr_sys_flush(stdout)
 
!-----------------------------------------------------------------------
!
!     compute time step and initialize time-related quantities
!
!-----------------------------------------------------------------------

      call init_time_manager1

      if ( my_task == master_task )  call shr_sys_flush(stdout)
 
!-----------------------------------------------------------------------
!
!     initialize equation of state
!
!-----------------------------------------------------------------------

      call init_state

      if ( my_task == master_task )  call shr_sys_flush(stdout)
 
!-----------------------------------------------------------------------
!
!     initialize horizontal mixing variables
!
!-----------------------------------------------------------------------

      call init_horizontal_mix
 
      if ( my_task == master_task )  call shr_sys_flush(stdout)
 

!-----------------------------------------------------------------------
!
!     initialize advection variables
!
!-----------------------------------------------------------------------

      call init_advection
 
      if ( my_task == master_task )  call shr_sys_flush(stdout)
 
!-----------------------------------------------------------------------
!
!     initialize vertical mixing variables
!
!-----------------------------------------------------------------------

      call init_vertical_mix

      if ( my_task == master_task )  call shr_sys_flush(stdout)
 
!-----------------------------------------------------------------------
!
!     calculate time-independent stencil coefficients
!
!-----------------------------------------------------------------------
 
      !***
      !*** operators different if variable hmix and Laplacian mixing
      !*** chosen
      !***

      del_mom    = 0
      del_tracer = 0
      if (lvariable_hmix) then
        if (hmix_momentum_itype == hmix_momentum_type_del2) del_mom = 2
        if (hmix_tracer_itype == hmix_tracer_type_del2) del_tracer = 2
      endif

      call init_operators(del_mom, del_tracer, AMF, AHF)
      call init_solvers    ! initialize barotropic solver and operators

      if ( my_task == master_task )  call shr_sys_flush(stdout)
 
!-----------------------------------------------------------------------
!
!     initialize pressure gradient (pressure averaging)
!     initialize convective adjustment
!     initialize baroclinic (reset to freezing)
!
!-----------------------------------------------------------------------

      call init_pressure_grad
      call init_convection
      call init_baroclinic

      if ( my_task == master_task )  call shr_sys_flush(stdout)
 
!-----------------------------------------------------------------------
!
!     initialize prognostic fields
!
!-----------------------------------------------------------------------

      call init_prognostic
 
      if ( my_task == master_task )  call shr_sys_flush(stdout)

!-----------------------------------------------------------------------
!
!     initialize ice module
!
!-----------------------------------------------------------------------

      call init_ice

      if ( my_task == master_task )  call shr_sys_flush(stdout)

!-----------------------------------------------------------------------
!
!     set initial temperature and salinity profiles (includes read of
!     restart file
!
!-----------------------------------------------------------------------

      call init_restart
      call init_ts

      if ( my_task == master_task )  call shr_sys_flush(stdout)

!-----------------------------------------------------------------------
!
!     finish computing time-related quantities after restart info
!     available
!
!-----------------------------------------------------------------------

      call init_time_manager2

      if ( my_task == master_task )  call shr_sys_flush(stdout)

!-----------------------------------------------------------------------
!
!     initialize diagnostics
!
!-----------------------------------------------------------------------

      call init_diagnostics

      if ( my_task == master_task )  call shr_sys_flush(stdout)

!-----------------------------------------------------------------------
!
!     initialize fields for surface forcing
!
!-----------------------------------------------------------------------

      call init_forcing

      if ( my_task == master_task )  call shr_sys_flush(stdout)

!-----------------------------------------------------------------------
!
!     initialize time averaging
!
!-----------------------------------------------------------------------

      call init_tavg

      if ( my_task == master_task )  call shr_sys_flush(stdout)

!-----------------------------------------------------------------------
!
!     initialize global budget diagnostics
!
!-----------------------------------------------------------------------

      call init_budget_diagnostics

      if ( my_task == master_task )  call shr_sys_flush(stdout)

!-----------------------------------------------------------------------
!
!     initialize output
!
!-----------------------------------------------------------------------

      call init_output

      if ( my_task == master_task )  call shr_sys_flush(stdout)

!-----------------------------------------------------------------------
!
!     Initialize drifters, hydrographic sections and current meters
!
!-----------------------------------------------------------------------

      call init_drifters
      call init_hydro_sections
      call init_current_meters

      if ( my_task == master_task )  call shr_sys_flush(stdout)

!-----------------------------------------------------------------------
!
!     calculate topographic stress (maximum entropy) velocities
!
!-----------------------------------------------------------------------

      call init_topostress
      if (ltopostress .and. 
     &    (hmix_momentum_itype /= hmix_momentum_type_del2)) then
        if (my_task == master_task) then
          write(stdout,*) 
     &    'WARNING: Topo stress has no effect if mom mixing is not del2'
        endif
      endif
 
      if ( my_task == master_task )  call shr_sys_flush(stdout)

!-----------------------------------------------------------------------
!
!     initialize ms_balance
!
!-----------------------------------------------------------------------
 
      if (lcoupled .and. lms_balance) call init_ms_balance

!-----------------------------------------------------------------------
!
!     initialize X display 
!
!-----------------------------------------------------------------------

      call init_xdisplay

!-----------------------------------------------------------------------
!
!     write model information into log file
!
!-----------------------------------------------------------------------

      call document_POP
 

!-----------------------------------------------------------------------
!
!     output delimiter to log file
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        write(stdout,1030)
        write(stdout,*)
        write(stdout,'(" End of initialization")')
        write(stdout,*)
        write(stdout,1030)
        call shr_sys_flush(stdout)
      endif

!-----------------------------------------------------------------------
!
!     write model warnings into log file
!
!-----------------------------------------------------------------------

      call POP_warnings
 

!-----------------------------------------------------------------------

      end subroutine initialize_POP

!***********************************************************************

      subroutine init_ts

!-----------------------------------------------------------------------
!
!     initializes temperature and salinity
!     also initializes prognostic variables from restart if required
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     namelist input
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: nml_error ! namelist i/o error flag

      character (char_len) ::
     &  init_ts_option      ! option for initializing t,s
      character (char_len_long) ::
     &  init_ts_file

      namelist /init_ts_nml/ init_ts_option, init_ts_file

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  k,                  ! vertical level index
     &  n,                  ! tracer index
     &  kk,                 ! indices for interpolating levitus data
     &  nu                  ! i/o unit number

      logical (kind=log_kind) ::
     &  lredef_case         ! flag for telling restart to redefine casename
     &, lredef_date         ! flag for telling restart to redefine date     

      real (kind=dbl_kind) ::
     &  sinterp,            ! factor for interpolating levitus data
     &  dpth_meters         ! depth of level in meters

      real (kind=dbl_kind), dimension(km) ::
     &  tinit, sinit        ! mean initial state as function of depth

      !***
      !*** 1992 Levitus mean climatology for internal generation of t,s
      !***

      real (kind=dbl_kind), dimension(33) ::
     &  depth_levitus = (/   
     &               0.0_dbl_kind,   10.0_dbl_kind,   20.0_dbl_kind,
     &              30.0_dbl_kind,   50.0_dbl_kind,   75.0_dbl_kind,
     &             100.0_dbl_kind,  125.0_dbl_kind,  150.0_dbl_kind,
     &             200.0_dbl_kind,  250.0_dbl_kind,  300.0_dbl_kind,
     &             400.0_dbl_kind,  500.0_dbl_kind,  600.0_dbl_kind,
     &             700.0_dbl_kind,  800.0_dbl_kind,  900.0_dbl_kind,
     &            1000.0_dbl_kind, 1100.0_dbl_kind, 1200.0_dbl_kind,
     &            1300.0_dbl_kind, 1400.0_dbl_kind, 1500.0_dbl_kind,
     &            1750.0_dbl_kind, 2000.0_dbl_kind, 2500.0_dbl_kind,
     &            3000.0_dbl_kind, 3500.0_dbl_kind, 4000.0_dbl_kind,
     &            4500.0_dbl_kind, 5000.0_dbl_kind, 5500.0_dbl_kind /),
     &  tmean_levitus = (/ 
     &             18.27_dbl_kind, 18.22_dbl_kind, 18.09_dbl_kind,
     &             17.87_dbl_kind, 17.17_dbl_kind, 16.11_dbl_kind,
     &             15.07_dbl_kind, 14.12_dbl_kind, 13.29_dbl_kind,
     &             11.87_dbl_kind, 10.78_dbl_kind,  9.94_dbl_kind,
     &              8.53_dbl_kind,  7.35_dbl_kind,  6.38_dbl_kind,
     &              5.65_dbl_kind,  5.06_dbl_kind,  4.57_dbl_kind,
     &              4.13_dbl_kind,  3.80_dbl_kind,  3.51_dbl_kind,
     &              3.26_dbl_kind,  3.05_dbl_kind,  2.86_dbl_kind,
     &              2.47_dbl_kind,  2.19_dbl_kind,  1.78_dbl_kind,
     &              1.49_dbl_kind,  1.26_dbl_kind,  1.05_dbl_kind,
     &              0.91_dbl_kind,  0.87_dbl_kind,  1.00_dbl_kind /),
     &  smean_levitus = (/ 
     &             34.57_dbl_kind, 34.67_dbl_kind, 34.73_dbl_kind,
     &             34.79_dbl_kind, 34.89_dbl_kind, 34.97_dbl_kind,
     &             35.01_dbl_kind, 35.03_dbl_kind, 35.03_dbl_kind,
     &             34.98_dbl_kind, 34.92_dbl_kind, 34.86_dbl_kind,
     &             34.76_dbl_kind, 34.68_dbl_kind, 34.63_dbl_kind,
     &             34.60_dbl_kind, 34.59_dbl_kind, 34.60_dbl_kind,
     &             34.61_dbl_kind, 34.63_dbl_kind, 34.65_dbl_kind,
     &             34.66_dbl_kind, 34.68_dbl_kind, 34.70_dbl_kind,
     &             34.72_dbl_kind, 34.74_dbl_kind, 34.75_dbl_kind,
     &             34.74_dbl_kind, 34.74_dbl_kind, 34.73_dbl_kind,
     &             34.73_dbl_kind, 34.72_dbl_kind, 34.72_dbl_kind /)

!-----------------------------------------------------------------------
!
!     read input namelist and broadcast 
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=init_ts_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue

        write(stdout,*)
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('reading init_ts_nml',normal_exit=.false.)
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, init_ts_nml)
          write(stdout,*) ' '
          select case (init_ts_option)
            case ('continue', 'restart', 'branch', 'hybrid') 
               if (luse_pointer_files) then
                  write(stdout,*) ' In this case, the init_ts_file'
     &            // ' name will be read from the pointer file'
     &            // ' and the INIT_TS_FILE name will be ignored '
     &            // ' (see below) '
                  write(stdout,*) ' '
               endif
          end select
            
      endif

      call broadcast_scalar(init_ts_option,master_task)
      call broadcast_scalar(init_ts_file  ,master_task)

!-----------------------------------------------------------------------
!
!     initialize t,s or call restart based on init_ts_option
!
!-----------------------------------------------------------------------

      select case (init_ts_option)

!-----------------------------------------------------------------------
!
!     read full 3-d t,s from input file  (CCSM RUNTYPE = 'startup')  
!       (LANL designation is 'file', old CCSM was 'initial')
!
!-----------------------------------------------------------------------

      case ('startup', 'file', 'initial')
        first_step = .true.

        if (my_task == master_task) then
          write(stdout,*) 'Initial 3-d T,S read from file:',
     &                     trim(init_ts_file)
        endif
        call open_parallel_file(nu,init_ts_file,recl_dbl)
        call read_array(nu,TRACER(:,:,:,1,curtime))
        call read_array(nu,TRACER(:,:,:,2,curtime))
        call close_parallel_file(nu)

        do n=1,nt
          do k=1,km
            where (k > KMT) TRACER(:,:,k,n,curtime) = c0
          end do
        end do

        if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' file read: ', init_ts_file
        endif

        !*** convert salinity to model units
        TRACER(:,:,:,2,curtime) = TRACER(:,:,:,2,curtime)*ppt_to_salt

        if (topo_smooth) then
          do k=1,km
            call fill_points(k,TRACER(:,:,k,1,curtime))
            call fill_points(k,TRACER(:,:,k,2,curtime))
          enddo
        endif

        TRACER(:,:,:,:,newtime) = TRACER(:,:,:,:,curtime)
        TRACER(:,:,:,:,oldtime) = TRACER(:,:,:,:,curtime)

!-----------------------------------------------------------------------
!
!     set initial state from restart file  (CCSM RUNTYPE = 'continue')
!       (LANL designation is 'restart')
!
!-----------------------------------------------------------------------

      case ('continue', 'restart')
        first_step  = .false.
        lredef_date = .false.
        lredef_case = .false.
        if (my_task == master_task) then
          write(stdout,*) 'Initial T,S read from restart file.'
        endif
        call read_restart(init_ts_file,lredef_case, lredef_date)
        ltavg_restart = .true.

!-----------------------------------------------------------------------
!
!     set initial state from restart file  (CCSM RUNTYPE = 'branch')   
!       and redefine the casename
!
!-----------------------------------------------------------------------

      case ('branch')
        first_step  = .false.
        lredef_date = .false.
        lredef_case = .true.  
        if (my_task == master_task) then
          write(stdout,*) 'Initial T,S branching from restart file.'
        endif
        call read_restart(init_ts_file,lredef_case, lredef_date)
        ltavg_restart = .false.

!-----------------------------------------------------------------------
!
!     set initial state from restart file  (CCSM RUNTYPE = 'hybrid')  
!       but redefine the casename and the starting date
!
!-----------------------------------------------------------------------

      case ('hybrid')
        first_step  = .true.  
        lredef_date = .true.  
        lredef_case = .true.  
        if (my_task == master_task) then
          write(stdout,*) 'Initial T,S is from a restart file.'
        endif
        call read_restart(init_ts_file,lredef_case, lredef_date)
        ltavg_restart = .false.

!-----------------------------------------------------------------------
!
!     set up t,s from input mean state as function of depth
!
!-----------------------------------------------------------------------

      case ('mean')
        first_step = .true.

        !***
        !*** open input file and read t,s profile
        !***

        call get_unit(nu)
        if (my_task == master_task) then 
          write(stdout,*) 'Initial mean T,S profile read from file:',
     &                     trim(init_ts_file)
          open(nu, file=init_ts_file, status='old')
          do k = 1,km
            read(nu,*) tinit(k),sinit(k)
          enddo
          close (nu)
        endif
        call release_unit(nu)

        call broadcast_vector(tinit, master_task, km)
        call broadcast_vector(sinit, master_task, km)

        !***
        !*** fill tracer array with appropriate values
        !***

        do k=1,km
          where (k <= KMT)
           TRACER(:,:,k,1,curtime) = tinit(k)
           TRACER(:,:,k,2,curtime) = sinit(k)*ppt_to_salt
          endwhere
        enddo

        TRACER(:,:,:,:,newtime) = TRACER(:,:,:,:,curtime)
        TRACER(:,:,:,:,oldtime) = TRACER(:,:,:,:,curtime)

!-----------------------------------------------------------------------
!
!     set up initial profile from 1992 Levitus mean ocean data
!
!-----------------------------------------------------------------------

      case ('internal')
        first_step = .true.
        if (my_task == master_task) then
          write(stdout,*) 
     &  'Initial T,S profile computed internally from 1992 Levitus data'
        endif

        do k=1,km

          dpth_meters = zt(k)*mpercm

          intrp_loop: do kk=1,32
            if (dpth_meters >= depth_levitus(kk) .and.
     &          dpth_meters <  depth_levitus(kk+1)) exit intrp_loop
          end do intrp_loop

          sinterp = (dpth_meters - depth_levitus(kk))/
     &              (depth_levitus(kk+1) - depth_levitus(kk))
 
          tinit(k) = (c1 - sinterp)*tmean_levitus(kk) +
     &                     sinterp *tmean_levitus(kk+1)
          sinit(k) = (c1 - sinterp)*smean_levitus(kk) +
     &                     sinterp *smean_levitus(kk+1)

          !***
          !*** calculate potential temp. from from insitu temperature,
          !*** salinity, and depth (pressure)
          !*** this routine part of state module and sinterp is used
          !*** as a temporary space for potential temperature
          !*** 

          where (k <= KMT)
           TRACER(:,:,k,1,curtime) = tinit(k)
           TRACER(:,:,k,2,curtime) = sinit(k)*ppt_to_salt
          endwhere

        enddo

        TRACER(:,:,:,:,newtime) = TRACER(:,:,:,:,curtime)
        TRACER(:,:,:,:,oldtime) = TRACER(:,:,:,:,curtime)

!-----------------------------------------------------------------------
!
!     bad initialization option
!
!-----------------------------------------------------------------------

      case default
        call exit_POP('Unknown t,s initialization option' 
     &,                normal_exit=.false.)
      end select

!-----------------------------------------------------------------------
!
!     calculate RHO from TRACER at time levels curtime and oldtime
!
!-----------------------------------------------------------------------

      do k=1,km
        call state(k,k,TRACER(:,:,k,1,curtime),TRACER(:,:,k,2,curtime),
     &                 RHOOUT=RHO(:,:,k,curtime))
        call state(k,k,TRACER(:,:,k,1,oldtime),TRACER(:,:,k,2,oldtime),
     &                 RHOOUT=RHO(:,:,k,oldtime))
      enddo

!-----------------------------------------------------------------------

      end subroutine init_ts

      subroutine document_POP

!-----------------------------------------------------------------------
!
!     this routine writes POP model information to the output log file
!
!-----------------------------------------------------------------------




!-----------------------------------------------------------------------
!
!     output model constant information to log file
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        write(stdout,1000)
        write(stdout,1010)

        write(stdout,1020) 'grav',    grav,      'cm/s^2'  
        write(stdout,1020) 'omega',   omega,     'rad/s'
        write(stdout,1020) 'radius',  radius,    'cm'
        write(stdout,1020) 'cp_sw',   cp_sw,     'erg/g/K'
        write(stdout,1020) 'cp_air',  cp_air,    'J/kg/K'
        write(stdout,1020) 'rho_air', rho_air,   'kg/m^3'
        write(stdout,1020) 'rho_sw',  rho_sw,    'g/cm^3'
        write(stdout,1020) 'rho_fw',  rho_fw,    'g/cm^3'
        write(stdout,1020) 'sound',   sound,     'cm/s' 
        write(stdout,1020) 'vonkar',  vonkar,    ' '        
        write(stdout,1020) 'emissivity',emissivity, ' '
        write(stdout,1020) 'stefan_boltzmann', stefan_boltzmann,  
     &                        'W/m^2/K^4'
        write(stdout,1020) 'latent_heat_vapor',latent_heat_vapor,  
     &                        'J/kg'
        write(stdout,1020) 'latent_heat_fusion',latent_heat_fusion, 
     &                        'erg/g'
        write(stdout,1020) 'ocn_ref_salinity', ocn_ref_salinity, 'psu' 
        write(stdout,1020) 'sea_ice_salinity', sea_ice_salinity, 'psu' 
        write(stdout,1020) 'T0_Kelvin',        T0_Kelvin,        'K' 
        write(stdout,1020) 'pi',               pi,               ' ' 
 
        write(stdout,1010)
        write(stdout,1030)

        call shr_sys_flush(stdout)
 
      endif

 1000 format(72('*'))
 1010 format(' ')
 1020 format (5x, a20, ' = ', 1pe25.15, 2x, a)
 1030 format(72('-'))

!-----------------------------------------------------------------------

      end subroutine document_POP
 
      subroutine POP_warnings

!-----------------------------------------------------------------------
!
!     this routine writes POP-model warnings to the output log file.
!     these warning conditions usually involve conditions set in two or 
!     more different modules, but this routine also serves as a 
!     convenient place to collect all warning conditions. the warning
!     may result in model shut-down
!
!-----------------------------------------------------------------------

      character (char_len_long) :: message
      integer (kind=int_kind)   :: number_of_fatal_errors


!-----------------------------------------------------------------------
!
!     warning conditions
!     ==================
!
!-----------------------------------------------------------------------

 
      if (my_task == master_task) then
        write(stdout,1000)
        write(stdout,1010)

!-----------------------------------------------------------------------
!
!     'varthick' and dtuxcel /= dttxcel(1)
!
!-----------------------------------------------------------------------
        if (sfc_layer_type == sfc_layer_varthick .and.
     &      dtuxcel /= dttxcel(1) ) then
            message = 'Warning:  Surface tracer and momentum'
     &       //' timesteps are unequal -- may cause instability'
     &       //' when using variable-thickness surface layer.'
            write(stdout,1100) message
            call shr_sys_flush(stdout)
        endif

        write(stdout,1030)
        call shr_sys_flush(stdout)
      endif


!-----------------------------------------------------------------------
!
!     fatal error conditions
!     ====================== 
!
!-----------------------------------------------------------------------

      number_of_fatal_errors = 0
 
      if (my_task == master_task) then
        write(stdout,1000)
        write(stdout,1010)


!-----------------------------------------------------------------------
!
!      sfc_layer_varthick & lfw_as_salt_flx = .false.
!
!-----------------------------------------------------------------------
        if (sfc_layer_type == sfc_layer_varthick .and.
     &      .not. lfw_as_salt_flx) then
            number_of_fatal_errors = number_of_fatal_errors + 1
            message = 'Fatal error:  unresolved issues involving'
     &       //' the combination sfc_layer_type == sfc_layer_varthick' 
     &       //' and lfw_as_salt_flx = .false.'
     &       //' cause this combination of options to be unsupported'
     &       //' at this time.'
            write(stdout,1100) message
            call shr_sys_flush(stdout)
        endif

 
 
!-----------------------------------------------------------------------
!
!      sfc_layer_varthick  & liceform  & lfw_as_salt_flx = .false.
!                            ^^^^^^^^
!-----------------------------------------------------------------------
        if (sfc_layer_type == sfc_layer_varthick .and.
     &      liceform .and. .not. lfw_as_salt_flx) then
            number_of_fatal_errors = number_of_fatal_errors + 1
            message = 'Fatal error:  unresolved issues involving'
     &       //' the combination sfc_layer_type == sfc_layer_varthick' 
     &       //' and liceform = .true. and lfw_as_salt_flx = .false.'
     &       //' cause this combination of options to be unsupported'
     &       //' at this time.'
            write(stdout,1100) message
            call shr_sys_flush(stdout)
        endif

!-----------------------------------------------------------------------
!
!      num_ghost_cells < 2
!                          
!-----------------------------------------------------------------------
        if (num_ghost_cells < 2) then
  
          if (tadvect_itype == tadvect_upwind3) then
            number_of_fatal_errors = number_of_fatal_errors + 1
            message = 'Fatal error:  using the '
     &       //'3rd-order upwind advection for tracers requires '
     &       //' num_ghost_cells to be greater than 1'
            write(stdout,1100) message
            call shr_sys_flush(stdout)
          endif
  
          if (hmix_tracer_itype == hmix_tracer_type_gm) then
            number_of_fatal_errors = number_of_fatal_errors + 1
            message = 'Fatal error:  using the '
     &       //'Gent-McWilliams tracer mixing reuqires '
     &       //' num_ghost_cells to be greater than 1'
            write(stdout,1100) message
            call shr_sys_flush(stdout)
          endif
  
          if (hmix_tracer_itype == hmix_tracer_type_del4) then
            number_of_fatal_errors = number_of_fatal_errors + 1
            message = 'Fatal error:  using the '
     &       //'Biharmonic horizontal tracer mixing '
     &       //' num_ghost_cells to be greater than 1'
            write(stdout,1100) message
            call shr_sys_flush(stdout)
          endif
  
          if (ltavg_flags(ntavg_BSF)) then
            number_of_fatal_errors = number_of_fatal_errors + 1
            message = 'Fatal error:  using the '
     &       //'barotropic streamfunction diagnostic requires'
     &       //' num_ghost_cells to be greater than 1'
            write(stdout,1100) message
            call shr_sys_flush(stdout)
          endif
 
        endif

 
 
        write(stdout,1030)
        call shr_sys_flush(stdout)
      endif

      if (number_of_fatal_errors > 0 ) then
         call exit_POP (' correct the error condition(s)'
     &                //' listed above before continuing' 
     &,                 normal_exit = .false.) 
      endif
 
 1000 format(72('*'))
 1010 format(' ')
 1030 format(72('-'))
 1100 format(5x, a)   

!-----------------------------------------------------------------------

      end subroutine POP_warnings
!***********************************************************************

      end module initial

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
