!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module movie

!***********************************************************************
!
!     this module contains information necessary for writing movie
!     file output
!
!     CVS:$Id: movie.F,v 1.2.2.6 2002/03/04 23:33:36 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use prognostic
      use grid
      use io
      use operators
      use global_reductions
      use stencils
      use time_management
      use exit_mod
      use io_general

      implicit none
      save     movie_bin    ! redundant save for compilation on IBM
     &,        movie_hdr
      save

!-----------------------------------------------------------------------
!
!     module variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  movie_freq_iopt,    ! frequency option for writing movie
     &  movie_freq          ! frequency of movie output

      logical (kind=log_kind) ::
     &  lmovie_on           ! movie file output wanted

      character (char_len) ::
     &  movie_outfile       ! root filename for movie output

      !***
      !*** available movie fields
      !***

      integer (kind=int_kind) ::
     &  num_mov_fields

      integer (kind=int_kind), dimension(:), allocatable ::
     &  mov_fields

      character (char_len), dimension(:), allocatable ::
     &  mov_fields_descr

      integer (kind=int_kind), parameter ::
     &  mov_field_shgt    =  1,
     &  mov_field_utrans  =  2,
     &  mov_field_vtrans  =  3,
     &  mov_field_temp1_2 =  4,
     &  mov_field_salt1_2 =  5,
     &  mov_field_temp6   =  6,
     &  mov_field_salt6   =  7,
     &  mov_field_vort    =  8

      type (output_file)
     &                            :: movie_bin
     &,                              movie_hdr
 
!***********************************************************************

      contains

!***********************************************************************

      subroutine write_movie(file_suffix)

!-----------------------------------------------------------------------
!
!     write movie dump
!
!     fields written are:
!
!       PSURF/grav     ! surface height 
!       UBTROP,VBTROP  ! barotropic velocities
!       UVEL,VVEL      ! 3d velocities
!       TRACER         ! 3d tracers
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      character (char_len), intent(in) ::
     &  file_suffix         ! suffix to append to root filename 

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  nu,                 ! i/o unit for output file
     &  n,                  ! dummy tracer index
     &  k,                  ! vertical level index
     &  nfield              ! dummy field index

 
      character (char_len_long) ::
     &  movie_filename,   ! filename for movie data
     &  header_filename     ! filename for header file

      real (kind=real_kind), dimension(imt,jmt) ::
     &  WORK2D              ! single precision array for 2d output

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  WORK1, WORK2        ! work arrays for computed fields

!-----------------------------------------------------------------------
!
!     write header file with contents
!
!-----------------------------------------------------------------------

      
      call create_output_filename (movie_outfile,'.', file_suffix
     &,                           '.hdr',header_filename,movie_hdr)
 

      call get_unit(nu)

      if (my_task == master_task) then
        open (nu, file=header_filename, status='unknown')

        write(nu,1000) 
     &     num_mov_fields, nsteps_total, tday00,cyear,cmonth, cday
 
        do nfield=1,num_mov_fields
          write(nu,'(a80)') mov_fields_descr(nfield)
        end do

        close(nu)
      endif

      call release_unit(nu)

 1000 format(i3,1x,i10,1x,f14.6,1x,a4,'-',a2,'-',a2)

!-----------------------------------------------------------------------
!
!     write data file
!
!-----------------------------------------------------------------------


      call create_output_filename (movie_outfile,'.', file_suffix
     &,                            ' ',movie_filename,movie_bin)
 
      call open_parallel_file(nu,movie_filename,recl_real)

!-----------------------------------------------------------------------
!
!     write arrays to parallel file
!
!-----------------------------------------------------------------------

      do nfield=1,num_mov_fields

        select case(mov_fields(nfield))

        case(mov_field_shgt) !*** surface height
          WORK2D = PSURF(:,:,curtime)/grav
          call write_array(nu,WORK2D)
 
        case(mov_field_utrans) !*** zonal transport
          WORK2 = HU*UBTROP(:,:,curtime)
          call sw_4pt_nobndy(WORK1,AT0,ATS,ATW,ATSW,WORK2)
          WORK2D = WORK1
          call write_array(nu,WORK2D)
 
        case(mov_field_vtrans) !*** meridional transport
          WORK2 = HU*VBTROP(:,:,curtime)
          call sw_4pt_nobndy(WORK1,AT0,ATS,ATW,ATSW,WORK2)
          WORK2D = WORK1
          call write_array(nu,WORK2D)

        case(mov_field_temp1_2) !*** surface temp
          WORK2D = p5*(TRACER(:,:,1,1,curtime) + 
     &                 TRACER(:,:,2,1,curtime))
          call write_array(nu,WORK2D)

        case(mov_field_salt1_2) !*** surface salt
          WORK2D = p5*(TRACER(:,:,1,2,curtime) + 
     &                 TRACER(:,:,2,2,curtime))
          call write_array(nu,WORK2D)

        case(mov_field_temp6) !*** level6 temp
          WORK2D = TRACER(:,:,6,1,curtime)
          call write_array(nu,WORK2D)

        case(mov_field_salt6) !*** level6 salt
          WORK2D = TRACER(:,:,6,2,curtime)
          call write_array(nu,WORK2D)

        case(mov_field_vort) !*** relative vorticity at surface
          k = 1 ! Index for surface level
          call zcurl(k,WORK1,UVEL(:,:,1,curtime),VVEL(:,:,1,curtime))
          WORK2 = WORK1*TAREA_R
          WORK2D = WORK2
          call write_array(nu,WORK2D)

        end select
      end do

      call close_parallel_file(nu)

      call document('write_movie'
     &,             'file written: '//trim(movie_filename))

!-----------------------------------------------------------------------

      end subroutine write_movie

!***********************************************************************

      subroutine init_movie

!-----------------------------------------------------------------------
!
!     initializes movie output choices from input files
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input namelist
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  nu,                 ! unit for contents input file
     &  nfield,             ! dummy for field number
     &  n,                  ! dummy for position in character string
     &  nml_error           ! namelist i/o error flag

      character (char_len) ::
     &  movie_freq_opt,   ! choice for frequency of movie output
     &  movie_contents,   ! file with choices of fields in movie output
     &  char_temp         ! temporary for manipulating fields

      namelist /movie_nml/ movie_freq_opt, movie_freq,
     &                     movie_outfile, movie_contents

!-----------------------------------------------------------------------
!
!     read movie file output frequency and filenames from namelist
!
!-----------------------------------------------------------------------

      movie_freq_iopt = freq_opt_never
      movie_freq      = 100000
      movie_outfile   = 'm'
      movie_contents  = 'unknown_movie_contents'

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=movie_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading movie_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, movie_nml)
          write(stdout,*) ' '
      endif

      if (my_task == master_task) then
        select case (movie_freq_opt)
        case ('never')
          movie_freq_iopt = freq_opt_never
        case ('nyear')
          movie_freq_iopt = freq_opt_nyear
        case ('nmonth')
          movie_freq_iopt = freq_opt_nmonth
        case ('nday')
          movie_freq_iopt = freq_opt_nday
        case ('nhour')
          movie_freq_iopt = freq_opt_nhour
        case ('nsecond')
          movie_freq_iopt = freq_opt_nsecond
        case ('nstep')
          movie_freq_iopt = freq_opt_nstep
        case default
          movie_freq_iopt = -1000
        end select

      endif

      call broadcast_scalar(movie_freq_iopt, master_task)

      if (movie_freq_iopt == -1000) then
        call exit_POP('unknown option for movie file frequency')
      else if (movie_freq_iopt == freq_opt_never) then
        lmovie_on = .false.
      else
        lmovie_on = .true.
      endif

      if (lmovie_on) then
        call broadcast_scalar(movie_freq,     master_task)
        call broadcast_scalar(movie_outfile,  master_task)
        call broadcast_scalar(movie_contents, master_task)
      endif

!-----------------------------------------------------------------------
!
!     read contents file to determine which fields to dump
!
!-----------------------------------------------------------------------

      if (lmovie_on) then
        call get_unit(nu)

        if (my_task == master_task) then
          open(nu, file=movie_contents, status='old')

          read(nu,*) num_mov_fields
        endif

        call broadcast_scalar(num_mov_fields, master_task)

        allocate ( mov_fields(num_mov_fields),
     &             mov_fields_descr(num_mov_fields))

        if (my_task == master_task) then
          do nfield=1,num_mov_fields
            read(nu,'(a80)') char_temp

            mov_fields_descr(nfield) = char_temp
            char_temp(1:4) = '    '
            char_temp = adjustl(char_temp)
            n = index(char_temp,' ') - 1
        
            select case(char_temp(1:n))
            case('SHGT')
              mov_fields(nfield) = mov_field_shgt
            case('UTRANS')
              mov_fields(nfield) = mov_field_utrans
            case('VTRANS')
              mov_fields(nfield) = mov_field_vtrans
            case('TEMP1_2')
              mov_fields(nfield) = mov_field_temp1_2
            case('SALT1_2')
              mov_fields(nfield) = mov_field_salt1_2
            case('TEMP6')
              mov_fields(nfield) = mov_field_temp6
            case('SALT6')
              mov_fields(nfield) = mov_field_salt6
            case('VORT')
              mov_fields(nfield) = mov_field_vort
            case default
              mov_fields(nfield) = -1000
            end select
          end do
 
          close(nu)
        endif

        call release_unit(nu)
        call broadcast_vector(mov_fields, master_task, num_mov_fields)

        if (any(mov_fields == -1000)) then
          call exit_POP('unknown field requested for movie file')
        endif

      endif

!-----------------------------------------------------------------------

      end subroutine init_movie

!***********************************************************************

      end module movie

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
