!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module operators

!***********************************************************************
!
!     This module contains arrays, routines necessary for various
!     mathematical operators, including Laplacian, biharmonic, 
!     divergence, curl and a vertical velocity calculation.
!
!     CVS:$Id: operators.F,v 1.3.2.1 2001/03/13 22:55:03 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use prognostic
      use stencils
      use grid

      implicit none
      save

!-----------------------------------------------------------------------
!
!     DT{N,S,E,W} = {N,S,E,W} coefficients of 5-point stencil
!         for the Del**2 operator before b.c.s have been applied
!     DU{C,N,S,E,W} = central and {N,S,E,W} coefficients of 5-point
!         stencil for the Del**2 operator acting at U points
!         (without metric terms that mix U,V) 
!     DM{C,N,S,E,W} = central and {N,S,E,W} coefficients of 5-point
!         stencil for the metric terms that mix U,V
!     DUM = central coefficient for metric terms that
!         do not mix U,V
!     K{X,Y}U = k{x,y} coefficients for advective metric terms
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension (imt,jmt) :: 
     &   DTN,DTS,DTE,DTW,
     &   DUC,DUN,DUS,DUE,DUW,
     &   DMC,DMN,DMS,DME,DMW,
     &   DUM,KXU,KYU

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_operators(del_mom, del_tracer, AMF, AHF)

!-----------------------------------------------------------------------
!
!     calculate the coefficients of the 5-point stencils for 
!     the Del**2 operator acting on tracer and momentum fields.
!     also calculate coefficients for all metric terms (both
!     advective and diffusive).
!
!-----------------------------------------------------------------------
!
!     equations:
!
!     Del**2(T) = (1/dy)*Dx[dy*Dx(T)]
!               + (1/dx)*Dy[dx*Dy(T)]
!
!     Del**2(U) = (1/dy)*Dx[dy*Dx(U)]
!               + (1/dx)*Dy[dx*Dy(U)]
!               - U*[dxkx + dyky + 2(kx**2 + ky**2)]   ! metric
!               + 2*ky*Dx(V) - 2*kx*Dy(V)              ! terms
!
!     Del**2(V) = (1/dy)*Dx[dy*Dx(V)]
!               + (1/dx)*Dy[dx*Dy(V)]
!               - V*[dxkx + dyky + 2(kx**2 + ky**2)]   ! metric
!               - 2*ky*Dx(U) + 2*kx*Dy(U)              ! terms
!
!     kx = (1/dy)*Dx(dy)
!     ky = (1/dx)*Dy(dx)
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  del_mom,    ! =2 if variable and del2 momentum mixing requested
     &  del_tracer  ! =2 if variable and del2 tracer   mixing requested

      real (kind=dbl_kind), dimension(:,:), intent(in) ::
     &  AMF, AHF    ! variable mixing coefficient factors

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension (imt,jmt) :: 
     &   DXKX,DYKY,DXKY,DYKX,     ! d{x,y}k{x,y}
     &   WORK1,WORK2,WORK3,WORK4  ! temporary work space

!-----------------------------------------------------------------------
!
!     calculate {N,S,E,W} coefficients for Del**2 acting on tracer
!     fields (for tracers, the central coefficient is calculated as
!     minus the sum of these after boundary conditions are applied).
!
!-----------------------------------------------------------------------

      WORK1 = HTN/HUW
      if (del_tracer == 2) then
        call n_shift(WORK2,AHF)
        WORK1 = WORK1*p5*(AHF + WORK2)
      endif
      DTN = WORK1*TAREA_R
      call s_shift(DTS,WORK1)
      DTS = DTS*TAREA_R

      WORK1 = HTE/HUS
      if (del_tracer == 2) then
        call e_shift(WORK2,AHF)
        WORK1 = WORK1*p5*(AHF + WORK2)
      endif
      DTE = WORK1*TAREA_R
      call w_shift(DTW,WORK1)
      DTW = DTW*TAREA_R

!-----------------------------------------------------------------------
!
!     calculate central and {N,S,E,W} coefficients for 
!     Del**2 (without metric terms) acting on momentum. 
!
!-----------------------------------------------------------------------

      WORK1 = HUS/HTE
      if (del_mom == 2) then
        call s_shift(WORK2,AMF)
        WORK1 = WORK1*p5*(AMF + WORK2)
      endif
      DUS = WORK1*UAREA_R
      call n_shift(DUN,WORK1)
      DUN = DUN*UAREA_R

      WORK1 = HUW/HTN
      if (del_mom == 2) then
        call w_shift(WORK2,AMF)
        WORK1 = WORK1*p5*(AMF + WORK2)
      endif
      DUW = WORK1*UAREA_R
      call e_shift(DUE,WORK1)
      DUE = DUE*UAREA_R

!-----------------------------------------------------------------------
!
!     coefficients for metric terms in Del**2(U)
!     and for metric advection terms (KXU,KYU)
!
!-----------------------------------------------------------------------

      call e_shift(KXU,HUW)
      KXU = (KXU - HUW)*UAREA_R
      call n_shift(KYU,HUS)
      KYU = (KYU - HUS)*UAREA_R

      call w_shift(WORK1,HTE)
      WORK1 = (HTE - WORK1)*TAREA_R    ! KXT

      if (del_mom == 2) then
        call w_shift(WORK2,AMF)
        WORK2 = p5*(WORK2 + AMF)

        call n_shift(DXKX,WORK1)
        WORK2 = p5*(WORK1 + DXKX)*WORK2
        call e_shift(DXKX,WORK2)
        DXKX = (DXKX - WORK2)*DXUR
      else
        call e_shift(DXKX,WORK1)
        DXKX = DXKX - WORK1
        call n_shift(WORK2,DXKX)
        DXKX = p5*(DXKX + WORK2)*DXUR
      endif

      if (del_mom == 2) then
        call s_shift(WORK2,AMF)
        WORK2 = p5*(WORK2 + AMF)

        call e_shift(DYKX,WORK1)
        WORK2 = p5*(WORK1 + DYKX)*WORK2
        call n_shift(DYKX,WORK2)
        DYKX = (DYKX - WORK2)*DYUR
      else
        call n_shift(DYKX,WORK1)
        DYKX = DYKX - WORK1
        call e_shift(WORK2,DYKX)
        DYKX = p5*(DYKX + WORK2)*DYUR
      endif

      call s_shift(WORK1,HTN)
      WORK1 = (HTN - WORK1)*TAREA_R    ! KYT

      if (del_mom == 2) then
        call s_shift(WORK2,AMF)
        WORK2 = p5*(WORK2 + AMF)

        call e_shift(DYKY,WORK1)
        WORK2 = p5*(WORK1 + DYKY)*WORK2
        call n_shift(DYKY,WORK2)
        DYKY = (DYKY - WORK2)*DYUR
      else
        call n_shift(DYKY,WORK1)
        DYKY = DYKY - WORK1
        call e_shift(WORK2,DYKY)
        DYKY = p5*(DYKY + WORK2)*DYUR
      endif

      if (del_mom == 2) then
        call w_shift(WORK2,AMF)
        WORK2 = p5*(WORK2 + AMF)

        call n_shift(DXKY,WORK1)
        WORK2 = p5*(WORK1 + DXKY)*WORK2
        call e_shift(DXKY,WORK2)
        DXKY = (DXKY - WORK2)*DXUR
      else
        call e_shift(DXKY,WORK1)
        DXKY = DXKY - WORK1
        call n_shift(WORK2,DXKY)
        DXKY = p5*(DXKY + WORK2)*DXUR
      endif

      DUC = -(DUN + DUS + DUE + DUW)               ! scalar laplacian
      if (del_mom == 2) then
        DUM = -(DXKX + DYKY + c2*AMF*(KXU**2 + KYU**2))  ! metric terms
      else
        DUM = -(DXKX + DYKY + c2*(KXU**2 + KYU**2))  ! metric terms
      endif

!-----------------------------------------------------------------------
!
!     calculate central and {N,S,E,W} coefficients for 
!     metric mixing terms which mix U,V.
!
!-----------------------------------------------------------------------

      call e_shift(WORK3,HTN)
      call n_shift(WORK4,HTE)
      if (del_mom == 2) then
        call n_shift(WORK1,AMF)
        call s_shift(WORK2,AMF)
        WORK1 = (WORK1 - WORK2)/(HTE + WORK4)
        DME =  (c2*AMF*KYU + WORK1)/(HTN + WORK3)
        call e_shift(WORK1,AMF)
        call w_shift(WORK2,AMF)
        WORK1 = (WORK1 - WORK2)/(HTN + WORK3)
        DMN = -(c2*AMF*KXU + WORK1)/(HTE + WORK4)
      else
        DME =  c2*KYU/(HTN + WORK3)
        DMN = -c2*KXU/(HTE + WORK4)
      endif
      DMW = -DME
      DMS = -DMN
      DMC = DXKY - DYKX

!-----------------------------------------------------------------------

      end subroutine init_operators

!***********************************************************************

      subroutine del2t(k,D2TK)

!-----------------------------------------------------------------------
!
!     horizontal laplacian of tracers
!
!     Del**2(Tb) = dx*Dx[dy*Dx(Tb)]/da
!                + dy*Dy[dx*Dy(Tb)]/da
!
!     comments:
!
!   * boundary conditions are that gradients of tracers are zero
!     across lateral boundaries. 
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &   k                     ! vertical level

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,nt), intent(out) :: 
     &   D2TK                  ! Del**2(TRACER) at level k

!-----------------------------------------------------------------------
!
!     local variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n   ! dummy tracer index

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &   CC,CN,CS,CE,CW        ! coeff of 5pt stencil for Del**2

!-----------------------------------------------------------------------
!
!     implement boundary conditions by setting 
!     stencil coefficients to zero at land points.
!
!-----------------------------------------------------------------------

      CN = merge(DTN, c0, (k.le.KMTN).and.(k.le.KMT))
      CS = merge(DTS, c0, (k.le.KMTS).and.(k.le.KMT))
      CE = merge(DTE, c0, (k.le.KMTE).and.(k.le.KMT))
      CW = merge(DTW, c0, (k.le.KMTW).and.(k.le.KMT))

      CC = -(CN + CS + CE + CW)  ! central coefficient

!-----------------------------------------------------------------------
!
!     calculate Del**2(T) for each tracer n
!
!-----------------------------------------------------------------------

      do n = 1,nt
        call fivept_nobndy(D2TK(:,:,n),CC,CN,CS,CE,CW,
     &                     TRACER(:,:,k,n,mixtime))
      enddo

!-----------------------------------------------------------------------

      end subroutine del2t

!***********************************************************************

      subroutine del2u(k,D2UK,D2VK,UIN,VIN)

!-----------------------------------------------------------------------
!
!     horizontal laplacian of momentum
!
!     Del**2(Ub) = dx*Dx[dy*Dx(Ub)]/da
!                + dy*Dy[dx*Dy(Ub)]/da
!                - Ub*[dxkx + dyky + 2(kx**2 + ky**2)]   ! metric
!                + 2*hy*Dx(Vb) - 2*kx*Dy(Vb)             ! terms
!
!     Del**2(Vb) = dx*Dx[dy*Dx(Vb)]/da
!                + dy*Dy[dx*Dy(Vb)]/da
!                - Vb*[dxkx + dyky + 2(kx**2 + ky**2)]   ! metric
!                - 2*hy*Dx(Ub) + 2*kx*Dy(Ub)             ! terms
!
!     comments:
!
!   * boundary conditions are not explicitly imposed on
!     Del**2{(U,V)} since U = V = 0 on the boundaries.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! depth level

      real (kind=dbl_kind), dimension (imt,jmt), intent(in) :: 
     &   UIN,               ! input U,V velocities (which may be
     &   VIN                !   modified if topostress used

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension (imt,jmt), intent(out) :: 
     &   D2UK,              ! Del**2(UVEL) at level k
     &   D2VK               ! Del**2(VVEL) at level k

!-----------------------------------------------------------------------
!
!     local variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension (imt,jmt) :: 
     &   WORKX,WORKY,       ! local work space
     &   CC                 ! center fivept weight

!-----------------------------------------------------------------------
!
!     calculate Del**2(U) without metric terms that mix U,V
!
!-----------------------------------------------------------------------

      CC = DUC + DUM  ! add metric contribution to central coefficient

      call fivept_nobndy(D2UK,CC,DUN,DUS,DUE,DUW,UIN)

!-----------------------------------------------------------------------
!
!     calculate Del**2(V) without metric terms that mix U,V
!
!-----------------------------------------------------------------------

      call fivept_nobndy(D2VK,CC,DUN,DUS,DUE,DUW,VIN)

!-----------------------------------------------------------------------
!
!     add metric terms that mix U,V, and zero fields at land points
!
!-----------------------------------------------------------------------

      call fivept_nobndy(WORKX,DMC,DMN,DMS,DME,DMW,VIN)
      call fivept_nobndy(WORKY,DMC,DMN,DMS,DME,DMW,UIN)

      where (k.le.KMU)
        D2UK = D2UK + WORKX
        D2VK = D2VK - WORKY
      elsewhere
        D2UK = c0
        D2VK = c0
      endwhere

!-----------------------------------------------------------------------

      end subroutine del2u
 
!***********************************************************************

      subroutine del4t(k,D4TK,lvariable_hmix,AHF)

!-----------------------------------------------------------------------
!
!     horizontal biharmonic operator (Del**4) for tracers
!
!     Del**2(Tb) = dx*Dx[dy*Dx(Tb)]
!                + dy*Dy[dx*Dy(Tb)]
!
!     Del**4(Tb) = Del**2(Del**2(Tb))
!
!     comments:
!
!   * for tracers, the boundary conditions are the same on both
!     applications of the Del**2 operator:  the gradient of both
!     T and Del**2(T) vanishes across lateral boundaries.
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! vertical level

      logical (kind=log_kind), intent(in) ::
     &  lvariable_hmix         ! flag for spatial-variation of mixing

      real (kind=dbl_kind), dimension(:,:), intent(in) ::
     &  AHF                 ! spatial weighting of mixing coefficient

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,nt), intent(out) :: 
     &   D4TK               ! Del**4(T) of nth tracer at level k

!-----------------------------------------------------------------------
!
!     local variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n    ! dummy tracer index

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &   D2TK,              ! intermediate Del**2 result
     &   CC,CN,CS,CE,CW     ! five point stencil coefficients

!-----------------------------------------------------------------------
!
!     implement boundary conditions by setting 
!     stencil coefficients to zero at land points.
!
!-----------------------------------------------------------------------

      CN = merge(DTN, c0, (k.le.KMTN) .and. (k.le.KMT))
      CS = merge(DTS, c0, (k.le.KMTS) .and. (k.le.KMT))
      CE = merge(DTE, c0, (k.le.KMTE) .and. (k.le.KMT))
      CW = merge(DTW, c0, (k.le.KMTW) .and. (k.le.KMT))

      CC = -(CN + CS + CE + CW)  ! central coefficient

      do n = 1,nt
!-----------------------------------------------------------------------
!
!       calculate Del**2(T)
!
!-----------------------------------------------------------------------

        call fivept_nobndy(D2TK,CC,CN,CS,CE,CW,TRACER(:,:,k,n,mixtime))

        if (lvariable_hmix) then
          D2TK = AHF*D2TK  ! multiply by spatial depend of diffusivity
        endif

!-----------------------------------------------------------------------
!
!       calculate Del**2(Del**2(T))
!
!-----------------------------------------------------------------------

        call fivept_nobndy(D4TK(:,:,n),CC,CN,CS,CE,CW,D2TK)

      enddo

!-----------------------------------------------------------------------

      end subroutine del4t

!***********************************************************************

      subroutine del4u(k,D4UK,D4VK,lvariable_hmix,AMF)

!-----------------------------------------------------------------------
!
!     horizontal biharmonic operator (Del**4) for momentum
!
!     Del**2(Ub) = dx*Dx[dy*Dx(Ub)]/da
!                + dy*Dy[dx*Dy(Ub)]/da
!                - Ub*[dxkx _ dyky + 2(kx**2 + ky**2)]   ! metric
!                + 2*hy*Dx(Vb) - 2*kx*Dy(Vb)             ! terms
!
!     Del**2(Vb) = dx*Dx[dy*Dx(Vb)]/da
!                + dy*Dy[dx*Dy(Vb)]/da
!                - Vb*[dxkx _ dyky + 2(kx**2 + ky**2)]   ! metric
!                - 2*hy*Dx(Ub) + 2*kx*Dy(Ub)             ! terms
!
!     Del**4(Ub) = Del**2(Del**2(Ub))
!     Del**4(Vb) = Del**2(Del**2(Vb))
!
!     comments:
!
!   * boundary conditions are not explicitly imposed on
!     Del**2{(U,V)} since U = V = 0 on the boundaries.
!     in this version of the model, the  boundary conditions
!     on Del**2 acting on Del**2{(U,V)} are also that
!     Del**2{(U,V)} vanishes at land points
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! vertical level

      logical (kind=log_kind), intent(in) ::
     &  lvariable_hmix         ! flag for spatial-variation of mixing

      real (kind=dbl_kind), dimension(:,:), intent(in) ::
     &  AMF                 ! spatial weighting of mixing coefficient

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension (imt,jmt), intent(out) :: 
     &   D4UK,              ! Del**4(UVEL)
     &   D4VK               ! Del**4(VVEL)

!-----------------------------------------------------------------------
!
!     local variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension (imt,jmt) :: 
     &   D2UK,D2VK,         ! intermediate Del**2 results
     &   CC,                ! center 5-point weight
     &   WORKX,WORKY        ! local work space

!-----------------------------------------------------------------------
!
!     calculate Del**2(U,V) without metric terms that mix U,V
!
!-----------------------------------------------------------------------

      CC = DUC + DUM  ! add metric contribution to central coefficient

      call fivept_nobndy(D2UK,CC,DUN,DUS,DUE,DUW,UVEL(:,:,k,mixtime))
      call fivept_nobndy(D2VK,CC,DUN,DUS,DUE,DUW,VVEL(:,:,k,mixtime))

!-----------------------------------------------------------------------
!
!     add metric terms that mix U,V, and zero fields at land points
!
!-----------------------------------------------------------------------

      call fivept_nobndy(WORKX,DMC,DMN,DMS,DME,DMW,VVEL(:,:,k,mixtime))
      call fivept_nobndy(WORKY,DMC,DMN,DMS,DME,DMW,UVEL(:,:,k,mixtime))

      where (k.le.KMU)
        D2UK = D2UK + WORKX
        D2VK = D2VK - WORKY
      elsewhere
        D2UK = c0
        D2VK = c0
      endwhere

      if (lvariable_hmix) then
        D2UK = AMF*D2UK  ! multiply by spatial dependence
        D2Vk = AMF*D2VK  ! of viscocity
      endif

!-----------------------------------------------------------------------
!
!     calculate Del**2(Del**2(U,V)) without metric terms that mix U,V
!
!-----------------------------------------------------------------------

      call fivept_nobndy(D4UK,CC,DUN,DUS,DUE,DUW,D2UK)
      call fivept_nobndy(D4VK,CC,DUN,DUS,DUE,DUW,D2VK)

!-----------------------------------------------------------------------
!
!     add metric terms, and zero fields at land points
!
!-----------------------------------------------------------------------

      call fivept_nobndy(WORKX,DMC,DMN,DMS,DME,DMW,D2VK)
      call fivept_nobndy(WORKY,DMC,DMN,DMS,DME,DMW,D2UK)

      where (k.le.KMU)
        D4UK = D4UK + WORKX
        D4VK = D4VK - WORKY
      elsewhere
        D4UK = c0
        D4VK = c0
      endwhere

!-----------------------------------------------------------------------

      end subroutine del4u

!***********************************************************************

      subroutine div(k,DIVU,UX,UY)

!-----------------------------------------------------------------------
!
!     divergence operator on vector fields defined at U-points
!
!     Div(Ux,Uy) =  (1/dy)*{Dx[Ay(dy*Ux)] + (1/dx)*Dy[Ax(dx*Uy)]}
!
!     comments:
!
!   * this routine returns divergence times cell area 
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k   ! vertical level

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   UX,UY              ! vector field defined at U-points

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &   DIVU               ! divergence at T-points times cell area

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &   WORKX,WORKY        ! local work space

!-----------------------------------------------------------------------
!
!     compute divergence calling 4 point stencil
!
!-----------------------------------------------------------------------

      WORKX = p5*UX*DYU
      WORKY = p5*UY*DXU

      call div_4pt(DIVU,WORKX,WORKY)

      where (k.gt.KMT) DIVU = c0
      
!-----------------------------------------------------------------------

      end subroutine div

!***********************************************************************

      subroutine grad(k,GRADX,GRADY,F)

!-----------------------------------------------------------------------
!
!     gradient operator on fields defined at T-points
!
!     Gradx(F) = Dx{Ay(F)}
!     Grady(F) = Dy{Ax(F)}
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! vertical level

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   F                  ! field defined at T points

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &   GRADX,GRADY        ! gradient in (i,j) direction at U points

!-----------------------------------------------------------------------
!
!     compute gradient with call to 4 point stencil
!
!-----------------------------------------------------------------------
      
      call grad_4pt(GRADX,GRADY,F)

      where (k.le.KMU)
        GRADX = DXUR*p5*GRADX
        GRADY = DYUR*p5*GRADY
      elsewhere
        GRADX = c0    ! zero at land points
        GRADY = c0
      endwhere
      
!-----------------------------------------------------------------------

      end subroutine grad

!***********************************************************************

      subroutine grad_nobndy(k,GRADX,GRADY,F)

!-----------------------------------------------------------------------
!
!     gradient operator on fields defined at T-points
!     this version calls the grad_4pt stencils which does not
!     perform boundary updates - useful when boundary info unnecessary
!
!     Gradx(F) = Dx{Ay(F)}
!     Grady(F) = Dy{Ax(F)}
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! vertical level

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   F                  ! field defined at T points

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &   GRADX,GRADY        ! gradient in (i,j) direction at U points

!-----------------------------------------------------------------------
!
!     compute gradient with call to 4 point stencil
!
!-----------------------------------------------------------------------
      
      call grad_4pt_nobndy(GRADX,GRADY,F)

      where (k.le.KMU)
        GRADX = DXUR*p5*GRADX
        GRADY = DYUR*p5*GRADY
      elsewhere
        GRADX = c0    ! zero at land points
        GRADY = c0
      endwhere
      
!-----------------------------------------------------------------------

      end subroutine grad_nobndy

!***********************************************************************

      subroutine zcurl(k,ZCURLU,UX,UY)

!-----------------------------------------------------------------------
!
!     z.curl operator on vector fields defined at U-points
!
!     z.curl(Ux,Uy) =  (1/dy)*Dx[Ay(dy*Uy)] - (1/dx)*Dy[Ax(dx*Ux)]
!
!     comments:
!
!   * this routine returns z.curl times cell area 
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! vertical level

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &   UX,UY              ! vector field defined at U-points

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &   ZCURLU             ! z.curl(Ux,Uy) at T-points times cell area

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &   WORKX,WORKY        ! local work space

!-----------------------------------------------------------------------
!
!     compute curl by call divergence 4 point stencil
!
!-----------------------------------------------------------------------

      WORKX =  p5*UY*DYU
      WORKY = -p5*UX*DXU

      call div_4pt(ZCURLU,WORKX,WORKY)

      where (k.gt.KMT) ZCURLU = c0
      
!-----------------------------------------------------------------------

      end subroutine zcurl

!***********************************************************************

      subroutine wcalc(WVELT)

!-----------------------------------------------------------------------
!
!     calculates vertical velocity WVEL in T-columns from horizontal
!     velocity field
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,km), intent(out) :: 
     &  WVELT          ! vertical velocity at T points for all levels

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: k     ! vert level index

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  FVN,FVS,FUE,FUW,    ! advective fluxes across sides of box
     &  WTK,WTKB,           ! vertical velocity at top, bottom of box
     &  WORK                ! local work space

!-----------------------------------------------------------------------
!
!     initialize vertical velocities to zero
!
!-----------------------------------------------------------------------

      WTKB = c0
      WTK  = c0

!-----------------------------------------------------------------------
!
!     integrate from bottom up
!
!-----------------------------------------------------------------------

      do k = km,1,-1

!-----------------------------------------------------------------------
!
!       advection fluxes
!
!-----------------------------------------------------------------------

        WORK = p5*UVEL(:,:,k,curtime)*DYU
        call s_shift(FUE,WORK)
        FUE = WORK + FUE
        call w_shift(FUW,FUE)

        WORK = p5*VVEL(:,:,k,curtime)*DXU
        call w_shift(FVN,WORK)
        FVN = WORK + FVN
        call s_shift(FVS,FVN)

!-----------------------------------------------------------------------
!
!       calculate vertical velocity at top of kth level
!       (vertical velocity is zero at bottom of T columns)
!
!-----------------------------------------------------------------------

        WORK = (FVN - FVS + FUE - FUW)*TAREA_R   ! central coefficient

        where (k <= KMT)
          WTK = WTKB - dz(k)*WORK
        endwhere

        WVELT(:,:,k) = WTK

        WTKB = WTK   ! top value becomes bottom value for next pass

      enddo

!-----------------------------------------------------------------------

      end subroutine wcalc

!***********************************************************************

      end module operators

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
