!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module pressure_grad

!***********************************************************************
!
!     contains routines for computing the pressure gradient force
!     pressure averaging is also managed by this module
!
!     CVS:$Id: pressure_grad.F,v 1.3.2.2 2001/09/07 19:52:18 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use prognostic
      use operators
      use grid
      use io
      use global_reductions
      use boundary
      use state_mod
      use time_management
      use exit_mod

      implicit none
      save

!-----------------------------------------------------------------------
!
!     module variables
!
!-----------------------------------------------------------------------

      logical (kind=log_kind) ::
     &  lpressure_avg       ! flag to turn on averaging of
                            ! pressure across three time levels

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  SUMX, SUMY,         ! incremental k sum of Grad{x,y}(P(k)) 
     &  RHOKMX, RHOKMY      ! x,y gradient of density at k-1 level 

!-----------------------------------------------------------------------
!
!     The factor, r(p), removes the contribution of pressure-
!     dependent compressibility from the density and thereby 
!     improves the accuracy of the Boussinesq approximation 
!     for the pressure gradient.
!
!     See Dukowicz, J. K., 2000: Reduction of Pressure and
!     Pressure Gradient Errors in Ocean Simulations, J. Phys.
!     Oceanogr., submitted.
!
!-----------------------------------------------------------------------

      logical (kind=log_kind) ::
     &  lbouss_correct      ! flag for correction to Bouss. approx.

      real (kind=dbl_kind), dimension(km) ::
     &  bouss               ! 1/r(p) factor for correction to Bouss.

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_pressure_grad

!-----------------------------------------------------------------------
!
!     initializes pressure gradient options
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  k,                     ! vertical level index
     &  nml_error              ! namelist i/o error flag

      namelist /pressure_grad_nml/ lpressure_avg, lbouss_correct

!-----------------------------------------------------------------------
!
!     read options from input namelist and broadcast
!
!-----------------------------------------------------------------------

      lpressure_avg = .true.
      lbouss_correct = .false.

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=pressure_grad_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading pressure_grad_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, pressure_grad_nml)
          write(stdout,*) ' '
      endif

      if (my_task == master_task) then
        write(stdout,1000)
        write(stdout,1010)
        write(stdout,1020)
        write(stdout,1010)
        write(stdout,1000)

 1000   format(72('-'))
 1010   format(' ')
 1020   format(' Pressure gradient options')

        if (lpressure_avg) then
          write(stdout,*) ' Pressure averaging enabled'
        else
          write(stdout,*) ' Pressure averaging disabled'
        endif
        if (lbouss_correct) then
          write(stdout,*) ' Density correction enabled'
        else
          write(stdout,*) ' Density correction disabled'
        endif
      endif

      call broadcast_scalar(lpressure_avg,  master_task)
      call broadcast_scalar(lbouss_correct, master_task)

!-----------------------------------------------------------------------
!
!     Density correction factor at level k.
!
!-----------------------------------------------------------------------

      if (lbouss_correct) then
        do k=1,km
          bouss(k) = c1/(1.02819_dbl_kind + 
     &                   4.4004e-5_dbl_kind*pressz(k) -
     &                   2.93161e-4_dbl_kind*
     &                   exp(-0.05_dbl_kind*pressz(k)))
        end do
      else
        bouss(:) = c1
      endif

!-----------------------------------------------------------------------

      end subroutine init_pressure_grad

!***********************************************************************

      subroutine gradp(k,PKX,PKY)

!-----------------------------------------------------------------------
!
!     gradient of hydrostatic pressure at level k
!
!     Gradx(P(k)) = grav*SUM(m=1,k){dz(m-1/2)
!                              *[Gradx(Rho(m)) + Gradx(Rho(m-1))]/2}
!
!     Grady(P(k)) = grav*SUM(m=1,k){dz(m-1/2)
!                              *[Grady(Rho(m)) + Grady(Rho(m-1))]/2}
!
!     where P(k) is the hydrostatic pressure at level k, 
!     Rho(k) is the density at level k, and Rho(0) = Rho(1).
!
!   * this routine must be called successively with k = 1,2,3,...
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k   ! depth level index

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &  PKX,PKY             ! {x,y} components of the pressure gradient

!-----------------------------------------------------------------------
! 
!     local or common variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind) ::
     &  factor

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  RHOKX, RHOKY,       ! x,y gradients of level k density
     &  RHOAVG              ! avg density when pressure avg on

!-----------------------------------------------------------------------
!
!     gradient of density at level k
!
!-----------------------------------------------------------------------

      RHOKX = c0  ! initialize before passing as argument
      RHOKY = c0

!-----------------------------------------------------------------------
!
!     calculate density at new time for pressure averaging
!
!-----------------------------------------------------------------------
 
      if (lpressure_avg .and. leapfrogts) then

        RHOAVG = p25*(RHO(:,:,k,newtime) + c2*RHO(:,:,k,curtime) + 
     &                RHO(:,:,k,oldtime))*bouss(k)

      else
        RHOAVG = RHO(:,:,k,curtime)*bouss(k)
      endif

      call grad_nobndy(k,RHOKX,RHOKY,RHOAVG)

!-----------------------------------------------------------------------
!
!     set Rho(0) = Rho(1) at top level, 
!     initialize sum for pressure gradients to zero.
!
!-----------------------------------------------------------------------

      if (k == 1) then
        RHOKMX = RHOKX
        RHOKMY = RHOKY
        SUMX = c0
        SUMY = c0
      endif

!-----------------------------------------------------------------------
!
!     obtain pressure gradient by incrementing sum of density gradients
!     from top level down to level k.
!
!-----------------------------------------------------------------------

      factor = dzw(k-1)*grav*p5
      SUMX = SUMX + factor*(RHOKX + RHOKMX)
      SUMY = SUMY + factor*(RHOKY + RHOKMY)

      PKX  = SUMX
      PKY  = SUMY
      
!-----------------------------------------------------------------------
!
!     overwrite level k-1 with level k density gradients for next pass
!
!-----------------------------------------------------------------------
       
      RHOKMX = RHOKX
      RHOKMY = RHOKY
      
!-----------------------------------------------------------------------

      end subroutine gradp

!***********************************************************************

      end module pressure_grad

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
