!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module restart

!***********************************************************************

!-----------------------------------------------------------------------
!
!     module for reading and writing restart dumps
!
!     CVS:$Id: restart.F,v 1.3.2.12 2002/01/31 22:30:50 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use prognostic
      use global_reductions
      use operators
      use grid
      use io
      use time_management
      use ice
      use forcing
      use forcing_coupled
      use exit_mod

      implicit none
      save

!-----------------------------------------------------------------------
!
!     module variables
!
!-----------------------------------------------------------------------

      character (char_len_long) ::
     &  restart_outfile       ! restart output filename root
        

      logical (kind=log_kind) ::
     &  pressure_correction   ! fix pressure for exact restart
     &, lrestart_on           ! flag to turn restarts on/off
     &, leven_odd_on          ! flag to turn even_odd restarts on/off

      !***
      !*** variables for frequency of output
      !***

      integer (kind=int_kind) ::
     &  even_odd_freq         ! even/odd restart files every freq steps 
     &, last_even_odd         ! last even/odd dump
     &, restart_flag          ! time flag id for restarts
     &, restart_freq_iopt     ! restart frequency option
     &, restart_freq          ! restart frequency 

      integer (kind=int_kind), parameter ::
     &  even = 0,             ! integer for which even/odd dump
     &  odd  = 1

      integer (kind=int_kind), private ::
     &   restart_cpl_ts       ! integer reference to 'coupled_ts' time flag

!-----------------------------------------------------------------------
!
!     data to be written/read from restart header file - use
!     namelist for clarity due to mix of options etc
!
!   *  restart header file contains:
!
!      iyear, imonth, iday, ihour, iminute, isecond
!      iyear0, imonth0, iday0, ihour0, iminute0, isecond0
!      elapsed_days,elapsed_months,elapsed_years
!      nsteps_total
!      these are generally included, but are actually optional
!        unless the relevent options are turned on:
!         if (liceform) tlast_ice
!
!   *  also included are the most recent times to which forcing data
!        has been interpolated:
!        {shf/sfwf/ws/ap/pt_interior/s_interior}_interp_last
!
!   *  also included are variables associated with bulk-NCEP salinity
!        forcing used for precipitation adjustment:
!        sal_initial(km), sum_precip, precip_fact, ssh_initial
!
!-----------------------------------------------------------------------

!***  namelist is included as a local variable in the read and write
!***  routines as some compilers complain if it is a module variable
!      namelist /restart_hdr_nml/ 
!     &         runid
!     &,        iyear,  imonth,  iday,  ihour,  iminute,  isecond
!     &,        iyear0, imonth0, iday0, ihour0, iminute0, isecond0
!     &,        dtt,    iday_of_year,   iday_of_year_last
!     &,        elapsed_days, elapsed_months, elapsed_years
!     &,        elapsed_days_this_year
!     &,        seconds_this_day,  seconds_this_day_next
!     &,        seconds_this_year, seconds_this_year_next
!     &,        nsteps_total
!     &,        eod, eod_last, eom, eom_last, eom_next, eoy, eoy_last
!     &,        midnight_last, adjust_year_next, newday, newhour
!     &,        leapyear, days_in_year, days_in_prior_year
!     &,        seconds_in_year, hours_in_year
!     &,        tlast_ice
!     &,        lcoupled_ts
!     &,        shf_interp_last, sfwf_interp_last, ws_interp_last
!     &,        ap_interp_last, pt_interior_interp_last
!     &,        s_interior_interp_last
!     &,        sal_initial, sum_precip, precip_fact, ssh_initial

!***********************************************************************

      contains

!***********************************************************************

      subroutine read_restart(restart_filename,lredef_case,lredef_date)

!-----------------------------------------------------------------------
!
!     reads restart dump
!
!   * prognostic fields read are:
!
!       UBTROP,VBTROP  ! barotropic velocities
!       PSURF          ! surface pressure
!       GRADPX,GRADPY  ! surface pressure gradient
!       PGUESS         ! next guess for pressure
!       UVEL,VVEL      ! 3d velocities
!       TRACER         ! 3d tracers
!
!-----------------------------------------------------------------------

      implicit none

!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      character (char_len_long), intent(in) ::
     &   restart_filename

      logical (kind=log_kind), intent(in) ::
     &   lredef_case          ! flag for redefining case name
     &,  lredef_date          ! flag for redefining starting date

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   n,k,          ! dummy counters
     &   nu,           ! i/o unit for various reads
     &   kfw,          ! index into 2d buffer array for FW_OLD field
     &   kfw_freeze,   ! index into 2d buffer array for FW_FREEZE field
     &   kaqice,       ! index into 2d buffer array for AQICE/QFLUX field
     &   nfields_2d    ! number of 2d fields
     &,  cindx,cindx2  ! indices into character strings

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &   WORK1,WORK2        ! work space for pressure correction

      real (kind=dbl_kind), dimension(:,:,:), allocatable :: 
     &   WORK2D             ! buffer for reading all 2-d fields together

      character (char_len_long) ::
     &   header_file
     &,  restart_file
     &,  restart_pointer_file
 
      character (char_len) ::
     &   save_runid   

      logical (kind=log_kind) ::
     &   lcoupled_ts  

      namelist /restart_hdr_nml/ 
     &         runid
     &,        iyear,  imonth,  iday,  ihour,  iminute,  isecond
     &,        iyear0, imonth0, iday0, ihour0, iminute0, isecond0
     &,        dtt,    iday_of_year,   iday_of_year_last
     &,        elapsed_days, elapsed_months, elapsed_years
     &,        elapsed_days_this_year
     &,        seconds_this_day,  seconds_this_day_next
     &,        seconds_this_year, seconds_this_year_next
     &,        nsteps_total
     &,        eod, eod_last, eom, eom_last, eom_next, eoy, eoy_last
     &,        midnight_last, adjust_year_next, newday, newhour
     &,        leapyear, days_in_year, days_in_prior_year
     &,        seconds_in_year, hours_in_year
     &,        tlast_ice
     &,        lcoupled_ts
     &,        shf_interp_last, sfwf_interp_last, ws_interp_last
     &,        ap_interp_last, pt_interior_interp_last
     &,        s_interior_interp_last
     &,        sal_initial, sum_precip, precip_fact, ssh_initial

!-----------------------------------------------------------------------
!
!     if pointer files are used, pointer file must be read to get 
!     actual filenames 
!
!-----------------------------------------------------------------------

      header_file = char_blank
      restart_file = char_blank
      restart_pointer_file = char_blank

      if (luse_pointer_files ) then
        call get_unit(nu)
 
        if (my_task == master_task) then
          restart_pointer_file = pointer_filename
          cindx = len_trim(pointer_filename) + 1
          cindx2= cindx + 7
          restart_pointer_file(cindx:cindx2) = '.restart'
          write(stdout,*) 'Reading pointer file: ',
     &                    trim(restart_pointer_file)
          open(nu, file=restart_pointer_file, form='formatted', 
     &             status='old')
          read(nu,1000) header_file
          read(nu,1000) restart_file
          close(nu)
        endif
        call release_unit(nu)

        call broadcast_scalar(header_file,  master_task)
        call broadcast_scalar(restart_file, master_task)

 1000 format(a)

!-----------------------------------------------------------------------
!
!     otherwise create filenames
!
!-----------------------------------------------------------------------

      else
        cindx2 = len_trim(restart_filename)
        restart_file(1:cindx2) = trim(restart_filename)

        cindx = 1
        cindx2= len_trim(restart_file)
        header_file(cindx:cindx2) = trim(restart_file)
        cindx = cindx2 + 1
        cindx2= cindx + 3
        header_file(cindx:cindx2) = '.hdr'
      endif

!-----------------------------------------------------------------------
!
!     read scalar quantities from header file 
!
!-----------------------------------------------------------------------

      call get_unit(nu)

      if (my_task == master_task) then

        !*** set some defaults for namelist variables not initialized
        !*** under some options

        tlast_ice = c0
        lcoupled_ts = .false.

        if (lredef_date) then
          ! don't actually read header file -- CCSM 'hybrid' case
        else
          write(stdout,*) ' attempting to read header file: ', 
     &                      trim(header_file)
        endif

        !*** if the user wants to use a casename other than that which is
        !*** written on the restart header file, then save "runid" here
        !*** and overwrite the value read from the restart file later
          
        if (lredef_case) then
            save_runid = runid
        endif
 
        
        if (lredef_date) then
            !CCSM RUNTYPE = 'hybrid'
            !if the dates from the restart file are NOT to be used, 
            ! do not read the header file at all -- use the 
            ! information from subroutine init_time_manager1 instead
        else
           open(nu,file=header_file,status='old')
           read (nu,nml=restart_hdr_nml)
           close(nu)

           write(stdout,*) ' '
           write(stdout,*) ' file read: ', trim(header_file)
        endif
 
 
        !*** if the runid from the restart file is NOT to be used, 
        !***   override it here
          
        if (lredef_case) then
            runid   = save_runid
        endif


      endif
 
      call release_unit(nu)


      if (.not. lredef_date) then
      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, restart_hdr_nml)
          write(stdout,*) ' '
      endif

!-----------------------------------------------------------------------
!
!       Broadcast header file information to other processors
!
!-----------------------------------------------------------------------

        call broadcast_scalar(runid,                   master_task)
        call broadcast_scalar(iyear,                   master_task)
        call broadcast_scalar(imonth,                  master_task)
        call broadcast_scalar(iday,                    master_task)
        call broadcast_scalar(ihour,                   master_task)
        call broadcast_scalar(iminute,                 master_task)
        call broadcast_scalar(isecond,                 master_task)
        call broadcast_scalar(iyear0,                  master_task)
        call broadcast_scalar(imonth0,                 master_task)
        call broadcast_scalar(iday0,                   master_task)
        call broadcast_scalar(ihour0,                  master_task)
        call broadcast_scalar(iminute0,                master_task)
        call broadcast_scalar(isecond0,                master_task)
        call broadcast_scalar(dtt,                     master_task)
        call broadcast_scalar(iday_of_year,            master_task)
        call broadcast_scalar(iday_of_year_last,       master_task)
        call broadcast_scalar(elapsed_days,            master_task)
        call broadcast_scalar(elapsed_months,          master_task)
        call broadcast_scalar(elapsed_years,           master_task)
        call broadcast_scalar(elapsed_days_this_year,  master_task)
        call broadcast_scalar(seconds_this_day,        master_task)
        call broadcast_scalar(seconds_this_day_next,   master_task)
        call broadcast_scalar(seconds_this_year,       master_task)
        call broadcast_scalar(seconds_this_year_next,  master_task)
        call broadcast_scalar(nsteps_total,            master_task)
        call broadcast_scalar(eod,                     master_task)
        call broadcast_scalar(eod_last,                master_task)
        call broadcast_scalar(eom,                     master_task)
        call broadcast_scalar(eom_last,                master_task)
        call broadcast_scalar(eom_next,                master_task)
        call broadcast_scalar(eoy,                     master_task)
        call broadcast_scalar(eoy_last,                master_task)
        call broadcast_scalar(midnight_last,           master_task)
        call broadcast_scalar(adjust_year_next,        master_task)
        call broadcast_scalar(newday,                  master_task)
        call broadcast_scalar(newhour,                 master_task)
        call broadcast_scalar(leapyear,                master_task)
        call broadcast_scalar(days_in_year,            master_task)
        call broadcast_scalar(days_in_prior_year,      master_task)
        call broadcast_scalar(seconds_in_year,         master_task)
        call broadcast_scalar(hours_in_year,           master_task)
        call broadcast_scalar(tlast_ice,               master_task)
        call broadcast_scalar(lcoupled_ts,             master_task)
        call broadcast_scalar(shf_interp_last,         master_task)
        call broadcast_scalar(sfwf_interp_last,        master_task)
        call broadcast_scalar(ws_interp_last,          master_task)
        call broadcast_scalar(ap_interp_last,          master_task)
        call broadcast_scalar(pt_interior_interp_last, master_task)
        call broadcast_scalar(s_interior_interp_last,  master_task)
        call broadcast_vector(sal_initial,             master_task, km)
        call broadcast_scalar(sum_precip,              master_task)
        call broadcast_scalar(precip_fact,             master_task)
        call broadcast_scalar(ssh_initial,             master_task)

        call int_to_char (4,iyear,cyear)
        call int_to_char (2,imonth,cmonth)
        call int_to_char (2,iday,cday)
        cmonth3 = month3_all(imonth)

!-----------------------------------------------------------------------
!
!       set old value for the time flag 'coupled_ts'
!
!-----------------------------------------------------------------------

        if (lcoupled_ts) then
          restart_cpl_ts = init_time_flag('coupled_ts')
          call set_time_flag_last (restart_cpl_ts, lcoupled_ts)
        endif
 
      endif ! lredef_date

!-----------------------------------------------------------------------
!
!     allocate array for reading 2d fields
!
!-----------------------------------------------------------------------

      nfields_2d = 11     ! standard set of 2d fields
      if (sfc_layer_type == sfc_layer_varthick) then
        nfields_2d = nfields_2d + 1
        kfw = nfields_2d
        nfields_2d = nfields_2d + 1
        kfw_freeze = nfields_2d
      endif
      if (liceform) then
        nfields_2d = nfields_2d + 1
        kaqice = nfields_2d
      endif

      allocate(WORK2D(imt,jmt,nfields_2d))

!-----------------------------------------------------------------------
!
!     read prognostic arrays
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
          write(stdout,*) ' attempting to read restart file: ',
     &                      trim(restart_file)
      endif
 
      call open_parallel_file(nu,restart_file,recl_dbl)

      call read_array(nu,WORK2D)

      UBTROP(:,:,curtime) = WORK2D(:,:,1)
      UBTROP(:,:,oldtime) = WORK2D(:,:,2)
      VBTROP(:,:,curtime) = WORK2D(:,:,3)
      VBTROP(:,:,oldtime) = WORK2D(:,:,4)
      PSURF (:,:,curtime) = WORK2D(:,:,5)
      PSURF (:,:,oldtime) = WORK2D(:,:,6)
      GRADPX(:,:,curtime) = WORK2D(:,:,7)
      GRADPX(:,:,oldtime) = WORK2D(:,:,8)
      GRADPY(:,:,curtime) = WORK2D(:,:,9)
      GRADPY(:,:,oldtime) = WORK2D(:,:,10)
      PGUESS              = WORK2D(:,:,11)
      if (sfc_layer_type == sfc_layer_varthick) then
        FW_OLD            = WORK2D(:,:,kfw)
        FW_FREEZE         = WORK2D(:,:,kfw_freeze)
      endif
      if (liceform) then
!-------------------------------------------------------------------------
!       should test lcoupled, but it isn't set until after read_restart
!       is called. Because in a coupled run, lcoupled_ts = .true. in the
!       restart file, it is used instead.
!-------------------------------------------------------------------------
        if (lcoupled_ts) then   
          QFLUX = WORK2D(:,:,kaqice)
        else
          AQICE = WORK2D(:,:,kaqice)
        endif
      endif
      deallocate(WORK2D)

      call read_array(nu,  UVEL(:,:,:,curtime))
      call read_array(nu,  UVEL(:,:,:,oldtime))
      call read_array(nu,  VVEL(:,:,:,curtime))
      call read_array(nu,  VVEL(:,:,:,oldtime))
      call read_array(nu,TRACER(:,:,:,1,curtime))
      call read_array(nu,TRACER(:,:,:,2,curtime))
      call read_array(nu,TRACER(:,:,:,1,oldtime))
      call read_array(nu,TRACER(:,:,:,2,oldtime))

      call close_parallel_file(nu)

      if (my_task == master_task) then
        write(stdout,*) ' '
        write(stdout,*) ' successfully completed reading: ', 
     &                      trim(restart_file)
      endif

!-----------------------------------------------------------------------
!
!     zero prognostic variables at land points
!
!-----------------------------------------------------------------------

      where (.not. CALCU)
        UBTROP(:,:,curtime) = c0
        VBTROP(:,:,curtime) = c0
        GRADPX(:,:,curtime) = c0
        GRADPY(:,:,curtime) = c0
        UBTROP(:,:,oldtime) = c0
        VBTROP(:,:,oldtime) = c0
        GRADPX(:,:,oldtime) = c0
        GRADPY(:,:,oldtime) = c0
      endwhere

      where (.not. CALCT)
        PSURF(:,:,curtime) = c0
        PSURF(:,:,oldtime) = c0
        PGUESS = c0
      endwhere

      if (sfc_layer_type == sfc_layer_varthick) then
        where (.not. CALCT) 
          FW_OLD    = c0
          FW_FREEZE = c0
        endwhere
      endif

      do k = 1,km
        where (k.gt.KMU)
          UVEL(:,:,k,curtime) = c0
          VVEL(:,:,k,curtime) = c0
        endwhere
      enddo

      do n = 1,nt
       do k = 1,km
         where (k.gt.KMT)
           TRACER(:,:,k,n,curtime) = c0
           TRACER(:,:,k,n,oldtime) = c0
         endwhere
       enddo
      enddo

!-----------------------------------------------------------------------
!
!     reset PSURF(oldtime) to eliminate error in barotropic continuity 
!     eqn due to (possible) use of different timestep after restart
!
!     NOTE: use pressure_correction = .false. for exact restart
!
!-----------------------------------------------------------------------

      if (pressure_correction) then
         WORK1 = HU*UBTROP(:,:,curtime)
         WORK2 = HU*VBTROP(:,:,curtime)
         !*** use PSURF(oldtime) as temp
         call div(1,PSURF(:,:,oldtime),WORK1,WORK2)
         PSURF(:,:,oldtime) = PSURF(:,:,curtime) + 
     &                        grav*dtp*PSURF(:,:,oldtime)*TAREA_R
      endif

!-----------------------------------------------------------------------

      end subroutine read_restart

!***********************************************************************

      subroutine write_restart(file_suffix)

!-----------------------------------------------------------------------
!
!     write restart dump
!
!   * prognostic fields written are:
!
!       UBTROP,VBTROP  ! barotropic velocities
!       PSURF          ! surface pressure
!       GRADPX,GRADPY  ! surface pressure gradient
!       PGUESS         ! next guess for pressure
!       UVEL,VVEL      ! 3d velocities
!       TRACER         ! 3d tracers
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input files
!
!-----------------------------------------------------------------------

      character (char_len), intent(in) ::
     &   file_suffix        ! suffix to append to root filename

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   n,k,          ! dummy counters
     &   nu,           ! i/o unit for various reads
     &   kfw,          ! index into 2d buffer array for FW_OLD field
     &   kfw_freeze,   ! index into 2d buffer array for FW_FREEZE field
     &   kaqice,       ! index into 2d buffer array for AQICE/QFLUX field
     &   nfields_2d    ! total number of 2d fields
     &,  cindx,cindx2  ! indices into character strings

      logical (kind=log_kind) ::
     &   lcoupled_ts  

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &   WORK1,WORK2        ! work space for pressure correction

      real (kind=dbl_kind), dimension(:,:,:), allocatable :: 
     &   WORK2D             ! buffer for reading all 2-d fields together

      character (char_len_long) ::
     &   header_filename,   ! filename for restart header file
     &   restart_filename   ! filename for restart binary data
     &,  restart_pointer_file ! file containing loc/name of restart file

      namelist /restart_hdr_nml/ 
     &         runid
     &,        iyear,  imonth,  iday,  ihour,  iminute,  isecond
     &,        iyear0, imonth0, iday0, ihour0, iminute0, isecond0
     &,        dtt,    iday_of_year,   iday_of_year_last
     &,        elapsed_days, elapsed_months, elapsed_years
     &,        elapsed_days_this_year
     &,        seconds_this_day,  seconds_this_day_next
     &,        seconds_this_year, seconds_this_year_next
     &,        nsteps_total
     &,        eod, eod_last, eom, eom_last, eom_next, eoy, eoy_last
     &,        midnight_last, adjust_year_next, newday, newhour
     &,        leapyear, days_in_year, days_in_prior_year
     &,        seconds_in_year, hours_in_year
     &,        tlast_ice
     &,        lcoupled_ts
     &,        shf_interp_last, sfwf_interp_last, ws_interp_last
     &,        ap_interp_last, pt_interior_interp_last
     &,        s_interior_interp_last
     &,        sal_initial, sum_precip, precip_fact, ssh_initial

!-----------------------------------------------------------------------
!
!     query time_flag routine for present value of lcoupled_ts
!
!-----------------------------------------------------------------------

      lcoupled_ts = check_time_flag (restart_cpl_ts)

!-----------------------------------------------------------------------
!
!     write scalar time quantities to header file
!
!-----------------------------------------------------------------------

 
      call get_unit(nu)

      if (my_task == master_task) then

        header_filename = char_blank
        if (trim(file_suffix) .ne. char_blank) then
           write(header_filename,1000)trim(restart_outfile),'.'
     &,                      trim(file_suffix), '.hdr'
        else
           write(header_filename,1000)trim(restart_outfile),'.hdr'
        endif

        open(nu,file=header_filename,status='unknown',
     &          delim='apostrophe')
        write(nu,nml=restart_hdr_nml)
        close(nu)

      endif
 
      call document('write_restart' 
     &,             'file written: '//trim(header_filename))
      call release_unit(nu)

!-----------------------------------------------------------------------
!
!     create filename for user-supplied root and append date
!
!-----------------------------------------------------------------------

      restart_filename = char_blank
      
      if (trim(file_suffix) .ne. char_blank) then
         write(restart_filename,1000)trim(restart_outfile),'.'
     &,                    trim(file_suffix)
      else
         write(restart_filename,1000)trim(restart_outfile)
      endif



!-----------------------------------------------------------------------
!
!     write parallel arrays
!
!-----------------------------------------------------------------------

      call open_parallel_file(nu,restart_filename,recl_dbl)

!-----------------------------------------------------------------------
!
!     allocate array for reading 2d fields
!
!-----------------------------------------------------------------------

      nfields_2d = 11     ! standard set of 2d fields
      if (sfc_layer_type == sfc_layer_varthick) then
        nfields_2d = nfields_2d + 1
        kfw = nfields_2d
        nfields_2d = nfields_2d + 1
        kfw_freeze = nfields_2d
      endif
      if (liceform) then
        nfields_2d = nfields_2d + 1
        kaqice = nfields_2d
      endif

      allocate(WORK2D(imt,jmt,nfields_2d))

!-----------------------------------------------------------------------
!
!     write 2-d arrays
!
!-----------------------------------------------------------------------

      WORK2D(:,:, 1) = UBTROP(:,:,curtime)
      WORK2D(:,:, 2) = UBTROP(:,:,oldtime)
      WORK2D(:,:, 3) = VBTROP(:,:,curtime)
      WORK2D(:,:, 4) = VBTROP(:,:,oldtime)
      WORK2D(:,:, 5) = PSURF (:,:,curtime)
      WORK2D(:,:, 6) = PSURF (:,:,oldtime)
      WORK2D(:,:, 7) = GRADPX(:,:,curtime)
      WORK2D(:,:, 8) = GRADPX(:,:,oldtime)
      WORK2D(:,:, 9) = GRADPY(:,:,curtime)
      WORK2D(:,:,10) = GRADPY(:,:,oldtime)
      WORK2D(:,:,11) = PGUESS 
      if (sfc_layer_type == sfc_layer_varthick) then
        WORK2D(:,:,kfw)        = FW_OLD
        WORK2D(:,:,kfw_freeze) = FW_FREEZE
      endif
      if (liceform) then
        if (lcoupled) then
          WORK2D(:,:,kaqice) = QFLUX
        else
          WORK2D(:,:,kaqice) = AQICE
        endif
      endif

      call write_array(nu,WORK2D)
      deallocate(WORK2D)

!-----------------------------------------------------------------------
!
!     write 3-d arrays
!
!-----------------------------------------------------------------------

      call write_array(nu,  UVEL(:,:,:,curtime))
      call write_array(nu,  UVEL(:,:,:,oldtime))
      call write_array(nu,  VVEL(:,:,:,curtime))
      call write_array(nu,  VVEL(:,:,:,oldtime))
      call write_array(nu,TRACER(:,:,:,1,curtime))
      call write_array(nu,TRACER(:,:,:,2,curtime))
      call write_array(nu,TRACER(:,:,:,1,oldtime))
      call write_array(nu,TRACER(:,:,:,2,oldtime))

      call close_parallel_file(nu)

      call document('write_restart' 
     &,             'file written: '//trim(restart_filename))

!-----------------------------------------------------------------------
!
!     if pointer files are used, write filenames to pointer file
!
!-----------------------------------------------------------------------

      if (luse_pointer_files) then
        call get_unit(nu)
        if (my_task == master_task) then
          restart_pointer_file = pointer_filename
          cindx = len_trim(pointer_filename) + 1
          cindx2= cindx + 7
          restart_pointer_file(cindx:cindx2) = '.restart'

          open(nu, file=restart_pointer_file, form='formatted', 
     &             status='unknown')
          write(nu,1000) header_filename
          write(nu,1000) restart_filename
          close(nu)
        endif
        call release_unit(nu)
      endif


 1000 format(a,:,a,:,a,:,a)

!-----------------------------------------------------------------------

      end subroutine write_restart

!***********************************************************************

      subroutine init_restart

!-----------------------------------------------------------------------
!
!     initializes restart quantities
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input namelist
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  nml_error             ! namelist i/o error flag

      character (char_len) ::
     &  restart_freq_opt    ! input option for freq of restart dumps

      namelist /restart_nml/ restart_freq_opt, restart_freq,
     &                       restart_outfile, 
     &                       leven_odd_on, even_odd_freq,
     &                       pressure_correction

!-----------------------------------------------------------------------
!
!     read namelist input and broadcast variables
!
!-----------------------------------------------------------------------

      restart_outfile   = 'd'
      restart_freq_iopt = freq_opt_never
      restart_freq      = 100000
      leven_odd_on      = .false.
      even_odd_freq     = 100000
      pressure_correction = .false.

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=restart_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading restart_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, restart_nml)
          write(stdout,*) ' '
      endif

      if (my_task == master_task) then
        select case (restart_freq_opt)
        case ('never')
          restart_freq_iopt = freq_opt_never
        case ('nyear')
          restart_freq_iopt = freq_opt_nyear
        case ('nmonth')
          restart_freq_iopt = freq_opt_nmonth
        case ('nday')
          restart_freq_iopt = freq_opt_nday
        case ('nhour')
          restart_freq_iopt = freq_opt_nhour
        case ('nsecond')
          restart_freq_iopt = freq_opt_nsecond
        case ('nstep')
          restart_freq_iopt = freq_opt_nstep
        case default
          restart_freq_iopt = -1000
        end select

      endif

      call broadcast_scalar (restart_outfile,      master_task)
      call broadcast_scalar (restart_freq_iopt,    master_task)
      call broadcast_scalar (restart_freq,         master_task)
      call broadcast_scalar (leven_odd_on,         master_task)
      call broadcast_scalar (even_odd_freq,        master_task)
      call broadcast_scalar (pressure_correction,  master_task)

      if (restart_freq_iopt == -1000) then
        call exit_POP('unknown restart frequency option')
      else if (restart_freq_iopt == freq_opt_never) then
        lrestart_on = .false.
      else
        lrestart_on = .true.
      endif

      if (leven_odd_on) last_even_odd = even

      restart_flag   = init_time_flag('restart',default=.false.,
     &                               freq_opt = restart_freq_iopt,
     &                               freq     = restart_freq)

      restart_cpl_ts = init_time_flag('coupled_ts')

!-----------------------------------------------------------------------

      end subroutine init_restart

!***********************************************************************

      end module restart

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
