!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module solvers

!***********************************************************************
!
!     This module contains arrays and routines necessary for the
!     barotropic solvers.
!
!     CVS:$Id: solvers.F,v 1.2.2.3 2001/09/12 22:20:00 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use global_reductions
      use stencils
      use grid
      use io
      use time_management
      use exit_mod

      implicit none
      save

!-----------------------------------------------------------------------
!
!       A{C,N,S,E,W,NE,SE,NW,SW} coefficients of 9-point stencil
!       for barotropic operator
!       PC{C,N,S,E,W,NE,SE,NW,SW} coefficients of 9-point stencil
!         for preconditioner
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension (imt,jmt) :: 
     &   A0,AC,AN,AS,AE,AW
     &  ,ANE,ASE,ANW,ASW 

      real (kind=dbl_kind), dimension (:,:), allocatable :: 
     &  PCC,PCN,PCS,PCE,PCW
     & ,PCNE,PCSE,PCNW,PCSW 

!-----------------------------------------------------------------------
!
!     scalar convergence-related variables
!
!-----------------------------------------------------------------------

      logical (kind=log_kind) ::
     &   lprecond         ! true if computed preconditioner to be used

      real (kind=dbl_kind) ::
     &   solv_convrg,     ! convergence error criterion
     &   sor,             ! for jacobi solver
     &   resid_norm,      ! residual normalization
     &   rms_residual     ! residual (also a diagnostic)

      integer (kind=int_kind), parameter ::
     &   solv_pcg = 1,  ! predefined solver types
     &   solv_cgr = 2,
     &   solv_jac = 3

      integer (kind=int_kind) ::
     &   solv_itype,      ! integer solver method (1=pcg, 2=cgr, 3=jac)
     &   solv_max_iters,  ! max number of iterations for solver
     &   solv_sum_iters,  ! accumulated no of iterations (diagnostic)
     &   solv_ncheck      ! check convergence every ncheck iterations

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_solvers

!-----------------------------------------------------------------------
!     calculate the coefficients of the 9-point stencils for the
!     operator in the elliptic equation for the surface pressure.
!-----------------------------------------------------------------------
!     equation:
!
!     A*F = B   (A = operator, F = field, B = r.h.s.)
!
!     A*F = da*Div{H*Grad(F)}   (da = dx*dy, cell area)
!
!     da*Div{H*Grad(F)} = dx*Dx{Ay[H*dy*Dx{Ay(F)}]}
!                       + dy*Dy{Ax[H*dx*Dy{Ax(F)}]} 
!-----------------------------------------------------------------------
!     comments:
!
!   * this routine is only called once, during initialization.
!
!   * in the free-surface formulation, an extra term is added to
!     the diagonal in 'barotropic_fs'.
!-----------------------------------------------------------------------
!     output variables:
!
!       A{C,N,S,E,W,NE,SE,NW,SW} coefficients of 9-point stencil
!
!       PC{C,N,S,E,W,NE,SE,NW,SW} coefficients of 9-point stencil
!
!     local or common variables:
!
!       {X,Y}{NE,SE,NW,SW} = contribution to {ne,se,nw,sw} coefficients 
!         from {x,y} components of divergence
!       HU = depth at U points
!
!-----------------------------------------------------------------------

      implicit none

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  XNE,XSE,XNW,XSW,YNE,YSE,YNW,YSW

      character (char_len) ::
     &   solv_type        ! which solver to use

      character (char_len) ::
     &   precond_file     ! file containing preconditioner

      namelist /solver_nml/ solv_convrg, solv_max_iters, solv_ncheck,
     &                      lprecond, solv_type, precond_file

      integer (kind=int_kind) ::
     &   n,               ! dummy counter
     &   nu,              ! I/O unit number and status flag
     &   nml_error        ! namelist i/o error flag

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   WORK             ! temp space for normalization

      integer (kind=int_kind), dimension(:,:), allocatable :: 
     &  IWORK1, IWORK2    ! local work arrays

      logical (kind=log_kind), dimension(:,:), allocatable :: 
     &  MLANDNE, MLANDNW, MLANDSE, MLANDSW,    ! check land masks
     &  LCHECK

!-----------------------------------------------------------------------
!
!     read solver choice and solver constants from namelist input
!     (namelist input file opened in initial.F)
!
!-----------------------------------------------------------------------

      solv_convrg    = eps
      solv_max_iters = 1000
      solv_ncheck    = 10
      lprecond       = .false.
      solv_type      = 'pcg'
      precond_file   = 'empty'

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=solver_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading solver_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, solver_nml)
          write(stdout,*) ' '
      endif

      if (my_task == master_task) then
        write(stdout,1000)
        write(stdout,1010)
        write(stdout,1020)
        write(stdout,1010)
        write(stdout,1000)

        write(stdout,*) ' Solver type:'
        select case(solv_type(1:3))
        case('pcg')
          write(stdout,*) '  preconditioned conjugate gradient'
          solv_itype = solv_pcg
        case('cgr')
          write(stdout,*) '  Conjugate Gradient Residual'
          solv_itype = solv_cgr
        case('jac')
          write(stdout,*) '  Jacobi'
          solv_itype = solv_jac
        case default
          solv_itype = -1000
        end select

        write(stdout,1030) solv_convrg
        write(stdout,1040) solv_max_iters
        write(stdout,1050) solv_ncheck

 1000   format(72('-'))
 1010   format(' ')
 1020   format(' Solver options (barotropic solver)')
 1030   format(' Solver converged for err < ',1pe12.5)
 1040   format(' Solver maximum iterations = ',i6) 
 1050   format(' Solver convergence checked every ',i6,
     &           ' iterations') 

      endif

      call broadcast_scalar(solv_convrg,    master_task)
      call broadcast_scalar(solv_max_iters, master_task)
      call broadcast_scalar(solv_ncheck,    master_task)
      call broadcast_scalar(lprecond,       master_task)
      call broadcast_scalar(solv_itype,     master_task)
      call broadcast_scalar(precond_file,   master_task)

      if (solv_itype == -1000) then
        call exit_POP('unknown solver type: must be pcg, cgr or jacobi')
      endif

!-----------------------------------------------------------------------
!
!     calculate normalization constant (darea,darea) for rms_residual
!     in cgr routine.
!
!-----------------------------------------------------------------------

      WORK = TAREA**2
      resid_norm = c1/global_sum(WORK,RCALCT)
      solv_convrg = solv_convrg**2/resid_norm

!-----------------------------------------------------------------------
!
!     set sor for jacobi solver
!
!-----------------------------------------------------------------------

      sor = p25     ! should be < 1/2

!-----------------------------------------------------------------------
!
!     compute nine point operator coefficients
!
!-----------------------------------------------------------------------

      XNE = p25*HU*DXUR*DYU
      YNE = p25*HU*DYUR*DXU

      call s_shift(XSE,XNE)
      call w_shift(XNW,XNE)
      call s_shift(XSW,XNW)

      call s_shift(YSE,YNE)
      call w_shift(YNW,YNE)
      call s_shift(YSW,YNW)

      ANE = XNE + YNE
      ASE = XSE + YSE
      ANW = XNW + YNW
      ASW = XSW + YSW

      AE  = XNE + XSE - YNE - YSE
      AW  = XNW + XSW - YNW - YSW
      AN  = YNE + YNW - XNE - XNW
      AS  = YSE + YSW - XSE - XSW

      AC  = -(ANE + ASE + ANW + ASW)   ! diagonal coefficient

      A0  = AC  ! initialize

!-----------------------------------------------------------------------
!
!     setup preconditioner if required
!
!-----------------------------------------------------------------------

      if (lprecond) then

        if (my_task == master_task) then
          write(stdout,*) ' Preconditioner read from file: ',
     &                    trim(precond_file)
        endif

        allocate ( PCC     (imt,jmt),
     &             PCN     (imt,jmt),
     &             PCS     (imt,jmt),
     &             PCE     (imt,jmt),
     &             PCW     (imt,jmt),
     &             PCNE    (imt,jmt),
     &             PCSE    (imt,jmt),
     &             PCNW    (imt,jmt),
     &             PCSW    (imt,jmt),
     &             IWORK1  (imt,jmt), 
     &             IWORK2  (imt,jmt),
     &             MLANDNE (imt,jmt),
     &             MLANDSE (imt,jmt),
     &             MLANDNW (imt,jmt),
     &             MLANDSW (imt,jmt))

!-----------------------------------------------------------------------
!
!       read preconditioner and check that it is consistent with
!       KMU field
!
!-----------------------------------------------------------------------

        call open_parallel_file(nu,precond_file,recl_dbl)
        call read_array(nu,PCC)
        call read_array(nu,PCN)
        call read_array(nu,PCS)
        call read_array(nu,PCE)
        call read_array(nu,PCW)
        call read_array(nu,PCNE)
        call read_array(nu,PCNW)
        call read_array(nu,PCSE)
        call read_array(nu,PCSW)
        call close_parallel_file(nu)

        if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' file read: ', precond_file
        endif

!-----------------------------------------------------------------------
!
!       check that PC is consistent with KMU field
!
!-----------------------------------------------------------------------

        IWORK1 = 0  ! initialize

        where (KMU == 0) 
          MLANDNE = .true.
        elsewhere
          MLANDNE = .false.
        end where

        call w_shift(IWORK1,KMU)

        where (IWORK1 == 0) 
          MLANDNW = .true.
        elsewhere
          MLANDNW = .false.
        end where

        call s_shift(IWORK1,KMU)

        where (IWORK1 == 0) 
          MLANDSE = .true.
        elsewhere
          MLANDSE = .false.
        end where

        call w_shift(IWORK2,IWORK1)

        where (IWORK2 == 0) 
          MLANDSW = .true.
        elsewhere
          MLANDSW = .false.
        end where
      
        LCHECK = .false.
        where (MLANDNE .and. (PCNE /= c0)) LCHECK = .true.
        where (MLANDNW .and. (PCNW /= c0)) LCHECK = .true.
        where (MLANDSE .and. (PCSE /= c0)) LCHECK = .true.
        where (MLANDSW .and. (PCSW /= c0)) LCHECK = .true.
        where (MLANDNE .and. MLANDNW .and. (PCN /= c0)) LCHECK = .true.
        where (MLANDNE .and. MLANDSE .and. (PCE /= c0)) LCHECK = .true.
        where (MLANDNW .and. MLANDSW .and. (PCW /= c0)) LCHECK = .true.
        where (MLANDSE .and. MLANDSW .and. (PCS /= c0)) LCHECK = .true.
        where (MLANDNE .and. MLANDSE .and. 
     &         MLANDNW .and. MLANDSW .and. (PCC /= c0)) LCHECK = .true.

        if (global_count(LCHECK) /= 0) then
          call exit_POP('PC and KMU are incompatible')
        endif

        deallocate (IWORK1, IWORK2, 
     &              LCHECK, MLANDNE, MLANDNW, MLANDSE, MLANDSW)

      else ! no preconditioner

        if (my_task == master_task) then
          write(stdout,*) ' No preconditioner'
        endif

      endif

!-----------------------------------------------------------------------

      end subroutine init_solvers

!***********************************************************************

      subroutine pcg(X,B)

!-----------------------------------------------------------------------
!     conjugate-gradient solver with preconditioner
!-----------------------------------------------------------------------
!     equations:
!
!     Ax = b , A = operator, x = field, b = r.h.s.
!-----------------------------------------------------------------------
!     comments:
!
!   * the matrix A is a 9-point operator given by the
!     coefficient arrays A{0,N,S,E,W,NE,SE,NW,SW}.
!     the input X array is an initial guess.
!
!   * every ncheck steps the residual norm is calculated
!     in order to test for convergence.
!
!   * the preconditioner PC is a 9-point operator and is
!     applied once each iteration
!
!   * diagonal preconditioning is used if the 9-point preconditioner
!     is not available.
!-----------------------------------------------------------------------
!
!     input variables:
!
!       A{0,N,S,E,W,NE,SE,NW,SW} arrays of coefficients for 9-point
!         operator
!       PC{C,N,S,E,W,NE,SE,NW,SW} arrays of coefficients for 9-point
!         preconditioner
!       B = r.h.s.
!       X input as initial guess
!
!     output variables:
!
!       X returned as solution
!
!     local or common variables:
!
!       R  = residual (b - Ax)
!       beta = {alpha,theta} on {leapfrog,matsuno} timesteps
!       S  = conjugate direction vector
!       eta{0,1} are scalars in the cg routine
!       rms_residual = sqrt{(r,r)/(da,da)}  (da = area of T cell)
!       ncheck:  calculate residual norm every ncheck steps 
!
!-----------------------------------------------------------------------

      character (char_len) :: noconvrg

      integer (kind=int_kind) :: m   ! local iteration counter

      real (kind=dbl_kind) ::
     &   eta0,eta1,rr

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &   X,B,R,S,Q,WORK0,WORK1       ! various cg intermediate results

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &   A0R                          ! diagonal preconditioner

!-----------------------------------------------------------------------

      R     = c0  ! initialize arrays before passing as arguments
      Q     = c0
      WORK1 = c0

!-----------------------------------------------------------------------
!
!       diagonal preconditioner if preconditioner not specified
!
!-----------------------------------------------------------------------

      if (.not. lprecond) then
        where (A0 /= c0)
          A0R = c1/A0
        elsewhere
          A0R = c0
        endwhere
      endif

!-----------------------------------------------------------------------
!
!     compute initial residual
!
!-----------------------------------------------------------------------

      call ninept_4(R,A0,AN,AE,ANE,X)
      ! call ninept_9(R,A0,AN,AS,AE,AW,ANE,ASE,ANW,ASW,X)

      R = B - R

!-----------------------------------------------------------------------
!
!     initialize fields and scalars
!
!-----------------------------------------------------------------------

      S = c0
      eta0 =c1 
      solv_sum_iters = solv_max_iters
 
!-----------------------------------------------------------------------
!
!     iterate
!
!-----------------------------------------------------------------------

      do m = 1, solv_max_iters

!-----------------------------------------------------------------------
!
!         calculate (PC)r 
!
!-----------------------------------------------------------------------

        if (lprecond) then
          call ninept_9(WORK1,PCC,PCN,PCS,PCE,PCW,PCNE,PCSE,PCNW,PCSW,R)
        else
          WORK1 = R*A0R   ! use diagonal preconditioner
        endif

!-----------------------------------------------------------------------
!
!       update conjugate direction vector s
!
!-----------------------------------------------------------------------

        WORK0 = R*WORK1
        eta1 = global_sum(WORK0,RCALCT)  ! (r,(PC)r)

        S =  WORK1 + S*(eta1/eta0) 

!-----------------------------------------------------------------------
!
!       compute As
!
!-----------------------------------------------------------------------

        call ninept_4(Q,A0,AN,AE,ANE,S)
        ! call ninept_9(Q,A0,AN,AS,AE,AW,ANE,ASE,ANW,ASW,S)

!-----------------------------------------------------------------------
!
!       compute next solution and residual
!
!-----------------------------------------------------------------------

        eta0 = eta1
        WORK0 = Q*S
        eta1 = eta0/global_sum(WORK0,RCALCT)

        X = X + eta1*S
        R = R - eta1*Q

!-----------------------------------------------------------------------
!
!       test for convergence
!
!-----------------------------------------------------------------------

        if (mod(m,solv_ncheck).eq.0) then

          call ninept_4(R,A0,AN,AE,ANE,X)
          ! call ninept_9(R,A0,AN,AS,AE,AW,ANE,ASE,ANW,ASW,X)
          R = B - R

          WORK0 = R*R
          rr = global_sum(WORK0,RCALCT)   ! (r,r)
          
          if (rr .lt. solv_convrg) then
            solv_sum_iters = m
            go to 100
          endif

        endif

      enddo

100   continue

      rms_residual = sqrt(rr*resid_norm)

      if (solv_sum_iters == solv_max_iters) then
        if (solv_convrg /= c0) then
         write(noconvrg,'(a28,i11)') 
     &        ' no convergence at time step ', nsteps_total
         call exit_POP(noconvrg)
        endif
      endif

!-----------------------------------------------------------------------

      end subroutine pcg

!***********************************************************************

      subroutine cgr(X,B)

!-----------------------------------------------------------------------
!     conjugate-residual solver
!-----------------------------------------------------------------------
!     equations:
!
!     Ax = b , A = operator, x = field, b = r.h.s.
!-----------------------------------------------------------------------
!     comments:
!
!   * the matrix A is a 9-point operator given by the
!     coefficient arrays A{0,N,S,E,W,NE,SE,NW,SW}.
!     the input X array is an initial guess.
!
!   * every ncheck steps the residual norm is recalculated as
!     r = b - Ax to avoid roundoff error in the accumulated
!     residual (r,r) = old (r,r) - eta0*q.  if the recalculated
!     (r,r) is not less than its previously calculated value, 
!     then the solution is not converging due to machine roundoff
!     error, and the routine is exited.
!-----------------------------------------------------------------------
!
!     input variables:
!
!       A{0,N,S,E,W,NE,SE,NW,SW} arrays of coefficients for 9-point
!         operator
!       B = r.h.s.
!       X input as initial guess
!
!     output variables:
!
!       X returned as solution
!
!     local or common variables:
!
!       R  = residual (b - Ax)
!       S  = conjugate direction vector
!       Q  = As, operator acting on s
!       AR = operator acting on r
!       eta{0,1},eps{0,1} are scalars in the cgr routine
!       rr = sum(r*r) residual norm
!       rms_residual = sqrt{(r,r)/(da,da)}  (da = area of T cell)
!       ncheck:  recalculate residual norm from operator (r,r) = b - Ax
!         every ncheck steps 
!       rrold = b - Ax from last recalculation of (r,r), used to
!         check that (r,r) is not increasing due to roundoff error
!-----------------------------------------------------------------------

      character (char_len) :: noconvrg

      integer (kind=int_kind) :: m  !local iteration counter

      real (kind=dbl_kind) ::
     &   eps0,eps1,eta0,eta1,rr,rrold

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &   X,B,R,S,Q,AR,WORK

!-----------------------------------------------------------------------

      R  = c0  ! initialize arrays before passing as arguments
      AR = c0

!-----------------------------------------------------------------------
!
!     compute initial residual and its norm
!
!-----------------------------------------------------------------------

      call ninept_9(R,A0,AN,AS,AE,AW,ANE,ASE,ANW,ASW,X)
      R = B - R

      WORK = R*R
      rr = global_sum(WORK,RCALCT)   ! initial (r,r)
      rrold = rr
 
!-----------------------------------------------------------------------
!
!     initialize fields and scalars
!
!-----------------------------------------------------------------------

      S = c0
      Q = c0
      eps1 = c1 
      solv_sum_iters = solv_max_iters
 
!-----------------------------------------------------------------------
!
!     iterate
!
!-----------------------------------------------------------------------

      do m = 1, solv_max_iters

!-----------------------------------------------------------------------
!
!       calculate Ar
!
!-----------------------------------------------------------------------

        call ninept_9(AR,A0,AN,AS,AE,AW,ANE,ASE,ANW,ASW,R)

!-----------------------------------------------------------------------
!
!       update conjugate direction vector s, and q = As
!
!-----------------------------------------------------------------------

        WORK = R*AR
        eps0 = global_sum(WORK,RCALCT)  ! (r,Ar)
        eta1 = eps0/eps1

        S =  R +  eta1*S 
        Q = AR +  eta1*Q 

!-----------------------------------------------------------------------
!
!       compute next solution and residual, update (r,r).
!       every ncheck steps recalculate (r,r) = b - Ax and
!       exit if it is not decreasing (due to roundoff error).
!
!-----------------------------------------------------------------------

        WORK = Q*Q
        eps1 = global_sum(WORK,RCALCT)  ! (As,As)
        eta0 = eps0/eps1

        X = X + eta0*S

        if (mod(m,solv_ncheck).eq.0) then

          call ninept_9(R,A0,AN,AS,AE,AW,ANE,ASE,ANW,ASW,X)
          R = B - R

          WORK = R*R
          rr = global_sum(WORK,RCALCT)

          if (rr.gt.rrold) then
            solv_sum_iters = m
            go to 100  ! exit
          endif

          rrold = rr

        else

          R = R - eta0*Q

          rr = rr - eta0**2*eps1

        endif

        eps1 = eps0  ! update for next pass

!-----------------------------------------------------------------------
!
!       test for convergence
!
!-----------------------------------------------------------------------

        if (rr .lt. solv_convrg) then
          solv_sum_iters = m
          go to 100
        endif

      enddo

100   continue

      rms_residual = sqrt(abs(rr)*resid_norm)

      if (solv_sum_iters == solv_max_iters) then
        if (solv_convrg /= c0) then
         write(noconvrg,'(a28,i5)') 
     &        ' no convergence at time step ', nsteps_total
         call exit_POP(noconvrg)
        endif
      endif

!-----------------------------------------------------------------------

      end subroutine cgr

!***********************************************************************

      subroutine jacobi(X,B)

!-----------------------------------------------------------------------
!     simple richardson-jacobi solver with diagonal preconditioning
!-----------------------------------------------------------------------
!     equations:
!
!     Ax = b , A = operator, x = field, b = r.h.s.
!-----------------------------------------------------------------------
!     comments:
!
!   * the matrix A is a 9-point operator given by the
!     coefficient arrays A{0,N,S,E,W,NE,SE,NW,SW}.
!     the input X array is an initial guess.
!
!   * every ncheck steps the residual norm is calculated
!     in order to test for convergence.
!
!   * the preconditioner PC is a 9-point operator and is
!     applied once each iteration
!
!   * diagonal preconditioning is used if the 9-point preconditioner
!     is not available, or if the streamfunction formulation is used.
!-----------------------------------------------------------------------
!
!     input variables:
!
!       A{0,N,S,E,W,NE,SE,NW,SW} arrays of coefficients for 9-point
!         operator
!       PC{C,N,S,E,W,NE,SE,NW,SW} arrays of coefficients for 9-point
!         preconditioner
!       B = r.h.s.
!       X input as initial guess
!
!     output variables:
!
!       X returned as solution
!
!     local or common variables:
!
!       R = residual (b - Ax)
!       rms_residual = sqrt{(r,r)/(da,da)}  (da = area of T cell)
!       ncheck:  calculate residual norm every ncheck steps 
!-----------------------------------------------------------------------

      character (char_len) :: noconvrg

      integer (kind=int_kind) :: m  ! local iteration counter

      real (kind=dbl_kind) :: rr

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &   X,B,R,WORK1,WORK0

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &   A0R                        ! diagonal preconditioner

!-----------------------------------------------------------------------

      R    = c0  ! initialize before passing as arguments
      WORK1 = c0

!-----------------------------------------------------------------------
!
!     diagonal preconditioner
!
!-----------------------------------------------------------------------

      if (.not. lprecond) then
        where (A0.ne.c0)
          A0R = c1/A0
        elsewhere
          A0R = c0
        endwhere
      endif

      solv_sum_iters = solv_max_iters
 
!-----------------------------------------------------------------------
!
!     iterate
!
!-----------------------------------------------------------------------

      do m = 1, solv_max_iters

!-----------------------------------------------------------------------
!
!       calculate residual r = b - Ax
!
!-----------------------------------------------------------------------

        call ninept_9(WORK1,A0,AN,AS,AE,AW,ANE,ASE,ANW,ASW,X)

        R = B - WORK1

!-----------------------------------------------------------------------
!
!       calculate (PC)r 
!
!-----------------------------------------------------------------------

        if (lprecond) then
          call ninept_9(WORK1,PCC,PCN,PCS,PCE,PCW,PCNE,PCSE,PCNW,PCSW,X)
        else
          WORK1 = R*A0R   ! use diagonal preconditioner
        endif

!-----------------------------------------------------------------------
!
!       compute next solution
!
!-----------------------------------------------------------------------

        X = X + sor*WORK1

!-----------------------------------------------------------------------
!
!       test for convergence
!
!-----------------------------------------------------------------------

        if (mod(m,solv_ncheck).eq.0) then

          WORK0 = R*R
          rr = global_sum(WORK0,RCALCT)   ! (r,r)
          
          if (rr .lt. solv_convrg) then
            solv_sum_iters = m
            go to 100
          endif

        endif

      enddo

100   continue

      rms_residual = sqrt(rr*resid_norm)

      if (solv_sum_iters == solv_max_iters) then
        if (solv_convrg /= c0) then
         write(noconvrg,'(a28,i5)') 
     &        ' no convergence at time step ', nsteps_total
         call exit_POP(noconvrg)
        endif
      endif

!-----------------------------------------------------------------------

      end subroutine jacobi

!***********************************************************************

      end module solvers

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
