!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module state_mod

!***********************************************************************
!
!     contains routines necessary for computing the density from 
!     temperature and salinity using an equation of state
!
!     the module supports four forms of EOS:
!     1. the UNESCO equation of state computed using the PT-based
!        bulk modulus from Jackett and McDougall, JTECH, Vol.12,
!        pp 381-389, April, 1995.
!     2. a polynomial fit to the full UNESCO EOS
!     3. a simple linear EOS based on constant expansion coeffs
!     4. The faster and more accurate alternative to the UNESCO eos
!        of McDougall, Wright, Jackett and Feistel (hereafter MWJF, 2001 
!        submission to JTECH).
!     *  An earlier, preliminary version of MWJF, used in the earliest
!        CCSM 2.0 runs, is also included here for documentation purposes
!        only (hereafter MWJ). This version is not supported, and if
!        it is selected, the model will stop.
!  
!     CVS:$Id: state_mod.F,v 1.3.2.11 2002/05/10 22:42:47 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use grid
      use io
      use global_reductions
      use time_management
      use exit_mod
      use timers
      use shr_vmath_mod

      implicit none

      integer (kind=int_kind), save ::
     &   timer_state    
 
      save

!-----------------------------------------------------------------------
!
!     valid ranges and pressure as function of depth
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(km) :: 
     &  tmin, tmax,         ! valid temperature range for level k
     &  smin, smax,         ! valid salinity    range for level k
     &  pressz              ! ref pressure (bars) at each level

!-----------------------------------------------------------------------
!
!     choices for eos type
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), parameter ::
     &  state_type_jmcd       = 1,
     &  state_type_polynomial = 2,
     &  state_type_linear     = 3,
     &  state_type_mwjf       = 4,
     &  state_type_mwj        = 5

      integer (kind=int_kind) ::
     &  state_itype            ! input state type chosen

      integer (kind=int_kind) ::
     &  state_range_iopt,   ! option for checking valid T,S range
     &  state_range_freq    ! freq (in steps) for checking T,S range

      integer (kind=int_kind), parameter ::
     &  state_range_ignore  = 1,  ! do not check T,S range
     &  state_range_check   = 2,  ! check T,S range and report invalid
     &  state_range_enforce = 3   ! force polynomial eval within range

!-----------------------------------------------------------------------
!
!     UNESCO EOS constants and JMcD bulk modulus constants
!
!-----------------------------------------------------------------------

      !*** for density of fresh water (standard UNESCO)

      real (kind=dbl_kind), parameter ::
     &  unt0 =   999.842594_dbl_kind,
     &  unt1 =  6.793952e-2_dbl_kind,
     &  unt2 = -9.095290e-3_dbl_kind,
     &  unt3 =  1.001685e-4_dbl_kind,
     &  unt4 = -1.120083e-6_dbl_kind,
     &  unt5 =  6.536332e-9_dbl_kind

      !*** for dependence of surface density on salinity (UNESCO)

      real (kind=dbl_kind), parameter ::
     &  uns1t0 =  0.824493_dbl_kind ,
     &  uns1t1 = -4.0899e-3_dbl_kind,
     &  uns1t2 =  7.6438e-5_dbl_kind,
     &  uns1t3 = -8.2467e-7_dbl_kind,
     &  uns1t4 =  5.3875e-9_dbl_kind,
     &  unsqt0 = -5.72466e-3_dbl_kind,
     &  unsqt1 =  1.0227e-4_dbl_kind,
     &  unsqt2 = -1.6546e-6_dbl_kind,
     &  uns2t0 =  4.8314e-4_dbl_kind

      !*** from Table A1 of Jackett and McDougall

      real (kind=dbl_kind), parameter ::
     &  bup0s0t0 =  1.965933e+4_dbl_kind,
     &  bup0s0t1 =  1.444304e+2_dbl_kind,
     &  bup0s0t2 = -1.706103_dbl_kind   ,
     &  bup0s0t3 =  9.648704e-3_dbl_kind,
     &  bup0s0t4 = -4.190253e-5_dbl_kind

      real (kind=dbl_kind), parameter ::
     &  bup0s1t0 =  5.284855e+1_dbl_kind,
     &  bup0s1t1 = -3.101089e-1_dbl_kind,
     &  bup0s1t2 =  6.283263e-3_dbl_kind,
     &  bup0s1t3 = -5.084188e-5_dbl_kind

      real (kind=dbl_kind), parameter ::
     &  bup0sqt0 =  3.886640e-1_dbl_kind,
     &  bup0sqt1 =  9.085835e-3_dbl_kind,
     &  bup0sqt2 = -4.619924e-4_dbl_kind

      real (kind=dbl_kind), parameter ::
     &  bup1s0t0 =  3.186519_dbl_kind   ,
     &  bup1s0t1 =  2.212276e-2_dbl_kind,
     &  bup1s0t2 = -2.984642e-4_dbl_kind,
     &  bup1s0t3 =  1.956415e-6_dbl_kind 

      real (kind=dbl_kind), parameter ::
     &  bup1s1t0 =  6.704388e-3_dbl_kind,
     &  bup1s1t1 = -1.847318e-4_dbl_kind,
     &  bup1s1t2 =  2.059331e-7_dbl_kind,
     &  bup1sqt0 =  1.480266e-4_dbl_kind 

      real (kind=dbl_kind), parameter ::
     &  bup2s0t0 =  2.102898e-4_dbl_kind,
     &  bup2s0t1 = -1.202016e-5_dbl_kind,
     &  bup2s0t2 =  1.394680e-7_dbl_kind,
     &  bup2s1t0 = -2.040237e-6_dbl_kind,
     &  bup2s1t1 =  6.128773e-8_dbl_kind,
     &  bup2s1t2 =  6.207323e-10_dbl_kind

!-----------------------------------------------------------------------
!
!     MWJF EOS coefficients
!
!-----------------------------------------------------------------------

      !*** these constants will be used to construct the numerator
      !*** factor unit change (kg/m^3 -> g/cm^3) into numerator terms

      real (kind=dbl_kind), parameter ::
     &  mwjfnp0s0t0 =   9.99843699e+2_dbl_kind * p001,
     &  mwjfnp0s0t1 =   7.35212840e+0_dbl_kind * p001,
     &  mwjfnp0s0t2 =  -5.45928211e-2_dbl_kind * p001,
     &  mwjfnp0s0t3 =   3.98476704e-4_dbl_kind * p001,
     &  mwjfnp0s1t0 =   2.96938239e+0_dbl_kind * p001,
     &  mwjfnp0s1t1 =  -7.23268813e-3_dbl_kind * p001,
     &  mwjfnp0s2t0 =   2.12382341e-3_dbl_kind * p001,
     &  mwjfnp1s0t0 =   1.04004591e-2_dbl_kind * p001,
     &  mwjfnp1s0t2 =   1.03970529e-7_dbl_kind * p001,
     &  mwjfnp1s1t0 =   5.18761880e-6_dbl_kind * p001,
     &  mwjfnp2s0t0 =  -3.24041825e-8_dbl_kind * p001,
     &  mwjfnp2s0t2 =  -1.23869360e-11_dbl_kind * p001

      !*** these constants will be used to construct the denominator

      real (kind=dbl_kind), parameter ::
     &  mwjfdp0s0t0 =   1.0e+0_dbl_kind,
     &  mwjfdp0s0t1 =   7.28606739e-3_dbl_kind,
     &  mwjfdp0s0t2 =  -4.60835542e-5_dbl_kind,
     &  mwjfdp0s0t3 =   3.68390573e-7_dbl_kind,
     &  mwjfdp0s0t4 =   1.80809186e-10_dbl_kind,
     &  mwjfdp0s1t0 =   2.14691708e-3_dbl_kind,
     &  mwjfdp0s1t1 =  -9.27062484e-6_dbl_kind,
     &  mwjfdp0s1t3 =  -1.78343643e-10_dbl_kind,
     &  mwjfdp0sqt0 =   4.76534122e-6_dbl_kind,
     &  mwjfdp0sqt2 =   1.63410736e-9_dbl_kind,
     &  mwjfdp1s0t0 =   5.30848875e-6_dbl_kind,
     &  mwjfdp2s0t3 =  -3.03175128e-16_dbl_kind,
     &  mwjfdp3s0t1 =  -1.27934137e-17_dbl_kind



!-----------------------------------------------------------------------
!
!     MWJ EOS coefficients
!
!-----------------------------------------------------------------------

      !*** these constants will be used to construct the numerator and
      !*** depend on salinity, potential temperature and pressure
      !*** factor unit change (kg/m^3 -> g/cm^3) into numerator terms

      real (kind=dbl_kind), parameter ::
     &  mwjp0s0t0 =   1.96637339e8_dbl_kind * p001,
     &  mwjp0s0t1 =   1.44604544e6_dbl_kind * p001,
     &  mwjp0s0t2 =  -1.07691980e4_dbl_kind * p001,
     &  mwjp0s0t3 =   8.52955498e1_dbl_kind * p001,
     &  mwjp0s1t0 =   5.79854265e5_dbl_kind * p001,
     &  mwjp0s1t1 =  -1.47731528e3_dbl_kind * p001,
     &  mwjp0s2t0 =   4.19489109e2_dbl_kind * p001,
     &  mwjp1s0t0 =   2.00122349e3_dbl_kind * p001,
     &  mwjp1s0t2 =   2.13851025e-2_dbl_kind * p001,
     &  mwjp1s1t0 =   9.68774141e-1_dbl_kind * p001,
     &  mwjp2s0t0 =  -6.54785602e-3_dbl_kind * p001,
     &  mwjp2s0t2 =  -2.50468726e-6_dbl_kind * p001,
     &  mwjp2s0t3 =   6.28902345e-8_dbl_kind * p001,
     &  mwjp3s0t1 =   2.82414763e-9_dbl_kind * p001

      !*** these constants will be used to construct the denominator
      !*** and depend on salinity and potential temperature

      real (kind=dbl_kind), parameter ::
     &  mwjs0t0 =   1.96668928e5_dbl_kind,
     &  mwjs0t1 =   1.43302205e3_dbl_kind,
     &  mwjs0t2 =  -9.09231525e0_dbl_kind,
     &  mwjs0t3 =   7.91654429e-2_dbl_kind,
     &  mwjs0t4 =   3.98630534e-5_dbl_kind,
     &  mwjs1t0 =   4.17831720e2_dbl_kind,
     &  mwjs1t1 =  -1.87581316e0_dbl_kind,
     &  mwjs1t3 =  -3.87902837e-5_dbl_kind,
     &  mwjsqt0 =   1.00765828e0_dbl_kind,
     &  mwjsqt2 =   3.12912597e-4_dbl_kind

!-----------------------------------------------------------------------
!
!     coeffs and reference values for polynomial eos
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(:), allocatable :: 
     &  to,                 ! reference temperature for level k
     &  so,                 ! reference salinity    for level k
     &  sigo                ! reference density     for level k

      real (kind=dbl_kind), dimension(:,:), allocatable :: 
     &  state_coeffs        ! coefficients for polynomial eos

!-----------------------------------------------------------------------
!
!     parameters for linear eos
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), parameter :: 
     &  alf = 2.5e-4_dbl_kind, ! expansion coeff (drho/dT) for lin eos 
     &  bet = 7.6e-4_dbl_kind  ! expansion coeff (drho/dS) for lin eos

!***********************************************************************

      contains

!***********************************************************************

      subroutine state(k,kk,TEMPK,SALTK,RHOOUT,RHOFULL,DRHODT,DRHODS)

!-----------------------------------------------------------------------
!
!     calculate density of water at level k from equation of state
!     rho = rho(depth,pot.temperature,salinity). the density can be
!     returned as a perturbation (RHOOUT) or as the full density
!     (RHOFULL). 
!
!     This routine also computes derivatives of density with respect
!     to temperature and salinity at level k from equation of state
!     if requested (ie the optional arguments are present)
!
!   * normally kk = k, but kk may = k+1 for adiabatic displacement
!     to next lower level
!
!     this routine supports three forms of the EOS (see module comments
!     at the top)
!
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  k,                     ! depth level index
     &  kk                     ! level to which water is adiabatically 
                               ! displaced (k or k+1)

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &  TEMPK,              ! temperature at level k
     &  SALTK               ! salinity    at level k

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), optional, intent(out) :: 
     &  RHOOUT,  ! density of water at level k after adiabatic
     &           !    displacement to level kk (as a perturbation).
     &  RHOFULL, ! full density of water at level k after adiabatic disp
     &  DRHODT,  ! derivative of density with respect to temperature
     &  DRHODS   ! derivative of density with respect to salinity

!-----------------------------------------------------------------------
!
!     local variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  out_of_range        ! counter for out-of-range T,S values

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  TQ,SQ,              ! adjusted T,S
     &  BULK_MOD,           ! Bulk modulus
     &  RHO0,               ! density at the surface
     &  DRDT0,              ! d(density)/d(temperature), for surface
     &  DRDS0,              ! d(density)/d(salinity   ), for surface
     &  DKDT,               ! d(bulk modulus)/d(pot. temp.)
     &  DKDS,               ! d(bulk modulus)/d(salinity  )
     &  SQR,DENOMK,         ! work arrays
     &  WORK1, WORK2, WORK3, WORK4, T2

      !*** MWJF numerator coefficients including pressure

      real (kind=dbl_kind) ::
     &  mwjfnums0t0, mwjfnums0t1, mwjfnums0t2, mwjfnums0t3,
     &  mwjfnums1t0, mwjfnums1t1, mwjfnums2t0,
     &  mwjfdens0t0, mwjfdens0t1, mwjfdens0t2, mwjfdens0t3, mwjfdens0t4,
     &  mwjfdens1t0, mwjfdens1t1, mwjfdens1t3,
     &  mwjfdensqt0, mwjfdensqt2


      !*** MWJ numerator coefficients including pressure

      real (kind=dbl_kind) ::
     &  mwjnums0t0, mwjnums0t1, mwjnums0t2, mwjnums0t3,
     &  mwjnums1t0, mwjnums1t1, mwjnums2t0

      real (kind=dbl_kind) :: p, p2 ! temporary pressure scalars

      call timer_start(timer_state)
!-----------------------------------------------------------------------
!
!     first check for valid range if requested
!
!-----------------------------------------------------------------------

      select case (state_range_iopt)
      case (state_range_ignore)

          TQ = TEMPK
          SQ = SALTK

      case (state_range_check)

        if (time_to_do(freq_opt_nstep, state_range_freq)) then
          out_of_range = global_count((TEMPK < tmin(kk) .or.
     &                                 TEMPK > tmax(kk)) .and. k<=KMT)
          if (out_of_range /= 0 .and. my_task == master_task) then
            write(stdout,*) 'WARNING: ',out_of_range,
     &         'points outside of valid temp range at level ',kk
          endif
          out_of_range = global_count((SALTK < smin(kk) .or.
     &                                 SALTK > smax(kk)) .and. k<=KMT)
          if (out_of_range /= 0 .and. my_task == master_task) then
            write(stdout,*) 'WARNING: ',out_of_range,
     &         'points outside of valid salinity range at level ',kk
            endif
        endif

        TQ = TEMPK
        SQ = SALTK

      case (state_range_enforce)

        TQ = min(TEMPK,tmax(kk))
        TQ = max(TQ,tmin(kk))

        SQ = min(SALTK,smax(kk))
        SQ = max(SQ,smin(kk))

      end select

!-----------------------------------------------------------------------
!
!     now compute density or expansion coefficients
!
!-----------------------------------------------------------------------

      select case (state_itype)

!-----------------------------------------------------------------------
!
!     Jackett and McDougall EOS
!
!-----------------------------------------------------------------------

      case (state_type_jmcd)

        p   = pressz(kk)
        p2  = p*p

        SQ  = c1000*SQ
        SQR = sqrt(SQ)
        T2  = TQ*TQ


        !***
        !*** first calculate surface (p=0) values from UNESCO eqns.
        !***

        WORK1 = uns1t0 + uns1t1*TQ + 
     &         (uns1t2 + uns1t3*TQ + uns1t4*T2)*T2
        WORK2 = SQR*(unsqt0 + unsqt1*TQ + unsqt2*T2)

        RHO0 = unt1*TQ + (unt2 + unt3*TQ + (unt4 + unt5*TQ)*T2)*T2
     &                 + (uns2t0*SQ + WORK1 + WORK2)*SQ


        !***
        !*** now calculate bulk modulus at pressure p from 
        !*** Jackett and McDougall formula
        !***

        WORK3 = bup0s1t0 + bup0s1t1*TQ +
     &         (bup0s1t2 + bup0s1t3*TQ)*T2 +
     &          p *(bup1s1t0 + bup1s1t1*TQ + bup1s1t2*T2) +
     &          p2*(bup2s1t0 + bup2s1t1*TQ + bup2s1t2*T2)
        WORK4 = SQR*(bup0sqt0 + bup0sqt1*TQ + bup0sqt2*T2 +
     &               bup1sqt0*p)

        BULK_MOD  = bup0s0t0 + bup0s0t1*TQ +
     &              (bup0s0t2 + bup0s0t3*TQ + bup0s0t4*T2)*T2 + 
     &              p *(bup1s0t0 + bup1s0t1*TQ + 
     &                 (bup1s0t2 + bup1s0t3*TQ)*T2) +
     &              p2*(bup2s0t0 + bup2s0t1*TQ + bup2s0t2*T2) +
     &              SQ*(WORK3 + WORK4)

        DENOMK = c1/(BULK_MOD - p)

        !***
        !*** now calculate required fields
        !***

        if (present(RHOOUT)) then
          RHOOUT = merge(((unt0 + RHO0)*BULK_MOD*DENOMK)*p001,
     &                    c0, KMT >= k)
        endif

        if (present(RHOFULL)) then
          RHOFULL = merge(((unt0 + RHO0)*BULK_MOD*DENOMK)*p001,
     &                    c0, KMT >= k)
        endif

        if (present(DRHODT)) then
          DRDT0 =  unt1 + c2*unt2*TQ + 
     &             (c3*unt3 + c4*unt4*TQ + c5*unt5*T2)*T2 +
     &             (uns1t1 + c2*uns1t2*TQ + 
     &              (c3*uns1t3 + c4*uns1t4*TQ)*T2 +
     &              (unsqt1 + c2*unsqt2*TQ)*SQR )*SQ

          DKDT  = bup0s0t1 + c2*bup0s0t2*TQ + 
     &            (c3*bup0s0t3 + c4*bup0s0t4*TQ)*T2 + 
     &            p *(bup1s0t1 + c2*bup1s0t2*TQ + c3*bup1s0t3*T2) +
     &            p2*(bup2s0t1 + c2*bup2s0t2*TQ) +
     &            SQ*(bup0s1t1 + c2*bup0s1t2*TQ + c3*bup0s1t3*T2 +
     &                p  *(bup1s1t1 + c2*bup1s1t2*TQ) +
     &                p2 *(bup2s1t1 + c2*bup2s1t2*TQ) +
     &                SQR*(bup0sqt1 + c2*bup0sqt2*TQ))

          DRHODT = merge ((DENOMK*(DRDT0*BULK_MOD -
     &         pressz(kk)*(unt0+RHO0)*DKDT*DENOMK))*p001,
     &         c0, KMT >= k)
        endif

        if (present(DRHODS)) then
          DRDS0  = c2*uns2t0*SQ + WORK1 + c1p5*WORK2
          DKDS = WORK3 + c1p5*WORK4

          DRHODS = merge (DENOMK*(DRDS0*BULK_MOD -
     &         pressz(kk)*(unt0+RHO0)*DKDS*DENOMK),
     &         c0, KMT >= k)

        endif
 
!-----------------------------------------------------------------------
!
!     polynomial EOS
!
!-----------------------------------------------------------------------

      case (state_type_polynomial)

        TQ = TQ - to(kk)
        SQ = SQ - so(kk) - 0.035_dbl_kind

        if (present(RHOOUT)) then

          !*** density as perturbation from a k-level reference

          RHOOUT = merge((state_coeffs(1,kk) + 
     &                   (state_coeffs(4,kk) + 
     &                    state_coeffs(7,kk)*SQ)*SQ +
     &                   (state_coeffs(3,kk) + 
     &                    state_coeffs(8,kk)*SQ + 
     &                    state_coeffs(6,kk)*TQ)*TQ)*TQ +
     &                   (state_coeffs(2,kk) + 
     &                   (state_coeffs(5,kk) + 
     &                    state_coeffs(9,kk)*SQ)*SQ)*SQ
     &                ,c0, KMT >= k)
        endif

        if (present(RHOFULL)) then

          !*** full density (adding the reference density)

          RHOFULL=merge(((state_coeffs(1,kk) + 
     &                   (state_coeffs(4,kk) + 
     &                    state_coeffs(7,kk)*SQ)*SQ +
     &                   (state_coeffs(3,kk) + 
     &                    state_coeffs(8,kk)*SQ + 
     &                    state_coeffs(6,kk)*TQ)*TQ)*TQ +
     &                   (state_coeffs(2,kk) + 
     &                   (state_coeffs(5,kk) + 
     &                    state_coeffs(9,kk)*SQ)*SQ)*SQ) +
     &                    sigo(kk)*p001 + c1
     &                ,c1, KMT >= k)
        endif

        if (present(DRHODT)) then
          DRHODT = merge (state_coeffs(1,kk) + 
     &                   (state_coeffs(4,kk) + 
     &                    state_coeffs(7,kk)*SQ)*SQ +
     &                (c2*state_coeffs(3,kk) + 
     &                 c2*state_coeffs(8,kk)*SQ + 
     &                 c3*state_coeffs(6,kk)*TQ)*TQ,
     &             c0, KMT >= k)
        endif

        if (present(DRHODS)) then
          DRHODS = merge((state_coeffs(4,kk) + 
     &                 c2*state_coeffs(7,kk)*SQ + 
     &                    state_coeffs(8,kk)*TQ)*TQ +
     &                    state_coeffs(2,kk) + 
     &                (c2*state_coeffs(5,kk) + 
     &                 c3*state_coeffs(9,kk)*SQ)*SQ,
     &             c0, KMT >= k)
        endif
 
!-----------------------------------------------------------------------
!
!     linear EOS
!
!-----------------------------------------------------------------------

      case (state_type_linear)

        if (present(RHOOUT)) then
          RHOOUT = bet*SALTK - alf*TEMPK
        endif

        if (present(RHOFULL)) then
          RHOFULL = bet*SALTK - alf*TEMPK
        endif

        if (present(DRHODT)) then
          DRHODT = -alf
        endif

        if (present(DRHODS)) then
          DRHODS =  bet
        endif

!-----------------------------------------------------------------------
!
!     McDougall, Wright, Jackett, and Feistel EOS
!     test value : rho = 1.033213387 for
!     S = 35.0 PSU, theta = 20.0, pressz = 200.0
!
!-----------------------------------------------------------------------

      case (state_type_mwjf)

        p   = c10*pressz(kk)

        SQ  = c1000*SQ
        call shr_vmath_sqrt(SQ, SQR, imt*jmt)

        !***
        !*** first calculate numerator of MWJF density [P_1(S,T,p)]
        !***

        mwjfnums0t0 = mwjfnp0s0t0 + p*(mwjfnp1s0t0 + p*mwjfnp2s0t0)
        mwjfnums0t1 = mwjfnp0s0t1 
        mwjfnums0t2 = mwjfnp0s0t2 + p*(mwjfnp1s0t2 + p*mwjfnp2s0t2)
        mwjfnums0t3 = mwjfnp0s0t3
        mwjfnums1t0 = mwjfnp0s1t0 + p*mwjfnp1s1t0
        mwjfnums1t1 = mwjfnp0s1t1
        mwjfnums2t0 = mwjfnp0s2t0

        WORK1 = mwjfnums0t0 + TQ * (mwjfnums0t1 + TQ * (mwjfnums0t2 +
     $       mwjfnums0t3 * TQ)) + SQ * (mwjfnums1t0 +
     $       mwjfnums1t1 * TQ + mwjfnums2t0 * SQ)

        !***
        !*** now calculate denominator of MWJF density [P_2(S,T,p)]
        !***

        mwjfdens0t0 = mwjfdp0s0t0 + p*mwjfdp1s0t0
        mwjfdens0t1 = mwjfdp0s0t1 + p**3 * mwjfdp3s0t1
        mwjfdens0t2 = mwjfdp0s0t2
        mwjfdens0t3 = mwjfdp0s0t3 + p**2 * mwjfdp2s0t3
        mwjfdens0t4 = mwjfdp0s0t4
        mwjfdens1t0 = mwjfdp0s1t0
        mwjfdens1t1 = mwjfdp0s1t1
        mwjfdens1t3 = mwjfdp0s1t3
        mwjfdensqt0 = mwjfdp0sqt0
        mwjfdensqt2 = mwjfdp0sqt2

        WORK2 = mwjfdens0t0 + TQ * (mwjfdens0t1 + TQ * (mwjfdens0t2 +
     $       TQ * (mwjfdens0t3 + mwjfdens0t4 * TQ))) +
     $       SQ * (mwjfdens1t0 + TQ * (mwjfdens1t1 + TQ*TQ*mwjfdens1t3)+
     $       SQR * (mwjfdensqt0 + TQ*TQ*mwjfdensqt2))

        DENOMK = c1/WORK2

        if (present(RHOOUT)) then
          RHOOUT  = WORK1*DENOMK
        endif

        if (present(RHOFULL)) then
          RHOFULL = WORK1*DENOMK
        endif

        if (present(DRHODT)) then
           WORK3 =              ! dP_1/dT
     $          mwjfnums0t1 + TQ * (c2*mwjfnums0t2 +
     $          c3*mwjfnums0t3 * TQ) + mwjfnums1t1 * SQ

           WORK4 =              ! dP_2/dT
     $          mwjfdens0t1 + SQ * mwjfdens1t1 +
     $          TQ * (c2*(mwjfdens0t2 + SQ*SQR*mwjfdensqt2) +
     $          TQ * (c3*(mwjfdens0t3 + SQ * mwjfdens1t3) + 
     $          TQ *  c4*mwjfdens0t4))

           DRHODT = (WORK3 - WORK1*DENOMK*WORK4)*DENOMK
        endif

        if (present(DRHODS)) then
           WORK3 =              ! dP_1/dS
     $          mwjfnums1t0 + mwjfnums1t1 * TQ + c2*mwjfnums2t0 * SQ

           WORK4 = mwjfdens1t0 +    ! dP_2/dS
     $          TQ * (mwjfdens1t1 + TQ*TQ*mwjfdens1t3) +
     $          c1p5*SQR*(mwjfdensqt0 + TQ*TQ*mwjfdensqt2)

           DRHODS = (WORK3 - WORK1*DENOMK*WORK4)*DENOMK * c1000
        endif
 


!-----------------------------------------------------------------------
!
!     McDougall, Wright and Jackett EOS
!     test value : rho = 1.033213242 for
!     S = 35.0 PSU, theta = 20.0, pressz = 200.0
!
!-----------------------------------------------------------------------

      case (state_type_mwj)

        p   = c10*pressz(kk)

        SQ  = c1000*SQ
        call shr_vmath_sqrt(SQ, SQR, imt*jmt)

        !***
        !*** first calculate numerator of MWJ density (depends on pressure)
        !***

        mwjnums0t0 = mwjp0s0t0 + p*(mwjp1s0t0 + p*mwjp2s0t0)
        mwjnums0t1 = mwjp0s0t1 + p*p*p*mwjp3s0t1
        mwjnums0t2 = mwjp0s0t2 + p*(mwjp1s0t2 + p*mwjp2s0t2)
        mwjnums0t3 = mwjp0s0t3 + p*p*mwjp2s0t3
        mwjnums1t0 = mwjp0s1t0 + p*mwjp1s1t0
        mwjnums1t1 = mwjp0s1t1
        mwjnums2t0 = mwjp0s2t0

        WORK1 = mwjnums0t0 + TQ * (mwjnums0t1 + TQ * (mwjnums0t2 +
     $       mwjnums0t3 * TQ)) + SQ * (mwjnums1t0 +
     $       mwjnums1t1 * TQ + mwjnums2t0 * SQ)

        !***
        !*** now calculate PT- and S-dependent portion of denominator
        !*** (independent of pressure)
        !***

        WORK2 = mwjs0t0 + TQ * (mwjs0t1 + TQ * (mwjs0t2 +
     $       TQ * (mwjs0t3 + mwjs0t4 * TQ))) +
     $       SQ * (mwjs1t0 + TQ * (mwjs1t1 + TQ*TQ*mwjs1t3) +
     $       SQR * (mwjsqt0 + TQ*TQ*mwjsqt2))

        DENOMK = c1/(WORK2+p)

        if (present(RHOOUT)) then
          RHOOUT  = WORK1*DENOMK
        endif

        if (present(RHOFULL)) then
          RHOFULL = WORK1*DENOMK
        endif

        if (present(DRHODT)) then
           WORK3 =              ! pressure-dependent term
     $          mwjnums0t1 + TQ * (c2*mwjnums0t2 +
     $          c3*mwjnums0t3 * TQ) + mwjnums1t1 * SQ

           WORK4 =              ! independent of pressure
     $          mwjs0t1 + SQ * mwjs1t1 +
     $          TQ * (c2*(mwjs0t2 + SQ*SQR*mwjsqt2) +
     $          TQ * (c3*(mwjs0t3 + SQ * mwjs1t3) + TQ * c4*mwjs0t4))

           DRHODT = (WORK3 - WORK1*DENOMK*WORK4)*DENOMK
        endif

        if (present(DRHODS)) then
           WORK3 =              ! pressure-dependent term
     $          mwjnums1t0 + mwjnums1t1 * TQ + c2*mwjnums2t0 * SQ

           WORK4 = mwjs1t0 +    ! independent of pressure
     $          TQ * (mwjs1t1 + TQ*TQ*mwjs1t3) +
     $          c1p5*SQR*(mwjsqt0 + TQ*TQ*mwjsqt2)

           DRHODS = (WORK3 - WORK1*DENOMK*WORK4)*DENOMK * c1000
        endif
 
!-----------------------------------------------------------------------

      end select

      call timer_stop (timer_state)
!-----------------------------------------------------------------------

      end subroutine state

!***********************************************************************

      subroutine init_state

!-----------------------------------------------------------------------
!
!     initializes eos choice and initializes coefficients for
!     equation of state
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     namelist for input choice and coefficient file
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: k,
     &  nu                  ! unit number for coeff input file
     & ,m, nbits,
     &  nml_error           ! namelist i/o error flag

      character (char_len) ::
     &  state_choice,       ! choice of which EOS to use
     &  state_file,         ! file containing polynomial eos coeffs
     &  state_range_opt     ! option for checking valid T,S range

      namelist /state_nml/ state_choice, state_file, 
     &                     state_range_opt, state_range_freq

!-----------------------------------------------------------------------
!
!     read input choice for EOS type
!
!-----------------------------------------------------------------------

      state_itype = state_type_polynomial
      state_file  = 'internal'
      state_range_iopt = state_range_ignore
      state_range_freq = 100000

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=state_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('reading state_nml',normal_exit=.false.)
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, state_nml)
          write(stdout,*) ' '
      endif

      if (my_task == master_task) then
        select case (state_choice(1:len_trim(state_choice)))
        case ('jmcd')
          state_itype = state_type_jmcd
        case ('polynomial')
          state_itype = state_type_polynomial
        case ('linear')
          state_itype = state_type_linear
        case ('mwjf')
          state_itype = state_type_mwjf
        case ('mwj')
          state_itype = state_type_mwj
          call exit_POP('obsolete state_choice -- '
     &          // 'are you certain you want to use this option?'
     &,             normal_exit = .false.)
        case default
          state_itype = -1000
        end select

        select case (state_range_opt)
        case ('ignore')
          state_range_iopt = state_range_ignore
        case ('check')
          state_range_iopt = state_range_check
        case ('enforce')
          state_range_iopt = state_range_enforce
        case default
          state_range_iopt = -1000
        end select

      endif

      call broadcast_scalar(state_itype,      master_task)
      call broadcast_scalar(state_file,       master_task)
      call broadcast_scalar(state_range_iopt, master_task)
      call broadcast_scalar(state_range_freq, master_task)

      if (state_itype == -1000) then
        call exit_POP('unknown choice for equation of state type' 
     &,                normal_exit=.false.)
      endif
      if (state_range_iopt == -1000) then
        call exit_POP('unknown choice for state range option' 
     &,                normal_exit=.false.)
      endif

!-----------------------------------------------------------------------
!
!     write out header for eos options to stdout, then state_choice.
!
!-----------------------------------------------------------------------

      if (my_task == master_task) then
        write(stdout,1001)
        write(stdout,1000)
        write(stdout,1001)
        write(stdout,1002)
        write(stdout,1001)
        write(stdout,1000)
        write(stdout,*)'EOS choice: ',trim(state_choice)
        if (state_file == 'internal') then
          write(stdout,*)'calculating EOS coefficients'
        else
          write(stdout,*)'reading file containing EOS coefficients: '
     &          , state_file
        endif
        select case (state_range_iopt)
        case (state_range_ignore)
          write(stdout,*) 'No checking of valid T,S range'
        case (state_range_check)
          write(stdout,*) 'Valid T,S range checked every ',
     &                    state_range_freq,' steps.'
        case (state_range_enforce)
          write(stdout,*) 'EOS computed as if in valid T,S range'
        end select
      endif

 1000 format (72('-'))
 1001 format (' ')
 1002 format ('equation of state options')

!-----------------------------------------------------------------------
!
!     calculate pressure as function of depth
!
!-----------------------------------------------------------------------

      do k=1,km
        pressz(k) = pressure(zt(k)*mpercm)
      end do

!-----------------------------------------------------------------------
!
!     initialize various arrays, constants for each state type
!
!-----------------------------------------------------------------------

      select case (state_itype)

!-----------------------------------------------------------------------
!
!     McDougall, Wright and Jackett or
!     McDougall, Wright, Jackett, and Feistel
!
!-----------------------------------------------------------------------

      case (state_type_mwj, state_type_mwjf)

        tmin = -2.0_dbl_kind    ! limited on the low end
        tmax = 999.0_dbl_kind   ! unlimited on the high end
        smin =  0.0_dbl_kind    ! limited on the low end
        smax = 0.999_dbl_kind   ! unlimited on the high end

!-----------------------------------------------------------------------
!
!     Jackett and McDougall
!
!-----------------------------------------------------------------------

      case (state_type_jmcd)

        tmin = -2.0_dbl_kind  ! valid pot. temp. range for level k
        tmax = 40.0_dbl_kind 
        smin =  0.0_dbl_kind  ! valid salinity   range for level k
        smax = 0.042_dbl_kind 

!-----------------------------------------------------------------------
!
!     polynomial - initialize polynomial coefficients and ranges
!
!-----------------------------------------------------------------------

      case (state_type_polynomial)

        allocate (to(km), so(km), sigo(km))
        allocate (state_coeffs(9,km))

        if (state_file == 'internal') then
          call init_state_coeffs
        else
          call get_unit(nu)
          if (my_task == master_task) then
            nbits = 64
            open(nu,file=state_file,access='direct',form='unformatted',
     &        recl=km*nbits/8,status='unknown')
            read(nu,rec=1) to
            read(nu,rec=2) so
            read(nu,rec=3) sigo
            do m = 1,9
              read(nu,rec=m+3) state_coeffs(m,:)
            enddo
            if (state_range_iopt /= state_range_ignore) then
              read(nu,rec=13) tmin
              read(nu,rec=14) tmax
              read(nu,rec=15) smin
              read(nu,rec=16) smax
            endif
            close (nu)
          endif
          call release_unit(nu)

          call broadcast_vector(to,   master_task, km)
          call broadcast_vector(so,   master_task, km)
          call broadcast_vector(sigo, master_task, km)
          do m = 1,9
            call broadcast_vector(state_coeffs(m,:), master_task, km)
          enddo
          if (state_range_iopt /= state_range_ignore) then
            call broadcast_vector(tmin, master_task, km)
            call broadcast_vector(tmax, master_task, km)
            call broadcast_vector(smin, master_task, km)
            call broadcast_vector(smax, master_task, km)
          endif
        endif

        !***
        !*** convert smin,smax to model units
        !***

        if (state_range_iopt /= state_range_ignore) then
          smin = smin*ppt_to_salt
          smax = smax*ppt_to_salt
        endif

!-----------------------------------------------------------------------
!
!     linear EOS - intialize valid range
!
!-----------------------------------------------------------------------

      case (state_type_linear)

        tmin = -2.0_dbl_kind  ! valid pot. temp. range for level k
        tmax = 40.0_dbl_kind 
        smin =  0.0_dbl_kind  ! valid salinity   range for level k
        smax = 0.042_dbl_kind 

      end select

      
      call get_timer(timer_state, 'EQUATION_OF_STATE')
 
!-----------------------------------------------------------------------

      end subroutine init_state

!***********************************************************************

       subroutine init_state_coeffs

!-----------------------------------------------------------------------
!
!     calculate coefficients for POP polynomial equation of state option
!
!     THIS WAS TAKEN FROM THE GFDL MOM MODEL and extensively modified
!       to improve readability (February (1999)
!
!       This program calculates the 9 coefficients of a third order
!     polynomial approximation to the equation of state for sea water.
!     The coefficients are calculated by first sampling a range of
!     temperature and salinity at each depth.  The density is then
!     computed at each of the sampled points using a full equation
!     of state.  A least squares method is used to fit a polynomial
!     (cubic in temp and salinity) to the sampled points.  More 
!     specifically, the densities calculated from the polynomial 
!     formula are in the form of sigma anomalies.  The method is
!     taken from that described by Bryan & Cox (1972).  
!       By default, the program uses the equation of state set by the
!     Joint Panel on Oceanographic Tables & Standards (UNESCO, 1981)
!     an described by Gill (1982).
!       Subroutine lsqsl2 performs the iterative least-squares
!     polynomial fitting for the overdetermined system.  The algorithm
!     is outlined by Hanson and Lawson (1969), and the code is a
!     slightly modified (to get rid of old Fortran IV stuff) version
!     of the original routine.
!
!     ACCURACY IMPROVEMENTS (March 1999)
!
!       There are two improvements: (a)  An more accurate depth-
!     to-pressure conversion based on the Levitus_94 climatology,
!     and (b) Use of a more recent formula for the computation of 
!     potential temperature (Bryden, 1973) in place of an older
!     algorithm (Fofonoff, 1962). See Dukowicz (2000).
!
!     references:
!        Bryan, K. & M. Cox, An approximate equation of state
!          for numerical models of ocean circulation, J. Phys.
!          Oceanogr., 2, 510-514, 1972.
!        Bryden, H.L., New polynomials for thermal expansion, adiabatic
!          temperature gradient and potential temperature of sea water,
!          Deap-Sea Res., 20, 401-408, 1973.
!        Dukowicz, J. K., 2000: Reduction of Pressure and Pressure
!          Gradient Errors in Ocean Simulations, J. Phys. Oceanogr.,
!          submitted.
!        Fofonoff, N., The Sea: Vol 1, (ed. M. Hill). Interscience,
!          New York, 1962, pp 3-30.
!        Gill, A., Atmosphere-Ocean Dynamics: International Geophysical
!          Series No. 30. Academic Press, London, 1982, pp 599-600.
!        Hanson, R., & C. Lawson, Extensions and applications of the
!          Householder algorithm for solving linear least squares
!          problems. Math. Comput., 23, 787-812, 1969.
!        UNESCO, 10th report of the joint panel on oceanographic tables
!          and standards. UNESCO Tech. Papers in Marine Sci. No. 36,
!          Paris, 1981.
!
!    inputs:
!      The user needs only to specify the number of model levels km
!    and the model layer thicknesses [cm] dz(1..km).
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), parameter ::
     &  nsample_salt = 5,              ! number of pts to sample in S 
     &  nsample_temp = 2*nsample_salt, ! number of pts in sample in T
     &  nsample_all  = nsample_salt*nsample_temp, ! total sample points
     &  itmax = 4,                     ! max iters for least squares
     &  sbdim = nsample_all+72,        ! array dims for work arrays 
     &  rdim = nsample_all+36          !  in lsq routine

      integer (kind=int_kind) ::
     &  i,j,k,n,        ! dummy loop counters
     &  nsample,        ! sample number
     &  irank, in, it,  ! used by lsqsl2 routine
     &  ieq,            !
     &  nlines,         ! number of data lines in km output
     &  nwords_last_line ! num data words in last line of km output

      real (kind=dbl_kind) :: 
     &  dtemp, dsalt,   ! T,S increments for curve fit
     &  temp_sample,    ! sample temperature in temp range
     &  salt_sample,    ! sample salinity    in salt range
     &  density,        ! density in MKS units
     &  tanom, sanom,   ! T,S anomalies (from level average)
     &  enorm           ! norm of least squares residual

      real (kind=dbl_kind), dimension(km) ::
     &  avg_theta       ! average of sample pot. temps for level k

      real (kind=dbl_kind), dimension(nsample_all) ::
     &  tsamp,          ! temperature     at each sample point 
     &  ssamp,          ! salinity        at each sample point
     &  thsamp,         ! potential temp  at each sample point
     &  sigma,          ! density         at each sample point
     &  sigman          ! density anomaly at each sample point

      real (kind=dbl_kind), dimension(nsample_all,9) ::
     &  lsqarray,       ! least squares array to find coeffs
     &  lsqc            ! lsq work array

      real (kind=dbl_kind), dimension(9) ::
     &  lsqcoeffs       ! polynomial coeffs returned by lsq routine

      real (kind=dbl_kind), dimension(9,9) ::
     &  homog           ! solution to homogeneous eqns (used by lsq)

      real (kind=dbl_kind), dimension(sbdim) ::
     &  lsqsb           ! lsq work array

      real (kind=dbl_kind), dimension(rdim) ::
     &  lsqr            ! lsq work array

      !***
      !*** bounds for polynomial fit using a reference model of 
      !*** 33 levels from surface to 8000m at z=(k-1)*250 meters
      !*** The user should review the appropriateness of the 
      !*** reference values set below, and modify them if the 
      !*** intended modelling application could be expected to yield 
      !*** temperature and salinity values outside of the ranges set 
      !*** by default.
      !***

      real (kind=dbl_kind), dimension(33) ::
     &  trefmin = ( / -2.0_dbl_kind, -2.0_dbl_kind, -2.0_dbl_kind,
     &                -2.0_dbl_kind, -1.0_dbl_kind, -1.0_dbl_kind,
     &                -1.0_dbl_kind, -1.0_dbl_kind, -1.0_dbl_kind,
     &                -1.0_dbl_kind, -1.0_dbl_kind, -1.0_dbl_kind,
     &                -1.0_dbl_kind, -1.0_dbl_kind, -1.0_dbl_kind,
     &                -1.0_dbl_kind, -1.0_dbl_kind, -1.0_dbl_kind,
     &                -1.0_dbl_kind,  0.0_dbl_kind,  0.0_dbl_kind,
     &                 0.0_dbl_kind,  0.0_dbl_kind,  0.0_dbl_kind,
     &                 0.0_dbl_kind,  0.0_dbl_kind,  0.0_dbl_kind,
     &                 0.0_dbl_kind,  0.0_dbl_kind,  0.0_dbl_kind,
     &                 0.0_dbl_kind,  0.0_dbl_kind,  0.0_dbl_kind / ),
     &  trefmax = ( / 29.0_dbl_kind, 19.0_dbl_kind, 14.0_dbl_kind,
     &                11.0_dbl_kind,  9.0_dbl_kind,  7.0_dbl_kind,
     &                 7.0_dbl_kind,  7.0_dbl_kind,  7.0_dbl_kind,
     &                 7.0_dbl_kind,  7.0_dbl_kind,  7.0_dbl_kind,
     &                 7.0_dbl_kind,  7.0_dbl_kind,  7.0_dbl_kind,
     &                 7.0_dbl_kind,  7.0_dbl_kind,  7.0_dbl_kind,
     &                 7.0_dbl_kind,  7.0_dbl_kind,  7.0_dbl_kind,
     &                 7.0_dbl_kind,  7.0_dbl_kind,  7.0_dbl_kind,
     &                 7.0_dbl_kind,  7.0_dbl_kind,  7.0_dbl_kind,
     &                 7.0_dbl_kind,  7.0_dbl_kind,  7.0_dbl_kind,
     &                 7.0_dbl_kind,  7.0_dbl_kind,  7.0_dbl_kind / ),
     &  srefmin = ( / 28.5_dbl_kind, 33.7_dbl_kind, 34.0_dbl_kind,
     &                34.1_dbl_kind, 34.2_dbl_kind, 34.4_dbl_kind,
     &                34.5_dbl_kind, 34.5_dbl_kind, 34.6_dbl_kind,
     &                34.6_dbl_kind, 34.6_dbl_kind, 34.6_dbl_kind,
     &                34.6_dbl_kind, 34.6_dbl_kind, 34.6_dbl_kind,
     &                34.6_dbl_kind, 34.6_dbl_kind, 34.6_dbl_kind,
     &                34.6_dbl_kind, 34.6_dbl_kind, 34.6_dbl_kind,
     &                34.6_dbl_kind, 34.6_dbl_kind, 34.7_dbl_kind,
     &                34.7_dbl_kind, 34.7_dbl_kind, 34.7_dbl_kind,
     &                34.7_dbl_kind, 34.7_dbl_kind, 34.7_dbl_kind,
     &                34.7_dbl_kind, 34.7_dbl_kind, 34.7_dbl_kind / ),
     &  srefmax = ( / 37.0_dbl_kind, 36.6_dbl_kind, 35.8_dbl_kind,
     &                35.7_dbl_kind, 35.3_dbl_kind, 35.1_dbl_kind,
     &                35.1_dbl_kind, 35.0_dbl_kind, 35.0_dbl_kind,
     &                35.0_dbl_kind, 35.0_dbl_kind, 35.0_dbl_kind,
     &                35.0_dbl_kind, 35.0_dbl_kind, 35.0_dbl_kind,
     &                35.0_dbl_kind, 35.0_dbl_kind, 35.0_dbl_kind,
     &                35.0_dbl_kind, 35.0_dbl_kind, 35.0_dbl_kind,
     &                35.0_dbl_kind, 35.0_dbl_kind, 35.0_dbl_kind,
     &                35.0_dbl_kind, 35.0_dbl_kind, 35.0_dbl_kind,
     &                35.0_dbl_kind, 35.0_dbl_kind, 35.0_dbl_kind,
     &                35.0_dbl_kind, 35.0_dbl_kind, 35.0_dbl_kind / )

!-----------------------------------------------------------------------
!
!     set the temperature and salinity ranges to be used for each
!     model level when performing the polynomial fitting
!
!-----------------------------------------------------------------------

      do k=1,km
        i = int(zt(k)*mpercm/250.0_dbl_kind) + 1
        tmin(k) = trefmin(i)
        tmax(k) = trefmax(i)
        smin(k) = srefmin(i)
        smax(k) = srefmax(i)
      end do

!-----------------------------------------------------------------------
!
!  loop over all model levels
!
!-----------------------------------------------------------------------

      do k=1,km

!-----------------------------------------------------------------------
!
!       sample the temperature range with nsample_temp points
!       and the salinity range with nsample_salt points and
!       create an array of possible combinations of T,S samples
!
!-----------------------------------------------------------------------

        dtemp = (tmax(k)-tmin(k)) / (nsample_temp-c1)
        dsalt = (smax(k)-smin(k)) / (nsample_salt-c1)

        nsample = 0
        do i=1,nsample_temp
          temp_sample = tmin(k) + (i-1)*dtemp
          do j=1,nsample_salt
            nsample = nsample + 1
            salt_sample = smin(k) + (j-1)*dsalt
            tsamp(nsample) = temp_sample
            ssamp(nsample) = salt_sample
          end do
        end do

!-----------------------------------------------------------------------
!
!       loop over the number of samples
!
!-----------------------------------------------------------------------

        !***
        !*** initialize averaging sums
        !***

        to  (k) = c0
        so  (k) = c0
        sigo(k) = c0
        avg_theta(k) = c0

        do n=1,nsample_all

!-----------------------------------------------------------------------
!
!         calculate density (sigma) for each t,s combintion at
!         this depth using unesco equation of state
!         unesco returns density (kg per m**3)
!
!-----------------------------------------------------------------------

          call unesco(tsamp(n),ssamp(n),pressz(k),density)

          sigma(n) = density - 1.0e3_dbl_kind

!-----------------------------------------------------------------------
!
!         calculate potential temp. from from insitu temperature,
!         salinity, and pressure
!
!-----------------------------------------------------------------------

          call potem(tsamp(n),ssamp(n),pressz(k),thsamp(n))

!-----------------------------------------------------------------------
!
!         accumulate level averages and end loop over samples
!
!-----------------------------------------------------------------------

          to  (k) = to  (k) + tsamp(n)
          so  (k) = so  (k) + ssamp(n)
          sigo(k) = sigo(k) + sigma(n)
          avg_theta(k) = avg_theta(k) + thsamp(n)

        end do ! loop over samples

!-----------------------------------------------------------------------
!
!       complete layer averages
!
!-----------------------------------------------------------------------

        to  (k) = to  (k)/real(nsample_all)
        so  (k) = so  (k)/real(nsample_all)
        sigo(k) = sigo(k)/real(nsample_all)
        avg_theta(k) = avg_theta(k)/real(nsample_all)

!-----------------------------------------------------------------------
!
!       recompute average (reference) density based on level average 
!       values of T, S, and pressure.
!       use average potential temperature in place of average temp
!
!-----------------------------------------------------------------------

        call unesco (to(k), so(k), pressz(k), density)

        sigo(k) = density - 1.0e3_dbl_kind
        to  (k) = avg_theta(k)

!-----------------------------------------------------------------------
!
!       fill array for least squares routine with anomalies
!       and their products (the terms in the desired cubic
!       polynomial.
!
!-----------------------------------------------------------------------

        do n=1,nsample_all

          tsamp(n) = thsamp(n)   !*** replace temp with potential temp

          tanom = tsamp(n) - to(k)
          sanom = ssamp(n) - so(k)
          sigman(n) = sigma(n) - sigo(k)

          lsqarray(n,1) = tanom
          lsqarray(n,2) = sanom
          lsqarray(n,3) = tanom * tanom
          lsqarray(n,4) = tanom * sanom
          lsqarray(n,5) = sanom * sanom
          lsqarray(n,6) = tanom * tanom * tanom
          lsqarray(n,7) = sanom * sanom * tanom
          lsqarray(n,8) = tanom * tanom * sanom
          lsqarray(n,9) = sanom * sanom * sanom

        end do

!-----------------------------------------------------------------------
!
!       LSQL2 is  a Jet Propulsion Laboratory subroutine that
!       computes the least squares fit in an iterative manner for
!       overdetermined systems. it is called here to iteratively
!       determine polynomial coefficients for cubic polynomials
!       that will best fit the density at the sampled points
!
!-----------------------------------------------------------------------

        irank = 0
        in = 1
        it = 0
        ieq = 2
        call lsqsl2 (nsample_all, lsqarray, nsample_all, 9, sigman, 
     &               lsqcoeffs, irank, in, itmax, it, ieq, enorm, 
     &               1.0e-7_dbl_kind, 9, homog, lsqc, lsqr, lsqsb)

!-----------------------------------------------------------------------
!
!       store coefficients
!
!-----------------------------------------------------------------------

        state_coeffs(:,k) = lsqcoeffs

!-----------------------------------------------------------------------
!
!       end of loop over levels
!
!-----------------------------------------------------------------------

      end do

!-----------------------------------------------------------------------
!
!     rescale some of the coefficients and reference values for correct 
!     units
!
!-----------------------------------------------------------------------

      do k=1,km
        sigo(k) = p001 * sigo(k)
        so  (k) = ppt_to_salt * so  (k) - 0.035_dbl_kind

        state_coeffs(1,k) = 1.e-3_dbl_kind * state_coeffs(1,k)
        state_coeffs(3,k) = 1.e-3_dbl_kind * state_coeffs(3,k)
        state_coeffs(5,k) = 1.e+3_dbl_kind * state_coeffs(5,k)
        state_coeffs(6,k) = 1.e-3_dbl_kind * state_coeffs(6,k)
        state_coeffs(7,k) = 1.e+3_dbl_kind * state_coeffs(7,k)
        state_coeffs(9,k) = 1.e+6_dbl_kind * state_coeffs(9,k)
      end do
      do k=1,km
        sigo(k) = sigo(k) * c1000
      end do

!-----------------------------------------------------------------------

      end subroutine init_state_coeffs

!***********************************************************************

      subroutine potem (temp, salt, pbars, theta)

!-----------------------------------------------------------------------
!
!     this subroutine calculates potential temperature as a function
!     of in-situ temperature, salinity, and pressure.
!
!     reference:
!        Bryden, H.L., New polynomials for thermal expansion, adiabatic
!           temperature gradient and potential temperature of sea water,
!           Deap-Sea Res., 20, 401-408, 1973.
!
!-----------------------------------------------------------------------

      implicit none

!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), intent(in) ::
     &  temp,           ! in-situ temperature [degrees centigrade]
     &  salt,           ! salinity [per mil]
     &  pbars           ! pressure [bars]

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), intent(out) ::
     &  theta           ! potential temperature [degrees centigrade]

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind) :: prss2, prss3, potmp

!-----------------------------------------------------------------------
!
!     compute powers of several variables
!
!-----------------------------------------------------------------------

      prss2 = pbars*pbars
      prss3 = prss2*pbars

!-----------------------------------------------------------------------
!
!     compute potential temperature from polynomial
!
!-----------------------------------------------------------------------

      potmp = pbars*(3.6504e-4_dbl_kind + temp*(8.3198e-5_dbl_kind +
     &        temp*(-5.4065e-7_dbl_kind + temp*4.0274e-9_dbl_kind))) +
     &        pbars*(salt - 35.0_dbl_kind)*(1.7439e-5_dbl_kind -
     &        temp*2.9778e-7_dbl_kind) + prss2*(8.9309e-7_dbl_kind +
     &        temp*(-3.1628e-8_dbl_kind + temp*2.1987e-10_dbl_kind)) -
     &        4.1057e-9_dbl_kind*prss2*(salt - 35.0_dbl_kind) +
     &        prss3*(-1.6056e-10_dbl_kind + temp*5.0484e-12_dbl_kind)

      theta = temp - potmp

!-----------------------------------------------------------------------

      end subroutine potem

!***********************************************************************

      subroutine unesco (temp, salt, pbars, rho)

!-----------------------------------------------------------------------
!
!     this subroutine calculates the density of seawater using the
!     standard equation of state recommended by unesco (1981).
!
!     references:
!        Gill, A., Atmosphere-Ocean Dynamics: International Geophysical
!         Series No. 30. Academic Press, London, 1982, pp 599-600.
!        UNESCO, 10th report of the joint panel on oceanographic tables
!          and standards. UNESCO Tech. Papers in Marine Sci. No. 36,
!          Paris, 1981.
!
!-----------------------------------------------------------------------

      implicit none

!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), intent(in) ::
     &  temp,           ! in-situ temperature [degrees centigrade]
     &  salt,           ! salinity [practical salinity units]
     &  pbars           ! pressure [bars]

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), intent(out) ::
     &  rho             ! density in kilograms per cubic meter

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind) :: rw, rsto, xkw, xksto, xkstp,
     &                    tem2, tem3, tem4, tem5, slt2, st15, pbar2 

!-----------------------------------------------------------------------
!
!     compute powers of several variables
!
!-----------------------------------------------------------------------

      tem2 = temp**2
      tem3 = temp**3
      tem4 = temp**4
      tem5 = temp**5
      slt2 = salt**2
      st15 = salt**(1.5_dbl_kind) 
      pbar2 = pbars**2

!-----------------------------------------------------------------------
!
!     compute density
!
!-----------------------------------------------------------------------

      rw = unt0 + unt1*temp + unt2*tem2 + unt3*tem3 + 
     &                        unt4*tem4 + unt5*tem5

      rsto = rw + 
     &       (uns1t0 + uns1t1*temp + uns1t2*tem2 + 
     &                 uns1t3*tem3 + uns1t4*tem4)*salt
     &     + (unsqt0 + unsqt1*temp + unsqt2*tem2)*st15 + uns2t0*slt2

      xkw =    1.965221e+4_dbl_kind + 
     &         1.484206e+2_dbl_kind*temp - 2.327105e+0_dbl_kind*tem2 +
     &         1.360477e-2_dbl_kind*tem3 - 5.155288e-5_dbl_kind*tem4

      xksto =  xkw + 
     &    (5.46746e+1_dbl_kind      - 6.03459e-1_dbl_kind*temp + 
     &     1.09987e-2_dbl_kind*tem2 - 6.1670e-5_dbl_kind*tem3)*salt
     &  + (7.944e-2_dbl_kind  + 
     &     1.6483e-2_dbl_kind *temp - 5.3009e-4_dbl_kind*tem2)*st15

      xkstp =  xksto + 
     &   (3.239908e+0_dbl_kind     + 1.43713e-3_dbl_kind*temp + 
     &    1.16092e-4_dbl_kind*tem2 - 5.77905e-7_dbl_kind*tem3)*pbars
     & + (2.2838e-3_dbl_kind       - 1.0981e-5_dbl_kind *temp - 
     &    1.6078e-6_dbl_kind *tem2)*pbars*salt
     & + 1.91075e-4_dbl_kind       *pbars*st15
     & + (8.50935e-5_dbl_kind      - 6.12293e-6_dbl_kind*temp + 
     &                               5.2787e-8_dbl_kind *tem2)*pbar2
     & + (-9.9348e-7_dbl_kind      + 2.0816e-8_dbl_kind *temp + 
     &    9.1697e-10_dbl_kind*tem2)*pbar2*salt

      rho =   rsto / (c1 - pbars/xkstp)

!-----------------------------------------------------------------------

      end subroutine unesco

!***********************************************************************

      function pressure(depth)

!-----------------------------------------------------------------------
!
!     this function computes pressure in bars from depth in meters
!     by using a mean density derived from depth-dependent global 
!     average temperatures and salinities from Levitus_94, and 
!     integrating using hydrostatic balance.
!
!     references:
!        Levitus, S., R. Burgett, and T.P. Boyer, World Ocean Atlas 
!          1994, Volume 3: Salinity, NOAA Atlas NESDIS 3, US Dept. of 
!          Commerce, 1994.
!        Levitus, S. and T.P. Boyer, World Ocean Atlas 1994, 
!          Volume 4: Temperature, NOAA Atlas NESDIS 4, US Dept. of 
!          Commerce, 1994.
!        Dukowicz, J. K., 2000: Reduction of Pressure and Pressure
!          Gradient Errors in Ocean Simulations, J. Phys. Oceanogr.,
!          submitted.
!
!-----------------------------------------------------------------------

      implicit none

!-----------------------------------------------------------------------
!
!     input variable
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), intent(in) :: depth    ! depth in meters

!-----------------------------------------------------------------------
!
!     output variable
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind) :: pressure   ! pressure [bars]

!-----------------------------------------------------------------------
!
!     convert depth in meters to pressure in bars
!
!-----------------------------------------------------------------------

      pressure = 0.059808_dbl_kind*(exp(-0.025_dbl_kind*depth) - c1)
     &         + 0.100766_dbl_kind*depth + 2.28405e-7_dbl_kind*depth**2

!-----------------------------------------------------------------------

      end function pressure

!***********************************************************************

      subroutine lsqsl2 (ndim,a,d,w,b,x,irank,in,itmax,it,ieq,enorm,
     &                   eps1,nhdim,h,aa,r,s)

!-----------------------------------------------------------------------
!
!     this routine is a modification of lsqsol. march,1968. r. hanson.
!     linear least squares solution
!
!     this routine finds x such that the euclidean length of
!     (*) ax-b is a minimum.
!
!     here a has k rows and n columns, while b is a column vector with
!     k components.
!
!     an orthogonal matrix q is found so that qa is zero below
!     the main diagonal.
!     suppose that rank (a)=r
!     an orthogonal matrix s is found such that
!     qas=t is an r x n upper triangular matrix whose last n-r columns
!     are zero.
!     the system tz=c (c the first r components of qb) is then
!     solved. with w=sz, the solution may be expressed
!     as x = w + sy, where w is the solution of (*) of minimum euclid-
!     ean length and y is any solution to (qas)y=ty=0.
!
!     iterative improvements are calculated using residuals and
!     the above procedures with b replaced by b-ax, where x is an
!     approximate solution.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!      input variables
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!      output variables
!
!-----------------------------------------------------------------------
!      subroutine lsqsl2 (ndim,a,d,w,b,x,irank,in,itmax,it,ieq,enorm,
!     &                   eps1,nhdim,h,aa,r,s)
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: ndim,d,w,irank,in,itmax,it,ieq,nhdim

      real (kind=dbl_kind) :: a,b,x,enorm,eps1,h,aa,r,s
      real (kind=dbl_kind) :: sj,dp,up,bp,aj

      logical (kind=log_kind) :: erm

!
!     in=1 for first entry.
!                   a is decomposed and saved. ax-b is solved.
!     in = 2 for subsequent entries with a new vector b.
!     in=3 to restore a from the previous entry.
!     in=4 to continue the iterative improvement for this system.
!     in = 5 to calculate solutions to ax=0, then store in the array h.
!     in  =  6   do not store a  in aa.  obtain  t = qas, where t is
!     min(k,n) x min(k,n) and upper triangular. now return.do not obtain
!     a solution.
!     no scaling or column interchanges are performed.
!     in  =  7   same as with  in = 6  except that soln. of min. length
!                is placed into x. no iterative refinement.  now return.
!     column interchanges are performed. no scaling is performed.
!     in  = 8    set addresses. now return.
!
!     options for computing  a matrix product   y*h  or  h*y are
!     available with the use of the entry points  myh and mhy.
!     use of these options in these entry points allow a great saving in
!     storage required.
!
!
      dimension a(d,w),b(1),aa(d,w),s(1), x(1),h(nhdim,nhdim),r(1)


      integer (kind=int_kind) :: 
     &  i,j,k,l,m,n,isw,ip,lm,irp1,irm1,n1,ns,k1,k2

      real (kind=dbl_kind) :: epsloc,am,a1,sp,top,top1,top2,enm1,a2

      integer (kind=int_kind) :: j1, j2, j3, j4, j5, j6, j7, j8, j9

!     d = depth of matrix.
!     w = width of matrix.

      k=d
      n=w

!-----------------------------------------------------------------------
!
!     if it=0 on entry, the possible error message will be suppressed.
!
!-----------------------------------------------------------------------

      if (it /= 0) then
        erm = .true.
      else
        erm = .false.
      endif

!-----------------------------------------------------------------------
!
!     set desired relative accuracy (eps1  controls (eps) rank)
!
!-----------------------------------------------------------------------

      epsloc = 1.e-16_dbl_kind

!-----------------------------------------------------------------------
!
!     ieq = 2      if column scaling by least max. column length is
!     to be performed.
!
!     ieq = 1       if scaling of all components is to be done with
!     the scalar max(abs(aij))/k*n.
!
!     ieq = 3 if column scaling as with in =2 will be retained in
!     rank deficient cases.
!
!     the array s must contain at least max(k,n) + 4n + 4min(k,n) cells
!        the   array r must contain k+4n s.p. cells.
!
!-----------------------------------------------------------------------

      isw = 1
      l   = min(k,n)
      m   = max(k,n)
      j1  = m
      j2  = j1 + n
      j3  = j2 + n
      j4  = j3 + l
      j5  = j4 + l
      j6  = j5 + l
      j7  = j6 + l
      j8  = j7 + n
      j9  = j8 + n

      if (irank >= 1 .and. irank <= l) then
        lm = irank
      else
        lm  = l
      endif

      select case (in)
      case (1)
        goto 10
      case (2)
        goto 360
      case (3)
        goto 810
      case (4)
        goto 390
      case (5)
        goto 830
      case (6)
        lm = l
        goto 10
      case (7)
        goto 10
      case (8)
        return
      case default
        call exit_POP('unknown choice for in in lsqsl2'
     &,                normal_exit=.false.)
      end select

!-----------------------------------------------------------------------
!
!     equilibrate columns of a (1)-(2).
!
!-----------------------------------------------------------------------

   10 continue

!-----------------------------------------------------------------------
!
!     save data when in = 1.
!
!-----------------------------------------------------------------------

      if (in == 1) then
        aa = a
      endif

      if (ieq /= 1) then
        do j=1,n
          am = c0
          do i=1,k
            am = max(am,abs(a(i,j)))
          end do

          !*** s(m+n+1)-s(m+2n) contains scaling for output variables.

          if (in == 6) am = c1
          s(j2+j) = c1/am
          do i=1,k
            a(i,j) = a(i,j)*s(j2+j)
          end do
        end do
      else
        am = c0
        do j=1,n
          do i=1,k
            am = max(am,abs(a(i,j)))
          end do
        end do
        am = am/real(k*n)
        if (in == 6) am = c1
        do j=1,n
          s(j2+j) = c1/am
          do i=1,k
            a(i,j) = a(i,j)*s(j2+j)
          end do
        end do
      endif

!-----------------------------------------------------------------------
!
!     compute column lengths
!
!-----------------------------------------------------------------------

      do j=1,n
        s(j7+j) = s(j2+j)
      end do

!-----------------------------------------------------------------------
!
!      s(m+1)-s(m+ n) contains variable permutations.
!     set permutation to identity.
!
!-----------------------------------------------------------------------

      do j=1,n
        s(j1+j) = j
      end do

!-----------------------------------------------------------------------
!
!     begin elimination on the matrix a with orthogonal matrices .
!     ip=pivot row
!
!-----------------------------------------------------------------------

      do 250 ip=1,lm

        dp = c0
        k2 = ip
        do j=ip,n
          sj = c0
          do i=ip,k
            sj = sj + a(i,j)**2
          end do
          if (sj >= dp) then
            dp = sj
            k2 = j
            if (in == 6) go to 160
          endif
        end do

!-----------------------------------------------------------------------
!
!       maximize (sigma)**2 by column interchange.
!        supress column interchanges when in=6.
!        exchange columns if necessary.
!
!-----------------------------------------------------------------------

        if (k2 == ip) go to 160
        do i=1,k
          a1      = a(i,ip)
          a(i,ip) = a(i,k2)
          a(i,k2) = a1
        end do

!-----------------------------------------------------------------------
!
!       record permutation and exchange squares of column lengths.
!
!-----------------------------------------------------------------------

        a1       = s(j1+k2)
        s(j1+k2) = s(j1+ip)
        s(j1+ip) = a1

        a1       = s(j7+k2)
        s(j7+k2) = s(j7+ip)
        s(j7+ip) = a1

  160   if (ip /= 1) then
          a1 = c0
          do i=1,ip-1
            a1 = a1 + a(i,ip)**2
          end do
          if (a1 > c0) go to 190
        end if
        if (dp > c0) go to 200

!-----------------------------------------------------------------------
!
!       test for rank deficiency.
!
!-----------------------------------------------------------------------

  190   if (sqrt(dp/a1) > eps1 .or. in == 6) go to 200
        if (erm) write (6,1140) irank,eps1,ip-1,ip-1
        irank = ip-1
        erm = .false.
        go to 260

!-----------------------------------------------------------------------
!
!       (eps1) rank is deficient.
!
!-----------------------------------------------------------------------

  200   sp = sqrt(dp)

!-----------------------------------------------------------------------
!
!       begin front elimination on column ip.
!       sp=sqroot(sigma**2).
!
!-----------------------------------------------------------------------

        bp = c1/(dp+sp*abs(a(ip,ip)))

!-----------------------------------------------------------------------
!
!       store beta in s(3n+1)-s(3n+l).
!
!-----------------------------------------------------------------------

        if (ip == k) bp = c0
        r(k+2*n+ip) = bp
        up = sign(sp + abs(a(ip,ip)), a(ip,ip))
        if (ip < k) then
          if (ip < n) then
            do j=ip+1,n
              sj = c0
              do i=ip+1,k
                sj=sj+a(i,j)*a(i,ip)
              end do
              sj = sj + up*a(ip,j)
              sj = bp*sj  !*** sj=yj now
  
              do i=ip+1,k
                a(i,j) = a(i,j) - a(i,ip)*sj
              end do
              a(ip,j) = a(ip,j) - sj*up
            end do
          endif
          a(ip,ip)    = -sign(sp,a(ip,ip))
          r(k+3*n+ip) = up
        endif

  250 continue
      irank = lm
  260 irp1 = irank+1
      irm1 = irank-1

      if (irank == 0 .or. irank == n) go to 360

!-----------------------------------------------------------------------
!
!     begin back processing for rank deficiency case
!      if irank is less than n.
!
!-----------------------------------------------------------------------

      if (ieq /= 3) then
        do j=1,n
          l = min(j,irank)

          !*** unscale columns for rank deficient matrices when ieq /= 3
          do i=1,l
            a(i,j) = a(i,j)/s(j7+j)
          end do

          s(j7+j) = c1
          s(j2+j) = c1
        end do
      endif

      ip = irank

  300 sj = c0
      do j=irp1,n
        sj = sj + a(ip,j)**2
      end do
      sj = sj + a(ip,ip)**2
      aj = sqrt(sj)
      up = sign(aj+abs(a(ip,ip)), a(ip,ip))

!-----------------------------------------------------------------------
!
!     ip th element of u vector calculated.
!     bp = 2/length of u squared.
!
!-----------------------------------------------------------------------

      bp = c1/(sj+abs(a(ip,ip))*aj)

      if (ip-1 > 0) then
        do i=1,ip-1
          dp = a(i,ip)*up
          do j=irp1,n
            dp = dp + a(i,j)*a(ip,j)
          end do
          dp = dp/(sj+abs(a(ip,ip))*aj)

          !*** calc. (aj,u), where aj=jth row of a

          a(i,ip) = a(i,ip) - up*dp

          !*** modify array a.

          do j=irp1,n
            a(i,j) = a(i,j) - a(ip,j)*dp
          end do
        end do
      endif

      a(ip,ip) = -sign(aj,a(ip,ip))

!-----------------------------------------------------------------------
!
!     calc. modified pivot.
!     save beta and ip th element of u vector in r array.
!
!-----------------------------------------------------------------------

      r(k+ip)   = bp
      r(k+n+ip) = up

!-----------------------------------------------------------------------
!
!     test for end of back processing.
!
!-----------------------------------------------------------------------

      if (ip-1 > 0) then
        ip=ip-1
        go to 300
      endif

  360 if (in == 6) return

      do j=1,k
        r(j) = b(j)
      end do
      it=0

!-----------------------------------------------------------------------
!
!     set initial x vector to zero.
!
!-----------------------------------------------------------------------

      do j=1,n
        x(j) = c0
      end do
      if (irank == 0) go to 690

!-----------------------------------------------------------------------
!
!     apply q to rt. hand side.
!
!-----------------------------------------------------------------------

  390 do ip=1,irank
        sj = r(k+3*n+ip)*r(ip)
        if (ip+1 <= k) then
          do i=ip+1,k
            sj = sj + a(i,ip)*r(i)
          end do
        endif
        bp = r(k+2*n+ip)
        if (ip+1 <= k) then
          do i=ip+1,k
            r(i) = r(i) - bp*a(i,ip)*sj
          end do
        endif
        r(ip) = r(ip) - bp*r(k+3*n+ip)*sj
      end do

      do j=1,irank
        s(j) = r(j)
      end do

      enorm = c0
      if (irp1.gt.k) go to 510
      do j=irp1,k
        enorm = enorm + r(j)**2
      end do
      enorm = sqrt(enorm)
      go to 510

  460 do j=1,n
        sj = c0
        ip = s(j1+j)
        do i=1,k
          sj = sj + r(i)*aa(i,ip)
        end do

        !*** apply to rt. hand side.  apply scaling.
c
        r(k+n+j) = sj*s(j2+ip)
      end do

      s(1) = r(k+n+1)/a(1,1)
      if (n /= 1) then
        do j=2,n
          sj = c0
          do i=1,j-1
            sj = sj + a(i,j)*s(i)
          end do
          s(j) = (r(k+j+n)-sj)/a(j,j)
        end do
      endif

!-----------------------------------------------------------------------
!
!     entry to continue iterating.  solves tz = c = 1st irank
!     components of qb .
!
!-----------------------------------------------------------------------

  510 s(irank) = s(irank)/a(irank,irank)
      if (irm1 /= 0) then
        do j=1,irm1
          n1 = irank-j
          sj = 0.
          do i=n1+1,irank
            sj = sj + a(n1,i)*s(i)
          end do
          s(n1) = (s(n1)-sj)/a(n1,n1)
        end do
      endif

!-----------------------------------------------------------------------
!
!     z calculated.  compute x = sz.
!
!-----------------------------------------------------------------------

      if (irank /= n) then
        do j=irp1,n
          s(j) = c0
        end do
        do i=1,irank
          sj = r(k+n+i)*s(i)
          do j=irp1,n
            sj = sj + a(i,j)*s(j)
          end do
          do j=irp1,n
            s(j) = s(j) - a(i,j)*r(k+i)*sj
          end do
          s(i) = s(i) - r(k+i)*r(k+n+i)*sj
        end do
      endif

!-----------------------------------------------------------------------
!
!     increment for x of minimal length calculated.
!
!-----------------------------------------------------------------------

      do i=1,n
        x(i) = x(i) + s(i)
      end do

      if (in.eq.7) go to 750

!-----------------------------------------------------------------------
!
!     calc. sup norm of increment and residuals
!
!-----------------------------------------------------------------------

      top1 = c0
      do j=1,n
        top1 = max(top1,abs(s(j))*s(j7+j))
      end do

      do i=1,k
        sj = c0
        do j=1,n
          ip = s(j1+j)
          sj = sj + aa(i,ip)*x(j)*s(j2+ip)
        end do
        r(i) = b(i) - sj
      end do

      if (itmax <= 0) go to 750

!-----------------------------------------------------------------------
!
!     calc. sup norm of x.
!
!-----------------------------------------------------------------------

      top = c0
      do j=1,n
        top = max(top,abs(x(j))*s(j7+j))
      end do

!-----------------------------------------------------------------------
!
!     compare relative change in x with tolerance eps .
!
!-----------------------------------------------------------------------

      if (top1-top*epsloc > 0) then
        if (it-itmax < 0) then
          it = it+1
          if (it /= 1) then
            if (top1 > p25*top2) go to 690
          endif
          top2 = top1
          if (isw == 1) then
            go to 390
          else if (isw ==2) then
            go to 460
          endif
        endif
        it=0
      endif
  690 sj=c0
      do j=1,k
        sj = sj + r(j)**2
      end do
      enorm = sqrt(sj)

      if (irank == n .and. isw == 1) then
        enm1 = enorm

        !*** save x array

        do j=1,n
          r(k+j) = x(j)
        end do
        isw = 2
        it = 0
        go to 460
      endif

!-----------------------------------------------------------------------
!
!     choose best solution
!
!-----------------------------------------------------------------------

      if (irank >= n .and. enorm > enm1) then
        do j=1,n
          x(j) = r(k+j)
        end do
        enorm = enm1
      endif

!-----------------------------------------------------------------------
!
!     norm of ax - b located in the cell enorm .
!     rearrange variables.
!
!-----------------------------------------------------------------------

  750 do j=1,n
        s(j) = s(j1+j)
      end do
      do j=1,n
        do i=j,n
          ip = s(i)
          if (j == ip) go to 780
        end do
  780   s(i) = s(j)
        s(j) = j
        sj   = x(j)
        x(j) = x(i)
        x(i) = sj
      end do

!-----------------------------------------------------------------------
!
!     scale variables.
!
!-----------------------------------------------------------------------

      do j=1,n
        x(j) = x(j)*s(j2+j)
      end do
      return

!-----------------------------------------------------------------------
!
!     restore a.
!
!-----------------------------------------------------------------------

  810 do j=1,n
        do i=1,k
          a(i,j) = aa(i,j)
        end do
      end do
      return

!-----------------------------------------------------------------------
!
!     generate solutions to the homogeneous equation ax = 0.
!
!-----------------------------------------------------------------------

  830 if (irank == n) return
      ns = n-irank
      do i=1,n
        do j=1,ns
          h(i,j) = c0
        end do
      end do

      do j=1,ns
        h(irank+j,j) = c1
      end do

      if (irank == 0) return

      do j=1,irank
        do i=1,ns
          sj = r(k+n+j)*h(j,i)
          do k1=irp1,n
            sj = sj + h(k1,i)*a(j,k1)
          end do
          bp = r(k+j)
          dp = bp*r(k+n+j)*sj
          a1 = dp
          a2 = dp-a1
          h(j,i) = h(j,i)-(a1+c2*a2)
          do k1=irp1,n
            dp = bp*a(j,k1)*sj
            a1 = dp
            a2 = dp - a1
            h(k1,i)=h(k1,i)-(a1+c2*a2)
          end do
        end do
      end do

!-----------------------------------------------------------------------
!
!     rearrange rows of solution matrix.
!
!-----------------------------------------------------------------------

      do j=1,n
        s(j) = s(j1+j)
      end do
      do j=1,n
        do i=j,n
          ip = s(i)
          if (j == ip) go to 900
        end do
  900   s(i) = s(j)
        s(j) = j
        do k1=1,ns
          a1      = h(j,k1)
          h(j,k1) = h(i,k1)
          h(i,k1) = a1
        end do
      end do

!-----------------------------------------------------------------------

 1140 format (/'warning. irank has been set to',i4,'  but(',1pe10.3,
     & ') rank is',i4,'.  irank is now taken as ',i4)

!-----------------------------------------------------------------------

      end subroutine lsqsl2

!***********************************************************************

      end module state_mod

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
