!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module step_mod

!***********************************************************************
!
!     contains the routine for stepping the model forward one timestep
!
!     CVS:$Id: step_mod.F,v 1.3.2.4 2001/08/10 19:50:27 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use prognostic
      use solvers
      use global_reductions
      use boundary
      use stencils
      use timers
      use grid
      use io
      use diagnostics
      use state_mod
      use time_management
      use xdisplay
      use baroclinic
      use barotropic
      use surface_hgt
      use tavg
      use forcing
      use forcing_coupled
      use ice
      use budget_diagnostics

      implicit none
      save

      integer (kind=int_kind), private ::
     &  cpl_stop_now          ! flag id for stop_now flag

!***********************************************************************

      contains

!***********************************************************************

      subroutine step

!-----------------------------------------------------------------------
!
!     advance one leapfrog timestep
!
!   * this routine controls logic for leapfrog timesteps.
!     it also performs time-averaging (or robert filtering).
!     prognostic variables are updated for the next timestep near
!     the end of the routine.
!
!   * averaging is only allowed on leapfrog timesteps, averaging
!     and matsuno mixing cannot occur on the same timestep.
!
!   * on matsuno steps, the time (n) velocity and tracer arrays
!     UBTROP,VBTROP,UVEL,VVEL,TRACER contain the predicted new 
!     velocities from the 1st pass for use in the 2nd pass.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     local or common variables:
!
!-----------------------------------------------------------------------
 
      integer k,i,j,n,m,tmptime

      logical (kind=log_kind), save :: first_call = .true.
      logical (kind=log_kind), save :: first_global_budget = .true.

      integer (kind=int_kind), save :: timer_baroclinic, 
     &                                 timer_barotropic

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  WORK1,WORK2         ! local work space

!-----------------------------------------------------------------------
!
!     if this is the first call to step, start some timers
!
!-----------------------------------------------------------------------

      if (first_call) then

        cpl_stop_now  = init_time_flag('stop_now',default=.false.)
        call get_timer(timer_baroclinic,'BAROCLINIC')
        call get_timer(timer_barotropic,'BAROTROPIC')


        first_call = .false.
      endif

#ifdef hydrographic_stations
!-----------------------------------------------------------------------
!
!     Gather data for comparison with hydrographic data
!
!-----------------------------------------------------------------------

      if(newday) call data_stations
      if(newday .and. (mod(iday-1,3).eq.0) ) call data_slices

#endif
#ifdef current_meters
!-----------------------------------------------------------------------
!
!     Gather data for comparison with current meter data
!
!-----------------------------------------------------------------------

      if(newday) call data_cmeters

#endif

!-----------------------------------------------------------------------
!
!     initialize the global budget arrays
!
!-----------------------------------------------------------------------

        if ( first_global_budget         .and. 
     &       ldiag_global_tracer_budgets .and. 
     &       time_to_start(tavg_start_iopt, tavg_start) ) then
             call diag_for_tracer_budgets (tracer_mean_initial,
     &                                     volume_t_initial)

          if ( my_task == master_task ) then
            write (stdout,1001) volume_t_initial,
     &                          tracer_mean_initial(1),
     &                          salt_to_ppt * tracer_mean_initial(2)
          endif
 
          first_global_budget = .false.
        endif
 
!-----------------------------------------------------------------------
!
!     read fields for surface forcing
!
!-----------------------------------------------------------------------

      call set_surface_forcing

      if (check_time_flag(cpl_stop_now)) RETURN

!-----------------------------------------------------------------------
!
!     update timestep counter, set corresponding model time, set
!     time-dependent logical switches to determine program flow.
!
!-----------------------------------------------------------------------

      call time_manager(lcoupled, liceform)

!-----------------------------------------------------------------------
!
!     compute and initialize some time-average diagnostics
!
!-----------------------------------------------------------------------

      call tavg_set_flag

      if (ltavg_on) then
        call tavg_2d_compute
      endif

!-----------------------------------------------------------------------
!
!     set timesteps and time-centering parameters for leapfrog or
!     matsuno steps.
!
!-----------------------------------------------------------------------

      mix_pass = 0

1000  continue        ! return here for 2nd pass in matsuno timestep

      if (matsuno_ts) mix_pass = mix_pass + 1

      if (leapfrogts) then  ! leapfrog timestep
        mixtime = oldtime
        beta  = alpha
        do k = 1,km
           c2dtt(k) = c2*dt(k)
        enddo
        c2dtu = c2*dtu
        c2dtp = c2*dtp    ! barotropic timestep = baroclinic timestep
        c2dtq = c2*dtu    ! turbulence timestep = mean flow timestep
      else
        mixtime = curtime
        beta  = theta
        do k = 1,km
           c2dtt(k) = dt(k)
        enddo
        c2dtu = dtu
        c2dtp = dtp       ! barotropic timestep = baroclinic timestep
        c2dtq = dtu       ! turbulence timestep = mean flow timestep
      endif

!-----------------------------------------------------------------------
!
!     on 1st pass of matsuno, set time (n-1) variables equal to
!     time (n) variables.
!
!-----------------------------------------------------------------------

      if (mix_pass == 1) then

        UBTROP(:,:,oldtime) = UBTROP(:,:,curtime)
        VBTROP(:,:,oldtime) = VBTROP(:,:,curtime)
        UVEL(:,:,:,oldtime) = UVEL(:,:,:,curtime)
        VVEL(:,:,:,oldtime) = VVEL(:,:,:,curtime)
        TRACER(:,:,:,:,oldtime) = TRACER(:,:,:,:,curtime)
        RHO(:,:,:,oldtime) = RHO(:,:,:,curtime)

      endif

!-----------------------------------------------------------------------
!
!     initialize diagnostic flags and sums
!
!-----------------------------------------------------------------------

      call diag_init_sums

!-----------------------------------------------------------------------
!
!     calculate change in surface height dh/dt from surface pressure
!
!-----------------------------------------------------------------------

      call dhdt

!-----------------------------------------------------------------------
!
!     integrate baroclinic equations explicitly to find tracers and
!     baroclinic velocities at new time.
!
!-----------------------------------------------------------------------

      call timer_start(timer_baroclinic)
      call baroclinic_driver
      call timer_stop(timer_baroclinic)

      call boundary_baroclinic(ZX, ZY)

!-----------------------------------------------------------------------
!
!     solve barotropic equations implicitly to find surface pressure
!     (or streamfunction) and barotropic velocities.
!
!-----------------------------------------------------------------------

      call timer_start(timer_barotropic)
      call barotropic_driver
      call timer_stop(timer_barotropic)

!-----------------------------------------------------------------------
!
!     update tracers using surface height at new time
!     also peform adjustment-like physics (convection, ice formation)
!
!-----------------------------------------------------------------------

      call baroclinic_correct_adjust

      call boundary_barotropic(UBTROP(:,:,newtime),
     &                         VBTROP(:,:,newtime),
     &                           UVEL(:,:,:,newtime),
     &                           VVEL(:,:,:,newtime),
     &                            RHO(:,:,:,newtime),
     &                         TRACER(:,:,:,:,newtime))

!-----------------------------------------------------------------------
!
!     add barotropic to baroclinic velocities at new time
!
!-----------------------------------------------------------------------

      do k=1,km
        where (k.le.KMU)
          UVEL(:,:,k,newtime) = UVEL(:,:,k,newtime) + 
     &                          UBTROP(:,:,newtime)
          VVEL(:,:,k,newtime) = VVEL(:,:,k,newtime) + 
     &                          VBTROP(:,:,newtime)
        endwhere
      enddo

!-----------------------------------------------------------------------
!
!     on matsuno mixing steps update variables and return for 2nd pass
!     note: first step is forward only.
!
!-----------------------------------------------------------------------

      if (mix_pass == 1) then

        UBTROP(:,:,curtime) = UBTROP(:,:,newtime)
        VBTROP(:,:,curtime) = VBTROP(:,:,newtime)
        UVEL(:,:,:,curtime) = UVEL(:,:,:,newtime)
        VVEL(:,:,:,curtime) = VVEL(:,:,:,newtime)
        TRACER(:,:,:,:,curtime) = TRACER(:,:,:,:,newtime)
        RHO(:,:,:,curtime) = RHO(:,:,:,newtime)

        go to 1000    ! return for 2nd pass in matsuno step

      endif

!-----------------------------------------------------------------------
!
!     extrapolate next guess for pressure from three known time levels
!
!-----------------------------------------------------------------------

      PGUESS = c3*(PSURF(:,:,newtime) - PSURF(:,:,curtime)) + 
     &             PSURF(:,:,oldtime)

!-----------------------------------------------------------------------
!
!     compute some global diagnostics 
!     before updating prognostic variables
!
!-----------------------------------------------------------------------

      call diag_global_preupdate

!-----------------------------------------------------------------------
!
!     update prognostic variables for next timestep:
!       on normal timesteps
!         (n) -> (n-1)
!         (n+1) -> (n) 
!       on averaging timesteps
!         [(n) + (n-1)]/2 -> (n-1)
!         [(n+1) + (n)]/2 -> (n)
!
!-----------------------------------------------------------------------

      if ( avg_ts .or. back_to_back ) then     ! averaging step

        UBTROP(:,:,oldtime) = p5*(UBTROP(:,:,oldtime) + 
     &                            UBTROP(:,:,curtime))
        VBTROP(:,:,oldtime) = p5*(VBTROP(:,:,oldtime) + 
     &                            VBTROP(:,:,curtime))
        UVEL(:,:,:,oldtime) = p5*(UVEL(:,:,:,oldtime) + 
     &                            UVEL(:,:,:,curtime))
        VVEL(:,:,:,oldtime) = p5*(VVEL(:,:,:,oldtime) + 
     &                            VVEL(:,:,:,curtime))
        UBTROP(:,:,curtime) = p5*(UBTROP(:,:,curtime) + 
     &                            UBTROP(:,:,newtime))
        VBTROP(:,:,curtime) = p5*(VBTROP(:,:,curtime) + 
     &                            VBTROP(:,:,newtime))
        UVEL(:,:,:,curtime) = p5*(UVEL(:,:,:,curtime) + 
     &                            UVEL(:,:,:,newtime))
        VVEL(:,:,:,curtime) = p5*(VVEL(:,:,:,curtime) + 
     &                            VVEL(:,:,:,newtime))

        if (sfc_layer_type == sfc_layer_varthick) then

          TRACER(:,:,2:km,:,oldtime) = p5*(TRACER(:,:,2:km,:,oldtime) + 
     &                                  TRACER(:,:,2:km,:,curtime))
          TRACER(:,:,2:km,:,curtime) = p5*(TRACER(:,:,2:km,:,curtime) + 
     &                                  TRACER(:,:,2:km,:,newtime))
          do n = 1,nt
           TRACER(:,:,1,n,oldtime) = p5*(
     &      (dz(1) + PSURF(:,:,oldtime)/grav)*TRACER(:,:,1,n,oldtime) + 
     &      (dz(1) + PSURF(:,:,curtime)/grav)*TRACER(:,:,1,n,curtime) ) 
           TRACER(:,:,1,n,curtime) = p5*(
     &      (dz(1) + PSURF(:,:,curtime)/grav)*TRACER(:,:,1,n,curtime) + 
     &      (dz(1) + PSURF(:,:,newtime)/grav)*TRACER(:,:,1,n,newtime) ) 
          enddo

          PSURF(:,:,oldtime) = p5*(PSURF(:,:,oldtime) + 
     &                             PSURF(:,:,curtime))
          PSURF(:,:,curtime) = p5*(PSURF(:,:,curtime) + 
     &                             PSURF(:,:,newtime))
          do n = 1,nt
            TRACER(:,:,1,n,oldtime) = TRACER(:,:,1,n,oldtime)
     &                        /(dz(1) + PSURF(:,:,oldtime)/grav)
            TRACER(:,:,1,n,curtime) = TRACER(:,:,1,n,curtime)
     &                        /(dz(1) + PSURF(:,:,curtime)/grav)
          enddo

        else

          TRACER(:,:,:,:,oldtime) = p5*(TRACER(:,:,:,:,oldtime) +
     &                                  TRACER(:,:,:,:,curtime))
          TRACER(:,:,:,:,curtime) = p5*(TRACER(:,:,:,:,curtime) +
     &                                  TRACER(:,:,:,:,newtime))
          PSURF (:,:,oldtime) = p5*(PSURF (:,:,oldtime) +
     &                              PSURF (:,:,curtime))
          PSURF (:,:,curtime) = p5*(PSURF (:,:,curtime) +
     &                              PSURF (:,:,newtime))

        endif

        do k = 1,km  ! recalculate densities from averaged tracers
          call state(k,k,TRACER(:,:,k,1,oldtime),
     &                   TRACER(:,:,k,2,oldtime),
     &                 RHOOUT=RHO(:,:,k,oldtime))
          call state(k,k,TRACER(:,:,k,1,curtime),
     &                   TRACER(:,:,k,2,curtime),
     &                 RHOOUT=RHO(:,:,k,curtime))
        enddo 

        GRADPX(:,:,oldtime) = p5*(GRADPX(:,:,oldtime) + 
     &                            GRADPX(:,:,curtime))
        GRADPY(:,:,oldtime) = p5*(GRADPY(:,:,oldtime) + 
     &                            GRADPY(:,:,curtime))
        GRADPX(:,:,curtime) = p5*(GRADPX(:,:,curtime) + 
     &                            GRADPX(:,:,newtime))
        GRADPY(:,:,curtime) = p5*(GRADPY(:,:,curtime) + 
     &                            GRADPY(:,:,newtime))
        PGUESS = p5*(PGUESS + PSURF(:,:,newtime)) ! correct after avg

        FW_OLD = p5*(FW + FW_OLD)

      else                         ! non-averaging step

        if (mix_pass == 2) then ! reset time n variables on 2nd pass matsuno

          UBTROP(:,:,curtime) = UBTROP(:,:,oldtime)
          VBTROP(:,:,curtime) = VBTROP(:,:,oldtime)
          UVEL(:,:,:,curtime) = UVEL(:,:,:,oldtime)
          VVEL(:,:,:,curtime) = VVEL(:,:,:,oldtime)
          TRACER(:,:,:,:,curtime) = TRACER(:,:,:,:,oldtime)
          RHO(:,:,:,curtime) = RHO(:,:,:,oldtime)

        endif

        FW_OLD = FW

        tmptime = oldtime
        oldtime = curtime
        curtime = newtime
        newtime = tmptime

      endif

!-----------------------------------------------------------------------
!
!     end of timestep, all variables updated
!     compute and print some more diagnostics
!
!-----------------------------------------------------------------------

      call ice_flx_to_coupler

      call diag_global_afterupdate
      call diag_print
      call diag_transport

      if ( ldiag_global_tracer_budgets  .and.
     &     time_to_do(tavg_freq_iopt, tavg_freq) )
     &  call tracer_budgets

!-----------------------------------------------------------------------
!
!       display vertically integrated velocity and surface height in 
!       X window if requested
!
!-----------------------------------------------------------------------

      if (lxdisplay .and. 
     &    time_to_do(freq_opt_nstep,nstep_xdisplay)) then

        WORK2 = HU*sqrt(UBTROP(:,:,curtime)**2 + 
     &                  VBTROP(:,:,curtime)**2)
        call sw_4pt_nobndy(WORK1,AT0,ATS,ATW,ATSW,WORK2)  ! HU at t-pts
        WORK2 = PSURF(:,:,curtime)/grav ! surface height
        call display(WORK1,WORK2)

      endif

!-----------------------------------------------------------------------

 1001 format (/, 10x, 'VOLUME AND TRACER BUDGET INITIALIZATION:',
     &        /, 10x, '========================================',
     &        /,  5x, ' volume_t (cm^3)           = ', e18.12,
     &        /,  5x, ' SUM [volume*T] (C   cm^3) = ', e18.12,
     &        /,  5x, ' SUM [volume*S] (ppt cm^3) = ', e18.12 )

      end subroutine step

!***********************************************************************

      end module step_mod

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
