!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module surface_hgt

!***********************************************************************
!
!     contains routine for computing change in surface height
!
!     CVS:$Id: surface_hgt.F,v 1.2.2.1 2001/03/13 22:55:06 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use prognostic
      use stencils
      use operators
      use grid
      use time_management

      implicit none
      save

!***********************************************************************

      contains

!***********************************************************************

      subroutine dhdt

!-----------------------------------------------------------------------
!
!     calculate change in surface height dh/dt from surface pressure
!     and including freshwater flux for a variable-thickness surface 
!     layer (dh/dt - Fw)
!
!     surface pressure contribution at T points:  
!       dh/dt = (p - pb)/(grav*dt)
!
!     at U points:  dh/dt (or dh/dt - Fw) is the area-weighted average 
!     of value at surrounding T points
!
!   * in the advection routines 'advt' and 'advu', the vertical 
!     velocity W = {Wt,Wu} in {T,U} columns is determined by
!     integrating the continuity equation L(1) = 0 from the surface
!     down to level k.  Depending on surface layer type, the 
!     integration starts with:
!        W = 0           (rigid lid)
!        W = dh/dt       (old free surface)
!        W = dh/dt - Fw  (variable thickness surface layer)
!     at the surface (dh/dt is the time change of the surface height,
!     and in T columns satisfies the barotropic continuity equation:
!     dh/dt + Div{H*([U],[V])} = (0 or Fw), where  ([U],[V]) is the 
!     barotropic velocity).
!
!   * on the 2nd pass of a matsuno timestep, the predicted pressure
!     from the 1st pass is contained in array PSURF(:,:,newtime) 
!     (PSURF(:,:,curtime) is not updated until the end of the matsuno 
!     step).
!
!   * note that DH, DHU represents dh/dt - Fw at T, U points
!     in variable thickness surface layer
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

!     real (kind=dbl_kind), dimension(imt,jmt) :: WORK,WORKX,WORKY
!     real (kind=dbl_kind) :: div_error, div_mag, dh_dt,dfw,dhtot

!-----------------------------------------------------------------------
!
!     calculate dh/dt or dh/dt - Fw at T points
!
!-----------------------------------------------------------------------

      select case (sfc_layer_type)

      case (sfc_layer_varthick)
        if (mix_pass == 2) then
          DH = (PSURF(:,:,newtime) - PSURF(:,:,curtime))/(grav*dtp)
     &       - FW
        else
          DH = (PSURF(:,:,curtime) - PSURF(:,:,oldtime))/(grav*dtp)
     &       - FW_OLD
        endif

      case (sfc_layer_rigid)
        DH = c0

      case (sfc_layer_oldfree)
        if (mix_pass == 2) then
          DH = (PSURF(:,:,newtime) - PSURF(:,:,curtime))/(grav*dtp)
        else
          DH = (PSURF(:,:,curtime) - PSURF(:,:,oldtime))/(grav*dtp)
        endif

      end select

!-----------------------------------------------------------------------
!
!     Some debugging diagnostics
!     ...check barotropic continuity equation:
!
!-----------------------------------------------------------------------
!
!     WORKX = HU*UBTROP(:,:,curtime)
!     WORKY = HU*VBTROP(:,:,curtime)
!     call div(1,WORK,WORKX,WORKY)
!     WORKX = (DH*TAREA + WORK)**2
!     WORKY = TAREA**2
!     div_error=sqrt(global_sum(WORKX,RCALCT)/global_sum(WORKY,RCALCT))
!     WORKX = (DH*TAREA)**2
!     WORKY = TAREA**2
!     div_mag=sqrt(global_sum(WORKX,RCALCT)/global_sum(WORKY,RCALCT))
!     if (my_task == master_task) then
!       write(stdout,*) ' div error: ', div_error, div_mag
!     endif
!     WORK = -WORK/TAREA
!
!     WORKX = TAREA*(PSURF(:,:,curtime)-PSURF(:,:,oldtime))/grav
!     WORKY = dtp*TAREA*FW_OLD
!     WORK  = TAREA*PSURF(:,:,curtime)/grav
!     dh_dt  = global_sum(WORKX,RCALCT)/area_t
!     dfw   = global_sum(WORKY,RCALCT)/area_t
!     dhtot  = global_sum(WORK,RCALCT)/area_t
!     if (my_task == master_task) then
!       write(stdout,*) ' '
!       write(stdout,*) 'mean surface height           ', dhtot
!       write(stdout,*) 'mean change in surface height ', dh_dt
!       write(stdout,*) 'mean change from freshwater   ', dfw
!     endif
!
!-----------------------------------------------------------------------
!
!     calculate dh/dt (or dh/dt - Fw) at U points as the area-weighted 
!     average of value at T points.
!
!-----------------------------------------------------------------------

      call ne_4pt_nobndy(DHU,AU0,AUN,AUE,AUNE,DH)

      where (.not. CALCU) DHU = c0  ! zero at land points

!-----------------------------------------------------------------------

      end subroutine dhdt

!***********************************************************************

      end module surface_hgt

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
