!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module timers

!***********************************************************************
!
!     this module contains routine for supporting multiple CPU timers
!
!     CVS:$Id: timers.F,v 1.2.2.3 2001/09/07 19:52:32 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use io
      use exit_mod

      implicit none
      save

!-----------------------------------------------------------------------

      integer (kind=int_kind), parameter ::
     &   max_timers    = 50,   ! max number of timers
     &   timer_free    = 0,    ! timer status constants
     &   timer_inuse   = 1,
     &   timer_started = 2,
     &   timer_stopped = 3

      integer (kind=int_kind), dimension(max_timers) ::
     &   timer_status          ! status of each timer
     &,  cycles1               ! cycle number at start for each timer
     &,  cycles2               ! cycle number at stop  for each timer

      character (char_len), dimension(max_timers) ::
     &   timer_name       ! name assigned to each timer

      integer (kind=int_kind) :: 
     &   cycles_max       ! max value of clock allowed by system

      real (kind=dbl_kind), dimension(max_timers) ::
     &   accum_time       ! accumulated time

      real (kind=dbl_kind) ::  
     &   clock_rate       ! clock_rate in seconds for each cycle

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_timers

!-----------------------------------------------------------------------
!
!     This routine initializes some machine parameters necessary for
!     computing cpu time from F90 intrinsics.
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: cycles ! count rate return by sys_clock

!-----------------------------------------------------------------------

      timer_status = timer_free
      timer_name   = 'unknown_timer_name'
      accum_time   = c0
      cycles1      = 0
      cycles2      = 0
      clock_rate   = c0

      !---
      !--- Call F90 intrinsic system_clock to determine clock rate
      !--- and maximum cycles.  If no clock available, print message.
      !---

      call system_clock(count_rate=cycles, count_max=cycles_max)

      if (cycles /= 0) then
        clock_rate = c1/real(cycles)
      else
        clock_rate = c0
        write(stdout,*)  '--- No system clock available ---'
      endif

!-----------------------------------------------------------------------
      
      end subroutine init_timers

!***********************************************************************

      subroutine get_timer(num_timer, name_choice)

!-----------------------------------------------------------------------

      character (*), intent(in) ::
     &  name_choice         ! input name for this timer

      integer (kind=int_kind), intent(out) ::
     &  num_timer           ! timer number assigned to this timer 

      integer (kind=int_kind) :: n

!-----------------------------------------------------------------------
!
!     search for next free timer
!
!-----------------------------------------------------------------------

      do n=1,max_timers
        if (timer_status(n) == timer_free) then
          num_timer = n
          timer_status(n) = timer_inuse
          timer_name  (n) = name_choice
          return
        endif
      end do

      call exit_POP('Exceeded maximum number of timers')

!-----------------------------------------------------------------------

      end subroutine get_timer

!***********************************************************************

      subroutine timer_clear(n)

!-----------------------------------------------------------------------
!
!     This routine resets a given timer.
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &   n                ! timer number

!-----------------------------------------------------------------------

      accum_time(n) = c0

!-----------------------------------------------------------------------

      end subroutine timer_clear

!***********************************************************************

      subroutine timer_start(n)

!-----------------------------------------------------------------------
!
!     This routine starts a given timer.
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &   n                ! timer number

      real (kind=dbl_kind) :: elapse   ! dummy for returned elapse time

!-----------------------------------------------------------------------

      if (timer_status(n) == timer_started) call timer_stop(n)
      timer_status(n) = timer_started

      call system_clock(count=cycles1(n))

!-----------------------------------------------------------------------

      end subroutine timer_start
 
!***********************************************************************

      subroutine timer_stop(n)

!-----------------------------------------------------------------------
!
!     This routine stops a given timer.
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &   n                ! timer number

      real (kind=dbl_kind) ::
     &   end_time,        ! end time returned by system counter 
     &   elapse           ! elapsed time returned by system counter

!-----------------------------------------------------------------------

      call system_clock(count=cycles2(n))

      !---
      !--- check and correct for cycle wrapping
      !---

      if (cycles2(n) >= cycles1(n)) then
        accum_time(n) = accum_time(n) + clock_rate*  
     &                                  (cycles2(n) - cycles1(n))
      else
        accum_time(n) = accum_time(n) + clock_rate*  
     &                     (cycles_max + cycles2(n) - cycles1(n))
      endif

      timer_status(n) = timer_stopped

!-----------------------------------------------------------------------

      end subroutine timer_stop
 
!***********************************************************************

      subroutine timer_print(n)

!-----------------------------------------------------------------------
!
!     prints the accumulated time for a given timer
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &   n                ! timer number

      logical (kind=log_kind) ::
     &   lrestart_timer

!-----------------------------------------------------------------------

      if (timer_status(n) == timer_started) then
        call timer_stop(n)
        lrestart_timer = .true.
      else
        lrestart_timer = .false.
      endif

      if (my_task == master_task) then
        write (stdout,*) 'Time in timer: ',trim(timer_name(n))
        write (stdout,100) n,accum_time(n)
      endif

  100 format ('Timer number',i3,' =',f11.2,' seconds')

      if (lrestart_timer) call timer_start(n)

!-----------------------------------------------------------------------

      end subroutine timer_print

!***********************************************************************

      subroutine timer_print_all

!-----------------------------------------------------------------------
!
!     prints accumulated time for all timers in use
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: n

!-----------------------------------------------------------------------

      if (my_task == master_task) then
        write(stdout,*) ' '
        write(stdout,*) 'Timing information:'
        write(stdout,*) ' '
      endif

      do n=1,max_timers
        if (timer_status(n) /= timer_free) then
          call timer_print(n)
        endif
      end do

!-----------------------------------------------------------------------

      end subroutine timer_print_all

!***********************************************************************

      subroutine timer_check(n)

!-----------------------------------------------------------------------
!
!     This routine checks a given timer.  This is primarily used to
!     periodically accumulate time in the timer to prevent timer cycles
!     from wrapping around max_cycles.
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::  
     &    n            ! timer number

!-----------------------------------------------------------------------

      if (timer_status(n) == timer_started) then
        call timer_stop (n)
        call timer_start(n)
      endif

!-----------------------------------------------------------------------

      end subroutine timer_check

!***********************************************************************

      end module timers

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
