!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module topostress

!***********************************************************************
!
!     This module contains routines necessary for computing stress
!     due to bottom topography.
!
!     CVS:$Id: topostress.F,v 1.2.2.2 2001/09/07 19:52:34 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use io
      use stencils
      use operators
      use grid
      use global_reductions
      use exit_mod

      implicit none
      save

!-----------------------------------------------------------------------
!
!     Topographic Stress Velocities
!
!-----------------------------------------------------------------------

      logical (kind=log_kind)
     &  ltopostress         ! true if topographic stress desired

      real (kind=dbl_kind), dimension(:,:), allocatable :: 
     &  TSU, TSV

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_topostress

!-----------------------------------------------------------------------
!
!     this routine allocates stress arrays if topographic stress is
!     chosen.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input namelist to choose topostress
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: nml_error ! namelist i/o error flag

      namelist /topostress_nml/ltopostress

!-----------------------------------------------------------------------
!
!     read namelist to see if topostress desired
!
!-----------------------------------------------------------------------

      ltopostress = .false.
      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=topostress_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading topostress_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, topostress_nml)
          write(stdout,*) ' '
      endif

      if (my_task == master_task) then
        write(stdout,1000)
        write(stdout,1010)
        write(stdout,1020)
        write(stdout,1010)
        write(stdout,1000)
        if (ltopostress) then
          write(stdout,1030)
        else
          write(stdout,1040)
        endif

 1000   format(72('-'))
 1010   format(' ')
 1020   format(' Topographic stress options')
 1030   format(' Topographic stress enabled')
 1040   format(' Topographic stress disabled')

      endif

      call broadcast_scalar(ltopostress,master_task)

!-----------------------------------------------------------------------
!
!     allocate the topographic stress velocity arrays if required
!
!-----------------------------------------------------------------------

      if (ltopostress) then
        allocate (TSU(imt,jmt), TSV(imt,jmt))

        call topo_stress
      endif

!-----------------------------------------------------------------------

      end subroutine init_topostress

!***********************************************************************

      subroutine topo_stress

!-----------------------------------------------------------------------
!
!     Calculate topographic stress (maximum entropy) velocities.  These
!     are time-independent 2d fields.
!
!     Psistar = -f*L*L*H
!
!     H*Ustar = -Grady(Psistar)
!     H*Vstar = +Gradx(Psistar)
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   iter, max_iter       ! local iteration counters

      integer (kind=int_kind), dimension(imt,jmt) :: 
     &   KMZ,                 ! smoothed topography 
     &   IWORK                ! local work array for smoothing topog.

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &   TSP,
     &   SCALE,
     &   HTNEW

      real (kind=dbl_kind), parameter :: 
     &   tslse = 12.0e5_dbl_kind, 
     &   tslsp = 3.0e5_dbl_kind

!-----------------------------------------------------------------------
!
!     compute array to indicate 'interior' stream function boxes that
!     is the minimum of surrounding KMU field.
!
!-----------------------------------------------------------------------

      ! call w_shift(IWORK,KMU)
      ! KMZ = min(KMU,IWORK)
      ! call s_shift(IWORK,KMZ)
      ! KMZ = min(KMZ,IWORK)  ! KMZ = southwest min of surrounding KMUs

      KMZ = KMT  ! try this instead....

!-----------------------------------------------------------------------
!
!     calculate the topographic stress equilibrium stream function.
!     first we need latitudes at T-points.
!
!-----------------------------------------------------------------------

      SCALE = tslsp + (tslse - tslsp)*(p5 + p5*cos(c2*TLAT))

      TSP = HT  ! initialize
      max_iter = 0  ! no smoothing?
      do iter = 1, max_iter
        call smooth_topo2(TSP,HTNEW)
        TSP = HTNEW
      enddo

      if (max_iter <= 0) HTNEW = HT
      where (KMZ /= 0)
        TSP = -FCORT*SCALE*SCALE*TSP
      elsewhere
        TSP = c0
      end where

!-----------------------------------------------------------------------
!
!     calculate the topographic stress velocities from stream function.
!
!-----------------------------------------------------------------------

      call grad(1,TSV,TSU,TSP)  !  gradient at U pts from T point field

      where (KMU /= 0)
         TSU = -TSU*HUR
         TSV =  TSV*HUR
      elsewhere
         TSU = c0
         TSV = c0
      endwhere

      !  apply only in 'deep' water
      ! where (KMU.le.3)
      !   TSU = c0
      !   TSV = c0
      ! endwhere

!-----------------------------------------------------------------------

      end subroutine topo_stress

!***********************************************************************

      subroutine smooth_topo2(HTOLD,HTNEW)

!-----------------------------------------------------------------------
!
!     smooth topography
!
!   * this routine is nearly identical to the smooth topography
!     routine in the grid module
!
!   * HT is smoothed by by application of a
!     9-point averaging stencil:
!
!              1 -- 2 -- 1
!              |    |    |
!              2 -- 4 -- 2
!              |    |    |
!              1 -- 2 -- 1
!
!     land points are not included in the smoothing, and the
!     stencil is modified to include only ocean points in the
!     averaging.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &  HTOLD               ! old HT field to be smoothed

!-----------------------------------------------------------------------
!
!     output:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &  HTNEW               ! smoothed HT field

!-----------------------------------------------------------------------
!
!     local or common variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt) :: 
     &  NB,                 ! array to compute number of ocean neighbors
     &  IWORK               ! local work space

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  WORK                ! local work space

!-----------------------------------------------------------------------
!
!     smooth topography
!
!-----------------------------------------------------------------------

      where (KMT > 0) 
        NB = 1
        HTNEW = HTOLD
      elsewhere
        NB = 0
        HTNEW = c0
      endwhere

      call e_shift(WORK,HTNEW)
      HTNEW = HTNEW + WORK
      call w_shift(WORK,HTNEW)
      HTNEW = HTNEW + WORK
      call n_shift(WORK,HTNEW)
      HTNEW = HTNEW + WORK
      call s_shift(WORK,HTNEW)
      HTNEW = HTNEW + WORK

      call e_shift(IWORK,NB)
      NB = NB + IWORK
      call w_shift(IWORK,NB)
      NB = NB + IWORK
      call n_shift(IWORK,NB)
      NB = NB + IWORK
      call s_shift(IWORK,NB)
      NB = NB + IWORK

!-----------------------------------------------------------------------
!
!     new depth field
!
!-----------------------------------------------------------------------

      where ((KMT /= 0) .and. (NB /= 0)) 
        HTNEW = HTNEW/real(NB)
      elsewhere
        HTNEW = c0
      endwhere

!-----------------------------------------------------------------------

      end subroutine smooth_topo2

!***********************************************************************

      end module topostress

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
