!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module vertical_mix

!***********************************************************************
!
!     this module contains the main driver routines for computing
!     vertical mixing.  routines for computing the diffusion
!     coefficients themselves are in separate modules for the
!     specific parameterizations which must be chosen at compile time.  
!     currently, the options for vertical mixing include:
!       constant:  a constant (in space and time) vertical diffusion
!       Rich no:   vertical diffusion based on local Richardson number
!                  (e.g. Pacanowski and Philander)
!       KPP:       k-profile vertical mixing (from Large et al.)
!       Turb closure:  implementation of Mellor-Yamada turbulence
!                  closure
!
!     CVS:$Id: vertical_mix.F,v 1.2.2.5 2001/09/07 19:52:36 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use prognostic
      use grid
      use io
      use timers
      use tavg
      use forcing
      use time_management
      use vmix_const
      use vmix_rich
      use vmix_kpp
      use exit_mod

      implicit none
      save

!-----------------------------------------------------------------------
!
!     mixing coefficient arrays
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(:,:,:,:), allocatable, public ::
     &  VDC                 ! tracer diffusivity

      real (kind=dbl_kind), dimension(:,:,:), allocatable, public ::
     &  VVC                 ! momentum viscosity

!-----------------------------------------------------------------------
!
!     scalar vertical mixing quantities
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), parameter, public ::
     &  vmix_type_const = 1,   ! integer identifiers for desired
     &  vmix_type_rich  = 2,   ! mixing parameterization
     &  vmix_type_kpp   = 3

      integer (kind=int_kind), public ::
     &  vmix_itype             ! users choice for vmix parameterization

#ifdef impvmix
!-----------------------------------------------------------------------
!
!     vectors needed for implicit vertical mixing
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), private ::
     &   timer_impvmixt,     ! timer numbers for implicit vertical mix
     &   timer_impvmixu

      real (kind=dbl_kind), dimension(km), private :: 
     &   afac_u, afac_t, hfac_u, hfac_t
#endif

      integer (kind=int_kind), private ::
     &  timer_vmix_coeff
     &, timer_vdifft
     &, timer_vdiffu
 
!-----------------------------------------------------------------------
!
!     these arrays store flux information to avoid re-computing fluxes
!     for each k level
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,nt), private :: 
     &  VTF        ! vertical tracer flux at top of T-box
 
      real (kind=dbl_kind), dimension(imt,jmt), private :: 
     &  VUF,VVF    ! vertical momentum fluxes at top of box

!-----------------------------------------------------------------------
!
!     namelist input
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind) ::
     &  aidif,             ! time-centering parameter for implicit vmix
     &  bottom_drag,       ! drag coefficient for bottom drag
     &  bottom_heat_flx,       ! bottom heat (geothermal) flux         (W/m^2)
     &  bottom_heat_flx_depth  ! depth below which bottom_heat_flx is
                               ! applied                               (cm)

      character (char_len) ::
     &  vmix_choice        ! choice for desired parameterization

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_vertical_mix

!-----------------------------------------------------------------------
!
!     initializes various mixing quantities and calls initialization
!     routines for specific parameterizations.
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  k,                    ! vertical level index
     &  nu,                   ! i/o unit
     &  nml_error             ! namelist i/o error flag

      namelist /vertical_mix_nml/ vmix_choice, aidif, bottom_drag,
     &                            bottom_heat_flx, bottom_heat_flx_depth

!-----------------------------------------------------------------------
!
!     read input namelist and set mixing type
!
!-----------------------------------------------------------------------

      aidif = c1
      bottom_drag = 1.0e-3_dbl_kind
      bottom_heat_flx       = c0
      bottom_heat_flx_depth = 1000.0e2_dbl_kind

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=vertical_mix_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading vertical_mix_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, vertical_mix_nml)
          write(stdout,*) ' '
      endif

      if (my_task == master_task) then
        write(stdout,1000)
        write(stdout,*)
        write(stdout,'(a27)') 'Vertical mixing options'
        write(stdout,*)
        write(stdout,1000)
 1000   format(72('-')) 

#ifdef impvmix
        write(stdout,*) ' Implicit vertical mixing enabled'
#else
        write(stdout,*) ' Implicit vertical mixing disabled'
#endif

        select case (vmix_choice)

        case ('const')
          vmix_itype = vmix_type_const
          write(stdout,'(a32)') '  Using constant vertical mixing'

        case ('rich')
          vmix_itype = vmix_type_rich
          write(stdout,'(a32)') '  Using Richardson number mixing'

        case ('kpp')
          vmix_itype = vmix_type_kpp
          write(stdout,'(a32)') '  Using KPP vertical mixing     '

        case default
          vmix_itype = -1000
        end select

        write(stdout,'(a17,2x,1pe12.5)') '  aidif         =',aidif
        write(stdout,'(a17,2x,1pe12.5)') '  bottom_drag   =',bottom_drag
        write(stdout,'(a25,2x,1pe12.5)') '  bottom_heat_flx       =',
     &                                      bottom_heat_flx
        write(stdout,'(a25,2x,1pe12.5)') '  bottom_heat_flx_depth =',
     &                                      bottom_heat_flx_depth
      endif

      call broadcast_scalar(vmix_itype,  master_task)
      call broadcast_scalar(aidif,       master_task)
      call broadcast_scalar(bottom_drag, master_task)
      call broadcast_scalar(bottom_heat_flx,       master_task)
      call broadcast_scalar(bottom_heat_flx_depth, master_task)

      bottom_heat_flx = bottom_heat_flx * hflux_factor

      if (vmix_itype == -1000) then
        call exit_POP('Unknown vertical mixing choice')
      endif


!-----------------------------------------------------------------------
!
!     allocate VDC, VVC arrays and define options and timers for chosen 
!     parameterization
!
!-----------------------------------------------------------------------

      select case (vmix_itype)

      case(vmix_type_const)
#ifdef impvmix
        allocate (VDC(imt,jmt,km,1), VVC(imt,jmt,km))
#else
        allocate (VDC(imt,jmt,1,1), VVC(imt,jmt,1))
#endif
        call init_vmix_const(VDC,VVC)
        call get_timer(timer_vmix_coeff,'VMIX_COEFFICIENTS_CONSTANT')

      case(vmix_type_rich)
#ifdef impvmix
        allocate (VDC(imt,jmt,km,1), VVC(imt,jmt,km))
#else
        allocate (VDC(imt,jmt,1,1), VVC(imt,jmt,1))
#endif
        call init_vmix_rich(VDC,VVC)
        call get_timer(timer_vmix_coeff,'VMIX_COEFFICIENTS_RICH')

      case(vmix_type_kpp)
#ifndef impvmix
        call exit_POP('Implicit vertical mixing must be used with KPP')
#endif
        allocate (VDC(imt,jmt,0:km+1,2), VVC(imt,jmt,km))
        call init_vmix_kpp(VDC,VVC)
        call get_timer(timer_vmix_coeff,'VMIX_COEFFICIENTS_KPP')

      end select

      call get_timer(timer_vdifft,'VMIX_EXPLICIT_TRACER')
      call get_timer(timer_vdiffu,'VMIX_EXPLICIT_MOMENTUM') 

#ifdef impvmix
!-----------------------------------------------------------------------
!
!     set up implicit coefficients if implicit vertical mixing chosen
!
!-----------------------------------------------------------------------

      call get_timer(timer_impvmixt,'VMIX_IMPLICIT_TRACER')
      call get_timer(timer_impvmixu,'VMIX_IMPLICIT_MOMENTUM')

      do k=1,km
        afac_u(k) = aidif*dzwr(k)
        afac_t(k) = aidif*dzwr(k)
      enddo

#endif

 
!-----------------------------------------------------------------------

      end subroutine init_vertical_mix

!***********************************************************************

      subroutine vmix(k)

!-----------------------------------------------------------------------
!
!     this is a driver routine which calls the appropriate
!     parameterization to compute the mixing coefficients VDC, VVC
!     must be called successively for k=1,2,3...
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  k                      ! vertical level index

!-----------------------------------------------------------------------
!
!     call proper routine based on vmix choice
!
!-----------------------------------------------------------------------

      call timer_start (timer_vmix_coeff)
 
      select case(vmix_itype)

      case(vmix_type_const)
        call vmix_coeffs_const(k,VDC,VVC)

      case(vmix_type_rich)
        call vmix_coeffs_rich(k,VDC,VVC)

      case(vmix_type_kpp)
        if (k == 1) then     ! for KPP, compute all coeffs at once
          call vmix_coeffs_kpp(VDC,VVC)
        endif

      end select
     
      call timer_stop  (timer_vmix_coeff)
 
!-----------------------------------------------------------------------

      end subroutine vmix

!***********************************************************************

      subroutine vdifft(k,VDTK)

!-----------------------------------------------------------------------
!
!     computes vertical diffusion of tracers
!
!     VDiff(Tb) = Dz(vdc*Dz(Tb))
!
!     comments:
!
!   * this routine must be called successively with k = 1,2,3,...
!
!       VTF(:,:,n) = vertical tracer flux at top of T box
!         at level k for nth tracer
!       VTFB = vertical tracer flux at bottom of T box, level k 
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k   ! vertical level

!-----------------------------------------------------------------------
!
!     output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,nt), intent(out) :: 
     &   VDTK               ! returns VDIFF(Tracer(:,:,k,n))

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &   n,    ! dummy loop counter for tracer number
     &   kp1,  ! k+1
     &   kvdc, ! k index into mixing coefficient array
     &   mt2   ! index for use when temperature VDC is different

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &   VTFB  ! vertical tracer flux across bottom of T-box

 
      call timer_start (timer_vdifft) 
 
!-----------------------------------------------------------------------
!
!     set up vertical indices
!
!-----------------------------------------------------------------------

      if (k  <  km) then
        kp1 = k + 1
      else
        kp1 = km
      endif

      kvdc = min(k,size(VDC,DIM=3))  !*** reduce to 1 if VDC 2-d array 

!-----------------------------------------------------------------------
!
!     start loop over tracers 
!     set mt2 to be either 1 (if VDC same for all tracers) or n
!       if coefficients are different for tracers
!
!-----------------------------------------------------------------------

      do n = 1,nt
        mt2 = min(n,size(VDC,DIM=4))

!-----------------------------------------------------------------------
!
!       surface tracer fluxes
!
!-----------------------------------------------------------------------

        if (k == 1) then
          VTF(:,:,n) = merge(STF(:,:,n), c0, KMT >= 1)
        endif

!-----------------------------------------------------------------------
!
!       vertical tracer flux vdc*Dz(T) at bottom  of T box
!
!-----------------------------------------------------------------------

        VTFB = merge(VDC(:,:,kvdc,mt2)*
     &               (TRACER(:,:,k  ,n,oldtime)-
     &                TRACER(:,:,kp1,n,oldtime))*dzwr(k)
     &              ,c0, KMT > k)

!-----------------------------------------------------------------------
!
!       bottom tracer fluxes
!
!-----------------------------------------------------------------------

        if ( n == 1 ) then
          VTFB = merge( - bottom_heat_flx, VTFB,
     &                  k == KMT .and. zw(k) >= bottom_heat_flx_depth )
        endif

!-----------------------------------------------------------------------
!
!       calculate Dz(vdc*Dz(T))
!
!-----------------------------------------------------------------------

        VDTK(:,:,n) = merge( (VTF(:,:,n) - VTFB)*dzr(k), c0, k <= KMT )

!-----------------------------------------------------------------------
!
!       set top value of VTF to bottom value for next pass at level k+1
!
!-----------------------------------------------------------------------

        VTF(:,:,n) = VTFB  

      enddo
      
      call timer_stop  (timer_vdifft) 
!-----------------------------------------------------------------------

      end subroutine vdifft

!***********************************************************************

      subroutine vdiffu(k,VDUK,VDVK)

!-----------------------------------------------------------------------
!
!     vertical diffusion of momentum
!
!     VDiff(Ub) = Dz(vvc*Dz(Ub))
!
!     comments:
!
!   * this routine must be called successively with k = 1,2,3,...
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! vertical level index

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &   VDUK,              ! returns Vdiff(U)
     &   VDVK               ! returns Vdiff(V)

!-----------------------------------------------------------------------
!
!     local variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   kp1,     ! k+1
     &   kvvc     ! index into viscosity array

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &   VUFB,VVFB,         ! vertical momentum fluxes at bottom of box
     &   WORK               ! local work array for bottom drag

      call timer_start (timer_vdiffu) 
 
!-----------------------------------------------------------------------
!
!     set vertical level indices
!
!-----------------------------------------------------------------------

      if (k  <  km) then
        kp1 = k + 1
      else
        kp1 = km
      endif

      kvvc = min(k,size(VVC,DIM=3))  !*** reduce to 1 if VVC 2-d array 

!-----------------------------------------------------------------------
!
!     surface momentum fluxes (wind stress)
!
!-----------------------------------------------------------------------

      if (k == 1) then
        VUF = merge(SMF(:,:,1), c0, KMU >= 1)
        VVF = merge(SMF(:,:,2), c0, KMU >= 1)
      endif

!-----------------------------------------------------------------------
!
!     vertical momentum flux vvc*Dz{(Ub,Vb)} at bottom of U box
!
!-----------------------------------------------------------------------

      VUFB = VVC(:,:,kvvc)*(UVEL(:,:,k  ,oldtime) - 
     &                      UVEL(:,:,kp1,oldtime))*dzwr(k)
      VVFB = VVC(:,:,kvvc)*(VVEL(:,:,k  ,oldtime) - 
     &                      VVEL(:,:,kp1,oldtime))*dzwr(k)

!-----------------------------------------------------------------------
!
!     bottom momentum fluxes from quadratic drag law
!
!-----------------------------------------------------------------------

      where (k == KMU) 
        WORK = bottom_drag*sqrt(UVEL(:,:,k,oldtime)**2 + 
     &                          VVEL(:,:,k,oldtime)**2)
        VUFB = WORK*UVEL(:,:,k,oldtime)
        VVFB = WORK*VVEL(:,:,k,oldtime)
      endwhere

!-----------------------------------------------------------------------
!
!     calculate Dz(vdc*Dz{(U,V)})
!
!-----------------------------------------------------------------------

      VDUK = merge((VUF - VUFB)*dzr(k), c0, k <= KMU)
      VDVK = merge((VVF - VVFB)*dzr(k), c0, k <= KMU)

!-----------------------------------------------------------------------
!
!     set top value of (VUF,VVF) to bottom value for next pass
!     at level k+1 
!
!-----------------------------------------------------------------------

      VUF = VUFB  
      VVF = VVFB  

!-----------------------------------------------------------------------
!
!     accumulate time-average of vertical diffusion fluxes if reqd
!
!-----------------------------------------------------------------------

      if (ltavg_on .and. mix_pass /= 1) then
        call tavg_vdiffu(k,VUF,VVF)
      endif     
      
      call timer_stop  (timer_vdiffu) 
 
!-----------------------------------------------------------------------

      end subroutine vdiffu

#ifdef impvmix
!***********************************************************************

      subroutine impvmixt(nfirst,nlast,ssh_index,update_index,RHS)

!-----------------------------------------------------------------------
!
!    implicit vertical mixing of tracers
!    using tridiagonal solver in each vertical column
!
!    since top boundary condition (given tracer flux) does not depend
!    on the tracer value, its influence is accounted for in the explicit
!    part (subroutine vdifft).  the bottom b.c. is zero flux.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!     nfirst,nlast = range of tracers to update
!     nfirst = 3 for updating only passive tracers nt > 2
!     ssh_index = surface height time index
!     update_index = tracer time index to identify which time level
!                    to add delta(tracer)
!     RHS = optional right hand side. if this argument is absent
!           the solver RHS is assumed to be stored in TRACER(newtime)
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &  nfirst, nlast        ! update tracers w indices nfirst to nlast
     &, ssh_index            ! surface height time index
     &, update_index         ! tracer time index to add del(tracer)

      real (kind=dbl_kind), dimension(:,:,:), optional, intent(in) ::
     &  RHS                  ! optional right hand side

!-----------------------------------------------------------------------
!
!     output is an updated TRACER(newtime) field
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   i,j,k,n              ! dummy loop indices
     &,  mt2                  ! index for selecting tracer coefficient

      logical (kind=log_kind) ::
     &   rhs_present          ! true if RHS argument is present
                              ! if false RHS stored in TRACER(newtime)

      real (kind=dbl_kind) :: 
     &   a,b,c,d              ! various temporaries

      real (kind=dbl_kind), dimension(km) :: 
     &   e,f                  ! various work arrays

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &   H1            ! factor containing full thickness of sfc layer

!-----------------------------------------------------------------------
!
!     check and initialize some necessary quantities
!
!-----------------------------------------------------------------------
 
      if (nfirst > nlast .or. nfirst > nt) return

      call timer_start(timer_impvmixt)

      rhs_present = present(RHS)

      do k = 1,km
        hfac_t(k) = dz(k)/c2dtt(k)
      end do

      !*** correct hfac for full surface layer thickness

      if (sfc_layer_type == sfc_layer_varthick) then
        H1 = hfac_t(1) + PSURF(:,:,ssh_index)/(grav*c2dtt(1))
      else
        H1 = hfac_t(1)
      endif

!-----------------------------------------------------------------------
!
!     loop over tracers
!
!-----------------------------------------------------------------------

      do n = nfirst,nlast

        mt2 = min(n,size(VDC,DIM=4))

        do j=jphys_b,jphys_e
        do i=iphys_b,iphys_e

          !*** perform tridiagonal solve in the vertical for every
          !*** horizontal grid point

          a = afac_t(1)*VDC(i,j,1,mt2)
          d = H1(i,j) + a
          e(1) = a/d
          b = H1(i,j)*e(1)
          if (rhs_present) then
            f(1) = hfac_t(1)*RHS(i,j,n)/d
          else
            f(1) = hfac_t(1)*TRACER(i,j,1,n,newtime)/d
          endif
          f(KMT(i,j)+1:km) = c0

          do k=2,KMT(i,j)

            c = a
            a = afac_t(k)*VDC(i,j,k,mt2)
            if (k == KMT(i,j)) then
              d = hfac_t(k)+b
            else
              d = hfac_t(k)+a+b
            endif

            e(k) = a/d
            b = (hfac_t(k) + b)*e(k)
            if (rhs_present) then
              f(k) = c*f(k-1)/d
            else
              f(k) = (hfac_t(k)*TRACER(i,j,k,n,newtime) + c*f(k-1))/d
            endif

          end do

          !*** back substitution

          do k=KMT(i,j)-1,1,-1
            f(k) = f(k) + e(k)*f(k+1)
          end do

!-----------------------------------------------------------------------
!
!         The solution of the system (UZ) is DeltaTracer -- it has 
!         already been multiplied by dtt so we can advance the tracers 
!         directly
!
!-----------------------------------------------------------------------

          do k = 1,km
            TRACER(i,j,k,n,newtime) = 
     &      TRACER(i,j,k,n,update_index) + f(k)
          end do

        end do ! end of i-j loops
        end do

!-----------------------------------------------------------------------
!
!     end tracer loop and stop timer
!
!-----------------------------------------------------------------------

      end do
      call timer_stop(timer_impvmixt)

!-----------------------------------------------------------------------

      end subroutine impvmixt

!***********************************************************************

      subroutine impvmixu

!-----------------------------------------------------------------------
!
!    implicit vertical mixing of momentum
!    uses the tridiagonal solver routine (LU decomposition) found
!    in numerical recipes
!
!    since the top boundary condition (given wind stress) does not 
!    depend on the velocity, its influence is accounted for in the 
!    explicit part (subroutine vdiffu).  on the other hand, the bottom 
!    b.c. does depend on velocity (stress = bottom_drag*speed*(U,V)).  
!    here, we consider "speed" to be explicit, and the (U,V) to be 
!    implicit.  to make it fully explicit, remove any term with 
!    bottom_drag in it.
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &   i,j,k             ! dummy loop indices

      real (kind=dbl_kind) :: 
     &   a,b,c,d           ! various temp variables

      real (kind=dbl_kind), dimension(km) :: 
     &   e,f1,f2           ! various work arrays

!-----------------------------------------------------------------------
!-----------------------------------------------------------------------

      call timer_start(timer_impvmixu)

      do k=1,km
        hfac_u(k) = dz(k)/c2dtu
      end do

      do j=jphys_b,jphys_e
      do i=iphys_b,iphys_e

        !*** solve tridiagonal system at each grid point

        a = afac_u(1)*VVC(i,j,1)
        d = hfac_u(1) + a
        e(1) = a/d
        b = hfac_u(1)*e(1)
        f1(1) = hfac_u(1)*UVEL(i,j,1,newtime)/d
        f2(1) = hfac_u(1)*VVEL(i,j,1,newtime)/d
        f1(KMU(i,j)+1:km) = c0
        f2(KMU(i,j)+1:km) = c0

        do k=2,KMU(i,j)

          c = a
          a = afac_u(k)*VVC(i,j,k)
          if (k == KMU(i,j)) then
            d = hfac_u(k)+b
          else
            d = hfac_u(k)+a+b
          endif

          e(k) = a/d
          b = (hfac_u(k) + b)*e(k)
          f1(k) = (hfac_u(k)*UVEL(i,j,k,newtime) + c*f1(k-1))/d
          f2(k) = (hfac_u(k)*VVEL(i,j,k,newtime) + c*f2(k-1))/d

        end do

        do k=KMU(i,j)-1,1,-1
          f1(k) = f1(k) + e(k)*f1(k+1)
          f2(k) = f2(k) + e(k)*f2(k+1)
        end do

        do k=1,km
          UVEL(i,j,k,newtime) = f1(k)
          VVEL(i,j,k,newtime) = f2(k)
        end do

      end do
      end do

!-----------------------------------------------------------------------
!
!     UVEL,VVEL(newtime) now hold a modified rhs that has already been
!     multiplied by dtu to advance UVEL.
!
!-----------------------------------------------------------------------

      call timer_stop(timer_impvmixu)

!-----------------------------------------------------------------------

      end subroutine impvmixu

#endif
!***********************************************************************

      end module vertical_mix

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
