!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module vmix_const

!***********************************************************************
!
!     this module contains the initialization and vmix routine for
!     computing viscosity and diffusivity for constant vertical mixing
!
!     CVS:$Id: vmix_const.F,v 1.3.2.2 2001/09/07 19:52:38 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use io
      use diagnostics
      use state_mod
      use exit_mod
      use convection

      implicit none
      save

!-----------------------------------------------------------------------
!
!     mixing coefficients 
!       VVC(imt,jmt,vmix_levels), VDC(imt,jmt,vmix_levels,vmix_tracers)
!     vmix_levels can range from 1 to km+2 depending on mixing
!       parameterization or whether implicit mixing turned on.
!     vmix_tracers =1  if diffusivity same for all tracers
!                  =2  if diffusivity different for temperature only
!                  =nt if diffusivity different for each tracer
!     turbulence closure mixing adds additional diffusion coeffs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind) ::
     &  const_vvc,        ! constant value for viscosity
     &  const_vdc         ! constant value for diffusivity

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_vmix_const(VDC,VVC)

!-----------------------------------------------------------------------
!
!     In this routine we can initialize VVC and VDC since they
!     remain constant throughout the simulation.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input/output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(:,:,:,:), intent(inout) ::
     &  VDC                 ! vertical tracer diffusivity

      real (kind=dbl_kind), dimension(:,:,:), intent(inout) ::
     &  VVC                 ! vertical viscosity for momentum

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: nml_error

      namelist /vmix_const_nml/const_vvc, const_vdc

!-----------------------------------------------------------------------
!
!     set defaults for mixing coefficients, then read them from namelist
!
!-----------------------------------------------------------------------

      const_vvc = p25
      const_vdc = p25
      
      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=vmix_const_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading vmix_const_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, vmix_const_nml)
          write(stdout,*) ' '
      endif

      if (my_task == master_task) then
        write(stdout,'(a17,2x,1pe12.5)') '  const_vdc     =',const_vdc
        write(stdout,'(a17,2x,1pe12.5)') '  const_vvc     =',const_vvc
      endif

      call broadcast_scalar(const_vvc, master_task)
      call broadcast_scalar(const_vdc, master_task)

!-----------------------------------------------------------------------
!
!     set mixing coefficients for all time
!
!-----------------------------------------------------------------------

      VVC = const_vvc
      VDC = const_vdc

!-----------------------------------------------------------------------

      end subroutine init_vmix_const

!***********************************************************************

      subroutine vmix_coeffs_const(k,VDC,VVC)

!-----------------------------------------------------------------------
!
!     computes vertical diffusion coefficients
!
!     for this case, no operations need to be performed since the
!     coefficients are constant and set in the initialization routine
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! vertical level index

!-----------------------------------------------------------------------
!
!     input/output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(:,:,:,:), intent(inout) ::
     &  VDC                 ! vertical tracer diffusivity

      real (kind=dbl_kind), dimension(:,:,:), intent(inout) ::
     &  VVC                 ! vertical viscosity for momentum

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: kp1  ! vertical level index

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  RHOK, RHOKP

      real (kind=dbl_kind) ::
     &  vvconv             ! viscosity due to convection

!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     if using diffusion to simulate convection, enhance the vertical 
!     mixing coefficients if gravitationally unstable
!
!-----------------------------------------------------------------------

      if (convection_itype == convect_type_diff) then
        VVC(:,:,k  ) = const_vvc
        VDC(:,:,k,1) = const_vdc

        kp1 = min(k+1,km)

        call state(k  ,kp1,TRACER(:,:,k  ,1,curtime),
     &                     TRACER(:,:,k  ,2,curtime),RHOOUT=RHOK )
        call state(kp1,kp1,TRACER(:,:,kp1,1,curtime),
     &                     TRACER(:,:,kp1,2,curtime),RHOOUT=RHOKP)

        if (convect_visc /= c0) then
          vvconv = convect_visc
        else
          vvconv = const_vvc  ! convection only affects tracers
        endif

        where ((RHOK > RHOKP) .and. (k < KMT))
          VDC(:,:,k,1) = convect_diff
          VVC(:,:,k)   = vvconv
        endwhere
      endif

      call cfl_vdiff(k,VDC,VVC)

!-----------------------------------------------------------------------

      end subroutine vmix_coeffs_const

!***********************************************************************

      end module vmix_const

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
