!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module vmix_kpp

!***********************************************************************
!
!     this module contains routines for initializing and computing
!     vertical mixing coefficients for the KPP parameterization
!     (Large, McWilliams and Doney, Reviews of Geophysics, 32, 363 
!     November 1994).
!
!     CVS:$Id: vmix_kpp.F,v 1.3.2.4 2001/09/07 19:52:39 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use prognostic
      use stencils
      use grid
      use io
      use state_mod
      use diagnostics
      use exit_mod
      use convection
      use forcing
      use forcing_shf

      implicit none
      save

!-----------------------------------------------------------------------
!
!     mixing constants
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind) ::
     &  rich_mix         ! coefficient for rich number term

      real (kind=dbl_kind), dimension(km) ::
     &  bckgrnd_vvc,          ! background value for viscosity
     &  bckgrnd_vdc           ! background value for diffusivity

      logical (kind=log_kind) :: 
     &  lrich,         ! flag for computing Ri-dependent mixing
     &  ldbl_diff,     ! flag for computing double-diffusive mixing
     &  lshort_wave,   ! flag for computing short-wave forcing
     &  lcheckekmo     ! check Ekman, Monin-Obhukov depth limit

      integer (kind=int_kind) :: 
     &  num_v_smooth_Ri ! num of times to vertically smooth Ri

      real (kind=dbl_kind), parameter ::
     &  epssfc = 0.1_dbl_kind   ! non-dimensional extent of sfc layer

!-----------------------------------------------------------------------
!
!     parameters for subroutine "bldepth": computes bndy layer depth 
!
!     concv   = ratio of interior buoyancy frequency to 
!               buoyancy frequency at entrainment depth
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), parameter ::
     &   Ricr   = 0.3_dbl_kind, ! crit Rich no. for bndy layer depth
     &   cekman = 0.7_dbl_kind, ! coefficient for Ekman depth
     &   cmonob = 1.0_dbl_kind, ! coefficient for Monin-Obukhov depth
     &   concv  = 1.8_dbl_kind, !
     &   hbf    = 1.0_dbl_kind  ! frac of layer affected by sol forcing

!-----------------------------------------------------------------------
!
!     parameters for subroutine "ri_iwmix" which computes
!     vertical mixing coefficients below boundary layer due to shear
!     instabilities, internal waves and convection
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), parameter ::
     &   Riinfty = 0.8_dbl_kind,    ! Rich. no. limit for shear instab.
     &   BVSQcon = c0               ! Brunt-Vaisala square cutoff(s**-2)

!     &   BVSQcon = -0.2e-4_dbl_kind ! Brunt-Vaisala square cutoff(s**-2)

!-----------------------------------------------------------------------
!
!     parameters for subroutine "ddmix" (double-diffusive mixing)
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), parameter ::
     &   Rrho0  = 1.9_dbl_kind,  ! limit for double-diff density ratio
     &   dsfmax = 10.0_dbl_kind  ! max diffusivity for salt fingering

!-----------------------------------------------------------------------
!
!     parameters for subroutine "blmix": mixing within boundary layer
!
!     cstar   = proportionality coefficient for nonlocal transport
!     cg      = non-dimensional coefficient for counter-gradient term
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), parameter ::
     &  cstar = 10.0_dbl_kind ! coeff for nonlocal transport

      real (kind=dbl_kind) ::
     &  Vtc,  ! turbulent velocity shear coefficient (for bulk Ri no)
     &  cg    ! coefficient for counter-gradient term

!-----------------------------------------------------------------------
!
!     parameters for velocity scale function (from Large et al.)
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), parameter :: 
     &     zeta_m = -0.2_dbl_kind,
     &     zeta_s = -1.0_dbl_kind,
     &     c_m    =  8.38_dbl_kind,
     &     c_s    =  98.96_dbl_kind,
     &     a_m    =  1.26_dbl_kind,
     &     a_s    = -28.86_dbl_kind

!-----------------------------------------------------------------------
!
!     common arrays used by KPP mixing
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(0:km+1) :: 
     &  ZGRID,                ! depth at cell interfaces
     &  HWIDE                 ! layer thickness at interfaces

      real (kind=dbl_kind), dimension(:,:), allocatable :: 
     &  HMXL,                ! mixed layer depth
     &  HBLT                 ! boundary layer depth

      real (kind=dbl_kind), dimension(:,:,:,:), allocatable :: 
     &  SRC_KPP              ! non-local mixing (treated as source term)

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_vmix_kpp(VDC,VVC)

!-----------------------------------------------------------------------
!
!     Read in constants for the KPP parameterization
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     input/output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,km), intent(inout) ::
     &  VVC        ! viscosity for momentum diffusion

      real (kind=dbl_kind), dimension(imt,jmt,0:km+1,2),intent(inout) ::
     &  VDC        ! diffusivity for tracer diffusion

!-----------------------------------------------------------------------
!
!     input namelist
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  k,                    ! local dummy index for vertical lvl
     &  nml_error             ! namelist i/o error flag

      real (kind=dbl_kind) :: 
     &  bckgrnd_vdc1,         ! diffusivity at depth dpth
     &  bckgrnd_vdc2,         ! variation in diffusivity
     &  bckgrnd_vdc_dpth,     ! depth at which diff equals vdc1
     &  bckgrnd_vdc_linv,     ! inverse length for transition region
     &  Prandtl               ! Prandtl number

      namelist /vmix_kpp_nml/bckgrnd_vdc1, bckgrnd_vdc2, 
     &                       bckgrnd_vdc_dpth, bckgrnd_vdc_linv,
     &                       Prandtl, rich_mix,
     &                       num_v_smooth_Ri, lrich, ldbl_diff, 
     &                       lshort_wave, lcheckekmo

!-----------------------------------------------------------------------
!
!     set defaults for mixing coefficients, then read them from namelist
!
!-----------------------------------------------------------------------


      bckgrnd_vdc1    = 0.1_dbl_kind
      bckgrnd_vdc2    = c0
      bckgrnd_vdc_dpth= 2500.0e02_dbl_kind
      bckgrnd_vdc_linv= 4.5e-05_dbl_kind
      Prandtl         = 10.0_dbl_kind
      rich_mix        = 50.0_dbl_kind
      lrich           = .true.
      ldbl_diff       = .false.
      lshort_wave     = .false.
      lcheckekmo      = .false.
      num_v_smooth_Ri = 1

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=vmix_kpp_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading vmix_kpp_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, vmix_kpp_nml)
          write(stdout,*) ' '
      endif

      if (my_task == master_task) then
        write(stdout,1000) '  bckgrnd_vdc1    =', bckgrnd_vdc1
        write(stdout,1000) '  bckgrnd_vdc2    =', bckgrnd_vdc2
        write(stdout,1000) '  bckgrnd_vdc_dpth=', bckgrnd_vdc_dpth
        write(stdout,1000) '  bckgrnd_vdc_linv=', bckgrnd_vdc_linv
        write(stdout,1000) '  Prandtl         =', Prandtl
        write(stdout,1000) '  rich_mix        =', rich_mix
        write(stdout,1010) '  Ri mixing       =', lrich
        write(stdout,1010) '  double-diff     =', ldbl_diff
        write(stdout,1010) '  short_wave      =', lshort_wave
        write(stdout,1010) '  lcheckekmo      =', lcheckekmo
        write(stdout,1020) '  num_smooth_Ri   =', num_v_smooth_Ri
 1000   format (a19,2x,1pe12.5)
 1010   format (a19,2x,l7)
 1020   format (a19,2x,i5)

      endif

      call broadcast_scalar(bckgrnd_vdc1,    master_task)
      call broadcast_scalar(bckgrnd_vdc2,    master_task)
      call broadcast_scalar(bckgrnd_vdc_dpth,master_task)
      call broadcast_scalar(bckgrnd_vdc_linv,master_task)
      call broadcast_scalar(Prandtl,         master_task)
      call broadcast_scalar(rich_mix,        master_task)
      call broadcast_scalar(num_v_smooth_Ri, master_task)
      call broadcast_scalar(lrich,           master_task)
      call broadcast_scalar(ldbl_diff,       master_task)
      call broadcast_scalar(lshort_wave,     master_task)
      call broadcast_scalar(lcheckekmo,      master_task)

!-----------------------------------------------------------------------
!
!     define some non-dimensional constants 
!
!-----------------------------------------------------------------------

      Vtc     = concv*sqrt(0.2_dbl_kind/c_s/epssfc)/vonkar**2/Ricr
      cg      = cstar*vonkar*(c_s*vonkar*epssfc)**p33

!-----------------------------------------------------------------------
!
!     define vertical grid coordinates and cell widths
!     compute vertical profile of background (internal wave) 
!     diffusivity and viscosity
!
!     the vertical profile has the functional form
!
!     BCKGRND_VDC(z) = vdc1 + vdc2*atan((|z| - dpth)/L) or
!                    = vdc1 + vdc2*atan((|z| - dpth)*linv)
!
!       where
!
!     vdc1 = vertical diffusivity at |z|=D              (cm^2/s)
!     vdc2 = amplitude of variation                     (cm^2/s)
!     linv = inverse length scale for transition region (1/cm)
!     dpth = depth where diffusivity is vdc1            (cm)
!
!     the viscosity has the same form but multiplied by a constant
!     Prandtl number
!
!-----------------------------------------------------------------------

      if (bckgrnd_vdc2 /= c0 .and. my_task == master_task) then
        write (stdout,*)
        write (stdout,*) 'Vertical Profile for background diffusivity'
      endif

      zgrid(0) = eps
      hwide(0) = eps
      do k=1,km
        zgrid(k) = -zt(k)
        hwide(k) =  dz(k)

        bckgrnd_vdc(k) = bckgrnd_vdc1 + bckgrnd_vdc2*
     &                   atan(bckgrnd_vdc_linv*(zw(k)-bckgrnd_vdc_dpth))
        bckgrnd_vvc(k) = Prandtl*bckgrnd_vdc(k)

        if (bckgrnd_vdc2 /= c0 .and. my_task == master_task) then
          write (stdout,1030) bckgrnd_vdc(k)
        endif
      end do

      zgrid(km+1) = -zw(km)
      hwide(km+1) = eps

 1030 format (2x,e12.6)

!-----------------------------------------------------------------------
!
!     allocate and initialize KPP-specific arrays
!
!-----------------------------------------------------------------------

      allocate (HMXL   (imt,jmt),
     &          HBLT   (imt,jmt),
     &          SRC_KPP(imt,jmt,km,nt))

      HMXL     = c0
      HBLT     = c0
      SRC_KPP  = c0
      VDC      = c0
      VVC      = c0

!-----------------------------------------------------------------------

      end subroutine init_vmix_kpp

!***********************************************************************

      subroutine vmix_coeffs_kpp(VDC,VVC) 

!-----------------------------------------------------------------------
!
!     this is the main driver routine for the KPP mixing 
!     scheme as outlined in Large, McWilliams and Doney, Reviews of
!     Geophysics, 32, 363 (November 1994).
!
!     vertical mixing coefficients are module variables
!     and non-local mixing is treated as a source term.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input/output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,km), intent(inout) ::
     &  VVC        ! viscosity for momentum diffusion

      real (kind=dbl_kind), dimension(imt,jmt,0:km+1,2),intent(inout) ::
     &  VDC        ! diffusivity for tracer diffusion

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  k,                    ! vertical level index 
     &  n,                    ! tracer index
     &  mt2                   ! index for separating temp from other trcrs

      integer (kind=int_kind), dimension(imt,jmt) :: 
     &  KBL                   ! index of first lvl below hbl

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  USTAR,      ! surface friction velocity
     &  BFSFC,      ! surface buoyancy forcing
     &  STABLE      ! = 1 for stable forcing; = 0 for unstable forcing
 
      real (kind=dbl_kind), dimension(imt,jmt,km) :: 
     &  DBLOC,      ! buoyancy difference between adjacent levels
     &  DBSFC,      ! buoyancy difference between level and surface
     &  GHAT        ! non-local mixing coefficient

      real (kind=dbl_kind), dimension(imt,jmt,0:km+1) :: 
     &  VISC        ! local temp for viscosity

!-----------------------------------------------------------------------
!
!     compute buoyancy differences at each vertical level.
!
!-----------------------------------------------------------------------


      call buoydiff(DBLOC, DBSFC)

!-----------------------------------------------------------------------
!
!     compute mixing due to shear instability, internal waves and
!     convection
!
!-----------------------------------------------------------------------

      call ri_iwmix(DBLOC, VISC, VDC)

!-----------------------------------------------------------------------
!
!     compute double diffusion if desired
!
!-----------------------------------------------------------------------

      if (ldbl_diff) call ddmix(VDC)

!-----------------------------------------------------------------------
!
!     compute boundary layer depth
!
!-----------------------------------------------------------------------

      call bldepth (DBLOC, DBSFC, USTAR, BFSFC, STABLE, KBL)

!-----------------------------------------------------------------------
!
!     compute boundary layer diffusivities
!
!-----------------------------------------------------------------------

      call blmix(VISC, VDC, USTAR, BFSFC, STABLE, KBL, GHAT) 

!-----------------------------------------------------------------------
!
!     average mixing coeffs to V-grid and zero out land values
!     use USTAR as temporary storage
!
!-----------------------------------------------------------------------

      do k=1,km-1           

        call ne_4pt_nobndy(USTAR, AU0, AUN, AUE, AUNE, VISC(:,:,k))

        VVC(:,:,k) = merge(USTAR, c0, (k < KMU))

        where (k .ge. KMT)
          VDC(:,:,k,1) = c0
          VDC(:,:,k,2) = c0
        endwhere
      enddo

      VDC(:,:,km,:) = c0
      VVC(:,:,km)   = c0

!-----------------------------------------------------------------------
!
!     add ghatp term from previous computation to right-hand-side 
!     "source" term on current row
!
!-----------------------------------------------------------------------

      do n=1,nt
        mt2=min(n,2)
        SRC_KPP(:,:,1,n) = STF(:,:,n)/dz(1)
     &                  *(-VDC(:,:,1,mt2)*GHAT(:,:,1))
        do k=2,km
          SRC_KPP(:,:,k,n) = STF(:,:,n)/dz(k)
     &                    *( VDC(:,:,k-1,mt2)*GHAT(:,:,k-1) 
     &                      -VDC(:,:,k  ,mt2)*GHAT(:,:,k  ))
        enddo
      enddo

!-----------------------------------------------------------------------
!
!     compute diagnostic mixed layer depth (cm) using a max buoyancy 
!     gradient criterion.  Use USTAR and BFSFC as temps.
!
!-----------------------------------------------------------------------


        USTAR = c0
        where (KMT .eq. 1)
          HMXL = -zgrid(1)
        elsewhere
          HMXL = c0
        endwhere
        do k=2,km
          where (k .le. KMT)
            USTAR = max(DBSFC(:,:,k)/(-zgrid(k)),USTAR)
            HMXL = -zgrid(k)
          endwhere
        enddo

        VISC(:,:,1) = c0
        do k=2,km
          where (USTAR > c0 )
            VISC(:,:,k) = (DBSFC(:,:,k)-DBSFC(:,:,k-1))/
     &                    (zgrid(k-1) - zgrid(k))
          end where
          where (VISC(:,:,k) .ge. USTAR
     &      .and. (VISC(:,:,k)-VISC(:,:,k-1)) /= c0
     &      .and. USTAR > c0 )   ! avoid divide by zero
            BFSFC = (VISC(:,:,k) - USTAR)/
     &              (VISC(:,:,k)-VISC(:,:,k-1))
            HMXL = -p5*(zgrid(k)+zgrid(k-1))*(c1-BFSFC)
     &             -p5*(zgrid(k-1) + zgrid(k-2))*BFSFC
            USTAR(:,:) = c0
          endwhere
        enddo


!-----------------------------------------------------------------------
!
!     compute cfl diagnostics
!
!-----------------------------------------------------------------------

      call cfl_vdiff(km,VDC,VVC)

!-----------------------------------------------------------------------


!-----------------------------------------------------------------------

      end subroutine vmix_coeffs_kpp

!***********************************************************************

      subroutine ri_iwmix (DBLOC, VISC, VDC)

!-----------------------------------------------------------------------
!
!     compute viscosity and diffusivity coefficients for the interior
!     ocean due to shear instability (richardson number dependent),
!     internal wave activity, and to static instability (Ri < 0).
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     intent(in)
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,km), intent(in) :: 
     &  DBLOC             ! buoyancy difference between adjacent levels

!-----------------------------------------------------------------------
!
!     intent(out)
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,0:km+1), intent(out) :: 
     &  VISC              ! viscosity

!-----------------------------------------------------------------------
!
!     input/output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,0:km+1,2),intent(inout) ::
     &  VDC        ! diffusivity for tracer diffusion

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  k,                    ! index for vertical levels
     &  n                     ! vertical smoothing index

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  VSHEAR,             ! (local velocity shear)^2
     &  RI_LOC,             ! local Richardson number 
     &  FRI,                ! function of Ri for shear
     &  FCON                ! function of Ri for convection

!-----------------------------------------------------------------------
!
!     compute mixing at each level
!
!-----------------------------------------------------------------------

      VISC(:,:,0) = c0

      do k = 1,km

!-----------------------------------------------------------------------
!
!       compute velocity shear squared and average to T points:
!       VSHEAR = (UVEL(k)-UVEL(k+1))**2+(VVEL(k)-VVEL(k+1))**2
!       Use FRI here as a temporary.
!
!-----------------------------------------------------------------------

        if (k < km) then

          FRI = (UVEL(:,:,k,curtime)-UVEL(:,:,k+1,curtime))**2 +
     &          (VVEL(:,:,k,curtime)-VVEL(:,:,k+1,curtime))**2
          call sw_4pt_nobndy(VSHEAR, AT0, ATS, ATW, ATSW, FRI)

        else

          VSHEAR = c0

        endif

!-----------------------------------------------------------------------
!
!       compute local richardson number
!
!       use visc as temporary Ri storage to be smoothed
!
!-----------------------------------------------------------------------

        RI_LOC = DBLOC(:,:,k)*(zgrid(k)-zgrid(k+1))/(VSHEAR + eps)

        VISC(:,:,k)   = merge(RI_LOC, VISC(:,:,k-1), k <= KMT)
 
      enddo

!-----------------------------------------------------------------------
!
!     vertically smooth Ri num_v_smooth_Ri times with 1-2-1 weighting
!     result again stored temporarily in VISC and use RI_LOC and FRI
!     as temps
!
!-----------------------------------------------------------------------
 
      do n = 1,num_v_smooth_Ri
 
        FRI            =  p25 * VISC(:,:,1)
        VISC(:,:,km+1) =        VISC(:,:,km)
 
        do k=1,km
          RI_LOC      = VISC(:,:,k)
          where (KMT >= 3)
            VISC(:,:,k) = FRI + p5*RI_LOC + p25*VISC(:,:,k+1)
          end where
          FRI         = p25*RI_LOC
        enddo

      enddo

!-----------------------------------------------------------------------
!
!     now that we have a smoothed Ri field, finish computing coeffs
!     at each level
!
!-----------------------------------------------------------------------

      do k = 1,km

!-----------------------------------------------------------------------
!
!       compute function of Brunt-Vaisala squared for convection
!       RI_LOC = N**2,  FCON is function of N**2
!       FCON = 0 for N**2 > 0
!       FCON = [1-(1-Ri/BVSQcon)**2]**3 for BVSQcon < N**2 < 0
!       FCON = 1 for N**2 < BVSQcon
!
!-----------------------------------------------------------------------

        RI_LOC = DBLOC(:,:,k)/(zgrid(k) - zgrid(k+1))

        if (BVSQcon /= c0) then
          FRI    = min(c1-(max(RI_LOC,BVSQcon))/BVSQcon, c1)
          FCON   = (c1 - FRI*FRI)**3
        else
          where (RI_LOC > c0)
            FCON = c0
          elsewhere
            FCON = c1
          end where
        endif

!-----------------------------------------------------------------------
!
!       if Ri-number mixing requested,
!       evaluate function of Ri for shear instability:
!         for 0 < Ri < Riinfty, function = (1 - (Ri/Riinfty)**2)**3
!         for     Ri > Riinfty, function = 0
!         for     Ri < 0      , function = 1
!       compute contribution due to shear instability
!       VISC holds smoothed Ri at k, but replaced by real VISC
!
!       otherwise only add iw and convection pieces
!
!-----------------------------------------------------------------------

        if (lrich) then
          FRI    = min((max(VISC(:,:,k),c0))/Riinfty, c1)

          VISC(:,:,k  ) = bckgrnd_vvc(k) + convect_visc*FCON + 
     &                    rich_mix*(c1 - FRI*FRI)**3

          if ( k < km ) then
            VDC (:,:,k,2) = bckgrnd_vdc(k) + convect_diff*FCON +
     &                      rich_mix*(c1 - FRI*FRI)**3
            VDC(:,:,k,1) = VDC(:,:,k,2)
          endif
        else
          VISC(:,:,k  ) = bckgrnd_vvc(k) + convect_visc*FCON

          if ( k < km ) then
            VDC (:,:,k,2) = bckgrnd_vdc(k) + convect_diff*FCON
            VDC(:,:,k,1) = VDC(:,:,k,2)
          endif
        endif

!-----------------------------------------------------------------------
!
!       set seafloor values to zero
!
!-----------------------------------------------------------------------

        where ( k >= KMT )
          VISC(:,:,k  ) = c0
          VDC (:,:,k,1) = c0 
          VDC (:,:,k,2) = c0 
        endwhere

!-----------------------------------------------------------------------
!
!       move to next level.
!
!-----------------------------------------------------------------------

      end do

!-----------------------------------------------------------------------
!
!     fill extra coefficients for blmix
!
!-----------------------------------------------------------------------

      VISC(:,:,0  ) = c0
      VDC (:,:,0,:) = c0
      VISC(:,:,km+1  ) = c0 
      VDC (:,:,km+1,:) = c0 

!-----------------------------------------------------------------------
 
      end subroutine ri_iwmix

!***********************************************************************

      subroutine bldepth (DBLOC, DBSFC, USTAR, BFSFC, STABLE, KBL)

!-----------------------------------------------------------------------
!
!     this routine computes the ocean boundary layer depth defined as
!     the shallowest depth where the bulk Richardson number is equal to
!     a critical value, Ricr.
!
!     NOTE: bulk richardson numbers are evaluated by computing 
!           differences between values at zgrid(kl) < 0 and surface
!           reference values. currently, the reference values are equal 
!           to the values in the surface layer.  when using higher 
!           vertical grid resolution, these reference values should be 
!           computed as the vertical averages from the surface down to 
!           epssfc*zgrid(kl).
!
!     this routine also computes where surface forcing is stable 
!     or unstable (STABLE)
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     intent(in)
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,km), intent(in) :: 
     &  DBLOC,          ! buoyancy difference between adjacent levels
     &  DBSFC           ! buoyancy difference between level and surface

!-----------------------------------------------------------------------
!
!     intent(out)
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(out) :: 
     &  KBL                    ! index of first lvl below hbl

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &  BFSFC,              ! Bo+radiation absorbed to d
     &  STABLE,             ! =1 stable forcing; =0 unstab
     &  USTAR               ! surface friction velocity

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &  kup, kdn, ktmp, kl  ! vertical level indices

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  VSHEAR,             ! (velocity shear re sfc)^2
     &  SIGMA,              ! d/hbl
     &  WM, WS,             ! turb vel scale functions
     &  BO,                 ! surface buoyancy forcing
     &  BOSOL,              ! radiative buoyancy forcing
     &  TALPHA,             ! temperature expansion coeff
     &  SBETA,              ! salinity    expansion coeff
     &  RHO1,               ! density at the surface
     &  WORK                ! temp array


      real (kind=dbl_kind), dimension(imt,jmt,2) :: 
     &  RI_BULK             ! Bulk Ri number at 2 lvls

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  HEKMAN,             ! Eckman depth limit
     &  HMONOB,             ! Monin-Obukhov depth limit
     &  HLIMIT              ! limit to mixed-layer depth

      real (kind=dbl_kind) :: absorb_frac

!-----------------------------------------------------------------------
!
!     compute friction velocity USTAR.  compute on U-grid and average
!     to T-grid.
!
!-----------------------------------------------------------------------

      if (allocated(SMFT)) then
        USTAR = sqrt(sqrt(SMFT(:,:,1)**2 + SMFT(:,:,2)**2))
      else
        WORK = sqrt(sqrt(SMF(:,:,1)**2 + SMF(:,:,2)**2))
        call sw_4pt_nobndy(USTAR, AT0, ATS, ATW, ATSW, WORK)
      endif

!-----------------------------------------------------------------------
!
!     compute density and expansion coefficients at surface
!
!-----------------------------------------------------------------------

      WORK = merge(-c2,TRACER(:,:,1,1,curtime),
     &             TRACER(:,:,1,1,curtime) < -c2)

      call state(1,1,WORK,TRACER(:,:,1,2,curtime),RHOFULL=RHO1,
     &               DRHODT=TALPHA, DRHODS=SBETA)

!-----------------------------------------------------------------------
!
!     compute turbulent and radiative sfc buoyancy forcing
!
!-----------------------------------------------------------------------

      where (RHO1 /= c0)
        BO    = grav*(-TALPHA*STF(:,:,1) -
     &                  SBETA*STF(:,:,2))/RHO1

        BOSOL = -grav*TALPHA*SHF_QSW/RHO1
      elsewhere
        BO    = c0
        BOSOL = c0
      end where

!-----------------------------------------------------------------------
!
!     find bulk Richardson number at every grid level until > Ricr
!     max values when Ricr never satisfied are KBL = KMT and
!     HBLT = -zgrid(KMT)
!
!     NOTE: the reference depth is -epssfc/2.*zgrid(i,k), but the 
!           reference u,v,t,s values are simply the surface layer 
!           values and not the averaged values from 0 to 2*ref.depth,
!           which is necessary for very fine grids(top layer < 2m 
!           thickness)
!
!
!     initialize hbl and kbl to bottomed out values
!
!-----------------------------------------------------------------------

      kup = 1
      kdn = 2

      RI_BULK(:,:,kup) = c0 
c      KBL = max(KMT,1)
      KBL = merge(KMT, 1, (KMT > 1))

      do kl=1,km
        where (kl .eq. KBL) HBLT = -zgrid(kl)
      enddo

!-----------------------------------------------------------------------
!
!     compute velocity shear squared and average to T-grid
!
!-----------------------------------------------------------------------

      do kl = 2,km
 
        WORK = (UVEL(:,:,1,curtime)-UVEL(:,:,kl,curtime))**2 +
     &         (VVEL(:,:,1,curtime)-VVEL(:,:,kl,curtime))**2
        call sw_4pt_nobndy(VSHEAR, AT0, ATS, ATW, ATSW, WORK)

!-----------------------------------------------------------------------
!
!       compute bfsfc= Bo + radiative contribution down to hbf * hbl
!       add epsilon to BFSFC to ensure never = 0
!
!-----------------------------------------------------------------------

        if (lshort_wave) then
          call sw_absorb_frac(-zgrid(kl),jerlov_water_type,absorb_frac)
          BFSFC = BO + BOSOL * (c1 - absorb_frac)
        else
          BFSFC = BO
        endif

        STABLE = MERGE(c1, c0, BFSFC >= c0)

        BFSFC  = BFSFC + STABLE*eps

!-----------------------------------------------------------------------
!
!       compute velocity scales at sigma, for hbl = -zgrid(kl)
!
!-----------------------------------------------------------------------

        WORK  = -zgrid(kl)
        SIGMA = STABLE + (c1- STABLE) * epssfc

        call wscale(SIGMA, WORK, USTAR, BFSFC, 2, WM, WS)

!-----------------------------------------------------------------------
!
!       compute the turbulent shear contribution to RI_BULK and store
!       in WM.
!
!-----------------------------------------------------------------------

        WM = WORK*Vtc*WS*sqrt(abs(
     &       p5*( DBLOC(:,:,kl-1)/(zgrid(kl-1)-zgrid(kl  )) +
     &            DBLOC(:,:,kl  )/(zgrid(kl  )-zgrid(kl+1)) ) ))

!-----------------------------------------------------------------------
! 
!       compute bulk Richardson number at new level, dunder
!       note: Ritop needs to be zero on land and ocean bottom
!             points so that the following if statement gets triggered
!             correctly. otherwise, hbl might get set to (big) negative
!             values, that might exceed the limit for the "exp" function
!             in "swfrac"
!
!-----------------------------------------------------------------------

        WORK = MERGE( (zgrid(1)-zgrid(kl))*DBSFC(:,:,kl), 
     &                c0, KMT >= kl)

        RI_BULK(:,:,kdn) = WORK/(VSHEAR+WM+eps)

!-----------------------------------------------------------------------
!
!       linear interpolate to find hbl where Rib = Ricr
!
!-----------------------------------------------------------------------

        where ((KBL == KMT) .and. (RI_BULK(:,:,kdn) > Ricr))
          HBLT = -zgrid(kl-1) + (zgrid(kl-1)-zgrid(kl)) *
     &                          (Ricr             - RI_BULK(:,:,kup))/
     &                          (RI_BULK(:,:,kdn) - RI_BULK(:,:,kup))
          KBL = kl
        endwhere

!-----------------------------------------------------------------------
!
!       swap klevel index for RI_BULK and move to next level
!
!-----------------------------------------------------------------------

        ktmp = kup
        kup = kdn
        kdn = ktmp

      end do

!-----------------------------------------------------------------------
!
!     correct stability and buoyancy forcing for SW up to boundary layer
!
!-----------------------------------------------------------------------

      if (lshort_wave) then
        do j = 1, jmt
          do i = 1, imt
            call sw_absorb_frac(HBLT(i,j),jerlov_water_type,absorb_frac)
            BFSFC(i,j)  = BO(i,j) + BOSOL(i,j)*(c1 - absorb_frac) 
          enddo
        enddo
      endif

      STABLE = MERGE(c1, c0, BFSFC >= c0)

      BFSFC  = BFSFC + STABLE * eps ! ensures bfsfc never=0

!-----------------------------------------------------------------------
!
!     for stable forcing, bndy layer depth must not exceed h_ekman or
!     h_monin-obukhov.  if exceeds limit, compute new hbl, kbl.
!     for eckman, must compute coriolis parameter on T-grid
!
!-----------------------------------------------------------------------

      if (lcheckekmo) then

        where (BFSFC .gt. c0)
          HEKMAN = cekman*USTAR/(abs(FCORT)+eps)
          HMONOB = cmonob*USTAR*USTAR*USTAR/vonkar/BFSFC
          HLIMIT = STABLE*MIN(HEKMAN,HMONOB) + (STABLE-c1)*zgrid(km)

          HBLT = MIN(HBLT,HLIMIT)
          HBLT = MAX(HBLT,-zgrid(1))
        endwhere

        KBL = KMT
        do kl=2,km
          where ((KBL.eq.KMT) .and. (-zgrid(kl).gt.HBLT)) KBL = kl
        end do

!-----------------------------------------------------------------------
!
!     correct stability and buoyancy forcing for SW up to boundary layer
!
!-----------------------------------------------------------------------

        if (lshort_wave) then
          do j = 1, jmt
            do i = 1, imt
              call sw_absorb_frac(HBLT(i,j),jerlov_water_type
     &                           ,absorb_frac)
              BFSFC(i,j)  = BO(i,j) + BOSOL(i,j)*(c1 - absorb_frac) 
            enddo
          enddo

          STABLE = MERGE(c1, c0, BFSFC >= c0)
          BFSFC  = BFSFC + STABLE * eps ! ensures bfsfc never=0
        endif
      endif

!-----------------------------------------------------------------------

      end subroutine bldepth

!***********************************************************************

      subroutine blmix(VISC, VDC, USTAR, BFSFC, STABLE, KBL, GHAT) 

!-----------------------------------------------------------------------
!
!     this routine computes mixing coefficients within boundary layer 
!     which depend on surface forcing and the magnitude and gradient 
!     of interior mixing below the boundary layer ("matching").  These
!     quantities have been computed in other routines (ri_iwmix and
!     bldepth).
!
!     caution: if mixing bottoms out at hbl = -zgrid(km) then
!     fictitious layer at km+1 is needed with small but finite width 
!     hwide(km+1).
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     intent(in)
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(imt,jmt), intent(in) :: 
     &  KBL                    ! index of first lvl below hbl

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &  BFSFC,                 ! surface buoyancy forcing
     &  STABLE,                ! =1 stable forcing; =0 unstab
     &  USTAR                  ! surface friction velocity

!-----------------------------------------------------------------------
!
!     input/output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,0:km+1), intent(inout) :: 
     &  VISC               ! interior mixing coeff on input
                           ! combined interior/bndy layer coeff output

      real (kind=dbl_kind), dimension(imt,jmt,0:km+1,2),intent(inout) ::
     &  VDC        ! diffusivity for tracer diffusion

!-----------------------------------------------------------------------
!
!     intent(out)
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,km), intent(out) :: 
     &  GHAT                ! non-local mixing coefficient

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: k  ! dummy k level index

      integer (kind=int_kind), dimension(imt,jmt) :: 
     &  KN                  ! klvl closest to HBLT

      real (kind=dbl_kind), dimension(3,imt,jmt,km) :: 
     &  BLMC                ! bndy layer mixing coefs

      real (kind=dbl_kind), dimension(3,imt,jmt) :: 
     &  GAT1,               ! shape function at sigma=1
     &  DAT1,               ! derivative of shape function 
     &  DKM1                ! bndy layer difs at kbl-1 lvl

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  WM,WS,              ! turbulent velocity scales
     &  CASEA,              ! =1 in case A, =0 in case B     
     &  SIGMA,              ! normalized depth (d/hbl)
     &  VISCH,              ! viscosity at hbl
     &  DIFTH,              ! temp diffusivity at hbl
     &  DIFSH,              ! tracer diffusivity at hbl
     &  DELHAT, R, DVDZUP, DVDZDN,
     &  VISCP, DIFTP, DIFSP, F1

!-----------------------------------------------------------------------
!
!     compute velocity scales at hbl
!
!-----------------------------------------------------------------------

      SIGMA = STABLE + (c1- STABLE)*epssfc

      call wscale(SIGMA, HBLT, USTAR, BFSFC, 3, WM, WS)

!-----------------------------------------------------------------------
!
!     determine caseA = 0 if closer to KBL than KBL-1
!     KN is then the closest klevel to HBLT
!
!-----------------------------------------------------------------------

      do k=1,km
        where (k == KBL)
          CASEA  = p5 + SIGN(p5, -zgrid(k)-p5*hwide(k)-HBLT)
        endwhere
      enddo

      KN = NINT(CASEA)*(KBL-1) + (1-NINT(CASEA))*KBL

!-----------------------------------------------------------------------
!
!     find the interior viscosities and derivatives at hbl by 
!     interpolating derivative values at vertical interfaces.  compute
!     matching conditions for shape function.  
!
!-----------------------------------------------------------------------

      F1 = STABLE*c5*BFSFC/(USTAR**4+eps)

      do k=1,km
        where (k == KN)
          DELHAT = p5*hwide(k) - zgrid(k) - HBLT        
          R      = c1 - DELHAT / hwide(k)

          DVDZUP = (VISC(:,:,k-1) - VISC(:,:,k  ))/hwide(k)
          DVDZDN = (VISC(:,:,k  ) - VISC(:,:,k+1))/hwide(k+1)
          VISCP  = p5*( (c1-R)*(DVDZUP + abs(DVDZUP)) +
     &                      R *(DVDZDN + abs(DVDZDN)) )

          DVDZUP = (VDC(:,:,k-1,2) - VDC(:,:,k  ,2))/hwide(k)
          DVDZDN = (VDC(:,:,k  ,2) - VDC(:,:,k+1,2))/hwide(k+1)
          DIFSP  = p5*( (c1-R)*(DVDZUP + abs(DVDZUP)) +
     &                      R *(DVDZDN + abs(DVDZDN)) )

          DVDZUP = (VDC(:,:,k-1,1) - VDC(:,:,k  ,1))/hwide(k)
          DVDZDN = (VDC(:,:,k  ,1) - VDC(:,:,k+1,1))/hwide(k+1)
          DIFTP  = p5*( (c1-R)*(DVDZUP + abs(DVDZUP)) +
     &                      R *(DVDZDN + abs(DVDZDN)) )

          VISCH = VISC(:,:,k)  + VISCP*DELHAT
          DIFSH = VDC(:,:,k,2) + DIFSP*DELHAT
          DIFTH = VDC(:,:,k,1) + DIFTP*DELHAT

          GAT1(1,:,:) = VISCH / HBLT /(WM+eps)
          DAT1(1,:,:) = -VISCP/(WM+eps) + F1*VISCH

          GAT1(2,:,:) = DIFSH / HBLT /(WS+eps)
          DAT1(2,:,:) = -DIFSP/(WS+eps) + F1*DIFSH
 
          GAT1(3,:,:) = DIFTH / HBLT /(WS+eps)
          DAT1(3,:,:) = -DIFTP/(WS+eps) + F1*DIFTH

        endwhere
      enddo

      DAT1 = min(DAT1,c0)

!-----------------------------------------------------------------------
!
!     compute the dimensionless shape functions and diffusivities
!     at the grid interfaces.  also compute function for non-local
!     transport term (GHAT).
!
!-----------------------------------------------------------------------

      do k = 1,km       

        SIGMA = (-zgrid(k) + p5*hwide(k)) / HBLT        
        F1    = STABLE*SIGMA + (c1-STABLE)*min(SIGMA,epssfc)

        call wscale(F1, HBLT, USTAR, BFSFC, 3, WM, WS)

        BLMC(1,:,:,k) = HBLT*WM*SIGMA*(c1 + SIGMA*((SIGMA-c2)+
     &      (c3-c2*SIGMA)*GAT1(1,:,:)+(SIGMA-c1)*DAT1(1,:,:))) 
        BLMC(2,:,:,k) = HBLT*WS*SIGMA*(c1 + SIGMA*((SIGMA-c2)+     
     &      (c3-c2*SIGMA)*GAT1(2,:,:)+(SIGMA-c1)*DAT1(2,:,:)))
        BLMC(3,:,:,k) = HBLT*WS*SIGMA*(c1 + SIGMA*((SIGMA-c2)+    
     &      (c3-c2*SIGMA)*GAT1(3,:,:)+(SIGMA-c1)*DAT1(3,:,:)))

        GHAT(:,:,k) = (c1-STABLE)* cg/(WS*HBLT +eps)

      end do

!-----------------------------------------------------------------------
!
!     find diffusivities at kbl-1 grid level
!
!-----------------------------------------------------------------------

      do k=1,km
        where (k == KBL-1)
          SIGMA = -zgrid(k)/HBLT          
        endwhere
      enddo

      F1 = STABLE*SIGMA + (c1-STABLE)*min(SIGMA,epssfc)        
      call wscale(F1, HBLT, USTAR, BFSFC, 3, WM, WS)

      DKM1(1,:,:) = HBLT*WM*SIGMA*(c1+SIGMA*((SIGMA-c2)       
     &         +(c3-c2*SIGMA)*GAT1(1,:,:)+(SIGMA-c1)*DAT1(1,:,:)))
      DKM1(2,:,:) = HBLT*WS*SIGMA*(c1+SIGMA*((SIGMA-c2)      
     &         +(c3-c2*SIGMA)*GAT1(2,:,:)+(SIGMA-c1)*DAT1(2,:,:)))
      DKM1(3,:,:) = HBLT*WS*SIGMA*(c1+SIGMA*((SIGMA-c2)       
     &         +(c3-c2*SIGMA)*GAT1(3,:,:)+(SIGMA-c1)*DAT1(3,:,:)))

!-----------------------------------------------------------------------
!
!     compute the enhanced mixing
!
!-----------------------------------------------------------------------

      do k=1,km-1
        where (k == (KBL - 1))

          DELHAT = (HBLT + zgrid(k))/(zgrid(k)-zgrid(k+1))

          BLMC(1,:,:,k) = (c1-DELHAT)*VISC(:,:,k) +  DELHAT *( 
     &                    (c1-DELHAT)**2 *DKM1(1,:,:) + DELHAT**2 *
     &                    (CASEA*VISC(:,:,k)
     &                     +(c1-CASEA)*BLMC(1,:,:,k)))

          BLMC(2,:,:,k) = (c1-DELHAT)*VDC(:,:,k,2) + DELHAT*( 
     &                    (c1-DELHAT)**2 * DKM1(2,:,:) + DELHAT**2 * 
     &                    (CASEA*VDC(:,:,k,2)
     &                     +(c1-CASEA)*BLMC(2,:,:,k)))

          BLMC(3,:,:,k) = (c1-DELHAT)*VDC(:,:,k,1) + DELHAT *(        
     &                    (c1-DELHAT)**2 * DKM1(3,:,:) + DELHAT**2 * 
     &                    (CASEA*VDC(:,:,k,1)
     &                    +(c1-CASEA)*BLMC(3,:,:,k)))

          GHAT(:,:,k) = (c1-CASEA) * GHAT(:,:,k)

        endwhere
      end do

!-----------------------------------------------------------------------
!
!     combine interior and boundary layer coefficients and nonlocal term
!
!-----------------------------------------------------------------------

      do k=1,km
        where (k < KBL) 
          VISC(:,:,k)  = BLMC(1,:,:,k)
          VDC(:,:,k,2) = BLMC(2,:,:,k)
          VDC(:,:,k,1) = BLMC(3,:,:,k)
        elsewhere
          GHAT(:,:,k) = c0
        endwhere
      enddo

!-----------------------------------------------------------------------

      end subroutine blmix

!***********************************************************************

      subroutine wscale(SIGMA, HBL, USTAR, BFSFC, m_or_s, WM, WS)

!-----------------------------------------------------------------------
!
!     compute turbulent velocity scales.
!
!     For zeta >= 0, wm = ws = vonkar*ustar/(1+5*zeta)
!     For zeta_m <= zeta < 0, wm = vonkar*ustar*(1-16*zeta)**(1/4)
!     For zeta_s <= zeta < 0, ws = vonkar*ustar*(1-16*zeta)**(1/2)
!     For zeta < zeta_m, wm = vonkar*ustar*(a_m - c_m*zeta)**(1/3)
!     For zeta < zeta_s, ws = vonkar*ustar*(a_s - c_s*zeta)**(1/3)
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     inputs
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &  m_or_s              ! flag =1 for wm only, 2 for ws, 3 for both

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &  SIGMA,              ! normalized depth (d/hbl)
     &  HBL,                ! boundary layer depth
     &  BFSFC,              ! surface buoyancy forcing
     &  USTAR               ! surface friction velocity

!-----------------------------------------------------------------------
!
!     outputs
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &  WM,                 ! turb velocity scales: momentum
     &  WS                  ! turb velocity scales: tracer

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: i,j  ! dummy loop indices

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  ZETA  ! d/L or sigma*hbl/L(monin-obk)
     &, ZETAH ! sigma*hbl*vonkar*BFSFC or ZETA = ZETAH/USTAR**3

!-----------------------------------------------------------------------
!
!     compute zetah and zeta - surface layer is special case
!
!-----------------------------------------------------------------------

      ZETAH = SIGMA*HBL*vonkar*BFSFC
      ZETA  = ZETAH/(USTAR**3 + eps)

!-----------------------------------------------------------------------
!
!     compute velocity scales for momentum
!
!-----------------------------------------------------------------------

      if (m_or_s .eq. 1 .or. m_or_s .eq. 3) then
        do j=1,jmt
          do i=1,imt
            if (ZETA(i,j) >= c0) then ! stable region
              WM(i,j) = vonkar*USTAR(i,j)/(c1 + c5*ZETA(i,j))
            else if (ZETA(i,j) >= zeta_m) then
              WM(i,j) = vonkar*USTAR(i,j)*(c1 - c16*ZETA(i,j))**p25
            else
              WM(i,j) = vonkar*(a_m*(USTAR(i,j)**3)-c_m*ZETAH(i,j))**p33
            endif
          end do
        end do
      endif

!-----------------------------------------------------------------------
!
!     compute velocity scales for tracers
!
!-----------------------------------------------------------------------

      if (m_or_s .eq. 2 .or. m_or_s .eq. 3) then
        do j=1,jmt
          do i=1,imt
            if (ZETA(i,j) >= c0) then
              WS(i,j) = vonkar*USTAR(i,j)/(c1 + c5*ZETA(i,j))
            else if (ZETA(i,j) >= zeta_s) then
              WS(i,j) = vonkar*USTAR(i,j)*SQRT(c1 - c16*ZETA(i,j))
            else
              WS(i,j) = vonkar*(a_s*(USTAR(i,j)**3)-c_s*ZETAH(i,j))**p33
            endif
          end do
        end do
      endif

!-----------------------------------------------------------------------

      end subroutine wscale

!***********************************************************************

      subroutine ddmix(VDC)

!-----------------------------------------------------------------------
!
!     Rrho dependent interior flux parameterization.
!     Add double-diffusion diffusivities to Ri-mix values at blending
!     interface and below.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input/output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,0:km+1,2),intent(inout) ::
     &  VDC        ! diffusivity for tracer diffusion

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) ::  k,kup,knxt

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  ALPHADT,            ! alpha*DT  across interfaces
     &  BETADS,             ! beta *DS  across interfaces
     &  RRHO,               ! dd density ratio
     &  DIFFDD,             ! dd diffusivity scale
     &  PRANDTL             ! prandtl number

      real (kind=dbl_kind), dimension(imt,jmt,2) :: 
     &  TALPHA,             ! temperature expansion coeff
     &  SBETA               ! salinity    expansion coeff

!-----------------------------------------------------------------------
!
!     compute alpha*DT and beta*DS at interfaces.  use RRHO and
!     PRANDTL for temporary storage for call to state
!
!-----------------------------------------------------------------------

      kup  = 1
      knxt = 2

      PRANDTL = merge(-c2,TRACER(:,:,1,1,curtime),
     &                TRACER(:,:,1,1,curtime) < -c2)

      call state(1, 1, PRANDTL, TRACER(:,:,1,2,curtime), RHOFULL=RRHO,
     &                 DRHODT=TALPHA(:,:,kup), DRHODS=SBETA(:,:,kup))

      do k=1,km

        if ( k < km ) then

          PRANDTL = merge(-c2,TRACER(:,:,k+1,1,curtime),
     &                    TRACER(:,:,k+1,1,curtime) .lt. -c2)

          call state(k+1, k+1, PRANDTL, TRACER(:,:,k+1,2,curtime), 
     &                         RHOFULL=RRHO, DRHODT=TALPHA(:,:,knxt), 
     &                                       DRHODS= SBETA(:,:,knxt))

          ALPHADT = -p5*(TALPHA(:,:,kup) + TALPHA(:,:,knxt))
     &                 *(TRACER(:,:,k  ,1,curtime) - 
     &                   TRACER(:,:,k+1,1,curtime))

          BETADS  = p5*( SBETA(:,:,kup) +  SBETA(:,:,knxt))
     &                *(TRACER(:,:,k  ,2,curtime) - 
     &                  TRACER(:,:,k+1,2,curtime))

          kup  = knxt
          knxt = 3 - kup

        else

          ALPHADT = c0
          BETADS  = c0

        endif       

!-----------------------------------------------------------------------
!
!       salt fingering case
!
!-----------------------------------------------------------------------

        where ( ALPHADT > BETADS .and. BETADS > c0 )

          RRHO       = MIN(ALPHADT/BETADS, Rrho0)
          DIFFDD     = dsfmax*(c1-((RRHO-c1)/(Rrho0-c1))**2)**3
          VDC(:,:,k,1) = VDC(:,:,k,1) + 0.7_dbl_kind*DIFFDD
          VDC(:,:,k,2) = VDC(:,:,k,2) + DIFFDD

        endwhere

!-----------------------------------------------------------------------
!
!       diffusive convection
!
!-----------------------------------------------------------------------

        where ( ALPHADT < c0 .and. BETADS < c0 .and.
     &          ALPHADT > BETADS )

          RRHO    = ALPHADT / BETADS
          DIFFDD  = 1.5e-2_dbl_kind*0.909_dbl_kind*
     &              exp(4.6_dbl_kind*exp(-0.54_dbl_kind*(c1/RRHO-c1)))
          PRANDTL = 0.15_dbl_kind*RRHO
        elsewhere
          RRHO    = c0
          DIFFDD  = c0
          PRANDTL = c0
        endwhere

        where (RRHO > p5) 
     &         PRANDTL = (1.85_dbl_kind - 0.85_dbl_kind/RRHO)*RRHO

        VDC(:,:,k,1) = VDC(:,:,k,1) + DIFFDD
        VDC(:,:,k,2) = VDC(:,:,k,2) + PRANDTL*DIFFDD

      end do

!-----------------------------------------------------------------------

      end subroutine ddmix

!***********************************************************************

      subroutine buoydiff(DBLOC, DBSFC)

!-----------------------------------------------------------------------
!
!     this routine calculates the buoyancy differences at model levels.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     intent(out)
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,km), intent(out) :: 
     &  DBLOC,          ! buoyancy difference between adjacent levels
     &  DBSFC           ! buoyancy difference between level and surface

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  k,                  ! vertical level index
     &  kprev, klvl, ktmp   ! indices for 2-level TEMPK array

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  RHO1,               ! density of sfc t,s displaced to k
     &  RHOKM,              ! density of t(k-1),s(k-1) displaced to k
     &  RHOK,               ! density at level k
     &  TEMPSFC             ! adjusted temperature at surface

      real (kind=dbl_kind), dimension(imt,jmt,2) :: 
     &  TEMPK               ! temp adjusted for freeze at levels k,k-1

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  TALPHA,             ! temperature expansion coefficient
     &  SBETA               ! salinity    expansion coefficient

!-----------------------------------------------------------------------
!
!     calculate density and buoyancy differences at surface
!
!-----------------------------------------------------------------------

      TEMPSFC = merge(-c2,TRACER(:,:,1,1,curtime),
     &                    TRACER(:,:,1,1,curtime) < -c2)


      klvl  = 2
      kprev = 1

      TEMPK(:,:,kprev) = TEMPSFC
      DBSFC(:,:,1) = c0

!-----------------------------------------------------------------------
!
!     calculate DBLOC and DBSFC for all other levels
!
!-----------------------------------------------------------------------

      do k = 2,km

        TEMPK(:,:,klvl) = merge(-c2,TRACER(:,:,k,1,curtime),
     &                              TRACER(:,:,k,1,curtime) < -c2)

        call state(k, k, TEMPSFC, TRACER(:,:,1,2,curtime), RHOFULL=RHO1)

        call state(k, k, TEMPK(:,:,kprev), TRACER(:,:,k-1,2,curtime),
     &                   RHOFULL=RHOKM)

        call state(k, k, TEMPK(:,:,klvl), TRACER(:,:,k,2,curtime),
     &                   RHOFULL=RHOK)

        where (RHOK /= c0)
          DBSFC(:,:,k)   = grav*(c1 - RHO1/RHOK)
          DBLOC(:,:,k-1) = grav*(c1 - RHOKM/RHOK)
        elsewhere
          DBSFC(:,:,k)   = c0
          DBLOC(:,:,k-1) = c0
        end where

        where (k-1 >= KMT) DBLOC(:,:,k-1) = c0

        ktmp  = klvl
        klvl  = kprev
        kprev = ktmp

      enddo

      DBLOC(:,:,km) = c0

!-----------------------------------------------------------------------

      end subroutine buoydiff

!***********************************************************************

      subroutine swfrac(ZDPTH, SWFRC)

!-----------------------------------------------------------------------
!
!     this routine computes the fraction of solar short-wave flux
!     penetrating to the given depth ZDPTH.  
!     NOTE: ZDPTH is assumed to be < 0 (surface = 0).
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     intent(in)
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(in) :: 
     &  ZDPTH            ! z-coordinate where sw flux is to be computed

!-----------------------------------------------------------------------
!
!     intent(out)
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &  SWFRC               ! short wave fraction at depth ZDPTH

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), parameter :: 
     &  sol_coef   = 0.67_dbl_kind,
     &  sol_depth1 = 100.0_dbl_kind,
     &  sol_depth2 = 1700.0_dbl_kind

!-----------------------------------------------------------------------

      SWFRC =       sol_coef *exp(ZDPTH/sol_depth1)
     &      + (c1 - sol_coef)*exp(ZDPTH/sol_depth2)

!-----------------------------------------------------------------------

      end subroutine swfrac

!***********************************************************************

      end module vmix_kpp

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
