!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module vmix_rich

!***********************************************************************
!
!     this module contains routines for initializing and computing
!     vertical mixing coefficients for the Richardson number
!     parameterization (e.g. Pacanowski-Philander)
!
!     CVS:$Id: vmix_rich.F,v 1.3.2.2 2001/09/07 19:52:41 njn01 Exp $
!     CVS:$Name: ccsm2_0_beta46 $
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use prognostic
      use stencils
      use grid
      use io
      use state_mod
      use time_management
      use diagnostics
      use exit_mod
      use convection

      implicit none
      save

!-----------------------------------------------------------------------
!
!     scalar mixing constants
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind) ::
     &  bckgrnd_vvc,       ! background value for viscosity
     &  bckgrnd_vdc,       ! background value for diffusivity
     &  rich_mix           ! coefficient for rich number term

      real (kind=dbl_kind), dimension(km) ::
     &  critnuk            ! for min diffusion based on local stability

!-----------------------------------------------------------------------
!
!     average velocities to be saved to avoid re-computing at next
!     k level
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &  UTK, VTK            ! average velocities at T points level k

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_vmix_rich(VDC,VVC)

!-----------------------------------------------------------------------
!
!     Read in constants for the richardson number mixing 
!     parameterization
!
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!
!     input/output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(:,:,:,:), intent(inout) ::
     &  VDC                 ! vertical tracer diffusivity

      real (kind=dbl_kind), dimension(:,:,:), intent(inout) ::
     &  VVC                 ! vertical momentum viscosity

!-----------------------------------------------------------------------
!
!     input namelist
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  k,                    ! vertical level index
     &  nml_error             ! namelist i/o error flag

      namelist /vmix_rich_nml/bckgrnd_vvc, bckgrnd_vdc, rich_mix

!-----------------------------------------------------------------------
!
!     set defaults for mixing coefficients, then read them from namelist
!
!-----------------------------------------------------------------------

      bckgrnd_vvc = 1.0
      bckgrnd_vdc = 0.1
      rich_mix    = 50.0
      
      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=vmix_rich_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading vmix_rich_nml')
      endif


      if (my_task == master_task) then
          write(stdout,*) ' '
          write(stdout,*) ' Document Namelist Parameters:'
          write(stdout,*) ' ============================ '
          write(stdout,*) ' '
          write(stdout, vmix_rich_nml)
          write(stdout,*) ' '
      endif

      if (my_task == master_task) then
        write(stdout,'(a17,2x,1pe12.5)') '  bckgrnd_vvc   =',bckgrnd_vvc
        write(stdout,'(a17,2x,1pe12.5)') '  bckgrnd_vdc   =',bckgrnd_vdc
        write(stdout,'(a17,2x,1pe12.5)') '  rich_mix      =',rich_mix
      endif

      call broadcast_scalar(bckgrnd_vvc, master_task)
      call broadcast_scalar(bckgrnd_vdc, master_task)
      call broadcast_scalar(rich_mix,    master_task)

!-----------------------------------------------------------------------
!
!     set part of minimum diffusion coefficient based on stability
!     (divide by c2dtt later for full critical number)
!
!-----------------------------------------------------------------------

      do k=1,km
        critnuk(k) = p25*dz(k)*dzw(k)
      end do

!-----------------------------------------------------------------------
!
!     initialize VVC, VDC
!
!-----------------------------------------------------------------------

      VVC = c0
      VDC = c0

!-----------------------------------------------------------------------

      end subroutine init_vmix_rich

!***********************************************************************

      subroutine vmix_coeffs_rich(k,VDC,VVC)

!-----------------------------------------------------------------------
!
!     computes vertical diffusion coefficients for Richardson number
!     mixing parameterization
!
!   * this routine must be called successively with k = 1,2,3,...
!
!   * richardson numbers at T-points are first calculated, 
!     the the richardson number at U points is taken to be
!     the average of the richardson number at surrounding T points.
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! vertical level index

!-----------------------------------------------------------------------
!
!     input/output variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(:,:,:,:), intent(inout) ::
     &  VDC                 ! vertical tracer diffusivity

      real (kind=dbl_kind), dimension(:,:,:), intent(inout) ::
     &  VVC                 ! vertical momentum viscosity

!-----------------------------------------------------------------------
!
!     local or common variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  kp1,  ! k+1 for k < km, km otherwise
     &  kvc   ! k index into VVC, VDC array (to handle 2-d or 3-d cases)

      real (kind=dbl_kind) :: 
     &  critnu              ! minimum diffusion based on local stability

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  UTKP,VTKP,          ! avg velocities at T point at k+1 level
     &  RHOK,               ! level k density after adiab displ to k+1
     &  RICH,RICHU          ! richardson number at T,U points

!-----------------------------------------------------------------------
!
!     set up k+1 index and k-index for VVC,VDC arrays
!
!-----------------------------------------------------------------------

      kp1 = min(k+1, km)

      kvc = min(k,size(VVC,DIM=3)) ! defaults to 1 if VVC 2-d array

!-----------------------------------------------------------------------
!
!     average velocities at k=1 T points
!
!-----------------------------------------------------------------------

      if (k == 1) then
        call sw_4pt_nobndy(UTK,AT0,ATS,ATW,ATSW,UVEL(:,:,k,mixtime))
        call sw_4pt_nobndy(VTK,AT0,ATS,ATW,ATSW,VVEL(:,:,k,mixtime))
      endif 

!-----------------------------------------------------------------------
!
!     average velocities at T points at level k+1
!
!-----------------------------------------------------------------------

      call sw_4pt_nobndy(UTKP,AT0,ATS,ATW,ATSW,UVEL(:,:,kp1,mixtime))
      call sw_4pt_nobndy(VTKP,AT0,ATS,ATW,ATSW,VVEL(:,:,kp1,mixtime))

!-----------------------------------------------------------------------
!
!     density at level k after adiabatic displacement to k+1
!     for use in vertical gradient of potential density for
!     richardson number calculation
!
!-----------------------------------------------------------------------

      call state(k,kp1,TRACER(:,:,k,1,mixtime),
     &                 TRACER(:,:,k,2,mixtime),RHOOUT=RHOK)

!-----------------------------------------------------------------------
!
!     richardson number and vertical diffusivity at bottom of
!     T box, level k 
!
!-----------------------------------------------------------------------

      select case (convection_itype)
      case (convect_type_adjust)
        critnu = critnuk(k)/c2dtt(k)
      case (convect_type_diff)
        critnu = convect_diff
      end select

      where (k < KMT)

        RICH =  -grav*dzw(k)*(RHOK - RHO(:,:,kp1,mixtime))/
     &    ((UTK - UTKP)**2 + (VTK - VTKP)**2 + eps)

        VDC(:,:,kvc,1) = min(critnu,
     &    bckgrnd_vdc + (bckgrnd_vvc + 
     &                   rich_mix/(c1 + c5*RICH)**2)/(c1 + c5*RICH))

      elsewhere

        RICH = c0
        VDC(:,:,kvc,1) = c0

      endwhere

      where (RICH < c0) VDC(:,:,kvc,1) = critnu

!-----------------------------------------------------------------------
!
!     richardson number at bottom of U box, level k 
!     (average of richardson number at surrounding T points) 
!
!-----------------------------------------------------------------------

      call ne_4pt_nobndy(RICHU,AU0,AUN,AUE,AUNE,RICH)

!-----------------------------------------------------------------------
!
!     vertical viscosity at bottom of U box, level k 
!
!-----------------------------------------------------------------------
        
      if (convection_itype == convect_type_diff) then
        critnu = convect_visc
      endif

      where (k < KMU)

        VVC(:,:,kvc)  = min(critnu,
     &    bckgrnd_vvc + rich_mix/(c1 + c5*RICHU)**2)

      elsewhere

        RICHU = c0
        VVC(:,:,kvc)  = c0

      endwhere

      where (RICHU < c0) VVC(:,:,kvc) = critnu

!-----------------------------------------------------------------------
!
!     bottom values become top values for next pass
!
!-----------------------------------------------------------------------

      UTK = UTKP
      VTK = VTKP 
              
!-----------------------------------------------------------------------
!
!     compute cfl diagnostics if required
!
!-----------------------------------------------------------------------

      call cfl_vdiff(k,VDC,VVC) 

!-----------------------------------------------------------------------

      end subroutine vmix_coeffs_rich

!***********************************************************************

      end module vmix_rich

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
